/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "PDFScanManager.h"
#import "CreatePDFViewControllerNew.h"
#import "PDFScanManager+private.h"
#import "LTCanmeraViewControllerNew.h"
#import "Preference.h"

@interface PDFScanManager()
@property (nonatomic, strong, readwrite) PDFScanToolbarManager* toolbarManager;
@property (nonatomic, strong, readwrite) PDFScanMenuViewManager* menuViewManager;
@property (nonatomic, strong) LTDocumentSession* documentSession;
@end


@implementation PDFScanManager
- (instancetype)init{
    _topButtonsHidden = [[NSMutableDictionary alloc] init];
    return [super init];
}

static PDFScanManager *_manager = nil;
+ (PDFScanManager *)shareManager{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _manager = [[self alloc] init];
        _manager.toolbarManager = [[PDFScanToolbarManager alloc] init];
        _manager.menuViewManager = [[PDFScanMenuViewManager alloc] init];
    });
    return _manager;
}

static ScanPDFSaveAsCallBack _saveAsCallBack;
+ (void)setSaveAsCallBack:(ScanPDFSaveAsCallBack)saveAsCallBack{
    _saveAsCallBack = saveAsCallBack;
}

+ (ScanPDFSaveAsCallBack)saveAsCallBack{
    return _saveAsCallBack;
}

static ScanPDFDoneCallBack _doneCallBack;
+ (void)setDoneCallBack:(ScanPDFDoneCallBack)doneCallBack{
    _doneCallBack = doneCallBack;
}

+ (ScanPDFDoneCallBack)doneCallBack{
    return _doneCallBack;
}

static ScanPDFSessionPageCountChangeCallBack _pageCountCallBack;
+ (void)setPageCountCallBack:(ScanPDFSessionPageCountChangeCallBack)pageCountCallBack{
    _pageCountCallBack = pageCountCallBack;
}

+ (ScanPDFSessionPageCountChangeCallBack)pageCountCallBack{
    return _pageCountCallBack;
}

static FSErrorCode _errorCode = FSErrUnknown;
static BOOL _isTrial = NO;
+ (FSErrorCode)initializeScanner:(unsigned long)serial1 serial2:(unsigned long)serial2{
    _isTrial = NO;
    if (serial1 == 0 && serial2 == 0) _isTrial = YES;
    PDFScanManager *manager = [self shareManager];
    AppFrameworkConfiguration *appFrameworkConfiguration = [[AppFrameworkConfiguration alloc] init];
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
    if ( paths && ([paths count] > 0)) [appFrameworkConfiguration setDocumentsDirectory:[[paths objectAtIndex:0] stringByAppendingString:@"/InternalDocuments"]];
    manager.documentPath = appFrameworkConfiguration.documentsDirectory.copy;
    
    LTInitStatus status = AppFrameworkSdk_InitEx(serial1, serial2, appFrameworkConfiguration);
    
    switch (status)
    {
        case LTInitStatusSuccess:
            _errorCode =  FSErrSuccess;
            break;
        case LTInitStatusExpiredLicense:
            _errorCode = FSErrRightsExpired;
             break;
        case LTInitStatusInvalidLicense:
            _errorCode = FSErrInvalidLicense;
            break;
    }
    manager.documentManager = [[LTDocumentManager alloc] init];
    return _errorCode;
}

+ (FSErrorCode)initializeCompression:(unsigned long)serial1 serial2:(unsigned long)serial2{
    
    FSErrorCode errorCode = FSErrUnknown;
    
    LTInitStatus status = MobileCompressionSdk_Init(serial1, serial2);
    switch (status)
    {
        case LTInitStatusSuccess:
            errorCode = FSErrSuccess;
             break;
        case LTInitStatusExpiredLicense:
            errorCode = FSErrRightsExpired;
             break;
        case LTInitStatusInvalidLicense:
            errorCode = FSErrInvalidLicense;
             break;
    }
    return errorCode;
}

- (UIViewController*)getPDFScanView
{
    LTCanmeraViewControllerNew* cameraView = [[LTCanmeraViewControllerNew alloc] init];
    cameraView.documentSession = [LTDocumentSession createNewDocument];
    self.documentSession = cameraView.documentSession;
    cameraView.documentSession.rename = nil;
    cameraView.modalPresentationStyle = UIModalPresentationFullScreen;
    UIViewController *nav = [[FSNavigationController alloc] initWithRootViewController:cameraView];
    nav.modalPresentationStyle = UIModalPresentationFullScreen;
    [cameraView.navigationController setNavigationBarHidden:YES];
    
    
    return nav;
}

- (BOOL)isTrial{
    return _isTrial;
}

- (NSString *)documentPath{
    if (_isTrial) {
        return [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    }
    return _documentPath;
}

- (void)setItemHiddenWithType:(FSScanCameraControllerItemType)itemType hidden:(BOOL)hidden
{
    [_topButtonsHidden setValue:[NSNumber numberWithBool:hidden] forKey:[NSString stringWithFormat:@"%lud",itemType]];
}

- (void)saveAsPDF:(NSString*)pdfFilePath
{
    NSError* error = nil;
    if (!pdfFilePath) {
        error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileNoSuchFileError userInfo:nil];
        !_saveAsCallBack ? : _saveAsCallBack(error , nil);
        return;
    }
    if (![[pdfFilePath pathExtension] isEqualToString:@"pdf"])
    {
        error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteInvalidFileNameError userInfo:nil];
        !_saveAsCallBack ? : _saveAsCallBack(error , nil);
        return;
    }
    NSString* directory  = [pdfFilePath stringByDeletingLastPathComponent];
    if (![[NSFileManager defaultManager] fileExistsAtPath:directory]) {
        error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteFileExistsError userInfo:nil];
        !_saveAsCallBack ? : _saveAsCallBack(error , nil);
        return;
    }
    if (pdfFilePath) {
        self.pdfFilePathWithSaveas = pdfFilePath;
        NSString* documentName = [[pdfFilePath lastPathComponent] stringByDeletingPathExtension];
        [self.documentSession saveAsPDF:documentName];
    }
}

- (BOOL)savePDFFlieWithRealFilePath:(NSString*)realPDFFilePath
{
    if ([PDFScanManager shareManager].pdfFilePathWithSaveas.length > 0) {
        NSString* pdfFilePathWithSaveas = [PDFScanManager shareManager].pdfFilePathWithSaveas;
        NSError* error = nil;
        if ([pdfFilePathWithSaveas.lowercaseString isEqualToString:realPDFFilePath.lowercaseString]) {
            !_saveAsCallBack ? : _saveAsCallBack(error , realPDFFilePath);
            [PDFScanManager shareManager].pdfFilePathWithSaveas = @"";
            return  YES;
        }
        if ([[NSFileManager defaultManager] fileExistsAtPath:pdfFilePathWithSaveas])
        {
            if (![[NSFileManager defaultManager] removeItemAtPath:pdfFilePathWithSaveas error:&error])
            {
                NSLog(@"Could not remove old files. Error:%@",error);
            }
        }
        
        [[NSFileManager defaultManager] copyItemAtPath:realPDFFilePath toPath:pdfFilePathWithSaveas error:&error];
        [[NSFileManager defaultManager] removeItemAtPath:realPDFFilePath error:&error];
        [[NSFileManager defaultManager] fileExistsAtPath:pdfFilePathWithSaveas];
        
        !_saveAsCallBack ? : _saveAsCallBack(error , self.pdfFilePathWithSaveas);
        [PDFScanManager shareManager].pdfFilePathWithSaveas = @"";
        return YES;
    }
    return NO;
}
@end
