/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PDFScanMenuViewManager.h"
#import "PDFScanMenuViewManager+Private.h"

@interface PDFScanMenuViewManager ()

@property (nonatomic, strong) NSMutableDictionary <NSString *, id<PDFScanMenuView>>*menuViewTypes;
@property (nonatomic, strong) NSMutableDictionary <NSString *, NSMutableArray <PDFScanMenuItemGroup *>*> *customItemGroups;

@end

@interface PDFScanMenuItemGroup ()
@property (nonatomic, weak) NSMutableArray *supgroups;
@end

@implementation PDFScanMenuViewManager

- (instancetype)init
{
    self = [super init];
    if (self) {
        _menuViewTypes = @{}.mutableCopy;
        _customItemGroups = @{}.mutableCopy;
        [self setDefaultItemGroupWithTypeSaveAs];
    }
    return self;
}

- (void)setDefaultItemGroupWithTypeSaveAs
{
    NSMutableArray *items = @[].mutableCopy;
    [items addObject:[[PDFScanMenuItem alloc] initWithTitle:@"" image:nil actionTag:0]];
    PDFScanMenuItemGroup *group = [[PDFScanMenuItemGroup alloc] initWithTitle:FSLocalizedForKey(@"kFileName") items:items];
    [self addMenuItemGroup:group forMenuViewType:PDFScanMenuViewTypeSaveAs];
    
    NSMutableArray *formatItems = @[].mutableCopy;
    [formatItems addObject:[[PDFScanMenuItem alloc] initWithTitle:FSLocalizedForKey(@"kOriginalDocument") image:nil actionTag:1]];
    PDFScanMenuItemGroup *formatGroup = [[PDFScanMenuItemGroup alloc] initWithTitle:FSLocalizedForKey(@"kFormat") items:formatItems];
    [self addMenuItemGroup:formatGroup forMenuViewType:PDFScanMenuViewTypeSaveAs];
}

- (void)setMenuView:(id<PDFScanMenuView>)menuViewType forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSAssert(menuViewType, @"The menuViewType is not nil");
    NSAssert([menuViewType respondsToSelector:@selector(getCustomView)], @"The menuViewType must follow \"@protocol PDFScanMenuView\"");
    NSAssert(self.menuViewTypes[@(menuViewTypeType).stringValue] == nil, @"The addedCustomView has been added and is not allowed to be added repeatedly");
    NSAssert(menuViewTypeType >= PDFScanMenuViewTypeSaveAs && menuViewTypeType <= PDFScanMenuViewTypeSaveAs, @"The menuViewTypeType is not supported");
    
    [self.menuViewTypes setValue:menuViewType forKey:@(menuViewTypeType).stringValue];
}

- (id<PDFScanMenuView>)getMenuViewForMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    return self.menuViewTypes[@(menuViewTypeType).stringValue];
}

- (void)addMenuItemGroup:(PDFScanMenuItemGroup *)group forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[PDFScanMenuItemGroup class]], @"The group should be a subclass of PDFScanMenuItemGroup");
    NSAssert(menuViewTypeType >= PDFScanMenuViewTypeSaveAs && menuViewTypeType <= PDFScanMenuViewTypeSaveAs, @"menuViewTypeType is not supported");
    
    NSMutableArray *itemGroups = self.customItemGroups[@(menuViewTypeType).stringValue];
    if (!itemGroups)itemGroups = @[].mutableCopy;
    [itemGroups addObject:group];
    [self.customItemGroups setValue:itemGroups forKey:@(menuViewTypeType).stringValue];
    group.supgroups = itemGroups;
}

- (void)insertMenuItemGroup:(PDFScanMenuItemGroup *)group atIndex:(NSInteger)index forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[PDFScanMenuItemGroup class]], @"The group should be a subclass of PDFScanMenuItemGroup");
    
    NSMutableArray *itemGroups = self.customItemGroups[@(menuViewTypeType).stringValue];
    if (!itemGroups) itemGroups = @[].mutableCopy;
    
    NSAssert(index <= itemGroups.count, @"Index must be less than or equal to count of itemGroups");
    
    [itemGroups insertObject:group atIndex:index];
    [self.customItemGroups setValue:itemGroups forKey:@(menuViewTypeType).stringValue];
    group.supgroups = itemGroups;
}

- (void)exchangeMenuItemGroupAtIndex:(NSInteger)index1 withMenuItemGroupAtIndex:(NSInteger)index2 forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSMutableArray *itemGroups = self.customItemGroups[@(menuViewTypeType).stringValue];
    PDFScanMenuItemGroup *group1 = [itemGroups objectAtIndex:index1];
    
    NSAssert(group1, @"Index1 does not exist in itemGroups");
    
    PDFScanMenuItemGroup *group2 = [itemGroups objectAtIndex:index2];
    
    NSAssert(group2, @"Index2 does not exist in itemGroups");
    
    [itemGroups replaceObjectAtIndex:index1 withObject:group2];
    [itemGroups replaceObjectAtIndex:index2 withObject:group1];
}

- (void)insertMenuItemGroup:(PDFScanMenuItemGroup *)group beforeGroup:(PDFScanMenuItemGroup *)siblingGroup forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[PDFScanMenuItemGroup class]], @"The group should be a subclass of PDFScanMenuItemGroup");
    NSAssert(siblingGroup, @"The siblingGroup is not nil");
    
    NSMutableArray *itemGroups = self.customItemGroups[@(menuViewTypeType).stringValue];
    if (!itemGroups) itemGroups = @[].mutableCopy;
    
    NSAssert([itemGroups containsObject:siblingGroup], @"The siblingGroup needs to be added");
    
    NSUInteger index = [itemGroups indexOfObject:siblingGroup];
    [itemGroups insertObject:group atIndex:index];
    group.supgroups = itemGroups;

}
- (void)insertMenuItemGroup:(PDFScanMenuItemGroup *)group afterGroup:(PDFScanMenuItemGroup *)siblingGroup forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[PDFScanMenuItemGroup class]], @"The group should be a subclass of PDFScanMenuItemGroup");
    NSAssert(siblingGroup, @"The siblingGroup is not nil");
    
    NSMutableArray *itemGroups = self.customItemGroups[@(menuViewTypeType).stringValue];
    if (!itemGroups) itemGroups = @[].mutableCopy;
    
    NSAssert([itemGroups containsObject:siblingGroup], @"The siblingGroup needs to be added");
    
    NSUInteger index = [itemGroups indexOfObject:siblingGroup];
    [itemGroups insertObject:group atIndex:index+1];
    group.supgroups = itemGroups;
}

- (void)removeMenuItemGroup:(PDFScanMenuItemGroup *)group forMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    NSMutableArray *itemGroups = self.customItemGroups[@(menuViewTypeType).stringValue];
    if (itemGroups) {
        [itemGroups removeObject:group];
        if (!itemGroups.count) {
            [self.customItemGroups setValue:nil forKey:@(menuViewTypeType).stringValue];
        }
    }
}

- (NSArray <PDFScanMenuItemGroup *> *)getMenuItemGroupsForMenuViewType:(PDFScanMenuViewType)menuViewTypeType{
    return self.customItemGroups[@(menuViewTypeType).stringValue].count ? self.customItemGroups[@(menuViewTypeType).stringValue].copy : nil;
}

@end

@interface PDFScanMenuItem ()
@property (nonatomic, weak) PDFScanMenuItemGroup *supgroup;
@end

@implementation PDFScanMenuItem

- (instancetype)init{
    self = [super init];
    if (self) {
        _actionTag = -1;
    }
    return self;
}

- (instancetype)initWithTitle:(NSString *)title image:(nullable UIImage *)image action:(PDFScanMenuItemAction)action{
    self = [self init];
    if (self) {
        _image = image;
        _title = title;
        _action = [action copy];
    }
    return self;
}

- (NSUInteger)index{
    if (_supgroup) {
        return [_supgroup.items indexOfObject:self];;
    }
    return NSNotFound;
}

#pragma mark - private

- (instancetype)initWithTitle:(NSString *)title image:(UIImage *)image actionTag:(NSInteger)tag{
    self = [self init];
    if (self) {
        _image = image;
        _title = title;
        _actionTag = tag;
    }
    return self;
}

@end

@implementation PDFScanMenuItemGroup

- (instancetype)initWithTitle:(nullable NSString *)title items:(NSArray <PDFScanMenuItem *> *)items{
    self = [super init];
    if (self) {
        _items = items.mutableCopy;
        _title = title;
    }
    return self;
}

- (NSUInteger)index{
    if (_supgroups) {
        return [_supgroups indexOfObject:self];
    }
    return NSNotFound;
}

@end
