/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, PDFScanMenuViewType) {
    ///< The thumbnail controller clicks on the insertBlankPage pop-up view.
    PDFScanMenuViewTypeThumbnailInsertBlankPagePop = 0,
    ///< Extraction controller.
    PDFScanMenuViewTypeExtract,
    ///< Save as controller.
    PDFScanMenuViewTypeSaveAs,
};
@class PDFScanMenuItem;
/** @brief Callback for click action. */
typedef void (^PDFScanMenuItemAction)(PDFScanMenuItem *_item);

/** brief Definition for custom item.*/
@interface PDFScanMenuItem : NSObject
/** brief Get  index of the  custom Item in the group It will be NSNotFound if it is not added.*/
@property (nonatomic, assign, readonly) NSUInteger index;
/** brief The custom title text.*/
@property (nonatomic, copy) NSString *title;
/** brief The custom item image.*/
@property (nonatomic, strong, nullable) UIImage *image;
/** brief The callback action associated with this menu item.*/
@property (nonatomic, copy, nullable) PDFScanMenuItemAction action;
/** @brief Intialize custom Item.
 *
 * @param[in]    title        The Item title.
 * @param[in]    image        The custom item image.
 * @param[in]    action      The custom item action callback.
 *
 * @return    The custom Item instance.
 */
- (instancetype)initWithTitle:(NSString *)title image:(nullable UIImage *)image action:(PDFScanMenuItemAction)action;
@end
/** @brief The custom item group. */
@interface PDFScanMenuItemGroup : NSObject
/** @brief Get  index of the  custom group in the view. It will be NSNotFound if it is not added. */
@property (nonatomic, assign, readonly) NSUInteger index;
/** @brief The title for custom group.*/
@property (nonatomic, copy) NSString *title;
/** @brief Set/Get the custom items.*/
@property (nonatomic, strong) NSMutableArray<PDFScanMenuItem *> * items;
/** @brief Intialize custom group.
 *
 * @param[in]    title        The group title.
 * @param[in]    items        The custom group items.
 *
 * @return    The custom group instance.
 */
- (instancetype)initWithTitle:(nullable NSString *)title items:(NSArray <PDFScanMenuItem *> *)items;
@end
@protocol PDFScanMenuView <NSObject>

@property (nonatomic, weak) UIViewController *containerController;

- (UIView *)getCustomView;
@end

@interface PDFScanMenuViewManager : NSObject

/** @brief Set id<PDFScanMenuView> for menu vew type. If the current menu view uses the current interface, the interface {@link addMenuItemGroup}  {@link insertMenuItemGroup}  {@link removeMenuItemGroup} {@link exchangeMenuItemGroupAtIndex} are invalidated, and the id<PDFScanMenuView> is responsible for all behavior.
 *
 *  @param[in]    menuView        The  id<PDFScanMenuView>.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)setMenuView:(id<PDFScanMenuView>)menuView forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Get id<PDFScanMenuView>  for menu view type.   
 *
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 *
 *  @return     The id<PDFScanMenuView> .
 */
- (id<PDFScanMenuView>)getMenuViewForMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Add group for menu vew type.
 *
 *  @param[in]    group        The  id<PDFScanMenuItemGroup>.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)addMenuItemGroup:(PDFScanMenuItemGroup *)group forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Insert group  at index .
 *
 *  @param[in]    group        The  id<PDFScanMenuItemGroup>.
 *  @param[in]    index        Index of group in menu view.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)insertMenuItemGroup:(PDFScanMenuItemGroup *)group atIndex:(NSInteger)index forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Exchange  group1's index with group2's index .
 *
 *  @param[in]    index1        The  group1's index.
 *  @param[in]    index2        The  group2's index.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)exchangeMenuItemGroupAtIndex:(NSInteger)index1 withMenuItemGroupAtIndex:(NSInteger)index2 forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Insert the group before the sibling group.
 *
 *  @param[in]    group        The group that needs to be inserted.
 *  @param[in]    siblingGroup        Existing sibling group.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)insertMenuItemGroup:(PDFScanMenuItemGroup *)group beforeGroup:(PDFScanMenuItemGroup *)siblingGroup forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Insert the group after the sibling group.
 *
 *  @param[in]    group        The group that needs to be inserted.
 *  @param[in]    siblingGroup        Existing sibling group.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)insertMenuItemGroup:(PDFScanMenuItemGroup *)group afterGroup:(PDFScanMenuItemGroup *)siblingGroup forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Remove group  for menu view type.
 *
 *  @param[in]    group        The group that need to be removed.
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 */
- (void)removeMenuItemGroup:(PDFScanMenuItemGroup *)group forMenuViewType:(PDFScanMenuViewType)menuViewType;
/** @brief Get groups  for menu vew type.
 *
 *  @param[in]    menuViewType        The view type that supports menu view. Please refer to {@link PDFScanMenuViewType PDFScanMenuViewTypeXXX} values and it would be one of these values.
 *
 *  @return    The groups.
 */
- (NSArray <PDFScanMenuItemGroup *> *)getMenuItemGroupsForMenuViewType:(PDFScanMenuViewType)menuViewType;


@end

NS_ASSUME_NONNULL_END
