/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

/**@brief The style of the toolbar*/
typedef NS_ENUM(NSInteger, FSToolbarStyle) {
    ///The default style
    FSToolbarStyleDefault = 0,
    ///At the top of the bar
    FSToolbarStyleTop = 1,
    ///At the bottom of the bar
    FSToolbarStyleBottom = 2
};

/**@brief The item is in the bar*/
typedef NS_ENUM(NSInteger, FSItemPosition) {
    ///The item is on the left
    FSItemPositionLeft = 0,
    ///The item is on the top
    FSItemPositionTop = FSItemPositionLeft,
    ///The item is on the center
    FSItemPositionCenter,
    ///The item is on the right
    FSItemPositionRight,
    ///The item is on the bottom
    FSItemPositionBottom = FSItemPositionRight,
};

@interface FSToolbar : UIView
/**@brief The status bar, It may not show*/
@property (nonatomic, strong, readonly) UIView *stateBar;
/**@brief The color of the bar, default is #F4F4F4, #2D343D */
@property (nullable, nonatomic, strong) UIColor *barTintColor;
/**@brief If it is negative, the default is negative based on the default interval set internally */
@property (nonatomic, assign) CGFloat entiretyInterval;
/**@brief Set content bar height , only applies to FSToolbarStyleTop or FSToolbarStyleBottom */
@property (nonatomic, assign) CGFloat contentBarHeight;
/**@brief Is there a dividing line, like uitoolbar*/
@property (nonatomic, assign) BOOL hasDivide;
/**@brief Keys that specify a horizontal or vertical layout constraint between objects, only applies to FSToolbarStyleDefault */
@property (nonatomic, assign) UILayoutConstraintAxis axis;
/**@brief The style of the toolbar*/
@property (nonatomic, assign, readonly) FSToolbarStyle style;
/**@brief Adjust  the content view layout margins*/
@property (nonatomic, assign) UIEdgeInsets subviewContentViewLayoutMargins;

/**@brief Disable the default initialization.*/
+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;

/** @brief Initialization method
 *
 * @param[in]    style         The toolbar style . Please refer to {@link FSToolbarStyle FSToolbarStyleXXX} values and it would be one of these values.
 */
- (instancetype)initWithStyle:(FSToolbarStyle)style;
/** @brief Initialization method
 *
 * @param[in]    frame         The frame.
 * @param[in]    style         The toolbar style . Please refer to {@link FSToolbarStyle FSToolbarStyleXXX} values and it would be one of these values.
 */
- (instancetype)initWithFrame:(CGRect)frame style:(FSToolbarStyle)style NS_DESIGNATED_INITIALIZER;

/** @brief Indicates whether the toolbar contains the  item.
 *
 * @param[in]    item         The item.
 */
- (BOOL)containsItem:(__kindof UIView *)item;
/** @brief Get item for index and in the position ;
 *
 * @param[in]    position        The item is in the bar . Please refer to {@link FSItemPosition FSItemPositionXXX} values and it would be one of these values.
 * @param[in]    index                The index.
 */
- (__kindof UIView *)getItemAtPosition:(FSItemPosition)position index:(NSUInteger)index;
/** @brief Gets the placeholder width of the item  for  position ;
 *
 * @param[in]    position        The item is in the bar . Please refer to {@link FSItemPosition FSItemPositionXXX} values and it would be one of these values.
 */
- (CGFloat)getWidthWithPosition:(FSItemPosition)position;

- (CGFloat)getWidthWithPosition:(FSItemPosition)position layoutIfNeeded:(BOOL)layoutIfNeeded;
/** @brief Get item count  for  position ;
 *
 * @param[in]    position        The item is in the bar . Please refer to {@link FSItemPosition FSItemPositionXXX} values and it would be one of these values.
 */
- (NSUInteger)getItemCountWithPosition:(FSItemPosition)position;
- (void)addEvenItem:(__kindof UIView *)item;
- (void)addItem:(__kindof UIView *)item atPosition:(FSItemPosition)position;
- (void)addItem:(__kindof UIView *)item spacing:(CGFloat)interval atPosition:(FSItemPosition)position;
- (void)removeItem:(__kindof UIView *)item;
- (void)removeItem:(__kindof UIView *)item atPosition:(FSItemPosition)position;
- (void)removeItemAtPosition:(FSItemPosition)position;
- (void)removeItemAtPosition:(FSItemPosition)position index:(NSUInteger)index;
- (void)removeAllItems;


- (__kindof UIView *)getItemForIndex:(NSUInteger)index position:(FSItemPosition)position  API_DEPRECATED("Implement getItemAtPosition:index: instead", ios(2.0, 8.0));
- (CGFloat)getWidthForPosition:(FSItemPosition)position API_DEPRECATED_WITH_REPLACEMENT("getItemAtPosition:index:", ios(2.0, 8.0));
- (NSUInteger)getItemCountForPosition:(FSItemPosition)position API_DEPRECATED_WITH_REPLACEMENT("getItemCountWithPosition:", ios(2.0, 8.0));
- (void)addItem:(__kindof UIView *)item position:(FSItemPosition)position API_DEPRECATED_WITH_REPLACEMENT("addItem:atPosition:", ios(2.0, 8.0));
- (void)addItem:(__kindof UIView *)item interval:(CGFloat)interval position:(FSItemPosition)position API_DEPRECATED_WITH_REPLACEMENT("addItem:spacing:atPosition:", ios(2.0, 8.0));
- (void)removeItem:(__kindof UIView *)item position:(FSItemPosition)position API_DEPRECATED_WITH_REPLACEMENT("removeItem:atPosition:", ios(2.0, 8.0));
- (void)removeItemByIndex:(NSUInteger)index position:(FSItemPosition)position  API_DEPRECATED("Implement removeItemAtPosition:index: instead", ios(2.0, 8.0));
- (void)removeItemsByPosition:(FSItemPosition)position API_DEPRECATED_WITH_REPLACEMENT("removeItemAtPosition:", ios(2.0, 8.0));
@end

NS_ASSUME_NONNULL_END
