/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPagerView.h"
#import "FSConst.h"
#import "FSMacro.h"
#import "FSPreferences.h"
#import "UIScrollView+FSAdd.h"
#import "UIView+FSAdd.h"
#import "UIColor+FSAdd.h"
#import "Masonry+FSAdd.h"

 NSRange visibleRangWithOffset(CGFloat offset,CGFloat width, NSInteger maxIndex){
    if (width <= 0) {
        return NSMakeRange(0, 0);
    }
    NSInteger startIndex = offset/width;
    NSInteger endIndex = ceil((offset + width)/width);
    
    if (startIndex < 0) {
        startIndex = 0;
    } else if (startIndex > maxIndex) {
        startIndex = maxIndex;
    }
    
    if (endIndex > maxIndex) {
        endIndex = maxIndex;
    }
    
    NSUInteger length = endIndex - startIndex;
    if (length > 5) {
        length = 5;
    }
    return NSMakeRange(startIndex, length);
}

@interface FSPagerView ()<UIScrollViewDelegate> {
    BOOL        _needLayoutContent;
    CGFloat     _preOffsetX;
    BOOL        _isTapScrollMoved;
    BOOL        _isLayoutScrollMoved;
    // private
    struct {
        unsigned int viewDidScroll: 1;
        unsigned int viewWillBeginScrolling: 1;
        unsigned int viewDidEndScrolling: 1;
        unsigned int transitionFromIndexToIndexProgress :1;
    }_delegateFlags;
}
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, assign) NSInteger countOfPagerItems;
@property (nonatomic, assign) NSInteger curIndex;

@property (nonatomic, assign) NSRange visibleRange;
@property (nonatomic, assign) NSRange prefetchRange;

@property (nonatomic, strong) NSMapTable *addedSubViews;

@end

@implementation FSPagerView

#pragma mark - life cycle

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = [UIColor clearColor];
        [self initialize];
    }
    return self;
}

- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    if (self = [super initWithCoder:aDecoder]) {
        self.backgroundColor = [UIColor clearColor];
        [self initialize];
    }
    return self;
}

- (void)initialize{
    UIScrollView *contentView = [[UIScrollView alloc] init];
    contentView.frame = self.frame;
    contentView.delegate = self;
    contentView.pagingEnabled = YES;
    contentView.showsHorizontalScrollIndicator = NO;
    contentView.showsVerticalScrollIndicator = NO;
    contentView.bounces = NO;
    [self addSubview:contentView];
    
    if (@available(iOS 11.0, *)) {
        contentView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    }
    
    _scrollView = contentView;
    
    _scrollEnabled = YES;
    _curIndex = -1;
    _preOffsetX = 0;
    _addedSubViews = [NSMapTable mapTableWithKeyOptions:NSMapTableCopyIn valueOptions:NSMapTableWeakMemory];
    
    [self addScrollViewObservers];
}

- (void)dealloc {
    [self removeScrollViewObservers];
}


#pragma mark - Observer

- (void)addScrollViewObservers {
    [self addObserver:self forKeyPath:@keypath(self, scrollView.frame) options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld context:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary<NSKeyValueChangeKey,id> *)change context:(void *)context {
    if ([keyPath isEqualToString:@keypath(self, scrollView.frame)]) {
        CGRect newFrame = [[change objectForKey:NSKeyValueChangeNewKey]CGRectValue];
        CGRect oldFrame = [[change objectForKey:NSKeyValueChangeOldKey]CGRectValue];
        BOOL needLayoutContent = !CGRectEqualToRect(newFrame, oldFrame);
        if (needLayoutContent) {
            [self setNeedLayout];
        }
    }
}

- (void)removeScrollViewObservers {
    [self removeObserver:self forKeyPath:@keypath(self, scrollView.frame) context:nil];
}

- (NSInteger)countOfPagerViews {
    return [_dataSource numberOfViewsInPagerView:self];
}

- (void)setContentInset:(UIEdgeInsets)contentInset {
    _contentInset = contentInset;
    [self setNeedsLayout];
}

- (void)setDelegate:(id<FSPagerViewDelegate>)delegate {
    _delegate = delegate;
    
    _delegateFlags.viewDidScroll = [delegate respondsToSelector:@selector(pagerViewDidScroll:)];
    _delegateFlags.viewWillBeginScrolling = [delegate respondsToSelector:@selector(pagerViewWillBeginScrolling:animate:)];
    _delegateFlags.viewDidEndScrolling = [delegate respondsToSelector:@selector(pagerViewDidEndScrolling:animate:)];
    _delegateFlags.transitionFromIndexToIndexProgress = [delegate respondsToSelector:@selector(pagerView:transitionFromIndex:toIndex:progress:)];
}

#pragma mark - public

- (void)setScrollEnabled:(BOOL)scrollEnabled{
    _scrollEnabled = scrollEnabled;
    self.scrollView.scrollEnabled = scrollEnabled;
}

- (void)reloadData {
    [self setNeedLayout];
    _countOfPagerItems = [self countOfPagerViews];
    if (_curIndex >= _countOfPagerItems) {
        _curIndex = _countOfPagerItems - 1;
    }
    CGFloat contentWidth = CGRectGetWidth(_scrollView.frame);
    _scrollView.contentSize = CGSizeMake(_countOfPagerItems * contentWidth, 0);
    _scrollView.contentOffset = CGPointMake(MIN(_curIndex, _countOfPagerItems-1)*contentWidth, _scrollView.contentOffset.y);
    
    [self scrollViewDidScroll:_scrollView];
}

- (void)scrollToViewAtIndex:(NSInteger)index animate:(BOOL)animate {
    _isTapScrollMoved = YES;
    _curIndex = index;
    [_scrollView setContentOffset:CGPointMake(index * CGRectGetWidth(_scrollView.frame),0) animated:animate];
    if (!animate) {
        [self scrollViewDidEndScrollingAnimation:_scrollView];
    }
}

- (UIView *)viewForIndex:(NSInteger)idx {
    return [_dataSource pagerView:self viewForIndex:idx];
}

#pragma mark - layout content

- (void)layoutSubviews {
    [super layoutSubviews];
    _isLayoutScrollMoved = YES;
    _scrollView.frame = UIEdgeInsetsInsetRect(self.bounds,_contentInset);
    _isLayoutScrollMoved = NO;
}

- (void)setNeedLayout {
    if (_countOfPagerItems <= 0) {
        _countOfPagerItems = [_dataSource numberOfViewsInPagerView:self];
    }
    _needLayoutContent = YES;
    if (_curIndex >= _countOfPagerItems) {
        _curIndex = _countOfPagerItems - 1;
    }
    
    CGFloat contentWidth = CGRectGetWidth(_scrollView.frame);
    _scrollView.contentSize = CGSizeMake(_countOfPagerItems * contentWidth, 0);
    _scrollView.contentOffset = CGPointMake(MAX(_curIndex, 0)*contentWidth, _scrollView.contentOffset.y);
    
    [self layoutIfNeed];
}

- (void)layoutIfNeed {
    if (CGRectIsEmpty(_scrollView.frame)) {
        return;
    }
    _countOfPagerItems = [self countOfPagerViews];

    CGFloat offsetX = _scrollView.contentOffset.x;
    NSRange visibleRange = visibleRangWithOffset(offsetX, CGRectGetWidth(_scrollView.frame), _countOfPagerItems);
    if (NSEqualRanges(_visibleRange, visibleRange) && !_needLayoutContent) {
        return;
    }
    _visibleRange = visibleRange;
    _needLayoutContent = NO;
    
    for (UIView *view in [[[_addedSubViews objectEnumerator] allObjects] copy]) {
        [view removeFromSuperview];
        [_addedSubViews removeObjectForKey:[NSString stringWithFormat:@"%p",view]];
    }
    for (NSInteger idx = visibleRange.location ; idx < visibleRange.location + visibleRange.length; ++idx) {
        UIView *view = [_dataSource pagerView:self viewForIndex:idx];
        CGRect frame = CGRectMake(idx * CGRectGetWidth(_scrollView.frame), 0, CGRectGetWidth(_scrollView.frame), CGRectGetHeight(_scrollView.frame));
        view.frame = frame;
        [_scrollView addSubview:view];
        [_addedSubViews setObject:view forKey:[NSString stringWithFormat:@"%p",view]];
    }
    
}

#pragma mark - UIScrollViewDelegate

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    if (!scrollView.superview) {
        return;
    }
    if ([self progressCaculateEnable]) {
        CGFloat offsetX = scrollView.contentOffset.x;
        int direction = offsetX >= _preOffsetX ? 0 : 1;

        if (CGRectIsEmpty(_scrollView.frame)) {
            return;
        }
        if (_countOfPagerItems <= 0) {
            _curIndex = -1;
            return;
        }
        CGFloat width = CGRectGetWidth(_scrollView.frame);
        CGFloat floadIndex = offsetX/width;
        NSInteger floorIndex = floor(floadIndex);
        if (floorIndex < 0 || floorIndex >= _countOfPagerItems || floadIndex > _countOfPagerItems-1) {
            return;
        }
        
        CGFloat progress = offsetX/width-floorIndex;
        NSInteger fromIndex = 0, toIndex = 0;
        if (direction == 0) {
            fromIndex = floorIndex;
            toIndex = MIN(_countOfPagerItems -1, fromIndex + 1);
            if (fromIndex == toIndex && toIndex == _countOfPagerItems-1) {
                fromIndex = _countOfPagerItems-2;
                progress = 1.0;
            }else{
                if (floadIndex == floorIndex) {
                    fromIndex = floorIndex-1;
                    toIndex = MIN(_countOfPagerItems -1, fromIndex + 1);
                    progress = 1.0;
                }
            }
        }else {
            toIndex = floorIndex;
            fromIndex = MIN(_countOfPagerItems-1, toIndex +1);
            progress = 1.0 - progress;
        }
        
        if (_delegateFlags.transitionFromIndexToIndexProgress) {
            [_delegate pagerView:self transitionFromIndex:fromIndex toIndex:toIndex progress:progress];
        }
    }
    [self layoutIfNeed];
}

- (BOOL)progressCaculateEnable {
    return _delegateFlags.transitionFromIndexToIndexProgress && !_isTapScrollMoved && !_isLayoutScrollMoved;
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView{
    _preOffsetX = scrollView.contentOffset.x;
}

- (void)scrollViewWillScrollToView:(UIScrollView *)scrollView animate:(BOOL)animate {
    _preOffsetX = scrollView.contentOffset.x;
}

- (void)scrollViewDidEndScrollingAnimation:(UIScrollView *)scrollView{
    _isTapScrollMoved = NO;
}

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate{
    _isTapScrollMoved = NO;
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView{
    _curIndex = scrollView.fs_offsetX/scrollView.fs_width;
}

@end

@interface FSTabPagerItemCell : UICollectionViewCell
@property (nonatomic, weak) UIView *item;
@property (nonatomic, assign) BOOL itemSelected;
@property (nonatomic, assign) FSTabPagerBarItemContentMode itemContentMode;
@end


@interface FSTabPagerBar ()<UICollectionViewDelegateFlowLayout, UICollectionViewDataSource> {
    struct {
        unsigned int widthForItemAtIndex :1;
        unsigned int didSelectItemAtIndex :1;
        unsigned int transitionFromItemToItemAnimated :1;
    }_delegateFlags;
}

@property (nonatomic, weak) UICollectionView *collectionView;
@property (nonatomic, strong) UIView *progressView;
@property (nonatomic, strong) UIView *bottomLineView;
@property (nonatomic, assign) NSInteger countOfItems;

@property (nonatomic, assign) NSInteger curIndex;
@property (nonatomic, strong) NSMutableDictionary<NSNumber *, __kindof UIView *> *items;
@end

@implementation FSTabPagerBar

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = [UIColor clearColor];
        [self initialize];
    }
    return self;
}

- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    if (self = [super initWithCoder:aDecoder]) {
        self.backgroundColor = [UIColor clearColor];
        [self initialize];
    }
    return self;
}

- (void)initialize{
    [self addFixAutoAdjustInsetScrollView];
    [self addCollectionView];
    [self addUnderLineView];
    
    _itemEdging = 3;
    _itemWidth = 0;
    _items = @{}.mutableCopy;
}

- (void)addFixAutoAdjustInsetScrollView {
    UIView *view = [[UIView alloc]init];
    [self addSubview:view];
}

- (void)addCollectionView {
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.scrollDirection = UICollectionViewScrollDirectionHorizontal;
    UICollectionView *collectionView = [[UICollectionView alloc]initWithFrame:UIEdgeInsetsInsetRect(self.bounds, _contentInset) collectionViewLayout:layout];
    collectionView.backgroundColor = [UIColor fs_clearColor];
    collectionView.showsHorizontalScrollIndicator = NO;
    collectionView.showsVerticalScrollIndicator = NO;
    if ([collectionView respondsToSelector:@selector(setPrefetchingEnabled:)]) {
        collectionView.prefetchingEnabled = NO;
    }
    collectionView.delegate = self;
    collectionView.dataSource = self;
    [self addSubview:collectionView];
    _collectionView = collectionView;
    
    Class cls = [FSTabPagerItemCell class];
    [collectionView registerClass:cls forCellWithReuseIdentifier:NSStringFromClass(cls)];
}

- (void)addUnderLineView {
    
    UIView *bottomLineView = [[UIView alloc] init];
    bottomLineView.backgroundColor = self.fs_dividerColor_P1;
    [_collectionView addSubview:bottomLineView];
    self.bottomLineView = bottomLineView;
   
    UIView *progressView = [[UIView alloc] init];
    progressView.fs_height = DIVIDE_PROGRESS_HEIGHT;
    [_collectionView addSubview:progressView];
    _progressView.layer.zPosition = 0.01;
    _progressView = progressView;
    _progressView.hidden = YES;
}

#pragma mark - getter setter
- (void)setProgressColor:(UIColor *)progressColor{
    _progressView.backgroundColor = progressColor;
}

- (void)setDelegate:(id<FSTabPagerBarDelegate>)delegate {
    _delegate = delegate;
    _delegateFlags.widthForItemAtIndex = [delegate respondsToSelector:@selector(pagerTabBar:widthForItemAtIndex:)];
    _delegateFlags.didSelectItemAtIndex = [delegate respondsToSelector:@selector(pagerTabBar:didSelectItemAtIndex:)];
    _delegateFlags.transitionFromItemToItemAnimated = [delegate respondsToSelector:@selector(pagerTabBar:transitionFromItem:toItem:animated:)];
}


#pragma mark - public

- (void)reloadData {
    _countOfItems = [_dataSource numberOfItemsInPagerTabBar:self];
    if (_curIndex >= _countOfItems) {
        _curIndex = _countOfItems - 1;
    }
    for (UIView *item in _items.allValues) [item removeFromSuperview];
    _items = @{}.mutableCopy;
    
    [_collectionView reloadData];
    [self scrollToItemAtIndex:_curIndex animated:NO];
}
    
- (FSTabPagerItemCell *)cellForIndex:(NSInteger)index{
    if (index >= _countOfItems) {
        return nil;
    }
    return (FSTabPagerItemCell *)[_collectionView cellForItemAtIndexPath:[NSIndexPath indexPathForItem:index inSection:0]];
}

- (__kindof UIView *)itemForIndex:(NSInteger)index{
    return [self cellForIndex:index].item;
}

- (CGRect)cellFrameWithIndex:(NSInteger)index{
    if (index < 0) {
        return CGRectZero;
    }
    if (index >= _countOfItems) {
        return CGRectZero;
    }
    UICollectionViewLayoutAttributes * cellAttrs = [_collectionView layoutAttributesForItemAtIndexPath:[NSIndexPath indexPathForItem:index inSection:0]];
    if (!cellAttrs) {
        return CGRectZero;
    }
    return cellAttrs.frame;
}

- (void)scrollToItemFromIndex:(NSInteger)fromIndex toIndex:(NSInteger)toIndex progress:(CGFloat)progress{
    if (toIndex < _countOfItems && toIndex >= 0 && fromIndex < _countOfItems && fromIndex >= 0) {
        UIView *progressView = self.progressView;
        if (self.countOfItems == 0) {
            return;
        }
        
        if (progressView.isHidden) {
            progressView.hidden = NO;
        }
        
        CGRect fromCellFrame = [self cellFrameWithIndex:fromIndex];
        CGRect toCellFrame = [self cellFrameWithIndex:toIndex];
        if (fromIndex > toIndex) {
            progressView.fs_width = CGRectGetWidth(toCellFrame) + (progress * CGRectGetWidth(toCellFrame));
            progressView.fs_right = CGRectGetMaxX(fromCellFrame);
        }else{
            progressView.fs_left = CGRectGetMinX(fromCellFrame);
            progressView.fs_width = CGRectGetWidth(toCellFrame) + (progress * CGRectGetWidth(toCellFrame));
        }
        if (progress == 1) {
            [self scrollToItemFromIndex:fromIndex toIndex:toIndex animate:NO];
            if (_delegateFlags.transitionFromItemToItemAnimated) {
                [_delegate pagerTabBar:self transitionFromItem:[self itemForIndex:fromIndex] toItem:[self itemForIndex:toIndex] animated:YES];
            }
        }
    }
}

- (void)scrollToItemFromIndex:(NSInteger)fromIndex toIndex:(NSInteger)toIndex animate:(BOOL)animate {
    if (toIndex < _countOfItems && toIndex >= 0 && fromIndex < _countOfItems && fromIndex >= 0) {
        [self scrollToItemAtIndex:toIndex animated:animate];
    }
}

- (void)scrollToItemAtIndex:(NSInteger)index animated:(BOOL)animated{
    if (self.countOfItems == 0 ) {
        return;
    }
    [[self cellForIndex:_curIndex] setItemSelected:NO];
    UIView *item = [self itemForIndex:_curIndex];
    if ([item isKindOfClass:[UIImageView class]]) {
        UIImageView *imageView = (UIImageView *)item;
        [imageView setHighlighted:NO];
    }
    
    _curIndex = index;
    [_collectionView scrollToItemAtIndexPath:[NSIndexPath indexPathForItem:index inSection:0] atScrollPosition:UICollectionViewScrollPositionCenteredHorizontally animated:animated];
    
    UIView *progressView = self.progressView;
    if (progressView.isHidden) {
        progressView.hidden = NO;
    }
    CGRect toCellFrame = [self cellFrameWithIndex:index];
    progressView.fs_width = CGRectGetWidth(toCellFrame);
    progressView.fs_bottom = CGRectGetHeight(toCellFrame);
    
    if (animated) {
        [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
            progressView.fs_left = CGRectGetMinX(toCellFrame);
            } completion:^(BOOL finished) {
                if (finished) {
                    progressView.fs_width = CGRectGetWidth(toCellFrame);
                }
        }];
    }else{
        progressView.fs_left = CGRectGetMinX(toCellFrame);
    }
    [[self cellForIndex:_curIndex] setItemSelected:YES];
    item = [self itemForIndex:_curIndex];
    if ([item isKindOfClass:[UIImageView class]]) {
        UIImageView *imageView = (UIImageView *)item;
        [imageView setHighlighted:YES];
    }
}

#pragma mark - UICollectionViewDataSource

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    _countOfItems = [_dataSource numberOfItemsInPagerTabBar:self];
    return _countOfItems;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    FSTabPagerItemCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:NSStringFromClass([FSTabPagerItemCell class]) forIndexPath:indexPath];
    cell.itemContentMode = self.itemContentMode;
    UIView *item = _items[@(indexPath.item)];
    if (!item) {
        item = [_dataSource pagerTabBar:self cellForItemAtIndex:indexPath.item];
        _items[@(indexPath.item)] = item ? item : [UIView new];
    }
    cell.item = item;
    if (indexPath.item == _curIndex){
        [cell setItemSelected:YES];
        UIView *progressView = self.progressView;
        if (progressView.isHidden) {
            progressView.hidden = NO;
        }
    }else{
        [cell setItemSelected:NO];
    }
    return cell;
}

#pragma mark - UICollectionViewDelegateFlowLayout

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout minimumLineSpacingForSectionAtIndex:(NSInteger)section{
    return self.itemEdging;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    [self scrollToItemFromIndex:_curIndex toIndex:indexPath.item animate:YES];
    if (_delegateFlags.didSelectItemAtIndex) {
        [_delegate pagerTabBar:self didSelectItemAtIndex:_curIndex];
    }
}

- (void)setItemWidth:(CGFloat)itemWidth{
    _itemWidth = itemWidth;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    CGSize cellSize = CGSizeZero;
    if (self.itemWidth > 0) {
        cellSize = CGSizeMake(self.itemWidth, CGRectGetHeight(_collectionView.frame));
    }else if(_delegateFlags.widthForItemAtIndex){
        CGFloat width = [_delegate pagerTabBar:self widthForItemAtIndex:indexPath.item];
        cellSize = CGSizeMake(width, CGRectGetHeight(_collectionView.frame));
    }else {
        NSAssert(NO, @"you must return cell width!");
    }
    
    if (_progressView) {
        _progressView.fs_width = cellSize.width;
    }
    
    return cellSize;
}

#pragma mark - transition cell

-(void)layoutSubviews {
    [super layoutSubviews];

    CGRect frame = UIEdgeInsetsInsetRect(self.bounds, _contentInset);
    _collectionView.frame = frame;
    self.bottomLineView.frame = CGRectMake(0, CGRectGetMaxY(frame)-2, frame.size.width, 2);

    [_collectionView reloadData];
    
    CGRect cellFrame = [self cellFrameWithIndex:_curIndex];
    _progressView.fs_width = CGRectGetWidth(cellFrame);
    _progressView.fs_bottom = CGRectGetHeight(cellFrame);
    _progressView.fs_left = CGRectGetMinX(cellFrame);
}

@end

@implementation FSTabPagerItemCell
    
- (void)setItem:(UIView *)item{
    if (_item == item) {
        return;
    }
    if (_item.superview == self.contentView) {
        [_item removeFromSuperview];
    }
    self.accessibilityLabel = item.accessibilityLabel;
    self.accessibilityTraits = item.accessibilityTraits;
    _item = item;
    [self.contentView addSubview:item];
    
    if (self.itemContentMode == FSTabPagerBarItemContentModeScaleToFill) {
        [item mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.top.fs_equalToSuperview().priorityHigh();
        }];
    }else{
        [item mas_makeConstraints:^(MASConstraintMaker *make) {
            make.center.fs_equalToSuperview();
            make.size.mas_equalTo(item.fs_size).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
   
}

- (void)setItemSelected:(BOOL)selected{
    _itemSelected = selected;
    if ([self.item isKindOfClass:[UIControl class]]) [(UIControl *)self.item setSelected:selected];
    [self setSelected:selected];
}

- (void)setHighlighted:(BOOL)highlighted{
    [super setHighlighted:highlighted];
    if ([self.item isKindOfClass:[UIControl class]]) [(UIControl *)self.item setHighlighted:highlighted];
}

- (UIView *)hitTest:(CGPoint)point withEvent:(UIEvent *)event{
    UIView *view = [super hitTest:point withEvent:event];
    if (view == self.item) {
        return self;
    }
    return view;
}

- (BOOL)isAccessibilityElement
{
    return YES;
}
@end
