/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSSearchViewController.h"
#import "Masonry+FSAdd.h"
#import "UIScrollView+FSAdd.h"
#import "FSItem.h"
#import "FSPreferences.h"

typedef NS_ENUM(NSUInteger, FSSearchState){
    FSSearchStateNone = 0,
    FSSearchStateBefore,
    FSSearchStateInputing,
    FSSearchStateBegin,
    FSSearchStateEnd,
};

@interface FSSearchViewController ()
@property (nonatomic, strong) UIView *searchBarContentView;
@property (nonatomic, strong) FSItem *cancelItem;
@property (nonatomic, strong) UISearchBar *searchBar;
@property (nonatomic, assign) FSSearchState searchState;
@property (nonatomic)  NSTimer *beginSearchTimer;
@property (nonatomic, strong) UITableView *searchTableView;
@end

@implementation FSSearchViewController

#pragma mark - life cycle

- (void)viewDidLoad{
    [super viewDidLoad];
    
#if _MAC_CATALYST_
    [self setSearchBarHidden:NO];
#endif
}

#pragma mark - rewrite

- (void)initialize{
    [super initialize];
    _searchInputInterval = CHANGE_VALUE_BLOCK_INTERVAL;
}

- (void)setEditing:(BOOL)editing{
    [super setEditing:editing];
    if (!self.searchBarContentView.hidden) [self setSearchBarHidden:YES];
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    if (!self.keywords.length) return;
    NSArray *highlightLabels = [self keywordsHighlightLabels:cell];
    for (UILabel *label in highlightLabels) {
        if (!label.text) continue;
        NSRange match = [label.text.lowercaseString rangeOfString:self.keywords.lowercaseString];
        [BindThemeKeySel(label, setAttributedText:)
         fs_updateThemeCompleted:^(UILabel * _Nonnull target, SEL  _Nonnull sel) {
            NSMutableAttributedString *contentStr = [[NSMutableAttributedString alloc]initWithString:target.text];
            [contentStr addAttribute:NSForegroundColorAttributeName value:target.fs_primaryColor range:match];
            [target setAttributedText:contentStr];
        }];
    }
}

#pragma mark - Lazy loading
- (UISearchBar *)searchBar{
    if (!_searchBar) {
        _searchBar = [UISearchBar new];
        _searchBar.searchBarStyle = UISearchBarStyleMinimal;
        _searchBar.delegate = self;
        
        UITextField *searchTextField = nil;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        if (@available(iOS 13.0, *)) {
            searchTextField = _searchBar.searchTextField;
        }
        else
#endif
        {
            // Fallback on earlier versions
            searchTextField = [_searchBar valueForKey:@"_searchField"];
        }
        if (searchTextField) {
            searchTextField.layer.cornerRadius = 18;
            searchTextField.layer.masksToBounds = YES;
        }
    }
    return _searchBar;
}

- (FSItem *)cancelItem{
    if (!_cancelItem) {
        FSItem *cancelItem = [FSItem createItemWithTitle:_cancelTitle];
        [BindThemeKeySel(cancelItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem  *_Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        [cancelItem addTarget:self action:@selector(clickedSearchCancelItem) forControlEvents:UIControlEventTouchUpInside];
        _cancelItem = cancelItem;
    }
    return _cancelItem;
}

- (UIView *)searchBarContentView{
    if (!_searchBarContentView) {
        FSToolbar *searchBarTopbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        searchBarTopbar.contentBarHeight = TOOL_BAR_HEIGHT_NORMAL;
        if (self.shouldBindTemeForStatusBar)
            StateBarThemeKeyPath(searchBarTopbar.stateBar, backgroundColor);
        [BindThemeKeySel(searchBarTopbar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
            StateBarThemeKeyPath(target, backgroundColor);
        }];
        searchBarTopbar.backgroundColor = self.fs_backgroundColor_B1;
        searchBarTopbar.hidden = YES;
        
        [searchBarTopbar addSubview:self.searchBar];

        [self.searchBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.fs_equalToSuperview();
        }];
        
        _searchBarContentView = searchBarTopbar;
    }
    return _searchBarContentView;
}

- (NSPredicate *)searchPredicate{
    return [NSPredicate predicateWithFormat:@"SELF CONTAINS[c] %@", self.keywords];
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    if (self.keywords.length) {
        @try {
            if (self.searchResultSoruce.count > indexPath.row) {
                return self.searchResultSoruce[indexPath.row];
            }else{
                return nil;
            }
        } @catch (NSException *exception) {
            return nil;
        }
        return nil;
    }
    return [super dataForAtIndexPath:indexPath];
}

#pragma mark - getter-setter
- (void)setCancelTitle:(NSString *)cancelTitle{
    _cancelTitle = cancelTitle;
    [self.cancelItem setTitle:cancelTitle forState:UIControlStateNormal];
}

- (NSString *)keywords{
    return self.searchBar.text;
}

- (void)setKeywords:(NSString *)keywords{
    self.searchBar.text = keywords;
    [self searchBar:self.searchBar textDidChange:keywords];
}

- (NSString *)placeholder{
    return self.searchBar.placeholder;
}

- (void)setPlaceholder:(NSString *)placeholder{
    self.searchBar.placeholder = placeholder;
}

#pragma mark - public
- (BOOL)evaluateWithObject:(id)object{
    @try {
        return [self.searchPredicate evaluateWithObject:object];
    } @catch (NSException *exception) {
        return NO;
    }
    return NO;
}

- (void)enteredSearchState{
    if (self.shouldShowBottomBar) {
        [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview);
            make.left.right.bottom.fs_equalToSuperview();
        }];
    }
}
- (void)exitedSearchState{
    if (self.shouldShowBottomBar) {
        [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.tableView.superview);
            make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview);
            make.left.right.fs_equalToSuperview();
        }];
    }
}

- (void)clickedSearchCancelItem{
    self.searchBarContentView.backgroundColor = self.fs_backgroundColor_B1;
    [self cancelSearch];
}

- (NSArray<UILabel *> *)keywordsHighlightLabels:(__kindof UITableViewCell *)cell{
    return nil;
}
#pragma mark - private

- (void)cancelSearch{
    self.searchBar.text = @"";
    [self.searchBar resignFirstResponder];
    [self.cancelItem removeFromSuperview];
    [self setSearchBarHidden:YES];
    _enteredSearch = NO;
    _searchState = FSSearchStateNone;
    [self exitedSearchState];
    [self reloadData];
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar{
    
    self.searchBarContentView.backgroundColor = self.fs_backgroundColor_B2;
    
    if (!self.enteredSearch) {
        _enteredSearch = YES;
        [self enteredSearchState];
    }
    _searchState = FSSearchStateBefore;
    if (![self.cancelItem isDescendantOfView:self.searchBarContentView]) {
        [self.searchBarContentView addSubview:self.cancelItem];
    }
    [self setSearchBarHidden:NO];
}

- (void)setSearchBarHidden:(BOOL)hidden{
    if (hidden && DEVICE_MAC_CATALYST) {
        hidden = NO;
    }
    if ([self emptyDataSetDisplayed] && self.searchBarContentView.hidden) return;
    if (!self.isViewLoaded) return;
    if (!_searchBarContentView.superview) {
        [self.view addSubview:self.searchBarContentView];
        [self.searchBarContentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.fs_equalToSuperview();
            make.top.mas_equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.searchBarContentView.superview);
        }];
    }
    self.searchBarContentView.hidden = hidden;
    if (hidden) {
        self.tableView.contentInset = UIEdgeInsetsZero;
    }else {
        self.tableView.contentInset = UIEdgeInsetsMake(TOOL_BAR_HEIGHT_NORMAL, 0, 0, 0);
        if (self.cancelItem.superview && [self.cancelItem isDescendantOfView:_searchBarContentView]) {
            [self.searchBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.top.bottom.fs_equalToSuperview();
            }];
            
            [self.cancelItem mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.mas_equalTo(self.searchBar.mas_right).offset(ITEM_MARGIN_LITTLE_NORMAL);
                make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
                make.centerY.fs_equalToSuperview();
            }];
        }else{
            [self.searchBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.right.left.top.bottom.fs_equalToSuperview();
            }];
        }
    }
}

#pragma mark - UIScrollViewDelegate

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate{
    if ([self.searchBar isFirstResponder] || self.editing) return;
    if (scrollView.fs_offsetY < -ITEM_MARGIN_LITTLE_NORMAL && self.tableView.contentInset.top != TOOL_BAR_HEIGHT_NORMAL) {
        [self setSearchBarHidden:NO];
    }else if(self.tableView.contentInset.top == TOOL_BAR_HEIGHT_NORMAL && scrollView.fs_offsetY > -(TOOL_BAR_HEIGHT_NORMAL - ITEM_MARGIN_LITTLE_NORMAL)){
        [self setSearchBarHidden:YES];
    }
}

- (void)beginSearch{
    self.searchState = FSSearchStateBegin;
    if (self.searchState == FSSearchStateInputing)  return;
    if (self.searchResultSoruce.count) {
        self.searchResultSoruce = nil;
    }
    NSMutableArray *tmp = @[].mutableCopy;
    for (NSArray *data in self.dataSource) {
        for (id object in data) {
            if (self.searchState == FSSearchStateInputing)  break;
            if ([self evaluateWithObject:object]) {
                [tmp addObject:object];
            }
        }
    }
    if (self.searchState == FSSearchStateInputing)  return;
    self.searchResultSoruce = tmp.mutableCopy;
    [self reloadData];
    self.searchState = FSSearchStateEnd;
    
    [_beginSearchTimer invalidate];
    _beginSearchTimer = nil;
}

#pragma mark - UISearchBarDelegate

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar{
    [searchBar resignFirstResponder];
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText{
    _searchState = FSSearchStateInputing;
    if (!_searchInputInterval) {
        [self beginSearch];
        return;
    }
    if (_beginSearchTimer) [_beginSearchTimer invalidate];
    _beginSearchTimer = [NSTimer scheduledTimerWithTimeInterval:_searchInputInterval target:self selector:@selector(beginSearch) userInfo:nil repeats:NO];
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    if (self.keywords.length) {
        return 1;
    }
    return [super numberOfSectionsInTableView:tableView];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    if (self.keywords.length) {
        return self.searchResultSoruce.count;
    }
    return [super tableView:tableView numberOfRowsInSection:section];
}

@end
