/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSTableViewPopoverController.h"
#import "FSPresentedViewControllerAnimator.h"
#import "FSPreferences.h"
#import "Masonry+FSAdd.h"
#import "UIView+FSAdd.h"
#import "FSConst.h"

@interface FSTableViewPopoverController ()<UIPopoverPresentationControllerDelegate>
@property (nonatomic, assign) FSTableViewPopoverShowStyle popoverShowStyle;
@property (nonatomic, weak) FSPresentedViewControllerAnimator *animator;
@end

@implementation FSTableViewPopoverController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
}

- (instancetype)initWithServices:(id<FSViewControllerServices>)services showStyle:(FSTableViewPopoverShowStyle)showStyle groups:(NSArray <FSPopoverItemGroup *> *)groups{
    self = [super initWithServices:services];
    if (self) {
        _popoverShowStyle = showStyle;
        _popoverSize = CGSizeZero;
        _groups = groups;
        [self configureModalPresentationStyle];
    }
    return self;
}

- (void)setGroups:(NSArray<FSPopoverItemGroup *> *)groups{
    _groups = groups;
    [self autoCalculatePopoverHeight];
}

- (void)setPopoverSize:(CGSize)popoverSize{
    _popoverSize = popoverSize;
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = self.animator;
        animator.size = popoverSize;
        animator.maxVerticalHeigh = popoverSize.height;
    }else{
        self.preferredContentSize = popoverSize;
    }
}

- (void)setPopoverWidth:(CGFloat)popoverWidth{
    _popoverWidth = popoverWidth;
    CGFloat height = self.popoverSize.height;
    if (height == 0) {
        height = [self getAutoCalculatePopoverHeight];
    }
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = self.animator;
        animator.size = CGSizeMake(popoverWidth, height);
        animator.maxVerticalHeigh = height;
    }else{
        self.preferredContentSize = CGSizeMake(popoverWidth, height);
    }
}

- (void)autoCalculatePopoverHeight{
    CGSize preferredContentSize = self.preferredContentSize;
    CGFloat height = [self getAutoCalculatePopoverHeight];
    preferredContentSize.height = height;
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = self.animator;
        animator.size = preferredContentSize;
        animator.maxVerticalHeigh = height;
    }else{
        self.preferredContentSize = preferredContentSize;
    }
}

- (CGFloat)getAutoCalculatePopoverHeight{
    CGFloat height = 0;
    for (int i = 0; i < [self numberOfSectionsInTableView:self.tableView]; i ++) {
        height += [self tableView:self.tableView numberOfRowsInSection:i] * PREFERENCE_MENU_CELL_ROW_HEIGHT_NORMAL + [self tableView:self.tableView heightForHeaderInSection:i] + [self tableView:self.tableView heightForFooterInSection:i];
    }
    return height;
}

- (void)setDismissCompletion:(VoidBlock)dismissCompletion{
    [super setDismissCompletion:dismissCompletion];
    self.animator.dismissViewControllerCompletion = dismissCompletion;
}

- (void)setSplitScrrenDismissCompletion:(VoidBlock)splitScrrenDismissCompletion{
    [super setSplitScrrenDismissCompletion:splitScrrenDismissCompletion];
    self.animator.dismissSplitViewControllerCompletion = splitScrrenDismissCompletion;
}

- (void)configureModalPresentationStyle{
    CGFloat height = self.popoverSize.height;
    if (height == 0) {
        height = [self getAutoCalculatePopoverHeight];
    }
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = [[FSPresentedViewControllerAnimator alloc] init];
        animator.size = CGSizeMake(self.popoverSize.width, height);
        animator.presentationStyle = FSModalPresentationBottom;
        animator.maxVerticalHeigh = height;
        animator.dismissViewControllerCompletion = self.dismissCompletion;
        animator.dismissSplitViewControllerCompletion = self.splitScrrenDismissCompletion;
        [self fs_setCustomTransitionAnimator:animator];
        self.animator = animator;
    }else{
        self.tableView.scrollEnabled = NO;
        self.modalPresentationStyle = UIModalPresentationPopover;
        UIPopoverPresentationController *popover = [self popoverPresentationController];
        popover.backgroundColor = self.fs_backgroundColor_B1;
        popover.delegate = self;
        self.preferredContentSize = CGSizeMake(self.popoverSize.width, height);
        popover.permittedArrowDirections = UIPopoverArrowDirectionAny;
    }
}

- (void)makeConstraints{}

- (void)loadView{
    self.view = self.tableView;
}


- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    if (_popoverShowStyle == FSTableViewPopoverShowStyleBubble) {
        [self.view mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.fs_equalToSuperview();
            make.size.mas_equalTo(self.preferredContentSize).priorityHigh();
        }];
    }
}

- (void)loadData{}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller {
    return UIModalPresentationNone;
}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller traitCollection:(UITraitCollection *)traitCollection {
    return UIModalPresentationNone;
}

- (void)popoverPresentationControllerDidDismissPopover:(UIPopoverPresentationController *)popoverPresentationController{
    !self.dismissCompletion ?: self.dismissCompletion();
    !self.splitScrrenDismissCompletion ?: self.splitScrrenDismissCompletion();
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *view = [UIView new];
    view.backgroundColor = tableView.backgroundColor;
    
    UIView *handleView = nil;
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault && section == 0) {
        handleView = [UIView new];
        [handleView fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
        handleView.backgroundColor = self.fs_dividerColor_P2;
        [view addSubview:handleView];
        [handleView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerX.fs_equalToSuperview();
            make.top.fs_equalToSuperview().offset(ITEM_TOP_ICON_MARGIN);
            make.size.mas_equalTo(CGSizeMake(31.f, 4.f));
        }];
    }
    
    FSPopoverItemGroup *group = self.groups[section];
    if (group.title.length) {
        UILabel *titleLabel = [UILabel new];
        titleLabel.font = self.fs_standardWeakFont_WF1;
        titleLabel.backgroundColor = [UIColor clearColor];
        titleLabel.textColor = self.fs_textColor_T2;
        titleLabel.text = group.title;
        
        [view addSubview:titleLabel];
        [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            if (section == 0) {
                if (handleView) {
                    make.top.equalTo(handleView.mas_bottom).offset(ITEM_TOP_ICON_MARGIN);
                }else{
                    make.bottom.fs_equalToSuperview().offset(-ITEM_TOP_ICON_MARGIN);
                }
            }else{
                make.centerY.fs_equalToSuperview();
            }

            make.left.mas_offset(ITEM_MARGIN_NORMAL);
        }];
    }
    return view;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    FSPopoverItemGroup *group = self.groups[section];
    if (group.title.length) {
        if (section == 0) {
            return TOOL_BAR_HEIGHT_NORMAL - ITEM_MEDIUM_SPACE_NORMAL;
        }
        return TOOL_BAR_HEIGHT_NORMAL/2;
    }
    if (section == 0) {
        return ITEM_MARGIN_LITTLE_NORMAL;
    }
    return 0.001;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return self.groups.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return [self.groups[section] items].count;
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    FSPopoverItemGroup *group = self.groups[indexPath.section];
    if (group.items.count > indexPath.row) return group.items[indexPath.row];
    return nil;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSPopoverItem *)item{
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
    if (!cell.installed) {
        cell.leftMargin = POPOVER_LABEL_LEFT_ICON_MARGINT;
        cell.mediumSpace = cell.leftMargin;
    }
    if (!cell.leftView) {
        UIImageView *imageView = [UIImageView new];
        imageView.fs_size = CGSizeMake(40.f, 40.f);
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        cell.leftView = imageView;
    }
    ((UIImageView *)cell.leftView).image = item.image;
    cell.estimatedTitleLabel.text = item.title;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    FSPopoverItem *item = [self dataForAtIndexPath:indexPath];
    [self dismissViewControllerAnimated:YES completion:^{
        !item.action ? : item.action(item, (int)indexPath.row);
        !self.action ? : self.action(item, (int)indexPath.row);
        !self.dismissCompletion ? : self.dismissCompletion();
        !self.splitScrrenDismissCompletion ?: self.splitScrrenDismissCompletion();
    }];
}

@end

@implementation FSPopoverItem

- (instancetype)initWithTitle:(NSString *)title image:(UIImage *)image{
    self = [super init];
    if (self) {
        _image = image;
        _title = title;
    }
    return self;
}
- (instancetype)initWithTitle :(NSString *)title image:(UIImage *)image action:(FSPopoverItemAction)action{
    self = [self initWithTitle:title image:image];
    if (self) {
        _action = [action copy];
    }
    return self;
}

@end

@implementation FSPopoverItemGroup

- (instancetype)initWithTitle:(nullable NSString *)title items:(NSArray <FSPopoverItem *> *)items{
    self = [super init];
    if (self) {
        _items = items;
        _title = title;
    }
    return self;
}

- (instancetype)initWithTitle:(nullable NSString *)title titlesAndImageNames:(NSArray<NSArray<NSString *> *> *)titlesAndImageNames{
    if (titlesAndImageNames.count != 2) return nil;
    NSArray *titles = titlesAndImageNames[0];
    NSArray *images = titlesAndImageNames[1];
    NSAssert(titles.count == images.count, @"Titles is equal to images");
    NSMutableArray *tmp = @[].mutableCopy;
    for (int i = 0 ; i < titles.count ;i++) {
        FSPopoverItem *item = [[FSPopoverItem alloc] initWithTitle:titles[i] image:FSImageNamed(images[i])];
        [tmp addObject:item];
    }
    return [self initWithTitle:title items:tmp.copy];
}

@end

