/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSViewController.h"
#if __has_include(<YZHookHandler/YZHookHandler.h>)
#import <YZHookHandler/YZHookHandler.h>
#else
#import "YZHookHandler.h"
#endif
#import "FSViewControllerServicesImpl.h"
#import "FSPreferences.h"
#import "FSItem.h"
#import "Masonry+FSAdd.h"

@interface FSViewController ()
@property (nonatomic, strong, nullable) FSNavigationBar *navBar;
@property (nonatomic, strong, nullable) FSToolbar *bottomBar;
@property (nonatomic, strong, readwrite) id params;
@end

@implementation FSViewController

+ (instancetype)allocWithZone:(struct _NSZone *)zone {

    FSViewController *viewController = [super allocWithZone:zone];
    
    @weakify(viewController)
    [[viewController yz_hookForSelector:@selector(viewDidLoad)]
     after:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        [viewController makeConstraints];
    }];
    return viewController;
}

- (instancetype)initWithCoder:(NSCoder *)coder{
    self = [super initWithCoder:coder];
    if (self) {
        _services = [[FSViewControllerServicesImpl alloc] init];
        [self initialize];
    }
    return self;
}

- (instancetype)init{
    return [self initWithParams:nil];
}

- (instancetype)initWithParams:(nullable id)params{
    return [self initWithServices:[[FSViewControllerServicesImpl alloc] init] params:params];;
}

- (instancetype)initWithServices:(id<FSViewControllerServices>)services{
    return [self initWithServices:services params:nil];
}

- (instancetype)initWithServices:(id<FSViewControllerServices>)services params:(nullable id)params{
    self = [super init];
    if (self) {
        if (!services) {
            services = [[FSViewControllerServicesImpl alloc] init];
        }
        _services = services;
        self.params = params;
        [self initialize];
    }
    return self;
}

- (void)initialize {
    self.shouldShowNavigationBar = YES;
    self.shouldShowBottomBar = NO;
    self.shouldLoadDataOnViewDidLoad = NO;
    self.shouldBindTemeForStatusBar = YES;
}

- (void)dealloc{
    FSLog(@"%@ dealloc",[self class]);
}

- (void)viewDidLoad {
    
    if (self.shouldLoadDataOnViewDidLoad) [self loadData];
    
    [super viewDidLoad];
    
    self.navigationController.navigationBarHidden = YES;
    self.edgesForExtendedLayout = UIRectEdgeNone;
    if (@available(iOS 11.0, *)) { }else{ self.automaticallyAdjustsScrollViewInsets = NO; }
    self.extendedLayoutIncludesOpaqueBars = YES;
    
    [self setViewBackgroundColor:self.fs_backgroundColor_B1];
 
    if (self.shouldShowNavigationBar) {
        self.navBar = [[FSNavigationBar alloc] initWithShowStyle:(FSNavigationBarShowStyle)self.showStyle];
        if (self.shouldBindTemeForStatusBar) {
            StateBarThemeKeyPath(self.navBar.stateBar, backgroundColor);
            [BindThemeKeySel(self.navBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
                StateBarThemeKeyPath(target, backgroundColor);
            }];
               
            
        }
        self.navBar.delegate = self;
        self.navBar.title = self.title;
        if (_navBarLeftTitle.length) [self setNavBarLeftTitle:_navBarLeftTitle];
        [self.view addSubview:self.navBar.superview ? : self.navBar];
    }
    
    if (self.shouldShowBottomBar) {
        self.bottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        [self.view addSubview:self.bottomBar];
    }
}

- (UIStatusBarStyle)preferredStatusBarStyle{
    return [FSPreferences sharePreferences].preferredStatusBarStyle;
}

- (void)loadData{}

- (void)setTitle:(NSString *)title{
    [super setTitle:title];
    if (!self.shouldShowNavigationBar) return;
    self.navBar.title = title;
}

-(void)setNavBarLeftTitle:(NSString *)navbarLeftTitle{
    _navBarLeftTitle = navbarLeftTitle;
    if (!self.shouldShowNavigationBar || !self.navBar) return;
    [self.navBar.leftItem setImage:nil forState:UIControlStateNormal | UIControlStateHighlighted | UIControlStateDisabled];
    [self.navBar.leftItem setTitle:navbarLeftTitle forState:UIControlStateNormal];
    [self.navBar updateItem];
    self.navBar.leftItem.accessibilityIdentifier = navbarLeftTitle;
    self.navBar.leftItem.accessibilityLabel = navbarLeftTitle;
}

- (void)setNavBarRightTitle:(NSString *)navbarRightTitle{
    _navBarRightTitle = navbarRightTitle;
    [self.navBar.rightItem setTitle:navbarRightTitle forState:UIControlStateNormal];
    self.navBar.rightItem.accessibilityIdentifier = navbarRightTitle;
    self.navBar.rightItem.accessibilityLabel = navbarRightTitle;
}

- (void)makeConstraints{
    if (self.shouldShowNavigationBar) {
        [self.navBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.fs_equalToSuperview();
        }];
    }
    
    if (self.shouldShowBottomBar) {
        [self.bottomBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.fs_equalToSuperview();
        }];
    }
}

- (void)setViewBackgroundColor:(UIColor *)backgroundColor{
    self.view.backgroundColor = backgroundColor;
}

/*
- (void)presentViewController:(UIViewController *)viewControllerToPresent animated:(BOOL)flag completion:(void (^)(void))completion{
    UIViewController *topViewController = [self.services.navigationControllerStack topNavigationController];
    if (topViewController == self.navigationController || topViewController == self) {
        [self.services presentViewController:viewControllerToPresent animated:YES completion:completion];
    }else{
        [super presentViewController:viewControllerToPresent animated:flag completion:completion];
    }
}
 */

- (FSViewControllerShowStyle)showStyle{
    if (self.navigationController && self.navigationController.viewControllers.count > 1 && [self.navigationController.viewControllers containsObject:self]) {
        return FSViewControllerShowStylePush;
    }
    if (self.presentingViewController){
        return FSViewControllerShowStylePresent;
    }
    return FSViewControllerShowStyleUnknow;
}

- (void)dismissViewControllerAnimated:(BOOL)flag completion:(void (^)(void))completion{
    UIViewController *topViewController = [self.services.navigationControllerStack topNavigationController];
    if (topViewController && (topViewController == self.navigationController || topViewController == self)) {
        [self.services dismissViewControllerAnimated:flag completion:completion];
    }else{
        [super dismissViewControllerAnimated:flag completion:completion];
    }
}

#pragma mark -FSNavigationBarDelegate
- (void)navigationBarBackTouch{
    UINavigationController *topViewController = [[self.services navigationControllerStack] topNavigationController];
    BOOL useServices = NO;
    if ([topViewController isKindOfClass:[UINavigationController class]]) {
        useServices = [topViewController.viewControllers containsObject:self];
    }else if ((FSViewController *)topViewController == self){
        useServices = YES;
    }
    if (self.showStyle == FSViewControllerShowStylePush) {
        if (useServices) {
            [self.services popViewControllerAnimated:YES];
        }else{
            [self.navigationController popViewControllerAnimated:YES];
            if ([self.navigationController.viewControllers containsObject:self]) {
                goto dismissViewController;
            }
        }
    }else if(self.showStyle == FSViewControllerShowStylePresent){
        dismissViewController : {
            if (useServices) {
                [self.services dismissViewControllerAnimated:YES completion:self.dismissCompletion];
            }else{
                [self dismissViewControllerAnimated:YES completion:self.dismissCompletion];
            }
        }
    }
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
