/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMessageThrottle.h"

typedef NS_ENUM(NSUInteger, FSPerformMode) {
    FSPerformModeFirstly, //undefine
    FSPerformModeLast,
    FSPerformModeDebounce,
};

#define LOCK(...) dispatch_semaphore_wait(_lock, DISPATCH_TIME_FOREVER); \
__VA_ARGS__; \
dispatch_semaphore_signal(_lock);

@implementation FSMessageThrottle

static NSMutableDictionary *_timers;
dispatch_semaphore_t _lock;
+ (void)initialize{
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _timers = [[NSMutableDictionary alloc] init];
        _lock = dispatch_semaphore_create(1);
    });
}

+ (void)debouncePerformTarget:(id)target
             selector:(SEL)selector
            oncePerDuration:(NSTimeInterval)durationThreshold
taskBlock:(FSThrottleTaskBlock)taskBlock{
    NSString *identity = [NSString stringWithFormat:@"FS_%p_%@",target, NSStringFromSelector(selector)];
    [self debouncePerformIdentity:identity oncePerDuration:durationThreshold taskBlock:taskBlock];
}

+ (void)debouncePerformClass:(Class)cls
             selector:(SEL)selector
            oncePerDuration:(NSTimeInterval)durationThreshold
            taskBlock:(FSThrottleTaskBlock)taskBlock{
    NSString *identity = [NSString stringWithFormat:@"FS_%@_%@",NSStringFromClass(cls), NSStringFromSelector(selector)];
    [self debouncePerformIdentity:identity oncePerDuration:durationThreshold taskBlock:taskBlock];
}

+ (void)debouncePerformIdentity:(NSString *)identity
            oncePerDuration:(NSTimeInterval)durationThreshold
                   taskBlock:(FSThrottleTaskBlock)taskBlock{
    [self messageThrottlePerformIdentity:identity oncePerDuration:durationThreshold usingMode:FSPerformModeDebounce taskBlock:taskBlock];
}

+ (void)messageThrottlePerformIdentity:(NSString *)identity
            oncePerDuration:(NSTimeInterval)durationThreshold
                  usingMode:(FSPerformMode)mode
                   taskBlock:(FSThrottleTaskBlock)taskBlock{
    if (!identity.length && !taskBlock) return;
    NSString *timersKey = [NSString stringWithFormat:@"id_%@_duration_%f_mode_%lu",identity,durationThreshold,(unsigned long)mode];
    LOCK(dispatch_source_t gcdTimer = _timers[timersKey]);
    if (gcdTimer) {
        if (mode == FSPerformModeDebounce) {
            dispatch_cancel(gcdTimer);
        }else if (mode == FSPerformModeLast){
            return;
        }
    }
    dispatch_queue_t queue = dispatch_get_current_queue();
    gcdTimer = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER, 0, 0, queue);
    uint64_t start = 0;
    uint64_t interval = durationThreshold;
    dispatch_source_set_timer(gcdTimer, dispatch_time(DISPATCH_TIME_NOW, start * NSEC_PER_SEC),  interval * NSEC_PER_SEC,0);
    LOCK(_timers[timersKey] = gcdTimer);
    dispatch_source_set_event_handler(gcdTimer, ^{
        taskBlock();
        dispatch_cancel(gcdTimer);
        LOCK(_timers[timersKey] = nil);
    });
    dispatch_resume(gcdTimer);
}
@end
