/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSURIAnnotHandler.h"
#import <MobileCoreServices/MobileCoreServices.h>
#import "AudioVideoModule.h"
#import "FSAnnot+Extensions.h"
#import "FSPermissionUtilty.h"
#import "FSURIModule.h"

@interface FSURIAnnotHandler ()

@end

@implementation FSURIAnnotHandler{
    __weak UIExtensionsManager *_extensionsManager;
}

- (FSAnnotType)getType {
    return FSAnnotScreen;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super initWithUIExtensionsManager:extensionsManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _minPlayerLayerWidthInPage = 0.1;
        _maxPlayerLayerWidthInPage = 1.0;
        _minPlayerLayerHeightInPage = 0.1;
        _maxPlayerLayerHeightInPage = 1.0;
    }
    return self;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionNone;

    if (annot.canModify && [FSPermissionUtilty canAddEditAnnot:self.extensionsManager]) {
        options = FSMenuOptionPlay;
        if ([annot canDeleteWithExternsions:self.extensionsManager] && annot.type == FSAnnotScreen) {
            options |= FSMenuOptionDelete;
        }
    }else{
        options = FSMenuOptionPlay;
    }

    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionNone;
    }

    if (![FSPermissionUtilty canAddEditAnnot:self.extensionsManager]) {
        options = FSMenuOptionPlay;
    }
    return options;
}

- (BOOL)keepAspectRatioForResizingAnnot:(FSAnnot *)annot {
    if (![_extensionsManager annotsPermissionCanModify:annot]) {
        return NO;
    }
    return YES;
}

- (void)play{
    if (self.annot) {
        if (self.annot.type == FSAnnotScreen) {
            FSScreen *screen = [[FSScreen alloc] initWithAnnot:self.annot];
            FSScreenActionType screenActiontype = screen.screenActiontype;
            if (screenActiontype == FSScreenActionTypeAdditionalActionURI) {
                [self openAdditionalActionUrl:screen];
            }
        }
    }
    
    [_extensionsManager setCurrentAnnot:nil];
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:annot.pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:annot.pageIndex];
    if ([Utility isAnnot:self.extensionsManager.currentAnnot uniqueToAnnot:annot]) {
        if (!(pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint])) {
            [self.extensionsManager setCurrentAnnot:nil];
        } else if (self.extensionsManager.shouldShowMenu && !self.extensionsManager.shouldShowPropertyBar) {
            [self showMenu];
        }
    } else {
        if (annot.type == FSAnnotScreen) {
            FSScreen *screen = [[FSScreen alloc] initWithAnnot:annot];
            FSScreenActionType screenActiontype = screen.screenActiontype;
            if (screenActiontype == FSScreenActionTypeAdditionalActionURI) {
                [self openAdditionalActionUrl:screen];
            }
        }
    }
    return YES;
}

- (void)openAdditionalActionUrl:(FSScreen*)screen{
 
    FSAdditionalAction *aAction = [[FSAdditionalAction alloc] initWithAnnot:screen];
    if ([aAction isEmpty]) return;
    FSAction *action = [aAction getAction:FSAdditionalActionTriggerAnnotMouseButtonPressed];
    if ([action isEmpty]) {
        return;
    }
    FSActionType actionType = [action getType];
    if (actionType == FSActionTypeURI) {
        FSURIAction *uriAction = [[FSURIAction alloc] initWithAction:action];
        NSString *uri = [uriAction getURI];
        if ([uri hasPrefix:@"http:"] || [uri hasPrefix:@"https:"]) {
            [self openUrlWithSafari:uri];
        }
        
        if ([Utility isPathOfWindows:uri]) {
            return;
        }
        
        NSString *fullPath = [NSString stringWithFormat:@"%@/%@",[self.pdfViewCtrl.filePath stringByDeletingLastPathComponent],uri];
        BOOL isFullPathExist = [[NSFileManager defaultManager] fileExistsAtPath:fullPath];
        BOOL isPathExist = [[NSFileManager defaultManager] fileExistsAtPath:uri];
        if (isFullPathExist || isPathExist) {
            NSString *filePath = @"";
            if (isFullPathExist) {
                filePath = fullPath;
            }else{
                filePath = uri;
            }
            
            NSString *ext = [fullPath pathExtension];
            if ([ext isEqualToString:@"pdf"]) {
                [self setDocWithPath:fullPath];
            }else{
                id<IModule> module = [self.extensionsManager getModuleByName:Module_Audio_Video];
                AudioVideoModule *audioVideoModule = nil;
                if ([module isKindOfClass:[AudioVideoModule class]]) {
                    audioVideoModule = (AudioVideoModule *)module;
                }

                NSString *mediaType = [self mimeTypeForFileAtPath:filePath];
                if ([mediaType containsString:@"audio"]) {
                    if (audioVideoModule) {
                        [audioVideoModule audioPlayWithFile:[NSURL fileURLWithPath:filePath]];
                    }
                }else if ([mediaType containsString:@"video"]) {
                    if (audioVideoModule) {
                        [audioVideoModule videoPlayWithFile:[NSURL fileURLWithPath:filePath]];
                    }
                }else{
                    if (self.extensionsManager.extFileOpenDelegate && [self.extensionsManager.extFileOpenDelegate respondsToSelector:@selector(openFileAtPath:)]) {
                        [self.extensionsManager.extFileOpenDelegate openFileAtPath:filePath];
                    }
                }
            }
        }else{
            [self openUrlWithSafari:uri];
        }
    }
    
}

- (void)openUrlWithSafari:(NSString*)urlStr{
    NSURL *URL = [NSURL URLWithString:[urlStr stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]]];
    NSString *scheme = URL.scheme;
    if (!scheme || scheme.length <= 0) {
        NSString *urlStr = [URL absoluteString];
        if (urlStr) {
            urlStr = [@"http://" stringByAppendingString:urlStr];
        }
        URL = [NSURL URLWithString:urlStr];
    }
    
    if (!URL) {
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kInvalidLinkUrl") onCtrl:[_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController]];
        return;
    }
    
    BOOL isExit = [[UIApplication sharedApplication] canOpenURL:URL];
    if (isExit) {
        [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kSecurityWarning") message:[NSString stringWithFormat:@"%@%@",FSLocalizedForKey(@"kLinkingToWeb"),[urlStr stringByURLDecode]] actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            if (index == 1) {
                [[UIApplication sharedApplication] openURL:URL options:@{} completionHandler:nil];
            }
        } onCtrl:[_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController]];
        
    } else if (scheme && scheme.length > 0) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kNoAppropriateApplication") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
        [alertController addAction:action];
        [[_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    } else {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInvalidUrl") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
        [alertController addAction:action];
        [[_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    }
}


- (NSString *)mimeTypeForFileAtPath:(NSString *)path
{
    if (![[[NSFileManager alloc] init] fileExistsAtPath:path]) {
        return nil;
    }
    
    NSString *UTI = (__bridge_transfer NSString *)UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (__bridge CFStringRef)[path pathExtension], NULL);
    NSString *MIMEType = (__bridge_transfer NSString *)UTTypeCopyPreferredTagWithClass((__bridge CFStringRef)UTI, kUTTagClassMIMEType);
    
    if (!MIMEType) {
        NSString *mineType = @"";
        NSURL *fileURL = [NSURL fileURLWithPath:path];
        return  [NSString stringWithFormat:@"%@/%@",mineType,fileURL.pathExtension];
    }
    return MIMEType;
}

- (void)setDocWithPath:(NSString *)path {
    if (path == nil) {
        return;
    }
    
    __weak typeof(self) weakSelf = self;
    void (^jumpblock)(BOOL) = ^(BOOL isNeedSave){
        if (self.extensionsManager.isMultiFileMode) {
            BOOL isOpen = [weakSelf.extensionsManager.delegate uiextensionsManager:weakSelf.extensionsManager openNewDocAtPath:path shouldCloseCurrentDoc:NO completionCallBack:^{}];
            return ;
        }
        if (isNeedSave) {
            [weakSelf.extensionsManager saveAndCloseCurrentDoc:^(BOOL success) {
                if (success) {
                    __strong typeof(weakSelf) strongSelf = weakSelf;
                    strongSelf.extensionsManager.isDocModified = NO;
                    [strongSelf.pdfViewCtrl openDoc:path
                                           password:nil
                                         completion:^(FSErrorCode error) {
                                             
                                         }];
                }
            }];
        }else{
            weakSelf.extensionsManager.isDocModified = NO;
            __strong typeof(weakSelf) strongSelf = weakSelf;
            [strongSelf.pdfViewCtrl openDoc:path
                                   password:nil
                                 completion:^(FSErrorCode error) {
                                                    
                                 }];
        }
    };
    
    if (self.extensionsManager.isDocModified && !self.extensionsManager.isMultiFileMode) {
        
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kShouldSaveFileTips") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelaction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            jumpblock(NO);
        }];
        
        UIAlertAction *okaction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            jumpblock(YES);
        }];
        
        [alertController addAction:cancelaction];
        [alertController addAction:okaction];
        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
    }else{
        jumpblock(NO);
    }
}
@end

