/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAnnotationItem.h"
#import "FSPermissionUtilty.h"

@interface FSAnnotationItem ()
{
    @public
    NSMutableArray <FSAnnotationItem *>* _replies;
    NSMutableArray <FSAnnotationItem *>* _allReplies;
}
@property (nonatomic, weak, readwrite) FSAnnotationItem *replyToAnnotItem;
@property (nonatomic, weak, readwrite) FSTopReplyAnnotationItem *topReplyAnnotItem;
@end
@implementation FSAnnotationItem

- (instancetype)initWithAnnot:(FSAnnot *)annot isReply:(BOOL)isReply extensionsManager:(UIExtensionsManager*)extensionsManager{
    self = [super init];
    if (self) {
        _isReply = isReply;
        _annot = annot;
        _replies = @[].mutableCopy;
        _allReplies = @[].mutableCopy;
        _extensionsManager = extensionsManager;
        
        BOOL annotCanDelete = [annot canDeleteWithExternsions:self.extensionsManager];
        
        if (annotCanDelete) {
            annotCanDelete =  [FSPermissionUtilty canAddAnnotToDocument:extensionsManager];
        }

        _annotCanDelete = annotCanDelete;
    }
    return self;
}

- (NSArray *)replies{
    return _replies.copy;
}

- (NSArray *)allReplies{
    return _allReplies.copy;
}

- (BOOL)isTopReply{
    return [self isKindOfClass:[FSTopReplyAnnotationItem class]];
}

- (NSString *)getAnnotImageName{
    return [Utility getAnnotImageNameForAnnot:self.annot];
}

- (void)setLocPageItmes:(NSMutableArray<FSAnnotationItem *> *)locPageItmes{
    if (_locPageItmes == locPageItmes) return;
    _locPageItmes = locPageItmes;
    for (FSAnnotationItem *subItem in _replies) {
        subItem.locPageItmes = locPageItmes;
    }
}

- (void)setFilterLocPageItmes:(NSMutableArray<FSAnnotationItem *> *)filterLocPageItmes{
    if (_filterLocPageItmes == filterLocPageItmes) return;
    _filterLocPageItmes = filterLocPageItmes;
    for (FSAnnotationItem *subItem in _replies) {
        subItem.locPageItmes = filterLocPageItmes;
    }
}

- (void)removeReplyItem:(FSAnnotationItem *)replyItem{
    [_replies removeObject:replyItem];
    NSMutableArray *needRemoveObjects = @[replyItem].mutableCopy;
    [needRemoveObjects addObjectsFromArray:replyItem.allReplies];
    [_allReplies removeObjectsInArray:needRemoveObjects];
    [self.locPageItmes removeObjectsInArray:needRemoveObjects];
    [self.filterLocPageItmes removeObjectsInArray:needRemoveObjects];
    
    FSAnnotationItem *replyToAnnotItem = self.replyToAnnotItem;
    while ([replyToAnnotItem.allReplies containsObject:replyItem]) {
        [replyToAnnotItem->_allReplies removeObject:replyItem];
        if ([replyToAnnotItem.replies containsObject:replyItem]) {
            [replyToAnnotItem->_replies removeObject:replyItem];
        }
        replyToAnnotItem = replyToAnnotItem.replyToAnnotItem;
    }
}

- (void)addReplyItem:(FSAnnotationItem *)replyItem{
    if (![_replies containsObject:replyItem]) {
        replyItem.filterLocPageItmes = self.filterLocPageItmes;
        replyItem.locPageItmes = self.locPageItmes;
        
        void (^insertReplyItemToLocPageWithLastItem)(FSAnnotationItem *_replyItem, FSAnnotationItem *lastReplyItem) = ^(FSAnnotationItem *_replyItem, FSAnnotationItem *lastReplyItem){
            if (lastReplyItem) {
                NSUInteger index = [self.locPageItmes indexOfObject:lastReplyItem];
                if (![self.locPageItmes containsObject:_replyItem]) {
                    if (index != NSNotFound) {
                        [self.locPageItmes insertObject:_replyItem atIndex:index+1];
                    }else{
                        [self.locPageItmes addObject:_replyItem];
                    }
                }
                
                index = [self.filterLocPageItmes indexOfObject:lastReplyItem];
                if (![self.filterLocPageItmes containsObject:_replyItem]) {
                    if (index != NSNotFound) {
                        [self.filterLocPageItmes insertObject:_replyItem atIndex:index+1];
                    }else{
                        [self.filterLocPageItmes addObject:_replyItem];
                    }
                }
            }else{
                if (![self.locPageItmes containsObject:_replyItem]) {
                    [self.locPageItmes addObject:_replyItem];
                }
                if (![self.filterLocPageItmes containsObject:_replyItem]) {
                    [self.filterLocPageItmes addObject:_replyItem];
                }
            }
        };
        
        FSAnnotationItem *lastReplyItem = [_replies lastObject];
        if ((!self.isReply && replyItem.isTopReply) ||
            (self.isTopReply && ((FSTopReplyAnnotationItem *)self).unfold) ||
            self.topReplyAnnotItem.unfold) {
            if ((!self.isReply && replyItem.isTopReply && lastReplyItem.isTopReply)) {
                if (((FSTopReplyAnnotationItem *)lastReplyItem).unfold ) {
                    lastReplyItem = [lastReplyItem.allReplies lastObject];
                }
            }
            insertReplyItemToLocPageWithLastItem(replyItem, lastReplyItem ? lastReplyItem : self);
        }
        
        [_replies addObject:replyItem];
        replyItem.replyToAnnotItem = self;
        if (!replyItem.isTopReply) {
            replyItem.topReplyAnnotItem = self.topReplyAnnotItem;
        }
        [self updateAllRepliesWithReplyItem:replyItem];
    }
}

- (void)updateAllRepliesWithReplyItem:(FSAnnotationItem *)replyItem{
    if (self.replyToAnnotItem) {
        [self.replyToAnnotItem updateAllRepliesWithReplyItem:replyItem];
    }
    if (![_allReplies containsObject:replyItem]) {
        if (self.isReply) {
            if (replyItem.replyToAnnotItem == self) {
                [_allReplies addObject:replyItem];
            }else{
                FSAnnotationItem *lastReplyItem = [replyItem->_allReplies lastObject];
                if (lastReplyItem) {
                    NSUInteger index = [_allReplies indexOfObject:lastReplyItem];
                    if (index != NSNotFound) {
                        [_allReplies insertObject:replyItem atIndex:index+1];
                    }
                }else{
                    [_allReplies addObject:replyItem];
                }
            }
        }else{
            if (replyItem.isTopReply) {
                [_allReplies addObject:replyItem];
                for (FSAnnotationItem *_replyItem in replyItem.allReplies) {
                    if (![_allReplies containsObject:_replyItem]) {
                        [_allReplies addObject:_replyItem];
                    }
                }
            }else{
                [_allReplies addObject:replyItem];
            }
        }
    }
}

@end

@implementation FSTopReplyAnnotationItem

- (void)dealloc{
    MARK
}

- (instancetype)initWithAnnot:(FSAnnot *)annot  extensionsManager:(UIExtensionsManager*)extensionsManager{
    self = [super initWithAnnot:annot isReply:YES  extensionsManager:extensionsManager];
    if (self) {
        _unfold = NO;
        self.topReplyAnnotItem = self;
        [self addAllRepliesWithAnnotItem:self];
    }
    return self;
}

- (void)addAllRepliesWithAnnotItem:(FSAnnotationItem *)annotItem{
    NSArray *replyAnnots = annotItem.annot.replyAnnots;
    for (FSNote *note in replyAnnots) {
        FSAnnotationItem *noteItem = [[FSAnnotationItem alloc] initWithAnnot:note isReply:YES extensionsManager:self.extensionsManager];
        [annotItem addReplyItem:noteItem];
        [self addAllRepliesWithAnnotItem:noteItem];
    }
}

- (NSArray<NSString *> *)repliesNM{
    NSMutableArray *tmp = @[].mutableCopy;
    for (FSAnnotationItem *item in self.allReplies.copy) {
        [tmp addObject:item.annot.NM];
    }
    if (tmp.count) return tmp.copy;
    return nil;
}

@end
