/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AttachmentAnnotHandler.h"
#import "AttachmentController.h"
#import "FSPermissionUtilty.h"

@interface AttachmentAnnotHandler () <UIDocumentInteractionControllerDelegate>

@property (nonatomic, strong) UIDocumentInteractionController *documentPopoverController;

@end

@implementation AttachmentAnnotHandler

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
        self.annotSelectionEdgeInsets = UIEdgeInsetsMake(-20, -20, -20, -20);
        self.annotRefreshRectEdgeInsets = UIEdgeInsetsMake(-30, -30, -30, -30);
        self.annotBorderEdgeInsets = UIEdgeInsetsMake(-2, -2, -2, -2);
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotFileAttachment;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionOpen;
    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionOpen;
    }
    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options |= FSMenuOptionStyle;
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    return options;
}

- (void)comment {
    [self openAttachment:[[FSFileAttachment alloc] initWithAnnot:self.extensionsManager.currentAnnot]];
    [self.extensionsManager setCurrentAnnot:nil];
}

- (void)resetProperties{

    FSAnnot *annot = self.extensionsManager.currentAnnot;
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setPropertyColor:annot.color];
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setIcon:annot.icon];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];
    
    [propertyMainView resetProperties:@[ @(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity), @(FSPropertyTypeShapeAttachment)]];
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    if (self.extensionsManager.currentAnnot == annot) {
        if (pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint]) {
            return YES;
        } else {
            [self.extensionsManager setCurrentAnnot:nil];
            return YES;
        }
    } else {
        [self openAttachment:[[FSFileAttachment alloc] initWithAnnot:annot]];
        return YES;
    }
    return NO;
}

- (BOOL)canResizeAnnot:(FSAnnot *)annot {
    return NO;
}

#pragma mark private

- (void)openAttachment:(FSFileAttachment *)annot {
    if (![annot getFileSpec]) {
        return;
    }
    NSFileManager *defaultManager = [NSFileManager defaultManager];
    NSString *attachmentPath = [Utility getAttachmentTempFilePath:annot];
    if ([Utility isSupportFormat:attachmentPath]) {

        AttachmentController *attachmentCtr = [[AttachmentController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
        [self.extensionsManager.viewCtrlManager presentViewController:attachmentCtr animated:YES completion:^{
            dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC));
            dispatch_after(popTime, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                
                if (![defaultManager fileExistsAtPath:attachmentPath]) {
                    [Utility loadAttachment:annot toPath:attachmentPath];
                }
                dispatch_async(dispatch_get_main_queue(), ^{
                    AttachmentItem *attachmentItem = [AttachmentItem itemWithAttachmentAnnotation:annot];
                    attachmentItem.currentlevel = 1;
                    attachmentItem.isSecondLevel = YES;
                    
                    [attachmentCtr openDocument:attachmentItem];
                });
            });
        }];

    } else {
        if (![defaultManager fileExistsAtPath:attachmentPath]) {
            if (![Utility loadAttachment:annot toPath:attachmentPath]) {
                return;
            }
        }
        BOOL isPresent = NO;
        #if !_MAC_CATALYST_
        NSURL *urlFile = [NSURL fileURLWithPath:attachmentPath isDirectory:NO];
        self.documentPopoverController = [UIDocumentInteractionController interactionControllerWithURL:urlFile];
        self.documentPopoverController.delegate = self;
        if (DEVICE_iPHONE) {
            isPresent = [self.documentPopoverController presentOpenInMenuFromRect:self.pdfViewCtrl.frame inView:self.pdfViewCtrl animated:YES];
        } else {
            int pageIndex = annot.pageIndex;
            CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            CGRect dvRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvRect pageIndex:pageIndex];
            isPresent = [self.documentPopoverController presentOpenInMenuFromRect:dvRect inView:self.pdfViewCtrl animated:YES];
        }
        #endif
        if (!isPresent) {
            NSString *fileName = [[annot getFileSpec] getFileName];
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:FSLocalizedForKey(@"kFailedOpenAttachment"), fileName] message:nil preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:cancelAction];
            [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
        }
    }
}


@end
