/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AttachmentToolHandler.h"
#import "AttachmentAnnotHandler.h"

@interface AttachmentToolHandler ()
@end

@implementation AttachmentToolHandler {
    UIExtensionsManager * __weak _extensionsManager;
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotFileAttachment;
    }
    return self;
}

- (NSString *)getName {
    return Tool_Attachment;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
}



// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self || pageIndex < 0) {
        return NO;
    }
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [recognizer locationInView:pageView];
        [self addAnnot:pageIndex point:point];
    }
    return YES;
}

#define ATTACHMENT_WIDTH 20
#define ATTACHMENT_HEIGHT 24

- (BOOL)addAnnot:(int)pageIndex point:(CGPoint)point {
    float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000.0;
    
    FSRectF *iconfsrect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:NOTE_ANNOTATION_WIDTH top1:NOTE_ANNOTATION_WIDTH];
    CGRect iconrect = [_pdfViewCtrl convertPdfRectToPageViewRect:iconfsrect pageIndex:pageIndex];
    float iconwidth = iconrect.size.width;
    CGRect rect = CGRectMake(point.x - ATTACHMENT_WIDTH * scale / 2, point.y - ATTACHMENT_HEIGHT * scale / 2,  iconwidth, iconwidth);

    FSRectF *dibRect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:pageIndex];
    
    FSPointF *tempPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSRectF *annotTempRect = [[FSRectF alloc] init];
    annotTempRect.left = tempPoint.x;
    annotTempRect.bottom = tempPoint.y - NOTE_ANNOTATION_WIDTH;
    annotTempRect.right = annotTempRect.left + NOTE_ANNOTATION_WIDTH;
    annotTempRect.top = annotTempRect.bottom + NOTE_ANNOTATION_WIDTH;
    dibRect = annotTempRect;

    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:_extensionsManager.viewCtrlManager];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFile;
    selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeSingle;
    selectDestination.expectFileType = [[NSArray alloc] initWithObjects:@"*", nil];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        [controller dismissViewControllerAnimated:YES completion:^{
            if (destinationFolder.count > 0) {
                NSFileManager *fileManager = [NSFileManager defaultManager];
                NSDictionary *fileAttribute = [fileManager attributesOfItemAtPath:destinationFolder[0] error:nil];
                long long fileSize = [fileAttribute fileSize];
                if (fileSize > 50 * 1024 * 1024) { // 50MB
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kAttachmentMaxSize"), 50] preferredStyle:UIAlertControllerStyleAlert];
                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                    [alertController addAction:action];
                    [self->_pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                    return;
                }
                
                FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:pageIndex];
                if (!page || [page isEmpty])
                    return;
                
                FSFileAttachment *annot = [[FSFileAttachment alloc] initWithAnnot:[page addAnnot:self.type rect:dibRect]];
                annot.NM = [Utility getUUID];
                annot.author = self->_extensionsManager.annotAuthor;
                annot.icon = self.toolSettings.attachmentIcon;
                annot.color = self.toolSettings.settingsColor;
                annot.opacity = self.toolSettings.opacity;
                annot.subject = @"File Attachment";
                
                annot.flags = FSAnnotFlagPrint | FSAnnotFlagNoZoom | FSAnnotFlagNoRotate;
                
                FSFileSpec *attachFile = [[FSFileSpec alloc] initWithDocument:self->_pdfViewCtrl.currentDoc];
                if (attachFile && [attachFile embed:destinationFolder[0]]) {
                    NSString *fileName = [destinationFolder[0] lastPathComponent];
                    [attachFile setFileName:fileName];
                    [attachFile setCreationDateTime:[Utility convert2FSDateTime:[fileAttribute fileCreationDate]]];
                    [attachFile setModifiedDateTime:[Utility convert2FSDateTime:[fileAttribute fileModificationDate]]];
                    [annot setFileSpec:attachFile];
                    
                    annot.content = fileName;
                }
                
                FSDateTime *now = [Utility convert2FSDateTime:[NSDate date]];
                [annot setCreationDateTime:now];
                [annot setModifiedDateTime:now];
                
                id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
                [annotHandler addAnnot:annot];
            }
        }];
    };
    [self->_extensionsManager.viewCtrlManager presentViewController:selectDestination animated:YES completion:nil];
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    return [self addAnnot:pageIndex point:point];
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
}

@synthesize toolSettings;

@end
