/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AudioVideoAnnotHandler.h"
#import <MobileCoreServices/MobileCoreServices.h>
#import "FSAnnot+Extensions.h"
#import "FSMediaFileUtil.h"
#import "FSPermissionUtilty.h"
#import "AudioVideoModule.h"
#import "AudioPlayerControl.h"

@interface AudioVideoAnnotHandler ()<UIDocumentInteractionControllerDelegate>
@property (nonatomic, assign) FSMediaFileType mediaFileType;
@property (nonatomic, strong) UIDocumentInteractionController *documentPopoverController;
@end

@implementation AudioVideoAnnotHandler{
    __weak UIExtensionsManager *_extensionsManager;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager annotType:(FSAnnotType)annotType{
    self = [super initWithUIExtensionsManager:extensionsManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _annotType = annotType;
        _minPlayerLayerWidthInPage = 0.1;
        _maxPlayerLayerWidthInPage = 1.0;
        _minPlayerLayerHeightInPage = 0.1;
        _maxPlayerLayerHeightInPage = 1.0;
        if (annotType == FSAnnotSound) {
            self.annotRefreshRectEdgeInsets = UIEdgeInsetsMake(-25, -25, -25, -25);
        }
    }
    return self;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    return [[AudioVideoAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager annotType:FSAnnotScreen];
}

- (FSAnnotType)getType {
    return self.annotType;
}

- (AudioVideoModule *)audioVideoModule{
    return [self.extensionsManager getModuleByName:Module_Audio_Video];
}

- (FSMediaFileType)mediaFileType{
    FSScreen *screen = [[FSScreen alloc] initWithAnnot:self.annot];
    FSAction *action = screen.action;
    if ([action isEmpty]) {
        return FSMediaFileTypeUndefine;
    }
    FSRenditionAction *renditionAction = [[FSRenditionAction alloc] initWithAction:action];
    if (([renditionAction isEmpty] || renditionAction.getRenditionCount == 0)) {
        return FSMediaFileTypeUndefine;
    }
    FSRendition *rendition = [renditionAction getRendition:0];
    NSString *mineType = [rendition getMediaClipContentType];
    if ([mineType containsString:@"audio"]) {
        return FSMediaFileTypeAudio;
    }
    return FSMediaFileTypeVideo;
}

- (BOOL)keepAspectRatioForResizingAnnot:(FSAnnot *)annot {
    if (![_extensionsManager annotsPermissionCanModify:annot]) {
        return NO;
    }
    return YES;
}

- (CGSize)minSizeForResizingAnnot:(FSAnnot *)annot {
    FSAnnotType fileType = [annot getFilterAnnotType];
    if (fileType == FSAnnotAudio) {
        return CGSizeMake(40, 40);
    }
    UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
    CGFloat pageWidth = CGRectGetWidth(pageView.bounds);
    CGFloat pageHeight = CGRectGetHeight(pageView.bounds);
    CGFloat minW = self.minPlayerLayerWidthInPage * pageWidth;
    CGFloat minH = self.minPlayerLayerHeightInPage * pageHeight;
    return CGSizeMake(minW, minH);
}

- (CGSize)maxSizeForResizingAnnot:(FSAnnot *)annot {
    UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
    CGFloat pageWidth = CGRectGetWidth(pageView.bounds);
    CGFloat pageHeight = CGRectGetHeight(pageView.bounds);
    CGFloat maxW = self.maxPlayerLayerWidthInPage * pageWidth;
    CGFloat maxH = self.maxPlayerLayerHeightInPage * pageHeight;
    return CGSizeMake(maxW, maxH);
}

- (BOOL)canMoveAnnot:(FSAnnot *)annot {
    if ([annot getType] == FSAnnotSound) {
        return [super canMoveAnnot:annot];
    }
    if ([self.extensionsManager.permissionProvider checkPermission:FSFunctionMultiMediaEdit] == FSPermissionStateShow) {
        return YES;
    }else{
        return NO;
    }
}

- (BOOL)canResizeAnnot:(FSAnnot *)annot {
    if ([annot getType] == FSAnnotSound || [self.extensionsManager.permissionProvider checkPermission:FSFunctionMultiMediaEdit] == FSPermissionStateShow) {
        return [super canResizeAnnot:annot];
    }else{
        return NO;
    }
}

- (void)onAnnotSelected:(FSAnnot *)annot {
    if ([self.extensionsManager.permissionProvider checkPermission:FSFunctionMultiMediaEdit] == FSPermissionStateShow) {
        [super onAnnotSelected:annot];
    }else{
        self.extensionsManager.currentAnnot = nil;
    }
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:annot.pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:annot.pageIndex];
    if ([Utility isAnnot:self.extensionsManager.currentAnnot uniqueToAnnot:annot]) {
        if (!(pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint])) {
            [self.extensionsManager setCurrentAnnot:nil];
        } else if (self.extensionsManager.shouldShowMenu && !self.extensionsManager.shouldShowPropertyBar) {
            [self showMenu];
        }
    } else {
        [self play:annot];
    }
    return YES;
}

- (void) delete {
    @weakify(self)
    [self.extensionsManager.permissionProvider checkPermission:FSFunctionMultiMediaEdit checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            if (self.audioVideoModule.audioPlay) {
                NSString *medioFilePath = [Utility getDocumentRenditionTempFilePath:self.annot.ha_fileSpec PDFPath:self.extensionsManager.pdfViewCtrl.filePath];
                NSString *playFilePath = self.audioVideoModule.audioPlay.filePath.path;
                if ([medioFilePath isEqualToString:playFilePath]) {
                    [self.audioVideoModule.audioPlay dismiss];
                }
            }
            [super delete];
        }else{
            [self showMenu];
        }
    }];
}

- (void)play{
    [self play:self.annot];
    [_extensionsManager setCurrentAnnot:nil];
}

- (void)play:(FSAnnot *)annot{
    if (self.audioVideoModule.audioPlay) [self.audioVideoModule.audioPlay dismiss];
    NSURL * (^renditionResult)(FSFileSpec *) = ^(FSFileSpec *fileSpec){
        NSURL *fileURL = nil;
        NSString *medioFilePath = [Utility getDocumentRenditionTempFilePath:fileSpec PDFPath:self.extensionsManager.pdfViewCtrl.filePath];
        if (!medioFilePath) return fileURL;
        BOOL result = [Utility loadFileSpec:fileSpec toPath:medioFilePath];
        if (!result) fileURL = nil;
        fileURL = [NSURL fileURLWithPath:medioFilePath];
        return fileURL;
    };
    
    if (self.annotType == FSAnnotScreen && (annot.getType == FSAnnotScreen || annot.getType == FSAnnotRichMedia)) {

        if (annot.getType == FSAnnotScreen) {
            FSScreen *screen = [[FSScreen alloc] initWithAnnot:annot];
            FSAction *action = screen.action;
            FSRenditionAction *renditionAction = [[FSRenditionAction alloc] initWithAction:action];
            FSRendition *rendition = [renditionAction getRendition:0];
            FSFileSpec *fileSpec = rendition.mediaClipFile;
            NSLog(@"%@",fileSpec);
            NSURL *fileURL = renditionResult(fileSpec);
            if (!fileURL) return;
            if (![self isPlayableFileURLForAlertView:fileURL annot:screen]) return;
            NSInteger numberOfLoops = 0;
            if (rendition.repeatCount == 0) {
                numberOfLoops = -1;
            }else if (rendition.repeatCount > 0){
                numberOfLoops = rendition.repeatCount;
            }
            if (screen.screenActiontype == FSScreenActionTypeAudio) {
                [self.audioVideoModule audioPlayWithFile:fileURL numberOfLoops:numberOfLoops];
            }
            else if (screen.screenActiontype == FSScreenActionTypeVideo){
                [self.audioVideoModule videoPlayWithFile:fileURL numberOfLoops:numberOfLoops];
            }
        }else if (annot.getType == FSAnnotRichMedia){
            FSRichMedia *richMedia = [[FSRichMedia alloc] initWithAnnot:annot];
            FSFileSpec *fileSpec = [richMedia getFileSpec];
            NSLog(@"%@",fileSpec);
            NSURL *fileURL = renditionResult(fileSpec);
            if (!fileURL) return;
            if (![self isPlayableFileURLForAlertView:fileURL annot:richMedia]) return;
            NSString *medioFilePath = [Utility getDocumentRenditionTempFilePath:fileSpec PDFPath:self.extensionsManager.pdfViewCtrl.filePath];
            NSString *mediaType = [self mimeTypeForFileAtPath:medioFilePath];
            if ([mediaType containsString:@"audio"]) {
                [self.audioVideoModule audioPlayWithFile:fileURL];
            }else if ([mediaType containsString:@"video"]) {
                [self.audioVideoModule videoPlayWithFile:fileURL];
            }
        }

    }else if (self.annotType == FSAnnotSound && annot.getType == FSAnnotSound){
        [self.pdfViewCtrl fs_showHUDLoading:FSLocalizedForKey(@"kLoading") process:^{
            FSSound *sound = [[FSSound alloc] initWithAnnot:annot];
            FSFileSpec *fileSpec = sound.getFileSpec;
            NSURL *fileURL = renditionResult(fileSpec);
            if (!fileURL){
                NSString *medioFilePath = [Utility getDocumentSoundTempFilePath:sound PDFPath:self.extensionsManager.pdfViewCtrl.filePath];
                if (!medioFilePath) return;
                fileURL = [NSURL fileURLWithPath:medioFilePath];
            }
            dispatch_main_async_safe(^{
                [self.audioVideoModule audioPlayWithFile:fileURL];
            });
        }];
    }
}

- (NSString *)mimeTypeForFileAtPath:(NSString *)path
{
    if (![[[NSFileManager alloc] init] fileExistsAtPath:path]) {
        return nil;
    }
    
    NSString *UTI = (__bridge_transfer NSString *)UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (__bridge CFStringRef)[path pathExtension], NULL);
    NSString *MIMEType = (__bridge_transfer NSString *)UTTypeCopyPreferredTagWithClass((__bridge CFStringRef)UTI, kUTTagClassMIMEType);
    
    if (!MIMEType) {
        NSString *mineType = @"";
        NSURL *fileURL = [NSURL fileURLWithPath:path];
        return  [NSString stringWithFormat:@"%@/%@",mineType,fileURL.pathExtension];
    }
    return MIMEType;
}

- (BOOL)isPlayableFileURLForAlertView:(NSURL *)fileURL annot:(FSAnnot*)annot{
    AVURLAsset *asset = [AVURLAsset assetWithURL:fileURL];
    NSArray *tracks = asset.tracks;
    
    __block BOOL isPlayable = NO;
    [tracks enumerateObjectsUsingBlock:^(AVAssetTrack *track, NSUInteger idx, BOOL * _Nonnull stop) {
        if (track.isPlayable) {
            isPlayable = YES;
            *stop = YES;
        }
    }];
    if (!isPlayable) {
        BOOL canOpen = NO;
        #if !_MAC_CATALYST_
        CGRect dvRect = CGRectMake(SCREEN_WIDTH/2, SCREEN_HEIGHT/2, 20, 20);
        self.documentPopoverController = [UIDocumentInteractionController interactionControllerWithURL:fileURL];
        self.documentPopoverController.delegate = self;
        UIView *inView = self.extensionsManager.pdfViewCtrl;
        if (DEVICE_iPHONE){
            canOpen = [self.documentPopoverController presentOpenInMenuFromRect:inView.frame inView:inView animated:YES];
        }
        else{
            if (annot) {
                CGRect rect = [self getAnnotRect:annot];
                dvRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:rect pageIndex:self.annot.pageIndex];
            }
            canOpen = [self.documentPopoverController presentOpenInMenuFromRect:dvRect inView:inView animated:YES];
        }
        #endif
        if (!canOpen) {
            NSLog(@"can't open");
        }
    }
    return isPlayable;
}

- (UIColor *)borderColorForSelectedAnnot:(FSAnnot *)annot {
    return [UIColor colorWithRGB:0x179cd8];
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionPlay;

    if (annot.canModify && [FSPermissionUtilty canAddEditAnnot:self.extensionsManager]) {
        if ([annot canDeleteWithExternsions:self.extensionsManager] && self.annotType == FSAnnotScreen) {
            options |= FSMenuOptionDelete;
        }
    }

    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionPlay;
    }

    if (![FSPermissionUtilty canAddEditAnnot:self.extensionsManager]) {
        options = FSMenuOptionPlay;
    }
    return options;
}

#pragma mark <UIDocumentInteractionControllerDelegate>
- (void)documentInteractionControllerDidDismissOpenInMenu:(UIDocumentInteractionController *)controller{
    
}

@end
