/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AudioVideoModule.h"
#import "FSFileAndImagePicker.h"
#import "AudioVideoAnnotHandler.h"
#import "AudioVideoToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>
#import <AVFoundation/AVFoundation.h>
#import <AVKit/AVKit.h>
#import "AudioPlayerControl.h"

@interface  VideoPlayerControler : AVPlayerViewController;
@property (nonatomic, assign) NSInteger numberOfLoops;
@end

@interface AudioVideoModule ()
@property (nonatomic, strong) AudioPlayerControl *audioPlay;
@property (nonatomic, strong) NSArray *colors;

@end

@implementation AudioVideoModule{
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return Module_Audio_Video;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        AudioVideoToolHandler *toolHandler = [[AudioVideoToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        AudioVideoAnnotHandler *annotHandler = [[AudioVideoAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];
        
        [self loadModule];
    }
    return self;
}

- (void)loadModule {

    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeAudio withItemContentType:FSItemContentTypeImageAndText] atToolTag:FSMainTopbarToolTagEdit];
    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeVideo withItemContentType:FSItemContentTypeImageAndText] atToolTag:FSMainTopbarToolTagEdit];
    
    self.audioPlay = [AudioPlayerControl sharedControl];
    if (self.audioPlay) {
        [self.audioPlay registerEventListens:_extensionsManager];
    }
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionEdit checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            [self->_extensionsManager changeState:FSUIManagerStateAnnotTool];

            AudioVideoToolHandler *toolHanlder = (AudioVideoToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Video];
            toolHanlder.fileType = self.mediaFileType;
            [self->_extensionsManager setCurrentToolHandler:toolHanlder];
            toolHanlder.toolSettings = readToolbarItem.toolSettings;
            if (self.mediaFileType == FSMediaFileTypeAudio) {
                [self->_extensionsManager onUIInteractElementClicked:FSUIElementReadingEditBarAudio];
                
                [self fs_toastWithMsg:FSLocalizedForKey(@"kAudioTapNotice") inView:[self->_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController].view];
            }else if(self.mediaFileType == FSMediaFileTypeVideo){
                [self->_extensionsManager onUIInteractElementClicked:FSUIElementReadingEditBarVideo];
                
                [self fs_toastWithMsg:FSLocalizedForKey(@"kVideoTapNotice") inView:[self->_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController].view];
            }
        }else{
            readToolbarItem.toolbarItem.item.selected = NO;
        }
    }];
    
    
}

- (void)audioPlayWithFile:(NSURL *)fileURL{
    [self audioPlayWithFile:fileURL numberOfLoops:0];
}

- (void)audioPlayWithFile:(NSURL *)fileURL numberOfLoops:(NSInteger)numberOfLoops
{
    if (!self.audioPlay) {
        self.audioPlay = [AudioPlayerControl sharedControl];
    }
    self.audioPlay.filePath = fileURL;
    [self.audioPlay setNumberOfLoops:numberOfLoops];
    [self.audioPlay addSuperview:_extensionsManager.pdfViewCtrl];
    [self.audioPlay play];
}


- (void)videoPlayWithFile:(NSURL *)fileURL{
    [self videoPlayWithFile:fileURL numberOfLoops:0];
}

- (void)videoPlayWithFile:(NSURL *)fileURL numberOfLoops:(NSInteger)numberOfLoops
{
    AVURLAsset *asset = [AVURLAsset assetWithURL:fileURL];
    AVPlayerItem *playerItem = [AVPlayerItem playerItemWithAsset:asset];
    AVPlayer *player = [AVPlayer playerWithPlayerItem:playerItem];
    VideoPlayerControler *VC = [[VideoPlayerControler alloc] init];
    VC.player = player;
    VC.numberOfLoops = numberOfLoops;
    [_pdfViewCtrl.fs_viewController presentViewController:VC animated:YES completion:nil];
}

- (void)dismissAnnotationContinue {
    [Utility dismissAnnotationContinue:_pdfViewCtrl];
}

@end

@interface VideoPlayerControler ()<AVPlayerViewControllerDelegate>
@property (nonatomic, assign) NSInteger currentPlayCount;
@end
@implementation VideoPlayerControler

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad{
    [super viewDidLoad];
    self.delegate = self;
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(playerItemDidReachEnd:)
     name:AVPlayerItemDidPlayToEndTimeNotification
     object:nil];
    self.currentPlayCount = 1;
    [self.player play];
}

- (void)playerItemDidReachEnd:(NSNotification *)not{
    if (self.numberOfLoops < 0){
        return;
    }
    if (self.numberOfLoops == 0){
        [self dismissViewControllerAnimated:YES completion:nil];
        return;
    }
    if (self.currentPlayCount == self.numberOfLoops){
        [self dismissViewControllerAnimated:YES completion:nil];
    }
    self.currentPlayCount++;
    [self.player seekToTime:kCMTimeZero];
    [self.player play];
}
@end

