/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <MobileCoreServices/MobileCoreServices.h>
#import "AudioVideoToolHandler.h"
#import "AudioVideoAnnotHandler.h"
#import "FSFileAndImagePicker.h"
#import "AudioVideoModule.h"

@interface AudioVideoToolHandler ()<FSFileAndImagePickerDelegate,IDocEventListener>

@property (nonatomic,assign) CGPoint point;
@property (nonatomic,assign) int pageIndex;
// 0.0~1.0
@property (nonatomic,assign) CGFloat minPlayerLayerWidthInPage;
@property (nonatomic,assign) CGFloat maxPlayerLayerWidthInPage;
@property (nonatomic,assign) CGFloat minPlayerLayerHeightInPage;
@property (nonatomic,assign) CGFloat maxPlayerLayerHeightInPage;
@property (nonatomic,strong) FSFileAndImagePicker *mediaPicker;

@end

@implementation AudioVideoToolHandler {
    UIExtensionsManager * __weak _extensionsManager;
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    FSTaskServer *_taskServer;
}
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    return [[AudioVideoToolHandler alloc] initWithUIExtensionsManager:extensionsManager annotType:FSAnnotScreen];
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager annotType:(FSAnnotType)annotType {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _annotType = annotType;
        _minPlayerLayerWidthInPage = 0.1;
        _maxPlayerLayerWidthInPage = 0.5;
        _minPlayerLayerHeightInPage = 0.1;
        _maxPlayerLayerHeightInPage = 0.5;
        _pageIndex = -1;
        _point = CGPointZero;
        
        self.mediaPicker = [[FSFileAndImagePicker alloc] init];
        [extensionsManager.pdfViewCtrl registerDocEventListener:self];
    }
    return self;
}

- (void)dealloc
{
    [_extensionsManager.pdfViewCtrl unregisterDocEventListener:self];
}

- (NSString *)getName {
    if (self.annotType == FSAnnotScreen) {
        return Tool_Video;
    }
    return Tool_Audio;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    return [self handleLongPressAndPan:pageIndex gestureRecognizer:recognizer];
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return [self handleLongPressAndPan:pageIndex gestureRecognizer:recognizer];
}

- (BOOL)handleLongPressAndPan:(int)pageIndex gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer {
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    _point = point;
    _pageIndex = pageIndex;
    
    CGSize tapSize = CGSizeMake(5, 5);
    CGRect tapRect = CGRectMake(point.x - tapSize.width / 2, point.y - tapSize.height / 2, tapSize.width, tapSize.height);
    
    CGRect dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:tapRect pageIndex:self.pageIndex];

    self.mediaPicker.extensionsManager = _extensionsManager;
    self.mediaPicker.fileType = self.fileType;
    self.mediaPicker.delegate = self;
    [self.mediaPicker showMenu:dvRect];
    
    return YES;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return YES;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (CGRect)annotRectWithCenter:(CGPoint)center pageView:(UIView *)pageView {
    float pageWidth = CGRectGetWidth(pageView.frame);
    float pageHeight = CGRectGetHeight(pageView.frame);
    if (!CGRectContainsPoint(pageView.bounds, center)) {
        return CGRectZero;
    }
    
    if (self.fileType  == FSMediaFileTypeAudio) {
        
        float audioScaledWidth = [Utility convertWidth:20 fromPDFToPageView:_pdfViewCtrl pageIndex:_pageIndex];
        CGFloat audio_icon_width = audioScaledWidth;
        CGFloat audio_icon_height = audioScaledWidth;
        CGRect frame = CGRectMake(center.x - audio_icon_width / 2, center.y - audio_icon_height / 2, audio_icon_width, audio_icon_height);
        return [Utility boundedRectForRect:frame containerRect:pageView.bounds];
    }

    CGSize videoSize = [self getThumbailImageRequestAtTimeSecond:0]?[self getThumbailImageRequestAtTimeSecond:0].size:CGSizeMake(40.f, 40.f);
    CGFloat originWidth = videoSize.width;
    CGFloat originHeight = videoSize.height;
    
    CGFloat width = 0;
    CGFloat height = 0;
    if (!CGSizeEqualToSize(videoSize, CGSizeMake(40.f, 40.f))) {
        width = MIN(MAX(originWidth, self.minPlayerLayerWidthInPage * pageWidth), self.maxPlayerLayerWidthInPage * pageWidth);
        height = originHeight / originWidth * width;
        height = MIN(MAX(height, self.minPlayerLayerHeightInPage * pageHeight), self.maxPlayerLayerHeightInPage * pageHeight);
        width = originWidth / originHeight * height;
    }else{
        width = originWidth;
        height = originHeight;
    }

    CGRect frame = CGRectMake(center.x - width / 2, center.y - height / 2, width, height);
    return [Utility boundedRectForRect:frame containerRect:pageView.bounds];
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
}

+ (CGSize)sizeAspectRatioFitSize:(CGSize)originSize maxWidth:(CGFloat)maxWidth maxHeight:(CGFloat)maxHeight {
    CGSize size = originSize;
    CGFloat aspectRatio = originSize.height / originSize.width;
    if (size.width > maxWidth) {
        size.width = maxWidth;
        size.height = size.width * aspectRatio;
    }
    if (size.height > maxHeight) {
        size.height = maxHeight;
        size.width = size.height / aspectRatio;
    }
    return size;
}

+ (CGPoint)centerOfRect:(CGRect)rect {
    return CGPointMake(rect.origin.x + rect.size.width / 2, rect.origin.y + rect.size.height / 2);
}

- (void)addAnnotWithRect:(CGRect)rect pageIndex:(int)pageIndex {
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    FSRectF *fsrect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:pageIndex];
    FSScreen *annot = [[FSScreen alloc] initWithAnnot:[page addAnnot:FSAnnotScreen rect:fsrect]];
    if (![annot getCptr]) {
        return;
    }
    
    int pvcRotation = [_pdfViewCtrl getViewRotation];
    FSRotation rotation = (FSRotation)(pvcRotation) % 4;
    rotation = ([page getRotation] + rotation) % 4;
    if (rotation != 0) {
        rotation = 4 - rotation;
    }
    [annot setRotation:rotation];
    
    FSFileSpec *fileSpec = [[FSFileSpec alloc] initWithDocument:self->_pdfViewCtrl.currentDoc];
    if (fileSpec && [fileSpec embed:self.filePath]) {
        annot.NM = [Utility getUUID];
        annot.author = _extensionsManager.annotAuthor;
        annot.createDate = [NSDate date];
        annot.modifiedDate = [NSDate date];
        annot.flags = FSAnnotFlagPrint;
        NSString *mineType = @"";
        annot.intent = mineType.copy;
        annot.opacity = self.toolSettings.opacity;
        [annot setBorderInfo:({
                   FSBorderInfo *borderInfo = [[FSBorderInfo alloc] init];
                   [borderInfo setWidth:0];
                   [borderInfo setStyle:FSBorderInfoSolid];
                    borderInfo;
               })];
        FSImage *fsimage = nil;
        if (self.fileType == FSMediaFileTypeAudio) {
            fsimage = [Utility uiimageToFSImage:FSImageNamed(@"edit_audio_annote")];
        }
        if (self.fileType == FSMediaFileTypeVideo) {
            UIImage *image = [self getThumbailImageRequestAtTimeSecond:0];
            if (image == nil) {
                image = FSImageNamed(@"edit_video_default");
            }
            fsimage = [Utility uiimageToFSImage:image];
        }
        if (fsimage) {
            [annot setImage:fsimage frame_index:0 compress:0];
        }
        NSFileManager *fileManager = [NSFileManager defaultManager];
        NSDictionary *fileAttribute = [fileManager attributesOfItemAtPath:self.filePath error:nil];
        
        FSAction *action = [FSAction create:_extensionsManager.pdfViewCtrl.currentDoc action_type:FSActionTypeRendition];
        FSRenditionAction *renditionAction = [[FSRenditionAction alloc] initWithAction:action];
        renditionAction.operationType = FSRenditionActionOpTypeAssociate;
        renditionAction.screenAnnot = annot;
        
        FSRendition *rendition = [[FSRendition alloc] initWithPdf_doc:_extensionsManager.pdfViewCtrl.currentDoc rendition_dict:[FSPDFDictionary create]];
        
        NSString *fileName = [self.filePath lastPathComponent];
        [fileSpec setFileName:fileName];
        [fileSpec setCreationDateTime:[Utility convert2FSDateTime:[fileAttribute fileCreationDate]]];
        [fileSpec setModifiedDateTime:[Utility convert2FSDateTime:[fileAttribute fileModificationDate]]];
        [rendition setMediaClipFile:fileSpec];
        rendition.renditionName = fileName;
        rendition.mediaClipName = fileName;
        rendition.permission = FSRenditionMediaPermTempAccess;
        NSString *mimeStr = [self mimeTypeForFileAtPath:self.filePath];
        if ([mimeStr isEqualToString:@"audio/vnd.wave"]) {
            mimeStr = @"audio/x-wav";
        }
        rendition.mediaClipContentType = mimeStr;
        [renditionAction insertRendition:rendition index:-1];
        annot.action = renditionAction;
        annot.contents = fileSpec.fileName;
        
        [_pdfViewCtrl lockRefresh];
        [annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        [_pdfViewCtrl refresh:CGRectInset(rect, -5, -5) pageIndex:pageIndex needRender:YES];
        
        for (id<IAnnotHandler> annotHandler in _extensionsManager.annotHandlers) {
            if ([annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]) {
                [annotHandler addAnnot:annot addUndo:YES];
                break;
            }
        }
    }

    self.filePath = nil;
}

- (NSString *)mimeTypeForFileAtPath:(NSString *)path
{
    if (![[[NSFileManager alloc] init] fileExistsAtPath:path]) {
        return nil;
    }
    
    NSString *UTI = (__bridge_transfer NSString *)UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (__bridge CFStringRef)[path pathExtension], NULL);
    NSString *MIMEType = (__bridge_transfer NSString *)UTTypeCopyPreferredTagWithClass((__bridge CFStringRef)UTI, kUTTagClassMIMEType);
    
    if (!MIMEType) {
        NSString *mineType = @"";
        if (self.fileType == FSMediaFileTypeAudio) {
            mineType = @"audio";
        }
        else if (self.fileType == FSMediaFileTypeVideo) {
            mineType = @"video";
        }
        NSURL *fileURL = [NSURL fileURLWithPath:path];
        return  [NSString stringWithFormat:@"%@/%@",mineType,fileURL.pathExtension];
    }
    return MIMEType;
}

- (UIImage *)getThumbailImageRequestAtTimeSecond:(CGFloat)timeBySecond {
    
    NSURL *url = [NSURL fileURLWithPath:self.filePath];
    AVURLAsset *urlAsset = [AVURLAsset assetWithURL:url];
    AVAssetImageGenerator *imageGenerator = [AVAssetImageGenerator assetImageGeneratorWithAsset:urlAsset];
    imageGenerator.appliesPreferredTrackTransform = YES;
    CMTime time = CMTimeMake(timeBySecond, 10);
    CMTime actualTime;
    NSError *error = nil;
    CGImageRef cgImage = [imageGenerator copyCGImageAtTime:time
                                                actualTime:&actualTime
                                                     error:&error];
    UIImage *icon_img = FSImageNamedAndNormalColor(@"add_veido_kaishi");
    if (error) {
        FSLog(@"getThumbailImageRequestAtTimeSecond：%@",error.localizedDescription);
        return nil;
    }
    UIImage *image = [UIImage imageWithCGImage:cgImage];
    
    CGFloat tW = CGImageGetWidth(image.CGImage);
    CGFloat tH = CGImageGetHeight(image.CGImage);
    
    CGFloat w = tW/3;
    CGFloat h = tH/3;
    if(tW < tH){
        h = w;
    }else{
        w = h;
    }
    UIGraphicsBeginImageContext(CGSizeMake(tW, tH));
    [image drawInRect:CGRectMake(0, 0, tW, tH)];
    [icon_img drawInRect:CGRectMake((tW - w) /2, (tH - h)/2, w, h)];
    UIImage *resultImg = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    CGImageRelease(cgImage);
    return resultImg;
}

#pragma mark <FSFileAndImagePickerDelegate>

- (void)fileAndImagePicker:(FSFileAndImagePicker *)fileAndImagePicker didPickFileAtPath:(NSString *)filePath{
    
    if (!filePath) {
        return;
    }
    
    self.filePath = filePath;
    UIView *pageView = [_pdfViewCtrl getPageView:_pageIndex];
    CGRect frame = [self annotRectWithCenter:_point pageView:pageView];
    if (CGRectIsEmpty(frame)) {
        return;
    }
    [self addAnnotWithRect:frame pageIndex:_pageIndex];
    
    [self completed];
//    if (_extensionsManager.continueAddAnnot) {
//        return;
//    }
    [_extensionsManager setCurrentToolHandler:nil];

}

- (void)fileAndImagePicker:(FSFileAndImagePicker *)fileAndImagePicker didPickImage:(UIImage *)image imagePath:(NSString *)imagePath {
    
}
- (void)fileAndImagePickerDidCancel:(FSFileAndImagePicker *)fileAndImagePicker {
    [self completed];
}

- (void)completed{
    self.filePath = nil;
    _point = CGPointZero;
    _pageIndex = -1;
}

#pragma mark -- <IDocEventListener>
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error{
}

@synthesize type;

@synthesize toolSettings;

@end
