/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
*/


#import "FSAutoFlipModule.h"
#import "PanelController+private.h"
#import "FSSettingBar+private.h"
#import "DigitalSignaturePanel.h"
#import "AnnotationPanel.h"
#import "FSCircleProgressView.h"
#import "PageNavigationModule.h"

static NSInteger MAX_SECONDS = 100;
static NSInteger MIN_SECONDS = 20;
static NSInteger DEFAULT_SECONDS = 60;
static NSInteger SECONDS_FOR_ONCE = 10;

@interface FSAutoFlipModule()<IStateChangeListener,IFSViewSettingEventListener,IFullScreenListener,IWindowFrameChangedListener>
@property(nonatomic, strong)FSToolbar* topToolbar;
@property(nonatomic, strong)FSToolbar* botoomToolbar;
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, assign) NSInteger currentSecondsForSpeed;
@property(nonatomic, strong) FSItem* addSpeedItem;
@property(nonatomic, strong) FSItem* slowSpeedItem;
@property(nonatomic, strong) FSItem* exitItem;
@property(nonatomic, strong) NSTimer *timer;
@property (nonatomic, assign) NSInteger currentTimerCount;
@property (nonatomic, strong) FSCircleProgressView* circleProgressView;
@property (nonatomic, assign) BOOL isSelfState;
@property (nonatomic, strong) UIPanGestureRecognizer *pan;
@end

@implementation FSAutoFlipModule

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager
{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _currentTimerCount = 0;
        [self initModule];
    }
    return self;
}

- (void)initModule
{
    [_pdfViewCtrl registerGestureEventListener:self];
    [_pdfViewCtrl registerDocEventListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager registerStateChangeListener:self];
    [_extensionsManager registerFullScreenListener:self];
    [_extensionsManager.settingBar.viewSettingManager registerEventListener:self];
    [_extensionsManager.thumbnailModule registerThumbnailEventListener:self];
    [_extensionsManager.panelController registerPanelChangedListener:self];
    [_extensionsManager registerWindowFrameChangedListener:self];
    [self.topToolbar setHidden:YES];
    [self.botoomToolbar setHidden:YES];
    self.currentSecondsForSpeed = DEFAULT_SECONDS;
    self.currentTimerCount = 0;
}

- (void)enterAutoFlipMode:(BOOL)isShow
{
    if (!isShow || (isShow && !self.timer)) {
        [self showAutoFlipMode:isShow animation:YES];
    }
    
    if (isShow) {
        [self shouldStartTimer:YES];
    }
    else
    {
        [self startTimer:NO];
    }
}

- (void)shouldStartTimer:(BOOL)forced{
    
    if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT ||
        [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT ||
        [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE ) {
        
        if ([self.pdfViewCtrl getCurrentPage] >= [self.pdfViewCtrl getPageCount] - 2 &&
            !([self.pdfViewCtrl getPageCount]==2 && [self.pdfViewCtrl getCurrentPage] == 0)) {
            [self startTimer:NO];
            return;
        }
        
    }else if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO) {
        
        if ([self.pdfViewCtrl getCurrentPage] >= [self.pdfViewCtrl getPageCount] - 2) {
            [self startTimer:NO];
            return;
        }
        
    }else{
        if ([self.pdfViewCtrl getCurrentPage] >= [self.pdfViewCtrl getPageCount] - 1) {
            [self startTimer:NO];
            return;
        }
    }
    
    if (forced) {
        [self startTimer:YES];
    }else{
        if (!self.timer) {
            [self startTimer:YES];
        }
    }
}

- (void)startTimer:(BOOL)isStart
{
    if (isStart) {
        if (self.timer)
        {
            [self.timer invalidate];
            self.timer = nil;
        }
        self.currentTimerCount = 0;
        self.circleProgressView.progressValue = 0;
        self.timer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(doActionWithTimer) userInfo:nil repeats:YES];
    }
    else
    {
        self.currentTimerCount = 0;
        self.circleProgressView.progressValue = 0;
        if (self.timer)
        {
            
            [self.timer invalidate];
            self.timer = nil;
        }
    }
}

- (void)doActionWithTimer
{
    _currentTimerCount ++;
    [self updateProgressViewValue];
    if (_currentTimerCount >= self.currentSecondsForSpeed) {
        _currentTimerCount = 0;
        self.circleProgressView.progressValue = 0;
        
        if (self.pdfViewCtrl.isContinuous || [self.extensionsManager getState] == FSUIManagerStateReflow) {
       
            if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT || [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE){
                int pageIndex = [self.pdfViewCtrl getCurrentPage] + 2;
                if (pageIndex >= [self.pdfViewCtrl getPageCount] - 2) {
                    [self startTimer:NO];
                }
            }

        }

        if ([self.pdfViewCtrl getCurrentPage] < [self.pdfViewCtrl getPageCount] - 1) {
            [self.pdfViewCtrl gotoNextPage:YES];
        }

    }
}

- (void)updateProgressViewValue
{
    float progress = _currentTimerCount / (float)self.currentSecondsForSpeed;
    if (progress > 1) progress = 1.0;
    self.circleProgressView.progressValue = (int)(progress * 100);
}

- (FSToolbar *)topToolbar
{
    if (_topToolbar == nil) {
        FSToolbar* topToolbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        topToolbar.tag = TOPBAR_PAGESLIDER_TAG;
        [topToolbar setHidden:YES];
        _topToolbar = topToolbar;
        topToolbar.backgroundColor = self.fs_backgroundColor_B2;
        StateBarThemeKeyPath(topToolbar.stateBar, backgroundColor);
        [BindThemeKeySel(topToolbar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
            StateBarThemeKeyPath(target, backgroundColor);
        }];
        [_extensionsManager.containerView addSubview:topToolbar];
        FSItem* closeItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
        [closeItem setFont:self.fs_standardGeneralFont_GF1];
        [BindThemeKeySel(closeItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        @weakify(self)
        closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self->_extensionsManager changeState:FSUIManagerStateNormal];
        };
        [topToolbar addItem:closeItem atPosition:FSItemPositionLeft];
        FSItem* titleItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kAutoFlip")];
        titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
        titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
        [titleItem setFont:self.fs_standardGeneralFont_GF1];
        [titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [topToolbar addItem:titleItem atPosition:FSItemPositionCenter];
        
        [_extensionsManager.containerView addSubview:topToolbar];
        [topToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.mas_equalTo(topToolbar.superview);
        }];
    }
    return _topToolbar;
}

- (FSToolbar *)botoomToolbar
{
    if (_botoomToolbar == nil) {
        FSToolbar* bottomToolbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        [bottomToolbar setHidden:YES];
        _botoomToolbar = bottomToolbar;
        bottomToolbar.hasDivide = NO;
        
        FSItem* addSpeedItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"auto_flip_add_speed")];
        addSpeedItem.accessibilityLabel = FSLocalizedForKey(@"kAutoflipIncreaseVoiceOver");
        _addSpeedItem = addSpeedItem;
        @weakify(self)
        addSpeedItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
                [self.extensionsManager delayHideTopBottomToolBar];
            }
            self.currentSecondsForSpeed -= SECONDS_FOR_ONCE;
            if (self.currentSecondsForSpeed <= MIN_SECONDS)
            {
                [self.addSpeedItem setEnabled:NO];
            }else if (self.currentSecondsForSpeed < MAX_SECONDS)
            {
                [self.slowSpeedItem setEnabled:YES];
            }
            [self updateProgressViewValue];
        };
        
        _circleProgressView = [[FSCircleProgressView alloc] initWithFrame:CGRectMake(0, 0, 21, 21)];
        [BindThemeKeySel(_circleProgressView, setProgressColor:) fs_updateThemeCompleted:^(FSCircleProgressView*  _Nonnull target, SEL  _Nonnull sel) {
            [target setProgressColor:target.fs_primaryColor ];
        }];
        _circleProgressView.progressWidth = 1.5f;
        _circleProgressView.progressTrackWidth = 1.5f;
        _circleProgressView.progressTrackColor = self.fs_textColor_T2;
        _circleProgressView.progressTextFont = self.fs_standardWeakFont_WF3;
        _circleProgressView.progressTextColor = self.fs_textColor_T4;
        _circleProgressView.progressValue = 0;
        [_botoomToolbar addSubview:_circleProgressView];
        
        FSItem* slowSpeedItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"auto_flip_slow_speed")];
        slowSpeedItem.accessibilityLabel = FSLocalizedForKey(@"kAutoflipDecreaseVoiceOver");
        _slowSpeedItem = slowSpeedItem;

        slowSpeedItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
                [self.extensionsManager delayHideTopBottomToolBar];
            }
            self.currentSecondsForSpeed += SECONDS_FOR_ONCE;
            if (self.currentSecondsForSpeed >= MAX_SECONDS)
            {
                [self.slowSpeedItem setEnabled:NO];
                //[self.pdfViewCtrl gotoNextPage:YES];
            }else if (self.currentSecondsForSpeed > MIN_SECONDS)
            {
                [self.addSpeedItem setEnabled:YES];
            }
            [self updateProgressViewValue];
        };
        
        [_extensionsManager.containerView addSubview:bottomToolbar];
        [self reLayoutBottomToolbar];
    }
    return _botoomToolbar;
}

#pragma mark -- IFullScreenListener
- (void)onFullScreen:(BOOL)isFullScreen {
    if ([_extensionsManager getState] == FSUIManagerStateAutoFlip && !(isFullScreen && _extensionsManager.currentToolHandler != nil)) {
        [self showAutoFlipMode:!isFullScreen animation:YES];
    }
}

- (void)showAutoFlipMode:(BOOL)isShow animation:(BOOL)animation
{
    if (self.botoomToolbar.hidden != isShow) {
        return;
    }

    if (!isShow) {
        if (![self checkIsIphoneDevice]) {
            [_extensionsManager setHiddenTopToolbar:YES];
        }
        self.topToolbar.hidden = YES;
        self.botoomToolbar.hidden  = YES;
    } else {
        if (![self checkIsIphoneDevice]) {
            self.topToolbar.hidden = YES;
            [_extensionsManager setHiddenTopToolbar:NO];
            [self.botoomToolbar fs_setShadowColor:self.fs_dividerColor_P3];
        }
        else
        {
            [_extensionsManager setHiddenTopToolbar:YES];
            [_extensionsManager setHiddenBottomToolbar:YES];
            self.topToolbar.hidden = NO;
            [self.botoomToolbar fs_setShadowColor:nil];
        }
        self.botoomToolbar.hidden  = NO;
        [self.botoomToolbar.superview bringSubviewToFront:self.botoomToolbar];
    }
    if (animation) {
        [Utility addAnimation:self.topToolbar.layer type:self.topToolbar.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.topToolbar.hidden ? kCATransitionFromTop : kCATransitionFromBottom timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
        [Utility addAnimation:self.botoomToolbar.layer type:self.botoomToolbar.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.botoomToolbar.hidden ? kCATransitionFromBottom : kCATransitionFromTop timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
    }
    [(PageNavigationModule *)[_extensionsManager getModuleByName:@"PageNavigation"] showPageNum: isShow];
    
    [_extensionsManager setStatusBarHidden:_extensionsManager.currentIsFullScreen && _topToolbar.hidden];
    [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    [_extensionsManager.settingBar.viewSettingManager resetPdfViewCtrlSafeArea];
    
    [(PageNavigationModule *)[_extensionsManager getModuleByName:@"PageNavigation"] layoutPdfViewContentInsets:NO];
}

#pragma mark --IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    if ([_extensionsManager getState] == FSUIManagerStateAutoFlip) {
        [_extensionsManager setFullScreen:!self.botoomToolbar.hidden];
        return YES;
    }
    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

- (nonnull FSModuleStringName)getName {
    return NSStringFromClass([FSAutoFlipModule class]);
}


#pragma mark --IStateChangeListener

-(void)onStateChanged:(FSUIManagerState)state lastState:(FSUIManagerState)lastState
{
    if (state != FSUIManagerStateAutoFlip && state != FSUIManagerStateAnnotTool) {
        if (_isSelfState) {
            _isSelfState = NO;
            [self.botoomToolbar setHidden:YES];
            [self.topToolbar setHidden:YES];
            [self startTimer:NO];
            [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:AUTOFLIP selected:NO];
            [self->_extensionsManager setFullScreen:NO];
        }
    }
    else if (state == FSUIManagerStateAutoFlip)
    {
        _isSelfState = YES;
        if (lastState != FSUIManagerStateAnnotTool && lastState != FSUIManagerStateAutoFlip) {
            [self.extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:AUTOFLIP selected:YES];
            [self reLayoutBottomToolbar];
        }
    }
}

-(void)reLayoutBottomToolbar
{
    [_botoomToolbar removeAllItems];
    if ([self checkIsIphoneDevice]) {
        [self.botoomToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.mas_equalTo(self.botoomToolbar.superview);
        }];
        if (_exitItem) {
            [_exitItem removeFromSuperview];
        }
        if (_pan) {
            [_botoomToolbar removeGestureRecognizer:_pan];
        }
        
        [_botoomToolbar addItem:_addSpeedItem spacing:0 atPosition:FSItemPositionCenter];
        
        [_circleProgressView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.mas_equalTo(self.circleProgressView.superview);
            make.centerY.mas_equalTo(self.circleProgressView.superview);
            make.width.height.mas_equalTo(24);
        }];
        
        [_botoomToolbar addItem:_slowSpeedItem spacing:60.f atPosition:FSItemPositionCenter];
    }
    else
    {
        self.botoomToolbar.layer.cornerRadius = 6;
        [self.botoomToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.botoomToolbar.superview.fs_mas_bottom).offset(-40);
            make.centerX.mas_equalTo(self.botoomToolbar.superview);
            make.width.mas_equalTo(240);
        }];
        
        [_addSpeedItem removeFromSuperview];
        [_botoomToolbar addSubview:_addSpeedItem];
        [_addSpeedItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(16);
            make.centerY.mas_equalTo(self.addSpeedItem.superview);
            make.width.height.mas_equalTo(40);
        }];
        
        [_circleProgressView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(80);
            make.centerY.mas_equalTo(self.circleProgressView.superview);
            make.width.height.mas_equalTo(24);
        }];
 
        [_slowSpeedItem removeFromSuperview];
        [_botoomToolbar addSubview:_slowSpeedItem];
        [_slowSpeedItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(128);
            make.centerY.mas_equalTo(self.slowSpeedItem.superview);
            make.width.height.mas_equalTo(40);
        }];
        
        @fs_weakify(self)
        if (!_exitItem) {
            FSItem* exitItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_exit")];
            exitItem.accessibilityLabel = FSLocalizedForKey(@"kExitVoiceOver");
            _exitItem = exitItem;
            _exitItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                [self->_extensionsManager changeState:FSUIManagerStateNormal];
            };
        }
        if (!_exitItem.superview) {
            [_botoomToolbar addSubview:_exitItem];
            [_exitItem mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.right.mas_equalTo(-ITEM_LABEL_TOP_MARGIN_NORMAL);
                make.centerY.mas_equalTo(self.exitItem.superview);
                make.width.height.mas_equalTo(40);
            }];
        }
        if (!_pan) {
            _pan = [[UIPanGestureRecognizer alloc] initWithTarget:_extensionsManager.settingBar.viewSettingManager action:@selector(moveBottombarWithRecognizer:)];
        }
        if (![self.botoomToolbar.gestureRecognizers containsObject:_pan]) {
            [self.botoomToolbar addGestureRecognizer:_pan];
        }
    }
}

#pragma mark--pageEventListener methods

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    if ([_extensionsManager getState] == FSUIManagerStateAutoFlip)
    {
        _currentTimerCount = 0;
        self.circleProgressView.progressValue = 0;
        [self shouldStartTimer:NO];
    }
}

#pragma mark -- IDocEventListener
- (void)onDocWillClose:(FSPDFDoc *)document{
    if (_isSelfState) {
         [_extensionsManager changeState:FSUIManagerStateNormal];
    }
}

#pragma mark -- IThumbnailEventListener
- (void)onThumbnailChanged:(BOOL)isHidden
{
    if ([self.extensionsManager getState] == FSUIManagerStateAutoFlip) {
        _botoomToolbar.hidden = !isHidden;
        if (isHidden) {
            [self.extensionsManager setFullScreen:NO];
        }
    }
}

#pragma mark --IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStateAutoFlip) {
        if (![self checkIsIphoneDevice] && !isHidden) {
            [self.botoomToolbar.superview bringSubviewToFront:self.botoomToolbar];
        }
    }
}

#pragma mark-- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    if (DEVICE_iPAD && changed) {
        if ([_extensionsManager getState] == FSUIManagerStateAutoFlip) {
            [self reLayoutBottomToolbar];
            self.botoomToolbar.hidden = YES;
            [self.extensionsManager setCurrentIsFullScreen:NO];
            [self showAutoFlipMode:YES animation:NO];
        }else if([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]){
            [self showAutoFlipMode:NO animation:NO];
        }
    }
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}
@end
