/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "CalloutToolHandler.h"
#import "StringDrawUtil.h"
#import "FSEasyDoneTextView.h"
#import "FreetextUtil.h"
#import "FtAnnotHandler.h"
#import "PaintUtility.h"

@interface CalloutToolHandler () <UITextViewDelegate,IRotationEventListener,IWindowFrameChangedListener, IPageEventListener,IScrollViewEventListener, IPropertyViewEventListener>{
    FSEasyDoneTextView *_textView;
    
    CGPoint _startPoint;
    CGPoint _kneePoint;
    CGPoint _endPoint;
    CGFloat _polylineLength;
}

@property (nonatomic, strong) FSFreeText* annot;
@property (nonatomic, assign) int pageIndex;
@property (nonatomic, strong) FSRectF *oldRect;
@property (nonatomic, strong, nullable) UIImage *annotImage;
@property (nonatomic, assign) BOOL isLongPressing;
@property (nonatomic, assign) BOOL isLongPressingMoved;
@end

@implementation CalloutToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
    CGFloat minAnnotPageInset;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotFreeText;
        minAnnotPageInset = 5;
        _polylineLength = 20.0;
        
        [_extensionsManager registerRotateChangedListener:self];
        [_extensionsManager registerWindowFrameChangedListener:self];
        [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
        [extensionsManager.pdfViewCtrl registerScrollViewEventListener:self];
        [extensionsManager registerPropertyViewEvnetListener:self];
    }
    return self;
}

-(NSString*)getName {
    return Tool_Callout;
}

-(BOOL)isEnabled {
    return YES;
}

-(void)onActivate {
}

-(void)onDeactivate {
    [self save];
//    [_textView resignFirstResponder];
}


// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self ||
        ((recognizer.state == UIGestureRecognizerStateBegan || recognizer.state == UIGestureRecognizerStateChanged) &&pageIndex < 0)) {
        return NO;
    }
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.isLongPressingMoved = NO;
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [recognizer locationInView:pageView];
        [self addAnnot:pageIndex point:point longPress:YES];
        if (!self.annot) {
            return YES;
        }
        self.isLongPressing = YES;
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:self.annot];
        FSPointFArray *calloutPoints = [freetext getCalloutLinePoints];
        if ([calloutPoints getSize] > 0) {
            _startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:[calloutPoints getAt:0] pageIndex:self.annot.pageIndex];
        }
        
    }else if (recognizer.state == UIGestureRecognizerStateChanged){
        if (!self.annot || pageIndex != self.annot.pageIndex) {
            return YES;
        }
        self.isLongPressingMoved = YES;
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:self.annot];
        self.oldRect = self.annot.fsrect;
        CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        if (!self.annot.canModify) {
            return YES;
        }
        UIView *pageView  = [_pdfViewCtrl getPageView:self.pageIndex];
        CGRect borderRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:self.annot.pageIndex];
        CGRect newRect;
        
        [self saveCalloutPoints:freetext];
        CGRect innerRect = [_pdfViewCtrl convertPdfRectToPageViewRect:[freetext getInnerRect] pageIndex:self.annot.pageIndex];
        
        CGRect newInnerRect;
        newInnerRect = innerRect;
        newInnerRect.origin.x = point.x;
        newInnerRect.origin.y = point.y - innerRect.size.height/2;
        if (newInnerRect.origin.x + innerRect.size.width >= pageView.frame.size.width &&
            _startPoint.x + innerRect.size.width >= pageView.frame.size.width) {
            newInnerRect.origin.x = pageView.frame.size.width - innerRect.size.width;
            newInnerRect.origin.y = point.y - innerRect.size.height/2;
        }
        
        //calculate new points
        if (_endPoint.x <= innerRect.origin.x + 2.0) {
            //left
            _endPoint.x = newInnerRect.origin.x;
            _endPoint.y = newInnerRect.origin.y + newInnerRect.size.height / 2.0;
            _kneePoint.x = _endPoint.x - _polylineLength;
            _kneePoint.y = _endPoint.y;
        } else if (_endPoint.x >= (innerRect.origin.x + innerRect.size.width) - 2.0) {
            //right
            _endPoint.x = newInnerRect.origin.x + newInnerRect.size.width;
            _endPoint.y = newInnerRect.origin.y + newInnerRect.size.height / 2.0;
            _kneePoint.x = _endPoint.x + _polylineLength;
            if (_kneePoint.x > pageView.bounds.size.width) {
                _kneePoint.x  =  pageView.bounds.size.width;
            }
            _kneePoint.y = _endPoint.y;
        } else if (_endPoint.y <= innerRect.origin.y + 2.0) {
            //top
            _endPoint.y = newInnerRect.origin.y;
            _endPoint.x = newInnerRect.origin.x + newInnerRect.size.width / 2.0;
            _kneePoint.y = _endPoint.y - _polylineLength;
            _kneePoint.x = _endPoint.x;
        } else {
            //bottom
            _endPoint.y = newInnerRect.origin.y + newInnerRect.size.height;
            _endPoint.x = newInnerRect.origin.x + newInnerRect.size.width / 2.0;
            _kneePoint.y = _endPoint.y + _polylineLength;
            _kneePoint.x = _endPoint.x;
        }
        innerRect = newInnerRect;
        
        
        CGRect realPageRect = [_pdfViewCtrl getPageView:self.annot.pageIndex].bounds;
        borderRect = [FreetextUtil calculateBorderRectWithInnerRect:innerRect startPoint:_startPoint kneePoint:_kneePoint];
        if ( !CGRectContainsRect(realPageRect, borderRect) ) {
            return NO;
        }
        self.annot.fsrect = [_pdfViewCtrl convertPageViewRectToPdfRect:borderRect pageIndex:pageIndex];
        
        [freetext setInnerRect:[_pdfViewCtrl convertPageViewRectToPdfRect:innerRect pageIndex:pageIndex]];
        FSPointFArray *points = [[FSPointFArray alloc] init];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:_startPoint pageIndex:pageIndex]];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:_kneePoint pageIndex:pageIndex]];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:_endPoint pageIndex:pageIndex]];
        [freetext setCalloutLinePoints:points];
        
        
        // change annot's appearance and refresh
        CGRect oldRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:pageIndex];
        newRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
        CGRect allRect = CGRectUnion(newRect, oldRect);
        allRect = CGRectInset(allRect, -30, -30);
        dispatch_async(dispatch_get_main_queue(), ^{
            [self->_pdfViewCtrl lockRefresh];
            [self.annot resetAppearanceStream];
            [self->_pdfViewCtrl unlockRefresh];
            [self->_pdfViewCtrl refresh:allRect pageIndex:pageIndex needRender:NO];
            self.annotImage = [Utility getAnnotImage:self.annot pdfViewCtrl:self->_pdfViewCtrl];
        });
    }else if (recognizer.state == UIGestureRecognizerStateEnded ||
              recognizer.state == UIGestureRecognizerStateCancelled){
        if (!self.annot) {
            return YES;
        }
        
        if (!self.isLongPressingMoved) {
            CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
            [self refreshAnnotWhenLongPressing:point];
            return YES;
        }
        
        //state ended or candelled
        CGRect newRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:self.annot.pageIndex];
        CGRect oldRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:self.annot.pageIndex];
        
        newRect = CGRectUnion(newRect, oldRect);
        newRect = CGRectInset(newRect, -30, -30);

        [self addTextViewWhenLongPress];
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            self.isLongPressing = NO;
            self.annotImage = nil;
            [self->_pdfViewCtrl refresh:newRect pageIndex:self.annot.pageIndex needRender:YES];
        });
        self.isLongPressingMoved = NO;
    }
    return YES;
}

- (void)saveCalloutPoints:(FSFreeText *)annot {
    FSPointFArray *points = [annot getCalloutLinePoints];
    int pageindex = annot.pageIndex;
    _startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:0] pageIndex:pageindex];
    _kneePoint = [_pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:1] pageIndex:pageindex];
    _endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:2] pageIndex:pageindex];
}

- (BOOL)adjustCalloutDragpoint:(FSFreeText*)annot translationPoint:(CGPoint)translationPoint {
    CGFloat tw = translationPoint.x;
    CGFloat th = translationPoint.y;

    [self saveCalloutPoints:annot];
    _startPoint.x += tw;
    _startPoint.y += th;
    
    CGRect innerRect = [_pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:annot.pageIndex];
    CGRect realPageRect = [_pdfViewCtrl getPageView:annot.pageIndex].bounds;
    if ( CGRectContainsPoint(innerRect, _startPoint) || (!CGRectContainsPoint(realPageRect, _startPoint)) ) {
        return NO;
    }
    
    if (_startPoint.y < innerRect.origin.y) {
        //top
        _endPoint.x = innerRect.origin.x + innerRect.size.width / 2.0;
        _endPoint.y = innerRect.origin.y;
        _kneePoint.x = _endPoint.x;
        _kneePoint.y = _endPoint.y - _polylineLength;
    } else if (_startPoint.y > innerRect.origin.y + innerRect.size.height) {
        //bottom
        _endPoint.x = innerRect.origin.x + innerRect.size.width / 2.0;
        _endPoint.y = innerRect.origin.y + innerRect.size.height;
        _kneePoint.x = _endPoint.x;
        _kneePoint.y = _endPoint.y + _polylineLength;
    } else if (_startPoint.x < innerRect.origin.x) {
        //left
        _endPoint.x = innerRect.origin.x;
        _endPoint.y = innerRect.origin.y + innerRect.size.height / 2.0;
        _kneePoint.x = _endPoint.x - _polylineLength;
        _kneePoint.y = _endPoint.y;
    } else {
        //right
        _endPoint.x = innerRect.origin.x + innerRect.size.width;
        _endPoint.y = innerRect.origin.y + innerRect.size.height / 2.0;
        _kneePoint.x = _endPoint.x + _polylineLength;
        _kneePoint.y = _endPoint.y;
    }
    return YES;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (BOOL)addAnnot:(int)pageIndex point:(CGPoint)point {
    return [self addAnnot:pageIndex point:point longPress:NO];
}

- (BOOL)addAnnot:(int)pageIndex point:(CGPoint)point longPress:(BOOL)longPress{
    self.pageIndex = pageIndex;
//    if (self.annot) {
//        [self save];
//        return YES;
//    }
    BOOL isEditing = (_textView && !_isSaved);
    [self save];
    _isSaved = NO;
    
    if (_extensionsManager.currentToolHandler == self) {
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGRect pageRect = pageView.bounds;
        CGPoint startPoint = point;
        CGPoint kneePoint;
        CGPoint endPoint;
        
        
        FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:self.pageIndex];
        self.annot = [[FSFreeText alloc] initWithAnnot:[page addAnnot:FSAnnotFreeText rect:[_pdfViewCtrl convertPageViewRectToPdfRect:CGRectMake(0, 0, 1, 1) pageIndex:pageIndex]]];
        //appearance
        FSDefaultAppearance *appearance = [self.annot getDefaultAppearance];
        appearance.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
        NSString *fontName = self.toolSettings.textFace;
        int fontID = [Utility toStandardFontID:fontName];
        if (fontID == -1) {
            [appearance setFont:[[FSFont alloc] initWithName:fontName styles:0 charset:FSFontCharsetDefault weight:0]];
        } else {
            [appearance setFont:[[FSFont alloc] initWithFont_id:fontID]];
        }
        [appearance setText_size:self.toolSettings.textSize];
        [appearance setText_color:self.toolSettings.settingsColor];
        [self.annot setBorderColor:_extensionsManager.config.defaultSettings.annotations.callout.color.rgbValue];
        [self.annot setDefaultAppearance:appearance];
        
        //font
        float fontSize = [appearance getText_size] ?: 10;
        fontSize = [Utility convertWidth:fontSize fromPDFToPageView:_pdfViewCtrl pageIndex:self.pageIndex];
        FSFont *fsfont = [appearance getFont];
        if (fsfont && ![fsfont isEmpty]) {
            fontName = [fsfont getName];
        }
        if (fontName.length == 0) {
            fontName = @"Helvetica";
        }
        UIFont *font = [self getSysFont:fontName size:fontSize];
        CGSize textSize = [Utility getTestSize:font];
        
        //set kneePoint, endPoint, textView frame
        CGFloat textViewWidth = 100.0;
        CGRect textFrame;
        if (!longPress) {
            if (startPoint.y < pageRect.size.height / 2.0) {
                if (startPoint.x < pageRect.size.width / 2.0) {
                    kneePoint.x = startPoint.x + 50.0;
                    kneePoint.y = startPoint.y + 50.0;
                    endPoint.x = kneePoint.x;
                    endPoint.y = kneePoint.y + 20.0;
                } else {
                    kneePoint.x = startPoint.x - 50.0;
                    kneePoint.y = startPoint.y + 50.0;
                    endPoint.x = kneePoint.x;
                    endPoint.y = kneePoint.y + 20.0;
                }
                textFrame = CGRectMake(endPoint.x - textViewWidth / 2.0, endPoint.y, textViewWidth, textSize.height);
            } else {
                if (startPoint.x < pageRect.size.width / 2.0) {
                    kneePoint.x = startPoint.x + 50.0;
                    kneePoint.y = startPoint.y - 50.0;
                    endPoint.x = kneePoint.x;
                    endPoint.y = kneePoint.y - 20.0;
                } else {
                    kneePoint.x = startPoint.x - 50.0;
                    kneePoint.y = startPoint.y - 50.0;
                    endPoint.x = kneePoint.x;
                    endPoint.y = kneePoint.y - 20.0;
                }
                textFrame = CGRectMake(endPoint.x - textViewWidth / 2.0, endPoint.y - textSize.height, textViewWidth, textSize.height);
            }
        }else{

            kneePoint = startPoint;
            endPoint = startPoint;
            textFrame = CGRectMake(startPoint.x , startPoint.y , textViewWidth, textSize.height);
        }
        
        if (!longPress) {
            //set text view properties
            _textView = [[FSEasyDoneTextView alloc] initWithFrame:textFrame];
            _textView.accessibilityIdentifier = @"calloutaccessibilityID";
            @fs_weakify(self)
            _textView.doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                [self save];
            };
            _textView.font = font;
            [FreetextUtil setTextViewProperties:_textView UIExtensionsManager:_extensionsManager toolSettings:self.toolSettings isCallout:YES];
            CGRect frame = _textView.frame;
            CGSize constraintSize = CGSizeMake(frame.size.width, MAXFLOAT);
            CGSize size = [_textView sizeThatFits:constraintSize];
            _textView.frame = CGRectMake(frame.origin.x, frame.origin.y, frame.size.width, size.height);
            textFrame = _textView.frame;
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(keyboardWasShown:)
                                                         name:UIKeyboardDidShowNotification
                                                       object:nil];
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(keyboardWasHidden:)
                                                         name:UIKeyboardWillHideNotification
                                                       object:nil];
            _textView.delegate = self;
            [pageView addSubview:_textView];
            
            
            [_textView becomeFirstResponder];
        }
        
        //set annot
        CGRect innerRect = textFrame;
        CGRect borderRect = [FreetextUtil calculateBorderRectWithInnerRect:innerRect startPoint:startPoint kneePoint:kneePoint];
        
        self.annot.fsrect = [_pdfViewCtrl convertPageViewRectToPdfRect:borderRect pageIndex:pageIndex];
        self.annot.NM = [Utility getUUID];
        self.annot.author = _extensionsManager.annotAuthor;
        self.annot.opacity = self.toolSettings.opacity;
//        NSString *content = _textView.text;
        self.annot.contents = nil;
        self.annot.createDate = [NSDate date];
        self.annot.modifiedDate = [NSDate date];
        self.annot.subject = @"Callout";
        self.annot.intent = @"FreeTextCallout";
        self.annot.flags = FSAnnotFlagPrint;

        [self.annot setInnerRect:[_pdfViewCtrl convertPageViewRectToPdfRect:innerRect pageIndex:pageIndex]];
        FSPointFArray *points = [[FSPointFArray alloc] init];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:startPoint pageIndex:pageIndex]];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:kneePoint pageIndex:pageIndex]];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:endPoint pageIndex:pageIndex]];
        [self.annot setCalloutLinePoints:points];
        [self.annot setCalloutLineEndingStyle:FSMarkupEndingStyleOpenArrow];
        
        int rotation = ([page getRotation] +[_pdfViewCtrl getViewRotation]) % 4;
        rotation = rotation == 0 ? rotation : 4 - rotation;
        self.annot.rotation = rotation ;
        
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        
        
        if (self.annot) {
            id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:self.annot];
            if (longPress) {
                if ([annotHandler isKindOfClass: NSClassFromString(@"FtAnnotHandler")]){
                    ((FtAnnotHandler *)annotHandler).longPressingAnnot = self.annot;
                }
            }
            [annotHandler addAnnot:self.annot addUndo:NO];
        }
        return YES;
    }
    if (isEditing) {
        return YES;
    }
    return NO;
}


- (BOOL)refreshAnnotWhenLongPressing:(CGPoint)point{
    if (!self.annot) {
        return NO;
    }

    if (_extensionsManager.currentToolHandler == self) {
        UIView *pageView = [_pdfViewCtrl getPageView:self.pageIndex];
        CGRect pageRect = pageView.bounds;
        CGPoint startPoint = point;
        CGPoint kneePoint;
        CGPoint endPoint;
        
        FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:self.pageIndex];
        
        //appearance
        FSDefaultAppearance *appearance = [self.annot getDefaultAppearance];
        NSString *fontName = self.toolSettings.textFace;
        
        //font
        float fontSize = [appearance getText_size] ?: 10;
        fontSize = [Utility convertWidth:fontSize fromPDFToPageView:_pdfViewCtrl pageIndex:self.pageIndex];
        FSFont *fsfont = [appearance getFont];
        if (fsfont && ![fsfont isEmpty]) {
            fontName = [fsfont getName];
        }
        if (fontName.length == 0) {
            fontName = @"Helvetica";
        }
        UIFont *font = [self getSysFont:fontName size:fontSize];
        CGSize textSize = [Utility getTestSize:font];
        
        //set kneePoint, endPoint, textView frame
        CGFloat textViewWidth = 100.0;
        CGRect textFrame;
        
        if (startPoint.y < pageRect.size.height / 2.0) {
            if (startPoint.x < pageRect.size.width / 2.0) {
                kneePoint.x = startPoint.x + 50.0;
                kneePoint.y = startPoint.y + 50.0;
                endPoint.x = kneePoint.x;
                endPoint.y = kneePoint.y + 20.0;
            } else {
                kneePoint.x = startPoint.x - 50.0;
                kneePoint.y = startPoint.y + 50.0;
                endPoint.x = kneePoint.x;
                endPoint.y = kneePoint.y + 20.0;
            }
            textFrame = CGRectMake(endPoint.x - textViewWidth / 2.0, endPoint.y, textViewWidth, textSize.height);
        } else {
            if (startPoint.x < pageRect.size.width / 2.0) {
                kneePoint.x = startPoint.x + 50.0;
                kneePoint.y = startPoint.y - 50.0;
                endPoint.x = kneePoint.x;
                endPoint.y = kneePoint.y - 20.0;
            } else {
                kneePoint.x = startPoint.x - 50.0;
                kneePoint.y = startPoint.y - 50.0;
                endPoint.x = kneePoint.x;
                endPoint.y = kneePoint.y - 20.0;
            }
            textFrame = CGRectMake(endPoint.x - textViewWidth / 2.0, endPoint.y - textSize.height, textViewWidth, textSize.height);
        }
        
        
        //set text view properties
        _textView = [[FSEasyDoneTextView alloc] initWithFrame:textFrame];
        _textView.accessibilityIdentifier = @"calloutaccessibilityID";
        @fs_weakify(self)
        _textView.doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @fs_strongify(self)
            [self save];
        };
        _textView.font = font;
        [FreetextUtil setTextViewProperties:_textView UIExtensionsManager:_extensionsManager toolSettings:self.toolSettings isCallout:YES];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasShown:)
                                                     name:UIKeyboardDidShowNotification
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasHidden:)
                                                     name:UIKeyboardWillHideNotification
                                                   object:nil];
        _textView.delegate = self;
        [pageView addSubview:_textView];
        
        [_textView becomeFirstResponder];
        
        //set annot
        CGRect innerRect = textFrame;
        CGRect borderRect = [FreetextUtil calculateBorderRectWithInnerRect:innerRect startPoint:startPoint kneePoint:kneePoint];
        
        self.annot.fsrect = [_pdfViewCtrl convertPageViewRectToPdfRect:borderRect pageIndex:self.pageIndex];
        self.annot.NM = [Utility getUUID];
        self.annot.author = _extensionsManager.annotAuthor;
        self.annot.opacity = self.toolSettings.opacity;
        //        NSString *content = _textView.text;
        self.annot.contents = nil;
        self.annot.createDate = [NSDate date];
        self.annot.modifiedDate = [NSDate date];
        self.annot.subject = @"Callout";
        self.annot.intent = @"FreeTextCallout";
        self.annot.flags = FSAnnotFlagPrint;
        
        [self.annot setInnerRect:[_pdfViewCtrl convertPageViewRectToPdfRect:innerRect pageIndex:self.pageIndex]];
        FSPointFArray *points = [[FSPointFArray alloc] init];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:startPoint pageIndex:self.pageIndex]];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:kneePoint pageIndex:self.pageIndex]];
        [points add:[_pdfViewCtrl convertPageViewPtToPdfPt:endPoint pageIndex:self.pageIndex]];
        [self.annot setCalloutLinePoints:points];
        [self.annot setCalloutLineEndingStyle:FSMarkupEndingStyleOpenArrow];
        
        int rotation = ([page getRotation] +[_pdfViewCtrl getViewRotation]) % 4;
        rotation = rotation == 0 ? rotation : 4 - rotation;
        self.annot.rotation = rotation ;
        
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        
        id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:self.annot];
        if ([annotHandler isKindOfClass: NSClassFromString(@"FtAnnotHandler")]){
            ((FtAnnotHandler *)annotHandler).longPressingAnnot = nil;
        }
        
        CGRect refreshRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:self.annot.pageIndex];
        [_pdfViewCtrl refresh:refreshRect pageIndex:self.annot.pageIndex];
        
        return YES;
    }

    return NO;
}

- (void)addTextViewWhenLongPress{
    
    if (!self.annot || [self.annot isEmpty] || self.pageIndex < 0) {
        return;
    }
    UIView *pageView = [_pdfViewCtrl getPageView:self.pageIndex];
    //appearance
    FSDefaultAppearance *appearance = [self.annot getDefaultAppearance];
    NSString *fontName = self.toolSettings.textFace;
    //font
    float fontSize = [appearance getText_size] ?: 10;
    fontSize = [Utility convertWidth:fontSize fromPDFToPageView:_pdfViewCtrl pageIndex:self.pageIndex];
    FSFont *fsfont = [appearance getFont];
    if (fsfont && ![fsfont isEmpty]) {
        fontName = [fsfont getName];
    }
    if (fontName.length == 0) {
        fontName = @"Helvetica";
    }
    UIFont *font = [self getSysFont:fontName size:fontSize];

    CGRect textFrame;
    textFrame = [_pdfViewCtrl convertPdfRectToPageViewRect:[self.annot getInnerRect] pageIndex:self.pageIndex];
    _textView = [[FSEasyDoneTextView alloc] initWithFrame:textFrame];
    _textView.accessibilityIdentifier = @"calloutaccessibilityID";
    @weakify(self)
    
    _textView.doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self save];
    };
    _textView.font = font;
    [FreetextUtil setTextViewProperties:_textView UIExtensionsManager:_extensionsManager toolSettings:self.toolSettings isCallout:YES];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWasShown:)
                                                 name:UIKeyboardDidShowNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWasHidden:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
    _textView.delegate = self;
    [pageView addSubview:_textView];
    
    
    [_textView becomeFirstResponder];
    
    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:self.annot];
    if ([annotHandler isKindOfClass: NSClassFromString(@"FtAnnotHandler")]){
        ((FtAnnotHandler *)annotHandler).longPressingAnnot = nil;
    }

}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint startPoint = [recognizer locationInView:pageView];
    return [self addAnnot:pageIndex point:startPoint];
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer
{
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context{

    if (!self.annot || [self.annot isEmpty] || pageIndex != self.annot.pageIndex) {
        return;
    }

    if (pageIndex == self.annot.pageIndex) {
        self.oldRect = self.annot.fsrect;
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:self.annot];
        [PaintUtility drawCallout:freetext annotImage:self.annotImage inContext:context pdfViewCtrl:_pdfViewCtrl isLineDash:NO];
    }
}

- (UIFont *)getSysFont:(NSString *)name size:(float)size {
    UIFont *font = [UIFont fontWithName:[Utility convert2SysFontString:name] size:size];
    if (!font) {
        font = [UIFont systemFontOfSize:size];
    }
    return font;
}

#pragma mark - UITextViewDelegate
- (void)textViewDidChange:(UITextView *)textView {
    UIView *pageView = [_pdfViewCtrl getPageView:self.annot.pageIndex];
    if (pageView) {
        [FreetextUtil adjustTextViewFrame:textView inPageView:pageView forMinPageInset:minAnnotPageInset];
    }
    
    CGRect refreshRect = [FreetextUtil resetCalloutAppearanceWhenTextviewChanged:_pdfViewCtrl annot:self.annot textFrame:textView.frame];
    textView.frame = [_pdfViewCtrl convertPdfRectToPageViewRect:[self.annot getInnerRect] pageIndex:self.annot.pageIndex];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self->_pdfViewCtrl refresh:refreshRect pageIndex:self.annot.pageIndex needRender:NO];
    });
}

- (void)textViewDidEndEditing:(UITextView *)textView{
    [self save];
}

#pragma mark - keyboard

- (void)keyboardWasShown:(NSNotification*)notification
{
    if (_keyboardShown)
        return;
    _keyboardShown = YES;
    [FreetextUtil afterKeyboardShown:notification textView:_textView UIExtensionsManager:_extensionsManager pageIndex:self.pageIndex];
}

- (void)keyboardWasHidden:(NSNotification*)notification
{
    _keyboardShown = NO;
    [FreetextUtil adjustBottomOffset:_pdfViewCtrl pageIndex:self.pageIndex];
}

- (void)onPageChangedFrom:(int)oldIndex to:(int)newIndex
{
    [self save];
}

- (void)save
{

    if (!self.annot) {
        return;
    }
    if (_textView && !_isSaved) {
        _isSaved = YES;

        NSString *checker = [NSString stringWithFormat:@"%C", 8198];
        if ([_textView.text rangeOfString:checker].length) {
            _textView.text  = [_textView.text stringByReplacingOccurrencesOfString:checker withString:@""];
        }
        
        self.annot.contents = _textView.text;
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        
        id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:self.annot];
        FSPDFPage *page = [self.annot getPage];
        if (page && ![page isEmpty]) {
            [annotHandler modifyAnnot:self.annot addUndo:NO];
            FSAnnotAttributes *attributes = [FSAnnotAttributes attributesWithAnnot:self.annot];
            [_extensionsManager addUndoItem:[UndoItem itemForUndoAddAnnotWithAttributes:attributes page:page annotHandler:annotHandler]];
        }

        CGRect borderRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:self.annot.pageIndex];
        [_pdfViewCtrl refresh:borderRect pageIndex:self.annot.pageIndex];
        
        if (!_extensionsManager.continueAddAnnot) {
            [_extensionsManager setCurrentToolHandler:nil];
        }
        
        [_textView resignFirstResponder];
        [_textView removeFromSuperview];
        _textView = nil;
        self.annot = nil;
        
        [[NSNotificationCenter defaultCenter] removeObserver:self];
    }
}

- (UIView*)hitTest:(CGPoint)point withEvent:(UIEvent*)event onView:(UIView*)view
{
    if (_textView != nil) {
        CGPoint pt = [_textView convertPoint:point fromView:view];
        if (CGRectContainsPoint(_textView.bounds, pt)) {
            return _textView;
        }
    }
    return nil;
}

//-(CGPoint)rotateVec:(float)px py:(float)py ang:(float)ang isChlen:(BOOL)isChlen newLine:(float)newLen
//{
//    CGPoint point = CGPointMake(0, 0);
//    float vx = px * cosf(ang) - py * sinf(ang);
//    float vy = px * sinf(ang) + py * cosf(ang);
//    if (isChlen) {
//        float d = sqrtf(vx * vx + vy * vy);
//        vx = vx / d * newLen;
//        vy = vy / d * newLen;
//        point.x = vx;
//        point.y = vy;
//    }
//    return point;
//}

#pragma mark <IFSUndoEventListener>

- (void)onWillRedo {
    if (self.annot) {
        [self save];
    }
}

- (void)onWillUndo {
    if (self.annot) {
        [self save];
    }
}

#pragma mark -- IRotationEventListener
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation{
    [self save];
}

#pragma mark -- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD) {
        [self save];
    }
}

#pragma mark -- IPageEventListener
- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex{
    [self save];
}

#pragma mark <IScrollViewEventListener>
- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView {

    if (_textView && self.annot) {
        
        FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.annot];
        BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
        int pageIndex = annot.pageIndex;
        if (isCallout) {
            _textView.frame = [_pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:pageIndex];
        } else {
            _textView.frame = [_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        }
        
        //appearance
        FSDefaultAppearance *appearance = [self.annot getDefaultAppearance];
        NSString *fontName = self.toolSettings.textFace;
        
        //font
        float fontSize = [appearance getText_size] ?: 10;
        fontSize = [Utility convertWidth:fontSize fromPDFToPageView:_pdfViewCtrl pageIndex:self.pageIndex];
        FSFont *fsfont = [appearance getFont];
        if (fsfont && ![fsfont isEmpty]) {
            fontName = [fsfont getName];
        }
        if (fontName.length == 0) {
            fontName = @"Helvetica";
        }
        UIFont *font = [self getSysFont:fontName size:fontSize];
        
        _textView.font = font;
        [[_pdfViewCtrl getPageView:_pdfViewCtrl.getCurrentPage] addSubview:_textView];
    }
}

#pragma mark --IPropertyViewEventListener
- (void)onPropertyViewDidShow{
    [self save];
}

@synthesize toolSettings;

@end
