/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAnnotExtent.h"
#import "FSAnnotAttributes.h"

FSRectF *convertToFSRect(FSPointF *p1, FSPointF *p2) {
    FSRectF *rect = [[FSRectF alloc] init];
    rect.left = MIN([p1 getX], [p2 getX]);
    rect.right = MAX([p1 getX], [p2 getX]);
    rect.top = MAX([p1 getY], [p2 getY]);
    rect.bottom = MIN([p1 getY], [p2 getY]);
    return rect;
}

@implementation FSAnnot (useProperties)

- (FSPDFDoc *)document {
    return [[self getPage] getDocument];
}

- (int)pageIndex {
    if ([self isEmpty]) return -1;
    return [[self getPage] getIndex];
}

- (FSAnnotType)type {
    if ([self isEmpty]) return FSAnnotUnknownType;
    return [self getType];
}

- (FSRectF *)fsrect {
    return [self getRect];
}

- (void)setFsrect:(FSRectF *)fsrect {
    //Check if rect is valid.
    if(fsrect.right - fsrect.left <= 1E-5 || fsrect.top-fsrect.bottom <= 1E-5)
        return;
    if ([self getFilterAnnotType] != FSAnnotAreaHighlight) {
        if ([self getFilterAnnotType] == FSAnnotStamp){
            [self move:fsrect is_reset_appearance:NO];
        }else{
            [self move:fsrect];
        }
    }else{
        FSPointF *first = [[FSPointF alloc] initWithX:fsrect.left Y:fsrect.top];
        FSPointF *second = [[FSPointF alloc] initWithX:fsrect.right Y:fsrect.top];
        FSPointF *third = [[FSPointF alloc] initWithX:fsrect.left Y:fsrect.bottom];
        FSPointF *fourth = [[FSPointF alloc] initWithX:fsrect.right Y:fsrect.bottom];
        FSQuadPoints *fsqps = [[FSQuadPoints alloc] initWithFirst:first second:second third:third fourth:fourth];
        self.quads = @[fsqps];
    }
}

- (unsigned int)color {
    if (self.type == FSAnnotFreeText) {
        FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self];
        FSDefaultAppearance *ap = [annot getDefaultAppearance];
        return ap.text_color;
    }else if (self.type == FSAnnotWidget) {
        FSWidget *annot = [[FSWidget alloc] initWithAnnot:self];
        FSControl *control = [annot getControl];
        FSDefaultAppearance *ap = [control getDefaultAppearance];
        return ap.text_color;
    } else {
        return [self getBorderColor];
    }
}

- (void)setColor:(unsigned int)color {
    if (self.type == FSAnnotFreeText) {
        FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self];
        FSDefaultAppearance *ap = [annot getDefaultAppearance];
        ap.text_color = color;
        [annot setDefaultAppearance:ap];
    }else if (self.type == FSAnnotWidget) {
        FSWidget *annot = [[FSWidget alloc] initWithAnnot:self];
        FSControl *control = [annot getControl];
        FSDefaultAppearance *ap = [control getDefaultAppearance];
        ap.text_color = color;
        [control setDefaultAppearance:ap];
    }
    else {
        [self setBorderColor:color];
    }
}

- (unsigned int)exfillColor
{
    if (![self hasProperty:FSAnnotPropertyFillColor]) return 0xFFFFFF00;
    if (self.type == FSAnnotCloud || self.type == FSAnnotPolygon) {
        FSPolygon *annot = [[FSPolygon alloc] initWithAnnot:self];
        return annot.fillColor;
    }else if (self.type == FSAnnotSquare) {
        FSSquare *annot = [[FSSquare alloc] initWithAnnot:self];
        return annot.fillColor;
    }else if (self.type == FSAnnotCircle) {
        FSCircle *annot = [[FSCircle alloc] initWithAnnot:self];
        return annot.fillColor;
    }  else {
        return self.color;
    }
}

- (void)setExfillColor:(unsigned int)exfillColor
{
    if ([self hasProperty:FSAnnotPropertyFillColor] && exfillColor == 0xFFFFFF00) {
        [self removeProperty:FSAnnotPropertyFillColor];
        return;
    }
    if (exfillColor == 0xFFFFFF00){
        return;
    }
    if (self.type == FSAnnotCloud || self.type == FSAnnotPolygon) {
        FSPolygon *annot = [[FSPolygon alloc] initWithAnnot:self];
        annot.fillColor = exfillColor;
    }else if (self.type == FSAnnotSquare) {
        FSSquare *annot = [[FSSquare alloc] initWithAnnot:self];
        annot.fillColor = exfillColor;
    }else if (self.type == FSAnnotCircle) {
        FSCircle *annot = [[FSCircle alloc] initWithAnnot:self];
        annot.fillColor = exfillColor;
    }  else {
        return [self setColor:exfillColor];
    }
}

- (float)lineWidth {
    return [[self getBorderInfo] getWidth];
}
- (void)setLineWidth:(float)lineWidth {
    FSBorderInfo *borderInfo = [self getBorderInfo];
    [borderInfo setWidth:lineWidth];
    [self setBorderInfo:borderInfo];
}
- (unsigned int)flags {
    return [self getFlags];
}

- (NSString *)subject {
    if ([self isEmpty]) return nil;
    if (![self isMarkup])
        return nil;
    return [[[FSMarkup alloc] initWithAnnot:self] getSubject];
}

- (void)setSubject:(NSString *)subject {
    if (![self isMarkup])
        return;
    return [[[FSMarkup alloc] initWithAnnot:self] setSubject:subject];
}

- (NSString *)NM {
    NSString *NM = nil;
    @try {
        NM = [self getUniqueID];
    } @catch (NSException *exception) {
    }
    if (!NM.length) {
        NM = [NSString stringWithFormat:@"%d_%d", [[self getPage] getIndex],[self.getDict getObjNum]];
    }
    return NM;
}

- (void)setNM:(NSString *)NM {
    BOOL canDo = ![Utility isDocumentSigned:self.document];
    if (canDo && [Utility canAddAnnotToDocument:self.document]) canDo = self.canEdit;
    if (self.type == FSAnnotWidget && !canDo)  return;
    @try {
        [self setUniqueID:NM];
    } @catch (NSException *exception) {
        NSLog(@"failed setting annot NM: %@", exception.description);
    }
}

- (NSString *)uuidWithPageIndex {
    NSString *uniqueID = nil;
    @try {
        uniqueID = [self getUniqueID];
    } @catch (NSException *exception) {
        uniqueID = nil;
    }
    if (!uniqueID.length) {
        return self.NM;
    }
   return [[NSString stringWithFormat:@"%d_", [[self getPage] getIndex]] stringByAppendingString:uniqueID];
}

- (NSString *)replyTo {
    if ([self isEmpty]) return nil;
    if (FSAnnotNote != self.type) {
        return nil;
    }
    FSMarkup *markup = [[[FSNote alloc] initWithAnnot:self] getReplyTo];
    if (!markup || [markup isEmpty])
        return nil;
    return markup.NM;
}

- (NSArray *)replyAnnots{
    if ([self isEmpty]) return nil;
    if ([self isMarkup])
    {
        FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:self];
        NSMutableArray *replyAnnots = @[].mutableCopy;
        for (int i = 0; i < markup.getReplyCount; i++) {
            [replyAnnots addObject:[markup getReply:i]];
        }
        return replyAnnots.copy;
    }
    return nil;
}

- (NSArray *)replyAnnotsNM{
    if ([self isEmpty]) return nil;
    if ([self isMarkup])
    {
        FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:self];
        NSMutableArray *replyAnnots = @[].mutableCopy;
        for (int i = 0; i < markup.getReplyCount; i++) {
            [replyAnnots addObject:[markup getReply:i].NM];
        }
        return replyAnnots.copy;
    }
    return nil;
}

- (NSString *)author {
    if ([self isEmpty]) return nil;
    if ([self getType] == FSAnnotScreen) {
        return [[[FSScreen alloc] initWithAnnot:self] getTitle];
    }
    if (![self isMarkup])
        return nil;
    FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
    return [mk getTitle];
}
- (void)setAuthor:(NSString *)author {
    if ([self getType] == FSAnnotScreen) {
        [[[FSScreen alloc] initWithAnnot:self] setTitle:author];
    }else if ([self isMarkup]) {
        [[[FSMarkup alloc] initWithAnnot:self] setTitle:author];
    }
}
- (NSString *)contents {
    if ([self isEmpty]) return nil;
    @try {
        if (self.type == FSAnnotFreeText) {
            FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:self];
            NSMutableString *contents = @"".mutableCopy;
            if ([freeText getRichTextCount] >= 1) {
                for (int i = 0; i < [freeText getRichTextCount]; i++) {
                    [contents appendString:[freeText getRichTextContent:i]];
                }
                return contents;
            }
        }
        return [self getContent];
    } @catch (NSException *exception) {
        return nil;
    } 
    return nil;
}

- (void)setContents:(NSString *)contents {
    if (self.type == FSAnnotFreeText) {
        FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:self];
        if ([freeText getRichTextCount] == 1) {
            if (contents.length) {
                [freeText setRichTextContent:0 content:contents];
            }else{
                [freeText removeRichText:0];
            }
        }

    }
    [self setContent:contents];
}

- (NSDate *)modifiedDate {
    if ([self isEmpty]) return nil;
    @try {
    FSDateTime *dt = [self getModifiedDateTime];
    return dt ? [Utility convertFSDateTime2NSDate:dt] : nil;
    }
    @catch(NSException* e)
    {
        return nil;
    }
}
- (void)setModifiedDate:(NSDate *)modifiedDate {
    if (!modifiedDate) {
        return;
    }
    BOOL canDo = ![Utility isDocumentSigned:self.document];
    if (canDo && [Utility canAddAnnotToDocument:self.document]) canDo = self.canEdit;
    if (self.type == FSAnnotWidget && !canDo)  return;
    FSDateTime *dt = [Utility convert2FSDateTime:modifiedDate];
    [self setModifiedDateTime:dt];
}
- (NSDate *)createDate {
    if ([self isEmpty]) return nil;
    if (![self isMarkup])
        return nil;
    FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
    @try {
    FSDateTime *dt = [mk getCreationDateTime];
    return dt ? [Utility convertFSDateTime2NSDate:dt] : nil;
    }
    @catch(NSException* e)
    {
        return nil;
    }
}
- (void)setCreateDate:(NSDate *)createDate {
    if (!createDate || ![self isMarkup])
        return;
    FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
    FSDateTime *dt = [Utility convert2FSDateTime:createDate];
    [mk setCreationDateTime:dt];
}
- (NSString *)intent {
    if ([self isEmpty]) return nil;
    if (![self isMarkup])
        return nil;
    FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
    return [mk getIntent];
}
- (void)setIntent:(NSString *)intent {
    if (![self isMarkup])
        return;
    FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
    return [mk setIntent:intent];
}
- (NSString *)selectedText {
    if ([self isEmpty]) return nil;
    FSPDFPage *_fspage = [self getPage];
    FSTextPage *textPage = nil;
    BOOL parseSuccess = [Utility parsePage:_fspage flag:FSPDFPageParsePageTextOnly pause:nil];
    if (parseSuccess) {
        textPage = [[FSTextPage alloc] initWithPage:_fspage flags:FSTextPageParseTextNormal];
    }

    NSString *selectedText = @"";
    if (textPage) {
        NSArray *array = self.quads;
        for (int i = 0; i < array.count; i++) {
            FSQuadPoints *arrayQuad = [array objectAtIndex:i];
            FSRectF *rect = convertToFSRect(arrayQuad.getFirst, arrayQuad.getFourth);
            NSString *tmp = [textPage getTextInRect:rect];
            if (tmp) {
                selectedText = [selectedText stringByAppendingString:tmp];
            }
        }
    }
    return selectedText;
}
- (NSArray *)quads {
    if ([self isEmpty]) return nil;
    FSQuadPointsArray *quads = nil;
    FSAnnotType type = [self getType];
    if (type == FSAnnotHighlight || type == FSAnnotUnderline || type == FSAnnotStrikeOut || type == FSAnnotSquiggly) {
        FSTextMarkup *markup = [[FSTextMarkup alloc] initWithAnnot:self];
        quads = [markup getQuadPoints];
    } else if (type == FSAnnotLink) {
        FSLink *link = [[FSLink alloc] initWithAnnot:self];
        quads = [link getQuadPoints];
    } else if (type == FSAnnotRedact) {
        FSRedact *redact = [[FSRedact alloc] initWithAnnot:self];
        quads = [redact getQuadPoints];
    }else {
        return nil;
    }
    NSMutableArray *array = [NSMutableArray array];
    for (int i = 0; i < [quads getSize]; i++) {
        FSQuadPoints *quadPoints = [quads getAt:i];
        FSQuadPoints *clone = [[FSQuadPoints alloc] initWithQuad_points:quadPoints];
        [array addObject:clone];
    }
    return array;
}
- (void)setQuads:(NSArray *)quads {
    if (quads.count == 0)
        return;

    FSAnnotType type = [self getType];
    if (FSAnnotHighlight != type && FSAnnotUnderline != type && FSAnnotStrikeOut != type && FSAnnotSquiggly != type && FSAnnotLink != type && FSAnnotRedact != type)
        return;
    FSQuadPointsArray *array = [[FSQuadPointsArray alloc] init];
    for (FSQuadPoints *quad in quads) {
        [array add:quad];
    }
    if (FSAnnotLink == type) {
        [[FSLink alloc] initWithAnnot:self].quadPoints = array;
    }else if (FSAnnotRedact == type) {
        [[FSRedact alloc] initWithAnnot:self].quadPoints = array;
    }else {
        [[FSTextMarkup alloc] initWithAnnot:self].quadPoints = array;
    }
}

- (BOOL)simulatePSI{
    if (![self isEmpty] && self.getType == FSAnnotInk) {
        FSPDFDictionary *annotDict = self.getDict;
        if ([annotDict hasKey:@"FXInkType"]) {
            FSPDFObject *FXInkType = [annotDict getElement:@"FXInkType"];
            NSString *PSI = [FXInkType getWideString];
            if ([PSI isEqualToString:@"PSI"]) {
                return YES;
            }
        }
    }
    return NO;
}

- (void)setSimulatePSI:(BOOL)simulatePSI{
    if (![self isEmpty] && self.getType == FSAnnotInk) {
        FSPDFDictionary *annotDict = self.getDict;
        if (simulatePSI) {
            [annotDict setAtString:@"FXInkType" value:@"PSI"];
        }else{
            [annotDict removeAt:@"FXPressureList"];
        }
    }
}

- (FSMarkup *)canGetGroupHeader {
    if ([self isEmpty]) return nil;
    if (self.isMarkup) {
        FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:self];
        if (markup.isGrouped){
            if (markup.getGroupHeader && ![markup.getGroupHeader isEmpty]) {
                return markup.getGroupHeader;
            }else{
                if ([markup getGroupElements].getSize > 1) {
                    [markup.getPage setAnnotGroup:[markup getGroupElements] header_index:0];
                    return [[markup getGroupElements] getAt:0];
                }else{
                    [markup ungroup];
                }
            }
        }
    }
    return nil;
}

- (FSMarkupArray *)canGetGroupElements {
    if ([self isEmpty]) return nil;
    FSMarkup *groupHeader = self.canGetGroupHeader;
    if (groupHeader && ![groupHeader isEmpty]) {
        return [groupHeader getGroupElements];
    }
    return nil;
}

- (BOOL)canReplyGroup{
    if (self.canReply) {
        FSMarkupArray *groupElements = self.canGetGroupElements;
        if (groupElements) {
            for (int i = 0; i < groupElements.getSize; i++) {
                FSMarkup *markup = [groupElements getAt:i];
                if (!markup.canReply) return NO;
            }
            return YES;
        }
        return YES;
    }
    return NO;
}

- (BOOL)nonempty{
    return ![self isEmpty];
}

- (BOOL)canGroup{
    return !(self.flags & FSAnnotFlagLocked) && self.canModify;
}

- (BOOL)canReply {
    if (![self isMarkup] ||
        self.type == FSAnnotFreeText ||
        self.type == FSAnnotFileAttachment ||
        self.type == FSAnnotSound ||
        self.flags & FSAnnotFlagReadOnly ||
        ![self canModify]) {
        return NO;
    }
    return YES;
}

- (BOOL)canModify {
    if ([self isEmpty]) return NO;
    FSAnnotType fileType = [self getFilterAnnotType];
    if (fileType == FSAnnotAudio || fileType == FSAnnotVideo || [self getType] == FSAnnotLink) {
        return [Utility canAddEditAnnotToDocument:self.document];
    }else{
        return [Utility canAddAnnotToDocument:self.document];
    }
}

- (BOOL)canModifyWithExternsions:(UIExtensionsManager*)externsionsManager {
    if (![externsionsManager annotsPermissionCanModify:self]) {
        return NO;
    }
    return [self canModify];
}

- (BOOL)canCopyText {
    return [Utility canCopyTextInDocument:self.document];
}

- (BOOL)canApplyRedaction {
    return self.type == FSAnnotRedact && [Utility canApplyRedaction:self.document];
}

- (BOOL)canDisplay {
    return YES;
}

- (BOOL)canPrintable {
    return YES;
}

- (BOOL)canDoPreference{
    return !(self.flags & FSAnnotFlagLocked || self.flags & FSAnnotFlagReadOnly || self.type == FSAnnotSound);
}

- (BOOL)canMove {
    return self.canDoPreference;
}

- (BOOL)canMoveWithExternsions:(UIExtensionsManager*)externsionsManager {
    if (![externsionsManager annotsPermissionCanModify:self]) {
        return NO;
    }
    return self.canDoPreference;
}

- (BOOL)canResize {
    return self.canDoPreference && self.canModify;
}

- (BOOL)canResizeWithExternsions:(UIExtensionsManager*)externsionsManager {
    if (![externsionsManager annotsPermissionCanModify:self]) {
        return NO;
    }
    return self.canDoPreference;
}

- (BOOL)canRotate {
    return self.canModify && self.type == FSAnnotStamp && self.canDoPreference;
}

- (BOOL)canRotateWithExternsions:(UIExtensionsManager*)externsionsManager {
    if (![externsionsManager annotsPermissionCanModify:self]) {
        return NO;
    }
    return self.type == FSAnnotStamp && self.canDoPreference;
}

- (BOOL)canDelete {
    return self.canModify && self.canDoPreference;
}

- (BOOL)canDeleteWithExternsions:(UIExtensionsManager*)externsionsManager {
    
    if (self.type == FSAnnotRedact && !externsionsManager.isSupportRedaction) {
        return NO;
    }
    
    if (![externsionsManager annotsPermissionCanDelete:self]) {
        return NO;
    }
    return self.canModify && self.canDoPreference;
}

- (BOOL)canEdit {
    return self.canModify && self.canDoPreference;
}

- (BOOL)canEditWithExternsions:(UIExtensionsManager*)externsionsManager {
    if (![externsionsManager annotsPermissionCanModify:self]) {
        return NO;
    }
    return self.canModify && self.canDoPreference;
}

- (BOOL)canModifyAppearanceWithExternsions:(UIExtensionsManager*)externsionsManager {
    if (![externsionsManager annotsPermissionCanModify:self]) {
        return NO;
    }
    return [self canModifyAppearance];
}

- (BOOL)canModifyAppearance {
    return self.canModify && self.canDoPreference;
}

- (BOOL)canView {
    return YES;
}

- (float)opacity {
    if ([self isMarkup]) {
        FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
        return [mk getOpacity];
    } else if ([self getType] == FSAnnotScreen) {
        FSScreen *screen = [[FSScreen alloc] initWithAnnot:self];
        return screen.opacity;
    } else {
        return 1.0f;
    }
}

- (void)setOpacity:(float)opacity {
    if ([self isMarkup]) {
        FSMarkup *mk = [[FSMarkup alloc] initWithAnnot:self];
        [mk setOpacity:opacity];
    } else if ([self getType] == FSAnnotScreen) {
        FSScreen *screen = [[FSScreen alloc] initWithAnnot:self];
        [screen setOpacity:opacity];
    }
}

- (void)setIcon:(int)icon {
    FSAnnotType type = self.type;
    NSString *iconName = [Utility getIconNameWithIconType:icon annotType:type];
    if (iconName) {
        if (type == FSAnnotNote) {
            [[FSNote alloc] initWithAnnot:self].iconName = iconName;
        } else if (type == FSAnnotStamp) {
            [[FSStamp alloc] initWithAnnot:self].iconName = iconName;
        } else if (type == FSAnnotFileAttachment) {
            [[FSFileAttachment alloc] initWithAnnot:self].iconName = iconName;
        }
    }
}

- (int)icon {
    FSAnnotType type = [self getType];
    NSString *iconName = nil;
    if (type == FSAnnotNote) {
        iconName = [[FSNote alloc] initWithAnnot:self].iconName;
    } else if (type == FSAnnotStamp) {
        iconName = [[FSStamp alloc] initWithAnnot:self].iconName;
    } else if (type == FSAnnotFileAttachment) {
        iconName = [[FSFileAttachment alloc] initWithAnnot:self].iconName;
    }
    if (iconName) {
        return [Utility getIconTypeWithIconName:iconName annotType:type];
    }
    return FPDF_ICONTYPE_UNKNOWN;
}

- (void)applyAttributes:(FSAnnotAttributes *)attributes {
    [attributes resetAnnot:self];
}

- (BOOL)isEqualToAnnot:(FSAnnot *)annot {
    if (self == annot) {
        return YES;
    }
    if ([self isEmpty] || !annot || [annot isEmpty]  ) {
        return NO;
    }
    
    @try {
        if (self.type != annot.type) {
            return NO;
        }
    } @catch (NSException *exception) {
        return NO;
    }
    //Duplicate pages have the same uuid, using PageIndex as the identifier
    return [self.uuidWithPageIndex isEqualToString:annot.uuidWithPageIndex];
}

- (BOOL)isReplyToAnnot:(FSAnnot *)annot {
    if (!annot || self.replyTo.length == 0) {
        return NO;
    }
    return [self.replyTo isEqualToString:[annot NM]] && [[self getPage] getIndex] == [[annot getPage] getIndex];
}

#define FSAnnotReflow 116

- (FSAnnotType)getFilterAnnotType{
    if ([self isEmpty]) return FSAnnotUnknownType;
    const FSAnnotType type = [self getType];
    NSUInteger annotType = type;
    switch (type) {
    case FSAnnotCaret:
        {
            if ([Utility isReplaceText:[[FSCaret alloc] initWithAnnot:self]]) {
                annotType = FSAnnotReplace;
            }else{
                annotType = FSAnnotInsert;
            }
        }
        break;
    case FSAnnotReflow:
        if ([self.intent isEqualToString:@"AreaHi"]) {
            annotType = FSAnnotAreaHighlight;
                
        }else{
            annotType = FSAnnotHighlight;
        }
        break;
    case FSAnnotHighlight:
        if ([self.intent isEqualToString:@"AreaHi"]) {
            annotType = FSAnnotAreaHighlight;
            
        }else{
            annotType = FSAnnotHighlight;
        }
        break;
    case FSAnnotStrikeOut:
        {
            if ([Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:self]]) {
                annotType = FSAnnotReplace;
            }else{
                annotType = FSAnnotStrikeOut;
            }
        }
        break;
    case FSAnnotFreeText: {
        NSString *intent = [([[FSFreeText alloc] initWithAnnot:self]) getIntent];
        if ([intent caseInsensitiveCompare:@"FreeTextTypewriter"] == NSOrderedSame) {
            
        }else if ([intent caseInsensitiveCompare:@"FreeTextCallout"] == NSOrderedSame) {
            annotType = FSAnnotCallout;
        }else{
            annotType = FSAnnotTextbox;
        }
        break;
    }
    case FSAnnotLine: {
        FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:self];
        BOOL isArrow = [[markup getIntent] caseInsensitiveCompare:@"LineArrow"] == NSOrderedSame;
        if (isArrow) {
            annotType = FSAnnotArrowLine;
            break;
        }
        BOOL isDistance = [[markup getIntent] caseInsensitiveCompare:@"LineDimension"] == NSOrderedSame;
        if (isDistance) {
            annotType = FSAnnotDistance;
            break;
        }
        break;
    }
        case FSAnnotScreen: {
            FSScreen *screen = [[FSScreen alloc] initWithAnnot:self];
            if (screen.screenActiontype == FSScreenActionTypeUnknow || screen.screenActiontype == FSScreenActionTypeImage) {
                annotType = FSAnnotImage;
                break;
            }
            if (screen.screenActiontype == FSScreenActionTypeAudio) {
                annotType = FSAnnotAudio;
            }else if (screen.screenActiontype == FSScreenActionTypeVideo){
                annotType = FSAnnotVideo;
            }else if (screen.screenActiontype == FSScreenActionTypeAdditionalActionURI){
                annotType = FSAnnotURI;
            }
        break;
    }
    case FSAnnotPolygon: {
        FSBorderInfo *border = [[[FSPolygon alloc] initWithAnnot:self] getBorderInfo];
        BOOL isCloud = ([border getStyle] == FSBorderInfoCloudy);
        if (isCloud) {
            annotType = FSAnnotCloud;
        } else {
            annotType = FSAnnotPolygon;
        }
        break;
    }
    default:
        break;
    }
    return annotType;
}

@end

@implementation FSWidget (useProperties)

- (BOOL)isImageField{
    if (self.nonempty) {
        if ([[self getField] getType] == FSFieldTypePushButton) {
            FSAction *action = [self getAction];
            if (![action isEmpty] && [action getType] == FSActionTypeJavaScript) {
                FSJavaScriptAction *javaScriptAction = [[FSJavaScriptAction alloc] initWithAction:action];
                if (![javaScriptAction isEmpty]) {
                    NSString *script = [javaScriptAction getScript];
                    if ([script containsString:@"buttonImportIcon"]){
                        return YES;
                    }
                }
            }
        }
    }
    return NO;
}

- (BOOL)isComboBox{
    if (self.nonempty) {
        if ([[self getField] getType] == FSFieldTypeComboBox) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)isTextField{
    if (self.nonempty) {
        if ([[self getField] getType] == FSFieldTypeTextField) {
            return YES;
        }
    }
    return NO;
}

- (FSPDFObject *)getButtonPressedSoundObjt{
    if (self.nonempty) {
        FSAdditionalAction *aAction = [[FSAdditionalAction alloc] initWithAnnot:self];
        if (![aAction isEmpty]) {
            FSAction *action = [aAction getAction:FSAdditionalActionTriggerAnnotMouseButtonPressed];
            if (!action.isEmpty && action.getType == FSActionTypeSound) {
                FSPDFDictionary *dict = [action getDict];
                if (dict) {
                    FSPDFObject *soundObj = [dict getElement:@"Sound"];
                    return soundObj;
                }
            }
        }
    }
    return nil;
}

- (NSDateFormatter *)dateTextFieldFormat{
    if (self.nonempty) {
        if ([[self getField] getType] == FSFieldTypeTextField) {
            FSAdditionalAction *aAction = [[FSAdditionalAction alloc] initWithField:[self getField]];
            if (![aAction isEmpty]) {
                FSAction *action = [aAction getAction:FSAdditionalActionTriggerFieldWillFormat];
                if (![action isEmpty] && [action getType] == FSActionTypeJavaScript) {
                    FSJavaScriptAction *javaScriptAction = [[FSJavaScriptAction alloc] initWithAction:action];
                    if (![javaScriptAction isEmpty]) {
                        NSString *script = [javaScriptAction getScript];
                        if ([script containsString:@"AFDate_FormatEx"]){
                            NSString *dateFormat = [script stringByReplacingOccurrencesOfString:@"AFDate_FormatEx" withString:@""];
                            NSCharacterSet *trimSets = [NSCharacterSet characterSetWithCharactersInString:@"\"();"];
                            dateFormat = [dateFormat stringByTrimmingCharactersInSet:trimSets];
                            
                            NSMutableString *tmpStr = [NSMutableString string];
                            for (int i = 0 ; i < dateFormat.length; i++){
                                NSString *c = [dateFormat substringWithRange:NSMakeRange(i, 1)];
                                if ([c isEqualToString:@"M"]) {
                                    c = @"m";
                                } else if ([c isEqualToString:@"m"]) {
                                    c = @"M";
                                }
                                [tmpStr appendString:c];
                            }
                            dateFormat = tmpStr.copy;
                            NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
                            formatter.dateFormat = dateFormat;
                            formatter.locale = [NSLocale currentLocale];
                            return formatter;
                        }
                    }
                }
            }
        }
    }
    return nil;
}

- (BOOL)addAActionJavaScript:(NSString *)script actionTriggerEvent:(FSAdditionalActionTriggerEvent)actionTriggerEvent{
    if (self.nonempty) {
        FSField *field = [self getField];
        if (![field isEmpty]) {
            FSAdditionalAction *aAction = [[FSAdditionalAction alloc] initWithField:field];
            if (![aAction isEmpty]) {
                if (!script.length) {
                    [aAction removeAction:actionTriggerEvent];
                    return YES;
                }
                FSAction *action = [FSAction create:[[self getPage] getDocument] action_type:FSActionTypeJavaScript];
                FSJavaScriptAction *javaScriptAction = [[FSJavaScriptAction alloc] initWithAction:action];
                [javaScriptAction setScript:script];
                [aAction setAction:actionTriggerEvent action:javaScriptAction];
                return YES;
            }
        }
    }
    return NO;
}



@end

@implementation FSPointFArray (debug)

- (NSString *)description {
    NSMutableString *ret = @"".mutableCopy;
    int sz = [self getSize];
    [ret appendFormat:@"%d points:", sz];
    for (int i = 0; i < sz; i++) {
        FSPointF *pt = [self getAt:i];
        [ret appendFormat:@" (%.1f, %.1f)", pt.x, pt.y];
    }
    return ret;
}

@end
