/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "CommentModule.h"
#import "MenuView+Private.h"
#import "FSSummarizeCommentsViewController.h"
#import "ThumbnailModule+Private.h"
#import "FSPermissionUtilty.h"

@interface CommentModule ()<MoreItemActionProtocol ,IDocEventListener>
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;

@property (nonatomic, strong) MoreMenuItem *importItem;
@property (nonatomic, strong) MoreMenuItem *exportItem;
@property (nonatomic, strong) MoreMenuItem *summarizeItem;
@property (nonatomic, strong) MoreMenuItem *exportHighLightItem;

@end

@implementation CommentModule

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;
        [self loadModule];
    }
    return self;
}

- (void)loadModule{
    
    [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
    
    MoreMenuGroup *group = [_extensionsManager.more getGroup:TAG_GROUP_COMMENT_FIELD];
    if (!group) {
        group = [[MoreMenuGroup alloc] init];
        group.tag = TAG_GROUP_COMMENT_FIELD;
        
        group.title = FSLocalizedForKey(@"kComments&Fields");
        [_extensionsManager.more addGroup:group];
    }
    self.importItem = [[MoreMenuItem alloc] init];
    self.importItem.tag = TAG_ITEM_IMPORTCOMMENT;
    self.importItem.title = FSLocalizedForKey(@"kImportComments");
    self.importItem.callBack = self;
    self.importItem.image = FSImageNamedAndNormalColor(@"more_comment_import_comment");
    [_extensionsManager.more addMenuItem:TAG_GROUP_COMMENT_FIELD withItem:self.importItem];
    
    self.exportItem = [[MoreMenuItem alloc] init];
    self.exportItem.tag = TAG_ITEM_EXPORTCOMMENT;
    self.exportItem.title = FSLocalizedForKey(@"kExportComments");
    self.exportItem.callBack = self;
    self.exportItem.image = FSImageNamedAndNormalColor(@"more_comment_export_comment");
    [_extensionsManager.more addMenuItem:TAG_GROUP_COMMENT_FIELD withItem:self.exportItem];
    
    self.summarizeItem = [[MoreMenuItem alloc] init];
    self.summarizeItem.tag = TAG_ITEM_SUMARIZECOMMENT;
    self.summarizeItem.title = FSLocalizedForKey(@"kSummarizeComments");
    self.summarizeItem.callBack = self;
    self.summarizeItem.image = FSImageNamedAndNormalColor(@"more_comment_summarize_comment");
    [_extensionsManager.more addMenuItem:TAG_GROUP_COMMENT_FIELD withItem:self.summarizeItem];
    
    self.exportHighLightItem = [[MoreMenuItem alloc] init];
    self.exportHighLightItem.tag = TAG_ITEM_EXPORTHIGHLIGHTEDTEXT;
    self.exportHighLightItem.title = FSLocalizedForKey(@"kExportHighLight");
    self.exportHighLightItem.callBack = self;
    self.exportHighLightItem.image = FSImageNamedAndNormalColor(@"more_comment_export_high_light");
    [_extensionsManager.more addMenuItem:TAG_GROUP_COMMENT_FIELD withItem:self.exportHighLightItem];
}

- (NSString *)getName {
    return @"Comment";
}

- (void)dealloc
{
    [_extensionsManager.pdfViewCtrl unregisterDocEventListener:self];
}

#pragma MoreMenuItemAction
- (void)onClick:(MoreMenuItem *)item{
    
    BOOL canAddAnnot = [self.extensionsManager annotsPermissionCanAdd];
    BOOL isDynamicXFA = [_pdfViewCtrl isDynamicXFA];
    if ((!canAddAnnot && item.tag == TAG_ITEM_IMPORTCOMMENT) || isDynamicXFA) {
        FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kWarning"), FSLocalizedForKey(@"kRMSNoAccess"), self.pdfViewCtrl);
        return;
    }

    [self operateComments:item];

    FSUIElementType elementType = -1;
    if(item.tag == TAG_ITEM_IMPORTCOMMENT) {
        elementType = FSUIElementReadingMoreCommentFieldImortComments;
    }
    else if (item.tag == TAG_ITEM_EXPORTCOMMENT) {
        elementType = FSUIElementReadingMoreCommentFieldExportComments;
    }
    else if(item.tag == TAG_ITEM_SUMARIZECOMMENT) {
        elementType = FSUIElementReadingMoreCommentFieldSummarizeComments;
    }
    else if(item.tag == TAG_ITEM_EXPORTHIGHLIGHTEDTEXT) {
        elementType = FSUIElementReadingMoreCommentFieldExportHighlightedText;
    }
    
    [_extensionsManager onUIInteractElementClicked:elementType];
    
}

- (void)operateComments:(MoreMenuItem*)item{
    if (item.tag == TAG_ITEM_IMPORTCOMMENT ||
        item.tag == TAG_ITEM_EXPORTCOMMENT ||
        item.tag == TAG_ITEM_EXPORTHIGHLIGHTEDTEXT) {
        [self operateImportAndExportComments:item];
    }else if (item.tag == TAG_ITEM_SUMARIZECOMMENT){
        [self oprerateSumarizeComments:item];
    }
}

- (void)oprerateSumarizeComments:(MoreMenuItem*)item{
    
    if (![Utility canExtractContents:_pdfViewCtrl]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    FSSummarizeCommentsViewController *summarizeCommentsVC = [[FSSummarizeCommentsViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
    summarizeCommentsVC.summarizeBlock = ^(SummarizeType summarizeType) {
        FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
        selectDestination.isRootFileDirectory = YES;
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
        selectDestination.expectFileType = @[@"*"];
        
        [selectDestination loadFilesWithPath:DOCUMENT_PATH];
        selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
            
            NSString *filePath = [destinationFolder lastObject];
        
            [controller dismissViewControllerAnimated:YES completion:^{
                [self inputSummarizeFileNameWithPath:filePath controller:controller summarizeType:summarizeType];
            }];
            
        };
        
        FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
        [self.extensionsManager.viewCtrlManager presentViewController:selectDestinationNavController
                                         animated:YES
                                       completion:nil];
    };
    FSNavigationController *summarizeCommentsNav = [[FSNavigationController alloc] initWithRootViewController:summarizeCommentsVC];
    if (DEVICE_iPAD) {
        summarizeCommentsNav.modalPresentationStyle = UIModalPresentationFormSheet;
    }else{
        summarizeCommentsNav.modalPresentationStyle = UIModalPresentationFullScreen;
    }
    [self.extensionsManager.viewCtrlManager presentViewController:summarizeCommentsNav animated:YES completion:nil];
//    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:summarizeCommentsNav animated:YES completion:nil];
}

- (void)inputSummarizeFileNameWithPath:(NSString *)path controller:(FSFileSelectDestinationViewController *)controller summarizeType:(SummarizeType)type{
    
    NSString *titleStr = [[self.extensionsManager.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
    
    [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kInputNewFileName") message:@"" textFieldText:titleStr actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
            
            if (index == 1) {
                
                void (^saveFdocFailedTip)(void) = ^{
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kError"), FSLocalizedForKey(@"kFailSummary"), self.pdfViewCtrl);
                    });
                };
                
                NSString *saveFilePath = [NSString stringWithFormat:@"%@/%@.pdf",path, textField.text];
                
                void (^saveFdoc)(void) = ^{
                    
                    FSPDFDoc *doc = self.pdfViewCtrl.currentDoc;
                    if (![self pdfDocHasAnnote:doc]) {
                        //no annote
                        [self fs_toastWithMsg:FSLocalizedForKey(@"kNoCommentSummary") inView:[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view];
                        return;
                    }

                    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                        @try {
                            FSAnnotationSummary* annotationSummary = [[FSAnnotationSummary alloc] initWithDocument:doc];
                            FSAnnotationSummarySettings* settings = [[FSAnnotationSummarySettings alloc] init];
                            [settings setAnnotType:0 use_all:YES];
                            [settings setFontSize:FSAnnotationSummarySettingsFontSizeMedium];
                            [settings setSortType:FSAnnotationSummarySettingsTypeSortByPage];
                            [self setThumbnailForAnnotationsImage:settings];
                            
                            if (type == SummarizeTypeDocumentAndComments) {
                                //document and comment
                                [settings setSummaryLayout:FSAnnotationSummarySettingsSummaryLayoutSeparatePagesWithSequenceNumber];
                            }else if (type == SummarizeTypeCommentsOnly){
                                //comment only
                                [settings setSummaryLayout:FSAnnotationSummarySettingsSummaryLayoutAnnotationOnly];
                            }
                            
                            FSProgressive *progressive = [annotationSummary startSummarize:saveFilePath settings:settings pause:nil];
                            FSProgressiveState proState = FSProgressiveToBeContinued;
                            while (proState == FSProgressiveToBeContinued) {
                                proState = [progressive resume];
                            }
                            
                            dispatch_async(dispatch_get_main_queue(), ^{
                                if (proState == FSProgressiveFinished) {
                                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                                        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kSummarizeSuccessful") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                            [controller dismissViewControllerAnimated:YES completion:nil];
                                        } onCurrentSceneResponder:self.pdfViewCtrl];
                                    });
                                }else{
                                    saveFdocFailedTip();
                                }
                            });
                        } @catch (NSException *exception) {
                            saveFdocFailedTip();
                        } @finally {
                            
                        }
                    }];

                };
                NSFileManager *filemanager = [NSFileManager defaultManager];
                if ([filemanager fileExistsAtPath:saveFilePath]) {
                    UIAlertController *alertController = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kReplace") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                        if (index == 1) {
                            saveFdoc();
                        }
                    }];
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                    });
                    
                }else{
                    saveFdoc();
                }
                
                return ;
            }
            [controller dismissViewControllerAnimated:YES completion:nil];

            
    }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
        if (index == 1) {
              return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
          }
        return YES;
    }] fs_showByViewController:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]] ;
}

- (void)setThumbnailForAnnotationsImage:(FSAnnotationSummarySettings *)settings{
    @try {
        [settings setAnnotThumbnail:FSAnnotNote thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_note")]];
        [settings setAnnotThumbnail:FSAnnotFreeText thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_typewriter")]];
        [settings setAnnotThumbnail:FSAnnotLine thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_line")]];
        [settings setAnnotThumbnail:FSAnnotSquare thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_square")]];
        [settings setAnnotThumbnail:FSAnnotCircle thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_circle")]];
        [settings setAnnotThumbnail:FSAnnotPolygon thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_polygon")]];
        [settings setAnnotThumbnail:FSAnnotPolyLine thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_polyline")]];
        [settings setAnnotThumbnail:FSAnnotHighlight thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_highlight")]];
        [settings setAnnotThumbnail:FSAnnotUnderline thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_underline")]];
        [settings setAnnotThumbnail:FSAnnotSquiggly thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_squiggly")]];
        [settings setAnnotThumbnail:FSAnnotStrikeOut thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_strikeout")]];
        [settings setAnnotThumbnail:FSAnnotStamp thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_stamp")]];
        [settings setAnnotThumbnail:FSAnnotCaret thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_insert_text")]];
        [settings setAnnotThumbnail:FSAnnotInk thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_pencil")]];
        [settings setAnnotThumbnail:FSAnnotFileAttachment thumbnail:[Utility uiimageToFSBitmap:FSImageNamed(@"ic_annotation_summary_attachment")]];
    } @catch (NSException *exception) {
        
    } @finally {
        
    }

}

- (BOOL)pdfDocHasAnnote:(FSPDFDoc*)pdfDoc{
    int pageCount = [pdfDoc getPageCount];
    int annoteCount = 0;
    
    if (pdfDoc && ![pdfDoc isEmpty]) {

        for (int i=0; i<pageCount; i++) {
            FSPDFPage *pdfPage = [pdfDoc getPage:i];
            annoteCount = [pdfPage getAnnotCount];
            for (int i=0; i<annoteCount; i++) {
                FSAnnot* annote= [pdfPage getAnnot:i];
                if (annote.getType != FSAnnotRedact
                    && annote.getType != FSAnnotPopup
                    && annote.getType != FSAnnotLink
                    && annote.getType != FSAnnotSound
                    && annote.getType != FSAnnotWidget
                    && annote.getType != FSAnnotPrinterMark
                    && annote.getType != FSAnnotWatermark) {
                    return YES;
                }
            }
        }
    }
    
    return NO;
}

- (void)operateImportAndExportComments:(MoreMenuItem*)item{
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
    selectDestination.isRootFileDirectory = YES;
    
    if (item.tag == TAG_ITEM_IMPORTCOMMENT) {
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFile;
        selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeImport;
        selectDestination.expectFileType = @[ @"fdf", @"xfdf"];

    }else if (item.tag == TAG_ITEM_EXPORTCOMMENT || item.tag == TAG_ITEM_EXPORTHIGHLIGHTEDTEXT){
        if (!([_extensionsManager.permissionProvider checkPermission:FSFunctionCanSaveToLocal] == FSPermissionStateShow)) {
            return;
        }
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
        selectDestination.expectFileType = @[@"*"];
    }
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        
        NSString *filePath = [destinationFolder lastObject];
        if (item.tag == TAG_ITEM_IMPORTCOMMENT) {
            FSFDFDoc *importFdoc = [[FSFDFDoc alloc] initWithPath:filePath];
            if ([importFdoc isEmpty]) {
                [controller dismissViewControllerAnimated:YES completion:^{
                    FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kWarning"), FSLocalizedForKey(@"kNoCommentsImport"), self.pdfViewCtrl);
                }];
                return;
            }
            FSPDFDoc *doc = self.pdfViewCtrl.currentDoc;
            int type = 0x0002;
            FSRange *range = [[FSRange alloc] initWithStart_index:0 end_index:[self.pdfViewCtrl.currentDoc getPageCount] filter:FSRangeAll];
            
            void (^importFailed)(void) = ^{
                [controller dismissViewControllerAnimated:YES completion:^{
                    FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kError"), FSLocalizedForKey(@"kFailedImportData"), self.pdfViewCtrl);
                }];
            };
            @try {
                int pageCount = [doc getPageCount];
                int beforeAnnotCount = 0;
                for (int i = 0; i < pageCount; i++) {
                    FSPDFPage *page = [doc getPage:i];
                    beforeAnnotCount += [page getAnnotCount];
                }
                BOOL flag = [doc importFromFDF:importFdoc types:type page_range:range];
                if (flag) {
                    pageCount = [doc getPageCount];
                    int afterAnnotCount = 0;
                    for (int i = 0; i < pageCount; i++) {
                        FSPDFPage *page = [doc getPage:i];
                        afterAnnotCount += [page getAnnotCount];
                        [self.extensionsManager.thumbnailModule.thumbnailCache removeThumbnailCacheOfPageAtIndex:i];
                    }
                    [self.pdfViewCtrl refresh];
                    
                    self.extensionsManager.isDocModified = YES;
                    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
                    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAttachmentLsitUpdate object:nil];
                    [controller dismissViewControllerAnimated:YES completion:^{
                        FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kSuccess"), FSLocalizedForKey(@"kSuccessImportData"), self.pdfViewCtrl);
                    }];
                }else{
                    importFailed();
                }
            } @catch (NSException *exception) {
                importFailed();
            }
  
        }else if (item.tag == TAG_ITEM_EXPORTHIGHLIGHTEDTEXT){
            [controller dismissViewControllerAnimated:YES completion:^{
                [self inputHighlightedTextFileNameWithPath:filePath controller:controller textFieldText:nil];
            }];
        }else{
            [controller dismissViewControllerAnimated:YES completion:^{
                [self inputFileNameWithPath:filePath controller:controller textFieldText:nil];
            }];
        }
    };
    
    FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
    [self.extensionsManager.viewCtrlManager presentViewController:selectDestinationNavController
                                     animated:YES
                                   completion:nil];
}

- (void)inputFileNameWithPath:(NSString *)path controller:(FSFileSelectDestinationViewController *)controller textFieldText:(NSString *)textFieldText{
    
    NSString *titleStr = [[self.extensionsManager.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
    
    [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kInputNewFileName") message:@"" textFieldText:textFieldText.length != 0 ? textFieldText : titleStr actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
            
            if (index == 1) {
                FSFDFDoc *exportFdoc = [[FSFDFDoc alloc] initWithType:FSFDFDocFDF];
                FSRange *range = [[FSRange alloc] initWithStart_index:0 end_index:[self.pdfViewCtrl.currentDoc getPageCount] filter:FSRangeAll];
                FSPDFDoc *doc = self.pdfViewCtrl.currentDoc;
                int type = 0x0002;
                void (^saveFdocFailedTip)(void) = ^{
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kFailedExportData") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                            [self inputFileNameWithPath:path controller:controller textFieldText:textField.text];
                        }onCurrentSceneResponder:self.pdfViewCtrl];
                    });
                };
                
                NSString *saveFilePath = [NSString stringWithFormat:@"%@/%@.fdf",path, textField.text];
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                    @try {
                        BOOL flag = [doc exportToFDF:exportFdoc types:type page_range:range];
                        if (flag) {
                            void (^saveFdoc)(void) = ^{
                                BOOL isSuccess = [exportFdoc saveAs:saveFilePath];
                                if (isSuccess) {
                                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                                        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kSuccessExportData") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                            [controller dismissViewControllerAnimated:YES completion:nil];
                                        }onCurrentSceneResponder:self.pdfViewCtrl];
                                    });
                                }else{
                                    saveFdocFailedTip();
                                }
                                
                            };
                            NSFileManager *filemanager = [NSFileManager defaultManager];
                            if ([filemanager fileExistsAtPath:saveFilePath]) {
                                UIAlertController *alertController = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kReplace") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                    if (index == 1) {
                                        saveFdoc();
                                    }else{
                                        [self inputFileNameWithPath:path controller:controller textFieldText:textField.text];
                                    }
                                }];
                                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                                    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                                });
                                
                            }else{
                                saveFdoc();
                            }
                        }else{
                            saveFdocFailedTip();
                        }
                    } @catch (NSException *exception) {
                        saveFdocFailedTip();
                    }
                }];
                
                return ;
            }
            [controller dismissViewControllerAnimated:YES completion:nil];

            
    }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
        if (index == 1) {
              return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
        }
        return YES;
    }] fs_showByViewController:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]] ;
}

- (void)inputHighlightedTextFileNameWithPath:(NSString *)path controller:(FSFileSelectDestinationViewController *)controller textFieldText:(NSString *)textFieldText{
    
    NSString *titleStr = [[self.extensionsManager.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
    
    [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kInputNewFileName") message:@"" textFieldText:textFieldText.length != 0 ? textFieldText : titleStr actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
        
        if (index == 1) {
            
            void (^saveFdocFailedTip)(void) = ^{
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kFailedExportData") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                        [self inputHighlightedTextFileNameWithPath:path controller:controller textFieldText:textField.text];
                    }onCurrentSceneResponder:self.pdfViewCtrl];
                });
            };
            
            NSString *saveFilePath = [NSString stringWithFormat:@"%@/%@.txt",path, textField.text];
            
            void (^saveFdoc)(void) = ^{
                
                FSPDFDoc *pdfDoc = [self->_extensionsManager.pdfViewCtrl getDoc];
                if (!pdfDoc || [pdfDoc isEmpty]) {
                    saveFdocFailedTip();
                    return;
                }
                
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                    NSMutableString *highLightStr = [NSMutableString stringWithFormat:@""];
                    int pageCount = [pdfDoc getPageCount];
                    int annoteCount = 0;
                    
                    for (int i=0; i<pageCount; i++) {
                        @try {
                            FSPDFPage *pdfPage = [pdfDoc getPage:i];
                            annoteCount = [pdfPage getAnnotCount];
                            BOOL bAddPageNum = NO;
                            for (int j=0; j<annoteCount; j++) {
                                FSAnnot* curAnnote= [pdfPage getAnnot:j];
                                if (curAnnote.getType == FSAnnotHighlight) {
                                    if (!bAddPageNum) {
                                        bAddPageNum = YES;
                                        [highLightStr appendFormat:@"Page %d\r\n",i+1];
                                    }
                                    FSMarkup *annot = [[FSMarkup alloc] initWithAnnot:curAnnote];
                                    NSArray *quads = annot.quads;
                                    for (int cnt = 0; cnt < quads.count; cnt++) {
                                        @try {
                                            FSQuadPoints *arrayQuad = [quads objectAtIndex:cnt];
                                            FSRectF *rect = [Utility convertToFSRect:arrayQuad.getFirst p2:arrayQuad.getFourth];
                                            NSString *tmp = [[Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:annot.pageIndex] getTextInRect:rect];
                                            if (tmp) {
                                                if (cnt != 0)
                                                    [highLightStr appendString:@"\r\n"];
                                                [highLightStr appendString:tmp];
                                                if (cnt >= quads.count - 1) {
                                                    [highLightStr appendString:@"\r\n"];
                                                }
                                            }
                                        } @catch (NSException *exception) {
                                            [highLightStr appendString:@"\r\n"];
                                            continue;
                                        }
                                    }
                                }
                            }
                        } @catch (NSException *exception) {
                            continue;
                        }
                    }
                    
                    if (!highLightStr.length) {
                        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                            [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kNoHighlightedTextToExport") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                [controller dismissViewControllerAnimated:YES completion:nil];
                            }onCurrentSceneResponder:self.pdfViewCtrl];
                        });
                        return;
                    }
                    
                    if (![highLightStr writeToFile:saveFilePath atomically:YES encoding:NSUTF8StringEncoding error:nil]) {
                        saveFdocFailedTip();
                        return;
                    }
                        
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kSuccessExportData") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                            [controller dismissViewControllerAnimated:YES completion:nil];
                        }onCurrentSceneResponder:self.pdfViewCtrl];
                    });
                }];
            };
            
            NSFileManager *filemanager = [NSFileManager defaultManager];
            if ([filemanager fileExistsAtPath:saveFilePath]) {
                UIAlertController *alertController = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kReplace") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                    if (index == 1) {
                        saveFdoc();
                    }else{
                        [self inputHighlightedTextFileNameWithPath:path controller:controller textFieldText:textField.text];
                    }
                }];
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                });
                
            }else{
                saveFdoc();
            }
            
            return ;
        }
        [controller dismissViewControllerAnimated:YES completion:nil];
        
        
    }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
        if (index == 1) {
            return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
        }
        return YES;
    }] fs_showByViewController:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]] ;
}

#pragma mark --IDocEventListener
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error{
    
    if (self.importItem) {
        BOOL canAddAnnot = [self.extensionsManager annotsPermissionCanAdd];
        BOOL isDynamicXFA = [_pdfViewCtrl isDynamicXFA];
        if (!canAddAnnot|| isDynamicXFA) {
            self.importItem.enable = NO;
        }else{
            self.importItem.enable = YES;
        }
    }

    if (self.summarizeItem) {
        if (![Utility canExtractContents:_pdfViewCtrl]) {
            self.summarizeItem.enable = NO;
        }else{
            self.summarizeItem.enable = YES;
        }
    }
    if (self.exportItem) {
        self.exportItem.enable = [Utility canExtractContents:_pdfViewCtrl];
    }
    
    if (self.exportHighLightItem) {
        self.exportHighLightItem.enable = [Utility canExtractContents:_pdfViewCtrl];
    }
}

@end
