/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMenuController.h"
#import "FSWeakPointerArray.h"

#define ICONWIDTH 40
#define ARROWHEIGH 10
#define ARROWWIDTH 20

@interface FSMenuController()<UIViewAdapter>
@property (nonatomic, weak) UIView* targetView;
@property (nonatomic, assign) CGRect targetRect;

@property (nonatomic, strong) UIView *menuView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIView *arrowViewDown;
@property (nonatomic, strong) UIView *arrowViewUp;

@property (nonatomic, strong) UIControl *maskView;
@property (nonatomic, weak) UIView *fakeTargetView;
@property (nonatomic, strong) NSMutableArray<FSItem *> *interactionMenuItems;
@end


@implementation FSMenuController
static FSMenuController *menuController = nil;

+ (FSMenuController *)sharedMenuController{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        menuController = [[self alloc] init];
        menuController.arrowDirection = FSMenuControllerArrowDirectionUp;
    });
    return menuController;
}

- (UIControl *)maskView{
    if(!_maskView){
        _maskView = [UIControl new];
        [_maskView addTarget:self action:@selector(dismiss) forControlEvents:UIControlEventTouchUpInside];
    }
    return _maskView;
}

- (void)dismiss{
    [self setMenuVisible:NO animated:YES];
}

- (void)setMenuItems:(NSArray<FSMenuItem *> *)menuItems{
    _menuItems = menuItems.copy;
    [_menuView removeFromSuperview];
    _menuView = nil;
}

- (void)setMenuVisible:(BOOL)menuVisible animated:(BOOL)animated{
    if ([UIMenuController sharedMenuController].isMenuVisible && menuVisible) {
        [[UIMenuController sharedMenuController] setMenuVisible:NO animated:YES];
        return;
    }
    _menuVisible = menuVisible;
    if (menuVisible) {
        [_fakeTargetView removeFromSuperview];
        FSAdapterUIView *fakeTargetView = [FSAdapterUIView new];
        fakeTargetView.userInteractionEnabled = NO;
        fakeTargetView.hidden = YES;
        [_targetView.superview addSubview:fakeTargetView];
        [fakeTargetView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.top.fs_equalToSuperview();
        }];
        fakeTargetView.adapter = self;
        _fakeTargetView = fakeTargetView;
        
        UIView *menuView = self.menuView;
        [self initMenuContent];
        UIWindow *keywindow = _targetView.fs_currentSceneKeyWindow;
        [keywindow addSubview:menuView];
        
        [keywindow insertSubview:self.maskView belowSubview:self.menuView];
        [self.maskView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.top.fs_equalToSuperview();
        }];
        
        if (self.delegate && [self.delegate respondsToSelector:@selector(menuControllerEnableAtIndex:)]) {
            for (FSItem *item in _interactionMenuItems) {
                item.enabled = [self.delegate menuControllerEnableAtIndex:[_interactionMenuItems indexOfObject:item]];
            }
        }
    }else{
        [_fakeTargetView removeFromSuperview];
        [_menuView removeFromSuperview];
        [self.maskView removeFromSuperview];
        _menuView = nil;
        if (self.delegate && [self.delegate respondsToSelector:@selector(menuControllerDidDisappear)]) {
            [self.delegate menuControllerDidDisappear];
        }
    }
}

- (void)setTargetRect:(CGRect)targetRect inView:(UIView *)targetView{
    _targetRect = targetRect;
    _targetView = targetView;
}

- (void)updateTargetRect:(CGRect)targetRect inView:(nonnull UIView *)targetView
{
    _targetRect = targetRect;
    _targetView = targetView;
    [self caculateShowPositionAndFrame];
    
    if (self.delegate && [self.delegate respondsToSelector:@selector(menuControllerEnableAtIndex:)]) {
        for (FSItem *item in _interactionMenuItems) {
            item.enabled = [self.delegate menuControllerEnableAtIndex:[_interactionMenuItems indexOfObject:item]];
        }
    }
}

- (void)update{
    [self caculateShowPositionAndFrame];
    if (self.delegate && [self.delegate respondsToSelector:@selector(menuControllerEnableAtIndex:)]) {
        for (FSItem *item in _interactionMenuItems) {
            item.enabled = [self.delegate menuControllerEnableAtIndex:[_interactionMenuItems indexOfObject:item]];
        }
    }
}

- (UIView *)menuView
{
    if (_menuView == nil) {
        _menuView = [[UIView alloc] init];
        _contentView = [[UIView alloc] init];
        _contentView.layer.cornerRadius = 8.0;
        _contentView.layer.masksToBounds = YES;
        _contentView.backgroundColor = [UIColor blackColor];
        _arrowViewDown = [[UIView alloc] init];
        _arrowViewUp = [[UIView alloc] init];
        //_arrowView.backgroundColor = [UIColor blackColor];
        CAShapeLayer *upLayer = [[CAShapeLayer alloc] init];
        UIBezierPath *path = [[UIBezierPath alloc] init];
        [path moveToPoint:CGPointMake(0, 0)];
        [path addLineToPoint:CGPointMake(ARROWWIDTH * 0.5, ARROWHEIGH)];
        [path addLineToPoint:CGPointMake(ARROWWIDTH, 0)];
        [path closePath];
        upLayer.path = path.CGPath;

        [_arrowViewDown.layer addSublayer:upLayer];
        
        CAShapeLayer *downLayer = [[CAShapeLayer alloc] init];
        UIBezierPath *path1 = [[UIBezierPath alloc] init];
        [path1 moveToPoint:CGPointMake(0, ARROWHEIGH)];
        [path1 addLineToPoint:CGPointMake(ARROWWIDTH, ARROWHEIGH)];
        [path1 addLineToPoint:CGPointMake(ARROWWIDTH * 0.5, 0)];
        [path1 closePath];
        downLayer.path = path1.CGPath;

        [_arrowViewUp.layer addSublayer:downLayer];
        
        [_menuView addSubview:_contentView];
        [_menuView addSubview:_arrowViewDown];
        
        _interactionMenuItems = @[].mutableCopy;
        for (int i = 0; i < _menuItems.count; i++) {
            FSMenuItem *menuItem = _menuItems[i];
            FSItem *item = [[FSItem alloc] initWithRrelation:FSImageTextRelationBottom];
            item.accessibilityLabel = menuItem.accessibilityLabel;
            item.tag = i;
//            [item setTitle:menuItem.title forState:UIControlStateNormal];
            [item setImage:[menuItem.image fs_imageWithTintColor:[UIColor whiteColor]] forState:UIControlStateNormal];
            [item setImage:[menuItem.image fs_imageWithTintColor:self.fs_iconColor_I2] forState:UIControlStateDisabled];
            if (menuItem.action) {
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    ! menuItem.action ? : menuItem.action(menuItem);
                };
            }

            [item addTarget:self action:@selector(itemClick:) forControlEvents:UIControlEventTouchUpInside];
            item.frame = CGRectMake( i * (ICONWIDTH+1), 0, ICONWIDTH, ICONWIDTH);
            
            if (i > 0) {
                CALayer *line = [[CALayer alloc] init];
                line.frame = CGRectMake( CGRectGetMinX(item.frame)-1, 0, 1, ICONWIDTH);
                line.backgroundColor = [UIColor whiteColor].CGColor;
                [_contentView.layer addSublayer:line];
            }

            [_contentView addSubview:item];
            [_interactionMenuItems addObject:item];
        }
    }
    return _menuView;
}

- (void)caculateShowPositionAndFrame
{
    CGFloat menuContentWidth = self.menuItems.count * ICONWIDTH;
    CGSize menuContentSize = CGSizeMake(menuContentWidth, ICONWIDTH);
    
    UIWindow *keywindow = _targetView.fs_currentSceneKeyWindow;
    CGRect targetOnWindow = [_targetView convertRect:_targetRect toView:keywindow];
    CGRect targetViewOnWindow = [_targetView convertRect:_targetView.frame toView:keywindow];
    if (CGRectContainsRect(targetViewOnWindow, targetOnWindow)) targetViewOnWindow = targetOnWindow;
    if (!_targetView || targetOnWindow.origin.x + targetOnWindow.size.width < 0 || targetOnWindow.origin.y + targetOnWindow.size.height < 0 || targetOnWindow.origin.x > keywindow.frame.size.width || targetOnWindow.origin.y > keywindow.frame.size.height) {
        _menuView.hidden = YES;
        return;
    }
    _menuView.hidden = NO;
    
    CGFloat arrowx = targetOnWindow.origin.x + targetOnWindow.size.width * 0.5;
    if (arrowx - menuContentWidth * 0.5 >= 0 && arrowx + menuContentWidth * 0.5 <= keywindow.frame.size.width) {
        if (CGRectGetMinY(targetOnWindow) - ABS(CGRectGetMinY(targetOnWindow) - CGRectGetMinY(targetViewOnWindow)) > ARROWHEIGH + ICONWIDTH && self.arrowDirection == FSMenuControllerArrowDirectionDown) {
            CGRect menuFrame = CGRectMake(CGRectGetMidX(targetOnWindow) - menuContentWidth * 0.5, CGRectGetMinY(targetOnWindow) - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
            
        }else if (keywindow.frame.size.height - CGRectGetMaxY(targetOnWindow) > ARROWHEIGH + ICONWIDTH){
            CGRect menuFrame = CGRectMake(targetOnWindow.origin.x + targetOnWindow.size.width * 0.5 - menuContentWidth * 0.5, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
            //_arrowView.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, 0, ARROWWIDTH, ARROWHEIGH);
            if (_arrowViewDown.superview) {
                [_arrowViewDown removeFromSuperview];
                [_menuView addSubview:_arrowViewUp];
            }
            _arrowViewUp.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        }else{
            CGRect menuFrame = CGRectMake(targetOnWindow.origin.x + targetOnWindow.size.width * 0.5 - menuContentWidth * 0.5, targetOnWindow.size.height * 0.5 - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }
        return;
    }
    
    if (arrowx - menuContentWidth * 0.5 < 0 && arrowx + menuContentWidth * 0.5< keywindow.frame.size.width) {
        if (CGRectGetMinY(targetOnWindow) - ABS(CGRectGetMinY(targetOnWindow) - CGRectGetMinY(targetViewOnWindow)) > ARROWHEIGH + ICONWIDTH) {
            CGRect menuFrame = CGRectMake(0, targetOnWindow.origin.y - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 < 8 ? 8 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }else if (keywindow.frame.size.height - CGRectGetMaxY(targetOnWindow) > ARROWHEIGH + ICONWIDTH){
            CGRect menuFrame = CGRectMake(0, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
           
            if (_arrowViewDown.superview) {
                [_arrowViewDown removeFromSuperview];
                [_menuView addSubview:_arrowViewUp];
            }
             _arrowViewUp.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 < 8 ? 8 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        }else{
            CGRect menuFrame = CGRectMake(0, targetOnWindow.size.height * 0.5 - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 < 8 ? 8 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }
        return;
    }
    
    if (arrowx - menuContentWidth * 0.5 > 0 && arrowx + menuContentWidth * 0.5 > keywindow.frame.size.width) {
        if (CGRectGetMinY(targetOnWindow) - ABS(CGRectGetMinY(targetOnWindow) - CGRectGetMinY(targetViewOnWindow)) > ARROWHEIGH + ICONWIDTH && self.arrowDirection == FSMenuControllerArrowDirectionDown) {
            CGRect menuFrame = CGRectMake(keywindow.frame.size.width - menuContentWidth, targetOnWindow.origin.y - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
           
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 > menuFrame.size.width - 28 ? menuFrame.size.width - 28 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }else if (keywindow.frame.size.height - CGRectGetMaxY(targetOnWindow) > ARROWHEIGH + ICONWIDTH){
            CGRect menuFrame = CGRectMake(keywindow.frame.size.width - menuContentWidth, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewDown.superview) {
                [_arrowViewDown removeFromSuperview];
                [_menuView addSubview:_arrowViewUp];
            }
            _arrowViewUp.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 > menuFrame.size.width - 28 ? menuFrame.size.width - 28 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        }else{
            CGRect menuFrame = CGRectMake(keywindow.frame.size.width - menuContentWidth, targetOnWindow.size.height * 0.5 - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 > menuFrame.size.width - 28 ? menuFrame.size.width - 28 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }
        return;
    }
}


- (void)initMenuContent{
    [self caculateShowPositionAndFrame];
}

- (void)itemClick:(FSItem *)item{
    if (self.delegate && [self.delegate respondsToSelector:@selector(menuControllerDidClickedItem:atIndex:)]) {
        [self.delegate menuControllerDidClickedItem:_menuItems[item.tag] atIndex:item.tag];
    }
    if (self.delegate && [self.delegate respondsToSelector:@selector(menuControllerEnableAtIndex:)]) {
        for (FSItem *item in _interactionMenuItems) {
            item.enabled = [self.delegate menuControllerEnableAtIndex:[_interactionMenuItems indexOfObject:item]];
        }
    }
}

#pragma mark - <UIViewAdapter>
- (void)layoutSubviewsFromView:(__kindof UIView *)view{
    if (DEVICE_iPAD && ![_targetView fs_currentSceneiPadHorizontalCompact]) {
        [self setMenuVisible:NO animated:YES];
        return;
    }
    [self update];
}
@end
