/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMenuViewManager.h"

@interface FSMenuViewManager ()

@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) NSMutableDictionary <NSString *, id<FSMenuView>>*menuViewTypes;
@property (nonatomic, strong) NSMutableDictionary <NSString *, NSMutableArray <FSMenuItemGroup *>*> *menuItemGroups;

@end

@interface FSMenuItemGroup ()
@property (nonatomic, weak) NSMutableArray *supgroups;
@end

@implementation FSMenuViewManager

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _menuViewTypes = @{}.mutableCopy;
        _menuItemGroups = @{}.mutableCopy;
        [self addItemGroupForDefaultMenuViewType];
    }
    return self;
}


- (void)setMenuView:(id<FSMenuView>)menuView forMenuViewType:(FSMenuViewType)menuViewType{
    NSAssert(menuViewType >= FSMenuViewTypeThumbnailInsertBlankPagePop && menuViewType <= FSMenuViewTypeUnsupported, @"The menuViewType is not supported");
    
    [self.menuViewTypes setValue:menuView forKey:@(menuViewType).stringValue];
}

- (id<FSMenuView>)getMenuViewForMenuViewType:(FSMenuViewType)menuViewType{
    return self.menuViewTypes[@(menuViewType).stringValue];
}

- (void)addMenuItemGroup:(FSMenuItemGroup *)group forMenuViewType:(FSMenuViewType)menuViewType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[FSMenuItemGroup class]], @"The group should be a subclass of FSMenuItemGroup");
    NSAssert(menuViewType >= FSMenuViewTypeThumbnailInsertBlankPagePop && menuViewType <= FSMenuViewTypeUnsupported, @"menuViewType is not supported");
    
    NSMutableArray *itemGroups = self.menuItemGroups[@(menuViewType).stringValue];
    if (!itemGroups)itemGroups = @[].mutableCopy;
    [itemGroups addObject:group];
    [self.menuItemGroups setValue:itemGroups forKey:@(menuViewType).stringValue];
    group.supgroups = itemGroups;
}

- (void)insertMenuItemGroup:(FSMenuItemGroup *)group atIndex:(NSInteger)index forMenuViewType:(FSMenuViewType)menuViewType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[FSMenuItemGroup class]], @"The group should be a subclass of FSMenuItemGroup");
    
    NSMutableArray *itemGroups = self.menuItemGroups[@(menuViewType).stringValue];
    if (!itemGroups) itemGroups = @[].mutableCopy;
    
    NSAssert(index <= itemGroups.count, @"Index must be less than or equal to count of itemGroups");
    
    [itemGroups insertObject:group atIndex:index];
    [self.menuItemGroups setValue:itemGroups forKey:@(menuViewType).stringValue];
    group.supgroups = itemGroups;
}

- (void)exchangeMenuItemGroupAtIndex:(NSInteger)index1 withMenuItemGroupAtIndex:(NSInteger)index2 forMenuViewType:(FSMenuViewType)menuViewType{
    NSMutableArray *itemGroups = self.menuItemGroups[@(menuViewType).stringValue];
    FSMenuItemGroup *group1 = [itemGroups objectAtIndex:index1];
    
    NSAssert(group1, @"Index1 does not exist in itemGroups");
    
    FSMenuItemGroup *group2 = [itemGroups objectAtIndex:index2];
    
    NSAssert(group2, @"Index2 does not exist in itemGroups");
    
    [itemGroups replaceObjectAtIndex:index1 withObject:group2];
    [itemGroups replaceObjectAtIndex:index2 withObject:group1];
}

- (void)insertMenuItemGroup:(FSMenuItemGroup *)group beforeGroup:(FSMenuItemGroup *)siblingGroup forMenuViewType:(FSMenuViewType)menuViewType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[FSMenuItemGroup class]], @"The group should be a subclass of FSMenuItemGroup");
    NSAssert(siblingGroup, @"The siblingGroup is not nil");
    
    NSMutableArray *itemGroups = self.menuItemGroups[@(menuViewType).stringValue];
    if (!itemGroups) itemGroups = @[].mutableCopy;
    
    NSAssert([itemGroups containsObject:siblingGroup], @"The siblingGroup needs to be added");
    
    NSUInteger index = [itemGroups indexOfObject:siblingGroup];
    [itemGroups insertObject:group atIndex:index];
    group.supgroups = itemGroups;

}
- (void)insertMenuItemGroup:(FSMenuItemGroup *)group afterGroup:(FSMenuItemGroup *)siblingGroup forMenuViewType:(FSMenuViewType)menuViewType{
    NSAssert(group, @"The group is not nil");
    NSAssert([group isKindOfClass:[FSMenuItemGroup class]], @"The group should be a subclass of FSMenuItemGroup");
    NSAssert(siblingGroup, @"The siblingGroup is not nil");
    
    NSMutableArray *itemGroups = self.menuItemGroups[@(menuViewType).stringValue];
    if (!itemGroups) itemGroups = @[].mutableCopy;
    
    NSAssert([itemGroups containsObject:siblingGroup], @"The siblingGroup needs to be added");
    
    NSUInteger index = [itemGroups indexOfObject:siblingGroup];
    [itemGroups insertObject:group atIndex:index+1];
    group.supgroups = itemGroups;
}

- (void)removeMenuItemGroup:(FSMenuItemGroup *)group forMenuViewType:(FSMenuViewType)menuViewType{
    NSMutableArray *itemGroups = self.menuItemGroups[@(menuViewType).stringValue];
    if (itemGroups) {
        [itemGroups removeObject:group];
        if (!itemGroups.count) {
            [self.menuItemGroups setValue:nil forKey:@(menuViewType).stringValue];
        }
    }
}

- (NSArray <FSMenuItemGroup *> *)getMenuItemGroupsForMenuViewType:(FSMenuViewType)menuViewType{
    return self.menuItemGroups[@(menuViewType).stringValue].count ? self.menuItemGroups[@(menuViewType).stringValue].copy : nil;
}

- (void)addItemGroupForDefaultMenuViewType{
    [self addItemGroupForMenuViewTypeThumbnailInsertBlankPagePop];
    [self addItemGroupForMenuViewTypeExtract];
    [self addItemGroupForMenuViewTypeSaveAs];
}

- (void)addItemGroupForMenuViewTypeThumbnailInsertBlankPagePop{
    NSMutableArray *items = @[].mutableCopy;
    
    [items addObject:[[FSMenuItem alloc] initWithTitle:FSLocalizedForKey(@"kBlankPage") image:FSImageNamedAndNormalColor(@"thumbnail_blankpage") actionTag:0]];
    
    [items addObject:[[FSMenuItem alloc] initWithTitle:FSLocalizedForKey(@"kGallery") image:FSImageNamedAndNormalColor(@"thumbnail_image") actionTag:1]];
    
    [items addObject:[[FSMenuItem alloc] initWithTitle:FSLocalizedForKey(@"kCamera") image:FSImageNamedAndNormalColor(@"thumbnail_camera") actionTag:2]];
    
//    [items addObject:[[FSMenuItem alloc] initWithTitle:FSLocalizedForKey(@"kVideo") image:FSImageNamedAndNormalColor(@"thumbnail_video") actionTag:3]];
    
    [items addObject:[[FSMenuItem alloc] initWithTitle:FSLocalizedForKey(@"kImport") image:FSImageNamedAndNormalColor(@"thumbnail_import") actionTag:4]];
    
    FSMenuItemGroup *group = [[FSMenuItemGroup alloc] initWithTitle:FSLocalizedForKey(@"kAddPage") items:items];
    [self addMenuItemGroup:group forMenuViewType:FSMenuViewTypeThumbnailInsertBlankPagePop];
}

- (void)addItemGroupForMenuViewTypeExtract{
    
}

- (void)addItemGroupForMenuViewTypeSaveAs{
    
}

@end

@interface FSMenuItem ()
@property (nonatomic, weak) FSMenuItemGroup *supgroup;
@end

@implementation FSMenuItem

- (instancetype)init{
    self = [super init];
    if (self) {
        _actionTag = -1;
    }
    return self;
}

- (instancetype)initWithMenuItem:(MenuItem*)menuItem
{
    self = [self init];
    if (self) {
        _image = menuItem.image;
        _title = menuItem.title;
        self.target = menuItem.object;
        self.selAction = menuItem.action;
    }
    return self;
}

- (instancetype)initWithTitle:(NSString *)title image:(UIImage *)image action:(FSMenuItemAction)action{
    self = [self init];
    if (self) {
        _image = image;
        _title = title;
        self.action = [action copy];
    }
    return self;
}

- (NSUInteger)index{
    if (_supgroup) {
        return [_supgroup.items indexOfObject:self];;
    }
    return NSNotFound;
}

#pragma mark - private

- (instancetype)initWithTitle:(NSString *)title image:(UIImage *)image actionTag:(NSInteger)tag{
    self = [self init];
    if (self) {
        _image = image;
        _title = title;
        _actionTag = tag;
    }
    return self;
}

- (void)setAction:(FSMenuItemAction)action{
    _action = action;
    if (_action) {
        self.selAction = @selector(transfersAction);
    }else{
        self.selAction = nil;
    }
    
}

- (void)transfersAction{
    !_action ? : _action(self);
}

@end

@implementation FSMenuItemGroup

- (instancetype)initWithTitle:(nullable NSString *)title items:(NSArray <FSMenuItem *> *)items{
    self = [super init];
    if (self) {
        _items = items.mutableCopy;
        _title = title;
    }
    return self;
}

- (NSUInteger)index{
    if (_supgroups) {
        return [_supgroups indexOfObject:self];
    }
    return NSNotFound;
}

@end

@implementation FSActionControlAttribute @end
