/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSTextViewPopoverViewController.h"
#import "PanelController+private.h"

@interface FSTextViewPopoverViewController ()<UITextViewDelegate>{
    CGRect  _kbFrame;
}

@property (nonatomic, strong) UITextView *textView;

@property (nonatomic, assign) BOOL keyboardManagerEnable;
@property (nonatomic, assign) BOOL shouldResignOnTouchOutside;
@property (nonatomic, assign) CGSize viewSize;

@property (nonatomic, weak) FSPresentedViewControllerAnimator *animator;

@property (nonatomic, weak) FSItem *okItem;

@end

@implementation FSTextViewPopoverViewController

- (instancetype)initWithServices:(FSViewControllerManager *)viewManager params:(nullable id)params{
    self = [super initWithServices:viewManager params:params];
    if (self) {
        [self configureModalPresentationStyle];
    }
    return self;
}

- (void)configureModalPresentationStyle{
    if (DEVICE_iPHONE) {
        FSPresentedViewControllerAnimator *animator = [[FSPresentedViewControllerAnimator alloc] init];
        animator.size = self.viewSize;
        animator.presentationStyle = FSModalPresentationCenter;
        [self fs_setCustomTransitionAnimator:animator];
        self.animator = animator;
    }else{
        self.modalPresentationStyle = UIModalPresentationFormSheet;
    }
}

- (void)initialize{
    [super initialize];
    self.didClickedOKWithDismissSelf = YES;
    self.title = FSLocalizedForKey(@"kNote");
}

- (UITextView *)textView{
    if(!_textView){
        _textView = [UITextView new];
        _textView.accessibilityIdentifier = @"input_text_id";
        _textView.font = self.fs_standardGeneralFont_GF1;
        _textView.textColor = self.fs_textColor_T4;
        _textView.delegate = self;
    }
    return _textView;
}

- (FSViewControllerManager *)viewManager{
    return self.services;
}

- (FSItem *)okItem{
    if (!_okItem) {
        _okItem = self.navBar.rightItem;
        if (_okItemEnabled) {
            _okItem.enabled = self.okItemEnabled(self.textView.text);
        }else{
            _okItem.enabled = !(self.textView.text.hash == _text.hash);
        }
    }
    return _okItem;
}

- (void)viewDidLoad{
    [super viewDidLoad];
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    
    [self.view addSubview:self.textView];
    self.textView.backgroundColor = self.view.backgroundColor;
    
    @weakify(self)
    self.okItem.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        !self.didClickedOK ? : self.didClickedOK(self.textView.text);
        if (self.didClickedOKWithDismissSelf) [self dismissViewControllerAnimated:YES completion:nil];
    };
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    ///Fixed IQKeyboardManager first keyboard pop-up position calculation error
    _keyboardManagerEnable =  [IQKeyboardManager sharedManager].enable;
    _shouldResignOnTouchOutside = [IQKeyboardManager sharedManager].shouldResignOnTouchOutside;
    [[IQKeyboardManager sharedManager] setEnable:NO];
    [[IQKeyboardManager sharedManager] setShouldResignOnTouchOutside:NO];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHide:) name:UIKeyboardWillHideNotification object:nil];
    [self.textView becomeFirstResponder];
    [self.viewManager.extensionsManager cancelAutoFullScreen];

}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [[IQKeyboardManager sharedManager] setEnable:_keyboardManagerEnable];
    [[IQKeyboardManager sharedManager] setShouldResignOnTouchOutside:_shouldResignOnTouchOutside];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self.viewManager.extensionsManager delayHideTopBottomToolBar];
}

- (void)setText:(NSString *)text{
    _text = text;
    self.textView.text = text;
}

- (void)setOkItemEnabled:(BoolBlock_string)okItemEnabled{
    _okItemEnabled = okItemEnabled;
    if (_okItemEnabled) {
        self.okItem.enabled = self.okItemEnabled(self.textView.text);
    }
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    
    if (DEVICE_iPHONE) {
        FSPDFViewCtrl *pdfView = self.viewManager.extensionsManager.pdfViewCtrl;
        UIView *referView;
        if ([self.params isKindOfClass:[UIView class]] && pdfView) {
            referView = (UIView *)self.params;
        }
        CGSize size = CGSizeMake(MIN(pdfView.fs_width, pdfView.fs_height), MAX(pdfView.fs_width, pdfView.fs_height));
        UIInterfaceOrientation interfaceOrientation = [UIApplication sharedApplication].statusBarOrientation;
        if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight) {
            size = CGSizeMake(MIN(pdfView.fs_width, pdfView.fs_height), MIN(pdfView.fs_width, pdfView.fs_height));
            size.height = size.height - STATUS_BAR_HEIGHT_NORMAL * 2;
        }else{
            size.height = size.height - (referView != nil ? referView.fs_height * 2 : STATUS_BAR_HEIGHT_NORMAL * 4);
        }
        self.viewSize = size;
        if (self.animator) {
            self.animator.size = self.viewSize;
        }
    }else{self.viewSize = self.view.fs_size;}
    [self adjustTextViewFrame];
}

#define TextViewEdgeInsets UIEdgeInsetsMake(18.f, 18.f, 18.f, 18.f)
- (void)adjustTextViewFrame{
    if ([IQKeyboardManager sharedManager].isKeyboardShowing) {
        UIWindow *keyWindow = self.fs_currentSceneKeyWindow;
        CGRect selfViewConvertRect = [self.view convertRect:self.view.frame toView:keyWindow];
        CGRect intersectionRect =  CGRectIntersection(_kbFrame, selfViewConvertRect);
        self.textView.frame = CGRectMake(TextViewEdgeInsets.left, self.navBar.fs_bottom + TextViewEdgeInsets.top,self.viewSize.width - TextViewEdgeInsets.left - TextViewEdgeInsets.bottom, self.viewSize.height - TextViewEdgeInsets.top - self.navBar.fs_height - CGRectGetHeight(intersectionRect));
    }else{
        self.textView.frame = CGRectMake(TextViewEdgeInsets.left, self.navBar.fs_bottom + TextViewEdgeInsets.top,self.viewSize.width - TextViewEdgeInsets.left - TextViewEdgeInsets.bottom, self.viewSize.height - TextViewEdgeInsets.top - TextViewEdgeInsets.bottom - self.navBar.fs_height);
    }
}

#undef TextViewEdgeInsets
#pragma mark - UIKeyboad Notification methods
-(void)keyboardWillShow:(NSNotification*)aNotification{
    _kbFrame = [[aNotification userInfo][UIKeyboardFrameEndUserInfoKey] CGRectValue];
    [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        [self adjustTextViewFrame];
    } completion:^(BOOL finished) {
            
    }];

}
- (void)keyboardWillHide:(NSNotification*)aNotification{
    _kbFrame = CGRectZero;
    [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        [self adjustTextViewFrame];
    } completion:^(BOOL finished) {
            
    }];
}
#pragma  mark - UITextViewDelegate

- (void)textViewDidChange:(UITextView *)textView{
    if (self.okItemEnabled) {
        self.okItem.enabled =  !self.okItemEnabled ? : self.okItemEnabled(textView.text);
    }else{
        self.okItem.enabled = !(textView.text.hash == _text.hash);
    }
}

- (void)dealloc
{
    if (self.extensionsMgr) {
        [self.extensionsMgr showMenuIfNeed];
    }
}

@end

