/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSViewControllerManager.h"
#import "FSTableViewController.h"
#import "FSSearchViewController.h"
#import "FSMoreTableViewController.h"
#import "SaveAsViewController.h"
#import "FSToolsSortMenuViewController.h"

typedef UIView* (^SourceViewBlock)(UIPopoverPresentationController *popover);

@interface FSNavigationControllerStack ()
- (void)removeAllNavigationControllers;
@end
@interface FSViewController (){
    @public
    id<FSViewControllerServices> _services;
}
@end

@interface FSViewControllerManager ()<IViewAppleEventListener,IWindowFrameChangedListener>
@property(nonatomic, strong) FSTableViewPopoverController *curTablePopoverVC;
@property(nonatomic, copy) SourceViewBlock sourceView;
@property(nonatomic, strong) NSArray<FSPopoverItemGroup *> * groupItems;
@end
@implementation FSViewControllerManager

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        UIViewController *viewController = extensionsManager.pdfViewCtrl.fs_viewController;
        [self resetRootViewController:viewController];
        
        [extensionsManager.pdfViewCtrl registerViewAppleEventListener:self];
    }
    return self;
}

- (void)dealloc{
    [_extensionsManager.pdfViewCtrl unregisterViewAppleEventListener:self];
    [_extensionsManager unregisterWindowFrameChangedListener:self];
}

- (void)registerEventListens{
    [_extensionsManager registerWindowFrameChangedListener:self];
}

- (UIViewController *)topMostViewController{
    return [self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController];
}

- (FSViewController *)createViewCtrlType:(FSViewCtrlType)type{
    return [self createViewCtrlType:type params:nil];
}
- (FSViewController *)pushViewCtrlType:(FSViewCtrlType)type{
    return [self pushViewCtrlType:type params:nil];
}
- (FSViewController *)pushViewCtrlType:(FSViewCtrlType)type params:(nullable id)params{
    FSViewController *viewController = [self createViewCtrlType:type params:params];
    [self pushViewController:viewController animated:YES];
    return viewController;
}
- (FSViewController *)presentViewCtrlType:(FSViewCtrlType)type{
    return [self presentViewCtrlType:type params:nil];
}
- (__kindof FSViewController *)presentViewCtrlType:(FSViewCtrlType)type params:(nullable id)params{
    return [self presentViewCtrlType:type params:params completion:nil];
}
- (FSViewController *)presentViewCtrlType:(FSViewCtrlType)type completion:(nullable VoidBlock)completion{
    return [self presentViewCtrlType:type params:nil completion:completion];
}
- (FSViewController *)presentViewCtrlType:(FSViewCtrlType)type params:(nullable id)params completion:(nullable VoidBlock)completion{
    FSViewController *viewController = [self createViewCtrlType:type params:params];
    [self presentViewController:viewController animated:YES completion:completion];
    return viewController;
}

- (void)presentViewController:(UIViewController *)viewController animated:(BOOL)animated completion:(VoidBlock)completion{
    if (!viewController.transitioningDelegate) {
        if (![viewController isKindOfClass:UINavigationController.class]) {
            viewController = [[FSNavigationController alloc] initWithRootViewController:viewController];
        }
        if (DEVICE_iPHONE || [self.extensionsManager fs_currentSceneiPadHorizontalCompact]) {
            [viewController fs_setCustomTransitionAnimator:self.extensionsManager.presentedVCAnimator];
        }else{
            viewController.modalPresentationStyle = UIModalPresentationFormSheet;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
            if (@available(iOS 13.0, *)) viewController.modalInPresentation = YES;
#endif
        }
    }
    [super presentViewController:viewController animated:YES completion:completion];
}

- (FSViewPopoverController *)popPopoverCustomView:(UIView *)view dismissCompletion:(VoidBlock)dismissCompletion sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView
{
    NSAssert(!CGSizeEqualToSize(view.fs_size, CGSizeZero), @"popPopoverCustomView size is not CGSizeZero");
    FSViewPopoverController *popoverVC = [[FSViewPopoverController alloc] initWithServices:self showStyle:sourceView ? FSPopoverShowStyleStyleBubble: FSPopoverShowStyleDefault targetView:view];
    [popoverVC setPopOverDismissCompletion:dismissCompletion];
    [self popPopoverController:popoverVC popView:view sourceView:sourceView sourceRect:CGRectZero];
    return popoverVC;
}

- (FSViewPopoverController *)popPopoverCustomView:(UIView *)view sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    return [self popPopoverCustomView:view sourceView:sourceView sourceRect:CGRectZero];
}

- (FSViewPopoverController *)popPopoverCustomView:(UIView *)view sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView sourceRect:(CGRect)sourceRect{

    NSAssert(!CGSizeEqualToSize(view.fs_size, CGSizeZero), @"popPopoverCustomView size is not CGSizeZero");
    FSViewPopoverController *popoverVC = [[FSViewPopoverController alloc] initWithServices:self showStyle:sourceView ? FSPopoverShowStyleStyleBubble: FSPopoverShowStyleDefault targetView:view];

    [self popPopoverController:popoverVC popView:view sourceView:sourceView sourceRect:sourceRect];
    return popoverVC;
}

- (FSViewPopoverController *)popPopoverCustomViewCtrl:(UIViewController *)viewCtrl sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    return [self popPopoverCustomViewCtrl:viewCtrl sourceView:sourceView sourceRect:CGRectZero];
}

- (FSViewPopoverController *)popPopoverCustomViewCtrl:(UIViewController *)viewCtrl sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView sourceRect:(CGRect)sourceRect{
    NSAssert(!CGSizeEqualToSize(viewCtrl.view.fs_size, CGSizeZero), @"popPopoverCustomView size is not CGSizeZero");
    FSViewPopoverController *popoverVC = [[FSViewPopoverController alloc] initWithServices:self showStyle:sourceView ? FSPopoverShowStyleStyleBubble: FSPopoverShowStyleDefault targetViewController:viewCtrl];

    [self popPopoverController:popoverVC popView:viewCtrl.view sourceView:sourceView sourceRect:sourceRect];
    return popoverVC;
}

- (void)popPopoverController:(FSViewPopoverController *)popoverVC popView:(UIView*)popView sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView sourceRect:(CGRect)sourceRect{
    
    if (sourceView) {
        UIPopoverPresentationController *_popover = [popoverVC popoverPresentationController];
        UIView *popoverSourceView = sourceView(_popover);
        _popover.sourceView = popoverSourceView;
        if (CGRectEqualToRect(sourceRect, CGRectZero)) {
            _popover.sourceRect = popoverSourceView.bounds;
        }else{
            _popover.sourceRect = sourceRect;
        }
    }else{
    }

    if (CGSizeEqualToSize(popoverVC.popoverSize, CGSizeZero) && popoverVC.popoverWidth == 0) {
        if (sourceView) {
            popoverVC.popoverWidth = popView.fs_width;
            popoverVC.popoverSize = popView.fs_size;
        }else{
            if (self.extensionsManager.pdfViewCtrl.fs_width > self.extensionsManager.pdfViewCtrl.fs_height) {
                popoverVC.popoverSize = CGSizeMake(self.extensionsManager.pdfViewCtrl.fs_height, popView.fs_height);
            }else{
                popoverVC.popoverSize = CGSizeMake(self.extensionsManager.pdfViewCtrl.fs_width, popView.fs_height);
            }
        }
    }
    
    BOOL resignFirstResponderGestureEnabled = [[IQKeyboardManager sharedManager] resignFirstResponderGesture].enabled;
    [[self.extensionsManager fs_currentSceneTopMostViewController] presentViewController:popoverVC animated:YES completion:^{
        [[IQKeyboardManager sharedManager] resignFirstResponderGesture].enabled = NO;
    }];
    popoverVC.commonBlock = ^{
        [[IQKeyboardManager sharedManager] resignFirstResponderGesture].enabled = resignFirstResponderGestureEnabled;
    };

}

- (void)resetRootViewController:(UIViewController *)viewController{
    if (!viewController) return;
    [self.navigationControllerStack removeAllNavigationControllers];
    
    [self.navigationControllerStack pushNavigationController:(UINavigationController *)viewController];
}

- (FSTableViewPopoverController *)popPopoverCtrlClass:(Class)ctrlClass items:(NSArray<FSPopoverItem *> *)items sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    return [self popPopoverCtrlClass:ctrlClass groups:@[[[FSPopoverItemGroup alloc] initWithTitle:nil items:items]] sourceView:sourceView];
}

- (__kindof FSTableViewPopoverController *)popPopoverCtrlClass:(Class)ctrlClass groups:(NSArray<FSPopoverItemGroup *> *)groups sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    return [self popPopoverCtrlClass:ctrlClass groups:groups sourceView:sourceView autoAdapt:YES];
}

- (FSTableViewPopoverController *)popPopoverCtrlClass:(Class)ctrlClass groups:(NSArray<FSPopoverItemGroup *> *)groups sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView autoAdapt:(BOOL)autoAdapt{
    NSAssert([ctrlClass isSubclassOfClass:[FSTableViewPopoverController class]], @"ctrlClass must be FSTableViewPopoverController subclasses");
    
    FSTableViewPopoverController *popoverVC = [[ctrlClass alloc] initWithServices:self showStyle:(sourceView && ![self.extensionsManager fs_currentSceneiPadHorizontalCompact]) ? FSTableViewPopoverShowStyleBubble: FSTableViewPopoverShowStyleDefault groups:groups.copy];
    
    if (DEVICE_iPAD && autoAdapt) {
        self.curTablePopoverVC = popoverVC;
        self.groupItems = groups;
        self.sourceView = sourceView;
    }else{
        self.curTablePopoverVC = nil;
        self.groupItems = nil;
        self.sourceView = nil;
    }

    if (sourceView && ![self.extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        UIPopoverPresentationController *_popover = [popoverVC popoverPresentationController];
        UIView *popoverSourceView = sourceView(_popover);
        _popover.sourceView = popoverSourceView;
        _popover.sourceRect = popoverSourceView.bounds;
    }else{
        
    }
    if (CGSizeEqualToSize(popoverVC.popoverSize, CGSizeZero) && popoverVC.popoverWidth == 0) {
        if (sourceView && ![self.extensionsManager fs_currentSceneiPadHorizontalCompact]) {
            popoverVC.popoverWidth = 342.f;
        }else{
            popoverVC.popoverWidth = MIN(self.extensionsManager.pdfViewCtrl.fs_width, self.extensionsManager.pdfViewCtrl.fs_height) ;
        }
    }
    BOOL resignFirstResponderGestureEnabled = [[IQKeyboardManager sharedManager] resignFirstResponderGesture].enabled;
    [[self.extensionsManager fs_currentSceneTopMostViewController] presentViewController:popoverVC animated:YES completion:^{
        [[IQKeyboardManager sharedManager] resignFirstResponderGesture].enabled = NO;
    }];
    popoverVC.commonBlock = ^{
        [[IQKeyboardManager sharedManager] resignFirstResponderGesture].enabled = resignFirstResponderGestureEnabled;
    };
    
    if (DEVICE_iPAD && autoAdapt) {
        popoverVC.splitScrrenDismissCompletion = ^{
            self.curTablePopoverVC = nil;
        };
    }
    return popoverVC;
}

- (void)popPopoverWithGroups:(NSArray<FSPopoverItemGroup *> *)groups sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    if (!groups.count) return;
    [self popPopoverCtrlClass:[FSTableViewPopoverController class] groups:groups sourceView:sourceView];
}

- (void)popPopoverWithItems:(NSArray<FSPopoverItem *> *)items sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    [self popPopoverWithGroups:@[[[FSPopoverItemGroup alloc] initWithTitle:nil items:items]] sourceView:sourceView];
}

- (void)showMoreMenuWithItems:(NSArray<FSPopoverItem *> *)items sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    [self showMoreMenuWithItems:items dismissCompletion:nil sourceView:sourceView];
}

- (void)showMoreMenuWithItems:(NSArray<FSPopoverItem *> *)items dismissCompletion:(nullable VoidBlock)dismissCompletion sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    if (!items.count) return;
    [[self popPopoverCtrlClass:[FSMoreMenuViewController class] items:items sourceView:sourceView] setDismissCompletion:[dismissCompletion copy]];
}

- (void)showMoreMenuWithTitles:(NSArray *)titles images:(NSArray *)images action:(FSPopoverItemAction)action sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView{
    [self showMoreMenuWithTitles:titles images:images action:action dismissCompletion:nil sourceView:sourceView];
}

- (void)showMoreMenuWithTitles:(NSArray *)titles images:(NSArray *)images action:(FSPopoverItemAction)action dismissCompletion:(nullable VoidBlock)dismissCompletion sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView {
    if (!titles.count) return;
    NSAssert(titles.count == images.count, @"Titles is equal to images");
    NSMutableArray *tmp = @[].mutableCopy;
    for (int i = 0 ; i < titles.count ;i++) {
        FSPopoverItem *item = [[FSPopoverItem alloc] initWithTitle:titles[i] image:images[i] action:^(FSPopoverItem * _Nonnull item, int index) {
            ! action ? : action(item,index);
        }];
        [tmp addObject:item];
    }
    [self showMoreMenuWithItems:tmp dismissCompletion:dismissCompletion sourceView:sourceView];
}

- (FSViewController *)createViewCtrlType:(FSViewCtrlType)type params:(nullable id)params{
    FSViewController *VC = nil;
    switch (type) {
        case FSViewCtrlTypeViewBase:
        {
            VC = [[FSViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeTabViewBase:
        {
            VC = [[FSTableViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeSearchBase:
        {
            VC = [[FSSearchViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeTablePopoverBase:
        {
            VC =  [[FSTableViewPopoverController alloc] initWithServices:self showStyle:FSTableViewPopoverShowStyleDefault groups:params];
        }
            break;
        case FSViewCtrlTypeMoreTablePopover:
        {
            VC =  [[FSMoreTableViewController alloc] initWithServices:self showStyle:FSTableViewPopoverShowStyleDefault];
        }
            break;
        case FSViewCtrlTypeAnnotFilter:
        {
            VC = [[FSAnnotFilterViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeFileSelect:
        {
            VC = [[FSFileSelectDestinationViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeCheckCer:
        {
            VC = [self fs_storyBoardViewControllerWithName:@"CerOperationView" identifier:@"CheckCerViewCtr"];
            VC->_services = self;
            [(CheckCerViewCtr *)VC setCurrentSelectSign:params];
        }
            break;
        case FSViewCtrlTypeCerInfo:
        {
            VC = [self fs_storyBoardViewControllerWithName:@"CerOperationView" identifier:@"CerInfoViewController"];
            VC->_services = self;
            [(CerInfoViewController *)VC setCerInfoDic:params];
        }
            break;
        case FSViewCtrlTypeCerTrustedList:
        {
            VC = [self fs_storyBoardViewControllerWithName:@"CerOperationView" identifier:@"CerTrustedListViewCtr"];
            VC->_services = self;
        }
            break;
            
        case FSViewCtrlTypeTextViewPopover:
        {
            VC = [[FSTextViewPopoverViewController alloc] initWithServices:self params:params];
            VC.fs_presentByTopViewController = YES;
        }
            break;
        case FSViewCtrlTypeInsertPageConfig:
        {
            VC = [[FSInsertPageConfigVCtrl alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeInsertPageSizeOrOri:
        {
            VC = [[FSInsertPageSizeOrOriVCtrl alloc] initWithServices:self params:params];
            NSAssert([params isKindOfClass:[NSNumber class]], @"Must be a subclass of NSNumber %@'s params",NSStringFromClass([VC class]));
        }
            break;
        case FSViewCtrlTypeInsertPageColor:
        {
            VC = [[FSInsertPageColorViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeSaveAs:
        {
            //            VC = [[FSSaveAsViewController alloc] initWithServices:self params:params];
            VC = [[SaveAsViewController alloc] init];
            VC->_services = self;
            break;
        }
        case FSViewCtrlTypeCustomViewContainer:
        {
            NSAssert([params isKindOfClass:[UIView class]], @"Must be a subclass of UIView %@'s params",NSStringFromClass([FSCustomViewContainerController class]));
            VC = [[FSCustomViewContainerController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeAddTools:
        {
            NSAssert([params isKindOfClass:[FSMainTopbarToolTagItem class]], @"Must be a subclass of FSMainTopbarToolTagItem %@'s params",NSStringFromClass([FSMainTopbarToolTagItem class]));
            VC = [[FSAddToolsViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeSortTools:
        {
            NSAssert([params isKindOfClass:[FSMainTopbarToolTagItem class]], @"Must be a subclass of FSMainTopbarToolTagItem %@'s params",NSStringFromClass([FSMainTopbarToolTagItem class]));
            VC = [[FSToolsSortMenuViewController alloc] initWithServices:self params:params];
        }
            break;
        case FSViewCtrlTypeSignatureList:
        {
            VC = [[SignatureListViewController alloc] initWithServices:self params:params];
        }
            break;
            
    }
    assert(VC);
    return VC;
}

#pragma mark -IViewAppleEventListener

- (void)didMoveToWindowFromPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    UIViewController *viewController = pdfViewCtrl.fs_viewController;
    if ([[self.navigationControllerStack navigationControllers] firstObject] != viewController) {
        [self resetRootViewController:viewController];
    }
}

#pragma mark -IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed {
    if (DEVICE_iPAD && changed) {
        if (([self.extensionsManager fs_currentSceneiPadHorizontalCompact] && self.curTablePopoverVC && self.curTablePopoverVC.popoverShowStyle == FSTableViewPopoverShowStyleBubble) ||
            (![self.extensionsManager fs_currentSceneiPadHorizontalCompact] && self.curTablePopoverVC && self.curTablePopoverVC.popoverShowStyle != FSTableViewPopoverShowStyleBubble)) {
   
            Class popoverClass = [self.curTablePopoverVC class];
            VoidBlock dismissCompletion = [self.curTablePopoverVC.dismissCompletion copy];
            [self.curTablePopoverVC dismissViewControllerAnimated:YES completion:^{
                self.curTablePopoverVC = nil;
                [[self popPopoverCtrlClass:popoverClass groups:self.groupItems sourceView:self.sourceView] setDismissCompletion:dismissCompletion];
            }];
        }
    }
}
@end

@implementation UIViewController (FSViewControllerManager)

- (void)fs_presentViewController:(UIViewController *)viewController animated:(BOOL)flag completion:(void (^__nullable)(void))completion{
    if ([self isKindOfClass:[FSViewController class]]) {
        FSViewControllerManager *services;
        if ([((FSViewController *)self).services isKindOfClass:[FSViewControllerManager class]]) {
            services = (FSViewControllerManager *)((FSViewController *)self).services;
        }
        if (!viewController.transitioningDelegate) {
            if (![viewController isKindOfClass:UINavigationController.class]) {
                viewController = [[FSNavigationController alloc] initWithRootViewController:viewController];
            }
            if (DEVICE_iPHONE || [self fs_currentSceneiPadHorizontalCompact]) {
                [viewController fs_setCustomTransitionAnimator:services.extensionsManager.presentedVCAnimator];
            }else{
                viewController.modalPresentationStyle = UIModalPresentationFormSheet;
    #if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
                if (@available(iOS 13.0, *)) viewController.modalInPresentation = YES;
    #endif
            }
        }
    }
    [self presentViewController:viewController animated:flag completion:completion];
}

@end
