/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "MenuControl.h"
#import "MenuItem.h"
#import "ReplyTableViewController.h"
#import "SelectToolHandler.h"

#define  MAGIC_CLICKED_ @"magic_clicked_"
@interface MenuControl ()
@property (nonatomic, strong) UIMenuController *menuControl;
@property (nonatomic, assign) BOOL isEqualLastTargetRect;
@property (nonatomic, strong) NSArray *lastMenuItems;
@property (nonatomic, assign) CGRect lastFrame;
@property (nonatomic, assign) CGRect targetRect;
@end

@implementation MenuControl {
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    UIExtensionsManager * __weak _extensionsManager;
    UILongPressGestureRecognizer *_longpressGesture;
    UIPanGestureRecognizer *_panGesture;
    UITapGestureRecognizer *_tapGesture;
}

- (void)handlePanGesture:(UIPanGestureRecognizer *)recognizer {
    [_extensionsManager onPan:recognizer];
}

- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)recognizer {
    if (recognizer == _tapGesture || recognizer == _longpressGesture)
        return YES;
    if (recognizer == _panGesture) {
        return [_extensionsManager onShouldBegin:recognizer];
    }
    return [super gestureRecognizerShouldBegin:recognizer];
}

- (id)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = _extensionsManager.pdfViewCtrl;
        self.menuControl = [UIMenuController sharedMenuController];

        _panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePanGesture:)];
        [self addGestureRecognizer:_panGesture];
        self.shouldBecomeFirstResponder = YES;
    }
    return self;
}

- (void)_setMenuVisible:(BOOL)menuVisible animated:(BOOL)animated {
    if (menuVisible != [self isMenuVisible]) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.menuControl setMenuVisible:menuVisible animated:animated];
        });
    }
    if (!menuVisible) {
        self.frame = CGRectZero;
        [self resignFirstResponder];
    }
}

- (void)setMenuVisible:(BOOL)menuVisible animated:(BOOL)animated {
    [self _setMenuVisible:menuVisible animated:animated];
    if (menuVisible) {
        [self.delegate menuControlDidShow:self];
    } else {
        [self.delegate menuControlDidHide:self];
    }
}

- (BOOL)isMenuVisible {
    return self.menuControl.isMenuVisible || !CGRectEqualToRect(self.frame, CGRectZero);
}

- (void)hideMenu {
    [self setMenuVisible:NO animated:YES];
}

- (void)setRect:(CGRect)rect {
    [self setRect:rect margin:10];
}

- (void)setRect:(CGRect)rect margin:(float)margin {
    UIView *displayView = [_pdfViewCtrl getDisplayView];
    CGRect frame = CGRectInset(rect, -35, -35);
    if ((CGRectGetMinX(frame) < 0 || CGRectGetMinY(frame) < 0) && (CGRectGetWidth(frame) > CGRectGetWidth(displayView.frame) && CGRectGetHeight(frame) > CGRectGetHeight(displayView.frame))) {
        frame = CGRectUnion(frame, displayView.frame);
    }
    self.frame = frame;
    [[_pdfViewCtrl getDisplayView] insertSubview:self atIndex:0];
    CGRect targetRect = CGRectInset(self.bounds, 35 - margin, 35 - margin);
    _isEqualLastTargetRect = CGRectEqualToRect(targetRect, _targetRect) && CGRectEqualToRect(frame, _lastFrame);
    _targetRect = targetRect;
    _lastFrame = frame;
    [self.menuControl setTargetRect:_targetRect inView:self];
}

- (void)showMenu {
    
    if (_isEqualLastTargetRect && self.menuItems == self.lastMenuItems && self.menuControl.isMenuVisible) {
        return;
    }
    self.menuControl.arrowDirection = UIMenuControllerArrowDefault;
    if ([_extensionsManager.topToolbar contentView] && ![_extensionsManager.topToolbar contentView].hidden) {
        CGRect topbarRect = [[_extensionsManager.topToolbar contentView] convertRect:[_extensionsManager.topToolbar contentView].frame toView:[_pdfViewCtrl getDisplayView] ];
        if (CGRectIntersectsRect(topbarRect, CGRectInset(self.frame, -20, -20))) {
            self.menuControl.arrowDirection = UIMenuControllerArrowUp;
        }
    }
    
    CGPoint leftTopPt = [self.superview convertPoint:CGPointMake(CGRectGetMinX(self.frame), CGRectGetMinY(self.frame)) toView:_pdfViewCtrl.fs_currentSceneKeyWindow];
    
    CGPoint leftBottomPt = [self.superview convertPoint:CGPointMake(CGRectGetMinX(self.frame), CGRectGetMaxY(self.frame)) toView:_pdfViewCtrl.fs_currentSceneKeyWindow];
    
    CGPoint rightTopPt = [self.superview convertPoint:CGPointMake(CGRectGetMaxX(self.frame), CGRectGetMinY(self.frame)) toView:_pdfViewCtrl.fs_currentSceneKeyWindow];
    
    CGPoint rightBottomPt = [self.superview convertPoint:CGPointMake(CGRectGetMaxX(self.frame), CGRectGetMaxY(self.frame)) toView:_pdfViewCtrl.fs_currentSceneKeyWindow];
    
    CGRect viewRect = CGRectMake(leftTopPt.x, leftTopPt.y, rightTopPt.x - leftTopPt.x, rightBottomPt.y - leftTopPt.y);
    CGPoint viewCenterPt = CGPointMake(CGRectGetMidX(viewRect), CGRectGetMidY(viewRect));
    
    if ([[_pdfViewCtrl.fs_currentSceneKeyWindow hitTest:leftTopPt withEvent:nil] isDescendantOfView:_pdfViewCtrl] ||
        [[_pdfViewCtrl.fs_currentSceneKeyWindow hitTest:leftBottomPt withEvent:nil] isDescendantOfView:_pdfViewCtrl] ||
        [[_pdfViewCtrl.fs_currentSceneKeyWindow hitTest:rightTopPt withEvent:nil] isDescendantOfView:_pdfViewCtrl] ||
        [[_pdfViewCtrl.fs_currentSceneKeyWindow hitTest:rightBottomPt withEvent:nil] isDescendantOfView:_pdfViewCtrl] ||
        [[_pdfViewCtrl.fs_currentSceneKeyWindow hitTest:viewCenterPt withEvent:nil] isDescendantOfView:_pdfViewCtrl] ) {

        NSMutableArray *menuArray = [[NSMutableArray alloc] init];
        for (int i = 0; i < self.menuItems.count; i++) {
            MenuItem *mcItem = [self.menuItems objectAtIndex:i];
            UIMenuItem *menuItem = [[UIMenuItem alloc] initWithTitle:mcItem.title image:mcItem.image action:NSSelectorFromString([NSString stringWithFormat:@"%@%i",MAGIC_CLICKED_, i])];
            menuItem.accessibilityLabel = mcItem.title;
            [menuArray addObject:menuItem];
        }
        [self.menuControl setMenuItems:menuArray];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.menuControl setMenuVisible:YES animated:YES];
        });
        self.lastMenuItems = self.menuItems;
    }
    [self.delegate menuControlDidShow:self];
    
    if (_extensionsManager.shouldShowMenu && self.shouldBecomeFirstResponder) {
        if (!self.window.isKeyWindow){
            [self.window makeKeyAndVisible];
        }
         [self becomeFirstResponder];
    }
}

- (BOOL)canBecomeFirstResponder {
    return YES;
}

- (BOOL)canPerformAction:(SEL)action withSender:(id)sender {
    NSString *actionName = NSStringFromSelector(action);
    NSRange match = [actionName rangeOfString:MAGIC_CLICKED_];
    if (match.location == 0) {
        return YES;
    }
    return NO;
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)sel {
    if ([super methodSignatureForSelector:sel]) {
        return [super methodSignatureForSelector:sel];
    }
    NSString *selName = NSStringFromSelector(sel);
    NSRange match = [selName rangeOfString:MAGIC_CLICKED_];
    if (match.location == 0) {
        return [super methodSignatureForSelector:@selector(onClickedAtIndex:)];
    }
    return nil;
}

- (void)forwardInvocation:(NSInvocation *)anInvocation {
    NSString *selName = NSStringFromSelector([anInvocation selector]);
    NSString *prefix = MAGIC_CLICKED_;
    NSRange match = [selName rangeOfString:prefix];
    if (match.location == 0) {
        [self onClickedAtIndex:[[selName substringFromIndex:prefix.length] intValue]];
    } else {
        [super forwardInvocation:anInvocation];
    }
}

- (void)onClickedAtIndex:(int)index {
    [self _setMenuVisible:NO animated:YES];
    if (self.menuItems.count < index + 1)
        return;
    MenuItem *mcItem = [self.menuItems objectAtIndex:index];
    if (mcItem.action) {
        ((void (*)(id, SEL, MenuItem *))[mcItem.object methodForSelector:mcItem.action])(mcItem.object, mcItem.action, mcItem);
    }
}

- (UIViewController *)topViewController {
    return [self topViewControllerWithRootViewController:_extensionsManager.pdfViewCtrl.fs_currentSceneTopMostViewController];
}

- (UIViewController *)topViewControllerWithRootViewController:(UIViewController *)rootViewController {
    if ([rootViewController isKindOfClass:[UINavigationController class]]) {
        UINavigationController *nav = (UINavigationController *) rootViewController;
        return [self topViewControllerWithRootViewController:nav.visibleViewController];
    } else if (rootViewController.presentedViewController) {
        UIViewController *presentedViewController = rootViewController.presentedViewController;
        return [self topViewControllerWithRootViewController:presentedViewController];
    } else {
        return rootViewController;
    }
}

@end
