/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIExtensionsManager+MultipleScenes.h"
#import "FormAnnotHandler.h"
#import <objc/runtime.h>
#import "StampUtil.h"

@protocol _IFSUIExtensionsModifiedEventListener <IAnnotEventListener, IPageEventListener, IFSUIExtensionsModifiedEventListener>

@end

@interface UIExtensionsModifiedContext : NSObject <_IFSUIExtensionsModifiedEventListener>
@property (nonatomic, strong) FSWeakPointerArray *listeners;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager;
- (void)registerModifiedEventListener:(id<IFSUIExtensionsModifiedEventListener>)listener;
- (void)unregisterModifiedEventListener:(id<IFSUIExtensionsModifiedEventListener>)listener;
@end
static NSMutableSet *_connectedExtensionsManagers;
static UIExtensionsManager *_currentExtManager;

@interface UIExtensionsManager ()
{
    BOOL _isDocModified;
}
@end
@implementation UIExtensionsManager (SupportsMultipleScenes)

+ (void)initialize{
    [super initialize];
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        [FSPDFViewCtrl registerMultipleScenesListener:(id<FSMultipleScenesListener>)self];
    });
}

+ (void)willAddPdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    if (self.supportsMultipleScenes) {
        if ([pdfViewCtrl.extensionsManager isKindOfClass:[UIExtensionsManager class]]) {
            if (@available(iOS 13.0, *)) {
                [self.connectedExtensionsManagers addObject:(UIExtensionsManager *)pdfViewCtrl.extensionsManager];
            } else {
                // Fallback on earlier versions
            }
        }
    }
}

+ (void)willRemovePdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    if (self.supportsMultipleScenes) {
        if ([pdfViewCtrl.extensionsManager isKindOfClass:[UIExtensionsManager class]]) {
            if (@available(iOS 13.0, *)) {
                [self.connectedExtensionsManagers removeObject:(UIExtensionsManager *)pdfViewCtrl.extensionsManager];
            } else {
                // Fallback on earlier versions
            }
        }
    }
}

+ (BOOL)supportsMultipleScenes{
    return [FSPDFViewCtrl supportsMultipleScenes];
}

 + (nonnull NSSet<id<FSPDFMultipleScenes>> *)connectedScenes API_AVAILABLE(ios(13.0)) {
    return _connectedExtensionsManagers.copy;
}

+ (nonnull NSSet<UIExtensionsManager *> *)openedExtensionsManagers {
    return _connectedExtensionsManagers.copy;
}

+ (NSMutableSet<id<FSPDFMultipleScenes>> *)connectedExtensionsManagers API_AVAILABLE(ios(13.0)){
    if (self.supportsMultipleScenes) {
        if (!_connectedExtensionsManagers) {
            _connectedExtensionsManagers = [NSMutableSet set];
        }
    }
    return _connectedExtensionsManagers;
}
/**
       secneA or secneB data update
              /           \
             /               \
        secneA              secneB
           /                        \
          /                             \
    extManagerA            extManagerB
 */

+ (void)addExtensionsManager:(UIExtensionsManager *)extensionsManager API_AVAILABLE(ios(13.0)){
    if (self.supportsMultipleScenes) {
        [self.connectedExtensionsManagers addObject:extensionsManager];
    }
}

+ (UIExtensionsManager *)currentExtManager{
    return _currentExtManager;
}

+ (void)setCurrentExtManager:(UIExtensionsManager *)currentExtManager{
    if (_currentExtManager == currentExtManager) {
        return;
    }
    [self updateCurrentExtensionsManagerBehavior];
    _currentExtManager = currentExtManager;
}

+ (void)updateCurrentExtensionsManagerBehavior{
    if (_currentExtManager.currentAnnot.nonempty) {
        _currentExtManager.currentAnnot = nil;
    }
    if (_currentExtManager.currentWidget && !_currentExtManager.currentWidget.isEmpty) {
        _currentExtManager.currentWidget = nil;
    }

    if (_currentExtManager.fillSignModule) {
        [_currentExtManager.fillSignModule deactiveFillSign];
    }
}

- (UIExtensionsModifiedContext *)modifiedContext{
    UIExtensionsModifiedContext *modifiedContext = [self fs_getAssociatedForKey:@keypath(self, modifiedContext)];
    if(!modifiedContext){
        modifiedContext =  [[UIExtensionsModifiedContext alloc] initWithExtensionsManager:self];
        [self fs_addAssociatedObject:modifiedContext forKey:@keypath(self, modifiedContext)];
    }
    return modifiedContext;
}

- (void)updateCallbackSubstitute{
    [FSCallbackSubstitute annotIconProviderCallbackSubstitute].currentTarget = self.iconProvider;
    [FSCallbackSubstitute actionHandlerSubstitute].currentTarget = self.actionHandler;
}

- (void)adapterMultipleScenes{
    UIExtensionsManager.currentExtManager = self;
    [self updateCallbackSubstitute];
}

- (BOOL)shouldSyncOtherScenes{
    return UIExtensionsManager.currentExtManager != self && [[UIExtensionsManager.currentExtManager.pdfViewCtrl currentDoc] isEqual:[self.pdfViewCtrl currentDoc]];
}

- (NSArray <UIExtensionsManager *> *)someDocFromOtherScenes{
    NSMutableArray *tmp = @[].mutableCopy;
    for (UIExtensionsManager *ext in UIExtensionsManager.openedExtensionsManagers) {
        if (ext == self) continue;
        if ([[ext.pdfViewCtrl currentDoc] isEqual:[self.pdfViewCtrl currentDoc]]) {
            [tmp addObject:ext];
        }
    }
    return tmp.copy;
}

- (void)supportsMultipleScenesOp:(void(^)(UIExtensionsManager *self))op{
    NSArray *otherScenes = [self someDocFromOtherScenes];
    op(self);
    for (UIExtensionsManager *otherExt in otherScenes) {
        op(otherExt);
    }
}

- (void)supportsMultipleScenesOp:(void(^)(UIExtensionsManager *self, id))op obj:(id)obj{
    NSArray *otherScenes = [self someDocFromOtherScenes];
    op(self, obj);
    for (UIExtensionsManager *otherExt in otherScenes) {
        op(otherExt, obj);
    }
}

- (void)supportsMultipleScenesOp:(void(^)(UIExtensionsManager *self, id, id))op obj1:(id)obj1 obj2:(id)obj2{
    NSArray *otherScenes = [self someDocFromOtherScenes];
    op(self, obj1, obj2);
    for (UIExtensionsManager *otherExt in otherScenes) {
        op(otherExt, obj1, obj2);
    }
}

- (void)registerExtensionsModifiedEventListener:(id<IFSUIExtensionsModifiedEventListener>)listener{
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        [self.modifiedContext registerModifiedEventListener:listener];
        [self registerAnnotEventListener:self.modifiedContext];
        [self.pdfViewCtrl registerPageEventListener:self.modifiedContext];
    }];
}

- (void)unregisterExtensionsModifiedEventListener:(id<IFSUIExtensionsModifiedEventListener>)listener{
    BOOL canUnregister = YES;
    NSArray *otherScenes = [self someDocFromOtherScenes];
    for (UIExtensionsManager *otherExt in otherScenes) {
        if ([otherExt.modifiedContext.listeners containsObject:listener]) {
            canUnregister = NO;
        }
    }
    if (canUnregister) {
        [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
            [self.modifiedContext unregisterModifiedEventListener:listener];
            [self unregisterAnnotEventListener:self.modifiedContext];
            [self.pdfViewCtrl unregisterPageEventListener:self.modifiedContext];
        }];
    }
}

- (void)onDocOpenedSupportsMultiplehook{
    if ([self someDocFromOtherScenes].count) {
        UIExtensionsManager *ext = [[self someDocFromOtherScenes] firstObject];
        self.undoItems = ext.undoItems;
        self.redoItems = ext.redoItems;
        _isDocModified = ext.isDocModified;
        [self undoRedoStateChanged];
    }
    
    NSArray *otherScenes = [self someDocFromOtherScenes];
    for (UIExtensionsManager *otherExt in otherScenes) {
        [otherExt.modifiedContext.listeners enumerateObjectsUsingBlock:^(id<IFSUIExtensionsModifiedEventListener>  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            if (![self.modifiedContext.listeners containsObject:obj]) {
                [self registerExtensionsModifiedEventListener:obj];
            }
        }];
    }
}

- (void)onDocWillCloseSupportsMultiplehook{
    if ([self someDocFromOtherScenes].count) {
        UIExtensionsManager *someDocExt = [[self someDocFromOtherScenes] firstObject];
        for (int i = 0; i < self.undoItems.count; i++) {
            UndoItem *ocitem = self.undoItems[i];
            ocitem.extensionsManager = someDocExt;
        }

        for (int i = 0; i < self.redoItems.count; i++) {
            UndoItem *ocitem = self.redoItems[i];
            ocitem.extensionsManager = someDocExt;
        }
        self.undoItems = @[].mutableCopy;
        self.redoItems = @[].mutableCopy;
    }
}

- (void)onDocClosedSupportsMultiplehook{
    [self.modifiedContext.listeners enumerateObjectsUsingBlock:^(id<IFSUIExtensionsModifiedEventListener>  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        [self unregisterExtensionsModifiedEventListener:obj];
    }];
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (UIWindowScene *)fs_windowScene{
    return self.windowScene;
}
#endif

@end


@interface ExAnnotIconProviderCallback ()

@property (nonatomic, strong) NSMutableArray *iconDocs;

@end

@implementation ExAnnotIconProviderCallback

- (NSString *)getProviderID {
    return @"FX";
}

- (NSString *)getProviderVersion {
    return @"0";
}

- (BOOL)hasIcon:(FSAnnotType)annotType iconName:(NSString *)iconName {
    if (annotType == FSAnnotNote || annotType == FSAnnotFileAttachment || annotType == FSAnnotStamp) {
        return YES;
    }
    return NO;
}
- (BOOL)canChangeColor:(FSAnnotType)annotType iconName:(NSString *)iconName {
    if (annotType == FSAnnotNote || annotType == FSAnnotFileAttachment) {
        return YES;
    }
    return NO;
}

- (FSShadingColor *)getShadingColor:(FSAnnotType)annotType iconName:(NSString *)iconName refColor:(unsigned int)refColor shadingIndex:(int)shadingIndex;
{
    FSShadingColor *shadingColor = [[FSShadingColor alloc] init];
    [shadingColor setFirst_color:refColor];
    [shadingColor setSecond_color:refColor];
    return shadingColor;
}

- (NSNumber *)getDisplayWidth:(FSAnnotType)annotType iconName:(NSString *)iconName {
    return [NSNumber numberWithFloat:32];
}

- (NSNumber *)getDisplayHeight:(FSAnnotType)annotType iconName:(NSString *)iconName {
    return [NSNumber numberWithFloat:32];
}

- (FSPDFPage *)getIcon:(FSAnnotType)annotType iconName:(NSString *)iconName color:(unsigned int)color {
    if (self.takeOverCallBack && [self.takeOverCallBack respondsToSelector:@selector(shouldTakeOverGetIcon:iconName:color:returnValue:)]) {
        FSPDFPage *page;
        if ([self.takeOverCallBack shouldTakeOverGetIcon:annotType iconName:iconName color:color returnValue:&page]) {
            return page;
        }
    }
    static NSArray *arrayNames = nil;
    if (!arrayNames) {
        arrayNames = [Utility getAllIconLowercaseNames];
    }

    NSInteger iconIndex = -1;
    if (annotType == FSAnnotNote || annotType == FSAnnotFileAttachment || annotType == FSAnnotStamp) {
        iconName = [iconName lowercaseString];
        if ([arrayNames containsObject:iconName]) {
            iconIndex = [arrayNames indexOfObject:iconName];
        }
    }

    if (iconIndex >= 0 && iconIndex < arrayNames.count) {
        if (!self.iconDocs) {
            self.iconDocs = [NSMutableArray arrayWithCapacity:arrayNames.count];
            for (int i = 0; i < arrayNames.count; i++) {
                [self.iconDocs addObject:[NSNull null]];
            }
        }

        FSPDFDoc *iconDoc = self.iconDocs[iconIndex];
        if ([iconDoc isEqual:[NSNull null]]) {
            NSString *path = [[NSBundle bundleForClass:[self class]] pathForResource:iconName ofType:@"pdf"];
            if (path) {
                iconDoc = [[FSPDFDoc alloc] initWithPath:path];
                FSErrorCode err = [iconDoc load:nil];
                if (FSErrSuccess == err) {
                    self.iconDocs[iconIndex] = iconDoc;
                }
            }
        }
        return [iconDoc isEqual:[NSNull null]] ? nil : [iconDoc getPage:0];
    }

    return nil;
}

- (FSShadingColor *)getShadingColor:(FSAnnotType)annotType iconName:(NSString *)iconName refColor:(unsigned int)refColor shadingIndex:(int)shadingIndex ShadingColor:(FSShadingColor *)out_shading_color {
    return [[FSShadingColor alloc] init];
}

@end


int _formCurrentPageIndex = 0;

@implementation ExActionHandler

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
    }
    return self;
}

- (int)getCurrentPage:(FSPDFDoc *)pdfDoc {
    if (_pdfViewCtrl.currentDoc == pdfDoc)
        return _formCurrentPageIndex;
    else
        return 0;
}

- (void)setCurrentPage:(FSPDFDoc *)pdfDoc pageIndex:(int)pageIndex {
    if (_pdfViewCtrl.currentDoc == pdfDoc)
        _formCurrentPageIndex = pageIndex;
}

- (FSRotation)getPageRotation:(FSPDFDoc *)pdfDoc pageIndex:(int)pageIndex {
    return 0;
}

- (BOOL)setPageRotation:(FSPDFDoc *)pdfDoc pageIndex:(int)pageIndex rotation:(FSRotation)rotation {
    return NO;
}

- (int)alert:(NSString *)msg title:(NSString *)title type:(int)type icon:(int)icon {
    self.type = type;
    self.retCode = -1;
    
    if (type == 0 || type == 4) {
        self.retCode = 1;
    } else if (type == 1) {
        self.retCode = 1;
    } else if (type == 2) {
        self.retCode = 4;
    } else if (type == 3) {
        self.retCode = 4;
    }
    NSString *toastStr = [NSString stringWithFormat:@"Alert: %@",msg];
    dispatch_main_async_safe(^{
        [self.extensionsManager fs_currentSceneToastWithMsg:toastStr];
    });
    return self.retCode;
}

- (NSString *)browseFile:(BOOL)is_open_dialog file_format:(NSString *)file_format file_filter:(NSString *)file_filter{
    FormAnnotHandler *handler = (FormAnnotHandler *) [_extensionsManager getAnnotHandlerByType:FSAnnotWidget];
    if (handler && [handler imageFieldBrowFileWithFileFilter:file_filter]) {
        return @"";
    }
    NSString *docName = [[[self.pdfViewCtrl filePath] lastPathComponent] stringByDeletingPathExtension];
    if (!docName.length) docName = [Utility getUUID];
    NSString *extension = file_format ? [file_format lowercaseString] : @"";
    NSString *savePath = [DOCUMENT_PATH stringByAppendingPathComponent:[docName stringByAppendingPathExtension:extension]];
    int renameCount = 1;
    while ([[NSFileManager defaultManager] fileExistsAtPath:savePath]) {
        savePath = [DOCUMENT_PATH stringByAppendingPathComponent:[[NSString stringWithFormat:@"%@ (%d) ",docName,renameCount] stringByAppendingPathExtension:extension]];
        renameCount++;
    }
    return savePath;
}

- (FSIdentityProperties *)getIdentityProperties {
    FSIdentityProperties *ip = [[FSIdentityProperties alloc] init];
//    [ip setCorporation:@"Foxit"];
//    [ip setEmail:@"Foxit"];
//    [ip setLogin_name:_extensionsManager.annotAuthor];
    [ip setName:_extensionsManager.annotAuthor];
    NSString *jsonString = [[NSUserDefaults standardUserDefaults] objectForKey:Key_Identity];
    NSMutableDictionary *identity = [[NSMutableDictionary alloc] init];
    if(jsonString && ![jsonString isEqualToString:@""]){
        identity = [Utility dictionaryWithJsonString:jsonString];
        NSString *firstName = [identity objectForKey:FSLocalizedForKey(@"kStampFirstName")] != nil ? [identity objectForKey:FSLocalizedForKey(@"kStampFirstName")] : @"" ;
        NSString *lastName = [identity objectForKey:FSLocalizedForKey(@"kStampLastName")] != nil ? [identity objectForKey:FSLocalizedForKey(@"kStampLastName")] : @"" ;
        NSString *title = [identity objectForKey:FSLocalizedForKey(@"kStampTitle")] != nil ? [identity objectForKey:FSLocalizedForKey(@"kStampTitle")] : @"" ;
        NSString *organizationName = [identity objectForKey:FSLocalizedForKey(@"kStampOrganizationName")] != nil ? [identity objectForKey:FSLocalizedForKey(@"kStampOrganizationName")] : @"" ;
        NSString *organizationUnit = [identity objectForKey:FSLocalizedForKey(@"kStampOrganizationUnit")] != nil ? [identity objectForKey:FSLocalizedForKey(@"kStampOrganizationUnit")] : @"" ;
        NSString *email = [identity objectForKey:FSLocalizedForKey(@"kStampEmail")] != nil ? [identity objectForKey:FSLocalizedForKey(@"kStampEmail")] : @"" ;
        [ip setFirst_name:firstName];
        [ip setLast_name:lastName];
        [ip setTitle:title];
        [ip setCorporation:organizationName];
        [ip setDepartment:organizationUnit];
        [ip setEmail:email];
    }
    return ip;
}

- (NSString *)GetJapaneseEras:(int)year month:(int)month day:(int)day {
    return [StampUtil GetJapaneseEras:year month:month day:day];
}

-(unsigned int)verifySignature:(FSPDFDoc *)document signature:(FSSignature *)signature{
    return FSSignatureStateUnknown;
}

static int _annotCountRecord;
- (BOOL)invalidateRect:(FSPDFDoc *)document page_index:(int)page_index pdf_rect:(FSRectF *)pdf_rect{
    @try {
        FSPDFPage *currPage = [[self.pdfViewCtrl currentDoc] getPage:page_index];
        FSPDFPage *docPage = [document getPage:page_index];
        if ([currPage getDict].getObjNum  != [docPage getDict].getObjNum){
            return NO;
        }
    } @catch (NSException *exception) {
        return NO;
    }
    
    CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:pdf_rect pageIndex:page_index];
    [self.pdfViewCtrl refresh:rect pageIndex:page_index];
    
    FSPDFPage *page = [[self.pdfViewCtrl currentDoc] getPage:page_index];
    int annotCount = [page getAnnotCount];
    if (_annotCountRecord != annotCount) {
        dispatch_main_async_safe(^{
            [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
        });
        _annotCountRecord = annotCount;
   }
    return YES;
}

- (BOOL)setDocChangeMark:(FSPDFDoc*)document change_mark:(BOOL)change_mark {
    _extensionsManager.isDocModified = change_mark;
    return YES;
}

- (BOOL)getDocChangeMark:(FSPDFDoc*)document {
    return _extensionsManager.isDocModified;
}

- (BOOL)submitForm:(FSPDFDoc*)document formData:(NSData*)formData url:(NSString*)url file_format_type:(FSFileFormatType)file_format_type{
    return NO;
}

- (void)setCurrentPage:(FSPDFDoc *)pdfDoc destination:(FSDestination *)destination {}

- (NSString *)getAppInfo:(FSActionCallbackAppInfoType)type{
    NSString *info = @"Foxit PDF SDK For iOS";
    switch(type) {
        case FSActionCallbackAppInfoTypeFormsVersion:
            info = @"11";
            break;
        case FSActionCallbackAppInfoTypeViewerType:
            info = @"Exchange-Pro";
            break;
        case FSActionCallbackAppInfoTypeViewerVariation:
            info = @"11.007";
            break;
        case FSActionCallbackAppInfoTypeViewerVersion:
            info = @"5.6";
            break;
        case FSActionCallbackAppInfoTypeAppVersion:
            info = FS_APP_VERSION;
            break;
    }
    return info;
}
@end


static FSCallbackSubstitute *_annotIconProviderCallback;
static FSCallbackSubstitute *_actionHandler;
static FSCallbackSubstitute *_fillerAssistCallbackSubstitute;

@interface FSCallbackSubstitute ()
@property (nonatomic, strong) FSWeakPointerArray *weakTargets;
@end
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wobjc-protocol-method-implementation"
@implementation FSCallbackSubstitute

+ (instancetype)annotIconProviderCallbackSubstitute{
    if (!_annotIconProviderCallback) {
        _annotIconProviderCallback = [FSCallbackSubstitute alloc];
    }
    return _annotIconProviderCallback;
}

+ (instancetype)actionHandlerSubstitute{
    if (!_actionHandler) {
        _actionHandler = [FSCallbackSubstitute alloc];
    }
    return _actionHandler;
}

+ (instancetype)fillerAssistCallbackSubstitute{
    if (!_fillerAssistCallbackSubstitute) {
        _fillerAssistCallbackSubstitute = [FSCallbackSubstitute alloc];
    }
    return _fillerAssistCallbackSubstitute;
}

- (id)target{
    return _currentTarget;
}

- (NSArray *)targets{
    return [self.weakTargets allObjects];
}

- (FSWeakPointerArray *)weakTargets{
    if(!_weakTargets){
        _weakTargets = [FSWeakPointerArray array];
    }
    return _weakTargets;
}

//!!!!: - If enabled, some may have problems using only one callback. The best way to do this is to switch _currentTarget
//- (id)forwardingTargetForSelector:(SEL)selector {
//    return NULL;
//}

//- (void)forwardInvocation:(NSInvocation *)invocation {
//    if ([self.currentTarget respondsToSelector:[invocation selector]]){
//        [invocation invokeWithTarget:self.currentTarget];
        
//        [self.weakTargets enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
//            if (obj != self.currentTarget && [obj respondsToSelector:[invocation selector]]){
//                [invocation invokeWithTarget:obj];
//            }
//        }];
//        return;
//    }
//    [super forwardInvocation:invocation];
//}

- (void)setCurrentTarget:(id)currentTarget{
    if (!currentTarget && _currentTarget) {
        [self.weakTargets removeObject:_currentTarget];
    }
    _currentTarget = currentTarget;
    [self addTarget:currentTarget];
}

- (void)removeTarget:(id)target{
    if (![self.weakTargets containsObject:target]) return;
    [self.weakTargets removeObject:target];
}

- (void)addTarget:(id)target{
    if ([self.weakTargets containsObject:target]) return;
    [self.weakTargets addObject:target];
}

@end
#pragma clang diagnostic pop


@implementation UIExtensionsModifiedContext

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _listeners = [FSWeakPointerArray array];
    }
    return self;
}


- (void)registerModifiedEventListener:(id<IFSUIExtensionsModifiedEventListener>)listener{
    if (listener && ![self.listeners containsObject:listener]) {
        [self.listeners addObject:listener];
    }
}

- (void)unregisterModifiedEventListener:(id<IFSUIExtensionsModifiedEventListener>)listener{
    if ([self.listeners containsObject:listener]) {
        [self.listeners removeObject:listener];
    }
}

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onAnnotAdded:page annot:annot];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[page, annot]];
        }
    }
}

- (void)onAnnotDeleted:(FSPDFPage *)page annot:(FSAnnot *)annot{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onAnnotDeleted:page annot:annot];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[page, annot]];
        }
    }
}

- (void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onAnnotModified:page annot:annot];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[page, annot]];
        }
    }
}


- (void)onAnnotsAdded:(NSArray<FSAnnot*> *)annots{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onAnnotsAdded:annots];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[annots]];
        }
    }
}


- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onPageChanged:oldIndex currentIndex:currentIndex];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[@(oldIndex), @(currentIndex)]];
        }
    }
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onPagesRemoved:indexes];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[indexes]];
        }
    }
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onPagesMoved:indexes dstIndex:dstIndex];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[indexes, @(dstIndex)]];
        }
    }
}

- (void)onPagesRotated:(NSArray<NSNumber *> *)indexes rotation:(int)rotation{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onPagesRotated:indexes rotation:rotation];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[indexes, @(rotation)]];
        }
    }
}

- (void)onPagesInsertedAtRange:(NSRange)range{
    for (id<IFSUIExtensionsModifiedEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:_cmd]) {
            [(id<_IFSUIExtensionsModifiedEventListener>)listener onPagesInsertedAtRange:range];
        }else{
            [listener extensions:self.extensionsManager modifiedEventDealSel:_cmd args:@[[NSNumber valueWithRange:range]]];
        }
    }
}

- (void)modifiedEventDealSel:(SEL)sel args:(NSArray *)args {}

@end
