/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIExtensionsManager+Private.h"

#define MKA_POPUPWIDTH      200
#define MKA_POPUPHEIGHT     100

@implementation UIExtensionsManager (Private)
- (void)addPopupWithAnnot:(FSAnnot *)annot{
    if (self.annotPermissionDelegate && [self.annotPermissionDelegate respondsToSelector:@selector(canAddPopupAnnot:)]) {
        if (![self.annotPermissionDelegate canAddPopupAnnot:annot]) {
            [annot setContents:nil];
            return;
        }
    }
    if (!annot || [annot isEmpty] || ![annot isMarkup]) {
        return;
    }
    FSAnnotType annotType = [annot getType];
    if (annotType == FSAnnotSound || annotType == FSAnnotFreeText) {
        return;
    }
    if (annotType == FSAnnotNote) {
        FSNote *note = [[FSNote alloc] initWithAnnot:annot];
        FSMarkup *replyTo = [note getReplyTo];
        if (replyTo && ![replyTo isEmpty]) {
            return;
        }
    }
    FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
    if (markup.popup && ![markup.popup isEmpty]) {
        return;
    }
    
    FSPDFPage *page = annot.getPage;
    if (![page isParsed]) [page startParse:0 pause:nil is_reparse:NO];
    
    FSRectF *rectF = [[FSRectF alloc] initWithLeft1:[page getWidth] bottom1:annot.fsrect.top-MKA_POPUPHEIGHT right1:[page getWidth]+MKA_POPUPWIDTH top1:annot.fsrect.top];
    if ([page getRotation] == 1) {
        rectF = [[FSRectF alloc] initWithLeft1:annot.fsrect.left bottom1:[page getWidth]-MKA_POPUPHEIGHT right1:annot.fsrect.left+MKA_POPUPWIDTH top1:[page getWidth]];
    }
    else if ([page getRotation] == 2) {
        rectF = [[FSRectF alloc] initWithLeft1:0  bottom1:annot.fsrect.bottom-MKA_POPUPHEIGHT right1:MKA_POPUPWIDTH top1:annot.fsrect.bottom];
    }
    else if ([page getRotation] == 3) {
        rectF = [[FSRectF alloc] initWithLeft1:annot.fsrect.right bottom1:-MKA_POPUPHEIGHT right1:annot.fsrect.right+MKA_POPUPWIDTH  top1:0];
    }

    FSPopup *popup = [[FSPopup alloc] initWithAnnot:[annot.getPage addAnnot:FSAnnotPopup rect:rectF]];
    markup.popup = popup;
}

- (void)deletePopupWithAnnot:(FSAnnot *)annot{
    if (!annot || [annot isEmpty] || ![annot isMarkup]) {
        return;
    }
    FSAnnotType annotType = [annot getType];
    if (annotType == FSAnnotSound || annotType == FSAnnotFreeText) {
        return;
    }
    if (annotType == FSAnnotNote) {
        FSNote *note = [[FSNote alloc] initWithAnnot:annot];
        FSMarkup *replyTo = [note getReplyTo];
        if (replyTo && ![replyTo isEmpty]) {
            return;
        }
    }
    FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
    if (markup.popup && ![markup.popup isEmpty]) {
        [annot.getPage removeAnnot:markup.popup];
    }
}

- (FSReadToolSettings *)getPropertyToolSettingsForAnnot:(FSAnnot *)annot{
    if (annot && ![annot isEmpty]) {
        FSAnnotType type = [annot getType];
        NSUInteger settingsType = 0;
        switch (type) {
            case FSAnnotNote: {
                settingsType = FSReadToolSettingsTypeNote;
                break;
            }
            case FSAnnotLink: {
                settingsType = FSReadToolSettingsTypeLink;
                break;
            }
            case FSAnnotFreeText:
            {
                BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
                BOOL isTextbox = (annot.intent.length == 0);
                if (!isTextbox && !isCallout) {
                    settingsType = FSReadToolSettingsTypeTypewriter;
                }else if(isCallout){
                    settingsType = FSReadToolSettingsTypeCallout;
                }else{
                    settingsType = FSReadToolSettingsTypeTextbox;
                }
                break;
            }
            case FSAnnotLine: {
                settingsType = FSReadToolSettingsTypeLine;
                FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
                BOOL isArrow = [[markup getIntent] caseInsensitiveCompare:@"LineArrow"] == NSOrderedSame;
                if (isArrow) {
                    settingsType = FSReadToolSettingsTypeArrow;
                }
                BOOL isDistance = [[markup getIntent] caseInsensitiveCompare:@"LineDimension"] == NSOrderedSame;
                if (isDistance) {
                    settingsType = FSReadToolSettingsTypeMeasure;
                }
                break;
            }
            case FSAnnotSquare: {
                settingsType = FSReadToolSettingsTypeRectangle;
                break;
            }
            case FSAnnotCircle: {
                settingsType = FSReadToolSettingsTypeOval;
                break;
            }
            case FSAnnotPolygon: {
                FSBorderInfo *border = [[[FSPolygon alloc] initWithAnnot:annot] getBorderInfo];
                BOOL isCloud = ([border getStyle] == FSBorderInfoCloudy);
                if (isCloud) {
                    settingsType = FSReadToolSettingsTypeCloud;
                }else{
                    settingsType = FSReadToolSettingsTypePolygon;
                }
                break;
            }
            case FSAnnotPolyLine: {
                settingsType = FSReadToolSettingsTypePolyLine;
                break;
            }
            case FSAnnotHighlight: {
                if ([annot.intent isEqualToString:@"AreaHi"]) {
                    settingsType = FSReadToolSettingsTypeAreaHighlight;
                }else{
                    settingsType = FSReadToolSettingsTypeHighlight;
                }
                break;
            }
            case FSAnnotUnderline: {
                settingsType = FSReadToolSettingsTypeUnderline;
                break;
            }
            case FSAnnotSquiggly: {
                settingsType = FSReadToolSettingsTypeSquiggly;
                break;
            }
            case FSAnnotStrikeOut: {
                
                if ([Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]]) {
                    settingsType = FSReadToolSettingsTypeReplaceText;
                }else{
                    settingsType = FSReadToolSettingsTypeStrikeout;
                }
                break;
            }
            case FSAnnotStamp: {
                settingsType = FSReadToolSettingsTypeStamp;
                break;
            }
            case FSAnnotCaret: {
                if ([Utility isReplaceText:[[FSCaret alloc] initWithAnnot:annot]]) {
                    settingsType = FSReadToolSettingsTypeReplaceText;
                }else{
                    settingsType = FSReadToolSettingsTypeInsertText;
                }
                break;
            }
            case FSAnnotInk: {
                settingsType = FSReadToolSettingsTypePencil;
                break;
            }
            case FSAnnotFileAttachment: {
                settingsType = FSReadToolSettingsTypeAttachments;
                break;
            }
            case FSAnnotScreen: {
                FSScreen *screen = [[FSScreen alloc] initWithAnnot:annot];
                if (screen.screenActiontype == FSScreenActionTypeUnknow || screen.screenActiontype == FSScreenActionTypeImage) {
                    settingsType = FSReadToolSettingsTypeImageAnnotation;
                }
                break;
            }
            case FSAnnotRedact: {
                settingsType = FSReadToolSettingsTypeRedact;
                break;
            }
            default:
                break;
        }
        if (settingsType) {
            return [self getReadToolSettingsForType:settingsType];
        }
    }
    return self.currentToolSettings;
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wobjc-protocol-method-implementation"
#pragma mark- FSPropertyMainViewDelegate
- (void)propertyMainView:(FSPropertyMainView *)propertyMainView changedProperty:(FSPropertyType)property fromOldValue:(NSValue *)oldValue toValue:(NSValue *)newValue{
    FSAnnot *annot = self.currentAnnot;
    if (annot && ![annot isEmpty]) {
        //        BOOL addUndo;
        //        switch (annot.type) {
        //        case FSAnnotNote:
        //        case FSAnnotCircle:
        //        case FSAnnotSquare:
        //        case FSAnnotFreeText:
        //        // text markup
        //        case FSAnnotHighlight:
        //        case FSAnnotUnderline:
        //        case FSAnnotStrikeOut:
        //        case FSAnnotSquiggly:
        //
        //        case FSAnnotLine:
        //        case FSAnnotInk:
        //        case FSAnnotCaret:
        //        case FSAnnotStamp:
        //            addUndo = NO;
        //            break;
        //        default:
        //            addUndo = YES;
        //            break;
        //        }
        [self changeAnnot:annot property:property from:oldValue to:newValue];
        if (!self.canUpdateAnnotDefaultProperties) return;
    }
    
    FSReadToolSettings *settings =  [self getPropertyToolSettingsForAnnot:annot];

    switch (property) {
    case FSPropertyTypePropertyColor:
    case FSPropertyTypeTextColor:
    case FSPropertyTypeBackGroundColor:
    case FSPropertyTypeBorderColor:
        [settings setPropertyColor:[UIColor colorWithRGB:[(NSNumber *) newValue unsignedIntValue]]];
        break;
    case FSPropertyTypeOpacity:
        settings.opacity = [(NSNumber *) newValue floatValue];
        break;
    case FSPropertyTypeShapeNote:
        self.noteIcon = [(NSNumber *) newValue intValue];
        settings.noteIcon = self.noteIcon;
        break;
    case FSPropertyTypeShapeAttachment:
        self.attachmentIcon = [(NSNumber *) newValue intValue];
        settings.attachmentIcon = [(NSNumber *) newValue unsignedIntValue];
        break;
    case FSPropertyTypeLineWidth:
    case FSPropertyTypeBorderWidth:
        if (settings.settingsType == FSReadToolSettingsTypeEraser) {
            settings.eraserLineWidth = [(NSNumber *) newValue floatValue];
        }
        settings.thickness = [(NSNumber *) newValue floatValue];
        break;
    case FSPropertyTypeFontName:
        settings.textFace = (NSString *) [newValue nonretainedObjectValue];
        break;
    case FSPropertyTypeFontSizeFixed:
        settings.textSize = [(NSNumber *) newValue unsignedIntValue];
        break;
    case FSPropertyTypeDistanceUnit:
        settings.distanceUnit = (NSString  *)[newValue nonretainedObjectValue];
        break;
    case FSPropertyTypeRotation:
        settings.rotation = [(NSNumber *) newValue intValue];
        settings.screenAnnotRotation = [Utility rotationForValue:@(settings.rotation)];
        break;
    case FSPropertyTypePen:
        settings.pencilStyle = [(NSNumber *) newValue intValue];
        break;
    case FSPropertyTypeFillColor:
            settings.fillColor = (([(NSNumber *)newValue unsignedIntValue] == 0xFFFFFF00) ? [UIColor colorWithRGBA:[(NSNumber *) newValue unsignedIntValue]] : [UIColor colorWithRGB:[(NSNumber *) newValue unsignedIntValue]]);
        break;
    case FSPropertyTypeEarseStyle:
        settings.eraseStyle = [(NSNumber *) newValue intValue];
        break;
    default:
        break;
    }
    
    for (id<IAnnotPropertyListener> listener in self.annotPropertyListeners) {
        if ([listener respondsToSelector:@selector(onToolSettingsChanged:)]) {
            [listener onToolSettingsChanged:settings];
        }
    }
}
#pragma clang diagnostic pop
- (void)changeAnnot:(FSAnnot *)annot property:(FSPropertyType)property from:(NSValue *)oldValue to:(NSValue *)newValue // addUndo:(BOOL)addUndo
{
    int pageIndex = annot.pageIndex;
    CGRect oldRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
    FSRectF *oldAnnotRect = annot.fsrect;
    BOOL modified = YES;
    switch (property) {
    case FSPropertyTypePropertyColor:
        annot.color = [(NSNumber *) newValue unsignedIntValue];
        break;
    case FSPropertyTypeFillColor:
        annot.exfillColor = [(NSNumber *) newValue unsignedIntValue];
        break;
    case FSPropertyTypeBorderColor:
        annot.color = [(NSNumber *) newValue unsignedIntValue];
        break;
    case FSPropertyTypeTextColor:
        if (annot.type == FSAnnotRedact) {
            FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
            int color = [(NSNumber *) newValue unsignedIntValue];
            FSDefaultAppearance *ap = [redact getDefaultAppearance];
            ap.text_color = color;
            [redact setDefaultAppearance:ap];
        }
        break;
    case FSPropertyTypeBackGroundColor:
        if (annot.type == FSAnnotRedact) {
            FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
            redact.applyFillColor = [(NSNumber *) newValue unsignedIntValue];
        }
        break;
    case FSPropertyTypeOpacity:
        annot.opacity = [(NSNumber *) newValue floatValue];
        break;
    case FSPropertyTypeShapeNote:
        if (annot.type == FSAnnotNote) {
            ([[FSNote alloc] initWithAnnot:annot]).icon = [(NSNumber *) newValue unsignedIntValue];
        } else {
            modified = NO;
        }
        break;
    case FSPropertyTypeShapeAttachment:
        if (annot.type == FSAnnotFileAttachment) {
            ([[FSFileAttachment alloc] initWithAnnot:annot]).icon = [(NSNumber *) newValue unsignedIntValue];
        } else {
            modified = NO;
        }
        break;
    case FSPropertyTypeLineWidth:
    case FSPropertyTypeBorderWidth:
        annot.lineWidth = [(NSNumber *) newValue floatValue];
        break;
    case FSPropertyTypeFontSizeFixed:
    case FSPropertyTypeFontSizeAuto:{
            int newFontSize = [(NSNumber *) newValue unsignedIntValue];
            FSDefaultAppearance * (^updateDefaultAppearance)(FSDefaultAppearance *) = ^(FSDefaultAppearance *ap){
                ap.text_size = newFontSize;
                return ap;
            };
        if (annot.type == FSAnnotFreeText) {
            FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:annot];
            [freeText setDefaultAppearance:updateDefaultAppearance([freeText getDefaultAppearance])];
            [freeText allowTextOverflow:false];
            [freeText resetAppearanceStream];
            [freeText allowTextOverflow:true];
            CGRect overRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            if(overRect.size.width < oldRect.size.width || overRect.size.height < oldRect.size.height){
                annot.fsrect = oldAnnotRect;
            }

        }else if (annot.type == FSAnnotRedact){
            FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
            [redact setDefaultAppearance:updateDefaultAppearance([redact getDefaultAppearance])];
        }else if (annot.type == FSAnnotWidget){
            FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
            FSControl *control = [widget getControl];
            FSDefaultAppearance *ap = [control getDefaultAppearance];
            ap.flags |= FSDefaultAppearanceFlagFontSize;
            [control setDefaultAppearance:updateDefaultAppearance(ap)];
        }
        else {
            modified = NO;
        }
    }
        break;
    case FSPropertyTypeFontName:{
        NSString *newFontName = (NSString *) [newValue nonretainedObjectValue];
        FSDefaultAppearance * (^updateDefaultAppearance)(FSDefaultAppearance *) = ^(FSDefaultAppearance *ap){
            FSDefaultAppearance *updateAp = nil;
            FSFont *originFont = ap.font;
            int fontID = [Utility toStandardFontID:newFontName];
            FSFont *font = nil;
            if (fontID == -1) {
                font = [[FSFont alloc] initWithName:newFontName styles:0 charset:FSFontCharsetDefault weight:0];
            } else {
                font = [[FSFont alloc] initWithFont_id:fontID];
            }
            ap.font = font;
            if ([newFontName caseInsensitiveCompare:[originFont getName]] != NSOrderedSame) {
                updateAp = ap;
            }
            return updateAp;
        };
        if (annot.type == FSAnnotFreeText) {
            FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:annot];
            FSDefaultAppearance *ap = updateDefaultAppearance([freeText getDefaultAppearance]);
            if (ap) [freeText setDefaultAppearance:ap];
        }else if (annot.type == FSAnnotRedact){
            FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
            FSDefaultAppearance *ap = updateDefaultAppearance([redact getDefaultAppearance]);
            if (ap) [redact setDefaultAppearance:ap];
        }else if (annot.type == FSAnnotWidget){
            FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
            FSControl *control = [widget getControl];
            FSDefaultAppearance *ap = updateDefaultAppearance([control getDefaultAppearance]);
            if (ap) [control setDefaultAppearance:ap];
        }
        else {
            modified = NO;
        }
    }
        break;
    case FSPropertyTypeDistanceUnit:
        if (annot.type == FSAnnotLine) {
            FSLine *lineAnnot = [[FSLine alloc] initWithAnnot:annot];
            if ([[lineAnnot getIntent] isEqualToString:@"LineDimension"]) {
                NSString *unitName = (NSString *) [newValue nonretainedObjectValue];
                annot.subject = unitName;
            }
        }
        break;
    case FSPropertyTypeRotation:
        if (annot.type == FSAnnotScreen) {
            FSRotation rotation = [Utility rotationForValue:newValue];
            if (rotation != 0) {
                rotation = 4 - rotation;
            }
            [[[FSScreen alloc] initWithAnnot:annot] setRotation:rotation];
        } else {
            modified = NO;
        }
        break;
    case FSPropertyTypeTextRedact:{
        FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
        NSString *coverText = (NSString *) [newValue nonretainedObjectValue];
        redact.overlayText = coverText;
        }
        break;
    case FSPropertyTypeTextForm:{
        
        NSString *newName = (NSString *) [newValue nonretainedObjectValue];
        if (newName.length) {
            FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
            FSField *field = [widget getField];
            FSForm *form = [[FSForm alloc] initWithDocument:annot.getPage.getDocument];
            if (field.getControlCount == 1) {
                [form renameField:field new_field_name:newName];
            }else{
                 [form moveControl:widget.getControl field_name:newName];
            }
        }
    }
        break;
    case FSPropertyTypeFormOptions:{
        FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
        FSField *field = [widget getField];
        if ([newValue respondsToSelector:@selector(boolValue)]) {
            BOOL on = [(NSNumber *)newValue boolValue];
            if (field.getType == FSFieldTypeComboBox ) {
                if (on) {
                    field.flags |= FSFieldFlagComboEdit;
                }else{
                    field.flags &= ~FSFieldFlagComboEdit;
                }
            }
            else if (field.getType == FSFieldTypeListBox ) {
                if (on) {
                    field.flags |= FSFieldFlagChoiceMultiSelect;
                }else{
                    field.flags &= ~FSFieldFlagChoiceMultiSelect;
                }
            }
        }
        else if ([newValue respondsToSelector:@selector(nonretainedObjectValue)]) {
            FSChoiceOptionArray *options = [newValue nonretainedObjectValue];
            field.options = options;
        }
    }
    break;
    default:
        modified = NO;
        break;
    }
    if (modified) {
        FSDateTime *now = [Utility convert2FSDateTime:[NSDate date]];
        [annot setModifiedDateTime:now];

        FSRectF *oldAnnotRect = [[FSRectF alloc] initWithLeft1:annot.fsrect.left bottom1:annot.fsrect.bottom right1:annot.fsrect.right top1:annot.fsrect.top];
        [self.pdfViewCtrl lockRefresh];
        [annot resetAppearanceStream];
        [self.pdfViewCtrl unlockRefresh];
        
        if (annot.type == FSAnnotFreeText && (property == FSPropertyTypeFontSizeFixed || property == FSPropertyTypeFontName )) {
            CGRect refreshrect = [self.pdfViewCtrl convertPdfRectToPageViewRect:oldAnnotRect pageIndex:pageIndex];
            CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            refreshrect = CGRectUnion(refreshrect, newRect);
            refreshrect = CGRectInset(refreshrect, -30, -30);
            [self.pdfViewCtrl refresh:refreshrect pageIndex:pageIndex];
        }

        // keep annot in bounds
        {
            FSRectF *rect = annot.fsrect;
            FSPDFPage *page = [annot getPage];
            BOOL isOutOfBounds = NO;
            if (rect.bottom < 0) {
                rect.top -= rect.bottom;
                rect.bottom = 0;
                isOutOfBounds = YES;
            }
            float w = [page getWidth];
            if (rect.right > w) {
                rect.left -= rect.right - w;
                rect.right = w;
                isOutOfBounds = YES;
            }
            if (isOutOfBounds) {
                annot.fsrect = rect;
                [annot resetAppearanceStream];
            }
        }

        id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:annot];
        if ([annotHandler respondsToSelector:@selector(onAnnotChanged:property:from:to:)]) {
            [annotHandler onAnnotChanged:annot property:property from:oldValue to:newValue];
        }

        if ([self shouldDrawAnnot:annot]) {
            CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            rect = CGRectUnion(rect, oldRect);
            rect = CGRectInset(rect, -20, -20);
            [self.pdfViewCtrl refresh:rect pageIndex:pageIndex];
        }

        if (annot.type == FSAnnotCaret) {
            FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
            if ([markup isGrouped]) {
                FSMarkupArray *groupAnnots = [markup getGroupElements];
                for (int i = 0; i < [groupAnnots getSize]; i++) {
                    FSAnnot *groupAnnot = [groupAnnots getAt:i];
                    if (groupAnnot && ![groupAnnot isEmpty] && ![groupAnnot.NM isEqualToString:annot.NM]) {
                        [self changeAnnot:groupAnnot property:property from:oldValue to:newValue];
                    }
                }
            }
        }
    }
}


@end

@implementation DevicePosition

@end
