/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "DynamicXFAHandler.h"
#import "UIExtensionsConfig+private.h"
#import "MenuItem.h"
#import "MenuControl.h"
#import "PaintUtility.h"
#import "PageNavigationModule.h"
#import "FSPermissionUtilty.h"

typedef NS_ENUM(NSInteger, DXFAHandlerPreNextOption) {
    DXFAHandlerPreNextOptionPRE = 0,
    DXFAHandlerPreNextOptionNEXT = 1,
};

static NSString *FORM_CHAR_BACK = @"BACK";

typedef void (^FinishWidgetSelectedCallBack)(void);

@interface DynamicXFAHandler ()
@property (nonatomic, strong) UIFSDocProviderCallback *docProviderCallback;
@property (nonatomic, strong) PaintUtility *paintUtility;
@property (nonatomic, copy)  FinishWidgetSelectedCallBack finishSelectedCallback;
@end
@implementation DynamicXFAHandler {
    int _focusedWidgetIndex;
    int _focusedPageIndex;
    BOOL _lockKeyBoardPosition;
    BOOL _needRotate;
    dispatch_semaphore_t _lock;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.extensionsManager = extensionsManager;
        self.pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        [self.extensionsManager registerRotateChangedListener:self];
        [_pdfViewCtrl registerPageEventListener:(id)self];
        
        _lock = dispatch_semaphore_create(1);

        _keyboardHeight = 0;
        _handlerCurrentWidget = nil;
        _focusedWidgetIndex = -1;
        _focusedPageIndex = -1;
        _inputCharacterLocation = 0;
        _paintUtility = [PaintUtility new];

        
        if (!self.extensionsManager.config.defaultSettings.disableFormNavigationBar) {
            self.formNaviBar = [self buildFormNaviBar];
            UIView *superview = self.extensionsManager.containerView;
            [superview addSubview:self.formNaviBar];
            [self.formNaviBar mas_makeConstraints:^(MASConstraintMaker *make) {
                if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.left.right.bottom.mas_equalTo(self.pdfViewCtrl);
                }else{
                    make.centerX.mas_equalTo(superview);
                    make.bottom.mas_equalTo(superview.fs_mas_bottom).offset(-TOOL_BAR_HEIGHT_NORMAL);
                    make.width.offset([self.formNaviBar getWidthWithPosition:FSItemPositionLeft] + ITEM_MARGIN_NORMAL + [self.formNaviBar getWidthWithPosition:FSItemPositionRight]);
                }
            }];
            [self.formNaviBar setHidden:YES];
        }

        self.hiddenTextField = [[UITextField alloc] init];
        self.hiddenTextField.hidden = YES;
        self.hiddenTextField.delegate = self;
        self.hiddenTextField.text = @"";
        self.hiddenTextField.autocorrectionType = UITextAutocorrectionTypeNo;
        [self colorChange];
        self.lastText = @"";
        
        self.hiddenTextField.inputAccessoryView = [self inputAccessoryView];
        
//        [self.pdfViewCtrl addSubview:self.hiddenTextField];
        [self.pdfViewCtrl insertSubview:self.hiddenTextField belowSubview:_extensionsManager.topToolbar.contentView];
        _lockKeyBoardPosition = NO;
    }
    return self;
}

- (void)colorChange
{
    BOOL isFirst = [self.hiddenTextField isFirstResponder];
    if (isFirst) [self.hiddenTextField resignFirstResponder];
    [self.hiddenTextField setTintColor:[UIColor blackColor]];
    self.hiddenTextField.backgroundColor = [UIColor clearColor];
    if (isFirst) [self.hiddenTextField becomeFirstResponder];
}

- (void)dealloc{
    [self.extensionsManager unregisterRotateChangedListener:self];
}

- (NSString *)getName {
    return @"FSXFAWidget";
}

- (void)setDisplayCaretRect:(FSRectF *)displayCaretRect{
    _displayCaretRect = displayCaretRect;
    if (displayCaretRect) {
        @fs_weakify(self)
        _paintUtility.drawCaretBlock = ^(PaintUtility *paint) {
            @fs_strongify(self)
            if (self.handlerCurrentWidget && ![self.handlerCurrentWidget isEmpty] ) {
                [self.pdfViewCtrl refresh:self.handlerCurrentWidget.getXFAPage.getIndex needRender:NO];
            }
        };
    }else{
        _paintUtility.drawCaretBlock = nil;
    }
}

- (BOOL)canShowKeybord:(FSXFAWidget *)widget {
    FSXFAWidgetWidgetType widgetType = [widget getType];
    return widgetType == FSXFAWidgetWidgetTypeTextEdit || FSXFAWidgetWidgetTypeNumericEdit == widgetType || FSXFAWidgetWidgetTypePasswordEdit == widgetType || FSXFAWidgetWidgetTypeDateTimeEdit == widgetType || widgetType ==  FSXFAWidgetWidgetTypeBarcode;
}

- (BOOL)canFocusXFAWidget:(FSXFAWidget *)widget {
    FSXFAWidgetWidgetType widgetType = [widget getType];
    if (!(widgetType == FSXFAWidgetWidgetTypeArc || FSXFAWidgetWidgetTypeLine == widgetType || FSXFAWidgetWidgetTypePasswordEdit == widgetType || FSXFAWidgetWidgetTypeUnknown == widgetType) ) {
        return YES;
    }
    return NO;
}

- (void)gotoPreNextXFAWidget:(DXFAHandlerPreNextOption)option {
    int curPageIndex = (self.extensionsManager.currentWidget && ![self.extensionsManager.currentWidget isEmpty])?self.extensionsManager.currentWidget.getXFAPage.getIndex:[self.pdfViewCtrl getCurrentPage];
    FSXFADoc *xfaDoc =  [self.pdfViewCtrl getXFADoc];
    FSXFAPage *xfaPage = [xfaDoc getPage:curPageIndex];
    int pageCount = [xfaDoc getPageCount];
    int widgetCount = [xfaPage getWidgetCount];
    BOOL canShowKeybord = NO;
    
    int savedPos = _focusedWidgetIndex;
    int savedPageIndex = curPageIndex;
    
    int prePos = 0;
    int next = 0;
    
    if (option == DXFAHandlerPreNextOptionPRE) {
        prePos = _focusedWidgetIndex < 0 ? 0 : _focusedWidgetIndex - 1;
    }else{
        next = _focusedWidgetIndex + 1;
    }
    
    while (true) {
        if (option == DXFAHandlerPreNextOptionPRE) {
            if (prePos == -1) {
                if (curPageIndex == 0)
                    curPageIndex = pageCount - 1;
                else
                    curPageIndex--;
                
                xfaPage = [xfaDoc getPage:curPageIndex];
                widgetCount = [xfaPage getWidgetCount];
                prePos = widgetCount - 1;
                if (widgetCount == 0)
                    continue;
            }
            if (savedPageIndex == curPageIndex && savedPos == prePos)
                break;
            
            FSXFAWidget *widget = [xfaPage getWidget:prePos];
            
            if (![widget isEmpty] && [widget getPresence] == FSXFAWidgetPresenceVisible && [self canFocusXFAWidget:widget]) {
                canShowKeybord = [self canShowKeybord:widget];
                self.handlerCurrentWidget = widget;
                [self gotoWidget:widget canShowKeybord:canShowKeybord];
                _focusedWidgetIndex = prePos;
                break;
            }
            
            prePos--;
        }else{
            if (next == widgetCount) {
                if (curPageIndex == pageCount - 1)
                    curPageIndex = 0;
                else
                    curPageIndex++;
                
                xfaPage = [xfaDoc getPage:curPageIndex];
                widgetCount = [xfaPage getWidgetCount];
                next = 0;
                if (widgetCount == 0)
                    continue;
            }
            if (savedPageIndex == curPageIndex && next == savedPos)
                break;
            
            FSXFAWidget *widget = [xfaPage getWidget:next];
            if (![widget isEmpty] && [widget getPresence] == FSXFAWidgetPresenceVisible && [self canFocusXFAWidget:widget]) {
                        canShowKeybord = [self canShowKeybord:widget];
                        self.handlerCurrentWidget = widget;
                        [self gotoWidget:widget canShowKeybord:canShowKeybord];
                        _focusedWidgetIndex = next;
                        break;
                    }
            next++;
        }
    }
    
    if (canShowKeybord || (_MAC_CATALYST_ && self.editFormControlNeedTextInput)) {
        [self beginInput];
    } else{
        [self endTextInput];
    }
}

- (UIView *)inputAccessoryView{
    if (self.extensionsManager.config.defaultSettings.disableFormNavigationBar) return nil;
    UIView *contentView = [UIView new];
    contentView.fs_size = CGSizeMake(self.pdfViewCtrl.fs_width, TAB_BAR_HEIGHT_NORMAL);
    self.textFormNaviBar = [self buildFormNaviBar];
    contentView.backgroundColor = self.textFormNaviBar.backgroundColor;
    [contentView addSubview:self.textFormNaviBar];
    [self.textFormNaviBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.fs_equalToSuperview().priorityHigh();
        if (DEVICE_iPHONE) {
            make.left.right.fs_equalToSuperview();
        }else{
            make.centerX.fs_equalToSuperview();
            make.width.offset(IPAD_PREFERENCE_BAR_WIDTH);
            
            CGFloat spacing = (IPAD_PREFERENCE_BAR_WIDTH -  [self.textFormNaviBar getWidthWithPosition:FSItemPositionLeft] - ITEM_MARGIN_NORMAL - [self.textFormNaviBar getWidthWithPosition:FSItemPositionRight])/5;
            
            for (FSToolbarItem *item in self.textFormNaviBar.leftItems.items) {
                item.margin += spacing;
            }
            for (FSToolbarItem *item in self.textFormNaviBar.rightItems.items) {
                item.margin += spacing;
            }
            
            [self.textFormNaviBar readdItemsForPosition:FSItemPositionLeft];
            [self.textFormNaviBar readdItemsForPosition:FSItemPositionRight];
        }
    }];
    return contentView;
}

- (FSToolbar *)buildFormNaviBar {
    @weakify(self);
    FSToolbar *formNaviBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    if (DEVICE_iPAD && ![_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        [formNaviBar fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
    }
    
    FSItem *prevItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_previous") background:nil];
    prevItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        [self gotoPreNextXFAWidget:DXFAHandlerPreNextOptionPRE];
    };
    [formNaviBar addItem:prevItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionLeft];

    FSItem *nextItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_next") background:nil];
    nextItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        [self gotoPreNextXFAWidget:DXFAHandlerPreNextOptionNEXT];
    };
    [formNaviBar addItem:nextItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionLeft];
    
    FSItem *doneItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kDone")];
    [BindThemeKeySel(doneItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem  *_Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    doneItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        [self closeWidget];
    };
    [formNaviBar addItem:doneItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionRight];

    FSItem *resetItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kReset")];
    [BindThemeKeySel(resetItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem  *_Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    resetItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        FSXFADoc *xfadoc = [self.pdfViewCtrl getXFADoc];
        
        @try {
            [xfadoc killFocus];
        } @catch (NSException *exception) {
        }
        
        int curPageIndex = self->_focusedPageIndex;
        FSXFAPage *xfapage = [xfadoc getPage:curPageIndex];
        FSXFAWidget *widget = [xfapage getWidget:self->_focusedWidgetIndex];
        
        if (widget !=nil && ![widget isEmpty]) {
            [widget resetData];
        }
        [xfadoc setFocus:widget];
        
        CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[widget getRect] pageIndex:widget.getXFAPage.getIndex];
        newRect = CGRectInset(newRect, -30, -30);
        [self.pdfViewCtrl refresh:newRect pageIndex:widget.getXFAPage.getIndex needRender:YES];

        self.extensionsManager.isDocModified = YES;
    };
    [formNaviBar addItem:resetItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionRight];
    
    return formNaviBar;
}

-(void)closeWidget {
    self.handlerCurrentWidget = nil;
    [self endTextInput];
    self->_focusedWidgetIndex = -1;
    self->_focusedPageIndex = -1;
    [self.extensionsManager setCurrentWidget:nil];
    [self setFocus:nil isHidden:YES];
}

-(void)gotoWidget:(FSXFAWidget *)widget canShowKeybord:(BOOL)canShowKeybord{
    int curPageIndex = [[widget getXFAPage] getIndex];
    FSRectF *rect = [widget getRect];
    
    [self.extensionsManager setCurrentWidget:widget];
    [self setFocus:widget isHidden:NO];
    
    if (canShowKeybord) {
        [self gotoFormField];
    } else {
        CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:curPageIndex];
        CGPoint pvPt = CGPointZero;
        pvPt.x = pvRect.origin.x - (self.pdfViewCtrl.frame.size.width - pvRect.size.width) / 2;
        pvPt.y = pvRect.origin.y - (self.pdfViewCtrl.frame.size.height - pvRect.size.height) / 2;
        FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pvPt pageIndex:curPageIndex];
        [self.pdfViewCtrl gotoPage:curPageIndex withDocPoint:pdfPt animated:YES];
    }
}

- (BOOL)isHitWidget:(FSXFAWidget *)widget point:(FSPointF *)point {
    if ([widget getPresence] != FSXFAWidgetPresenceVisible) {
        return NO;
    }
    FSXFAWidgetHitTestArea hitAreaType = [widget onHitTest:point];
    if (hitAreaType != FSXFAWidgetHitTestAreaUnknown) {
        return YES;
    }
    return NO;
}

- (void)clickFormField:(FSXFAWidget *)xfawidget point:(FSPointF *)point isHidden:(BOOL)hidden {
    if (xfawidget != nil && ![xfawidget isEmpty]) {
        @try {
            [xfawidget onLButtonDown:point flags:0];
            
        } @catch (NSException *exception) {
        }
        @try {
            [xfawidget onMouseMove:point flags:0];
        } @catch (NSException *exception) {
        }
#if _MAC_CATALYST_
        @try {
            [xfawidget onLButtonUp:point flags:0];
        } @catch (NSException *exception) {
        }
#endif
    }
    
    if (!hidden) {
        FSXFAWidgetWidgetType fieldType = [xfawidget getType];
        if (FSXFAWidgetWidgetTypePushButton == fieldType) {
            hidden = YES;
        }
    }
    [self.formNaviBar setHidden:hidden];
}

- (void)setFocus:(FSXFAWidget *)widget isHidden:(BOOL)hidden {
    FSXFADoc *xfadoc = [_pdfViewCtrl getXFADoc];
    if (widget && ![widget isEmpty]) {
        @try {
            [xfadoc setFocus:widget];
        } @catch (NSException *exception) {
        }
    } else {
        @try {
            [xfadoc killFocus];
        } @catch (NSException *exception) {
        }
    }
    
    [self.formNaviBar setHidden:hidden];
    [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] setIsFormNavBarShowState:!hidden];
}

- (BOOL)compareTwoWidgetIsEqualTo:(FSXFAWidget *)widget compareWidget:(FSXFAWidget *)cwidget {
    if (cwidget == widget) {
        return YES;
    }
    if (!widget || [widget isEmpty] || !cwidget || [cwidget isEmpty]) {
        return NO;
    }
    if ([[widget getXFAPage] getIndex] != [[cwidget getXFAPage] getIndex]) {
        return NO;
    }
    return [widget getIndex] == [cwidget getIndex];
}

#pragma mark IAnnotHandler event
- (void)onXFAWidgetSelected:(FSXFAWidget *)widget {
    _isOver = NO;
    
    FSXFAWidgetWidgetType widgetType = [widget getType];
    if (widgetType != FSXFAWidgetWidgetTypeTextEdit)
        self.editFormControlNeedSetCursor = YES;
    if ([self canShowKeybord:widget])
        self.editFormControlNeedTextInput = YES;
    
    if (self.finishSelectedCallback) self.finishSelectedCallback();
}

- (void)onXFAWidgetDeselected:(FSXFAWidget *)widget {
    int pageIndex = widget.getXFAPage.getIndex;
    [self endForm:pageIndex];
    [_extensionsManager.thumbnailModule removeThumbnailCacheOfPageAtIndex:pageIndex];
    
    if ([self isXFAWidgetSignature:widget]) {
        [self hideAnnotMenu];
    }
}

// PageView Gesture+Touch
- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer widget:(FSXFAWidget *)widget {
    BOOL canFillForm = [FSPermissionUtilty canFillForm:self.extensionsManager];
    if (!canFillForm) {
        return NO;
    }

    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [gestureRecognizer locationInView:pageView];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    
    if (pageIndex == widget.getXFAPage.getIndex && [self isHitWidget:widget point:pdfPoint]) {
        return YES;
    }
    return NO;
}

-(void)gotoWidgetOnError:(FSXFAWidget *)widget{
    self.hiddenTextField.keyboardType = [widget getType] == FSXFAWidgetWidgetTypeNumericEdit ? UIKeyboardTypeNumberPad : UIKeyboardTypeDefault;
    
    _focusedWidgetIndex = [widget getIndex];
    _focusedPageIndex = [[widget getXFAPage] getIndex];
    _hasFormChanged = NO;
    _editFormControlNeedSetCursor = NO;
    
    BOOL isHasWidgetClick = self.extensionsManager.currentWidget != nil ? YES : NO;
    
    if (self.editFormControlNeedTextInput) {
        _lockKeyBoardPosition = NO;
        [self beginInput];
        
        _hasFormChanged = YES;
        
        if (isHasWidgetClick) {
            BOOL canShowKeybord = [self canShowKeybord:widget];
            if (canShowKeybord) {
                [self gotoFormField];
            }
        }
    } else {
        [self.formNaviBar setHidden:NO];
        [self endTextInput];
    }
    
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event widget:(FSXFAWidget *)widget {
    BOOL canFillForm = [FSPermissionUtilty canFillForm:self.extensionsManager];
    if (!canFillForm) {
        int wpageIndex = [[widget getXFAPage] getIndex];
        UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [[touches anyObject] locationInView:pageView];
        FSPointF *pdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:wpageIndex];
        
        if ([self compareTwoWidgetIsEqualTo:widget compareWidget:_extensionsManager.currentWidget]) {
            if (!(pageIndex == wpageIndex && [self isHitWidget:widget point:pdfPoint])) {
                [_extensionsManager setCurrentWidget:nil];
            }
        }
        return NO;
    }
//    BOOL needReturn = _isOver;
//    if (needReturn) {
//        return YES;
//    }
    _isInputError = NO;
    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [[touches anyObject] locationInView:pageView];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    
    if (![self isHitWidget:self.extensionsManager.currentWidget point:pdfPoint]) {
        _handlerCurrentWidget = [self.extensionsManager getXFAWidgetAtPoint:point pageIndex:pageIndex];
    }

    if (_handlerCurrentWidget == nil || [_handlerCurrentWidget isEmpty] || [_handlerCurrentWidget getPresence] != FSXFAWidgetPresenceVisible) {
        [self clickFormField:nil point:pdfPoint isHidden:YES];
        [self.extensionsManager setCurrentWidget:nil];
        [self setFocus:nil isHidden:YES];
        return NO;
    }

    self.hiddenTextField.keyboardType = [widget getType] == FSXFAWidgetWidgetTypeNumericEdit ? UIKeyboardTypeNumberPad : UIKeyboardTypeDefault;
    
    _focusedWidgetIndex = [_handlerCurrentWidget getIndex];
    _focusedPageIndex = pageIndex;
    [self clickFormField:_handlerCurrentWidget point:pdfPoint isHidden:NO];
    
    if (_isInputError) {
        [self clickFormField:nil point:pdfPoint isHidden:YES];
        return NO;
    }
    
    _hasFormChanged = NO;
    _editFormControlNeedSetCursor = NO;
    
    @fs_weakify(self)
    [self setFinishSelectedCallback:^{
        @fs_strongify(self)
        BOOL isHasWidgetClick = self.extensionsManager.currentWidget != nil ? YES : NO;
        if (self.editFormControlNeedTextInput) {
            self->_lockKeyBoardPosition = NO;
            [self beginInput];
            
            self->_hasFormChanged = YES;
            
            if (isHasWidgetClick) {
                BOOL canShowKeybord = [self canShowKeybord:widget];
                if (canShowKeybord) {
                    [self gotoFormField];
                }
            }
        } else {
            if ([self->_handlerCurrentWidget getType] != FSXFAWidgetWidgetTypePushButton ) {
                [self.formNaviBar setHidden:YES];
                [self endTextInput];
            }
            
            if ([self isXFAWidgetSignature:self->_handlerCurrentWidget] ) {
                @try {
                    FSSignature *signature = [self->_handlerCurrentWidget getSignature];
                    if ([signature isEmpty]) {
                        [self setFinishSelectedCallback:nil];
                        return;
                    }
                    
                    CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[self->_handlerCurrentWidget getRect] pageIndex:pageIndex];
                    CGRect resultRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:newRect pageIndex:pageIndex];

                    DynamicXFASignatureAnnotHandler *signatureAnnotHandler = self.extensionsManager.dynamicXFASignatureAnnotHandler;
                    signatureAnnotHandler.isDynamicXFAFile = YES;
                    signatureAnnotHandler.menuRect = resultRect;
                    signatureAnnotHandler.editXFAWidget = self->_handlerCurrentWidget;
                    [signatureAnnotHandler dynamicXFASignatureSelected:signature shouldShowMenu:YES];
                    
                    
                } @catch (NSException *exception) {
                }
            }
        }
        [self setFinishSelectedCallback:nil];
    }];
    [self.extensionsManager setCurrentWidget:_handlerCurrentWidget];
    return YES;
}

-(void)hideAnnotMenu {
    MenuControl *annotMenu = _extensionsManager.menuControl;
    if (annotMenu.isMenuVisible) {
        [annotMenu setMenuVisible:NO animated:YES];
    }
}

-(BOOL)isXFAWidgetSignature:(FSXFAWidget *)xfawidget {
    if (xfawidget == nil || [xfawidget isEmpty]) return NO;
    @try {
        if ([xfawidget getType] == FSXFAWidgetWidgetTypeSignature) return YES;
    } @catch (NSException *exception) {
    }
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event widget:(FSXFAWidget *_Nullable)widget{
    if (![self compareTwoWidgetIsEqualTo:self.extensionsManager.currentWidget compareWidget:widget]) {
        return NO;
    }
    
    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [[touches anyObject] locationInView:pageView];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    @try {
        [widget onMouseMove:pdfPoint flags:0];
    } @catch (NSException *exception) {
    }
    
    return YES;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context widget:(FSXFAWidget *_Nullable)widget{
    if (pageIndex == widget.getXFAPage.getIndex && [self compareTwoWidgetIsEqualTo:widget compareWidget:_handlerCurrentWidget] && [widget getPresence] == FSXFAWidgetPresenceVisible) {
        CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[widget getRect] pageIndex:pageIndex];
        if (rect.size.width > 0 && rect.size.height > 0 && ![self isXFAWidgetSignature:_handlerCurrentWidget]) {
            UIEdgeInsets paddingEdge = UIEdgeInsetsMake(-5, -5, -5, -5);
            if ([widget getType] == FSXFAWidgetWidgetTypeDateTimeEdit) {
                paddingEdge = UIEdgeInsetsMake(-5, -5, -5, -10);
            }
            rect = UIEdgeInsetsInsetRect(rect, paddingEdge);
            CGContextSetLineWidth(context, 2.0);
            CGFloat dashArray[] = {3, 3, 3, 3};
            CGContextSetLineDash(context, 3, dashArray, 4);
            UIColor *dashLineColor = [UIColor colorWithRGB:0x708090];
            CGContextSetStrokeColorWithColor(context, [dashLineColor CGColor]);
            CGContextStrokeRect(context, rect);
        }


        if (self.displayCaretRect) {
            CGRect caretRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.displayCaretRect pageIndex:pageIndex];
            [_paintUtility drawCaretWithRect:caretRect inContext:context];
        }
    }
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event widget:(FSXFAWidget *_Nullable)widget {
    if (![self compareTwoWidgetIsEqualTo:self.extensionsManager.currentWidget compareWidget:widget]) {
        return NO;
    }
    
    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [[touches anyObject] locationInView:pageView];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    
    @try {
        [widget onLButtonUp:pdfPoint flags:0];
    } @catch (NSException *exception) {
    }
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer widget:(FSXFAWidget *_Nullable)widget{
    return YES;

}

- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *)annot {
}

- (BOOL)addAnnot:(nonnull FSAnnot *)annot {
    return NO;
}

- (BOOL)addAnnot:(nonnull FSAnnot *)annot addUndo:(BOOL)addUndo {
    return NO;
}

- (FSAnnotType)getType {
    return 0;
}

- (BOOL)isHitAnnot:(nonnull FSAnnot *)annot point:(nonnull FSPointF *)point {
    return YES;
}

- (BOOL)modifyAnnot:(nonnull FSAnnot *)annot {
    return NO;
}

- (BOOL)modifyAnnot:(nonnull FSAnnot *)annot addUndo:(BOOL)addUndo {
    return NO;
}
- (void)onAnnotDeselected:(nonnull FSAnnot *)annot {
}

- (void)onAnnotSelected:(nonnull FSAnnot *)annot {
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(nonnull UIGestureRecognizer *)gestureRecognizer annot:(FSAnnot * _Nullable)annot {
    return YES;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(nonnull NSSet *)touches withEvent:(nonnull UIEvent *)event annot:(nonnull FSAnnot *)annot {
    return YES;
}

- (BOOL)removeAnnot:(nonnull FSAnnot *)annot {
    return NO;
}

- (BOOL)removeAnnot:(nonnull FSAnnot *)annot addUndo:(BOOL)addUndo {
    return NO;
}

- (void)endOp:(BOOL)needDelay {
    if (!_isOver) {
        _isOver = YES;
    }
    [self.pdfViewCtrl refresh:CGRectZero pageIndex:[self.pdfViewCtrl getCurrentPage]];
}

- (void)beginInput{
    #if !_MAC_CATALYST_
    [self.formNaviBar setHidden:YES];
    #endif
    [self.hiddenTextField becomeFirstResponder];
    self.hiddenTextField.text = [_handlerCurrentWidget isEmpty] ? @"" : [_handlerCurrentWidget getValue];
    
}

- (void)endTextInput {
    self.editFormControlNeedTextInput = NO;
    if (self.hiddenTextField) {
        [self.hiddenTextField resignFirstResponder];
        _isOver = NO;
        self.lastText = @"";
        self.hiddenTextField.text = @"";
        
    }
}

- (void)endForm:(int)pageIndex {
    if (self.extensionsManager.currentWidget) {
        [self endTextInput];
    }
}

- (void)textViewInputText:(NSString *)text{
    if(_inputCharacterLocation == 0xFFFF) return;
    _inputCharacterLocation = 0xFFFF;
    for(NSUInteger i=0; i<text.length; i++)
    {
        unichar cha = [text characterAtIndex:i];
        //Control character as space.
        if(cha == 0x2006) cha = FSFillerVkeySpace;
        [self formInputText:[[NSString alloc] initWithCharacters:&cha length:1]];
    }
    if (!self.editFormControlNeedTextInput) {
        [self endOp:YES];
    }
}

#pragma mark - UITextFieldDelegate
-(BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    if (!self.editFormControlNeedTextInput) return YES;
    if(range.length > 0 )
    {
        for(int i=0; i<range.length; i++)
            [self formInputText:FORM_CHAR_BACK];
    }
    else
    {
        if([string isEqualToString:@""])
            [self formInputText:FORM_CHAR_BACK];
    }
    
    _inputCharacterLocation = range.location;
    [self.pdfViewCtrl lockRefresh];
    [self textViewInputText:string];
    [self.pdfViewCtrl unlockRefresh];
    return YES;
}

-(void)textFieldDidEndEditing:(UITextField *)textField{
    self.editFormControlNeedTextInput = NO;
    if (self.hiddenTextField) {
        _isOver = NO;
        self.lastText = @"";
        self.hiddenTextField.text = @"";
    }
}

#pragma mark -
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"

- (void)gotoFormField {
    
     dispatch_semaphore_wait(_lock, DISPATCH_TIME_FOREVER);
    FSXFAWidget *widget = self.extensionsManager.currentWidget;
    if (widget && ![widget isEmpty]) {

        CGRect pvWidgetRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:widget.fsrect pageIndex:widget.pageIndex];
        CGRect dvWidgetRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvWidgetRect pageIndex:widget.pageIndex];
        
        CGFloat visibleHeight = CGRectGetHeight(self.pdfViewCtrl.bounds) - [self.extensionsManager topHeight];
        if (DEVICE_iPHONE) {
            CGFloat maxHeight = visibleHeight - _keyboardHeight - 88;
            if (CGRectGetHeight(dvWidgetRect) > maxHeight) {
                dvWidgetRect.size.height = maxHeight;
            }
        }
        
        PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
        if (_needRotate) {
            CGPoint annotPVOrigin = [self.pdfViewCtrl convertDisplayViewPtToPageViewPt:pvWidgetRect.origin pageIndex:widget.pageIndex];
            FSPointF *annotPVOriginf = [self.pdfViewCtrl convertPageViewPtToPdfPt:annotPVOrigin pageIndex:widget.pageIndex];
            [self.pdfViewCtrl gotoPage:widget.pageIndex withDocPoint:annotPVOriginf animated:YES];
            _needRotate = NO;
            pvWidgetRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:widget.fsrect pageIndex:widget.pageIndex];
            dvWidgetRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvWidgetRect pageIndex:widget.pageIndex];
            if ([self.pdfViewCtrl isContinuous]) {
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [self gotoFormField];
                });
                dispatch_semaphore_signal(_lock);
                return;
            }
        }
        
        CGFloat dvAnnotMaxYExtra = CGRectGetHeight(self.pdfViewCtrl.bounds) - CGRectGetMaxY(dvWidgetRect);
        if (dvAnnotMaxYExtra < _keyboardHeight) {
            float bottomOffset = ({
                float dvOffsetY = _keyboardHeight - fabs(dvAnnotMaxYExtra) + 20;
                
                if (dvOffsetY + CGRectGetHeight(pvWidgetRect) > CGRectGetHeight(self.pdfViewCtrl.bounds)) {
                    dvOffsetY =  CGRectGetHeight(self.pdfViewCtrl.bounds) - _keyboardHeight - 88;
                }
                dvOffsetY;
            });
            if ([self.pdfViewCtrl isContinuous]) {
                int pageIndex = widget.pageIndex;
                if (pageIndex == [self.pdfViewCtrl getPageCount] - 1) {
                    [self.pdfViewCtrl setBottomOffset:bottomOffset];
                }else if(pageIndex == [self.pdfViewCtrl getPageCount] - 2 && (pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT)){
                    if (pageLayoutMode == PDF_LAYOUT_MODE_TWO && [self.pdfViewCtrl getPageCount] % 2 == 0) {
                        [self.pdfViewCtrl setBottomOffset:bottomOffset];
                    }
                    if (pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT && [self.pdfViewCtrl getPageCount] % 2 == 1) {
                        [self.pdfViewCtrl setBottomOffset:bottomOffset];
                    }
                }else{
                    CGRect pvRectToPdfDisplayRect = [[self.pdfViewCtrl getPageView:pageIndex] convertRect:pvWidgetRect toView:[self.pdfViewCtrl getDisplayView]];
                    
                    CGRect visibleRect = self.pdfViewCtrl.bounds;
                    float topHeight = [self.extensionsManager topHeight];
                    visibleRect.origin.y += topHeight;
                    visibleRect.size.height -= topHeight;
                    
                    float y = CGRectGetMinY(pvWidgetRect) - CGRectGetHeight(visibleRect) + CGRectGetHeight(pvWidgetRect) + _keyboardHeight + 20;
                    if (y < 0) {
                        [self.pdfViewCtrl setBottomOffset:bottomOffset];
                    }else{
                        CGPoint pt = CGPointMake(pvWidgetRect.origin.x - pvRectToPdfDisplayRect.origin.x, y);
                        FSPointF *fspt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pt pageIndex:pageIndex];
                        [self.pdfViewCtrl gotoPage:widget.pageIndex withDocPoint:fspt animated:NO];
                    }
                }
            }else{
                [self.pdfViewCtrl setBottomOffset:bottomOffset];
            }
        }
//        else {
//            CGPoint dvPt = CGPointZero;
//            dvPt.x = pvWidgetRect.origin.x - (self.pdfViewCtrl.frame.size.width - pvWidgetRect.size.width) / 2;
//            dvPt.y = pvWidgetRect.origin.y - (self.pdfViewCtrl.frame.size.height - pvWidgetRect.size.height) / 2;
//            CGPoint pvPt = [self.pdfViewCtrl convertDisplayViewPtToPageViewPt:dvPt pageIndex:widget.pageIndex];
//            FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pvPt pageIndex:widget.pageIndex];
//            if (![self.pdfViewCtrl isContinuous]) {
//                [self.pdfViewCtrl gotoPage:widget.pageIndex withDocPoint:pdfPt animated:YES];
//            }else {
//                FSRectF *rect = widget.getRect;
//                int curPageIndex = widget.pageIndex;
//                CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:curPageIndex];
//
//                float orignY = pvRect.origin.y;
//                BOOL isTwoPagesMode = (pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_LEFT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE);
//                if (!isTwoPagesMode) {
//                    orignY = pvRect.origin.y - CGRectGetHeight(self.pdfViewCtrl.frame)/2 ;
//                }
//                CGPoint newOrign = CGPointMake(pvRect.origin.x - CGRectGetWidth(self.pdfViewCtrl.frame)/2, orignY);
//                FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:newOrign pageIndex:curPageIndex];
//                [self.pdfViewCtrl gotoPage:curPageIndex withDocPoint:pdfPt animated:YES];
//            }
//        }
    }
    dispatch_semaphore_signal(_lock);
}

- (void)keyboardWasShown:(NSNotification *)aNotification {
    if (_keyboardShown)
        return;
    _keyboardShown = YES;
    NSDictionary *info = [aNotification userInfo];
    // Get the frame of the keyboard.
    NSValue *frame = nil;
    frame = [info objectForKey:UIKeyboardBoundsUserInfoKey];
    CGRect keyboardFrame = [frame CGRectValue];
    _keyboardHeight = keyboardFrame.size.height;
    [self gotoFormField];
}

- (void)keyboardWasHidden:(NSNotification *)aNotification {
    _keyboardShown = NO;
    [self endOp:YES];
    FSXFAWidget *xfaWidget = self.extensionsManager.currentWidget;
    if (xfaWidget && ![xfaWidget isEmpty] && !_lockKeyBoardPosition) {
        PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
        int pageIndex = xfaWidget.getXFAPage.getIndex;
        if (pageLayoutMode == PDF_LAYOUT_MODE_SINGLE || pageIndex == [self.pdfViewCtrl.currentDoc getPageCount] - 1 || pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_LEFT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE) {
            [self.pdfViewCtrl setBottomOffset:0];
        }
        else if ([self.pdfViewCtrl isContinuous]) {
            [self.pdfViewCtrl setBottomOffset:0];
        }
    }
}

- (void)formInputText:(NSString *)character {
    @synchronized(self) {
        unsigned int code = 0;

        if (!character || character.length == 0) {
            return;
        } else if ([character isEqualToString:@"\n"]) {
            code = 0x0D; //enter key
        } else if ([character isEqualToString:FORM_CHAR_BACK]) {
            code = 0x08; //backspace key
        } else {
            NSData *myD = [character dataUsingEncoding:NSUTF16LittleEndianStringEncoding];
            Byte *bytes = (Byte *) [myD bytes];
            //byte to hex
            NSString *hexStr = @"";
            for (int i = 0; i < [myD length]; i++) {
                NSString *newHexStr = [NSString stringWithFormat:@"%x", bytes[i] & 0xff]; //hex
                if ([newHexStr length] == 1)
                    hexStr = [NSString stringWithFormat:@"0%@%@", newHexStr, hexStr];
                else
                    hexStr = [NSString stringWithFormat:@"%@%@", newHexStr, hexStr];
            }
            code = (unsigned int) strtoul([[hexStr substringWithRange:NSMakeRange(0, hexStr.length)] UTF8String], 0, 16);
        }

        [_handlerCurrentWidget onKeyDown:code flags:0];
        [_handlerCurrentWidget onChar:code flags:0];
        [_handlerCurrentWidget onKeyUp:code flags:0];
        
        _hasFormChanged = YES;
    }
}

#pragma mark IDocEventListener
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    _handlerCurrentWidget = nil;
    
    if ([_pdfViewCtrl isDynamicXFA]) {
        FSXFADoc *xfadoc = [_pdfViewCtrl getXFADoc];
        UIFSDocProviderCallback *xfadocProviderCB = [[UIFSDocProviderCallback alloc] initWithExtensionsManager:_extensionsManager];
        [xfadoc setDocProviderCallback:xfadocProviderCB];
        xfadocProviderCB.dynamicXFAHandler = self;
        _docProviderCallback = xfadocProviderCB;
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasShown:)
                                                     name:UIKeyboardDidShowNotification
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasHidden:)
                                                     name:UIKeyboardWillHideNotification
                                                   object:nil];
    }
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    [self.formNaviBar setHidden:YES];
    if (document)
        _handlerCurrentWidget = nil;
    
    if ([_pdfViewCtrl isDynamicXFA]) {
        _docProviderCallback = nil;
        [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardDidShowNotification object:nil];
        [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
    }

}

#pragma mark IRotationEventListener

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration{
    if (_keyboardShown) {
        [self endTextInput];
        _needRotate = YES;
    }
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    self.hiddenTextField.inputAccessoryView = [self inputAccessoryView];
    if (_needRotate) {                                                                                                                                                                                                        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self beginInput];
        });
    }
}

@end
