/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSThumbnailChildViewController.h"
#import "FSThumbnailCell.h"
#import "FSReorderableCollectionView.h"
#import "FSPermissionUtilty.h"

@interface FSThumbnailChildViewController ()<FSReorderableCollectionViewDataSource, FSReorderableCollectionViewReorderDelegate, FSThumbnailCellDelegate>{
    FSReorderableCollectionView *_collectionView;
}

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSPDFDoc *document;

@property (nonatomic, strong, readwrite) FSViewControllerManager *services;
@property (nonatomic) CGSize cellSize;

// thumbnail loading
@property (nonatomic, strong) NSOperationQueue *operationQueue;
@property (nonatomic, strong) NSMutableDictionary<NSIndexPath *, NSBlockOperation *> *operations;

// misc
@property (nonatomic) NSInteger insertIndex;
@property (nonatomic) BOOL needPasteItem;
@property (nonatomic, strong) FSPDFDoc *copyedDoc;

@property (nonatomic, assign) BOOL didMovePageNeedUpdate;
@property (nonatomic, assign) BOOL needReloadData;
@property (nonatomic, assign) BOOL isUpdateCurrentRow;

@property (nonatomic, assign) NSInteger maxSelectedIndex;

@property (nonatomic, weak) FSThumbnailCell *currentPageCell;
@end

@implementation FSThumbnailChildViewController

@dynamic services;

static NSString *const reuseIDForThumbnailCell = @"thumbnailCell";
static NSString *const reuseIDForPlaceholderCell = @"placeholderCell";
static NSString *const ReuseIDForPastePlaceholderCell = @"ReuseIDForPastePlaceholderCell";

- (instancetype)initWithServices:(id<FSViewControllerServices>)services params:(nullable id)params{
    self = [super initWithServices:services params:params];
    if (self) {
        self.extensionsManager = self.services.extensionsManager;
        self.pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        
        self.document = self.pdfViewCtrl.currentDoc;
        self.isEditing = NO;
        self.operationQueue = ({
            NSOperationQueue *queue = [[NSOperationQueue alloc] init];
            queue.name = @"load thumbnail queue";
            queue.maxConcurrentOperationCount = 1;
            queue;
        });
        self.operations = [NSMutableDictionary<NSIndexPath *, NSBlockOperation *> dictionary];
        self.cellSize = [self calculateCellSize];
        _isUpdateCurrentRow = YES;
    }
    return self;
}

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.shouldLoadDataOnViewDidLoad = YES;
}

- (FSReorderableCollectionView *)collectionView{
    if (!_collectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
        CGSize tmpCellSize = self.cellSize;
        layout.itemSize = tmpCellSize;

        _collectionView = [[FSReorderableCollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout isModifiable:[FSPermissionUtilty canAssemble:self.extensionsManager] && ![self.document isXFA]];
        _collectionView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        _collectionView.backgroundColor = self.fs_dividerColor_P1;
        
        [_collectionView registerClass:[FSThumbnailCell class] forCellWithReuseIdentifier:reuseIDForThumbnailCell];
        [_collectionView registerClass:[FSReorderableCollectionViewPlaceholderCell class] forCellWithReuseIdentifier:reuseIDForPlaceholderCell];
        [_collectionView registerClass:[FSPastePlaceholderCollectionViewCell class] forCellWithReuseIdentifier:ReuseIDForPastePlaceholderCell];
        
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        _collectionView.allowsMultipleSelection = YES;
    }
    return _collectionView;
}

- (FSThumbnailChildType)thumbnailChildType{
    return [self.params intValue];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    if (self.needReloadData) [self reloadData];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.operationQueue cancelAllOperations];
    self.needReloadData = NO;
}

- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    if (self.thumbnailChildType == FSThumbnailChildTypeAll && _isUpdateCurrentRow) {
        NSIndexPath *currentIndexPath = [NSIndexPath indexPathForItem:[_pdfViewCtrl getCurrentPage] inSection:0];
        [self.collectionView scrollToItemAtIndexPath:currentIndexPath atScrollPosition:UICollectionViewScrollPositionCenteredVertically animated:YES];
        _isUpdateCurrentRow = NO;
    }
}

- (void)dealloc {
    [self.operationQueue cancelAllOperations];
}

- (void)loadData{
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {
        [self loadAnnotPageIndex];
    }else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark){
        [self sortBookmarks];
    }
}
- (void)reloadData{
    NSArray *selectedItems = self.collectionView.indexPathsForSelectedItems;
    [super reloadData];
    for (NSIndexPath *indexPath in selectedItems) {
        [self selectItem:YES atIndexPath:indexPath];
    }
}

- (void)loadAnnotPageIndex{
    _annotedPageIndexs = @[].mutableCopy;
    NSBlockOperation *op;
    __weak __block NSBlockOperation *weakOp;
    @weakify(self)
    weakOp = op = [NSBlockOperation blockOperationWithBlock:^{
        if (weakOp.isCancelled) {
            return;
        }
        @strongify(self)
        
        int pageCount = [self.document getPageCount];
        for (int pageIndex = 0; pageIndex < pageCount; pageIndex++) {
            FSPDFPage *page = [self.document getPage:pageIndex];
            if ([page isEmpty]) continue;
            if (page.getAnnotCount) {
                [self.annotedPageIndexs addObject:@(pageIndex)];
            }
        }
    }];
    weakOp.completionBlock = ^{
        @strongify(self)
        if (self.thumbnailChildType != FSThumbnailChildTypeAll) {
            dispatch_main_async_safe(^{
                if (!self.parentViewController) {
                    self.needReloadData = YES;
                }else{
                    [self reloadData];
                }
            });
        }
    };
    [self.operationQueue addOperation:op];
}

- (void)sortBookmarks{

    NSArray <NSNumber *>*sortPages = [self.bookmarkPageIndexs sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
        return [obj1 intValue] < [obj2 intValue] ? NSOrderedAscending : NSOrderedDescending;
    }];
    [self.bookmarkPageIndexs removeAllObjects];
    [self.bookmarkPageIndexs addObjectsFromArray:sortPages];
}

- (BOOL)emptyDataSetShouldDisplay:(UICollectionView *)collectionView{
    return ![self getNumberOfItemsInCollectionView];
}

- (NSString *)titleForEmptyDataSet{
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) return FSLocalizedForKey(@"kNoAnnotatedPages");
    return FSLocalizedForKey(@"kNoBookmarkedPages");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) return FSImageNamedAndPrimaryColor(@"panel_comment_no_data");
    return FSImageNamedAndPrimaryColor(@"panel_bookmark_no_data_gray");
}

- (int)getPageCount{
    BOOL isDynamciXFA = [_pdfViewCtrl isDynamicXFA];
    if (!isDynamciXFA) {
        return [self.document getPageCount];
    }
    return [[_pdfViewCtrl getXFADoc] getPageCount];
}

-(NSUInteger)getNumberOfItemsInCollectionView{
    NSUInteger count = 0;
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {
        count = self.annotedPageIndexs.count;
    }
    else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
        count = self.bookmarkPageIndexs.count;
    }else{
        count = [self getPageCount];
    }
    if (self.needPasteItem) count ++;
    return count;
}

- (int)getPageIndexInCollectionViewAtItem:(NSInteger)item{
    if (self.needPasteItem) {
        if (self.insertIndex < item) item --;
    }
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {
        if (item >= self.annotedPageIndexs.count) {
            return [self.annotedPageIndexs.lastObject intValue] + (int)(item - self.annotedPageIndexs.count + 1);
        }
        return [self.annotedPageIndexs[item] intValue];
    }
    else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
        if (self.bookmarkPageIndexs.count > item) {
            return  [self.bookmarkPageIndexs[item] intValue];
        }
    }
    return (int)item;
}
#pragma mark <UICollectionViewDataSource>

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    FSReorderableCollectionView *reorderCollectionView = (FSReorderableCollectionView *) collectionView;
    
    return [self getNumberOfItemsInCollectionView] - reorderCollectionView.indexSetForDraggingCells.count + (reorderCollectionView.indexPathForPlaceholderCell ? 1 : 0);
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {

    FSReorderableCollectionView *reorderableCollectionView = (FSReorderableCollectionView *) collectionView;
    if (self.needPasteItem) {
        FSPastePlaceholderCollectionViewCell *cell;
        NSIndexPath *indexPathForPastePlaceholderCell = reorderableCollectionView.indexPathForPastePlaceholderCell;
        if (indexPathForPastePlaceholderCell) {
            if (self.insertIndex != indexPathForPastePlaceholderCell.item) {
                self.insertIndex = indexPathForPastePlaceholderCell.item;
            }
            if (indexPathForPastePlaceholderCell == indexPath || indexPathForPastePlaceholderCell.item == indexPath.item) {
                cell = [reorderableCollectionView dequeueReusableCellWithReuseIdentifier:ReuseIDForPastePlaceholderCell forIndexPath:indexPath];
            }
        }else{
            if (indexPath.item == self.insertIndex) {
                reorderableCollectionView.indexPathForPastePlaceholderCell = indexPath;
                cell = [reorderableCollectionView dequeueReusableCellWithReuseIdentifier:ReuseIDForPastePlaceholderCell forIndexPath:indexPath];
            }
        }
        @weakify(self)
        cell.onTapClickClose = ^{
            @strongify(self)
            [self deleteCopyItem];
        };
        if (cell) return cell;
    }
    if ([indexPath isEqual:reorderableCollectionView.indexPathForPlaceholderCell]) {
        return [reorderableCollectionView dequeueReusableCellWithReuseIdentifier:reuseIDForPlaceholderCell forIndexPath:indexPath];
    }
    
    FSThumbnailCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:reuseIDForThumbnailCell forIndexPath:indexPath];
    [cell fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
    cell.contentView.backgroundColor = self.fs_backgroundColor_B3;
    cell.delegate = self;
    cell.isEditing = self.isEditing;
    // remove spinner
    for (UIView *subview in cell.contentView.subviews) {
        if ([subview isKindOfClass:[UIActivityIndicatorView class]]) {
            [subview removeFromSuperview];
            break;
        }
    }
    int pageIndex = [self getPageIndexInCollectionViewAtItem:[reorderableCollectionView getOriginalIndexPathForIndexPath:indexPath].item];
    NSString *title;
    if (self.extensionsManager.useLogicalPageNumbers){
        title = [Utility getPageLabelTitle:pageIndex doc:[self.pdfViewCtrl currentDoc]];
    }
    if (title.length){
        [cell.labelNumber setTitle:title forState:UIControlStateDisabled];
    }else{
        [cell.labelNumber setTitle:[NSString stringWithFormat:@"%d", pageIndex + 1] forState:UIControlStateDisabled];
    }
    cell.accessibilityLabel = [NSString stringWithFormat:FSLocalizedForKey(@"kSpeechPage"),pageIndex + 1];
    // update button frames
    
    float width = 600;
    float height = 800;
    @try{
        if (![_pdfViewCtrl isDynamicXFA]) {
            FSPDFPage *page = [self.document getPage:pageIndex];
            if (![page isEmpty]) {
                width = [page getWidth];
                height = [page getHeight];
            }
        }else{
            FSXFAPage *page = [[_pdfViewCtrl getXFADoc] getPage:pageIndex];
            if (![page isEmpty]) {
                width = [page getWidth];
                height = [page getHeight];
            }
        }
        
    }
   @catch(NSException* e)
    {
    }
    
    FSRotation rotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    if (rotation == FSRotation90 || rotation == FSRotation270) {
        int tmp = width;
        width =  height;
        height = tmp;
    }
    
    CGFloat realWidth = MIN(self.cellSize.width, self.cellSize.height * width / height);
    [cell updateButtonFramesWithThumbnailWidth:realWidth];
    
    @weakify(self)
    [BindThemeKeySel(cell, updateButtonFramesWithThumbnailWidth:) fs_updateThemeCompleted:^(FSThumbnailCell * _Nonnull target, SEL  _Nonnull sel) {
        @strongify(self)
        if (pageIndex == [self.pdfViewCtrl getCurrentPage]) {
            
            self.currentPageCell.layer.borderColor = target.fs_dividerColor_P1.CGColor;
            self.currentPageCell.labelNumber.backgroundColor = [[UIColor fs_blackColor] colorWithAlphaComponent:0.5];
            self.currentPageCell.layer.borderWidth = 1.f;
            
            self.currentPageCell = target;
            target.layer.borderWidth = 2.f;
            target.labelNumber.backgroundColor = [target.fs_primaryColor colorWithAlphaComponent:0.5];
            target.layer.borderColor = target.fs_primaryColor.CGColor;
        }else{
            target.layer.borderColor = target.fs_dividerColor_P1.CGColor;
            target.labelNumber.backgroundColor = [[UIColor fs_blackColor] colorWithAlphaComponent:0.5];
            target.layer.borderWidth = 1.f;
        }
    }];
    
    cell.bookmark.hidden = ![self.bookmarkPageIndexs containsObject:@(pageIndex)];

    BOOL selected = [[self.collectionView indexPathsForSelectedItems] containsObject:indexPath];
    cell.selected = selected;
    return cell;
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        if ([self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection]) {
            [self reloadData];
        }
    }
}
#endif

#pragma mark <FSReorderableCollectionViewDataSource>

- (void)reorderableCollectionView:(FSReorderableCollectionView *)collectionView moveItemAtIndexPaths:(NSArray<NSIndexPath *> *)sourceIndexPaths toIndex:(NSUInteger)destinationIndex {
    if (collectionView.isDraggingPasteCell) return;
    if (!self.didMovePageNeedUpdate) return;
    NSMutableArray *pageIndexs = @[].mutableCopy;
    for (NSIndexPath *sourceIndexPath in sourceIndexPaths) {
        [pageIndexs addObject:@([self getPageIndexInCollectionViewAtItem:sourceIndexPath.item])];
    }
    [self.thumbnailParentViewController movePageIndexs:pageIndexs toIndex:destinationIndex beforeOrAfter:self.maxSelectedIndex > destinationIndex];
}

#pragma mark <UICollectionViewDelegate>
- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if (!self.isEditing) {
        int pageIndex = (int) [self getPageIndexInCollectionViewAtItem:indexPath.item];
        [self.thumbnailParentViewController.delegate thumbnailViewController:self.thumbnailParentViewController openPage:pageIndex];
        return;
    }
    if (self.thumbnailParentViewController.hasPasteItemInChildViewController && !self.needPasteItem){
        [collectionView deselectItemAtIndexPath:indexPath animated:NO];
        return;
    }
    if (self.needPasteItem && indexPath.item == self.insertIndex) {
        [collectionView deselectItemAtIndexPath:indexPath animated:NO];
        [self deleteCopyItem];
        @weakify(self)
        [self.thumbnailParentViewController.view fs_showHUDLoading:FSLocalizedForKey(@"kPasting")];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.thumbnailParentViewController duplicatePages:^(NSString * _Nonnull errorInfo) {
                @strongify(self)
                self.insertIndex = -1;
                [self.thumbnailParentViewController onCellSelectedOrDeselected];
                [self.thumbnailParentViewController.view fs_hideHUDLoading];
            }];
        });
        return;
    }
    if (indexPath.item > self.maxSelectedIndex) {
        self.maxSelectedIndex = indexPath.item;
    }
    [self.thumbnailParentViewController onCellSelectedOrDeselected];
}

- (void)collectionView:(UICollectionView *)collectionView didDeselectItemAtIndexPath:(nonnull NSIndexPath *)indexPath {
    if (self.thumbnailParentViewController.hasPasteItemInChildViewController && !self.needPasteItem){
        [collectionView selectItemAtIndexPath:indexPath animated:NO scrollPosition:UICollectionViewScrollPositionNone];
        return;
    }
    if (indexPath.item == self.maxSelectedIndex) {
        NSArray<NSNumber *> *selectedItemsForOrderedAscending = [self getSelectedItemsForOrderedAscending];
        if (selectedItemsForOrderedAscending.count) {
            self.maxSelectedIndex = selectedItemsForOrderedAscending.lastObject.intValue;
        }else{
            self.maxSelectedIndex = -1;
        }
    }
    [self.thumbnailParentViewController onCellSelectedOrDeselected];
}

- (void)collectionView:(UICollectionView *)collectionView willDisplayCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    if (self.needPasteItem && self.insertIndex == indexPath.item) return;
    if (![cell isKindOfClass:[FSThumbnailCell class]]) {
        return;
    }
    UIActivityIndicatorView *spinner = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    spinner.center = CGPointMake(cell.contentView.bounds.size.width / 2, cell.contentView.bounds.size.height / 2);
    [cell.contentView addSubview:spinner];
    [spinner startAnimating];
    ((FSThumbnailCell *) cell).imageView.image = nil;

    FSReorderableCollectionView *reorderCollectionView = (FSReorderableCollectionView *) collectionView;
    NSUInteger pageIndex = [self getPageIndexInCollectionViewAtItem:[reorderCollectionView getOriginalIndexPathForIndexPath:indexPath].item];

    CGSize thumbnailSize = ({
        float width = 600;
        float height = 800;
        @try {
            if (![_pdfViewCtrl isDynamicXFA]) {
                FSPDFPage *page = [self.document getPage:(int)pageIndex];
                if (![page isEmpty]) {
                    width = [page getWidth];
                    height = [page getHeight];
                }
            }else{
                FSXFAPage *page = [[_pdfViewCtrl getXFADoc] getPage:(int)pageIndex];
                if (![page isEmpty]) {
                    width = [page getWidth];
                    height = [page getHeight];
                }
            }
        }
        @catch(NSException* e)
        {}
        CGFloat aspectRatio = width / height;
        CGFloat thumbnailWidth = MAX(self.cellSize.width, self.cellSize.height * aspectRatio);
        CGFloat thumbnailHeight = thumbnailWidth / aspectRatio;
        CGSizeMake(thumbnailWidth, thumbnailHeight);
    });

    FSRotation rotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    if (rotation == FSRotation90 || rotation == FSRotation270) {
        int tmp = thumbnailSize.width;
        thumbnailSize.width =  thumbnailSize.height;
        thumbnailSize.height = tmp;
    }
    
    NSBlockOperation *op;
    __weak __block NSBlockOperation *weakOp;
    @weakify(self)
    weakOp = op = [NSBlockOperation blockOperationWithBlock:^{
        if (weakOp.isCancelled) {
            return;
        }
        @strongify(self)
        [self.thumbnailParentViewController.delegate thumbnailViewController:self.thumbnailParentViewController
                        getThumbnailForPageAtIndex:pageIndex
                                     thumbnailSize:thumbnailSize
                                         needPause:^BOOL {
                                             return !weakOp || weakOp.isCancelled;
                                         }
                                          callback:^(UIImage *thumbnailImage) {
                                              dispatch_async(dispatch_get_main_queue(), ^{
                                                  @strongify(self)
                                                  if (!weakOp.isCancelled && [self.collectionView.visibleCells containsObject:cell]) {
                                                      ((FSThumbnailCell *) cell).imageView.image = thumbnailImage;
                                                      [spinner stopAnimating];
                                                      [spinner removeFromSuperview];
                                                  }
                                              });
                                          }];
    }];
    

    [self.operationQueue addOperation:op];
    if (self.operations[indexPath]) {
        [self.operations[indexPath] cancel];
    }
    self.operations[indexPath] = op;

    BOOL selected = [[self.collectionView indexPathsForSelectedItems] containsObject:indexPath];
    cell.selected = selected;
}

- (void)collectionView:(UICollectionView *)collectionView didEndDisplayingCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    if (![self.collectionView.indexPathsForVisibleItems containsObject:indexPath]) {
        NSBlockOperation *operation = self.operations[indexPath];
        [operation cancel];
        self.operations[indexPath] = nil;
    }
}

#pragma mark <UICollectionViewDelegateFlowLayout>

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    return self.cellSize;
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewFlowLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets cellInsets = [self cellInsets];
    UIEdgeInsets sectionInset = [self sectionInsets];
    CGFloat margin = sectionInset.left + cellInsets.left;
    CGFloat remainSpace = collectionView.fs_width - (self.cellSize.width + margin * 2) * 2;
    if (remainSpace > 0 && (remainSpace < self.cellSize.width + margin * 2)) {
        margin = collectionView.fs_width / 2 - self.cellSize.width - 15.f;
        if (margin < 0) {
            margin = ABS((collectionView.fs_width / 2 - self.cellSize.width)/2);
        }
    }
    return UIEdgeInsetsMake(sectionInset.top, margin, sectionInset.bottom, margin);
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout minimumLineSpacingForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets cellInsets = [self cellInsets];
    return cellInsets.top + cellInsets.bottom;
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets cellInsets = [self cellInsets];
    return cellInsets.left + cellInsets.right;
}

#pragma mark <FSThumbnailCellDelegate>

- (void)cell:(FSThumbnailCell *)cell rotateClockwise:(BOOL)clockwise {
    NSIndexPath *indexPath = [self.collectionView indexPathForCell:cell];
    if (indexPath) {
        [self cancelDrawingThumbnailsAtIndexPaths:@[ indexPath ]];
        if ([self.thumbnailParentViewController.pageManipulationDelegate rotatePagesAtIndexes:@[ @(indexPath.item) ] clockwise:clockwise]) {
            BOOL selected = [self.collectionView.indexPathsForSelectedItems containsObject:indexPath];
            [self.collectionView reloadItemsAtIndexPaths:@[ indexPath ]];
            if (selected) {
                [self.collectionView selectItemAtIndexPath:indexPath animated:NO scrollPosition:UICollectionViewScrollPositionNone];
            }
        } else {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailRotatePage") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
        }
    }
}

- (void)deleteCell:(FSThumbnailCell *)cell {
    NSIndexPath *indexPath = [self.collectionView indexPathForCell:cell];
    if (indexPath) {
        [self deleteItemsAtIndexPaths:indexPath completion:nil];
    }
}

- (void)didShowEditButtonsInCell:(FSThumbnailCell *)cell {
    for (FSThumbnailCell *visibleCell in self.collectionView.visibleCells) {
        if (visibleCell != cell) {
        }
    }
}

- (void)cell:(FSThumbnailCell *)cell insertBeforeOrAfter:(BOOL)beforeOrAfter {
    NSIndexPath *indexPath = [self.collectionView indexPathForCell:cell];
    if (indexPath) {
        self.insertIndex = (beforeOrAfter ? indexPath.item : indexPath.item + 1);
    }
}


#pragma mark <FSReorderableCollectionViewReorderDelegate>

- (void)reorderableCollectionView:(FSReorderableCollectionView *)collectionView willMoveItemsAtIndexPaths:(NSArray<NSIndexPath *> *)sourceIndexPaths toIndex:(NSUInteger)destinationIndex {
    if (collectionView.isDraggingPasteCell) {
        self.insertIndex = collectionView.indexPathForPlaceholderCell.item;
        return;
    }
    self.didMovePageNeedUpdate = YES;
    if (sourceIndexPaths.count == 1) {
        if (sourceIndexPaths.lastObject.item == destinationIndex) {
            self.didMovePageNeedUpdate = NO;
            return;
        }
    }
    [self cancelDrawingThumbnails];
}

- (void)reorderableCollectionView:(FSReorderableCollectionView *)collectionView didMoveItemsAtIndexPaths:(NSArray<NSIndexPath *> *)sourceIndexPaths toIndex:(NSUInteger)destinationIndex {
    if (collectionView.isDraggingPasteCell) return;
    if (!self.didMovePageNeedUpdate) return;
    __block NSUInteger maxIndex = destinationIndex;
    __block NSUInteger minIndex = destinationIndex;
    [sourceIndexPaths enumerateObjectsUsingBlock:^(NSIndexPath *_Nonnull obj, NSUInteger idx, BOOL *_Nonnull stop) {
        if (obj.item > maxIndex) {
            maxIndex = obj.item;
        }
        if (obj.item < minIndex) {
            minIndex = obj.item;
        }
    }];
    
    NSIndexPath *currentPageIndexPath = [NSIndexPath indexPathForItem:[self.pdfViewCtrl getCurrentPage] inSection:0];
    if ([sourceIndexPaths containsObject:currentPageIndexPath]) {
        BOOL selected = [collectionView.indexPathsForSelectedItems containsObject:currentPageIndexPath];
        [self reloadItemsAtIndexPaths:currentPageIndexPath completion:^(BOOL finished) {}];
        if (selected) [collectionView selectItemAtIndexPath:currentPageIndexPath animated:NO scrollPosition:UICollectionViewScrollPositionNone];
    }
    [self updatePageNumberLabelsInRange:NSMakeRange(minIndex, maxIndex - minIndex + 1)];
    [self.thumbnailParentViewController onCellSelectedOrDeselected];
    NSArray<NSNumber *> *selectedItemsForOrderedAscending = [self getSelectedItemsForOrderedAscending];
    if (selectedItemsForOrderedAscending.count) {
        self.maxSelectedIndex = selectedItemsForOrderedAscending.lastObject.intValue;
    }else{
        self.maxSelectedIndex = -1;
    }
    [self movePageIndexsAfterReload];
}

#pragma mark event handler

- (void)setIsEditing:(BOOL)isEditing {
    if (_isEditing != isEditing) {
        _isEditing = isEditing;
        //update tool bars
        
        //        //update collection view
        [self.collectionView.visibleCells enumerateObjectsUsingBlock:^(__kindof FSThumbnailCell *_Nonnull cell, NSUInteger idx, BOOL *_Nonnull stop) {
            if ([cell respondsToSelector:@selector(setIsEditing:)]) {
                cell.isEditing = isEditing;
            }
        }];
        [self deselectAll];
        
        _collectionView.longPress.enabled = isEditing && !(!self.needPasteItem && self.thumbnailParentViewController.hasPasteItemInChildViewController);
        
        self.maxSelectedIndex = -1;
    }
}

- (void)selectOrDeselectAll {
    NSUInteger pageCount = [self getNumberOfItemsInCollectionView] - (self.needPasteItem ? 1 : 0);
    BOOL isSelectAll = (self.collectionView.indexPathsForSelectedItems.count == pageCount);
    
    if (!isSelectAll) {
        [self selectAll];
    } else {
        [self deselectAll];
    }
}

#pragma mark insert pages and images
- (void)convenientAlert:(NSString *)title {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:nil preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                           style:UIAlertActionStyleCancel
                                                         handler:nil];
    [alertController addAction:cancelAction];
    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - inner methods


- (void)selectAll {
    NSUInteger pageCount = [self getNumberOfItemsInCollectionView];
    for (int i = 0; i < pageCount; i++) {
        if (self.needPasteItem && i == self.insertIndex) {
            continue;
        }
        [self.collectionView selectItemAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0] animated:NO scrollPosition:UICollectionViewScrollPositionNone];
    }
    [self.thumbnailParentViewController onCellSelectedOrDeselected];
}

- (void)deselectAll {
    for (NSIndexPath *selectedIndexPath in self.collectionView.indexPathsForSelectedItems) {
        [self.collectionView deselectItemAtIndexPath:selectedIndexPath animated:NO];
    }
    assert(0 == self.collectionView.indexPathsForSelectedItems.count);
    [self.thumbnailParentViewController onCellSelectedOrDeselected];
}


- (void) willDeletePageIndexs:(NSArray <NSNumber *> *)pageIndexs{
    for (NSNumber *pageIndex in  pageIndexs) {
        [self.operations[[NSIndexPath indexPathForItem:pageIndex.intValue inSection:0]] cancel];
    }

}
- (void) didDeletePageIndexs:(NSArray <NSNumber *> *)pageIndexs{
    if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
        [self sortBookmarks];
    }
    NSMutableArray *indexPaths = @[].mutableCopy;
    NSArray *annotedPageIndexs = self.annotedPageIndexs.copy;
    NSArray *bookmarkPageIndexs = self.bookmarkPageIndexs.copy;
    for (NSNumber *pageIndex in  pageIndexs) {
        if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {
            NSUInteger index = [annotedPageIndexs indexOfObject:pageIndex];
            if (index != NSNotFound) {
                [self.annotedPageIndexs removeObject:pageIndex];
                [indexPaths addObject:[NSIndexPath indexPathForItem:index inSection:0]];
            }
        } else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
            NSUInteger index = [bookmarkPageIndexs indexOfObject:pageIndex];
            if (index != NSNotFound) {
                [indexPaths addObject:[NSIndexPath indexPathForItem:index inSection:0]];
            }
        }else{
            [indexPaths addObject:[NSIndexPath indexPathForItem:pageIndex.intValue inSection:0]];
        }
    }
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {
        if (self.parentViewController) {
            [self deleteItemsAtIndexPaths:indexPaths completion:^(BOOL finished) {
                if (self.annotedPageIndexs.count) {
                    [self loadAnnotPageIndex];
                }
            }];
        }else{
            self.needReloadData = YES;
            if (self.annotedPageIndexs.count) {
                [self loadAnnotPageIndex];
            }
        }
    } else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
        if (self.parentViewController) {
            [self deleteItemsAtIndexPaths:indexPaths completion:^(BOOL finished) {
                if (self.bookmarkPageIndexs.count) {
                    [self updatePageNumberLabels];
                }
            }];
        }else{
            self.needReloadData = YES;
            if (self.bookmarkPageIndexs.count) {
                [self updatePageNumberLabels];
            }
        }
    }else{
        NSMutableArray *needDeleteIndexPaths = @[].mutableCopy;
        for (NSIndexPath *indexPath in self.collectionView.indexPathsForVisibleItems){
            if ([indexPaths containsObject:indexPath]){
                [needDeleteIndexPaths addObject:indexPath];
            }
        }
        [self deleteItemsAtIndexPaths:needDeleteIndexPaths completion:nil];
        [self reloadItemsAtIndexPaths:[NSIndexPath indexPathForItem:[self.pdfViewCtrl getCurrentPage] inSection:0] completion:nil];
        [self updatePageNumberLabels];
    }
}

- (void)movePageIndexsAfterReload{
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {
        [self loadAnnotPageIndex];
    }else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
        [self sortBookmarks];
        [self updatePageNumberLabels];
    }else{
        [self reloadData];
    }
}

- (NSUInteger)findIndexInAnnotedPageIndexsForPageIndex:(NSUInteger)pageIndex{
    
    NSUInteger index = 0;
    for (int i = 0; i <  self.annotedPageIndexs.count; i++) {
        NSNumber *_pageIndex = self.annotedPageIndexs[i];
        if ((index < pageIndex && pageIndex < _pageIndex.intValue) || pageIndex == _pageIndex.intValue) {
            return i;
        }
        index = _pageIndex.intValue;
    }
    return self.annotedPageIndexs.count;
}

- (void)insertPagesInIndexPaths:(NSArray<NSIndexPath *> *)indexPaths atIndex:(NSUInteger)index{
    if (self.thumbnailChildType == FSThumbnailChildTypeAnnoted) {

        NSUInteger findIndex = [self findIndexInAnnotedPageIndexsForPageIndex:index];
        if (self.insertIndex == index) findIndex = index;
        NSUInteger indexPathItem = findIndex;
        
        NSMutableArray *insertPages = @[].mutableCopy;
        NSMutableArray *insertIndexPaths = @[].mutableCopy;
        for (NSIndexPath *insertPage in indexPaths) {
            int pageIndex = (int )insertPage.item;
            FSPDFPage *page = [self.document getPage:pageIndex];
            if (page.getAnnotCount) {
                [insertPages addObject:@(pageIndex)];
                [insertIndexPaths addObject:[NSIndexPath indexPathForItem:indexPathItem inSection:0]];
                indexPathItem ++;
            }
        }
        BOOL needUpdate = NO;
        if (insertPages.count) {
            [self.annotedPageIndexs fs_insertObjects:insertPages atIndex:findIndex];
            [self.collectionView insertItemsAtIndexPaths:insertIndexPaths];
            for (NSUInteger i = index + insertPages.count; i < self.annotedPageIndexs.count; i++) {
                NSInteger tmpPageIndex = [self.annotedPageIndexs[i] intValue];
                self.annotedPageIndexs[i] = @(tmpPageIndex + indexPaths.count);
            }
            needUpdate = (index + insertPages.count < self.annotedPageIndexs.count);
        }else{
            for (NSUInteger i = findIndex; i < self.annotedPageIndexs.count; i++) {
                NSInteger tmpPageIndex = [self.annotedPageIndexs[i] intValue];
                self.annotedPageIndexs[i] = @(tmpPageIndex + indexPaths.count);
            }
            needUpdate = (findIndex < self.annotedPageIndexs.count);
        }
        if (needUpdate) {
            [self updatePageNumberLabelsInRange:NSMakeRange(findIndex, [self getNumberOfItemsInCollectionView] - findIndex)];
        }
    }else if (self.thumbnailChildType == FSThumbnailChildTypeBookmark) {
        [self sortBookmarks];
        [self updatePageNumberLabels];
    }else{
        if (self.thumbnailParentViewController.currentThumbnailChildViewController != self) {
            [self reloadData];
        }else{
            [self.collectionView insertItemsAtIndexPaths:indexPaths];
        }
        [self updatePageNumberLabelsInRange:NSMakeRange(index, [self getPageCount] - index)];
    }
}

- (void)insertCopyItem{
    
    for (FSItem *segmentItem in self.thumbnailParentViewController.segmentItems) {
        if (segmentItem == self.thumbnailParentViewController.segmentCurrentItem) {
            continue;
        }
        segmentItem.enabled = NO;
    }
    if (self.thumbnailChildType == FSThumbnailChildTypeAll) {
        self.insertIndex = [self.pdfViewCtrl getCurrentPage] + 1;
        self.needPasteItem = YES;
        [self insertItemsAtIndexPaths:[NSIndexPath indexPathForItem:self.insertIndex inSection:0] completion:nil];
    }else{
        self.insertIndex = [self getNumberOfItemsInCollectionView];
        self.needPasteItem = YES;
        NSIndexPath *insertIndexPath = [NSIndexPath indexPathForItem:self.insertIndex inSection:0];
        [self insertItemsAtIndexPaths:insertIndexPath completion:nil];
        [self.collectionView scrollToItemAtIndexPath:insertIndexPath atScrollPosition:UICollectionViewScrollPositionNone animated:NO];
    }
}

- (void)copyDocForSelectedPageIndexs{
    self.copyedDoc = [self getDocForSelectedPageIndexs];
}

- (void)deleteCopyItem{
    for (FSItem *segmentItem in self.thumbnailParentViewController.segmentItems) {
        segmentItem.enabled = YES;
    }
    if (self.needPasteItem) {
        self.needPasteItem = NO;
        _collectionView.indexPathForPastePlaceholderCell = nil;
        [self deleteItemsAtIndexPaths:[NSIndexPath indexPathForItem:self.insertIndex inSection:0] completion:nil];
        [self.thumbnailParentViewController onCellSelectedOrDeselected];
    }
}

- (FSPDFDoc *)getDocForSelectedPageIndexs{
    NSArray<NSNumber *>  *sortPages = [self getSelectedItemsForOrderedAscending];
    if (!sortPages.count) return nil;
    //create an empty doc
    FSPDFDoc *doc = [[FSPDFDoc alloc] init];
    FSRange *pageRanges = [[FSRange alloc] init];
    for (NSUInteger i = 0; i < sortPages.count; i++) {
        int pageIndex = [self getPageIndexInCollectionViewAtItem:[sortPages[i] intValue]];
        [pageRanges addSingle:pageIndex];
    }
    //import by srcDoc
    FSProgressiveState state = FSProgressiveError;
    FSProgressive* progress = nil;
    @try {
        progress = [doc startImportPages:0 src_doc:self.document flags:0 layer_name:nil page_range:pageRanges pause:nil];
        if(!progress)
            state = FSProgressiveFinished;
        else
            state = FSProgressiveToBeContinued;

    } @catch (NSException *exception) {
        NSLog(@"FSPDFDoc::startImportPages EXCEPTION NAME:%@", exception.name);
        if ([exception.name isEqualToString:@"FSErrUnsupported"]) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kUnsupportedDocFormat") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
        }
        return nil;
    }

    while (state == FSProgressiveToBeContinued) {
        state = [progress resume];
    }
    if (FSProgressiveFinished != state) {
        return nil;
    }
    return doc;
}

- (NSArray <NSNumber *> *)getSelectedItemsForOrderedAscending{
    NSArray<NSNumber *> *pages = [self.collectionView.indexPathsForSelectedItems valueForKey:@"item"];
    if (pages.count == 0) {
        return nil;
    }
    NSArray <NSNumber *>*sortPages = [pages sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
        return [obj1 intValue] < [obj2 intValue] ? NSOrderedAscending : NSOrderedDescending;
    }];
    return sortPages;
}

- (void)updatePageNumberLabels{
    for (NSIndexPath *indexPath in self.collectionView.indexPathsForVisibleItems) {
        FSThumbnailCell *cell = (FSThumbnailCell *) [self.collectionView cellForItemAtIndexPath:indexPath];
        if ([cell isKindOfClass:[FSThumbnailCell class]]) {
            if (self.extensionsManager.useLogicalPageNumbers){
                NSString *title = [Utility getPageLabelTitle:[self getPageIndexInCollectionViewAtItem:indexPath.item] doc:[self.pdfViewCtrl currentDoc]];
                if (title.length){
                    [cell.labelNumber setTitle:title forState:UIControlStateDisabled];
                    continue;
                }
            }
            [cell.labelNumber setTitle:[NSString stringWithFormat:@"%d", [self getPageIndexInCollectionViewAtItem:indexPath.item] + 1] forState:UIControlStateDisabled];
        }
    }

}

- (void)updatePageNumberLabelsInRange:(NSRange)range {
    if (range.location == NSNotFound || range.length == 0) {
        return;
    }
    for (NSUInteger i = range.location; i < range.location + range.length; i++) {
        FSThumbnailCell *cell = (FSThumbnailCell *) [self.collectionView cellForItemAtIndexPath:[NSIndexPath indexPathForItem:i inSection:0]];
        if ([cell isKindOfClass:[FSThumbnailCell class]]) {
            if (self.extensionsManager.useLogicalPageNumbers){
                NSString *title = [Utility getPageLabelTitle:[self getPageIndexInCollectionViewAtItem:i] doc:[self.pdfViewCtrl currentDoc]];
                if (title.length){
                    [cell.labelNumber setTitle:title forState:UIControlStateDisabled];
                    continue;
                }
            }
            [cell.labelNumber setTitle:[NSString stringWithFormat:@"%d", [self getPageIndexInCollectionViewAtItem:i] + 1] forState:UIControlStateDisabled];
        }

    }
}

- (void)cancelDrawingThumbnails {
    [self.operationQueue cancelAllOperations];
    [self.operations removeAllObjects];
}

- (void)cancelDrawingThumbnailsAtIndexPaths:(NSArray<NSIndexPath *> *)indexPaths {
    [indexPaths enumerateObjectsUsingBlock:^(NSIndexPath *_Nonnull obj, NSUInteger idx, BOOL *_Nonnull stop) {
        [self.operations[obj] cancel];
        self.operations[obj] = nil;
    }];
}

- (void)viewWillLayoutSubviews{
    [super viewWillLayoutSubviews];
    
    CGSize cellSize = [self calculateCellSize];
    if (!CGSizeEqualToSize(cellSize, self.cellSize) || CGSizeEqualToSize(CGSizeZero, self.cellSize)) {
        self.cellSize = cellSize;
        [self.collectionView.collectionViewLayout invalidateLayout];
    //    [self.collectionView reloadData];
    }
}

#pragma mark layout sizes

- (CGSize)calculateCellSize {
    CGSize cellSize;
#define OVERVIEW_IMAGE_WIDTH_IPHONE 142.0
#define OVERVIEW_IMAGE_HEIGHT_IPHONE 177.0
#define OVERVIEW_IMAGE_WIDTH 160.0
#define OVERVIEW_IMAGE_HEIGHT 200.0
    if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        cellSize = CGSizeMake(OVERVIEW_IMAGE_WIDTH_IPHONE, OVERVIEW_IMAGE_HEIGHT_IPHONE);
    } else {
        cellSize = CGSizeMake(OVERVIEW_IMAGE_WIDTH, OVERVIEW_IMAGE_HEIGHT);
    }
    
    return cellSize;
}

- (UIEdgeInsets)cellInsets {
    UIInterfaceOrientation orientation = [UIApplication sharedApplication].statusBarOrientation;
    ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
    if (UIInterfaceOrientationIsPortrait(orientation)) {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(5, 12, 5, 12);
            }  else {
                return UIEdgeInsetsMake(5, 6, 5, 6);
            }
        } else {
            return UIEdgeInsetsMake(10, 17, 10, 10);
        }
    } else {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (sizeMode == ScreenSizeMode_40) {
                return UIEdgeInsetsMake(5, 0, 5, 0);
            } else if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(5, 9, 5, 8);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(5, 2, 5, 1);
            } else {
                return UIEdgeInsetsMake(5, 7, 5, 7);
            }
        } else {
            return UIEdgeInsetsMake(10, 20, 10, 20);
        }
    }
}

- (UIEdgeInsets)sectionInsets {
    CGFloat topInset = (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) ? 10 : 20;
    
    UIInterfaceOrientation orientation = [UIApplication sharedApplication].statusBarOrientation;
    if (UIInterfaceOrientationIsPortrait(orientation)) {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
            if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(topInset, 20, 44, 16);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(topInset, 0, 44, 0);
            } else {
                return UIEdgeInsetsMake(topInset, 6, 44, 5);
            }
        } else {
            return UIEdgeInsetsMake(topInset, 6, 44, 5);
        }
    } else {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
            if (sizeMode == ScreenSizeMode_40) {
                return UIEdgeInsetsMake(topInset, 0, 44, 0);
            } else if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(topInset, 15, 44, 15);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(topInset, 4, 44, 4);
            } else {
                return UIEdgeInsetsMake(topInset, 5, 44, 5);
            }
        } else {
            return UIEdgeInsetsMake(topInset, 10, 44, 5);
        }
    }
}

@end
