/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FillSignModule.h"
#import "DigitalSignatureAnnotHandler.h"
#import "FSSettingBar+private.h"
#import "Utility.h"
#import "UIButton+Extensions.h"
#import "UIExtensionsConfig.h"
#import "FillSignToolHandler.h"
#import "FillSignProfileController.h"
#import "FillSignMenu.h"
#import "MenuView+Private.h"
#import "FSFillSignUndoItem.h"
#import "FSPermissionUtilty.h"

@interface FillSignModule ()<MoreItemActionProtocol, IAnnotEventListener, IFSMainTopbarEventListener, IStateChangeListener, IThumbnailEventListener, IPanelChangedListener, IMoreMenuViewListener, IDocModifiedEventListener, IFSViewSettingEventListener, IFSUndoEventListener>
@property (nonatomic, strong) TbBaseItem *signItem;

@property (nonatomic, strong) FSToolbar *topToolBar;
@property (nonatomic, strong) FSToolbar *bottomToolBar;
@property (nonatomic, assign) int oldState;
@property (nonatomic, strong) UIButton *cancelBtn;
@property (nonatomic, weak) FSItem *selectedItem;
@property (nonatomic, copy) NSArray *preDefineContent;
@property (nonatomic, copy) NSString *selectPreDefineStr;

@property (nonatomic, strong) NSMutableArray *fillSignItems;
@end

@implementation FillSignModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return Module_Fill_Sign;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _fillSignMenu = [[FillSignMenu alloc] initWithExtensionsManager:extensionsManager];
        [self loadModule];
        FillSignToolHandler* fillSignToolHandler = [[FillSignToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:fillSignToolHandler];
        [_extensionsManager registerRotateChangedListener:fillSignToolHandler];
        [_fillSignMenu registerFillSignMenuEvent:fillSignToolHandler];
        self.fillSignToolHandler = fillSignToolHandler;
        
        DigitalSignatureAnnotHandler* annotHandler = [[DigitalSignatureAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_pdfViewCtrl registerScrollViewEventListener:annotHandler];
        [_extensionsManager registerRotateChangedListener:annotHandler];
        [_extensionsManager registerAnnotHandler:annotHandler];
        [_extensionsManager registerUndoEventListener:self];

    }
    return self;
}

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)loadModule {

    [_pdfViewCtrl registerDocEventListener:self];
    [_extensionsManager registerToolEventListener:self];
    [_extensionsManager registerStateChangeListener:self];
    [_extensionsManager registerAnnotEventListener:self];
    [_extensionsManager registerDocModifiedEventListener:self];

    [_extensionsManager.settingBar.viewSettingManager registerEventListener:self];
    [_extensionsManager.more registerEventListener:self];
    [_extensionsManager.toolbarManager registerTopbarEventListener:self];
    [_extensionsManager.thumbnailModule registerThumbnailEventListener:self];
    [_extensionsManager.panelController registerPanelChangedListener:self];
    
    _fillSignItems = @[].mutableCopy;
    for (int i = FSReadToolSettingsTypeFillSignText; i <= FSReadToolSettingsTypeFillSignRoundRectangle; i++) {
        FSReadToolbarItem *fillSignItem = [[FSReadToolbarItem alloc] initWithToolSettingType:i withItemContentType:FSItemContentTypeImageAndText];
        [_extensionsManager.toolbarManager addReadItem:fillSignItem atToolTag:FSMainTopbarToolTagSign];
        [_fillSignItems addObject:fillSignItem];
    }
    
    self.preDefineContent = [FillSignProfileController getPreDefineConetArray];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(removeUndoItems:) name:FSNotificationNameFlattenAnnot object:_extensionsManager];
}

- (void)removeUndoItems:(NSNotification *)not{
    if (not.object == _extensionsManager) {
        NSMutableArray *needRemoveItems = @[].mutableCopy;
        for (UndoItem *undoItem in _extensionsManager.undoItems) {
            if ([undoItem isKindOfClass:[FSFillSignUndoItem class]]) {
                [needRemoveItems addObject:undoItem];
            }
        }
        for (UndoItem *undoItem in _extensionsManager.redoItems) {
            if ([undoItem isKindOfClass:[FSFillSignUndoItem class]]) {
                [needRemoveItems addObject:undoItem];
            }
        }
        [_extensionsManager removeUndoItems:needRemoveItems];
    }
}

#pragma mark - MoreMenuItemAction
- (void)onClick:(MoreMenuItem *)item {
    [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeCerTrustedList];
}

- (void)updateFillSignItm{
    for (FSReadToolbarItem *fillSignItem in _fillSignItems) {
        BOOL canAddFillSign = [self canAddFillSign];
        fillSignItem.enabled = canAddFillSign;
    }
}

- (BOOL)canAddFillSign{
    BOOL isDynamicXFA = [_pdfViewCtrl isDynamicXFA];
    if (isDynamicXFA) {
        return NO;
    }
    BOOL modifyDoc = [FSPermissionUtilty canModifyContentsInDocument:_extensionsManager];
    //BOOL hasForm = [[_pdfViewCtrl currentDoc] hasForm];
    BOOL sign = [FSPermissionUtilty canAddSign:_extensionsManager];
    BOOL fill = [FSPermissionUtilty canFillForm:_extensionsManager];
    return  !(!modifyDoc || !sign || !fill);
}

#pragma mark IHandlerEventListener

- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName {
    if ([lastToolName isEqualToString:Tool_Signature] && ![toolName isEqualToString:Tool_Fill_Sign]) {
        if (_extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag != FSMainTopbarToolTagSign) {
            [_extensionsManager changeState:FSUIManagerStateNormal];
        }
    }else if ([toolName isEqualToString:Tool_Signature]) {
        if (self.fillSignToolHandler.editFillHandler) {
            [self.fillSignToolHandler deactiveFillSign];
        }
        FillSignMenu *fillSignMenu = self.fillSignMenu;
        if (fillSignMenu.isMenuVisible) {
            [fillSignMenu setMenuVisible:NO animated:NO];
        }
    }
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    
    if (self.fillSignMenu.isMenuVisible) {
        [self.fillSignToolHandler deactiveFillSign];
    }
    if ([UIMenuController sharedMenuController].isMenuVisible) {
        [[UIMenuController sharedMenuController] setMenuVisible:NO animated:YES];
    }
    
    FillSignToolHandler *toolHandler = self.fillSignToolHandler;
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    

    if (readToolbarItem.settingsType == FSReadToolSettingsTypeFillSignProfile) {
        FillSignProfileController *profileCtr = [[FillSignProfileController alloc] init];
        profileCtr.selectCall = ^(FillSignProfileController * _Nonnull ctr, NSString * _Nonnull selectStr) {
            self.selectedItem = readToolbarItem.toolbarItem.item;
            self.selectPreDefineStr = selectStr;
            if (!selectStr.length) {
                [self cancelSelectItem];
                if (self->_extensionsManager.currentToolHandler) {
                    self->_extensionsManager.currentToolHandler = nil;
                }
            }else{
                toolHandler.fillSignType = FSFillSignTypeProfile;
                [self->_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kFillClickPrompt")];
            }
        };
        profileCtr.updateCall = ^(FillSignProfileController * _Nonnull ctr, NSArray * _Nonnull contenArray) {
            self.preDefineContent = [NSArray arrayWithArray:contenArray];
        };
        
        FSNavigationController *nav = [[FSNavigationController alloc] initWithRootViewController:profileCtr];
        if (DEVICE_iPHONE){
            nav.modalPresentationStyle = UIModalPresentationFullScreen;
        }else{
            nav.modalPresentationStyle = UIModalPresentationFormSheet;
        }
        [_pdfViewCtrl.fs_viewController presentViewController:nav animated:YES completion:^{
            
        }];
    }else{
        switch (readToolbarItem.settingsType) {
            case FSReadToolSettingsTypeFillSignText:
                toolHandler.fillSignType = FSFillSignTypeText;
                break;
            case FSReadToolSettingsTypeFillSignSegText:
                toolHandler.fillSignType = FSFillSignTypeSegText;
                break;
            case FSReadToolSettingsTypeFillSignCheckMark:
                toolHandler.fillSignType = FSFillSignTypeCheck;
                break;
            case FSReadToolSettingsTypeFillSignCrossMark:
                toolHandler.fillSignType = FSFillSignTypeX;
                break;
            case FSReadToolSettingsTypeFillSignDot:
                toolHandler.fillSignType = FSFillSignTypeDot;
                break;
            case FSReadToolSettingsTypeFillSignLine:
                toolHandler.fillSignType = FSFillSignTypeLine;
                break;
            case FSReadToolSettingsTypeFillSignRoundRectangle:
                toolHandler.fillSignType = FSFillSignTypeRoundrect;
                break;
            default:
                break;
        }
    }
    
    
    FSUIElementType elementType = -1;
    switch (readToolbarItem.toolSettings.settingsType) {
        case FSReadToolSettingsTypeFillSignText:
            elementType = FSUIElementReadingFillSignTypeWriter;
            break;
        case FSReadToolSettingsTypeFillSignSegText:
            elementType = FSUIElementReadingFillSignComboText;
            break;
        case FSReadToolSettingsTypeFillSignProfile:
            elementType = FSUIElementReadingFillSignPredefined;
            break;
        case FSReadToolSettingsTypeFillSignCheckMark:
            elementType = FSUIElementReadingFillSignV;
            break;
        case FSReadToolSettingsTypeFillSignCrossMark:
            elementType = FSUIElementReadingFillSignX;
            break;
        case FSReadToolSettingsTypeFillSignDot:
            elementType = FSUIElementReadingFillSignDot;
            break;
        case FSReadToolSettingsTypeFillSignLine:
            elementType = FSUIElementReadingFillSignLine;
            break;
        case FSReadToolSettingsTypeFillSignRoundRectangle:
            elementType = FSUIElementReadingFillSignRectangle;
            break;
        case FSReadToolSettingsTypeDigitalSignature:
            elementType = FSUIElementReadingFillSignSign;
            break;
        default:
            break;
    }
    
    [_extensionsManager onUIInteractElementClicked:elementType];
}

#pragma mark - IStateChangeListener

- (void)onStateChanged:(FSUIManagerState)state {}

- (void)onStateChanged:(FSUIManagerState)state lastState:(FSUIManagerState)lastState{
    if (state == FSUIManagerStateFillSign) {
        if (_extensionsManager.topToolbar.currentTagItem.toolTag != FSMainTopbarToolTagSign) {
            NSArray<FSMainTopbarToolTagItem*> *toolTagItems = _extensionsManager.topToolbar.tagItems;
            FSMainTopbarToolTagItem *signToolTagItem = nil;
            for (FSMainTopbarToolTagItem *toolTagItem in toolTagItems) {
                if (toolTagItem.toolTag == FSMainTopbarToolTagSign) {
                    signToolTagItem = toolTagItem;
                    break;
                }
            }
            if (signToolTagItem) {
                _extensionsManager.topToolbar.currentTagItem = signToolTagItem;
            }
        }
    }else if (state == FSUIManagerStateNormal && _extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag == FSMainTopbarToolTagSign) {
        [_extensionsManager changeState:FSUIManagerStateFillSign];
    }else {
        if (lastState == FSUIManagerStateFillSign) {
            [self cancelSelectItem];
            
            if (_extensionsManager.currentToolHandler) {
                [_extensionsManager setCurrentToolHandler:nil];
            }
            [self deactiveFillSign];
        }
    }
}

#pragma mark <IDocEventListener>
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    
    if (document) {
        [self updateFillSignItm];
    }
}

- (void)onDocWillClose:(FSPDFDoc *)document
{
    [self deactiveFillSign];
}

#pragma mark <FSMainTopbarEventListener>
- (void)didSetCurrentTagItem:(FSMainTopbarToolTagItem *)curTagItem lastTagItem:(FSMainTopbarToolTagItem *)lastTagItem{
    if (curTagItem != lastTagItem && curTagItem.toolTag == FSMainTopbarToolTagSign) {
        if (_extensionsManager.currentState != FSUIManagerStateFillSign) {
            [_extensionsManager changeState:FSUIManagerStateFillSign];
        }
    }else if(curTagItem != lastTagItem && lastTagItem.toolTag == FSMainTopbarToolTagSign){
        FillSignMenu *fillSignMenu = self.fillSignMenu;
        if (fillSignMenu.isMenuVisible) {
            [fillSignMenu setMenuVisible:NO animated:NO];
        }
        [_extensionsManager changeState:FSUIManagerStateNormal];
    }
}

- (void)cancelSelectItem
{
    if (self.selectedItem) {
        self.selectedItem.selected = NO;
        [self.selectedItem setBackgroundColor:[UIColor fs_clearColor]];
        self.selectedItem = nil;
        self.fillSignToolHandler.fillSignType = FSFillSignTypeUnknow;
    }
}

- (NSString *)getPreDefineStr{
    return self.selectPreDefineStr;
}

- (NSArray *)getPreDefineContentArray{
    return self.preDefineContent;
}

- (void)deactiveFillSign{
    [self.fillSignToolHandler deactiveFillSign];
}

#pragma mark -- IMoreMenuViewListener
- (void)onMoreMenuChanged:(BOOL)isHidden{
    if ([_extensionsManager getState] == FSUIManagerStateFillSign) {
        if (!isHidden) {
            [self deactiveFillSign];
        }
    }
}

#pragma mark -- IThumbnailEventListener
- (void)onThumbnailChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStateFillSign) {
        if (!isHidden) {
            [self deactiveFillSign];
        }
    }
}

#pragma mark - IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden{
    if ([_extensionsManager getState] == FSUIManagerStateFillSign) {
        if (!isHidden) {
            [self deactiveFillSign];
        }
    }
}

- (void)onShowViewSettings:(BOOL)isShow{
    if ([_extensionsManager getState] == FSUIManagerStateFillSign) {
        if (isShow) {
            [self deactiveFillSign];
        }
    }
}

#pragma mark - IDocModifiedEventListener
- (void)onDocModified:(FSPDFDoc *)doc{
    [self updateFillSignItm];
}

#pragma mark - IFSUndoEventListener
- (void)onWillUndo{
    [self deactiveFillSign];
}

- (void)onWillRedo{
    [self deactiveFillSign];
}

@end
