/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureModule+Private.h"
#import "DigitalSignatureAnnotHandler.h"
#import "Utility.h"
#import "UIButton+Extensions.h"
#import "UIExtensionsConfig.h"
#import "CerTrustedListViewCtr.h"
#import "MenuView+Private.h"
#import "SignToolHandler.h"
#import "FSPermissionUtilty.h"

@interface SignatureModule ()<MoreItemActionProtocol, IReadToolSettingsModule>

@property (nonatomic, assign) int oldState;

@property (nonatomic, strong) SignToolHandler *toolHandler;

@property (nonatomic, strong) FSReadToolbarItem *signatureItem;

@property (nonatomic, strong) MoreMenuItem *cerItem;
@property (nonatomic, strong) FSReadToolbarItem *cerReadToolbarItem;

@property (nonatomic, strong) DigitalSignatureAnnotHandler* annotHandler;
@end

@implementation SignatureModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return Module_Signature;
}

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        _toolHandler = [[SignToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerRotateChangedListener:_toolHandler];
        [_extensionsManager registerToolHandler:_toolHandler];
        [_extensionsManager.pdfViewCtrl registerDocEventListener:_toolHandler];
        [_extensionsManager.pdfViewCtrl registerScrollViewEventListener:_toolHandler];
        _annotHandler = [[DigitalSignatureAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_pdfViewCtrl registerScrollViewEventListener:_annotHandler];
        [_extensionsManager registerRotateChangedListener:_annotHandler];
        [_extensionsManager registerAnnotHandler:_annotHandler];
        [_pdfViewCtrl registerDocEventListener:self];
    }
    return self;
}

- (void)loadModule {
    [self initToolBar];
    
    [_extensionsManager registerToolEventListener:self];
    
    if (_extensionsManager.config.loadSignature) {
        self.signatureItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeDigitalSignature withItemContentType:FSItemContentTypeImageAndText];
        [_extensionsManager.toolbarManager addReadItem:self.signatureItem atToolTag:FSMainTopbarToolTagSign];
        MoreMenuGroup *group = [_extensionsManager.more getGroup:TAG_GROUP_PROTECT];
        if (!group) {
            group = [[MoreMenuGroup alloc] init];
            group.title = FSLocalizedForKey(@"kProtect");
            group.tag = TAG_GROUP_PROTECT;
            [_extensionsManager.more addGroup:group];
        }
        
        MoreMenuItem *cerItem = [[MoreMenuItem alloc] init];
        _cerItem = cerItem;
        cerItem.tag = TAG_ITEM_CERTIFICATE;
        cerItem.title = FSLocalizedForKey(@"kTrustedCert");
        cerItem.callBack = self;
        cerItem.image = FSImageNamedAndNormalColor(@"more_protect_cert_protect");
        [_extensionsManager.more addMenuItem:TAG_GROUP_PROTECT withItem:cerItem];
        
        _cerReadToolbarItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTrutedCertificates withItemContentType:FSItemContentTypeImageAndText];
        [_extensionsManager.toolbarManager addReadItem:_cerReadToolbarItem atToolTag:FSMainTopbarToolTagProtect];
    }
}

- (UIImage *)getSignatureViewImage{
    if (self.getSignatureViewImageBlock) {
        return self.getSignatureViewImageBlock();
    }
    return nil;
}

#pragma mark - MoreMenuItemAction
- (void)onClick:(MoreMenuItem *)item {
    [self openCerTrustedViewCtrl];
    if(item.tag == TAG_ITEM_CERTIFICATE) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreProtectTrustedCertificates];
    }
}

- (void)openCerTrustedViewCtrl{
    UIViewController *vc = [_extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeCerTrustedList];
    FSNavigationController *navigationCtrl = [[FSNavigationController alloc] initWithRootViewController:vc];
    if (DEVICE_iPAD) {
        navigationCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
    }else{
        navigationCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
    }
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:navigationCtrl animated:YES completion:nil];

}

- (void)initToolBar {
    FSNavigationBar *navbar = [[FSNavigationBar alloc] initWithStyle:FSToolbarStyleTop];
    navbar.rightItem.enabled = NO;
    _signatureViewTopBar = [[FSMainToolbar alloc] initWithToolbar:navbar];
}

- (void)cancelSignature {
    [self.toolHandler delete];
    [_extensionsManager setCurrentToolHandler:nil];
    [_extensionsManager changeState:FSUIManagerStateNormal];
}

#pragma mark IHandlerEventListener

- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName {
    if ([toolName isEqualToString:Tool_Signature]) {

    } else if ([lastToolName isEqualToString:Tool_Signature]) {
        if (_extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag != FSMainTopbarToolTagSign) {
            [_extensionsManager changeState:FSUIManagerStateNormal];
        }
    }
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    if (readToolbarItem.settingsType == FSReadToolSettingsTypeTrutedCertificates){
        [self onClick:self.cerItem];
    }else{
        self.toolHandler.toolSettings = readToolbarItem.toolSettings;
        [_extensionsManager setCurrentToolHandler:self.toolHandler];
        [self.toolHandler signList];
        
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingFillSignSign];
    }
}


#pragma mark <IDocEventListener>
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
//    if(_annotHandler.isAddCertSignature){
//        [_annotHandler gotoSignPage];
//    }
//    if(_toolHandler.isAddCertSignature){
//        [_toolHandler gotoSignPage];
//    }
    
    if (document) {
        self.signatureItem.enabled = [FSPermissionUtilty canAddSignToDocument:_extensionsManager] && [FSPermissionUtilty canAddSign:_extensionsManager] && ![_pdfViewCtrl isDynamicXFA] && [FSPermissionUtilty canFillForm:_extensionsManager] && ([_extensionsManager.permissionProvider checkPermission:FSFunctionPDFSign] != FSPermissionStateHide);
    }
}

@end
