/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFillFormHistoryDataView.h"
#import "FSHistoryDataCollectionCell.h"
#import "FSHistoryDataModule+Private.h"
#import "FormAnnotHandler.h"

@interface FSFillFormHistoryDataCollectionCell : FSHistoryDataCollectionCell
@property (nonatomic, strong) UIButton *deleteBtn;
@end
@implementation FSFillFormHistoryDataCollectionCell

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        [self addDeleteBtn];
        
    }
    return self;
}

- (void)addDeleteBtn{
    self.titleLabel.textColor = [UIColor fs_whiteColor];
    _deleteBtn = [UIButton new];
    _deleteBtn.accessibilityLabel = FSLocalizedForKey(@"kRemoveSuggestionBtnVoiceover");
    [_deleteBtn setImage:FSImageNamed(@"fill_form_history_item_delete") forState:UIControlStateNormal];
    [self.contentView addSubview:_deleteBtn];
    
    [self.titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.fs_equalToSuperview();
        make.left.equalTo(self).offset(TITLE_LEFT_ICON_MARGINT);
    }];
    
    [_deleteBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.fs_equalToSuperview();
        make.left.equalTo(self.titleLabel.mas_right).offset(TITLE_LEFT_ICON_MARGINT);
        make.right.equalTo(self).offset(-TITLE_LEFT_ICON_MARGINT);
    }];
}

@end

@interface FSFillFormHistoryDataView ()
@property (nonatomic, weak) FormAnnotHandler *formAnnotHandler;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSHistoryDataConfig *historyDataConfig;
@property (nonatomic, assign) BOOL beginEditing;
@property (nonatomic, copy) NSString *lastKeyword;
@end
@implementation FSFillFormHistoryDataView

- (instancetype)initWithFormAnnotHandler:(FormAnnotHandler *)formAnnotHandler{
    self = [super init];
    if (self) {
        _formAnnotHandler = formAnnotHandler;
        _extensionsManager = formAnnotHandler.extensionsManager;
        Class cls = [FSFillFormHistoryDataCollectionCell class];
        [self.collectionView registerClass:cls forCellWithReuseIdentifier:NSStringFromClass(cls)];
        self.dataSource = self.historyDataConfig.items;
        self.collectionView.backgroundColor = [UIColor fs_clearColor];
    }
    return self;
}

- (FSHistoryDataConfig *)historyDataConfig{
    if (!_historyDataConfig) {
        FSHistoryDataModule *historyDataModule = [_extensionsManager getModuleByName:Module_History_Data];
        _historyDataConfig = [historyDataModule configForScene:FSInteractiveSceneFillingForms];
    }
    return _historyDataConfig;
}

- (void)textDidBeginEditing:(NSString *)text{
    _beginEditing = YES;
    if (!self.historyDataConfig.items.count && !self.dataSource.count) return;
    [self updateDataSourceByKeyword:text];
}

- (void)textDidChange:(NSString *)text{
    if (!self.historyDataConfig.items.count && !self.dataSource.count) return;
    [self updateDataSourceByKeyword:text];
}
- (void)textDidEndEditing:(NSString *)text{
    if (!_beginEditing || !text.length) return;
    [self addTextItem:text];
}

- (void)updateDataSourceByKeyword:(NSString *)keyword{
    if (!keyword.length) {
        self.dataSource = @[];
        [self.collectionView reloadData];
        return;
    }
    _lastKeyword = keyword;
    self.dataSource = [self.historyDataConfig sortItemsByKeyword:keyword];
    [self.collectionView reloadData];
}

- (void)addTextItem:(NSString *)text{
    if (!text.length) return;
    NSMutableArray *items = self.historyDataConfig.items ? [self.historyDataConfig.items mutableCopy] : @[].mutableCopy;
    for (FSHistoryDataItem *item in items) {
        if ([item.content isEqualToString:text] ) {
            return;
        }
    }
    FSHistoryDataItem *item = [FSHistoryDataItem new];
    item.createDate = [NSDate date];
    item.lastUseDate = [NSDate date];
    item.content = text;
    [items addObject:item];
    self.historyDataConfig.items = items;
}

- (void)updateItemByText:(NSString *)text{
    if (!text.length) return;
    NSMutableArray *items = [self.historyDataConfig.items mutableCopy];
    for (FSHistoryDataItem *item in items) {
        if ([item.content isEqualToString:text] ) {
            item.lastUseDate = [NSDate date];
            self.dataSource = [self.historyDataConfig sortItemsByKeyword:text];
            [self.collectionView reloadData];
            break;
        }
    }
}

- (__kindof FSHistoryDataCollectionCell *)collectionView:(nonnull UICollectionView *)collectionView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(FSHistoryDataItem *)dataItem{
    Class cls = [FSFillFormHistoryDataCollectionCell class];
    FSFillFormHistoryDataCollectionCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:NSStringFromClass(cls) forIndexPath:indexPath];
    return cell;
}

- (void)configureCell:(FSFillFormHistoryDataCollectionCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSHistoryDataItem *)dataItem{
    [cell fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
    BindThemeKeyPath(cell, backgroundColor);
    cell.titleLabel.text = dataItem.content;
    cell.titleLabel.accessibilityLabel = [NSString stringWithFormat:@"%@ %@",cell.titleLabel.text, FSLocalizedForKey(@"kButtonVoiceover")];
    @fs_weakify(self)
    [cell.deleteBtn setBlockForControlEvents:UIControlEventTouchUpInside block:^(id  _Nonnull sender) {
        [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kRemoveSuggestionTitle") message:FSLocalizedFormatForKey(@"kRemoveSuggestionMessage", dataItem.content) actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            if (index == 1) {
                @fs_strongify(self)
                NSMutableArray *items = [self.historyDataConfig.items mutableCopy];
                for (FSHistoryDataItem *item in items) {
                    if (item == dataItem) {
                        [items removeObject:item];
                        self.historyDataConfig.items = items;
                        [self updateDataSourceByKeyword:self.lastKeyword];
                        break;
                    }
                }
            }
        } onCtrl:[self fs_currentSceneTopMostViewController]];
    }];
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section{
    return UIEdgeInsetsMake(6, 8, 6, 8);
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    FSHistoryDataItem *item = [self dataForIndexPath:indexPath];
    CGSize size = [Utility getTextSize:item.content fontSize:[self.fs_standardGeneralFont_GF2 pointSize] maxSize:CGSizeMake(CGFLOAT_MAX, [self.fs_standardGeneralFont_GF2 pointSize] * 2)];
    return CGSizeMake(size.width + 40, 32.f);
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    FSHistoryDataItem *item = [self dataForIndexPath:indexPath];
    [_formAnnotHandler setTextToCurrentTextFiled:item.content];
    [self updateItemByText:item.content];
}

@end
