/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FormModule.h"
#import "FormToolHandler.h"
#import "FormAnnotHandler.h"
#import "MenuView+Private.h"
#import "FSFileSelectDestinationViewController.h"
#import "FSPermissionUtilty.h"

@interface FormModule ()<IStateChangeListener, IFSMainTopbarEventListener> {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    UIButton *currentItemBtn;
}
@property (nonatomic, strong) MoreMenuGroup *group;
@property (nonatomic, weak) MoreMenuView *moreMenu;
@property (nonatomic, strong) MoreMenuItem *createFormItem;
@property (nonatomic, strong) MoreMenuItem *exportFormItem;
@property (nonatomic, strong) MoreMenuItem *importFormItem;
@property (nonatomic, strong) MoreMenuItem *resetFormItem;
@property (nonatomic, strong) FSReadToolbarItem* textFieldReadItem;
@property (nonatomic, strong) FSReadToolbarItem* checkBoxReadItem;
@property (nonatomic, strong) FSReadToolbarItem* radioReadItem;
@property (nonatomic, strong) FSReadToolbarItem* comBoxReadItem;
@property (nonatomic, strong) FSReadToolbarItem* listBoxReadItem;
@property (nonatomic, strong) FSReadToolbarItem* signatureReadItem;
@property (nonatomic, strong) FSReadToolbarItem *imageFieldReadItem;
@end

@implementation FormModule

- (NSString *)getName {
    return @"Form";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];

        FormAnnotHandler *annotHandler = [[FormAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];
        FormToolHandler *toolHandler = [[FormToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        [_extensionsManager registerRotateChangedListener:annotHandler];
        [_extensionsManager registerStateChangeListener:self];
        
        [_pdfViewCtrl registerDocEventListener:annotHandler];
        [_pdfViewCtrl registerDocEventListener:self];
        [_extensionsManager.toolbarManager registerTopbarEventListener:self];
    }
    return self;
}

- (void)loadModule {
    _textFieldReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTextField withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_textFieldReadItem atToolTag:FSMainTopbarToolTagForm];
    _checkBoxReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeCheckBox withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_checkBoxReadItem atToolTag:FSMainTopbarToolTagForm];
    _radioReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeRadioButton withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_radioReadItem atToolTag:FSMainTopbarToolTagForm];
    _comBoxReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeComboBox withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_comBoxReadItem atToolTag:FSMainTopbarToolTagForm];
    _listBoxReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeListBox withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_listBoxReadItem atToolTag:FSMainTopbarToolTagForm];
    _signatureReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeSignatureField withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_signatureReadItem atToolTag:FSMainTopbarToolTagForm];
    
    _imageFieldReadItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeImageField withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_imageFieldReadItem atToolTag:FSMainTopbarToolTagForm];

    [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager registerAnnotEventListener:self];
    
    self.moreMenu = _extensionsManager.more;
    self.group = [self.moreMenu getGroup:TAG_GROUP_COMMENT_FIELD];
    if (!self.group) {
        self.group = [[MoreMenuGroup alloc] init];
        self.group.title = FSLocalizedForKey(@"kComments&Fields");
        self.group.tag = TAG_GROUP_COMMENT_FIELD;
        [self.moreMenu addGroup:self.group];
    }
    
    self.exportFormItem = [[MoreMenuItem alloc] init];
    self.exportFormItem.tag = TAG_ITEM_EXPORTFORM;
    self.exportFormItem.callBack = self;
    self.exportFormItem.title = FSLocalizedForKey(@"kExportForm");
    self.exportFormItem.image = FSImageNamedAndNormalColor(@"more_form_export_form");
    [self.moreMenu addMenuItem:self.group.tag withItem:self.exportFormItem];

    self.importFormItem = [[MoreMenuItem alloc] init];
    self.importFormItem.tag = TAG_ITEM_IMPORTFORM;
    self.importFormItem.callBack = self;
    self.importFormItem.title = FSLocalizedForKey(@"kImportForm");
    self.importFormItem.image = FSImageNamedAndNormalColor(@"more_form_import_form");
    [self.moreMenu addMenuItem:self.group.tag withItem:self.importFormItem];

    self.resetFormItem = [[MoreMenuItem alloc] init];
    self.resetFormItem.tag = TAG_ITEM_RESETFORM;
    self.resetFormItem.callBack = self;
    self.resetFormItem.title = FSLocalizedForKey(@"kResetFormFields");
    self.resetFormItem.image = FSImageNamedAndNormalColor(@"more_form_reset_form");
    [self.moreMenu addMenuItem:self.group.tag withItem:self.resetFormItem];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateItemState) name:NSNotificationNameFlattenDoc object:nil];
}

- (void)onClick:(MoreMenuItem *)item {
    
    [self operateFromModule:item];
    
    FSUIElementType elementType = -1;
    if (item.tag == TAG_ITEM_EXPORTFORM) {
        elementType = FSUIElementReadingMoreCommentFieldExportFormFields;
    }
    else if(item.tag == TAG_ITEM_IMPORTFORM) {
        elementType = FSUIElementReadingMoreCommentFieldImportFormFields;
    }
    else if(item.tag == TAG_ITEM_RESETFORM) {
        elementType = FSUIElementReadingMoreCommentFieldResetFormFields;
    }
    
    [_extensionsManager onUIInteractElementClicked:elementType];
}

- (void)operateFromModule:(MoreMenuItem *)item{
    BOOL isDynamicXFA = [_pdfViewCtrl isDynamicXFA];
    
    if (!isDynamicXFA && ![_pdfViewCtrl.currentDoc hasForm]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kNoFormAvailable") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    NSFileManager *filemanager = [NSFileManager defaultManager];

    BOOL canFillForm = [FSPermissionUtilty canFillForm:_extensionsManager];
    
    if (item.tag == TAG_ITEM_EXPORTFORM) {
        if (!([_extensionsManager.permissionProvider checkPermission:FSFunctionCanSaveToLocal] == FSPermissionStateShow)) {
            return;
        }
        
        FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:_extensionsManager.viewCtrlManager];
        selectDestination.isRootFileDirectory = YES;
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
        selectDestination.expectFileType = @[@"*"];
        [selectDestination loadFilesWithPath:DOCUMENT_PATH];
        selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
            __block void (^inputFileName)(void) = ^() {
                
                NSString *titleStr = [[self->_extensionsManager.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
                
                [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kInputNewFileName") message:@"" textFieldText:titleStr actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
                        
                    if (index == 1) {
                        NSString *fileName = textField.text;

                        if ([fileName rangeOfString:@"/"].location != NSNotFound) {
                            dispatch_async(dispatch_get_main_queue(), ^{
                                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kIllegalNameWarning") preferredStyle:UIAlertControllerStyleAlert];
                                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                                       style:UIAlertActionStyleCancel
                                                                                     handler:^(UIAlertAction *action) {
                                                                                         dispatch_async(dispatch_get_main_queue(), ^{
                                                                                             inputFileName();
                                                                                         });
                                                                                         return;
                                                                                     }];
                                [alertController addAction:cancelAction];
                                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                            });
                            return;
                        } else if (fileName.length == 0) {
                            dispatch_async(dispatch_get_main_queue(), ^{
                                inputFileName();
                            });
                            return;
                        }

                        void (^createXML)(NSString *xmlFilePath) = ^(NSString *xmlFilePath) {
                            NSString *tmpFilePath = [TEMP_PATH stringByAppendingPathComponent:[xmlFilePath lastPathComponent]];
                            BOOL isSuccess = NO;
                            if (isDynamicXFA) {
                                FSXFADoc *xfadoc = [self->_pdfViewCtrl getXFADoc];
                                @try {
                                    isSuccess = [xfadoc exportData:tmpFilePath export_type:FSXFADocExportDataTypeXML];
                                } @catch (NSException *exception) {
                                    NSLog(@"ExportToXML EXCEPTION NAME:%@", exception.name);
                                }
                            }else{
                                FSForm *form = [[FSForm alloc] initWithDocument:self->_pdfViewCtrl.currentDoc];
                                if (nil == [form getCptr])
                                    return;

                                @try {
                                    isSuccess = [form exportToXML:tmpFilePath];
                                } @catch (NSException *exception) {
                                    NSLog(@"ExportToXML EXCEPTION NAME:%@", exception.name);
                                }
                            }

                            if (isSuccess) {
                                double delayInSeconds = 0.4;
                                dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                                dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {

                                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kExportFormSuccess") preferredStyle:UIAlertControllerStyleAlert];
                                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                                    [alertController addAction:action];
                                    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                                    [filemanager moveItemAtPath:tmpFilePath toPath:xmlFilePath error:nil];
                                });
                            } else {
                                double delayInSeconds = 0.4;
                                dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                                dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
                                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kExportFormFailed") preferredStyle:UIAlertControllerStyleAlert];
                                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                                    [alertController addAction:action];
                                    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                                });
                            }

                        };

                        NSString *xmlFilePath = [destinationFolder[0] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.xml", fileName]];
                        NSFileManager *fileManager = [[NSFileManager alloc] init];
                        if ([fileManager fileExistsAtPath:xmlFilePath]) {
                            double delayInSeconds = 0.3;
                            dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                            dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
                                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                                       style:UIAlertActionStyleCancel
                                                                                     handler:^(UIAlertAction *action) {
                                                                                         dispatch_async(dispatch_get_main_queue(), ^{
                                                                                             inputFileName();
                                                                                         });
                                                                                     }];
                                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kReplace")
                                                                                 style:UIAlertActionStyleDefault
                                                                               handler:^(UIAlertAction *action) {
                                                                                   inputFileName = nil;
                                                                                   [fileManager removeItemAtPath:xmlFilePath error:nil];
                                                                                   createXML(xmlFilePath);
                                                                               }];
                                [alertController addAction:cancelAction];
                                [alertController addAction:action];
                                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                            });
                        } else {
                            inputFileName = nil;
                            createXML(xmlFilePath);
                        }

                    }
                        [controller dismissViewControllerAnimated:YES completion:nil];

                        
                }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
                    if (index == 1) {
                        return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
                      }
                    return YES;
                }] fs_showByViewController:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]] ;

            };

            [controller dismissViewControllerAnimated:YES completion:^{
                inputFileName();
            }];
        };
        FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
        [_extensionsManager.viewCtrlManager presentViewController:selectDestinationNavController
                                         animated:YES
                                       completion:nil];
    } else if (item.tag == TAG_ITEM_IMPORTFORM) {
        if (!canFillForm) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            return;
        }

        FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:_extensionsManager.viewCtrlManager];
//        FileSelectDestinationViewController *selectDestination = [[FileSelectDestinationViewController alloc] init];
        selectDestination.isRootFileDirectory = YES;
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFile;
        selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeImport;
        if (isDynamicXFA) {
            selectDestination.expectFileType = @[ @"xml", @"xdp"];
        }else{
            selectDestination.expectFileType = @[ @"xml", @"fdf"];
        }
        [selectDestination loadFilesWithPath:DOCUMENT_PATH];

        selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
            [controller dismissViewControllerAnimated:YES completion:^{
                if (destinationFolder.count > 0) {
                    BOOL isSuccess = NO;
                    if (isDynamicXFA) {
                        FSXFADoc *xfaDoc = [self->_pdfViewCtrl getXFADoc];
                        @try {
                            isSuccess = [xfaDoc importData:destinationFolder[0]];
                        } @catch (NSException *exception) {
                            NSLog(@"xfaDoc Importdata EXCEPTION NAME:%@", exception.name);
                        }
                    }else{
                        FSForm *form = [[FSForm alloc] initWithDocument:self->_pdfViewCtrl.currentDoc];
                        if (nil == [form getCptr] || [form isEmpty])
                            return;
                        
                        @try {
                            if ([[destinationFolder[0] pathExtension] isEqualToString:@"fdf"]){
                                FSFDFDoc *importFdoc = [[FSFDFDoc alloc] initWithPath:destinationFolder[0]];
                                isSuccess = [self->_pdfViewCtrl.currentDoc importFromFDF:importFdoc types:FSPDFDocForms];
                            }else{
                                isSuccess = [form importFromXML:destinationFolder[0]];
                            }
                        } @catch (NSException *exception) {
                            NSLog(@"ImportFromXML EXCEPTION NAME:%@", exception.name);
                        }
                    }
                    
                    if (isSuccess) {
                        self->_extensionsManager.isDocModified = YES;
                        double delayInSeconds = 0.3;
                        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                        dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
                            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kImportFormSuccess") preferredStyle:UIAlertControllerStyleAlert];
                            __weak UIExtensionsManager *weakExtMgr = self->_extensionsManager;
                            
                            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
                                if (isDynamicXFA) {
                                    int page_index = -1;
                                    [weakExtMgr.pdfViewCtrl reloadXFADoc:FSDocProviderCallbackPageViewEventTypeAdded page_index:@[@(page_index)]];
                                }
                            }];
                            [alertController addAction:action];
                            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController]  presentViewController:alertController animated:YES completion:nil];
                        });
                    } else {
                        double delayInSeconds = 0.3;
                        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                        dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
                            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kImportFormFailed") preferredStyle:UIAlertControllerStyleAlert];
                            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                            [alertController addAction:action];
                            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController]  presentViewController:alertController animated:YES completion:nil];
                        });
                    }
                }
            }];
            
        };

        FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
        [_extensionsManager.viewCtrlManager presentViewController:selectDestinationNavController animated:YES completion:nil];
    } else if (item.tag == TAG_ITEM_RESETFORM) {
        if (!canFillForm) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            return;
        }

        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kSureToResetFormFields") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo") style:UIAlertActionStyleCancel handler:nil];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                         style:UIAlertActionStyleDefault
                                                       handler:^(UIAlertAction *action) {
                                                           if (isDynamicXFA) {
                                                               FSXFADoc *xfadoc = [self->_pdfViewCtrl getXFADoc];
                                                               [xfadoc resetForm];
                                                           }else{
                                                               FSForm *form = [[FSForm alloc] initWithDocument:self->_pdfViewCtrl.currentDoc];
                                                               if (nil == [form getCptr])
                                                                   return;
                                                               @try {
                                                                    [form reset];

                                                               } @catch (NSException *exception) {
                                                               }
                                                           }
                                                           self->_extensionsManager.isDocModified = YES;
                                                           [self->_extensionsManager.thumbnailModule clearThumbnailCachesForPDFAtPath:self->_pdfViewCtrl.filePath];
                                                       }];
        [alertController addAction:cancelAction];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    }
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem{
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionEdit checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            if (self->_extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag != FSMainTopbarToolTagForm) {
                [self->_extensionsManager changeState:FSUIManagerStateAnnotTool];
            }
            FormToolHandler *toolHandler = [self->_extensionsManager getToolHandlerByName:Tool_Form];
            toolHandler.toolSettings = readToolbarItem.toolSettings;
            toolHandler.continueAdd = YES;
            [self->_extensionsManager setCurrentToolHandler:toolHandler];
        }else{
            readToolbarItem.toolbarItem.item.selected = NO;
        }
    }];
    
    
    FSUIElementType elementType = -1;
    switch (readToolbarItem.toolSettings.settingsType) {
        case FSReadToolSettingsTypeTextField:
            elementType = FSUIElementReadingFormBarTextField;
            break;
        case FSReadToolSettingsTypeCheckBox:
            elementType = FSUIElementReadingFormBarCheckBox;
            break;
        case FSReadToolSettingsTypeRadioButton:
            elementType = FSUIElementReadingFormBarRadioButton;
            break;
        case FSReadToolSettingsTypeComboBox:
            elementType = FSUIElementReadingFormBarComboBox;
            break;
        case FSReadToolSettingsTypeListBox:
            elementType = FSUIElementReadingFormBarListBox;
            break;
        case FSReadToolSettingsTypeSignatureField:
            elementType = FSUIElementReadingFormBarSignatureField;
            break;
        case FSReadToolSettingsTypeImageField:
            elementType = FSUIElementReadingFormBarImageField;
            break;
        default:
            break;
    }
    
    [_extensionsManager onUIInteractElementClicked:elementType];
    
}

- (void)resetMoreViewItem{
    
    if ([_pdfViewCtrl isDynamicXFA] ) {
        if ([_extensionsManager allAnnotPermission] && _extensionsManager.enableModifyDocument) {
            self.importFormItem.enable = YES;
            self.resetFormItem.enable = YES;
        }
        self.exportFormItem.enable = YES;
    }else if(![_pdfViewCtrl.currentDoc isEmpty]){
        BOOL canFillForm = [FSPermissionUtilty canFillForm:_extensionsManager];
        
        if (!canFillForm || ![_pdfViewCtrl.currentDoc hasForm] || !_extensionsManager.enableModifyDocument) {
            self.importFormItem.enable = NO;
            self.resetFormItem.enable = NO;
        }else{
            self.importFormItem.enable = YES;
            self.resetFormItem.enable = YES;
        }
        
        if (![_pdfViewCtrl.currentDoc hasForm]) {
            self.exportFormItem.enable = NO;
        }else{
            self.exportFormItem.enable = YES;
        }
    }
}

- (void)updateItemState{
    [self resetMoreViewItem];
}

#pragma mark - IStateChangeListener

- (void)onStateChanged:(FSUIManagerState)state {}
- (void)onStateChanged:(FSUIManagerState)state lastState:(FSUIManagerState)lastState{
    if (state == FSUIManagerStateCreateForm) {
        
    }else if (state == FSUIManagerStateNormal && _extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag == FSMainTopbarToolTagForm) {
        [_extensionsManager changeState:FSUIManagerStateCreateForm];
    }else {
        if (_extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag != FSMainTopbarToolTagForm && lastState == FSUIManagerStateCreateForm && _extensionsManager.fullScreenToolbarMgr.floatyToolbar.hidden) {
            if (_extensionsManager.currentToolHandler) {
                [_extensionsManager setCurrentToolHandler:nil];
            }
        }
    }
}

#pragma mark <FSMainTopbarEventListener>
- (void)didSetCurrentTagItem:(FSMainTopbarToolTagItem *)curTagItem lastTagItem:(FSMainTopbarToolTagItem *)lastTagItem{
    if (curTagItem != lastTagItem && curTagItem.toolTag == FSMainTopbarToolTagForm) {
        [_extensionsManager changeState:FSUIManagerStateCreateForm];
    }else if(curTagItem != lastTagItem && lastTagItem.toolTag == FSMainTopbarToolTagForm){
        [_extensionsManager changeState:FSUIManagerStateNormal];

    }
}

#pragma mark <IDocEventListener>
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    if (document) {
        BOOL enabled = ![FSPermissionUtilty canNotEnterForm:_extensionsManager];
        _textFieldReadItem.enabled = enabled;
        _checkBoxReadItem.enabled = enabled;
        _radioReadItem.enabled = enabled;
        _comBoxReadItem.enabled = enabled;
        _signatureReadItem.enabled = enabled;
        _listBoxReadItem.enabled = enabled;
        _imageFieldReadItem.enabled = enabled;
        if (![_extensionsManager allowFormPermission]) {
            _textFieldReadItem.enabled = NO;
            _checkBoxReadItem.enabled = NO;
            _radioReadItem.enabled = NO;
            _comBoxReadItem.enabled = NO;
            _signatureReadItem.enabled = NO;
            _listBoxReadItem.enabled = NO;
            _imageFieldReadItem.enabled = NO;
        }
    }
    
    [self resetMoreViewItem];
}

#pragma mark --IPageEventListener
- (void)onPagesInsertedAtRange:(NSRange)range{
    [self resetMoreViewItem];
}

#pragma mark --IAnnotEventListener
- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot{
    [self resetMoreViewItem];
}

- (void)onAnnotsAdded:(NSArray<FSAnnot *> *)annots{
    [self resetMoreViewItem];
}

- (void)onAnnotDeleted:(FSPDFPage *)page annot:(FSAnnot *)annot{
    [self resetMoreViewItem];
}

-(void)onAnnotFlattened:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self resetMoreViewItem];
}

@end
