/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FormToolHandler.h"
#import "PaintUtility.h"

@interface FormToolHandler ()<IStateChangeListener>
@property (nonatomic, strong) FSAnnot *annot;
@property (nonatomic, strong) FSPointF *startPoint;
@property (nonatomic, strong) FSPointF *endPoint;
@property (nonatomic, assign) BOOL isEndDrawing;
@property (nonatomic, assign) int currentPageIndex;

@property (nonatomic, strong) PaintUtility *paintUtility;
@property (nonatomic, strong) NSMutableDictionary *tempPointsDict;

@property (nonatomic, copy) NSString *radioGroupName;
@end

@implementation FormToolHandler{
    UIExtensionsManager * __weak _extensionsManager;
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        self.type = FSAnnotWidget;
        self.tempPointsDict = @{}.mutableCopy;
        [extensionsManager registerStateChangeListener:self];
    }
    return self;
}

- (BOOL)isEnabled {
    return YES;
}

- (FSFieldType)fieldType{
    switch (self.toolSettings.settingsType) {
        case FSReadToolSettingsTypeTextField:
            return FSFieldTypeTextField;
        case FSReadToolSettingsTypeCheckBox:
            return FSFieldTypeCheckBox;
        case FSReadToolSettingsTypeRadioButton:
            return FSFieldTypeRadioButton;
        case FSReadToolSettingsTypeComboBox:
            return FSFieldTypeComboBox;
        case FSReadToolSettingsTypeListBox:
            return FSFieldTypeListBox;
        case FSReadToolSettingsTypeSignatureField:
            return FSFieldTypeSignature;
        case FSReadToolSettingsTypeImageField:
            return FSFieldTypePushButton;
        default:
            break;
    }
    return FSFieldTypeUnknown;
}

- (void)setContinueAdd:(BOOL)continueAdd{
    _continueAdd = continueAdd;
    self.radioGroupName = nil;
}

- (void)onActivate {
    _isEndDrawing = NO;
    self.annot = nil;
    
    self.tempPointsDict = @{}.mutableCopy;
}

- (void)onDeactivate {
    _isEndDrawing = NO;
    if (self.annot) {
        FSAnnot *annot = self.annot;
        [_pdfViewCtrl lockRefresh];
        [self updateAnnotRect];
        [annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        [_pdfViewCtrl refresh:self.annot.pageIndex];
    }
    self.annot = nil;
    [self.tempPointsDict removeAllObjects];
    self.tempPointsDict = nil;
}

-(void) updateAnnotRect {
    if(self.startPoint && self.endPoint)
        self.annot.fsrect = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

- (NSString *)getName {
    return Tool_Form;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    self.startPoint = nil;
    self.endPoint = nil;
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    
    [self.tempPointsDict setObject:@(point) forKey:@(pageIndex)];
    
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSRectF *dibRect = nil;
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.currentPageIndex = pageIndex;
        _isEndDrawing = NO;
        
        dibRect = [[FSRectF alloc] init];
        [dibRect setLeft:dibPoint.x];
        [dibRect setBottom:dibPoint.y];
        [dibRect setRight:dibPoint.x + 0.1];
        [dibRect setTop:dibPoint.y + 0.1];
        
        self.startPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        self.endPoint   = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        
        if (self.annot) {
            id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:self.type];
            @try {
                [self updateAnnotRect];
                [annotHandler addAnnot:self.annot addUndo:YES];
            } @catch (NSException *exception) {
            } @finally {
                self.annot = nil;
            }
        }
        self.annot = [self addAnnotToPage:pageIndex withRect:dibRect];
        if (!self.annot) {
            return YES;
        }
            
        self.startPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        self.endPoint   = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        
        FSRectF *lineFSRect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:self.annot.lineWidth top1:self.annot.lineWidth];
        CGRect lineRect = [_pdfViewCtrl convertPdfRectToPageViewRect:lineFSRect pageIndex:pageIndex];
        float lineWidth = lineRect.size.width;
        
        self.paintUtility = [[PaintUtility alloc] init];
        self.paintUtility.annotLineWidth = lineWidth;
        
        if (!_extensionsManager.enableHighlightForm) {
            self.paintUtility.annotColor = _extensionsManager.config.defaultSettings.highlightFormColor.rgbValue;
            self.paintUtility.annotOpacity = [_extensionsManager.config.defaultSettings.highlightFormColor alpha];
        }else{
            self.paintUtility.annotColor = _extensionsManager.highlightFormColor.rgbValue;
            self.paintUtility.annotOpacity = [_extensionsManager.highlightFormColor alpha];
        }
        
        
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (pageIndex == self.currentPageIndex) {
            float marginX = 2;
            float marginY = 2;
            UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
            if (point.x < marginX) {
                point.x = marginX;
            }else if (point.x > (pageView.bounds.size.width - marginX)) {
                point.x = pageView.bounds.size.width - marginX;
            }
            
            if (point.y < marginY) {
                point.y = marginY;
            }else if (point.y > (pageView.bounds.size.height - marginY)) {
                point.y = pageView.bounds.size.height - marginY;
            }
            self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
            [_pdfViewCtrl refresh:pageIndex needRender:NO];
        }
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        if (pageIndex != self.annot.pageIndex) {
            point = [self.tempPointsDict[@(self.annot.pageIndex)] CGPointValue];
            pageIndex = self.annot.pageIndex;
        }
        float marginX = [Utility getAnnotMinXMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
        float marginY = [Utility getAnnotMinYMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
        
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        
        if (point.y < marginY) {
            point.y = marginY;
        }
        
        if (point.y > pageview.bounds.size.height - marginY) {
            point.y = pageview.bounds.size.height - marginY;
        }
        
        if (point.x < marginX) {
            point.x = marginX;
        }
        
        if (point.x > pageview.bounds.size.width - marginX) {
            point.x = pageview.bounds.size.width - marginX;
        }
        
        self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        dibRect = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
        
        [_pdfViewCtrl lockRefresh];
        self.annot.fsrect = dibRect;
        [_pdfViewCtrl unlockRefresh];
        
        if (self.type == FSAnnotWidget) {
            if (self.annot.fsrect.top - self.annot.fsrect.bottom < 5.0 || self.annot.fsrect.right - self.annot.fsrect.left < 5.0) {
                FSAnnot* annot = self.annot;
                [self removeAnnot:annot];
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [self->_pdfViewCtrl refresh:pageIndex needRender:YES];
                });
                return NO;
            }
        }
        if (self.annot) {
            [_pdfViewCtrl lockRefresh];
            [self.annot resetAppearanceStream];
            [_pdfViewCtrl unlockRefresh];
            id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:self.type];
            @try {
                [annotHandler addAnnot:self.annot addUndo:YES];
            } @catch (NSException *exception) {
            } @finally {
                self.annot = nil;
            }
        }
        _isEndDrawing = YES;
        return YES;
    }
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (void)removeAnnot:(FSAnnot *)annot {
    if ([annot isEmpty] || !annot) {
        return;
    }
    FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
    FSField *field = [widget getField];
    FSForm *form = [[FSForm alloc] initWithDocument:_pdfViewCtrl.currentDoc];
    @try {
        [form removeField:field];
    } @catch (NSException *exception) {
    }
    self.annot = nil;
    self.startPoint = nil;
    self.endPoint = nil;
}

- (FSAnnot *)addAnnotToPage:(int)pageIndex withRect:(FSRectF *)rect {
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty])
        return nil;
    FSForm *form = [[FSForm alloc] initWithDocument:_pdfViewCtrl.currentDoc];
    FSControl *control = nil;
    FSWidget *widget = nil;
    if (self.fieldType == FSFieldTypeSignature) {
        FSSignature *signature = [page addSignature:rect];
        control = [signature getControl:0];
        if (![control isEmpty]) {
            widget = [control getWidget];
        }
    }else{
        NSString *name;
        if (self.fieldType == FSFieldTypeRadioButton) {
            if (self.continueAdd && self.radioGroupName) {
                name = self.radioGroupName;
            }else{
                name = [Utility formAddControlFieldName:form forType:self.fieldType isImageField:NO];
                self.radioGroupName = name;
            }
        }else{
            name = [Utility formAddControlFieldName:form forType:self.fieldType isImageField:self.toolSettings.settingsType == FSReadToolSettingsTypeImageField];
        }
        control = [form addControl:page field_name:name field_type:self.fieldType rect:rect];
        widget = [control getWidget];
        if ([widget isEmpty]) return nil;
        FSField *field = [widget getField];
        FSDefaultAppearance *appearance = [control getDefaultAppearance];
        NSString *fontName = nil;
        if (self.fieldType == FSFieldTypeTextField) {
            appearance.text_color = self.toolSettings.settingsColor;
            appearance.text_size =  self.toolSettings.textSize;
            fontName = self.toolSettings.textFace;
        }else if (self.fieldType == FSFieldTypeCheckBox){
            appearance.text_color = self.toolSettings.settingsColor;
        }else if (self.fieldType == FSFieldTypeRadioButton) {
            appearance.text_color = self.toolSettings.settingsColor;
        }else if (self.fieldType == FSFieldTypeComboBox) {
            appearance.text_color = self.toolSettings.settingsColor;
            appearance.text_size = self.toolSettings.textSize;
            fontName = self.toolSettings.textFace;
            field.flags |= self.toolSettings.customText ? FSFieldFlagComboEdit : 0;
        }else if (self.fieldType == FSFieldTypeListBox) {
            appearance.text_color = self.toolSettings.settingsColor;
            appearance.text_size = self.toolSettings.textSize;
            fontName = self.toolSettings.textFace;
            field.flags |= self.toolSettings.multipleSelection ? FSFieldFlagChoiceMultiSelect : 0;
        }else if (self.fieldType == FSFieldTypePushButton) {
            [widget setMKBackgroundColor:0xFFFFFF];
            unsigned int rgb = ((unsigned int)(0.752941f * 255)) | (((unsigned int)(0.752941f * 255)) << 8) | (((unsigned int)(0.752941f * 255)) << 16);
            [widget setMKBorderColor:rgb];
            FSIconFit *iconFit = [FSIconFit new];
            iconFit.fit_bounds = NO;
            iconFit.horizontal_fraction = 0.5f;
            iconFit.vertical_fraction = 0.5f;
            iconFit.is_proportional_scaling = YES;
            iconFit.scale_way_type = FSIconFitScaleWayAlways;
            [widget setMKIconFit:iconFit];
            [widget setMKIconCaptionRelation:FSAnnotMKRelationNoCaption];
            
            UIImage *placeholderImage = FSImageNamed(@"read_tool_image_field_picture");
            FSImage *image = [Utility uiimageToFSImage:placeholderImage];
            [widget setMKNormalIconImage:image frame_index:0];
            
            FSBorderInfo *bi = [FSBorderInfo new];
            bi.style = FSBorderInfoSolid;
            bi.width = 1.0f;
            [widget setBorderInfo:bi];
            appearance.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
            appearance.font = [[FSFont alloc] initWithFont_id:FSFontStdIDHelvetica];
            appearance.text_size = 0;
            appearance.text_color = 0;

            FSAction *action = [FSAction create:_pdfViewCtrl.currentDoc action_type:FSActionTypeJavaScript];
            FSJavaScriptAction *jsaction = [[FSJavaScriptAction alloc] initWithAction:action];
            [jsaction setScript:@"event.target.buttonImportIcon()"];
            [widget setAction:jsaction];
            [widget resetAppearanceStream];
        }
        if (appearance.text_size) {
            appearance.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
        }
        if (fontName) {
            FSFont *font = nil;
            int fontID = [Utility toStandardFontID:fontName];
            if (fontID == -1) {
                font = [[FSFont alloc] initWithName:fontName styles:0 charset:FSFontCharsetDefault weight:0];
            } else {
                font = [[FSFont alloc] initWithFont_id:fontID];
            }
            appearance.font = font;
        }

        [control setDefaultAppearance:appearance];
        
    }
    widget.NM = [NSString stringWithFormat:@"FoxitPDFSDK_%@",[Utility getUUID]];
    widget.author = _extensionsManager.annotAuthor;
    widget.opacity = self.toolSettings.opacity;
    widget.createDate = [NSDate date];
    widget.modifiedDate = [NSDate date];
    widget.flags = FSAnnotFlagPrint;
    widget.mKRotation = ([page getRotation] + [_pdfViewCtrl getViewRotation])%4;
    return widget;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (_extensionsManager.currentToolHandler != self) {
        return;
    }
    if (!self.startPoint || !self.endPoint || self.currentPageIndex != pageIndex) {
        return;
    }

    CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:pageIndex];
    CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:pageIndex];
    
    CGRect rect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
    if (_isEndDrawing) {
        CGContextClearRect(context, rect);
        return;
    }
    [self.paintUtility drawMultipleSelectWithRect:rect inContext:context];
}

- (CGRect)getRectFromStartPoint:(CGPoint)startPoint endPoint:(CGPoint)endPoint {
    CGFloat xminValue = MIN(startPoint.x, endPoint.x);
    CGFloat yminValue = MIN(startPoint.y, endPoint.y);
    
    CGFloat xmaxValue = MAX(startPoint.x, endPoint.x);
    CGFloat ymaxValue = MAX(startPoint.y, endPoint.y);
    
    CGRect result = CGRectMake(xminValue, yminValue, fabs(xmaxValue-xminValue), fabs(ymaxValue-yminValue));
    return result;
}

#pragma mark - IStateChangeListener

- (void)onStateChanged:(FSUIManagerState)state{

}

@synthesize toolSettings;

@end
