/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FtAnnotHandler.h"
#import "FSAnnotAttributes.h"
#import "MenuItem.h"
#import "ShapeUtil.h"
#import "StringDrawUtil.h"
#import "FreetextUtil.h"
#import "FSEasyDoneTextView.h"
#import "FreetextModule.h"
#import "FSPermissionUtilty.h"
#import "PaintUtility.h"
#import "CalloutToolHandler.h"

@interface FtAnnotHandler () <UITextViewDelegate, IRotationEventListener, IWindowFrameChangedListener,IPageEventListener,IAnnotHandler,IScrollViewEventListener, IPropertyViewEventListener>

@property (nonatomic, assign) BOOL shouldShowMenu;
@property (nonatomic, assign) BOOL shouldShowPropertyBar;
@property (nonatomic, strong) FSRectF *oldRect;
@property (nonatomic, assign) CGPoint originPt;
@property (nonatomic, assign) BOOL isModify;
@end

@implementation FtAnnotHandler {
    FSEasyDoneTextView *_textView;
    BOOL _isSaved;
    BOOL _keyboardShown;
    
    NSString* _savedText;
    
    //for callout
    BOOL _isMovingCalloutStartPoint;
    CGPoint _startPoint;
    CGPoint _kneePoint;
    CGPoint _endPoint;
    CGFloat _polylineLength;
}

- (id)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
//        self.isShowStyle = NO;
        self.editType = FSAnnotEditTypeFull;
        _isMovingCalloutStartPoint = NO;
        _startPoint = CGPointZero;
        _kneePoint = CGPointZero;
        _endPoint = CGPointZero;
        _polylineLength = 20.0;
        
        [extensionsManager registerRotateChangedListener:self];
        [extensionsManager registerWindowFrameChangedListener:self];
        [extensionsManager.pdfViewCtrl registerPageEventListener:self];
        [extensionsManager.pdfViewCtrl registerScrollViewEventListener:self];
        [extensionsManager registerPropertyViewEvnetListener:self];
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotFreeText;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionNone;
    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionNone;
    }
    BOOL isRichText = NO;
    if (annot.nonempty && annot.getType == FSAnnotFreeText) {
        FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:annot];
        isRichText = [freeText getRichTextCount];
    }

    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        if (!isRichText) {
            options |= FSMenuOptionStyle;
            if (annot.canEdit) {
                if ([self.extensionsManager annotsPermissionCanModify:annot]) {
                    options |= FSMenuOptionEdit;
                }
            }
        }
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    if (annot.canCopyText && [Utility canCopyText:self.extensionsManager.pdfViewCtrl] && self.extensionsManager.config.copyText) {
        options |= FSMenuOptionCopyText;
    }
    return options;
}

- (void)onAnnotSelected:(FSAnnot *)annot {
    [super onAnnotSelected:annot];
    _savedText = annot.contents;
    _isSaved = YES;
    if ([annot.intent isEqualToString:@"FreeTextCallout"]) {
        
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:annot];
        FSPointFArray *calloutPoints = [freetext getCalloutLinePoints];
        int numPoints = [calloutPoints getSize];
        if (numPoints < 2) {
            FSPointF *startPoint = [[FSPointF alloc] initWithX:0 Y:0];
            FSPointF *kneePoint = [[FSPointF alloc] initWithX:0 Y:0];
            FSPointF *endPoint = [[FSPointF alloc] initWithX:0 Y:0];
            
            FSPointFArray *points = [[FSPointFArray alloc] init];
            [points add:startPoint];
            [points add:kneePoint];
            [points add:endPoint];
            [freetext setCalloutLinePoints:points];
        }
        
        self.annotImage = [Utility getAnnotImage:annot pdfViewCtrl:self.pdfViewCtrl];
    }else if ([annot.intent isEqualToString:@"FreeTextTypewriter"]){
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:annot];
        [freetext allowTextOverflow:YES];
    }
}

- (UIFont *)getUIFontForFreeText:(FSFreeText *)freeText {
    FSDefaultAppearance *appearance = [freeText getDefaultAppearance];
    float fontSize = [appearance getText_size] ?: 10;
    fontSize = [Utility convertWidth:fontSize fromPDFToPageView:self.pdfViewCtrl pageIndex:freeText.pageIndex];
    FSFont *font = [appearance getFont];
    NSString *fontName = nil;
    if (font && ![font isEmpty]) {
        fontName = [font getName];
    }
    if (fontName.length == 0) {
        fontName = @"Helvetica";
    }
    return [self getSysFont:fontName size:fontSize];
}

- (void)edit {
    self.extensionsManager.shouldShowMenu = NO;
    _isSaved = NO;
    FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];
    BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
    BOOL isTypeWriter = [annot.intent isEqualToString:@"FreeTextTypewriter"];
    BOOL isTextbox = !isCallout && !isTypeWriter;
    int pageIndex = annot.pageIndex;
    {
        FSRotation rotation = [annot.getPage getRotation];
        FSRotation viewRotation = (FSRotation)[self.pdfViewCtrl getViewRotation];
        FSRotation annotRotation = [annot getRotation];
        rotation = (FSRotation) ((annotRotation + rotation + viewRotation) % 4);
        CGRect textFrame;
        FSRectF *annotRect;
        if (isCallout) {
            annotRect = [annot getInnerRect];
        } else {
            annotRect = annot.fsrect;
        }
        FSMatrix2D *mat = [self.pdfViewCtrl getDisplayMatrix:pageIndex];
        FSMatrix2D *annotMat = [annot getDisplayMatrix:mat];
        [annotMat transformRect:annotRect];
        textFrame = [Utility FSRectF2CGRect:annotRect];//[self.pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:pageIndex];
        CGPoint pt = CGPointMake(CGRectGetMidX(textFrame) , CGRectGetMidY(textFrame));
        CGAffineTransform transform = CGAffineTransformConcat(CGAffineTransformMakeTranslation(-pt.x, -pt.y), CGAffineTransformMakeRotation( -M_PI_2 * rotation));
        transform = CGAffineTransformConcat(transform, CGAffineTransformMakeTranslation(pt.x, pt.y));
        textFrame = CGRectApplyAffineTransform(textFrame, transform);
        
        _textView = [[FSEasyDoneTextView alloc] initWithFrame:textFrame];
        _textView.textContainer.lineFragmentPadding = 0;
        _textView.textContainerInset = UIEdgeInsetsZero;
        _textView.delegate = self;
        if (isTextbox) {
            _textView.layer.borderColor = [UIColor clearColor].CGColor;
        }
        if (isCallout) {
            _textView.layer.borderColor = [UIColor clearColor].CGColor;
            _textView.textContainerInset = UIEdgeInsetsMake(2.5, -4, 0, -4);
            _textView.layer.borderWidth = 1;
        }
        _textView.showsVerticalScrollIndicator = NO;
        _textView.showsHorizontalScrollIndicator = NO;
        _textView.scrollEnabled = NO;
        _textView.clipsToBounds = NO;
        _textView.backgroundColor = [UIColor clearColor];
        if (annot.fillColor) _textView.backgroundColor = [UIColor colorWithRGB:annot.fillColor];
        if (!isTypeWriter) {
            if (annot.borderColor) [_textView fs_setLayerBorderWidth:1 borderColor:[UIColor colorWithRGB:annot.borderColor]];
        }
        _textView.textColor = [UIColor colorWithRGB:annot.color alpha:annot.opacity];
        _textView.font = [self getUIFontForFreeText:annot];
        
        _textView.transform = CGAffineTransformMakeRotation(M_PI_2 * rotation);
        [_textView becomeFirstResponder];
        _textView.text = annot.contents;
        [self textViewDidChange:_textView];
        self.isModify = NO;
        @weakify(self)
        _textView.doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self.extensionsManager setCurrentAnnot:nil];
        };
        [[self.pdfViewCtrl getPageView:pageIndex] addSubview:_textView];

        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasShown:)
                                                     name:UIKeyboardDidShowNotification
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasHidden:)
                                                     name:UIKeyboardWillHideNotification
                                                   object:nil];
        if (isCallout || isTextbox) {
            CGRect refreshRect = CGRectInset(textFrame, -30, -30);
            if (isCallout) {
                refreshRect = [FreetextUtil resetCalloutAppearanceWhenTextviewChanged:self.pdfViewCtrl annot:annot textFrame:_textView.frame];
            }
            if (isTextbox) {
                self.annotImage = nil;
            }else{
                self.annotImage = [Utility getAnnotImage:annot pdfViewCtrl:self.pdfViewCtrl];
            }
            if (isCallout) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self.pdfViewCtrl refresh:refreshRect pageIndex:pageIndex];
                });
            }else{
                [self.pdfViewCtrl refresh:refreshRect pageIndex:pageIndex needRender:NO];
            }
        }
        else{
            self.annotImage = nil;
            textFrame = CGRectInset(textFrame, -30, -30);
            [self.pdfViewCtrl refresh:textFrame pageIndex:pageIndex needRender:NO];
        }
    }
}

- (void)resetProperties{
    [super resetProperties];
    FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];
    [self fixInvalidFontForFreeText:annot];
    FSDefaultAppearance *appearance = [annot getDefaultAppearance];
    float fontSize = [appearance getText_size];
    FSFont *font = [appearance getFont];
    NSString *fontName = @"";
    if (font && ![font isEmpty]) {
        fontName = [font getName];
    }
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setPropertyColor:annot.color];
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];
    [propertyMainView setFontName:fontName];
    [propertyMainView setFontSize:fontSize];
    
    [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity), @(FSPropertyTypeFontName), @(FSPropertyTypeFontSizeFixed)]];
}

- (void)fixInvalidFontForFreeText:(FSFreeText *)annot {
    FSDefaultAppearance *ap = [annot getDefaultAppearance];
    if (!ap.font || [ap.font isEmpty] || ap.text_size == 0) {
        if (!ap.font || [ap.font isEmpty]) {
            FSFont *font = [[FSFont alloc] initWithFont_id:FSFontStdIDHelvetica];
            ap.font = font;
        }
        if (ap.text_size == 0) {
            ap.text_size = 10;
        }
        ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
        [annot setDefaultAppearance:ap];
    }
}

- (void)endEdit:(BOOL *_Nonnull)shouldRemoveAnnot {
    *shouldRemoveAnnot = NO;
    
    if (!_isSaved) {
        _isSaved = YES;
        if (self.isModify){
            NSString *checker = [NSString stringWithFormat:@"%C", 8198];
            if ([_textView.text rangeOfString:checker].length) {
                _textView.text  = [_textView.text stringByReplacingOccurrencesOfString:checker withString:@""];
                _savedText = _textView.text;
            }

            _textView.selectedRange = NSMakeRange(_textView.text.length, 0);
            FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];
            BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
            BOOL isTextBox = [annot.subject isEqualToString:@"Textbox"];
            if (isTextBox) {
                [annot allowTextOverflow:NO];
            }
            if (_savedText.length == 0 && (!isCallout && !isTextBox)) {
                *shouldRemoveAnnot = YES;
            } else {
                annot.contents = _savedText;
                [self fixInvalidFontForFreeText:annot];
                CGRect textFrame = _textView.frame;
                
                // move annot if exceed page edge, as the annot size may be changed after reset ap (especially for Chinese char, the line interspace is changed)
                if (textFrame.origin.y + textFrame.size.height > [self.pdfViewCtrl getPageView:annot.pageIndex].bounds.size.height) {
                    textFrame.origin.y = [self.pdfViewCtrl getPageView:annot.pageIndex].bounds.size.height - textFrame.size.height;
                }
                if (!isCallout) {
                    annot.fsrect = [self.pdfViewCtrl convertPageViewRectToPdfRect:_textView.frame pageIndex:annot.pageIndex];
                }
                [self.pdfViewCtrl lockRefresh];
                [annot resetAppearanceStream];
                [self.pdfViewCtrl unlockRefresh];
            }
            [annot allowTextOverflow:YES];
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.extensionsManager setCurrentAnnot:nil];
            [self->_textView resignFirstResponder];
            [self->_textView removeFromSuperview];
            self->_textView = nil;
            [[NSNotificationCenter defaultCenter] removeObserver:self];
        });
    }
}

- (void)deleteAnnot {
    FSAnnot *annot = self.extensionsManager.currentAnnot;
    FSTask *task = [[FSTask alloc] init];
    task.run = ^() {
        [self removeAnnot:annot];
    };
    [self.extensionsManager.taskServer executeSync:task];
    [self.extensionsManager setCurrentAnnot:nil];
}

- (void)onAnnotDeselected:(FSAnnot *)annot {
    BOOL shouldRemoveAnnot;
    [self endEdit:&shouldRemoveAnnot];
    if (shouldRemoveAnnot) {
        [self removeAnnot:annot];
    } else {
        [super onAnnotDeselected:annot];
    }
}

- (void)saveCalloutPoints:(FSFreeText *)annot {
    FSPointFArray *points = [annot getCalloutLinePoints];
    int pageindex = annot.pageIndex;
    _startPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:0] pageIndex:pageindex];
    _kneePoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:1] pageIndex:pageindex];
    _endPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:2] pageIndex:pageindex];
}

- (BOOL)adjustCalloutDragpoint:(FSFreeText*)annot translationPoint:(CGPoint)translationPoint {
    CGFloat tw = translationPoint.x;
    CGFloat th = translationPoint.y;

    [self saveCalloutPoints:annot];
    _startPoint.x += tw;
    _startPoint.y += th;
    
    CGRect innerRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:annot.pageIndex];
//    CGRect realPageRect = [self.pdfViewCtrl getPageView:annot.pageIndex].bounds;
//    if ( CGRectContainsPoint(innerRect, _startPoint) || (!CGRectContainsPoint(realPageRect, _startPoint)) ) {
//        return NO;
//    }
    
    [FreetextUtil resetKneeAndEndingPt:self.pdfViewCtrl pageIndex:annot.pageIndex textRect:innerRect pStartingPt:&_startPoint pKneePt:&_kneePoint pEndingPt:&_endPoint];
    
//    if (_startPoint.y < innerRect.origin.y) {
//        //top
//        _endPoint.x = innerRect.origin.x + innerRect.size.width / 2.0;
//        _endPoint.y = innerRect.origin.y;
//        _kneePoint.x = _endPoint.x;
//        _kneePoint.y = _endPoint.y - _polylineLength;
//    } else if (_startPoint.y > innerRect.origin.y + innerRect.size.height) {
//        //bottom
//        _endPoint.x = innerRect.origin.x + innerRect.size.width / 2.0;
//        _endPoint.y = innerRect.origin.y + innerRect.size.height;
//        _kneePoint.x = _endPoint.x;
//        _kneePoint.y = _endPoint.y + _polylineLength;
//    } else if (_startPoint.x < innerRect.origin.x) {
//        //left
//        _endPoint.x = innerRect.origin.x;
//        _endPoint.y = innerRect.origin.y + innerRect.size.height / 2.0;
//        _kneePoint.x = _endPoint.x - _polylineLength;
//        _kneePoint.y = _endPoint.y;
//    } else {
//        //right
//        _endPoint.x = innerRect.origin.x + innerRect.size.width;
//        _endPoint.y = innerRect.origin.y + innerRect.size.height / 2.0;
//        _kneePoint.x = _endPoint.x + _polylineLength;
//        _kneePoint.y = _endPoint.y;
//    }
    return YES;
}

- (CGRect)calculateRectWithEditType:(CGRect)rect translationPoint:(CGPoint)translationPoint {
    CGFloat tw = translationPoint.x;
    CGFloat th = translationPoint.y;
    if (self.editType == FSAnnotEditTypeLeftTop || self.editType == FSAnnotEditTypeMiddleTop || self.editType == FSAnnotEditTypeRightTop) {
        // top
        rect.origin.y += th;
        rect.size.height -= th;
    } else if (self.editType == FSAnnotEditTypeLeftBottom || self.editType == FSAnnotEditTypeMiddleBottom || self.editType == FSAnnotEditTypeRightBottom) {
        // bottom
        rect.size.height += th;
    }
    if (self.editType == FSAnnotEditTypeLeftTop || self.editType == FSAnnotEditTypeLeftMiddle || self.editType == FSAnnotEditTypeLeftBottom) {
        // left
        rect.origin.x += tw;
        rect.size.width -= tw;
    } else if (self.editType == FSAnnotEditTypeRightTop || self.editType == FSAnnotEditTypeRightMiddle || self.editType == FSAnnotEditTypeRightBottom) {
        // right
        rect.size.width += tw;
    }
    return rect;
}

- (CGRect)correctBorderRect:(CGRect)borderRect pageIndex:(int)pageIndex
{
   CGRect realPageRect = [self.pdfViewCtrl getPageView:pageIndex].bounds;
   //    FSRectF *pageViewRect = [Utility CGRect2FSRectF:realPageRect];

   //    FSRectF *rect = [Utility CGRect2FSRectF:borderRect];
       CGFloat left = borderRect.origin.x;
       CGFloat right = borderRect.origin.x + borderRect.size.width;
       CGFloat top = borderRect.origin.y;
       CGFloat bottom = borderRect.origin.y + borderRect.size.height;
       if (borderRect.size.width <= realPageRect.size.width &&
           borderRect.size.height <= realPageRect.size.height) {
           if (left < realPageRect.origin.x) {
               float diferrence = realPageRect.origin.x - left + 5;
               left += diferrence;
               right += diferrence;
           }
           if (top < realPageRect.origin.y) {
               float diferrence = realPageRect.origin.y - top + 5;
               top += diferrence;
               bottom += diferrence;
           }
           if (right > (realPageRect.origin.x + realPageRect.size.width)) {
               float diferrence =  right - (realPageRect.origin.x + realPageRect.size.width) + 5;
               right -= diferrence;
               left -= diferrence;
           }
           if (bottom > (realPageRect.origin.y+realPageRect.size.height)) {
               float diferrence = bottom - (realPageRect.origin.y+realPageRect.size.height) + 5;
               top -= diferrence;
               bottom -= diferrence;
           }
       }
       float fontSize = 0.0;
       if (self.annot && ![self.annot isEmpty]) {
           FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:self.annot];
           fontSize = freeText.getDefaultAppearance.text_size - 6;
       }
       borderRect.origin.x = left;
       borderRect.origin.y = top;
       borderRect.size.width = right - left;
       borderRect.size.height = bottom - top;
   //    CGRect newRect = [Utility FSRectF2CGRect:rect];
       return borderRect;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    if ([_textView isFirstResponder]) return NO;
    BOOL canMove = [self canMoveAnnot:annot];
    BOOL canResize = [self canResizeAnnot:annot];
    if (!canMove && !canResize) {
        if (self.extensionsManager.currentAnnot) {
            [self showMenu];
        }
        return YES;
    }
    if (![Utility isAnnot:annot uniqueToAnnot:self.extensionsManager.currentAnnot]) {
        return NO;
    }
    FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:annot];
    BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
    BOOL isTypeWriter = [annot.intent isEqualToString:@"FreeTextTypewriter"];

    if (isTypeWriter) {
        [freetext allowTextOverflow:YES];
    }
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        //state began
        self.shouldShowMenu = self.extensionsManager.shouldShowMenu;
        self.shouldShowPropertyBar = self.extensionsManager.shouldShowPropertyBar;
        [self hideMenu];
        [self.extensionsManager hidePropertyBar];
        
        CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
        _originPt = pvRect.origin;
        if (!isCallout) {
            UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
            CGPoint point = [recognizer locationInView:pageView];
            self.editType = (FSAnnotEditType) [ShapeUtil getEditTypeWithPoint:point rect:CGRectInset(pvRect, -12, -12) defaultEditType:FSAnnotEditTypeFull];
            _originPt = pvRect.origin;
        }  else {
            pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[freetext getInnerRect] pageIndex:annot.pageIndex];
            UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
            CGPoint point = [recognizer locationInView:pageView];
            self.editType = (FSAnnotEditType) [ShapeUtil getEditTypeWithPoint:point rect:CGRectInset(pvRect, -12, -12) defaultEditType:FSAnnotEditTypeFull];

            if (self.editType == FSAnnotEditTypeFull) {
                FSPointFArray *calloutPoints = [freetext getCalloutLinePoints];
                if ([calloutPoints getSize] > 0) {
                    _startPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:[calloutPoints getAt:0] pageIndex:annot.pageIndex];
                }
                CGFloat diff1 = ABS(_startPoint.x - point.x) + ABS(_startPoint.y - point.y);
                const CGFloat tolerance = 25.0;
                _isMovingCalloutStartPoint = (diff1 < tolerance);
            }
        }

    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        //state changed
        self.oldRect = annot.fsrect;
        CGPoint translationPoint = [recognizer translationInView:[self.pdfViewCtrl getPageView:pageIndex]];
        [recognizer setTranslation:CGPointZero inView:[self.pdfViewCtrl getPageView:pageIndex]];
        if (!annot.canModify) {
            return YES;
        }
        CGFloat tw = translationPoint.x;
        CGFloat th = translationPoint.y;
        
        CGRect borderRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
        CGRect newRect;
        
        if (isCallout) {
            [self saveCalloutPoints:freetext];
            CGRect innerRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[freetext getInnerRect] pageIndex:annot.pageIndex];

            if (_isMovingCalloutStartPoint) {
                if (![self adjustCalloutDragpoint:freetext translationPoint:translationPoint]) {
                    return NO;
                }
            } else {
                CGRect newInnerRect;
                if (self.editType == FSAnnotEditTypeFull) {
                    newInnerRect = innerRect;
                    newInnerRect.origin.x += tw;
                    newInnerRect.origin.y += th;
                } else {
                    newInnerRect = [self calculateRectWithEditType:innerRect translationPoint:translationPoint];
                    if (newInnerRect.size.height < _polylineLength || newInnerRect.size.width < _polylineLength) {
                        return NO;
                    }
                }
                //calculate new points
                if (![self adjustCalloutDragpoint:freetext translationPoint:CGPointZero]) {
                    return NO;
                }
                innerRect = newInnerRect;
            }
            
            CGRect realPageRect = [self.pdfViewCtrl getPageView:annot.pageIndex].bounds;
            borderRect = [FreetextUtil calculateBorderRectWithInnerRect:innerRect startPoint:_startPoint kneePoint:_kneePoint];
            if ( !CGRectContainsRect(realPageRect, borderRect) ) {
                return NO;
            }
            annot.fsrect = [self.pdfViewCtrl convertPageViewRectToPdfRect:borderRect pageIndex:pageIndex];
            FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:annot];
            [freetext setInnerRect:[self.pdfViewCtrl convertPageViewRectToPdfRect:innerRect pageIndex:pageIndex]];
            FSPointFArray *points = [[FSPointFArray alloc] init];
            [points add:[self.pdfViewCtrl convertPageViewPtToPdfPt:_startPoint pageIndex:pageIndex]];
            [points add:[self.pdfViewCtrl convertPageViewPtToPdfPt:_kneePoint pageIndex:pageIndex]];
            [points add:[self.pdfViewCtrl convertPageViewPtToPdfPt:_endPoint pageIndex:pageIndex]];
            [freetext setCalloutLinePoints:points];
        } else {
            if (self.editType == FSAnnotEditTypeFull) {
                borderRect.origin.x += tw;
                borderRect.origin.y += th;
            } else {
                borderRect = [self calculateRectWithEditType:borderRect translationPoint:translationPoint];
            }
            newRect = [self correctBorderRect:borderRect pageIndex:pageIndex];
            if (CGRectEqualToRect(newRect, CGRectZero)) {
                [self.pdfViewCtrl refresh:pageIndex needRender:NO];
                return NO;
            }
            annot.fsrect = [self.pdfViewCtrl convertPageViewRectToPdfRect:newRect pageIndex:pageIndex];
        }
        
        // change annot's appearance and refresh
        CGRect oldRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:pageIndex];
        newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        CGRect allRect = CGRectUnion(newRect, oldRect);
        allRect = CGRectInset(allRect, -30, -30);
        [self.pdfViewCtrl refresh:allRect pageIndex:pageIndex needRender:NO];
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        //state ended or candelled
        self.editType = FSAnnotEditTypeUnknown;
        _isMovingCalloutStartPoint = NO;

        CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
        CGRect oldRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:annot.pageIndex];
        if (self.shouldShowMenu) {
            [self showMenu];
        } else if (self.shouldShowPropertyBar) {
            [self.extensionsManager showPropertyBar];
        }
        
        _originPt = CGPointZero;

        newRect = CGRectUnion(newRect, oldRect);
        newRect = CGRectInset(newRect, -30, -30);
        [self.pdfViewCtrl lockRefresh];
        [annot resetAppearanceStream];
        [self.pdfViewCtrl unlockRefresh];
        
        if (annot.canModify) {
            [self modifyAnnot:annot addUndo:NO];
        }
        
        self.annotImage = [Utility getAnnotImage:annot pdfViewCtrl:self.pdfViewCtrl];
        [self.pdfViewCtrl refresh:pageIndex needRender:NO];
    }
    return YES;
}

- (UIFont *)getSysFont:(NSString *)name size:(float)size {
    UIFont *font = [UIFont fontWithName:[Utility convert2SysFontString:name] size:size];
    if (!font) {
        font = [UIFont systemFontOfSize:size];
    }
    return font;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:annot.pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:annot.pageIndex];
    if ([Utility isAnnot:self.extensionsManager.currentAnnot uniqueToAnnot:annot]) {

        if (self.extensionsManager.currentAnnot) {
            FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];
            BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
            if (isCallout) {
                self.extensionsManager.shouldShowMenu = YES;
            }
        }

        if (!(pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint])) {
            [self.extensionsManager setCurrentAnnot:nil];
        } else if (self.extensionsManager.shouldShowMenu && !self.extensionsManager.shouldShowPropertyBar) {
            [self showMenu];
        }
    } else {
        [self.extensionsManager setCurrentAnnot:annot];
    }
    return YES;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *)annot {
    if (![annot isMarkup] && pageIndex == annot.pageIndex && [Utility isAnnot:annot uniqueToAnnot:self.annot])
        return;
    FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:annot];
    BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
    
    if (pageIndex == annot.pageIndex) {
        if (_textView) {
            if (([annot.intent isEqualToString:@"FreeTextCallout"] || annot.intent.length == 0)&& self.annotImage) {
                [PaintUtility drawCallout:freetext annotImage:self.annotImage inContext:context pdfViewCtrl:self.pdfViewCtrl isLineDash:NO];
            } else {

            }
        } else {
            CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect  pageIndex:annot.pageIndex];
            self.oldRect = annot.fsrect;

            if (self.annotImage) {
                
                CGContextSaveGState(context);
                CGRect rect = CGRectMake(ceilf(_originPt.x), ceilf(_originPt.y), ceilf(self.annotImage.size.width), ceilf(self.annotImage.size.height));
                if (CGPointEqualToPoint(_originPt, CGPointZero)) {
                    rect.origin = pvRect.origin;
                }
                CGContextTranslateCTM(context, rect.origin.x, rect.origin.y);
                CGContextTranslateCTM(context, 0, rect.size.height);
                CGContextScaleCTM(context, 1.0, -1.0);
                CGContextTranslateCTM(context, -rect.origin.x, -rect.origin.y);
                CGContextDrawImage(context, rect, [self.annotImage CGImage]);
                CGContextRestoreGState(context);
                
                if (isCallout && !CGPointEqualToPoint(_originPt, CGPointZero)) {
                    [PaintUtility drawCallout:freetext annotImage:self.annotImage inContext:context pdfViewCtrl:self.pdfViewCtrl isLineDash:YES];
                }
            }

            if (self.extensionsManager.currentAnnot == annot) {
                BOOL canMove = [self canMoveAnnot:annot];
                if (isCallout) {
                    pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[freetext getInnerRect] pageIndex:annot.pageIndex];
                }
                
                CGRect rect = canMove ? CGRectInset(pvRect, -5, -5) : CGRectInset(pvRect, -2, -2);
                if ([annot.intent isEqualToString:@"FreeTextTypewriter"]) {
                    rect = canMove ? CGRectInset(pvRect, -8, -5) : CGRectInset(pvRect, -2, -2);
                }
                CGContextSetLineWidth(context, 2.0);
                CGFloat dashArray[] = {3, 3, 3, 3};
                CGContextSetLineDash(context, 3, dashArray, 4);
                CGContextSetStrokeColorWithColor(context, [[UIColor colorWithRGB:annot.color] CGColor]);
                CGContextStrokeRect(context, rect);

                if (canMove && [self canResizeAnnot:annot]) {
                    UIImage *dragDot = [UIImage imageNamed:@"annotation_drag.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
                    CGFloat dragWidth = dragDot.size.width;
                    NSMutableArray* moveablePointArray = (NSMutableArray*)[ShapeUtil getMovePointInRect:rect];
                    if (isCallout) {
                        FSPointFArray *calloutPoints = [freetext getCalloutLinePoints];
                        if ([calloutPoints getSize] > 0) {
                            _startPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:[calloutPoints getAt:0] pageIndex:annot.pageIndex];
                        }

                        [moveablePointArray addObject:[NSValue valueWithCGRect:CGRectMake(_startPoint.x - dragWidth / 2.0, _startPoint.y - dragWidth / 2.0, 1.0, 1.0)]];
                    }
                    
                    [moveablePointArray enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                        CGRect dotRect = [obj CGRectValue];
                        CGPoint point = CGPointMake(dotRect.origin.x, dotRect.origin.y);
                        [dragDot drawAtPoint:point];
                    }];
                }
            }
        }
    }
}

- (BOOL)shouldDrawAnnot:(FSAnnot *)annot inPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    if ([annot isEqualToAnnot:self.longPressingAnnot]) {
        return NO;
    }
    if (self.extensionsManager.currentToolHandler && [self.extensionsManager.currentToolHandler getName] == Tool_Callout) {
        if ([annot isEqualToAnnot:[(CalloutToolHandler *)self.extensionsManager.currentToolHandler annot]]) {
            return NO;
        }
    }
    return [super shouldDrawAnnot:annot inPDFViewCtrl:pdfViewCtrl];
}

#pragma mark - UITextViewDelegate
- (void)textViewDidChange:(UITextView *)textView {
    CGSize oneSize = [Utility getTestSize:textView.font];
    CGPoint point = textView.frame.origin;
    
    if (!self.extensionsManager.currentAnnot) return;

    FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];

    int pageIndex = annot.pageIndex;
    BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
    if (annot.intent.length == 0) {
        CGAffineTransform textViewTransform = textView.transform;
        textView.transform = CGAffineTransformIdentity;
        CGRect frame = textView.frame;
        CGSize constraintSize = CGSizeMake(frame.size.width, MAXFLOAT);
        CGSize size = [textView sizeThatFits:constraintSize];
        textView.frame = CGRectMake(frame.origin.x, frame.origin.y, frame.size.width, MAX(size.height, frame.size.height));
        textView.transform = textViewTransform;
    } else if (isCallout) {
        UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
        [FreetextUtil adjustTextViewFrame:textView inPageView:pageView forMinPageInset:5];
    } else {
        CGAffineTransform textViewTransform = textView.transform;
        textView.transform = CGAffineTransformIdentity;
        NSMutableParagraphStyle *paragraphStyle = [[NSMutableParagraphStyle alloc] init];
        paragraphStyle.lineBreakMode = NSLineBreakByWordWrapping;
        CGSize size = [textView.text boundingRectWithSize:CGSizeMake([self.pdfViewCtrl getPageViewWidth:pageIndex] - point.x, 99999) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : textView.font, NSParagraphStyleAttributeName : paragraphStyle} context:nil].size;
        size.height += 5;
        CGRect frame = textView.frame;
        frame.size = size;
        textView.frame = frame;
        textView.transform = textViewTransform;
    }
    float textViewHeight = textView.frame.origin.y + textView.frame.size.height;
    if (textViewHeight >= ([self.pdfViewCtrl getPageViewHeight:pageIndex] - 20)) {
        CGAffineTransform textViewTransform = textView.transform;
        textView.transform = CGAffineTransformIdentity;
        CGRect textViewFrame = textView.frame;
        textViewFrame.origin.y -= (textViewHeight - [self.pdfViewCtrl getPageViewHeight:pageIndex]);
        textView.frame = textViewFrame;
        textView.transform = textViewTransform;
    }
    CGRect textViewFrame = textView.frame;
    if (isCallout) {
        CGRect refreshRect = [FreetextUtil resetCalloutAppearanceWhenTextviewChanged:self.pdfViewCtrl annot:annot textFrame:textViewFrame];
        self.annotImage = [Utility getAnnotImage:annot pdfViewCtrl:self.pdfViewCtrl];
        textView.frame = [self.pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:annot.pageIndex];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.pdfViewCtrl refresh:refreshRect pageIndex:pageIndex];
        });
    }
    _savedText = textView.text;
    if (textView.frame.size.height >= ([self.pdfViewCtrl getPageViewHeight:pageIndex] - 20)) {
//        [textView endEditing:YES];
    }
    self.isModify = YES;
}

- (void)textViewDidEndEditing:(UITextView *)textView{
    [_textView removeFromSuperview];
    _textView = nil;
    BOOL endEdit;
    [self endEdit:&endEdit];
    self.annotImage = [Utility getAnnotImage:self.annot pdfViewCtrl:self.pdfViewCtrl];
}

#pragma mark - keyboard
- (void)keyboardWasShown:(NSNotification *)notification {
    if (_keyboardShown)
        return;
    _keyboardShown = YES;
    FSAnnot *annot = self.extensionsManager.currentAnnot;
    
    [FreetextUtil afterKeyboardShown:notification textView:_textView UIExtensionsManager:self.extensionsManager pageIndex:annot.pageIndex];
}

- (void)keyboardWasHidden:(NSNotification *)notification {
    int pageIndex = [self.pdfViewCtrl getCurrentPage];
    [self.pdfViewCtrl refresh:pageIndex];
    _keyboardShown = NO;
    
    [FreetextUtil adjustBottomOffset:self.pdfViewCtrl pageIndex:pageIndex];
}

#pragma mark -- IRotationEventListener
- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration{
    if (_textView && _textView.superview) {
        [self textViewDidEndEditing:_textView];
    }
}

#pragma mark -- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD) {
        if (_textView && _textView.superview) {
            [self textViewDidEndEditing:_textView];
        }
    }
}

#pragma mark -- IPageEventListener
- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex{
    if (_textView && _textView.superview) {
        [self textViewDidEndEditing:_textView];
    }
}

#pragma mark <IScrollViewEventListener>
- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView {

    if (_textView && self.extensionsManager.currentAnnot) {
        
        FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];
        BOOL isCallout = [annot.intent isEqualToString:@"FreeTextCallout"];
        int pageIndex = annot.pageIndex;
        if (isCallout) {
            _textView.frame = [self.pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:pageIndex];
        } else {
            _textView.frame = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        }
        _textView.font = [self getUIFontForFreeText:annot];
        [[self.pdfViewCtrl getPageView:self.pdfViewCtrl.getCurrentPage] addSubview:_textView];
    }
    if (self.annotImage && self.extensionsManager.currentAnnot) {
        FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:self.extensionsManager.currentAnnot];
        self.annotImage = [Utility getAnnotImage:annot pdfViewCtrl:self.pdfViewCtrl];
        [self.pdfViewCtrl refreshOverlay];
    }
}

#pragma mark --IPropertyViewEventListener
- (void)onPropertyViewDidShow{
    if (_textView && _textView.superview) {
        [self textViewDidEndEditing:_textView];
    }
}

@end
