/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSHistoryDataModule.h"

@implementation FSHistoryDataItem

- (NSString *)description{
    return FSGetObjectPropertyDescription(self, [super description]);
}

- (void)encodeWithCoder:(NSCoder *)coder{
    [self fs_encodeWithCoder:coder];
}
 
- (instancetype)initWithCoder:(NSCoder *)decoder{
    if (self = [super init]) {
        [self fs_decoderWithCoder:decoder];
    }
    return self;
}

@end

@implementation FSHistoryDataConfig

- (void)encodeWithCoder:(NSCoder *)coder{
    [self fs_encodeWithCoder:coder];
}
 
- (instancetype)initWithCoder:(NSCoder *)decoder{
    if (self = [super init]) {
        [self fs_decoderWithCoder:decoder];
    }
    return self;
}


- (instancetype)init{
    self = [super init];
    if (self) {
        _enable = YES;
        _maxCount = 10;
    }
    return self;
}

- (NSArray<FSHistoryDataItem *> *)sortItemsByKeyword:(NSString *)keyword{
    NSMutableArray<FSHistoryDataItem *> *items = self.items.mutableCopy;
    if (keyword.length) {
        NSPredicate *predicate = [NSPredicate predicateWithBlock:^BOOL(FSHistoryDataItem *item, NSDictionary *bindings) {
            BOOL res = NO;
            if (self.sortRule & FSHistoryDataMatchCase) {
                if (self.sortRule & FSHistoryDataMatcConsecutive) {
                    res = [item.content hasPrefix:keyword];
                }else{
                    res = [item.content containsString:keyword];
                }
            }else{
                if (self.sortRule & FSHistoryDataMatcConsecutive) {
                    res = [item.content.lowercaseString hasPrefix:keyword.lowercaseString];
                }else{
                    res = [item.content.lowercaseString containsString:keyword.lowercaseString];
                }
            }
            return res;
        }];
        [items filterUsingPredicate:predicate];
    }
    switch (self.sortKey) {
        case FSHistoryDataSortKeyDate:
        case FSHistoryDataSortKeyChar:
            [items sortUsingComparator:^NSComparisonResult(FSHistoryDataItem * _Nonnull obj1, FSHistoryDataItem * _Nonnull obj2) {
                return [obj1.content compare:obj2.content options:NSNumericSearch];
            }];
            if (self.sortOrder == FSDataSortOrderedDescending) items = [[items reverseObjectEnumerator] allObjects].mutableCopy;
            break;
        default:
            [items sortUsingComparator:^NSComparisonResult(FSHistoryDataItem * _Nonnull obj1, FSHistoryDataItem * _Nonnull obj2) {
                if (obj1.lastUseDate.timeIntervalSince1970 > obj2.lastUseDate.timeIntervalSince1970) {
                    if (self.sortOrder == FSDataSortOrderedAscending) {
                        return NSOrderedDescending;
                    }
                    return NSOrderedAscending;
                }
                if (self.sortOrder == FSDataSortOrderedAscending) {
                    return NSOrderedAscending;
                }
                return NSOrderedDescending;
            }];
            break;
    }
    return [items subarrayWithRange:NSMakeRange(0, items.count > self.maxCount ? self.maxCount : items.count)];
}

@end

static NSMutableDictionary <NSNumber *, FSHistoryDataConfig *>* _historyDataItemsFromScene;

@implementation FSHistoryDataModule

+ (void)initialize{
    [super initialize];
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _historyDataItemsFromScene = @{}.mutableCopy;
    });
}

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        FSHistoryDataConfig *fillFormConfig = [[FSHistoryDataConfig alloc] init];
        fillFormConfig.scene = FSInteractiveSceneFillingForms;
        fillFormConfig.sortOrder = FSDataSortOrderedDescending;
        fillFormConfig.sortRule = FSHistoryDataMatcConsecutive;
        [self configScene:fillFormConfig];
    }
    return self;
}

- (FSModuleStringName)getName{
    return Module_History_Data;
}

- (void)configScene:(FSHistoryDataConfig *)config{
    _historyDataItemsFromScene[@(config.scene)] = config;
}

- (FSHistoryDataConfig *)configForScene:(FSInteractiveScene)scene{
    return _historyDataItemsFromScene[@(scene)];
}

- (NSArray<FSHistoryDataItem *> *)currentItemsFromScene:(FSInteractiveScene)scene{
    return _historyDataItemsFromScene[@(scene)].items.copy;
}

- (NSArray<FSHistoryDataItem *> *)sortItemsByKeyword:(NSString *)keyword
                                           fromScene:(FSInteractiveScene)scene{
    FSHistoryDataConfig *config = _historyDataItemsFromScene[@(scene)];
    
    return [config sortItemsByKeyword:keyword];
}
@end
