/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ImageToolHandler.h"
#import "FSFileAndImagePicker.h"
#import "PaintUtility.h"

static void swapValues(CGFloat *v1, CGFloat *v2) {
    CGFloat tmp = *v1;
    *v1 = *v2;
    *v2 = tmp;
}

@interface ImageToolHandler ()<FSFileAndImagePickerDelegate>

@property (nonatomic) CGPoint point;
@property (nonatomic, strong) UIImageView *imageView;
@property (nonatomic) int pageIndex;

// 0.0~1.0
@property (nonatomic) CGFloat minImageWidthInPage;
@property (nonatomic) CGFloat maxImageWidthInPage;
@property (nonatomic) CGFloat minImageHeightInPage;
@property (nonatomic) CGFloat maxImageHeightInPage;
@property (nonatomic,strong) FSFileAndImagePicker *mediaPicker;

@property (nonatomic, strong) FSPointF *startPoint;
@property (nonatomic, strong) FSPointF *endPoint;
@property (nonatomic, strong) PaintUtility *paintUtility;
@property (nonatomic, strong) NSMutableDictionary *tempPointsDict;
@property (nonatomic, assign) CGRect imageRect;
@property (nonatomic, assign) CGRect rect;
@end

@implementation ImageToolHandler {
    UIExtensionsManager * __weak _extensionsManager;
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotScreen;
        _minImageWidthInPage = 0.1;
        _maxImageWidthInPage = 0.3;
        _minImageHeightInPage = 0.1;
        _maxImageHeightInPage = 0.3;
        _image = nil;
        _imageView = nil;
        _pageIndex = -1;
        _point = CGPointZero;
        
        self.mediaPicker = [[FSFileAndImagePicker alloc] init];
    }
    return self;
}

- (NSString *)getName {
    return Tool_Image;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
    self.imageRect = CGRectZero;
    self.startPoint = nil;
    self.endPoint = nil;
    [_pdfViewCtrl refresh:self.pageIndex];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    return [self handleLongPressAndPan:pageIndex gestureRecognizer:recognizer];
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)handleLongPressAndPan:(int)pageIndex gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer {
    if (_extensionsManager.currentToolHandler != self ) {
        return NO;
    }
    
    CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    
    [self.tempPointsDict setObject:@(point) forKey:@(pageIndex)];
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSRectF *dibRect = nil;
    
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
        _point = point;
        self.pageIndex = pageIndex;
        self.imageRect = CGRectZero;
        self.rect = CGRectZero;
        
        dibRect = [[FSRectF alloc] init];
        [dibRect setLeft:dibPoint.x];
        [dibRect setBottom:dibPoint.y];
        [dibRect setRight:dibPoint.x + 0.1];
        [dibRect setTop:dibPoint.y + 0.1];
        
        self.startPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        self.endPoint   = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];

        self.paintUtility = [[PaintUtility alloc] init];
        self.paintUtility.annotLineWidth = 0;
        self.paintUtility.annotColor = UIColor.blueColor.rgbValue;
        self.paintUtility.annotOpacity = 0.3;
        
    } else if (gestureRecognizer.state == UIGestureRecognizerStateChanged) {
        if (pageIndex != self.pageIndex) {
            return YES;
        }
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        float marginX = [Utility getAnnotMinXMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
        float marginY = [Utility getAnnotMinYMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
        if (point.y < marginY) {
            point.y = marginY;
        }
        if (point.y > pageview.bounds.size.height - marginY) {
            point.y = pageview.bounds.size.height - marginY;
        }
        if (point.x < marginX) {
            point.x = marginX;
        }
        if (point.x > pageview.bounds.size.width - marginX) {
            point.x = pageview.bounds.size.width - marginX;
        }
        self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:self.pageIndex];
        CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:self.pageIndex];
        self.imageRect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
        [_pdfViewCtrl refresh:CGRectUnion(self.imageRect, self.rect) pageIndex:pageIndex needRender:NO];
        self.rect = self.imageRect;

    } else if (gestureRecognizer.state == UIGestureRecognizerStateEnded || gestureRecognizer.state == UIGestureRecognizerStateCancelled) {
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        if (pageIndex != self.pageIndex) {
            pageIndex = self.pageIndex;
            pageview = [_pdfViewCtrl getPageView:pageIndex];
            point = [gestureRecognizer locationInView:pageview];
            _pageIndex = pageIndex;
            _point = point;
        }
        
        CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:pageIndex];
        CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:pageIndex];
        if (CGPointEqualToPoint(startPoint, endPoint)) {
            self.startPoint = nil;
            self.endPoint = nil;
            self.imageRect = CGRectZero;
            [self onPageViewTap:pageIndex recognizer:(UITapGestureRecognizer*)gestureRecognizer];
            return YES;
        }
        
        
        endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:self.pageIndex];
        self.imageRect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
    
        CGRect dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:self.imageRect pageIndex:pageIndex];
        self.mediaPicker.extensionsManager = _extensionsManager;
        self.mediaPicker.fileType = FSMediaFileTypeImage;
        self.mediaPicker.delegate = self;
        [self.mediaPicker showMenu:dvRect];
        [_pdfViewCtrl refresh:pageIndex];
        
        return YES;
    }
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    _point = point;
    _pageIndex = pageIndex;
    
    CGSize tapSize = CGSizeMake(5, 5);
    CGRect tapRect = CGRectMake(point.x - tapSize.width / 2, point.y - tapSize.height / 2, tapSize.width, tapSize.height);
    
    CGRect dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:tapRect pageIndex:self.pageIndex];

    self.mediaPicker.extensionsManager = _extensionsManager;
    self.mediaPicker.fileType = FSMediaFileTypeImage;
    self.mediaPicker.delegate = self;
    [self.mediaPicker showMenu:dvRect];
 
    self.imageRect = CGRectZero;
    self.startPoint = nil;
    self.endPoint = nil;
    [_pdfViewCtrl refresh:pageIndex];
    return YES;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (CGRect)annotRectWithCenter:(CGPoint)center pageView:(UIView *)pageView {
    float pageWidth = CGRectGetWidth(pageView.frame);
    float pageHeight = CGRectGetHeight(pageView.frame);
    if (!CGRectContainsPoint(pageView.bounds, center)) {
        return CGRectZero;
    }
    CGFloat originWidth = self.image.size.width;
    CGFloat originHeight = self.image.size.height;
    if (self.toolSettings.screenAnnotRotation == FSRotation90 || self.toolSettings.screenAnnotRotation == FSRotation270) {
        swapValues(&originWidth, &originHeight);
    }

    CGFloat width = MIN(MAX(originWidth, self.minImageWidthInPage * pageWidth), self.maxImageWidthInPage * pageWidth);
    CGFloat height = originHeight / originWidth * width;
    height = MIN(MAX(height, self.minImageHeightInPage * pageHeight), self.maxImageHeightInPage * pageHeight);
    width = originWidth / originHeight * height;

    CGRect frame = CGRectMake(center.x - width / 2, center.y - height / 2, width, height);
    return [Utility boundedRectForRect:frame containerRect:pageView.bounds];
}

#pragma mark draw by CGContext
- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    
    if (!CGRectEqualToRect(self.imageRect, CGRectZero) && pageIndex == self.pageIndex) {
        [self.paintUtility drawMultipleSelectWithRect:self.imageRect inContext:context];
    }
}

- (CGRect)getRectFromStartPoint:(CGPoint)startPoint endPoint:(CGPoint)endPoint {
    CGFloat xminValue = MIN(startPoint.x, endPoint.x);
    CGFloat yminValue = MIN(startPoint.y, endPoint.y);
    
    CGFloat xmaxValue = MAX(startPoint.x, endPoint.x);
    CGFloat ymaxValue = MAX(startPoint.y, endPoint.y);
    
    CGRect result = CGRectMake(xminValue, yminValue, fabs(xmaxValue-xminValue), fabs(ymaxValue-yminValue));
    return result;
}

+ (CGSize)sizeAspectRatioFitSize:(CGSize)originSize maxWidth:(CGFloat)maxWidth maxHeight:(CGFloat)maxHeight {
    CGSize size = originSize;
    CGFloat aspectRatio = originSize.height / originSize.width;
    if (size.width > maxWidth) {
        size.width = maxWidth;
        size.height = size.width * aspectRatio;
    }
    if (size.height > maxHeight) {
        size.height = maxHeight;
        size.width = size.height / aspectRatio;
    }
    return size;
}

+ (CGPoint)centerOfRect:(CGRect)rect {
    return CGPointMake(rect.origin.x + rect.size.width / 2, rect.origin.y + rect.size.height / 2);
}

- (void)addAnnotWithRect:(CGRect)rect pageIndex:(int)pageIndex {
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    FSRectF *fsrect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:pageIndex];
    FSScreen *annot = [[FSScreen alloc] initWithAnnot:[page addAnnot:FSAnnotScreen rect:fsrect]];
    if (![annot getCptr]) {
        return;
    }
    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;
    annot.intent = @"Img";
    annot.opacity = self.toolSettings.opacity;

    int pvcRotation = [_pdfViewCtrl getViewRotation];
    FSRotation rotation = (FSRotation)(pvcRotation + self.toolSettings.screenAnnotRotation) % 4;
    rotation = ([page getRotation] + rotation) % 4;
    if (rotation != 0) {
        rotation = 4 - rotation;
    }
    FSBorderInfo *borderinfo = [FSBorderInfo new];
    borderinfo.width = 0.0f;
    [annot setBorderInfo:borderinfo];
    [annot setRotation:rotation];
    
    if (self.filePath.length) {
        annot.contents = [self.filePath lastPathComponent];
    }else{
        NSString *dateStr = ({
            NSDate *date = [NSDate date];
            NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
            [formatter setDateFormat:@"yyyyMMddHHmmssSSS"];
            [formatter setLocale:[NSLocale currentLocale]];
            [formatter stringFromDate:date];
        });
        annot.contents = [NSString stringWithFormat:@"%@.png",dateStr];
    }
    FSImage *fsimage = [Utility uiimageToFSImage:self.image];
    if (fsimage) {
        [annot setImage:fsimage frame_index:0 compress:0];
    }
    [_pdfViewCtrl lockRefresh];
    [annot resetAppearanceStream];
    [_pdfViewCtrl unlockRefresh];
    [_pdfViewCtrl refresh:CGRectInset(rect, -5, -5) pageIndex:pageIndex needRender:YES];

    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:self.type];
    [annotHandler addAnnot:annot addUndo:YES];
}


#pragma mark <FSFileAndImagePickerDelegate>

- (void)fileAndImagePicker:(FSFileAndImagePicker *)fileAndImagePicker didPickImage:(UIImage *)image imagePath:(NSString *)imagePath{
    
    self.filePath = imagePath;
    if (!image) {
        return;
    }
    self.image = image;
    MenuControl *annotMenu = _extensionsManager.menuControl;
    if ([annotMenu isMenuVisible]) {
        [annotMenu hideMenu];
    }
    CGRect imageRect = self.imageRect;
    if (CGRectEqualToRect(self.imageRect, CGRectZero)) {
        UIView *pageView = [_pdfViewCtrl getPageView:_pageIndex];
        imageRect = [self annotRectWithCenter:_point pageView:pageView];
    }
    if (CGRectIsEmpty(imageRect)) {
        return;
    }

    UIImage *imageNew = [UIImage imageWithCGImage:[self.image CGImage]
                                         scale:[self.image scale]
                                   orientation:[Utility imageOrientationForRotation:self.toolSettings.screenAnnotRotation]];
    self.imageView = [[UIImageView alloc] initWithImage:imageNew];
    self.imageView.frame = ({
        CGRect frame = self.imageView.frame;
        frame.size = [self.class sizeAspectRatioFitSize:frame.size maxWidth:imageRect.size.width maxHeight:imageRect.size.height];
        frame;
    });
    self.imageView.center = [self.class centerOfRect:imageRect];

    self.startPoint = nil;
    self.endPoint = nil;
    self.imageRect = CGRectZero;
    [_pdfViewCtrl refresh:_pageIndex];
    
    [self addAnnotWithRect:imageRect pageIndex:self.pageIndex];
    self.pageIndex = -1;
    self.image = nil;
    self.imageView = nil;
}
- (void)fileAndImagePickerDidCancel:(FSFileAndImagePicker *)fileAndImagePicker {
    [self completed];
}

- (void)completed{
    self.image = nil;
    self.filePath = nil;
    _point = CGPointZero;
    _pageIndex = -1;
}
@synthesize toolSettings;

@end
