/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSLinkThumbnailViewController.h"
#import "FSThumbnailCell.h"
#import "FSReorderableCollectionView.h"
#import "FSPermissionUtilty.h"

@interface FSLinkThumbnailViewController ()<FSReorderableCollectionViewDataSource, FSReorderableCollectionViewReorderDelegate, FSThumbnailCellDelegate>{
    FSReorderableCollectionView *_collectionView;
}

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSPDFDoc *document;

@property (nonatomic, strong, readwrite) FSViewControllerManager *services;
@property (nonatomic) CGSize cellSize;

// thumbnail loading
@property (nonatomic, strong) NSOperationQueue *operationQueue;
@property (nonatomic, strong) NSMutableDictionary<NSIndexPath *, NSBlockOperation *> *operations;

// misc
@property (nonatomic) NSInteger insertIndex;
@property (nonatomic) BOOL needPasteItem;
@property (nonatomic, strong) FSPDFDoc *copyedDoc;

@property (nonatomic, assign) BOOL didMovePageNeedUpdate;
@property (nonatomic, assign) BOOL needReloadData;

@property (nonatomic, assign) NSInteger maxSelectedIndex;

@property (nonatomic, weak) FSThumbnailCell *currentPageCell;
@property (nonatomic, assign) BOOL bShowNotice;
@property (nonatomic, assign) BOOL bScrollToCurrentPage;
@end

@implementation FSLinkThumbnailViewController

@dynamic services;

static NSString *const reuseIDForThumbnailCell = @"thumbnailCell";

- (instancetype)initWithServices:(id<FSViewControllerServices>)services{
    self = [super initWithServices:services];
    if (self) {
        self.extensionsManager = self.services.extensionsManager;
        self.pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        
        self.document = self.pdfViewCtrl.currentDoc;
        self.operationQueue = ({
            NSOperationQueue *queue = [[NSOperationQueue alloc] init];
            queue.name = @"load link thumbnail queue";
            queue.maxConcurrentOperationCount = 1;
            queue;
        });
        self.operations = [NSMutableDictionary<NSIndexPath *, NSBlockOperation *> dictionary];
        self.cellSize = [self calculateCellSize];
    }
    return self;
}

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = YES;
    self.shouldLoadDataOnViewDidLoad = YES;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.

    self.isEditing = YES;
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kDone");
    self.title = FSLocalizedForKey(@"kThumbnail");
    self.navBar.rightItem.enabled = NO;
    [self.navBar.rightItem addTarget:self action:@selector(donePress) forControlEvents:UIControlEventTouchUpInside];
    self.view.backgroundColor = self.fs_dividerColor_P1;
}

- (void)donePress{
    
    NSArray <NSNumber *> *array = [self getSelectedItemsForOrderedAscending];
    int pageIndex = self.extensionsManager.pdfViewCtrl.getCurrentPage;
    if (array.count > 0) {
        NSNumber * number = array[0];
        int tmpInt = [number intValue];
        if (tmpInt >= 0 && tmpInt < self.extensionsManager.pdfViewCtrl.getPageCount) {
            pageIndex = tmpInt;
        }
    }
    
    [self dismissViewControllerAnimated:YES completion:^{
        if (self.doneBlock) {
            self.doneBlock(pageIndex);
        }
    }];
    
}

- (FSReorderableCollectionView *)collectionView{
    if (!_collectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
        CGSize tmpCellSize = self.cellSize;
        layout.itemSize = tmpCellSize;

        _collectionView = [[FSReorderableCollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout isModifiable:[FSPermissionUtilty canAssemble:self.extensionsManager] && ![self.document isXFA]];
        _collectionView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        _collectionView.backgroundColor = self.fs_dividerColor_P1;
        
        [_collectionView registerClass:[FSThumbnailCell class] forCellWithReuseIdentifier:reuseIDForThumbnailCell];
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        _collectionView.allowsMultipleSelection = NO;
    }
    return _collectionView;
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    if (self.needReloadData) [self reloadData];
    if (!self.bShowNotice) {
        self.bShowNotice = YES;
    }
    
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.operationQueue cancelAllOperations];
    self.needReloadData = NO;
}

-(void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    
    if (!self.bScrollToCurrentPage) {
        int currentPage = self.startPageIndex;
        if (currentPage < 0 || currentPage > self.extensionsManager.pdfViewCtrl.getPageCount) {
            currentPage = [self.extensionsManager.pdfViewCtrl getCurrentPage];
        }

        [self.collectionView selectItemAtIndexPath:[NSIndexPath indexPathForItem:currentPage inSection:0] animated:NO scrollPosition:UICollectionViewScrollPositionNone];
        [self.collectionView scrollToItemAtIndexPath:[NSIndexPath indexPathForItem:currentPage inSection:0] atScrollPosition:UICollectionViewScrollPositionCenteredVertically animated:NO];
        if ([self.collectionView.delegate respondsToSelector:@selector(collectionView:didSelectItemAtIndexPath:)]) {
            [self.collectionView.delegate collectionView:self.collectionView didSelectItemAtIndexPath:[NSIndexPath indexPathForItem:currentPage inSection:0]];
        }
        
        self.bScrollToCurrentPage = YES;
    }

}

- (void)dealloc {
    [self.operationQueue cancelAllOperations];
}

- (void)reloadData{
    NSArray *selectedItems = self.collectionView.indexPathsForSelectedItems;
    [super reloadData];
    for (NSIndexPath *indexPath in selectedItems) {
        [self selectItem:YES atIndexPath:indexPath];
    }
}

- (BOOL)emptyDataSetShouldDisplay:(UICollectionView *)collectionView{
    return ![self getNumberOfItemsInCollectionView];
}

- (int)getPageCount{
    BOOL isDynamciXFA = [_pdfViewCtrl isDynamicXFA];
    if (!isDynamciXFA) {
        return [self.document getPageCount];
    }
    return [[_pdfViewCtrl getXFADoc] getPageCount];
}

-(NSUInteger)getNumberOfItemsInCollectionView{
    NSUInteger count = 0;
    count = [self getPageCount];
    return count;
}

- (int)getPageIndexInCollectionViewAtItem:(NSInteger)item{

    return (int)item;
}

- (void)viewWillLayoutSubviews{
    [super viewWillLayoutSubviews];
    
    self.cellSize = [self calculateCellSize];
    [self.collectionView.collectionViewLayout invalidateLayout];
}

#pragma mark <UICollectionViewDataSource>

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    FSReorderableCollectionView *reorderCollectionView = (FSReorderableCollectionView *) collectionView;
    
    return [self getNumberOfItemsInCollectionView] - reorderCollectionView.indexSetForDraggingCells.count + (reorderCollectionView.indexPathForPlaceholderCell ? 1 : 0);
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {

    FSReorderableCollectionView *reorderableCollectionView = (FSReorderableCollectionView *) collectionView;
    
    FSThumbnailCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:reuseIDForThumbnailCell forIndexPath:indexPath];
    [cell fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
    cell.contentView.backgroundColor = self.fs_backgroundColor_B3;
    cell.delegate = self;
    cell.isEditing = self.isEditing;
    // remove spinner
    for (UIView *subview in cell.contentView.subviews) {
        if ([subview isKindOfClass:[UIActivityIndicatorView class]]) {
            [subview removeFromSuperview];
            break;
        }
    }
    int pageIndex = [self getPageIndexInCollectionViewAtItem:[reorderableCollectionView getOriginalIndexPathForIndexPath:indexPath].item];
    [cell.labelNumber setTitle:[NSString stringWithFormat:@"%d", pageIndex + 1] forState:UIControlStateDisabled];
    // update button frames
    
    float width = 600;
    float height = 800;
    @try{
        if (![_pdfViewCtrl isDynamicXFA]) {
            FSPDFPage *page = [self.document getPage:pageIndex];
            if (![page isEmpty]) {
                width = [page getWidth];
                height = [page getHeight];
            }
        }else{
            FSXFAPage *page = [[_pdfViewCtrl getXFADoc] getPage:pageIndex];
            if (![page isEmpty]) {
                width = [page getWidth];
                height = [page getHeight];
            }
        }
        
    }
   @catch(NSException* e)
    {
    }
    
    FSRotation rotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    if (rotation == FSRotation90 || rotation == FSRotation270) {
        int tmp = width;
        width =  height;
        height = tmp;
    }
    
    CGFloat realWidth = MIN(self.cellSize.width, self.cellSize.height * width / height);
    [cell updateButtonFramesWithThumbnailWidth:realWidth];
    
    @weakify(self)
    [BindThemeKeySel(cell, updateButtonFramesWithThumbnailWidth:) fs_updateThemeCompleted:^(FSThumbnailCell * _Nonnull target, SEL  _Nonnull sel) {
        @strongify(self)
        if (pageIndex == [self.pdfViewCtrl getCurrentPage]) {
            
            self.currentPageCell.layer.borderColor = target.fs_dividerColor_P1.CGColor;
            self.currentPageCell.labelNumber.backgroundColor = [[UIColor fs_blackColor] colorWithAlphaComponent:0.5];
            self.currentPageCell.layer.borderWidth = 1.f;
            
            self.currentPageCell = target;
            target.layer.borderWidth = 2.f;
            target.labelNumber.backgroundColor = [target.fs_primaryColor colorWithAlphaComponent:0.5];
            target.layer.borderColor = target.fs_primaryColor.CGColor;
        }else{
            target.layer.borderColor = target.fs_dividerColor_P1.CGColor;
            target.labelNumber.backgroundColor = [[UIColor fs_blackColor] colorWithAlphaComponent:0.5];
            target.layer.borderWidth = 1.f;
        }
    }];
    
    cell.bookmark.hidden = YES;

    BOOL selected = [[self.collectionView indexPathsForSelectedItems] containsObject:indexPath];
    cell.selected = selected;
    return cell;
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        [self reloadData];
    }
}

#pragma mark <UICollectionViewDelegate>

- (BOOL)collectionView:(UICollectionView *)collectionView shouldSelectItemAtIndexPath:(nonnull NSIndexPath *)indexPath {
    if (!self.isEditing) {
        return NO;
    } else {
        return YES;
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {

    if (indexPath.item > self.maxSelectedIndex) {
        self.maxSelectedIndex = indexPath.item;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self.collectionView.indexPathsForSelectedItems.count > 0) {
            self.navBar.rightItem.enabled = YES;
        }else{
            self.navBar.rightItem.enabled = NO;
        }
    });

}

- (void)collectionView:(UICollectionView *)collectionView didDeselectItemAtIndexPath:(nonnull NSIndexPath *)indexPath {

    if (indexPath.item == self.maxSelectedIndex) {
        NSArray<NSNumber *> *selectedItemsForOrderedAscending = [self getSelectedItemsForOrderedAscending];
        if (selectedItemsForOrderedAscending.count) {
            self.maxSelectedIndex = selectedItemsForOrderedAscending.lastObject.intValue;
        }else{
            self.maxSelectedIndex = -1;
        }
    }

    dispatch_async(dispatch_get_main_queue(), ^{
        if (self.collectionView.indexPathsForSelectedItems.count > 0) {
            self.navBar.rightItem.enabled = YES;
        }else{
            self.navBar.rightItem.enabled = NO;
        }
    });
}

- (void)collectionView:(UICollectionView *)collectionView willDisplayCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    if (self.needPasteItem && self.insertIndex == indexPath.item) return;
    if (![cell isKindOfClass:[FSThumbnailCell class]]) {
        return;
    }
    UIActivityIndicatorView *spinner = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    spinner.center = CGPointMake(cell.contentView.bounds.size.width / 2, cell.contentView.bounds.size.height / 2);
    [cell.contentView addSubview:spinner];
    [spinner startAnimating];
    ((FSThumbnailCell *) cell).imageView.image = nil;

    FSReorderableCollectionView *reorderCollectionView = (FSReorderableCollectionView *) collectionView;
    NSUInteger pageIndex = [self getPageIndexInCollectionViewAtItem:[reorderCollectionView getOriginalIndexPathForIndexPath:indexPath].item];

    CGSize thumbnailSize = ({
        float width = 600;
        float height = 800;
        @try {
            if (![_pdfViewCtrl isDynamicXFA]) {
                FSPDFPage *page = [self.document getPage:(int)pageIndex];
                if (![page isEmpty]) {
                    width = [page getWidth];
                    height = [page getHeight];
                }
            }else{
                FSXFAPage *page = [[_pdfViewCtrl getXFADoc] getPage:(int)pageIndex];
                if (![page isEmpty]) {
                    width = [page getWidth];
                    height = [page getHeight];
                }
            }
        }
        @catch(NSException* e)
        {}
        CGFloat aspectRatio = width / height;
        CGFloat thumbnailWidth = MAX(self.cellSize.width, self.cellSize.height * aspectRatio);
        CGFloat thumbnailHeight = thumbnailWidth / aspectRatio;
        CGSizeMake(thumbnailWidth, thumbnailHeight);
    });

    FSRotation rotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    if (rotation == FSRotation90 || rotation == FSRotation270) {
        int tmp = thumbnailSize.width;
        thumbnailSize.width =  thumbnailSize.height;
        thumbnailSize.height = tmp;
    }
    
    NSBlockOperation *op;
    __weak __block NSBlockOperation *weakOp;
    @weakify(self)
    weakOp = op = [NSBlockOperation blockOperationWithBlock:^{
        if (weakOp.isCancelled) {
            return;
        }
        @strongify(self)
        
        [self.extensionsManager.thumbnailModule.thumbnailCache getThumbnailForPageAtIndex:pageIndex withThumbnailSize:thumbnailSize needPause:^BOOL{
            return !weakOp || weakOp.isCancelled;
        } callback:^(UIImage * _Nullable thumbnailImage) {
            dispatch_async(dispatch_get_main_queue(), ^{
                @strongify(self)
                if (!weakOp.isCancelled && [self.collectionView.visibleCells containsObject:cell]) {
                    ((FSThumbnailCell *) cell).imageView.image = thumbnailImage;
                    [spinner stopAnimating];
                    [spinner removeFromSuperview];
                }
            });
        }];

    }];
    

    [self.operationQueue addOperation:op];
    if (self.operations[indexPath]) {
        [self.operations[indexPath] cancel];
    }
    self.operations[indexPath] = op;

    BOOL selected = [[self.collectionView indexPathsForSelectedItems] containsObject:indexPath];
    cell.selected = selected;
}

- (void)collectionView:(UICollectionView *)collectionView didEndDisplayingCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    if (![self.collectionView.indexPathsForVisibleItems containsObject:indexPath]) {
        NSBlockOperation *operation = self.operations[indexPath];
        [operation cancel];
        self.operations[indexPath] = nil;
    }
}

#pragma mark <UICollectionViewDelegateFlowLayout>
- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    return self.cellSize;
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewFlowLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets cellInsets = [self cellInsets];
    UIEdgeInsets sectionInset = [self sectionInsets];
    CGFloat margin = sectionInset.left + cellInsets.left;
    CGFloat remainSpace = collectionView.fs_width - (self.cellSize.width + margin * 2) * 2;
    if (remainSpace > 0 && (remainSpace < self.cellSize.width + margin * 2)) {
        margin = collectionView.fs_width / 2 - self.cellSize.width - 15.f;
        if (margin < 0) {
            margin = ABS((collectionView.fs_width / 2 - self.cellSize.width)/2);
        }
    }
    return UIEdgeInsetsMake(sectionInset.top, margin, sectionInset.bottom, margin);
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout minimumLineSpacingForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets cellInsets = [self cellInsets];
    return cellInsets.top + cellInsets.bottom;
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section {
    UIEdgeInsets cellInsets = [self cellInsets];
    return cellInsets.left + cellInsets.right;
}

#pragma mark event handler
- (void)setIsEditing:(BOOL)isEditing {
    if (_isEditing != isEditing) {
        _isEditing = isEditing;
        //update tool bars
        
        //        //update collection view
        [self.collectionView.visibleCells enumerateObjectsUsingBlock:^(__kindof FSThumbnailCell *_Nonnull cell, NSUInteger idx, BOOL *_Nonnull stop) {
            if ([cell respondsToSelector:@selector(setIsEditing:)]) {
                cell.isEditing = isEditing;
            }
        }];
        [self deselectAll];
        
        
        self.maxSelectedIndex = -1;
    }
}

- (void)selectOrDeselectAll {
    NSUInteger pageCount = [self getNumberOfItemsInCollectionView] - (self.needPasteItem ? 1 : 0);
    BOOL isSelectAll = (self.collectionView.indexPathsForSelectedItems.count == pageCount);
    
    if (!isSelectAll) {
        [self selectAll];
    } else {
        [self deselectAll];
    }
}

#pragma mark insert pages and images
- (void)convenientAlert:(NSString *)title {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:nil preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                           style:UIAlertActionStyleCancel
                                                         handler:nil];
    [alertController addAction:cancelAction];
    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - inner methods


- (void)selectAll {
    NSUInteger pageCount = [self getNumberOfItemsInCollectionView];
    for (int i = 0; i < pageCount; i++) {
        if (self.needPasteItem && i == self.insertIndex) {
            continue;
        }
        [self.collectionView selectItemAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0] animated:NO scrollPosition:UICollectionViewScrollPositionNone];
    }
}

- (void)deselectAll {
    for (NSIndexPath *selectedIndexPath in self.collectionView.indexPathsForSelectedItems) {
        [self.collectionView deselectItemAtIndexPath:selectedIndexPath animated:NO];
    }
    assert(0 == self.collectionView.indexPathsForSelectedItems.count);
}

- (NSArray <NSNumber *> *)getSelectedItemsForOrderedAscending{
    NSArray<NSNumber *> *pages = [self.collectionView.indexPathsForSelectedItems valueForKey:@"item"];
    if (pages.count == 0) {
        return nil;
    }
    NSArray <NSNumber *>*sortPages = [pages sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
        return [obj1 intValue] < [obj2 intValue] ? NSOrderedAscending : NSOrderedDescending;
    }];
    return sortPages;
}

- (void)updatePageNumberLabels{
    for (NSIndexPath *indexPath in self.collectionView.indexPathsForVisibleItems) {
        FSThumbnailCell *cell = (FSThumbnailCell *) [self.collectionView cellForItemAtIndexPath:indexPath];
        if ([cell isKindOfClass:[FSThumbnailCell class]]) {
            [cell.labelNumber setTitle:[NSString stringWithFormat:@"%d", [self getPageIndexInCollectionViewAtItem:indexPath.item] + 1] forState:UIControlStateDisabled];
        }
    }
    
}

- (void)updatePageNumberLabelsInRange:(NSRange)range {
    if (range.location == NSNotFound || range.length == 0) {
        return;
    }
    for (NSUInteger i = range.location; i < range.location + range.length; i++) {
        FSThumbnailCell *cell = (FSThumbnailCell *) [self.collectionView cellForItemAtIndexPath:[NSIndexPath indexPathForItem:i inSection:0]];
        if ([cell isKindOfClass:[FSThumbnailCell class]]) {
            [cell.labelNumber setTitle:[NSString stringWithFormat:@"%d", [self getPageIndexInCollectionViewAtItem:i] + 1] forState:UIControlStateDisabled];
        }
        
    }
}

- (void)cancelDrawingThumbnails {
    [self.operationQueue cancelAllOperations];
    [self.operations removeAllObjects];
}

- (void)cancelDrawingThumbnailsAtIndexPaths:(NSArray<NSIndexPath *> *)indexPaths {
    [indexPaths enumerateObjectsUsingBlock:^(NSIndexPath *_Nonnull obj, NSUInteger idx, BOOL *_Nonnull stop) {
        [self.operations[obj] cancel];
        self.operations[obj] = nil;
    }];
}

#pragma mark layout sizes

- (CGSize)calculateCellSize {
    CGSize cellSize;
#define OVERVIEW_IMAGE_WIDTH_IPHONE 142.0
#define OVERVIEW_IMAGE_HEIGHT_IPHONE 177.0
#define OVERVIEW_IMAGE_WIDTH 160.0
#define OVERVIEW_IMAGE_HEIGHT 200.0
    if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        cellSize = CGSizeMake(OVERVIEW_IMAGE_WIDTH_IPHONE, OVERVIEW_IMAGE_HEIGHT_IPHONE);
    } else {
        cellSize = CGSizeMake(OVERVIEW_IMAGE_WIDTH, OVERVIEW_IMAGE_HEIGHT);
    }
    
    return cellSize;
}

- (UIEdgeInsets)cellInsets {
    UIInterfaceOrientation orientation = [UIApplication sharedApplication].statusBarOrientation;
    ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
    if (UIInterfaceOrientationIsPortrait(orientation)) {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(5, 12, 5, 12);
            }  else {
                return UIEdgeInsetsMake(5, 6, 5, 6);
            }
        } else {
            return UIEdgeInsetsMake(10, 17, 10, 10);
        }
    } else {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (sizeMode == ScreenSizeMode_40) {
                return UIEdgeInsetsMake(5, 0, 5, 0);
            } else if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(5, 9, 5, 8);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(5, 2, 5, 1);
            } else {
                return UIEdgeInsetsMake(5, 7, 5, 7);
            }
        } else {
            return UIEdgeInsetsMake(10, 17, 10, 10);
        }
    }
}

- (UIEdgeInsets)sectionInsets {
    CGFloat topInset = (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) ? 10 : 20;
    
    UIInterfaceOrientation orientation = [UIApplication sharedApplication].statusBarOrientation;
    if (UIInterfaceOrientationIsPortrait(orientation)) {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
            if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(topInset, 20, 44, 16);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(topInset, 0, 44, 0);
            } else {
                return UIEdgeInsetsMake(topInset, 6, 44, 5);
            }
        } else {
            return UIEdgeInsetsMake(topInset, 6, 44, 5);
        }
    } else {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
            if (sizeMode == ScreenSizeMode_40) {
                return UIEdgeInsetsMake(topInset, 0, 44, 0);
            } else if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(topInset, 15, 44, 15);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(topInset, 4, 44, 4);
            } else {
                return UIEdgeInsetsMake(topInset, 5, 44, 5);
            }
        } else {
            return UIEdgeInsetsMake(topInset, 10, 44, 5);
        }
    }
}

@end
