/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <MobileCoreServices/MobileCoreServices.h>
#import "LinkToolHandler.h"
#import "AudioVideoAnnotHandler.h"
#import "FSFileAndImagePicker.h"
#import "AudioVideoModule.h"
#import "PaintUtility.h"
#import "FSLinkEditViewController.h"
#import "DigitalSignatureAnnotHandler.h"
#import "FormAnnotHandler.h"
#import "ImageAnnotHandler.h"

@interface LinkToolHandler ()<FSFileAndImagePickerDelegate>

@property (nonatomic,assign) CGPoint point;
@property (nonatomic,assign) int pageIndex;
@property (nonatomic, strong) PaintUtility *paintUtility;
@property (nonatomic, strong) NSMutableDictionary *tempPointsDict;
@property (nonatomic, assign) BOOL isEndDrawing;
@property (nonatomic, assign) int currentPageIndex;

@property (nonatomic, strong) FSLink *annot;
@property (nonatomic, strong) FSPointF *startPoint;
@property (nonatomic, strong) FSPointF *endPoint;

@end

@implementation LinkToolHandler {
    UIExtensionsManager * __weak _extensionsManager;
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    FSTaskServer *_taskServer;
}
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    return [[LinkToolHandler alloc] initWithUIExtensionsManager:extensionsManager annotType:FSAnnotLink];
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager annotType:(FSAnnotType)annotType {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _annotType = annotType;
        type = annotType;
        _pageIndex = -1;
        _point = CGPointZero;
        
    }
    return self;
}

- (NSString *)getName {
    return Tool_Link;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
    _isEndDrawing = NO;
    self.annot = nil;
}

- (void)onDeactivate {
    _isEndDrawing = NO;
    self.startPoint = nil;
    self.endPoint = nil;
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {

    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSRectF *dibRect = nil;
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.currentPageIndex = pageIndex;
        _isEndDrawing = NO;
        
        self.startPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        self.endPoint   = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];

        CGFloat red, green, blue, alpha;
        [_extensionsManager.linksHighlightColor getRed:&red green:&green blue:&blue alpha:&alpha];
        self.paintUtility = [[PaintUtility alloc] init];
        self.paintUtility.annotLineWidth = 0;
        self.paintUtility.annotColor = _extensionsManager.linksHighlightColor.rgbValue;
        self.paintUtility.annotOpacity = alpha;
        
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (self.currentPageIndex == pageIndex) {
            FSRectF *oldRectF = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
            CGRect oldRect = [_pdfViewCtrl convertPdfRectToPageViewRect:oldRectF pageIndex:pageIndex];
            FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
            float marginX = [Utility getAnnotMinXMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
            float marginY = [Utility getAnnotMinYMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
            FSRotation rotation = [page getRotation];
            CGFloat pdfPageWidth = (rotation == FSRotation0 || rotation == FSRotation180 || rotation == FSRotationUnknown) ? [page getWidth] : [page getHeight];
            CGFloat pdfPageHeight = (rotation == FSRotation0 || rotation == FSRotation180 || rotation == FSRotationUnknown) ? [page getHeight] : [page getWidth];
            if (!(dibPoint.x < marginX || dibPoint.y > pdfPageHeight - marginY || dibPoint.y < marginY || dibPoint.x > pdfPageWidth - marginX)) {
                self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
            }

            dibRect = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
            CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
            rect = CGRectInset(CGRectUnion(rect, oldRect), -30, -30);
            [_pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:NO];
        }
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
//        if (pageIndex != self.annot.pageIndex) {
//            pageIndex = self.annot.pageIndex;
//        }
        
        float marginX = [Utility getAnnotMinXMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
        float marginY = [Utility getAnnotMinYMarginInPDF:_pdfViewCtrl pageIndex:pageIndex];
        
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        
        if (point.y < marginY) {
            point.y = marginY;
        }
        
        if (point.y > pageview.bounds.size.height - marginY) {
            point.y = pageview.bounds.size.height - marginY;
        }
        
        if (point.x < marginX) {
            point.x = marginX;
        }
        
        if (point.x > pageview.bounds.size.width - marginX) {
            point.x = pageview.bounds.size.width - marginX;
        }
        
        FSRectF *oldRectF = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
        CGRect oldRect = [_pdfViewCtrl convertPdfRectToPageViewRect:oldRectF pageIndex:pageIndex];
        
        self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        dibRect = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
        CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
        rect = CGRectInset(CGRectUnion(rect, oldRect), -30, -30);
        
        _isEndDrawing = YES;
    
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self showLinkEditView:pageIndex];
            [self->_pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:NO];
        });
        
        return YES;
    }
    return YES;
}

- (void)addAnnotWithDic:(NSDictionary *)dict annntRect:(FSRectF *)dibRect refreshRect:(CGRect)rect annotPageIndex:(int)annotPageIndex
{
    if (dict) {
        self.annot = [self addMarkup:dibRect annotPageIndex:annotPageIndex];
        int pageIndex = [[dict objectForKey:kLinkPageIndex] intValue] - 1;
        NSString *webAddr = [dict objectForKey:kLinkWebAddr];
        if (pageIndex < 0 || pageIndex >= self->_extensionsManager.pdfViewCtrl.getPageCount) {
            pageIndex = self->_extensionsManager.pdfViewCtrl.getCurrentPage;
        }
        if (!webAddr) {
            webAddr = @"";
        }
        
        // Add action for link annotation
        FSPDFDoc* doc = [[self.annot getPage] getDocument];
        if ([[dict objectForKey:kLinkSelectRow] intValue] == 0) {
            FSGotoAction* action = [[FSGotoAction alloc] initWithAction:[FSAction create:doc action_type:FSActionTypeGoto]];
            FSDestination* dest = [FSDestination createXYZ:self->_extensionsManager.pdfViewCtrl.getDoc page_index:pageIndex left:0 top:0 zoom_factor:1.0f];
            [action setDestination:dest];
            [self.annot setAction:action];
        }else{
            FSURIAction* action = [[FSURIAction alloc] initWithAction:[FSAction create:doc action_type:FSActionTypeURI]];
            [action setTrackPositionFlag:YES];
            [action setURI:webAddr];
            [self.annot setAction:action];
        }

        [self->_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [self->_pdfViewCtrl unlockRefresh];
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByType:self.type];
        @try {
            [annotHandler addAnnot:self.annot addUndo:YES];
        } @catch (NSException *exception) {
        } @finally {
            self.annot = nil;
            self.paintUtility = nil;
        }
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self->_extensionsManager setCurrentToolHandler:nil];
            [self->_pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:YES];
        });
    }
}

- (void)showLinkEditView:(int)pageIndex{
    FSLinkEditViewController *linkEditVC = [[FSLinkEditViewController alloc] init];
    linkEditVC.extensionsMgr = _extensionsManager;
    
    FSRectF *dibRect = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
    CGRect rect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
    rect = CGRectInset(rect, -30, -30);
    linkEditVC.doneActionBlock = ^(NSDictionary * _Nonnull dict) {
        [self addAnnotWithDic:dict annntRect:dibRect refreshRect:rect annotPageIndex:pageIndex];
    };
    linkEditVC.cancelActionBlock = ^{
        self.startPoint = nil;
        self.endPoint = nil;
        [self->_pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:NO];
    };
    FSNavigationController *navCtrl = [[FSNavigationController alloc] initWithRootViewController:linkEditVC];
    if (DEVICE_iPHONE) {
        navCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
    }else{
        navCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
            if (@available(iOS 13.0, *)) navCtrl.modalInPresentation = YES;
#endif
    }
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:navCtrl animated:YES completion:nil];
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    _point = point;
    _pageIndex = pageIndex;
    
    if (_extensionsManager.currentToolHandler == self) {
        FSAnnot* annot = [_extensionsManager getAnnotAtPoint:point pageIndex:pageIndex];
        if (annot || _extensionsManager.currentAnnot) {
            id<IAnnotHandler> annothandler = [_extensionsManager getAnnotHandlerByType:FSAnnotLink];
            if (annothandler) {
                return [annothandler onPageViewTap:pageIndex recognizer:recognizer annot:annot];
            }
        }
    }

    return YES;
}

- (id<IAnnotHandler>)getAnnotHandlerByAnnot:(FSAnnot *)annot {
    FSAnnotType type = [annot getType];
    if (type == FSAnnotSquiggly || type == FSAnnotStrikeOut || type == FSAnnotUnderline) {
        type = FSAnnotHighlight;
    }
    if (type == FSAnnotSquare) {
        type = FSAnnotCircle;
    }
    for (id<IAnnotHandler> annotHandler in _extensionsManager.annotHandlers) {
        if ([annotHandler respondsToSelector:@selector(getType)]) {
            if ([annotHandler getType] == type) {
                if (FSAnnotWidget == type) {
                    FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
                    FSField *field = [widget getField];
                    FSFieldType fieldType = [field isEmpty] ? FSFieldTypeUnknown : [field getType];
                    if (FSFieldTypeSignature == fieldType && [annotHandler isKindOfClass:[DigitalSignatureAnnotHandler class]]) {
                        return annotHandler;
                    }
                    if (FSFieldTypeSignature != fieldType && [annotHandler isKindOfClass:[FormAnnotHandler class]]) {
                        return annotHandler;
                    }
                }
                else {
                    if (type == FSAnnotScreen) {
                        FSScreen *screen = [[FSScreen alloc] initWithAnnot:annot];
                        FSScreenActionType screenActiontype = screen.screenActiontype;
                        if ((screenActiontype == FSScreenActionTypeUnknow || screenActiontype == FSScreenActionTypeImage) && [annotHandler isKindOfClass:[ImageAnnotHandler class]]) return annotHandler;
                        if (screenActiontype == FSScreenActionTypeAudio && [annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]) return annotHandler;
                        if (screenActiontype == FSScreenActionTypeVideo && [annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]) return annotHandler;
                        continue;
                    }
                     return annotHandler;
                }
            }
        }
    }
    return nil;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    FSAnnot *annot = _extensionsManager.currentAnnot;
    if (annot.nonempty && annot.type == FSAnnotLink) {
        id<IAnnotHandler> annothandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
        if (annothandler) {
            CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
            FSPointF *pdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
            if (!(pageIndex == annot.pageIndex && [annothandler isHitAnnot:annot point:pdfPoint])) {
                [_extensionsManager setCurrentAnnot:nil];
            }
        }
    }
    return YES;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}



- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {

    if (!self.startPoint || !self.endPoint || self.currentPageIndex != pageIndex || _extensionsManager.currentToolHandler != self) {
        return;
    }
    
    CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:pageIndex];
    CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:pageIndex];

    CGRect rect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
    [self.paintUtility drawMultipleSelectWithRect:rect inContext:context];

}

- (CGRect)getRectFromStartPoint:(CGPoint)startPoint endPoint:(CGPoint)endPoint {
    CGFloat xminValue = MIN(startPoint.x, endPoint.x);
    CGFloat yminValue = MIN(startPoint.y, endPoint.y);
    
    CGFloat xmaxValue = MAX(startPoint.x, endPoint.x);
    CGFloat ymaxValue = MAX(startPoint.y, endPoint.y);
    
    CGRect result = CGRectMake(xminValue, yminValue, fabs(xmaxValue-xminValue), fabs(ymaxValue-yminValue));
    return result;
}




- (BOOL)removeAnnot:(FSAnnot *)annot {
    FSPDFPage *page = [annot getPage];
    BOOL result = [page removeAnnot:annot];
    
    self.annot = nil;
    return result;
}

- (FSLink*)addMarkup:(FSRectF*)rect annotPageIndex:(int)pageIndex{
    
    // Add link annotation
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty])
        return nil;
    
    FSLink* annot = [[FSLink alloc] initWithAnnot:[page addAnnot:FSAnnotLink rect:rect]];
    [annot setHighlightingMode:FSAnnotHighlightingToggle];

    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;
    [annot setBorderInfo:({
               FSBorderInfo *borderInfo = [[FSBorderInfo alloc] init];
               [borderInfo setWidth:0];
               [borderInfo setStyle:FSBorderInfoSolid];
                borderInfo;
           })];
    int opacity = 0;

    if (type == FSAnnotLink) {
        annot.subject = @"Link";
    }
    annot.opacity = opacity;

    [_pdfViewCtrl lockRefresh];
    [annot resetAppearanceStream];
    [_pdfViewCtrl unlockRefresh];

    FSTask *task = [[FSTask alloc] init];
    task.run = ^() {

        CGRect cgRect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        cgRect = CGRectInset(cgRect, -20, -20);

        [self->_pdfViewCtrl refresh:cgRect pageIndex:pageIndex];
    };
    [_taskServer executeSync:task];
    
    return  annot;
}

@synthesize type;

@synthesize toolSettings;

@end
