/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SaveAsViewController.h"
#import "PasswordModule.h"
#import "QualityViewController.h"
#import "MenuView+Private.h"
#import "FSReduceFileSizeView.h"
#import "FSSelectStyleObj.h"
#import "FSPermissionUtilty.h"

static NSString *const SelectStyleTableViewCellID = @"SelectStyleTableViewCell";
@interface SaveAsViewController ()<UITableViewDelegate, UITableViewDataSource, IDocEventListener>

@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, strong) FSSelectStyleObj *currentCellObj;
@property (nonatomic, copy) NSString *pdfFilePath;
@property (nonatomic, copy) NSString *pdfFileName;
@property (nonatomic, copy) NSString *saveAsPdfFilePath;
@property (nonatomic, assign) BOOL canFlattenCopy;
@property (nonatomic, assign) BOOL canOptimizeDoc;
@property (nonatomic, assign) FSImageSettingsImageCompressQuality colorCompressQuality;
@property (nonatomic, assign) FSMonoImageSettingsMonoImageCompressQuality monoCompressQuality;
@property (nonatomic, assign) BOOL isCompress;
@property (nonatomic, assign) unsigned long long beforeCompressSize;
@property (nonatomic, assign) unsigned long long afterCompressSize;
@property (nonatomic, assign) BOOL isScanTrial;
@end

@implementation SaveAsViewController

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
    self.shouldLoadDataOnViewDidLoad = YES;
    self.cellSelectedColor = self.fs_dividerColor_P1;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = FSLocalizedForKey(@"kSaveAs");
    
    [self tableView:self.tableView selectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:1]];
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    _colorCompressQuality = FSImageSettingsImageCompressQualityMedium;
    _monoCompressQuality = FSMonoImageSettingsImageCompressQualityLossless;
    _isCompress = NO;
    _canOptimizeDoc = YES;

    [self.navBar.leftItem setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    [self.navBar.leftItem setImage:nil forState:UIControlStateNormal];
    [self.navBar.rightItem setTitle:FSLocalizedForKey(@"kSave") forState:UIControlStateNormal];
    [self.navBar.leftItem addTarget:self action:@selector(clickedCancel:) forControlEvents:UIControlEventTouchUpInside];
    [self.navBar.rightItem addTarget:self action:@selector(clickedOK:) forControlEvents:UIControlEventTouchUpInside];
    [self.navBar updateItem];
}

- (void)tableView:(UITableView *)tableView selectRowAtIndexPath:(NSIndexPath *)indexPath{
    
    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:0];
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)loadData{
    NSArray *tempArr = nil;
    if ([self.pdfViewCtrl getCropMode] != PDF_CROP_MODE_NONE) {
        tempArr = @[ FSLocalizedForKey(@"kOriginalDocument"), FSLocalizedForKey(@"kFlattenedCopy"), FSLocalizedForKey(@"kCroppedCopy"), FSLocalizedForKey(@"kReducedFileSizeCopy")];
    }else{
        tempArr = @[ FSLocalizedForKey(@"kOriginalDocument"), FSLocalizedForKey(@"kFlattenedCopy"), FSLocalizedForKey(@"kReducedFileSizeCopy")];
    }
    

    NSMutableArray *styleData = @[].mutableCopy;
    for (int i = 0 ; i < tempArr.count; i++) {
        
        NSString *styleText = tempArr[i];
        if ([styleText isEqualToString:FSLocalizedForKey(@"kFlattenedCopy")]) {
            if (!self.canFlattenCopy) continue;
        }else if ([styleText isEqualToString:FSLocalizedForKey(@"kReducedFileSizeCopy")]){
            @try {
                if (![FSLibrary hasModuleLicenseRight:FSModuleNameOptimizer] || !self.canOptimizeDoc) continue;
            } @catch (NSException *exception) {
                continue;
            }
        }
        FSSelectStyleObj *obj = [[FSSelectStyleObj alloc] init];
        obj.styleText = tempArr[i];
        obj.selected = NO;
        obj.styleID = i;
        if (i == 0) {
            obj.selected = YES;
            self.currentCellObj = obj;
        }
        [styleData addObject:obj];
    }
    self.dataSource = styleData.copy;
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return self.dataSource[indexPath.row];
}

- (void)setUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    _extensionsManager = extensionsManager;
    _pdfViewCtrl = extensionsManager.pdfViewCtrl;
    self.pdfFilePath = [self.pdfViewCtrl filePath];
    FSPDFViewCtrl *pdfViewCtrl = self.pdfViewCtrl;
    BOOL docHasSignature = [Utility isDocumentSigned:[pdfViewCtrl currentDoc]];
    BOOL allowEdit = docHasSignature ? NO : [FSPermissionUtilty canModifyContents:_extensionsManager];
    self.canFlattenCopy = !(!allowEdit || ![FSPermissionUtilty canAddAnnot:_extensionsManager]) && [_extensionsManager annotsPermissionCanModify] && [_extensionsManager annotsPermissionCanDelete];
    self.canOptimizeDoc = self.canFlattenCopy && ![[pdfViewCtrl currentDoc] isXFA] && ![Utility simpleCheckPDFA:[pdfViewCtrl currentDoc]];
}

- (void)setPdfFilePath:(NSString *)pdfFilePath{
    _pdfFilePath = pdfFilePath;
    if (pdfFilePath.length) {
        self.pdfFileName = [[pdfFilePath lastPathComponent] stringByDeletingPathExtension];
        self.saveAsPdfFilePath = [NSString stringWithFormat:@"%@/%@",DOCUMENT_PATH,self.pdfFileName];
    }
}

- (BOOL)saveDoc:(NSString *)file_path flag:(unsigned int)flag{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if ([fileManager fileExistsAtPath:file_path]){
        NSString *tmp_path = [NSTemporaryDirectory() stringByAppendingPathComponent:@"save_as_tmp.pdf"];
        if ([fileManager fileExistsAtPath:tmp_path]) {
            if (![fileManager removeItemAtPath:tmp_path error:nil]) return NO;
        }
        if ([self.pdfViewCtrl saveDoc:tmp_path flag:flag]) {
            if ([fileManager fileExistsAtPath:file_path]) {
                if (![fileManager removeItemAtPath:file_path error:nil]) return NO;
            }
            return [fileManager moveItemAtPath:tmp_path toPath:file_path error:nil];
        }
    }
    return [self.pdfViewCtrl saveDoc:file_path flag:flag];
}

- (IBAction)clickedCancel:(id)sender {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)clickedOK:(id)sender {
    if ([self.currentCellObj.styleText isEqualToString:FSLocalizedForKey(@"kReducedFileSizeCopy")]) {
        self.beforeCompressSize = [self pdfFileSize];
    }
    
    if (self.extensionsManager){
        [self.extensionsManager setCurrentToolHandler:nil];
    }
    
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
    selectDestination.expectFileType = @[@"*"];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        void (^saveAs)(NSString *) = ^(NSString *pdfFilePath){
            [controller dismissViewControllerAnimated:NO completion:nil];
            self.saveAsPdfFilePath = pdfFilePath;
            if (self.saveAsPdfFilePath.length) {
                    dispatch_semaphore_t wait = dispatch_semaphore_create(0);
                    void (^dismissViewAndOpenDoc)(void) = ^(){
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [self dismissViewControllerAnimated:YES completion:^{
                                [self reopenDoc:pdfFilePath];
                                dispatch_semaphore_signal(wait);
                            }];
                        });
                        dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
                    };

                    if ([self.currentCellObj.styleText isEqualToString:FSLocalizedForKey(@"kFlattenedCopy")]) {
                        UIAlertController *alertController = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning")
                                                  message:FSLocalizedForKey(@"kFlattenedCopyTip")
                                                  actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")]
                                               actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                                   if (index == 1) {
                                                       [self.view fs_showHUDLoading:FSLocalizedForKey(@"kSaving") process:^{
                                                           FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
                                                           if (![xfaDoc isEmpty] && xfaDoc && [xfaDoc getType] == FSXFADocDynamic) {
                                                               [xfaDoc flattenTo:self.saveAsPdfFilePath];
                                                           }else{
                                                               FSPDFDoc *doc = [self.pdfViewCtrl currentDoc];
                                                               for (int i = 0 ; i < [doc getPageCount]; i ++){
                                                                   FSPDFPage *page = [doc getPage:i];
                                                                   [page flatten:YES options:FSPDFPageFlattenAll];
                                                               }
                                                               [self saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagNormal];
                                                           }
                                                           dismissViewAndOpenDoc();
                                                       }];
                                                   }
                        }];
                        [self presentViewController:alertController animated:YES completion:nil];
                    }else if ([self.currentCellObj.styleText isEqualToString:FSLocalizedForKey(@"kCroppedCopy")]) {
                        
                        [self.view fs_showHUDLoading:FSLocalizedForKey(@"kSaving") process:^{
                            
                            FSPDFDoc *doc = [self.pdfViewCtrl currentDoc];
                            for (int i = 0 ; i < [doc getPageCount]; i ++){
                                UIEdgeInsets edgeInsets = [self.pdfViewCtrl getCropInsets:i];
                                FSPDFPage *page = [doc getPage:i];
                                if (!page || [page isEmpty]) {
                                    continue;;
                                }

                                FSRectF *mediaBox = [page getBox:FSPDFPageMediaBox];
                                FSRectF *oldCropRect = [page getBox:FSPDFPageCropBox];
                                if ([oldCropRect isEmpty]) {
                                    oldCropRect = mediaBox;
                                }

                                FSRectF *cropRect = [[FSRectF alloc] initWithLeft1:oldCropRect.left + edgeInsets.left bottom1:oldCropRect.bottom +  edgeInsets.bottom right1:(oldCropRect.right - edgeInsets.right) top1:(oldCropRect.top - edgeInsets.top)];
                                
                                [page setBox:FSPDFPageCropBox box:cropRect];
                            }
                            [self saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagNormal];
                            
                            dismissViewAndOpenDoc();
                        }];
                        
                    }else if ([self.currentCellObj.styleText isEqualToString:FSLocalizedForKey(@"kReducedFileSizeCopy")]) {

                        void (^block) (void) = ^(){
                            if (self.pdfViewCtrl) {
                                if ([[self.pdfViewCtrl currentDoc] getSignatureCount] > 0) {
                                    [self saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagIncremental];
                                }else{
                                    [self saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagNormal];
                                }
                                
                            }
                            [self reopenDoc:pdfFilePath];
                        };
                        [self dismissViewControllerAnimated:NO completion:^{

                            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kOptimizing") process:^{
                                
                                FSImageSettings *imageSettings = [[FSImageSettings alloc] init];
                                [imageSettings setQuality:self.colorCompressQuality];
                                [imageSettings setCompressionMode:FSImageSettingsImageCompressjpeg];
                                
                                FSMonoImageSettings *monoSettings = [[FSMonoImageSettings alloc] init];
                                [monoSettings setQuality:self.monoCompressQuality];
                                [monoSettings setCompressionMode:FSMonoImageSettingsImageCompressjbig2];
                                
                                FSOptimizerSettings *optitmizeSettings = [[FSOptimizerSettings alloc] init];
                                [optitmizeSettings setColorGrayImageSettings:imageSettings];
                                [optitmizeSettings setMonoImageSettings:monoSettings];
                                [optitmizeSettings setOptimizerOptions:FSOptimizerSettingsOptimizerCompressImages];
                                
                                FSProgressive *progressive;
                                
                                @try {
                                    progressive = [FSOptimizer optimize:[self.pdfViewCtrl currentDoc] settings:optitmizeSettings pause:nil];
                                    
                                    if (progressive) {
                                        FSProgressiveState proState = FSProgressiveToBeContinued;
                                        while (proState == FSProgressiveToBeContinued) {
                                            proState = [progressive resume];
                                        }
                                        if (proState != FSProgressiveFinished){
                                            return;
                                        }
                                    }
                                } @catch (NSException *exception) {
                                    dispatch_async(dispatch_get_main_queue(), ^{
                                        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kOptimizeFailed") onCtrl:self.pdfViewCtrl.fs_viewController];
                                    });
                                    return;
                                }
                                
                                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.08 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                                    block();
                                });
                            }];
                        }];
                        
                    }else{
                            
                            void (^saveBlock) (void) = ^(){
                                if (self.pdfViewCtrl) {
                                    if ([[self.pdfViewCtrl currentDoc] getSignatureCount] > 0) {
                                        [self saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagIncremental];
                                    }else{
                                        [self saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagNormal];
                                    }
                                    
                                }
                                dismissViewAndOpenDoc();
                            };
                            
                            [self.view fs_showHUDLoading:FSLocalizedForKey(@"kSaving") process:^{
                                saveBlock();
                            }];
                        }
                    
            }
        };
        
        NSString *pdfFilePath = [[destinationFolder[0] stringByAppendingPathComponent:self.pdfFileName] stringByAppendingPathExtension:[self.pdfFilePath pathExtension]];
        NSFileManager *fileManager = [[NSFileManager alloc] init];
        if ([fileManager fileExistsAtPath:pdfFilePath]) {
            
//            if (self.extensionsManager.isViewSignedDocument) {
//                FSAlertViewCtrlShow(FSLocalizedForKey(@"kWarning"), FSLocalizedForKey(@"kCouldNotSavedTip"));
//                return;
//            }
            
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                       style:UIAlertActionStyleCancel
                                                                     handler:^(UIAlertAction *action) {
                                                                         [self tableView:self.tableView didSelectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
                                                                         [controller.navBar.rightItem setEnabled:YES];
                                                                     }];
                UIAlertAction *replaceAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kReplace")
                                                                        style:UIAlertActionStyleDefault
                                                                      handler:^(UIAlertAction *action) {
                                                                        saveAs(pdfFilePath);
                                                                      }];
                [alertController addAction:cancelAction];
                [alertController addAction:replaceAction];
                [controller presentViewController:alertController animated:YES completion:nil];
            });
        } else {
            saveAs(pdfFilePath);
        }
    };

    FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
    if (DEVICE_iPHONE) {
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFullScreen;
    }else{
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFormSheet;
    }
    [self.extensionsManager.viewCtrlManager presentViewController:selectDestinationNavController
                                     animated:YES
                                   completion:nil];
    
}

- (void)reopenDoc:(NSString *)pdfFilePath
{
        if (self.extensionsManager) {
            void(^popCompressTipBlcok)(void) = ^(){
                if (self.beforeCompressSize > 0) {
                    self.afterCompressSize = [self pdfFileSize];
                  
                    BOOL isSuccess = YES;
                    if (self.afterCompressSize > self.beforeCompressSize) {
                        isSuccess = NO;
                    }
                    NSString *msgStr = [NSString stringWithFormat:FSLocalizedForKey(@"kOptimizeSizeTip"),[Utility displayFileSize:self.beforeCompressSize],[Utility displayFileSize:self.afterCompressSize]];
                    FSReduceFileSizeView *noticeView = [[FSReduceFileSizeView alloc] initWithState:isSuccess msg:msgStr];
                    FS_TRANSFER_WINDOW_SCENE(noticeView, self->_pdfViewCtrl);
                    [self.extensionsManager.pdfViewCtrl addSubview:noticeView];
                    [noticeView mas_makeConstraints:^(MASConstraintMaker *make) {
                        make.left.right.top.bottom.equalTo(self.extensionsManager.pdfViewCtrl);
                    }];
                }
            };
            if ([self.extensionsManager.delegate respondsToSelector:@selector(uiextensionsManager:openNewDocAtPath:shouldCloseCurrentDoc:completionCallBack:)]) {
                [self.extensionsManager.delegate uiextensionsManager:self.extensionsManager openNewDocAtPath:pdfFilePath shouldCloseCurrentDoc:YES completionCallBack:^{}];
                popCompressTipBlcok();
            } else {
                [self.extensionsManager.pdfViewCtrl openDoc:pdfFilePath
                                                   password:self.extensionsManager.passwordModule.inputPassword
                                                 completion:^(FSErrorCode error) {
                    if (error != FSErrSuccess) {
                        [self.extensionsManager.pdfViewCtrl closeDoc:nil];
                        if (self.extensionsManager.goBack)
                            self.extensionsManager.goBack();
                    }else{
                        popCompressTipBlcok();
                    }
                }];
            }
            self.extensionsManager.isDocModified = NO;
            self.extensionsManager.more.hidden = YES;
        }else{
            !self.saveAsCallBack ? : self.saveAsCallBack(nil, pdfFilePath);
        }

};

- (unsigned long long)pdfFileSize{
    unsigned long long fileSize = 0;
    NSFileManager *manager = [NSFileManager defaultManager];
    if ([manager fileExistsAtPath:self.pdfViewCtrl.filePath]) {
        fileSize = [[manager attributesOfItemAtPath:self.pdfViewCtrl.filePath error:nil] fileSize];
    }
    return fileSize;
}

#pragma mak UITableViewDataSource UITableViewDelegate

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0) {
        return 1;
    }
    return self.dataSource.count;
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    if (section == 0) {
        return FSLocalizedForKey(@"kFileName");
    }
    return FSLocalizedForKey(@"kFormat");
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSSelectStyleObj *)obj{
    if (indexPath.section == 0) {
        if (!cell.installed) {
            cell.rightMargin = ITEM_MARGIN_NORMAL;
        }
        cell.contentView.backgroundColor = self.fs_backgroundColor_B8;
        cell.estimatedTitleLabel.text = self.pdfFileName;
        
        if (!cell.rightView) {
            FSItem *clearItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_clear_gray")];
            @fs_weakify(self)
            @fs_weakify(cell)
            clearItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                @fs_strongify(cell)
                self.pdfFileName = @"";
                cell.estimatedTitleLabel.text = @"";
            };
            cell.rightView = clearItem;
            cell.rightMargin = ITEM_TOP_ICON_MARGIN;
        }
    }else{
        cell.contentView.backgroundColor = [UIColor clearColor];
        cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
        UIImageView *imageView = cell.leftView;
        if (!imageView) {
            imageView = [UIImageView new];
            imageView.hidden = YES;
            cell.leftView = imageView;
        }
        
        [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image = FSImageNamedAndPrimaryColor(@"common_select_check");
        }];
        
        if (!cell.rightView) {
            FSItem *accessoryItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_accessory_gray")];
            cell.rightView = accessoryItem;
        }
        
        if ([obj.styleText isEqualToString:FSLocalizedForKey(@"kReducedFileSizeCopy")]) {
            cell.rightView.hidden = NO;
            cell.rightView.accessibilityLabel = FSLocalizedForKey(@"kQuality");
            @weakify(self)
            ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                QualityViewController *qualityVC = [[QualityViewController alloc] init];
                qualityVC.colorCompressQuality = self.colorCompressQuality;
                qualityVC.monoCompressQuality = self.monoCompressQuality;
                qualityVC.didSelectColorBlock = ^(NSUInteger compressQuality, NSString * _Nullable qualityTitle) {
                    self.colorCompressQuality = compressQuality + 1;
                };
                qualityVC.didSelectMonoBlock = ^(NSUInteger compressQuality, NSString * _Nullable qualityTitle) {
                    self.monoCompressQuality = compressQuality + 1;
                };
                [self.navigationController pushViewController:qualityVC animated:YES];
            };
        }else{
            cell.rightView.hidden = YES;
        }
        
        cell.estimatedTitleLabel.text = obj.styleText;
        cell.leftView.hidden = !obj.selected;
        if (obj.selected) {
            cell.accessibilityTraits = UIAccessibilityTraitSelected;
        }else{
            cell.accessibilityTraits = UIAccessibilityTraitNone;
        }
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section == 1) {
        if (self.dataSource.count > 1) {
            NSIndexPath *lastIndexPath = [NSIndexPath indexPathForRow:[self.dataSource indexOfObject:self.currentCellObj] inSection:1];
            self.currentCellObj.selected = NO;
            FSSelectStyleObj *obj = self.dataSource[indexPath.row];
            obj.selected = YES;
            self.currentCellObj = obj;
            
            FSTitleEstimatedTableViewCell *lastCell = [tableView cellForRowAtIndexPath:lastIndexPath];
            [self configureCell:lastCell atIndexPath:lastIndexPath withDataItem:[self dataForAtIndexPath:lastIndexPath]];
            FSTitleEstimatedTableViewCell *currentCell = [tableView cellForRowAtIndexPath:indexPath];
            [self configureCell:currentCell atIndexPath:indexPath withDataItem:[self dataForAtIndexPath:indexPath]];
        }
    }else{
        
        [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kRename") message:@"" textFieldText:self.pdfFileName actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
            
            if (index == 1) {
                NSString *fileName = textField.text;
                
                if ([fileName rangeOfString:@"/"].location != NSNotFound) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        
                        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kIllegalNameWarning") preferredStyle:UIAlertControllerStyleAlert];
                        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                         style:UIAlertActionStyleCancel
                                                                       handler:nil];
                        [alertController addAction:action];
                        [self presentViewController:alertController animated:YES completion:nil];
                    });
                    return;
                } else if (fileName.length == 0) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        
                    });
                    return;
                }
                self.pdfFileName = fileName;
                FSTitleEstimatedTableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
                [self configureCell:cell atIndexPath:indexPath withDataItem:[self dataForAtIndexPath:indexPath]];
            }
            
        }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
            if (index == 1) {
                return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length && textField.text.hash != self.pdfFileName.hash;
            }
            return YES;
        }] fs_showByViewController:self] ;
   
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
   UIView *headerView = [super tableView:tableView viewForHeaderInSection:section];
   headerView.backgroundColor = self.fs_backgroundColor_B9;
   return headerView;
}
/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end

