/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "NoteToolHandler.h"

@interface NoteToolHandler () <IDocEventListener>
@end

@implementation NoteToolHandler {
    UIExtensionsManager * __weak _extensionsManager;
    FSPDFViewCtrl * __weak _pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotNote;
    }
    return self;
}

- (NSString *)getName {
    return Tool_Note;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self || pageIndex < 0) {
        return NO;
    }
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [recognizer locationInView:pageView];
        [self addAnnot:pageIndex point:point];
    }
    return YES;
}

- (BOOL)addAnnot:(int)pageIndex point:(CGPoint)point {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGRect rect1 = [pageView frame];
    CGSize size = rect1.size;
    if (point.x > size.width || point.y > size.height || point.x < 0 || point.y < 0)
        return NO;
    float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000.0;
    
    FSRectF *iconfsrect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:NOTE_ANNOTATION_WIDTH top1:NOTE_ANNOTATION_WIDTH];
    CGRect iconrect = [_pdfViewCtrl convertPdfRectToPageViewRect:iconfsrect pageIndex:pageIndex];
    float iconwidth = iconrect.size.width;
    CGRect rect = CGRectMake(point.x - NOTE_ANNOTATION_WIDTH * scale / 2, point.y - NOTE_ANNOTATION_WIDTH * scale / 2, iconwidth, iconwidth);
    
    FSRectF *dibRect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:pageIndex];
    
    FSPointF *tempPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSRectF *annotTempRect = [[FSRectF alloc] init];
    annotTempRect.left = tempPoint.x;
    annotTempRect.bottom = tempPoint.y - NOTE_ANNOTATION_WIDTH;
    annotTempRect.right = annotTempRect.left + NOTE_ANNOTATION_WIDTH;
    annotTempRect.top = annotTempRect.bottom + NOTE_ANNOTATION_WIDTH;
    dibRect = annotTempRect;

    FSTextViewPopoverViewController *noteDialog = [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:_extensionsManager.topToolbar.getToolbarContentView];
    noteDialog.okItemEnabled = ^BOOL(NSString *text) {
        return YES;
    };
    noteDialog.title = FSLocalizedForKey(@"kNote");
    self.noteDialog = noteDialog;
    @fs_weakify(self)
    noteDialog.didClickedOK = ^(NSString *text) {
        @fs_strongify(self)
        FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:pageIndex];
        if (!page || [page isEmpty])
            return;
        FSNote *annot = [[FSNote alloc] initWithAnnot:[page addAnnot:FSAnnotNote rect:dibRect]];
        annot.icon = self.toolSettings.noteIcon;
        annot.color = self.toolSettings.settingsColor;
        annot.opacity = self.toolSettings.opacity;
        annot.contents = text;
        annot.NM = [Utility getUUID];
        annot.author = self->_extensionsManager.annotAuthor;
        annot.subject = @"Note";
        annot.flags = FSAnnotFlagPrint | FSAnnotFlagNoZoom | FSAnnotFlagNoRotate;
        FSDateTime *now = [Utility convert2FSDateTime:[NSDate date]];
        [annot setCreationDateTime:now];
        [annot setModifiedDateTime:now];
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
        [annotHandler addAnnot:annot];
    };
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    return [self addAnnot:pageIndex point:point];
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
}

#pragma mark IDocEventListener

- (void)onDocWillClose:(FSPDFDoc *)document {
    if (self.noteDialog) {
        [self.noteDialog dismissViewControllerAnimated:NO completion:nil];
    }
}

@synthesize toolSettings;

@end
