/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReplyTableViewController.h"
#import "FSUndo.h"
#import "Masonry/Masonry.h"
#import "UINavigationItem+IOS7PaddingAdditions.h"
#import "MultipleSelectionToolHandler.h"
#import "FSAnnotHandler.h"
#import "FSPermissionUtilty.h"

@interface ReplyTableViewController () <AnnotationListCellDelegate, IAnnotEventListener, IPageEventListener>

@property (nonatomic, strong) NSMutableArray *rootAnnos;
@property (nonatomic, strong) NSMutableDictionary *annoStruct;
@property (nonatomic, strong) NSMutableDictionary *nodeAnnos;
@property (nonatomic, assign) BOOL canDeleteAnnot;
@end

@implementation ReplyTableViewController {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    __weak FSPDFDoc *_document;
}

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleDescription;
}

- (id)initWithStyle:(UITableViewStyle)style extensionsManager:(UIExtensionsManager *)extensionsManager {
    _extensionsManager = extensionsManager;
    _pdfViewCtrl = extensionsManager.pdfViewCtrl;
    _document = _pdfViewCtrl.currentDoc;

    if (self = [super initWithServices:extensionsManager.viewCtrlManager]) {
        _rootAnnos = [[NSMutableArray alloc] init];
        _annoStruct = [[NSMutableDictionary alloc] init];
        _nodeAnnos = [[NSMutableDictionary alloc] init];

        self.indexPath = nil;
        self.moreIndexPath = nil;
        self.isShowMore = NO;
        
        self.isShowMenu = extensionsManager.shouldShowMenu;
        extensionsManager.shouldShowMenu = NO;
    }
    [_pdfViewCtrl registerPageEventListener:self];
    return self;
}

- (void)setTableViewAnnotations:(NSArray *)annotatios {
    if (annotatios.count == 0)
        return;

    NSMutableArray *itemArray = [NSMutableArray array];

    for (FSAnnot *anno in annotatios) {
        AnnotationItem *item = [[AnnotationItem alloc] init];
        item.annot = anno;
        item.currentlevelshow = NO;
        item.currentlevel = 1;

        if (item.annot.replyTo.length == 0) {
            item.isSecondLevel = YES;
            [self.rootAnnos addObject:item];
        }

        [itemArray addObject:item];
    }

    [self.annoStruct addEntriesFromDictionary:[AnnotationStruct getAnnotationStructWithAnnos:itemArray]];

    for (AnnotationItem *anno in self.rootAnnos) {
        NSMutableArray *tempArray = [NSMutableArray array];
        [tempArray addObject:anno];
        NSString *nm = [anno.annot NM];

        if (1 > nm.length)
            return;
        [self.nodeAnnos setObject:tempArray forKey:nm];

        self.title =  anno.annot.author;
    }
    [self.tableView reloadData];
}

- (void)clearData {
    [_rootAnnos removeAllObjects];
    [_nodeAnnos removeAllObjects];
    [_annoStruct removeAllObjects];
}

- (void)dealloc {
    _extensionsManager.shouldShowMenu = self.isShowMenu;
    self.editingCancelHandler = nil;
    self.editingDoneHandler = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.canDeleteAnnot = YES;
    [self initNavigationBar];

    [self performSelector:@selector(openFirstLevel) withObject:nil afterDelay:0.3];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [_extensionsManager registerAnnotEventListener:self];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [_extensionsManager unregisterAnnotEventListener:self];
}


- (void)openFirstLevel {
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
   
    AnnotationItem *annotItem = [self dataForAtIndexPath:indexPath];
    [self getDetailReply:annotItem atIndexPath:indexPath];

    if (self.isNeedReply) {
        [self replyToAnnotation:annotItem];
    }
}

- (void)initNavigationBar {
    [self.navBar.leftItem addTarget:self action:@selector(deleteAction) forControlEvents:UIControlEventTouchUpInside];
    [self.navBar.rightItem addTarget:self action:@selector(doneAction) forControlEvents:UIControlEventTouchUpInside];

    self.navBarLeftTitle = FSLocalizedForKey(@"kDelete");
    self.navBarRightTitle = FSLocalizedForKey(@"kDone");

    if ([_extensionsManager.currentToolHandler isKindOfClass:[MultipleSelectionToolHandler class]]) {
        FSAnnot *multipleSelectionCurrentAnoot = [(MultipleSelectionToolHandler *)_extensionsManager.currentToolHandler currentAnoot];
        if(multipleSelectionCurrentAnoot && !multipleSelectionCurrentAnoot.canDelete){
            self.navBar.leftItem.enabled = NO;
        }
    }else{
        if (![_extensionsManager.currentAnnot canDeleteWithExternsions:_extensionsManager]) {
            self.navBar.leftItem.enabled = NO;
        }
    }
}

- (void)deleteAction {
    
    if ([_extensionsManager.currentToolHandler isKindOfClass:[MultipleSelectionToolHandler class]]) {
        FSAnnot *annot = [(MultipleSelectionToolHandler *)_extensionsManager.currentToolHandler currentAnoot];
        [(MultipleSelectionToolHandler *)_extensionsManager.currentToolHandler selectedGroupAnnotForDelete:annot];
        [_extensionsManager setCurrentToolHandler:nil];
        [self.navigationController dismissViewControllerAnimated:YES completion:nil];
        [self clearData];
    }else{
        AnnotationItem *rootItem = [self.rootAnnos objectAtIndex:0];
        [self deleteAnnotation:rootItem];

        [self.navigationController dismissViewControllerAnimated:YES completion:nil];
        [self clearData];
    }
}

- (void)doneAction {
    if (self.editAnnoItem || (self.replyanno && self.replyanno.isReply == YES)) {

        if (self.editAnnoItem) {
            self.editAnnoItem = nil;
        }

        if (self.replyanno && self.replyanno.isReply == YES) {
            self.replyanno.isReply = NO;
        }
    }

    [self.navigationController dismissViewControllerAnimated:YES completion:^{
        ! self.editingDoneHandler ? : self.editingDoneHandler();
    }];

    [self clearData];
    [_extensionsManager setCurrentAnnot:nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.rootAnnos.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section >= self.rootAnnos.count) {
        return 0;
    }
    AnnotationItem *item = [self.rootAnnos objectAtIndex:section];
    if (item.isDeleted)
        return 0;
    return [[self.nodeAnnos objectForKey:[[item annot] NM]] count];
}

- (AnnotationItem *)dataForAtIndexPath:(NSIndexPath *)indexPath{
    if (indexPath.section >= self.rootAnnos.count) {
        return nil;
    }
    AnnotationItem *item = (AnnotationItem *) self.rootAnnos[indexPath.section];
    NSArray *itemArray = self.nodeAnnos[item.annot.NM];
    if (indexPath.row >= itemArray.count) {
        return nil;
    }
    return itemArray[indexPath.row];
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(AnnotationItem *)annoItem{
    if (!annoItem.annot.contents.length) {
        identifier = NSStringFromClass([FSSubjectEstimatedTableViewCell class]);
    }
    return  [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:annoItem];
}

- (void)configureCell:(FSDescriptionEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(AnnotationItem *)annoItem{
    if (!cell.installed) {
        cell.editTopOffset = ITEM_IMAGE_TOP_MARGIN_NORMAL;
        cell.leftMargin = 9.f;
    }
    
    if (!cell.leftView) {
        UIView *leftView = [UIView new];
        cell.leftView = leftView;
    }

    if (!cell.rightView) {
        FSItem *moreItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
        moreItem.accessibilityIdentifier = @"more_item_id";
        cell.rightView = moreItem;
    }
    
    cell.accessibilityIdentifier = @"note_cell_id";
    cell.estimatedTitleLabel.text = annoItem.annot.author;
    cell.estimatedSubjectLabel.text = [Utility displayDateInMDYHM:annoItem.annot.modifiedDate];
    [cell.leftView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
    
    if (annoItem.annot.replyTo) {
        FSItem *unfold = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"panel_unfold_+_gray") imageSelected:FSImageNamedAndNormalColor(@"panel_unfold_-_gray")];
        unfold.selected = annoItem.currentlevelshow;
        unfold.tag = 200;
        ///Unfold and fold
        @weakify(self)
        unfold.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self getDetailReply:annoItem atIndexPath:indexPath];
        };
        
        AnnotationItem *rootAnnoItem = [self.rootAnnos objectAtIndex:0];
        if (![rootAnnoItem.annot.NM isEqualToString:annoItem.annot.replyTo]) {
            unfold.hidden = YES;
        }
        if (!unfold.hidden && [[self.annoStruct objectForKey:annoItem.annot.uuidWithPageIndex] count] <= 0) {
            unfold.hidden = YES;
        }

        annoItem.currentlevelbutton = unfold;
        
        UIImageView *turnImageView = [[UIImageView alloc] initWithImage:FSImageNamedAndNormalColor(@"panel_reply_arrow_gray")];
        [cell.leftView addSubview:unfold];
        [cell.leftView addSubview:turnImageView];
        
        [unfold mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.top.bottom.fs_equalToSuperview();
        }];

        [turnImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(unfold.mas_right).offset(0);
            make.size.mas_equalTo(turnImageView.image.size);
            make.centerY.mas_equalTo(unfold);
            make.right.fs_equalToSuperview();
        }];
        
        cell.estimatedTitleLabel.text = [NSString stringWithFormat:@"%@ to %@", annoItem.annot.author ? annoItem.annot.author : @"", annoItem.replytoauthor ? annoItem.replytoauthor : @""];
    }else{
        UIImageView *annotImage = [UIImageView new];
        annotImage.image = FSImageNamedAndNormalColor([Utility getAnnotImageNameForAnnot:annoItem.annot]);
        [cell.leftView addSubview:annotImage];
        
        [annotImage mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.top.right.bottom.fs_equalToSuperview();
        }];
    }
    
    BOOL canEdit = [self annotationListItemCanEdit:annoItem];
    NSMutableArray *items = @[].mutableCopy;
    @weakify(self)
    if ([self annotationListItemCanReply:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kReply") image:FSImageNamedAndNormalColor(@"panel_reply_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            @strongify(self)
            [self replyToAnnotation:annoItem];
        }]];
    }
    if (canEdit) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kNote") image:FSImageNamedAndNormalColor(@"panel_note_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            @strongify(self)
            [self addNoteToAnnotation:annoItem atIndexPath:indexPath];
        }]];
    }
    if ([self annotationListItemCanApplyRedaction:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kApply") image:FSImageNamedAndNormalColor(@"more_redact_apply") action:^(FSPopoverItem * _Nonnull item, int index) {
            @strongify(self)
            [self annotationListItemApplyRedaction:annoItem atIndexPath:indexPath];
        }]];
    }
    if ([self annotationListItemCanFlatten:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kFlatten") image:FSImageNamedAndNormalColor(@"panel_flatten_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            [_extensionsManager.permissionProvider checkPermission:FSFunctionFlatten checkPermissionState:^(FSPermissionState state) {
                @strongify(self)
                if (state == FSPermissionStateShow) {
                    [self annotationListItemFlatten:annoItem atIndexPath:indexPath];
                }
            }];
        }]];
    }
    if ([self annotationListItemCanDelete:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            @strongify(self)
            [self annotationListItemDelete:annoItem atIndexPath:indexPath];
        }]];
    }
    ((FSItem *)cell.rightView).enabled = items.count;
    ((FSItem *)cell.rightView).hidden = _isGroup;
    
    ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        
        UIColor *oriColor = _item.superview.backgroundColor;
        if (DEVICE_iPHONE) _item.superview.backgroundColor = self.fs_backgroundColor_B2;
        [self->_extensionsManager.viewCtrlManager showMoreMenuWithItems:items.copy dismissCompletion:DEVICE_iPHONE  ? ^{
            _item.superview.backgroundColor = oriColor;
        } : nil sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return _item;
        } : nil];
        
    };
    
    cell.editItem.hidden = ![self annotationListItemCanDelete:annoItem];
    if ([cell respondsToSelector:@selector(estimatedDescriptionLabel)]) {
        NSString *contents = annoItem.annot.contents;
        
        if ((contents == nil || contents.length == 0) && annoItem.annot.screenActiontype != FSScreenActionTypeUnScreen) {
            contents = annoItem.annot.ha_fileSpec.fileName;
        }
        cell.estimatedDescriptionLabel.accessibilityIdentifier = @"note_content_id";
        cell.estimatedDescriptionLabel.text = contents;
        
        UIButton *editNoteClicked = [cell.contentView viewWithTag:100];
        if (canEdit) {
            if (!editNoteClicked) {
                editNoteClicked = [UIButton new];
                editNoteClicked.tag = 100;
                [cell.contentView addSubview:editNoteClicked];
                
                [editNoteClicked mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.bottom.fs_equalToSuperview();
                    make.top.equalTo(cell.estimatedDescriptionLabel);
                }];
            }
            
            editNoteClicked.hidden = YES;
            
            if ([FSPermissionUtilty canAddAnnot:_extensionsManager] ) {
                editNoteClicked.hidden = NO;
                @weakify(cell)
                [editNoteClicked setBlockForControlEvents:UIControlEventTouchUpInside block:^(id  _Nonnull sender) {
                    @strongify(self)
                    @strongify(cell)
                    if (!cell.editing) [self addNoteToAnnotation:annoItem atIndexPath:indexPath];
                }];
            }
        }else{
            editNoteClicked.hidden = YES;
        }
    }
    [super configureCell:cell atIndexPath:indexPath withDataItem:annoItem];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editAnnoItem || (self.replyanno && self.replyanno.isReply == YES)) {

        if (self.editAnnoItem) {
            self.editAnnoItem = nil;
            _extensionsManager.currentAnnot = nil;
        }

        if (self.replyanno && self.replyanno.isReply == YES) {
            self.replyanno.isReply = NO;
            self.replyanno = nil;
        }

        [self.tableView reloadData];
    }

    [tableView deselectRowAtIndexPath:indexPath animated:YES];

    AnnotationItem *selectanno = [[self.nodeAnnos objectForKey:[[[self.rootAnnos lastObject] annot] NM]] objectAtIndex:indexPath.row];

    if (self.replyanno && self.replyanno == selectanno) {
        return;
    }

    if (![selectanno.annot canEditWithExternsions:_extensionsManager] || ![selectanno.annot canDeleteWithExternsions:_extensionsManager]  ) {
        return;
    }

    selectanno.isEdited = YES;

    self.editAnnoItem = [self dataForAtIndexPath:indexPath];
    _extensionsManager.currentAnnot = self.editAnnoItem.annot;
    self.indexPath = indexPath;
    
    [self addNoteToAnnotation:selectanno atIndexPath:indexPath];

    [self.tableView reloadData];
}


- (BOOL)annotationListItemCanEdit:(AnnotationItem *)item {
    if (![_extensionsManager annotsPermissionCanModify:item.annot]) {
        return NO;
    }
    if (!item) return NO;
    BOOL canEdit = item.annot.canEdit && item.annot.type != FSAnnotFreeText && item.annot.type != FSAnnotScreen;
    if (canEdit) {
        return [FSPermissionUtilty canAddAnnot:_extensionsManager];
    }
    return canEdit;
}

- (BOOL)annotationListItemCanDelete:(AnnotationItem *)item {
    if (_canDeleteAnnot) {
        _canDeleteAnnot = !(![_extensionsManager annotsPermissionCanDelete:item.annot] || ![item.annot canDeleteWithExternsions:_extensionsManager]);
        if (!_canDeleteAnnot) {
            self.navBar.leftItem.enabled = NO;
        }
    }

    return [item.annot canDeleteWithExternsions:_extensionsManager];
}

- (BOOL)annotationListItemCanReply:(AnnotationItem *)item {
    if (![_extensionsManager annotsPermissionCanReply:item.annot]) {
        return NO;
    }
    return item.annot.canReply;
}

- (BOOL)annotationListItemCanFlatten:(AnnotationItem *)item {
    if (![_extensionsManager annotPermissionCanFlatten:item.annot]) {
        return NO;
    }
    return !item.annot.replyTo && item.annot.canModify && ([_extensionsManager.permissionProvider checkPermission:FSFunctionFlatten] != FSPermissionStateHide);
}

- (BOOL)annotationListItemCanApplyRedaction:(AnnotationItem *)item{
    if (![_extensionsManager annotsPermissionCanModify:item.annot] || ![FSPermissionUtilty canApplyRedaction:_extensionsManager] || ![_extensionsManager allAnnotPermission]) {
        return NO;
    }
    return item.annot.canApplyRedaction && _extensionsManager.isSupportRedaction && ([_extensionsManager.permissionProvider checkPermission:FSFunctionRedaction] != FSPermissionStateHide);
}

- (void)addNoteToAnnotation:(AnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath {
    if (![_extensionsManager annotsPermissionCanModify:item.annot]) {
        return;
    }
    FSTextViewPopoverViewController *textViewVC =  [_extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeTextViewPopover params:self.navBar];
    textViewVC.text = item.annot.contents;
    [textViewVC setDidClickedOK:^(NSString *contents) {
        
        FSAnnot *annot = item.annot;
        
        NSString *oldContents = annot.contents;
        NSDate *oldDate = annot.modifiedDate;
        NSDate *now = [NSDate date];
        NSString *NM = annot.NM;
        FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:annot.pageIndex];
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
        
        UndoItem *undoItem = [UndoItem itemWithUndo:^(UndoItem *item) {
            FSAnnot *annot = [Utility getAnnotByNM:NM inPage:page];
            annot.contents = oldContents;
            annot.modifiedDate = oldDate;
            if ([annotHandler respondsToSelector:@selector(modifyAnnot:addUndo:)]) {
                [annotHandler modifyAnnot:annot addUndo:NO];
            } else {
                [annotHandler modifyAnnot:annot];
            }
        }
                        redo:^(UndoItem *item) {
                            FSAnnot *annot = [Utility getAnnotByNM:NM inPage:page];
                            annot.contents = contents;
                            annot.modifiedDate = now;
                            if ([annotHandler respondsToSelector:@selector(modifyAnnot:addUndo:)]) {
                                [annotHandler modifyAnnot:annot addUndo:NO];
                            } else {
                                [annotHandler modifyAnnot:annot];
                            }
                        }
                                          pageIndex:annot.pageIndex];
        [self->_extensionsManager addUndoItem:undoItem];
        
        undoItem.redo(undoItem);
        [self reloadRowsAtIndexPaths:indexPath];
        self->_extensionsManager.isDocModified = YES;
    }];
    
    self.replyanno = nil;
    
    [self presentViewController:textViewVC animated:YES completion:nil];
}

- (void)annotationListItemDelete:(AnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath{
    [self deleteAnnotation:item];
    if (!self.tableView.visibleCells.count) {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}

- (void)annotationListItemFlatten:(AnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kFlatten") message:FSLocalizedForKey(@"kFlattenTip") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
    UIAlertAction *flattenAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFlatten") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {

        [self->_extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
            if (ext.currentAnnot) {
                [ext setCurrentAnnot:nil];
            }
        }];
        // restore to original contents if annot being edited
        AnnotationItem *replytoannotation = item;
        NSMutableArray *deletearray = [NSMutableArray arrayWithObject:replytoannotation];
        [deletearray addObjectsFromArray:[AnnotationStruct getAllChildNodesWithSuperAnnotation:replytoannotation annoStruct:self.annoStruct]];
        [self deletaAnnotatios:deletearray];
        
        FSAnnot *annot = item.annot;
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
        if ([annotHandler flattenAnnot:annot]){
            [self dismissViewControllerAnimated:YES completion:nil];
        }
        item.isDeleted = YES;
    }];
    [alertController addAction:cancelAction];
    [alertController addAction:flattenAction];
    [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
}

- (void)annotationListItemApplyRedaction:(AnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath{
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionRedaction checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            UIAlertController *alert = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kRedaction") message:FSLocalizedForKey(@"kRedactTip") actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    FSRedact *annot = [[FSRedact alloc] initWithAnnot:item.annot];
                    [self->_extensionsManager setCurrentAnnot:nil];
                    int pageIndex = annot.pageIndex;
                    [Utility applyRedactAnnot:annot completed:^(NSArray<FSAnnotAttributes *> * _Nonnull replyAttributes) {
                        [self clearData];
                        [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
                        [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAttachmentLsitUpdate object:nil];
                        [self->_pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex needRender:YES clearCache:YES];
                        self->_extensionsManager.isDocModified = YES;
                        [self->_extensionsManager clearUndoRedo];
                    }];
                    [self dismissViewControllerAnimated:YES completion:nil];
                }
            }];
            [[self fs_currentSceneTopMostViewController] presentViewController:alert animated:YES completion:nil];
        }
    }];
}

#pragma mark Spread TableView
- (void)getDetailReply:(AnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath {
    NSUInteger currentsection = indexPath.section;
    NSUInteger currentrow = indexPath.row;
    AnnotationItem *currentanno = item;
    NSMutableArray *currentarray = [self.nodeAnnos objectForKey:[[[self.rootAnnos objectAtIndex:0] annot] NM]];
    NSMutableArray *addannoarray = [self.annoStruct objectForKey:currentanno.annot.uuidWithPageIndex];

    [addannoarray makeObjectsPerformSelector:@selector(addCurrentlevel:) withObject:[NSNumber numberWithLong:(currentanno.annot.replyTo == nil ? (currentanno.currentlevel + 1) : (currentanno.currentlevel))]];

    [addannoarray makeObjectsPerformSelector:@selector(setReplytoauthor:) withObject:currentanno.annot.author];

    [addannoarray makeObjectsPerformSelector:@selector(setcurrentlevelshow:) withObject:[NSNumber numberWithBool:NO]];

    if (currentanno.annot.replyTo == nil) {
        [addannoarray makeObjectsPerformSelector:@selector(setSecondLevel:) withObject:[NSNumber numberWithBool:YES]];
    }

    if (!currentanno.currentlevelshow) {
        currentanno.currentlevelshow = YES;

        NSUInteger insertrowindex = [currentarray indexOfObject:currentanno];

        [currentarray insertObjects:addannoarray atIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(insertrowindex + 1, addannoarray.count)]];

        NSMutableArray *arCells = [NSMutableArray array];

        for (int i = 0; i < addannoarray.count; i++) {
            currentrow = currentrow + 1;
            [arCells addObject:[NSIndexPath indexPathForRow:currentrow inSection:currentsection]];
        }

        [self.tableView insertRowsAtIndexPaths:arCells withRowAnimation:UITableViewRowAnimationTop];

        if (currentanno.annot.replyTo != nil) {
            for (AnnotationItem *sendOpen in addannoarray) {
                [sendOpen.currentlevelbutton sendActionsForControlEvents:UIControlEventTouchUpInside];
            }
        }

        if (currentanno.isSecondLevel) {
            currentanno.currentlevelshow = YES;
            [self.tableView performSelector:@selector(reloadData) withObject:nil afterDelay:0.2];
        }
    } else {
        NSMutableArray *deleteannos = [NSMutableArray array];

        [self getAboutAnnotatios:currentanno Annoarray:currentarray deleteArray:deleteannos];

        currentanno.currentlevelshow = NO;

        NSUInteger insertrowindex = [currentarray indexOfObject:currentanno];

        [currentarray removeObjectsAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(insertrowindex + 1, deleteannos.count)]];

        NSMutableArray *arCells = [NSMutableArray array];

        for (int i = 0; i < deleteannos.count; i++) {
            currentrow = currentrow + 1;
            [arCells addObject:[NSIndexPath indexPathForRow:currentrow inSection:currentsection]];
        }

        [self.tableView deleteRowsAtIndexPaths:arCells withRowAnimation:UITableViewRowAnimationTop];
        [self.tableView performSelector:@selector(reloadData) withObject:nil afterDelay:0.3];
    }
}

- (void)getAboutAnnotatios:(AnnotationItem *)searchanno Annoarray:(NSArray *)annoarray deleteArray:(NSMutableArray *)deletearray {
    NSArray *searcharray = [AnnotationStruct getAllChildNodesWithSuperAnnotation:searchanno annoStruct:self.annoStruct];

    for (AnnotationItem *annannotationo in searcharray) {
        if ([annoarray containsObject:annannotationo]) {
            [deletearray addObject:annannotationo];
        }
    }
}

#pragma mark AddandUpdateAnnotaion

- (void)deletaAnnotatios:(NSArray *)annos {
    if (annos.count == 0) {
        return;
    }

    NSMutableSet *readydeletes = [NSMutableSet set];

    for (AnnotationItem *childanno in annos) {
        [readydeletes addObject:childanno];
        [readydeletes addObjectsFromArray:[self.annoStruct objectForKey:childanno.annot.uuidWithPageIndex]];
    }

    for (AnnotationItem *deleteanno in readydeletes) {
        [AnnotationStruct deleteAnnotationFromAnnoStruct:self.annoStruct deleteAnnotation:deleteanno rootAnnotation:[self.rootAnnos lastObject]];

        [(NSMutableArray *)[self.nodeAnnos objectForKey:[[[self.rootAnnos lastObject] annot] NM]] removeObject:deleteanno];

        if (self.replyanno == deleteanno)
            self.replyanno = nil;
    }

    if (self.nodeAnnos.count == 0) {
        [self.rootAnnos removeAllObjects];
    }

    [self.tableView reloadData];
}

- (void)deleteAnnotation:(AnnotationItem *)item {
    if (item.isDeleted) {
        return;
    }

    // restore to original contents if annot being edited
    AnnotationItem *replytoannotation = item;

    NSMutableArray *deletearray = [NSMutableArray arrayWithObject:replytoannotation];

    [deletearray addObjectsFromArray:[AnnotationStruct getAllChildNodesWithSuperAnnotation:replytoannotation annoStruct:self.annoStruct]];

    [self deletaAnnotatios:deletearray];

    if (_extensionsManager.currentAnnot) {
        if (![_extensionsManager.currentAnnot isEqualToAnnot:replytoannotation.annot]) {
            _extensionsManager.currentAnnot = nil;
        }
    }
    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:replytoannotation.annot];
    [annotHandler removeAnnot:replytoannotation.annot];

    if (_extensionsManager.currentAnnot) {
        [_extensionsManager setCurrentAnnot:nil];
    }

    item.isDeleted = YES;
}

- (void)replyToAnnotation:(AnnotationItem *)item {
    FSTextViewPopoverViewController *textViewVC =  [_extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeTextViewPopover params:self.navBar];
    textViewVC.title = FSLocalizedForKey(@"kReply");
    textViewVC.okItemEnabled = ^BOOL(NSString *text) {
        return YES;
    };
    [textViewVC setDidClickedOK:^(NSString *contents) {
        AnnotationItem *replytoannotation = item;
        NSUInteger currentsection = 0;

        NSMutableArray *currentarray = [self.nodeAnnos objectForKey:[[[self.rootAnnos objectAtIndex:0] annot] NM]];

        if (replytoannotation.currentlevelshow == NO) {
            ! replytoannotation.currentlevelbutton.onTapClick ? : replytoannotation.currentlevelbutton.onTapClick(replytoannotation.currentlevelbutton);
        }

        self.pageIndex = replytoannotation.annot.pageIndex;
        self.replyanno = [[AnnotationItem alloc] init];

        FSPDFPage *page = [self->_document getPage:(int) self.pageIndex];
        if (!page)
            return;

        NSDate *now = [NSDate date];
        self.replyanno.annot = [[[FSMarkup alloc] initWithAnnot:replytoannotation.annot] addReply];
        self.replyanno.annot.fsrect = replytoannotation.annot.fsrect;
        self.replyanno.annot.NM = [Utility getUUID];
        self.replyanno.annot.author = self->_extensionsManager.annotAuthor;
        self.replyanno.annot.contents = @"";
        self.replyanno.annot.color = 0;
        self.replyanno.annot.opacity = 1;
        self.replyanno.annot.lineWidth = 2;
        self.replyanno.annot.icon = 0;
        self.replyanno.isReply = YES;
        self.replyanno.annot.modifiedDate = now;
        self.replyanno.annot.createDate = now;
        self.replyanno.annot.contents = contents;
        self.replyanno.rootannotation = replytoannotation.rootannotation;

        if (replytoannotation.annot.replyTo == nil) {
            self.replyanno.isSecondLevel = YES;
        } else {
            self.replyanno.isSecondLevel = NO;
        }

        [AnnotationStruct insertAnnotationToAnnoStruct:self.annoStruct insertAnnotation:self.replyanno SuperAnnotation:replytoannotation];

        NSMutableArray *addannoarray = [NSMutableArray arrayWithObject:self.replyanno];

        [addannoarray makeObjectsPerformSelector:@selector(addCurrentlevel:) withObject:[NSNumber numberWithLong:(replytoannotation.annot.replyTo == nil ? (replytoannotation.currentlevel + 1) : (replytoannotation.currentlevel))]];

        [addannoarray makeObjectsPerformSelector:@selector(setReplytoauthor:) withObject:replytoannotation.annot.author];

        NSUInteger insertrowindex = [currentarray indexOfObject:replytoannotation];

        NSMutableArray *arCells = [NSMutableArray array];

        if (replytoannotation.annot.replyTo == nil) {
            [currentarray insertObjects:addannoarray atIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange((insertrowindex + [[self.nodeAnnos objectForKey:[[[self.rootAnnos lastObject] annot] NM]] count]), addannoarray.count)]];

            [arCells addObject:[NSIndexPath indexPathForRow:(insertrowindex + [[self.nodeAnnos objectForKey:[[[self.rootAnnos lastObject] annot] NM]] count] - 1) inSection:currentsection]];
        } else {
            [currentarray insertObjects:addannoarray atIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(insertrowindex + [[self.annoStruct objectForKey:replytoannotation.annot.uuidWithPageIndex] count], addannoarray.count)]];

            [arCells addObject:[NSIndexPath indexPathForRow:insertrowindex + [[self.annoStruct objectForKey:replytoannotation.annot.uuidWithPageIndex] count] inSection:currentsection]];
        }

        self.indexPath = [arCells lastObject];

        [self.tableView insertRowsAtIndexPaths:arCells withRowAnimation:UITableViewRowAnimationNone];
        [self.tableView scrollToRowAtIndexPath:self.indexPath atScrollPosition:UITableViewScrollPositionTop animated:YES];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:self.replyanno.annot];
            [annotHandler addAnnot:self.replyanno.annot];
            if ([annotHandler isKindOfClass:[FSAnnotHandler class]] && [((FSAnnotHandler*)annotHandler).attributesBeforeModify.NM isEqualToString:item.annot.NM]) {
                ((FSAnnotHandler*)annotHandler).attributesBeforeModify = [FSAnnotAttributes attributesWithAnnot:item.annot];
            }
            [self.tableView reloadData];
        });
    }];
    [self presentViewController:textViewVC animated:YES completion:nil];
}


- (void)modifyAnnot:(FSAnnot *)annot withContents:(NSString *)contents {
    annot.contents = contents;
}

#pragma mark IAnnotEventListener

- (void)onAnnotWillDelete:(FSPDFPage *)page annot:(FSAnnot *)annot{
    AnnotationItem *item = [self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
    if ([annot isEqualToAnnot:item.annot]){
        [_pdfViewCtrl.fs_viewController dismissViewControllerAnimated:YES completion:nil];
    }
}

- (void)onAnnotWillFlatten:(FSPDFPage *)page annot:(FSAnnot *)annot{
    AnnotationItem *item = [self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
    if ([annot isEqualToAnnot:item.annot]){
        [_pdfViewCtrl.fs_viewController dismissViewControllerAnimated:YES completion:nil];
    }
}

- (void)onPagesWillFlatten:(int)index{
    AnnotationItem *item = [self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
    if ([[item.annot getPage] getIndex] == index) {
        dispatch_main_async_safe(^{
            [self->_pdfViewCtrl.fs_viewController dismissViewControllerAnimated:YES completion:nil];
        });
    }
}

#pragma mark rotation

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
}

@end

