/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPageColorView.h"
#import "FSPageColorCell.h"
#import "FSViewSettingManager.h"
#import "FSSettingBar+private.h"

static CGFloat HorzontalCellWidth = 32;
static CGFloat HorzontalCellHeight = 32;

@interface FSPageColorView ()<UICollectionViewDelegate, UICollectionViewDataSource>
@property (nonatomic, strong) UICollectionView* collectionView;
@property (nonatomic, strong) NSArray* dataSource;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@end

@implementation FSPageColorView

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager frame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self initData];
        [self initCollectionView];
    }
    return self;
}

- (void)initData
{
    _dataSource = @[ UIColor.whiteColor, UIColorHex(#E8E8E8), UIColorHex(#F0E8D8), UIColorHex(#CCC5B2), UIColorHex(#99CFA1), UIColorHex(#2E5073), UIColorHex(#644F47), UIColorHex(#9A9A9A)];
}

- (void)initCollectionView
{
    self.backgroundColor = self.fs_backgroundColor_B1;
    FSItem* title = [FSItem createItemWithTitle:FSLocalizedForKey(@"kPageColor")];
    title.accessibilityTraits = UIAccessibilityTraitNone;
    title.customAccessibilityTraits = UIAccessibilityTraitNone;
    [title setTitleColor:self.fs_textColor_T2 forState:UIControlStateNormal];
    title.titleLabel.font = self.fs_standardWeakFont_WF1;
    [self addSubview:title];
    [title mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(16);
        make.top.mas_equalTo([self checkIsIphoneDevice] ? 4 : 20);
        make.height.mas_equalTo(17);
    }];
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.scrollDirection = UICollectionViewScrollDirectionVertical;
    layout.itemSize = CGSizeMake(HorzontalCellWidth, HorzontalCellHeight);
    layout.sectionInset = UIEdgeInsetsMake(0, 16, 0, 16);
    _collectionView = [[UICollectionView alloc] initWithFrame:self.bounds collectionViewLayout:layout];
    _collectionView.allowsMultipleSelection = NO;
    [_collectionView registerClass:[FSPageColorCell class] forCellWithReuseIdentifier:NSStringFromClass([FSPageColorCell class])];
    
    [self addSubview:_collectionView];
    
    _collectionView.backgroundColor = self.fs_backgroundColor_B1;
    [_collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(title.mas_bottom).offset(21);
        make.left.right.bottom.mas_equalTo(0);
    }];
    _collectionView.delegate = self;
    _collectionView.dataSource = self;

    NSString* colorString = [_extensionsManager.settingBar.viewSettingManager getItemPageColor];
    UIColor* pageColor = nil;
    int iIndex = -1;
    if (colorString == nil) {
        pageColor = UIColor.whiteColor;
    }else{
        pageColor =  [UIColor colorWithHexString:colorString];
        for (int i = 0; i < self.dataSource.count; i++) {
            if (CGColorEqualToColor(((UIColor*)self.dataSource[i]).CGColor, pageColor.CGColor)) {
                iIndex = i;
                break;
            }
        }
    }
    
    if (!_pdfViewCtrl.isNightMode) {
        if (iIndex >= 0) {
            [_collectionView selectItemAtIndexPath:[NSIndexPath indexPathForRow:iIndex inSection:0] animated:NO scrollPosition:UICollectionViewScrollPositionNone];
            
            if (_extensionsManager.pdfViewCtrl.colorMode != FSRendererColorModeMappingGray || (_extensionsManager.pdfViewCtrl.colorMode == FSRendererColorModeMappingGray && !CGColorEqualToColor(_extensionsManager.pdfViewCtrl.mappingModeBackgroundColor.CGColor, pageColor.CGColor))) {
                [self.extensionsManager.settingBar.viewSettingManager setPageColor:pageColor];
            }
        }
    }
}


static DidSelectColorCallBack _didSelecteColorCallBack;

+ (void)setDidSelecteColorCallBack:(DidSelectColorCallBack)didSelecteColorCallBack
{
    _didSelecteColorCallBack = didSelecteColorCallBack;
}

+ (DidSelectColorCallBack)didSelecteColorCallBack
{
    return _didSelecteColorCallBack;
}

#pragma mark--UICollectionViewDelegate

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    return self.dataSource.count;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView
{
    return 1;
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    FSPageColorCell* cell  = (FSPageColorCell*)[collectionView dequeueReusableCellWithReuseIdentifier:NSStringFromClass([FSPageColorCell class]) forIndexPath:indexPath];
    [cell setCellInfo:self.dataSource[indexPath.row]];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    UIColor* color = self.dataSource[indexPath.row];
    
    if ((self.pdfViewCtrl.mappingModeBackgroundColor != color && [self.pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW) ||
        (self.pdfViewCtrl.reflowBackgroundColor != color && [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW)) {
        [self.extensionsManager.settingBar.viewSettingManager setPageColor:color];
        if (![self checkIsIphoneDevice]) {
            [self.extensionsManager.settingBar.viewSettingManager setIsNightMode:NO];
        }
        [self.extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:PAGECOLOR selected:YES];
    }
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section
{
    return (self.frame.size.width - HorzontalCellWidth*8-16*2)/7;
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}

- (void)refreshView
{
    if (self.collectionView) {
        [self.collectionView reloadData];
        [self.collectionView.collectionViewLayout invalidateLayout];
    }
}
@end
