/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AttachmentViewController.h"
#import "AnnotationItem.h"
#import "AnnotationListCell.h"
#import "AnnotationStruct.h"
#import "AttachmentController.h"
#import "AttachmentPanel.h"
#import "FSAnnotExtent.h"
#import "FileManageListViewController.h"
#import "FileSelectDestinationViewController.h"
#import "PanelController.h"
#import "PanelHost.h"
#import "UIExtensionsConfig.h"
#import "FSFileSelectDestinationViewController.h"
#import "FSGCDServer.h"
#import "NSString+FSAdd.h"
#import "FSPermissionUtilty.h"

#define kThumbnailSize CGSizeMake(32.f, 40.f);
@interface AttachmentViewController () <IPanelChangedListener, AnnotationListCellDelegate, IAnnotEventListener>

@property (nonatomic, retain) UIDocumentInteractionController *documentPopoverController;
@property (nonatomic, strong) NSOperationQueue *loadAttachmentQueue;
@property (nonatomic, strong) UITapGestureRecognizer *tapGesture;
@property (nonatomic, strong) UIViewController *originalPresentedViewController;

@property (nonatomic, assign) NSInteger canDeleteAttachmentCounts;
@property (nonatomic, assign) BOOL batchDelete;
@end

@implementation AttachmentViewController {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSPanelController *__weak _panelController;
    AttachmentPanel *__weak _attachmentPanel;
}

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager module:(AttachmentPanel *)attachmentPanel{
    self = [super initWithServices:extensionsManager.viewCtrlManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _panelController = attachmentPanel.panelController;
        _attachmentPanel = attachmentPanel;
        _allAttachmentsSections = [NSMutableArray array];
        [_panelController registerPanelChangedListener:self];
        [_extensionsManager registerAnnotEventListener:self];
        [_pdfViewCtrl registerDocEventListener:self];
        [_pdfViewCtrl registerPageEventListener:self];
        self.moreIndexPath = nil;
        self.originalPresentedViewController = nil;
    }
    return self;
}

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.shouldShowBottomBar = YES;
    self.sectionStyle = FSTableViewSectionStyleTitle;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleDescription;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
}

- (void)viewDidLoad {
    [super viewDidLoad];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadDataList:) name:NSNotificationNameAttachmentLsitUpdate object:nil];
    
    FSItem *addItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kAdd")];
    addItem.font = self.fs_standardGeneralFont_GF2;
    [addItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    [addItem addTarget:self action:@selector(clickedAddItem:) forControlEvents:UIControlEventTouchUpInside];
    [self.bottomBar addItem:addItem atPosition:FSItemPositionRight];
    _addItem = addItem;
    
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kSelectAll") forState:UIControlStateNormal];
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
    [self.editRightItem setTitle:@"" forState:UIControlStateNormal];
    [self.editRightItem setImage:FSImageNamedAndNormalColor(@"common_delete_gray") forState:UIControlStateNormal];
    self.cellSelectedColor = UIColor.clearColor;
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    [self reloadData];
}

- (void)clickedAddItem:(FSItem *)item{
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:_extensionsManager.viewCtrlManager];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFile;
    selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeAdd;
    selectDestination.expectFileType = [[NSArray alloc] initWithObjects:@"*", nil];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *files) {
        
        [controller dismissViewControllerAnimated:YES completion:^{
            [self.view fs_showHUDLoading:[NSString stringWithFormat:@"%@0/%lu",[FSLocalizedForKey(@"kDeleting") fs_stringByTrimOccurrences:@"..."], (unsigned long)files.count]];
            NSFileManager *fileManager = [NSFileManager defaultManager];
            for (int i = 0; i < files.count; i++) {
                [self.view fs_showHUDLoading:[NSString stringWithFormat:@"%@%d/%lu",[FSLocalizedForKey(@"kAdding") fs_stringByTrimOccurrences:@"..."],i+1,(unsigned long)files.count]];
                NSString *file = files[i];
                NSDictionary *fileAttribute = [fileManager attributesOfItemAtPath:file error:nil];
                long long fileSize = [fileAttribute fileSize];
                if (fileSize > 50 * 1024 * 1024) { // 50MB
                    [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kAttachmentMaxSize"), 50] onCtrl:self];
                    continue;
                }
                
                [self _addDocumentAttachemnt:file addUndo:NO];
                self->_extensionsManager.isDocModified = YES;
            }
            [self.view fs_hideHUDLoading];
        }];
    };
    [self->_extensionsManager.viewCtrlManager presentViewController:selectDestination animated:YES completion:nil];
}

- (void)setEditing:(BOOL)editing{
    [super setEditing:editing];
    _attachmentPanel.rightItem.selected = editing;
}

- (void)setSelectedDataSource:(NSArray *)selectedDataSource{
    [super setSelectedDataSource:selectedDataSource];
    self.editLeftItem.selected = self.selectedDataSource.count == self.canDeleteAttachmentCounts;
}

- (void)clickedEditLeftItem{
    if (self.allAttachmentsSections.count > 0) {
        self.editLeftItem.selected = !self.editLeftItem.selected;
        if (self.editLeftItem.selected) {
            NSInteger section = [self numberOfSectionsInTableView:self.tableView];
            for (int s = 0 ; s < section; s++ ) {
                NSInteger row = [self tableView:self.tableView numberOfRowsInSection:s];
                for (int r = 0; r < row; r++) {
                    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:r inSection:s];
                    AttachmentItem *item = [self dataForAtIndexPath:indexPath];
                    if ([self annotationListItemCanDelete:item]) {
                        [self selectRow:YES atIndexPath:indexPath];
                        [self selectDataSourceRow:YES atIndexPath:indexPath];
                    }
                }
            }
        }else{
            [self deselectAll];
        }
    }
}

- (void)clickedEditRightItem{
    if ([self selectedIndexPaths].count) {
        void(^block)(void) = ^{
            self.batchDelete = YES;
            for (int i = 0; i < self.selectedDataSource.count; i++) {
                NSIndexPath *indexPath = self.selectedIndexPaths[i];
                [self selectRow:NO atIndexPath:indexPath];
                AttachmentItem *item = self.selectedDataSource[i];
                [self deleteAnnotation:item];
            }
            self.batchDelete = NO;
            self.canDeleteAttachmentCounts -= self.selectedDataSource.count;
            if ([self emptyDataSetDisplayed] || !self.canDeleteAttachmentCounts) {
                self.editing = NO;
            }
            self->_extensionsManager.isDocModified = YES;
            [super clickedEditRightItem];
        };
        if (self.selectedDataSource.count != self.canDeleteAttachmentCounts) {
            block();
        }else{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kClearAttachments") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    block();
                }
            }onCtrl:[self fs_currentSceneTopMostViewController]];
        }

    }
}

- (void)updateCanDeleteAttachmentCounts{
    __block NSUInteger canDeleteAttachmentCounts = 0;

    FSGCDGroupServer *group = [FSGCDGroupServer new];
    for (NSArray *attachmentsSection in self.allAttachmentsSections) {
        [group executeBlock:^{
            for (AttachmentItem *attachmentItem in attachmentsSection) {
                if ([self annotationListItemCanDelete:attachmentItem]) {
                    canDeleteAttachmentCounts++;
                }
            }
        }];
    }
    dispatch_semaphore_t wait = dispatch_semaphore_create(0);
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [group notifyBlock:^{
            dispatch_semaphore_signal(wait);;
        }];
    });
    dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
    _canDeleteAttachmentCounts = canDeleteAttachmentCounts;
}


- (void)_addDocumentAttachemnt:(NSString *)attachmentFilePath addUndo:(BOOL)addUndo {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL isDir;
    if (![fileManager fileExistsAtPath:attachmentFilePath isDirectory:&isDir] || isDir) {
        return;
    }
    NSDictionary *attributes = [fileManager attributesOfItemAtPath:attachmentFilePath error:nil];
    if (!attributes) {
        return;
    }
    
    FSPDFNameTree *nameTree = [[FSPDFNameTree alloc] initWithDocument:_pdfViewCtrl.currentDoc type:FSPDFNameTreeEmbeddedFiles];
    void(^addFile)(NSString *fileName) = ^(NSString *fileName){
        FSFileSpec *fileSpec = [[FSFileSpec alloc] initWithDocument:self->_pdfViewCtrl.currentDoc];
        [fileSpec setFileName:fileName];
        bool isOK = false;
        @try {
            isOK = [fileSpec embed:attachmentFilePath];
        } @catch (NSException *e) {
            NSLog(@"failed to embed file, %@", e);
        }
        if (!isOK) {
            return;
        }
        [fileSpec setCreationDateTime:[Utility convert2FSDateTime:[attributes fileCreationDate]]];
        [fileSpec setModifiedDateTime:[Utility convert2FSDateTime:[attributes fileModificationDate]]];
        if ([nameTree add:fileName pdf_object:[fileSpec getDict]]) {
            AttachmentItem *attachmentItem = [AttachmentItem itemWithDocumentAttachment:fileName file:fileSpec PDFPath:self->_pdfViewCtrl.filePath];
            [self onDocumentAttachmentAdded:attachmentItem];
            self.canDeleteAttachmentCounts++;

            // undo/redo support
            if (addUndo) {
                [self->_extensionsManager addUndoItem:[UndoItem itemWithUndo:^(UndoItem *item) {
                                        if ([nameTree hasName:fileName]) {
                                            [nameTree removeObj:fileName];
                                        }
                                        [self onDocumentAttachmentDeleted:attachmentItem];
                                    }
                                                    redo:^(UndoItem *item) {
                                                        [self _addDocumentAttachemnt:attachmentFilePath addUndo:NO];
                                                    }
                                                    pageIndex:-1]];
            }
        } else {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kFailedAddAttachment"), 50] preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:cancelAction];
            [self presentViewController:alertController animated:YES completion:nil];
        }
    };
    
    __block NSString *fileName = attachmentFilePath.lastPathComponent;
    if ([nameTree hasName:fileName]) {
        [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailedAddAttachmentForExistedName") actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kRenameAndAttached")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            int renameCount = 1;
            NSString *newFileName = [[NSString stringWithFormat:@"%@-Copy",[fileName stringByDeletingPathExtension]] stringByAppendingPathExtension:[fileName pathExtension]];
            while ([nameTree hasName:newFileName]) {
                newFileName = [[NSString stringWithFormat:@"%@-Coyp(%d)",[fileName stringByDeletingPathExtension], renameCount] stringByAppendingPathExtension:[fileName pathExtension]];
                renameCount ++;
            }
            if (index == 1) {
                [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kRename") message:nil textFieldText:newFileName actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
                    if (index == 1) {
                        addFile(textField.text);
                    }
                }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
                    if (index == 1) {
                        return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length && ![nameTree hasName:textField.text];
                    }
                    return YES;
                }] fs_showByViewController:[self fs_currentSceneTopMostViewController]];
            }
        } onCtrl:[self fs_currentSceneTopMostViewController]];
    }else{
        addFile(fileName);
    }

}

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoAttachments");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"panel_attachment_no_data");
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    BOOL emptyDataSetShouldDisplay = [super emptyDataSetShouldDisplay:tableView];
    _attachmentPanel.rightItem.enabled = !emptyDataSetShouldDisplay && _canDeleteAttachmentCounts;
    if (!_attachmentPanel.rightItem.enabled && _attachmentPanel.rightItem.selected) {
        _attachmentPanel.rightItem.selected = NO;
    }
    if (emptyDataSetShouldDisplay) {
        if (self.editing) {
            self.editing = NO;
        }
    }
    return emptyDataSetShouldDisplay;
}

#pragma mark - IAnnotEventListener

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (annot.type == FSAnnotFileAttachment) {
        [self updateAllAttachments:[AttachmentItem itemWithAttachmentAnnotation:[[FSFileAttachment alloc] initWithAnnot:annot]] operation:FSAnnotationOperationAdd];
        [self updateCanDeleteAttachmentCounts];
    }
}

- (void)onAnnotWillDelete:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (annot.type == FSAnnotFileAttachment) {
        [self updateAllAttachments:[AttachmentItem itemWithAttachmentAnnotation:[[FSFileAttachment alloc] initWithAnnot:annot]] operation:FSAnnotationOperationDelete];
        if (!_batchDelete) {
            [self updateCanDeleteAttachmentCounts];
        }
    }
}

- (void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (annot.type == FSAnnotFileAttachment) {
        [self updateAllAttachments:[AttachmentItem itemWithAttachmentAnnotation:[[FSFileAttachment alloc] initWithAnnot:annot]] operation:FSAnnotationOperationModify];
    }
}

- (void)onAnnotWillFlatten:(FSPDFPage *)page annot:(FSAnnot *)annot{
    if (annot.type == FSAnnotFileAttachment) {
        [self updateAllAttachments:[AttachmentItem itemWithAttachmentAnnotation:[[FSFileAttachment alloc] initWithAnnot:annot]] operation:FSAnnotationOperationDelete];
        if (!_batchDelete) {
            [self updateCanDeleteAttachmentCounts];
        }
    }
}

- (void)onDocumentAttachmentAdded:(AttachmentItem *)attachmentItem {
    [self updateAllAttachments:attachmentItem operation:FSAnnotationOperationAdd];
}

- (void)onDocumentAttachmentDeleted:(AttachmentItem *)attachmentItem {
    [self updateAllAttachments:attachmentItem operation:FSAnnotationOperationDelete];
}

- (void)onDocumentAttachmentModified:(AttachmentItem *)attachmentItem {
    [self updateAllAttachments:attachmentItem operation:FSAnnotationOperationModify];
}

#pragma mark - IPanelChangedListener

- (void)onPanelChanged:(BOOL)isHidden {
    if (isHidden) {
        self.editing = NO;
    }
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    AttachmentItem *attachmentItem = [self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:section]];
    if (attachmentItem && attachmentItem.isDocumentAttachment) {
        return FSLocalizedForKey(@"kDocumentAttachmentTab");
    }
    return  [NSString stringWithFormat:FSLocalizedForKey(@"kPage"), attachmentItem.annot.pageIndex + 1];
}

- (NSString *)titles2ForAtSection:(NSUInteger)section{
    NSArray *array = [self.allAttachmentsSections copy];
    if (section < [array count]) {
        return  [NSString stringWithFormat:@"%lu", (unsigned long) [[array objectAtIndex:section] count]];
    }
    return nil;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.allAttachmentsSections.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    NSArray *array = [self.allAttachmentsSections objectAtIndex:section];
    return array.count;
}

#pragma mark -  UITableViewDelegate
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *headerView = [super tableView:tableView viewForHeaderInSection:section];
    headerView.backgroundColor = self.fs_backgroundColor_B2;
    return headerView;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    return 25;
}

- (AttachmentItem *)getAnnotationItemAtIndexPath:(NSIndexPath *)indexPath {
    if (!indexPath) {
        return nil;
    }
    if (indexPath.section >= self.allAttachmentsSections.count) {
        return nil;
    }
    NSArray *sectionAttachments = [self.allAttachmentsSections objectAtIndex:[indexPath section]];
    if (indexPath.item >= sectionAttachments.count) {
        return nil;
    }
    return [sectionAttachments objectAtIndex:indexPath.item];
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return [self getAnnotationItemAtIndexPath:indexPath];
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(AttachmentItem *)attachmentItem{
    if (!attachmentItem.description.length) {
        identifier = NSStringFromClass([FSSubjectEstimatedTableViewCell class]);
    }
    return  [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:attachmentItem];
}


- (void)configureCell:(FSDescriptionEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(AttachmentItem *)attachmentItem{
    if (!cell.installed) {
        cell.editTopOffset = ITEM_IMAGE_TOP_MARGIN_NORMAL + 7;
        cell.leftMargin = 9.f;
    }
    cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
    if (!cell.leftView) {
        UIImageView *imageView = [UIImageView new];
        imageView.fs_size = kThumbnailSize;
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        cell.leftView = imageView;
    }
    NSString *nameStr = attachmentItem.keyName;
    if (!nameStr || nameStr.length <= 0) {
        nameStr = attachmentItem.fileName;
    }
    ((UIImageView *)(cell.leftView)).image = FSImageNamed([Utility getIconName:nameStr]);
    if (!cell.rightView) {
        FSItem *rightView = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
        rightView.accessibilityLabel = FSLocalizedForKey(@"kMore");
        cell.rightView = rightView;
    }
    
    BOOL canEdit = [self annotationListItemCanDescript:attachmentItem];
    @weakify(self)
    @weakify(cell)
    ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        NSMutableArray *items = @[].mutableCopy;
        
        if ([self annotationListItemCanSave:attachmentItem]) {
            [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kSave") image:FSImageNamedAndNormalColor(@"common_save_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
                [self annotationListItemSave:attachmentItem];
            }]];
        }
        if (canEdit) {
            [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDescription") image:FSImageNamedAndNormalColor(@"panel_note_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
                [self addNoteToAnnotation:attachmentItem withIndexPath:indexPath];
            }]];
        }
        if ([self annotationListItemCanFlatten:attachmentItem]) {
            [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kFlatten") image:FSImageNamedAndNormalColor(@"panel_flatten_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
                [self annotationListItemFlatten:attachmentItem];
            }]];
        }
            
        if ([self annotationListItemCanDelete:attachmentItem]) {
            [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
                [self annotationListItemDelete:attachmentItem];
            }]];
        }
        @strongify(cell)
        if ([self fs_currentSceneIsCompact]) [cell setHighlighted:YES color:self.fs_backgroundColor_B2 custom:YES];
        attachmentItem.isShowMore = YES;
        [self->_extensionsManager.viewCtrlManager showMoreMenuWithItems:items.copy dismissCompletion: ^{
            [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
            attachmentItem.isShowMore = NO;
        }  sourceView:![self fs_currentSceneIsCompact] ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return _item;
        } : nil];
        
    };
    
    cell.didInstallConstraints = ^(FSDescriptionEstimatedTableViewCell *_cell) {
        [_cell.estimatedTitleLabel mas_updateConstraints:^(MASConstraintMaker *make) {
            make.width.equalTo(_cell.contentView).offset(- _cell.leftMargin - _cell.leftView.fs_width - _cell.rightMargin*2 - _cell.mediumSpace - 40);
        }];
    };
    
    cell.estimatedTitleLabel.text = [self fs_currentSceneIsCompact] ? attachmentItem.fileName : [attachmentItem.fileName stringByDeletingPathExtension];
    cell.estimatedTitleLabel.numberOfLines = 2;
    cell.estimatedSubjectLabel.text = [NSString stringWithFormat:@"%@%@ · %@",[self fs_currentSceneIsCompact] ? @"" : [NSString stringWithFormat:@"%@ · ",[[attachmentItem.fileName pathExtension] uppercaseString]], [Utility displayDateInMDY:attachmentItem.modifyDate], [Utility displayFileSize:attachmentItem.fileSize]];

    cell.editItem.hidden = ![self annotationListItemCanDelete:attachmentItem];
    if ([cell respondsToSelector:@selector(estimatedDescriptionLabel)]) {
        cell.estimatedDescriptionLabel.text = attachmentItem.description;
    
        UIButton *editNoteClicked = [cell.contentView viewWithTag:100];
        if (canEdit) {
            if (!editNoteClicked) {
                editNoteClicked = [UIButton new];
                editNoteClicked.tag = 100;
                [cell.contentView addSubview:editNoteClicked];
                
                [editNoteClicked mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.bottom.fs_equalToSuperview();
                    make.top.equalTo(cell.estimatedDescriptionLabel);
                }];
            }
            
            editNoteClicked.hidden = YES;
            
            if ([FSPermissionUtilty canAddAnnot:_extensionsManager] ) {
                editNoteClicked.hidden = NO;
                @weakify(self)
                [editNoteClicked setBlockForControlEvents:UIControlEventTouchUpInside block:^(id  _Nonnull sender) {
                    @strongify(self)
                    if (!cell.editing) [self addNoteToAnnotation:attachmentItem withIndexPath:indexPath];
                }];
            }
        }else{
            editNoteClicked.hidden = YES;
        }
    }

    [super configureCell:cell atIndexPath:indexPath withDataItem:attachmentItem];
    if ([self fs_currentSceneIsCompact]) [cell setHighlighted:attachmentItem.isShowMore color:self.fs_backgroundColor_B2 custom:YES];
    else [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editing) {
        [super tableView:tableView didSelectRowAtIndexPath:indexPath];
        return;
    }
    [tableView deselectRowAtIndexPath:indexPath animated:NO];

    NSArray *sectionAttachments = [self.allAttachmentsSections objectAtIndex:[indexPath section]];
    AttachmentItem *attachmentItem = [sectionAttachments objectAtIndex:[indexPath row]];

    [self openAttachment:attachmentItem];
}

- (void)updateAllAttachments:(AttachmentItem *)attachmentItem operation:(int)operation {
    @synchronized(self) {
        NSMutableArray *attachments = nil;
        for (NSMutableArray *array in self.allAttachmentsSections) {
            AttachmentItem *item = [array objectAtIndex:0];
            if (attachmentItem.pageIndex == item.pageIndex) {
                attachments = array;
                break;
            }
        }

        if (operation == FSAnnotationOperationAdd) {
            if (!attachments) {
                attachments = [NSMutableArray array];
                [self.allAttachmentsSections addObject:attachments];
            }
            [attachments addObject:attachmentItem];
        } else if (operation == FSAnnotationOperationModify) {
            int index = -1;
            for (AttachmentItem *tmpItem in attachments) {
                if (attachmentItem.annot) {
                    if ([tmpItem.annot.NM isEqualToString:attachmentItem.annot.NM]) {
                        index = (int) [attachments indexOfObject:tmpItem];
                        break;
                    }
                } else {
                    if ([tmpItem.fileName isEqualToString:attachmentItem.fileName]) {
                        index = (int) [attachments indexOfObject:tmpItem];
                        break;
                    }
                }
            }
            if (index != -1) {
                [attachments replaceObjectAtIndex:index withObject:attachmentItem];
            }
        } else if (operation == FSAnnotationOperationDelete) {
            for (AttachmentItem *tmpItem in attachments) {
                if (attachmentItem.annot) {
                    if ([tmpItem.annot.NM isEqualToString:attachmentItem.annot.NM]) {
                        [attachments removeObject:tmpItem];
                        if (attachments.count == 0) {
                            [self.allAttachmentsSections removeObject:attachments];
                        }
                        break;
                    }
                } else {
                    if ([tmpItem.keyName isEqualToString:attachmentItem.keyName]) {
                        [attachments removeObject:tmpItem];
                        if (attachments.count == 0) {
                            [self.allAttachmentsSections removeObject:attachments];
                        }
                        break;
                    }
                }
            }
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [self resortAllAttachments];
            [self reloadData];
        });
    }
}

- (void)openAttachment:(AttachmentItem *)attachmentItem {
    if (!attachmentItem.fileName) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailedOpenAttachmentBadFileName") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                         style:UIAlertActionStyleCancel
                                                       handler:^(UIAlertAction *_Nonnull action) {
                                                           [self popoverDidCancel];
                                                       }];
        [alertController addAction:action];
        [self presentAlertController:alertController];
        return;
    }

    if (!attachmentItem.fileSpec) {
        return;
    }
    if ([Utility isSupportFormat:attachmentItem.filePath]) {

        AttachmentController *attachmentCtr = [[AttachmentController alloc] initWithServices:_extensionsManager.viewCtrlManager];
        [_extensionsManager.viewCtrlManager presentViewController:attachmentCtr animated:YES completion:^{
            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^{
                if (![Utility loadFileSpec:attachmentItem.fileSpec toPath:attachmentItem.filePath]) return;
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [attachmentCtr openDocument:attachmentItem];
                });
            });
        }];

    } else {
        if (![Utility loadFileSpec:attachmentItem.fileSpec toPath:attachmentItem.filePath]) {
            return;
        }
        BOOL isPresent = NO;
        #if !_MAC_CATALYST_
        NSURL *urlFile = [NSURL fileURLWithPath:attachmentItem.filePath isDirectory:NO];
        self.documentPopoverController = [UIDocumentInteractionController interactionControllerWithURL:urlFile];
        self.documentPopoverController.delegate = self;
        if (DEVICE_iPHONE) {
            isPresent = [self.documentPopoverController presentOpenInMenuFromRect:_pdfViewCtrl.frame inView:_pdfViewCtrl animated:YES];
        } else {
            CGRect dvRect = CGRectMake(CGRectGetWidth(_pdfViewCtrl.bounds) / 2, CGRectGetHeight(_pdfViewCtrl.bounds) / 2, 20, 20);
            isPresent = [self.documentPopoverController presentOpenInMenuFromRect:dvRect inView:_pdfViewCtrl animated:YES];
        }
        #endif
        if (!isPresent) {
            NSString *fileName = [attachmentItem.fileSpec getFileName];
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:FSLocalizedForKey(@"kFailedOpenAttachment"), fileName] message:nil preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:cancelAction];
            [self presentViewController:alertController animated:YES completion:nil];
        }
    }
}

- (void)setIsShowMore:(BOOL)isShowMore {
    _isShowMore = isShowMore;
}

- (void)loadAllAnnotAttachments:(NSArray *)annotAttachments {
    NSMutableArray *attachmentItems = [NSMutableArray arrayWithCapacity:annotAttachments.count];
    for (FSFileAttachment *annot in annotAttachments) {
        AttachmentItem *attachmentItem = [AttachmentItem itemWithAttachmentAnnotation:annot];
        attachmentItem.currentlevel = 1;
        attachmentItem.isSecondLevel = YES;
        if ([self annotationListItemCanDelete:attachmentItem]) {
            self.canDeleteAttachmentCounts ++;
        }
        [attachmentItems addObject:attachmentItem];
    }
    if (attachmentItems.count > 0) {
        [self.allAttachmentsSections addObject:attachmentItems];
    }
}

- (void)loadAllDocumentAttachments:(FSPDFDoc *)document {
    if (![self documentHasNameTree:document]) return;
    NSMutableArray<AttachmentItem *> *attachmentItems = [NSMutableArray<AttachmentItem *> array];
    FSPDFNameTree *nameTree = [[FSPDFNameTree alloc] initWithDocument:document type:FSPDFNameTreeEmbeddedFiles];
    for (int i = 0; i < [nameTree getCount]; i++) {
        NSString *name = [nameTree getName:i];
        FSPDFObject *dict = [nameTree getObj:name];
        if (dict && dict.getType == FSPDFObjectDictionary) {
            FSFileSpec *file = [[FSFileSpec alloc] initWithDocument:document pdf_object:dict];
            if (file) {
                AttachmentItem *attachmentItem = [AttachmentItem itemWithDocumentAttachment:name file:file PDFPath:_pdfViewCtrl.filePath];
                attachmentItem.currentlevel = 1;
                attachmentItem.isSecondLevel = YES;
                if ([self annotationListItemCanDelete:attachmentItem]) {
                    self.canDeleteAttachmentCounts ++;
                }
                [attachmentItems addObject:attachmentItem];
            }
        }
    }
    if (attachmentItems.count > 0) {
        [self.allAttachmentsSections addObject:attachmentItems];
    }
}

- (BOOL)documentHasNameTree:(FSPDFDoc *)document{
    @try {
        FSPDFDictionary *catalog = document.getCatalog;
        return [catalog hasKey:@"Names"];
    } @catch (NSException *exception) {
        NSLog(@"%@",exception);
    }
    return NO;
}

- (void)resortAllAttachments {
    if (self.allAttachmentsSections.count > 1) {
        [self.allAttachmentsSections sortUsingComparator:^NSComparisonResult(NSArray *obj1, NSArray *obj2) {
            AttachmentItem *attachmentItem1 = [obj1 objectAtIndex:0];
            AttachmentItem *attachmentItem2 = [obj2 objectAtIndex:0];
            if (attachmentItem2.isDocumentAttachment) {
                return NSOrderedDescending;
            } else if (attachmentItem1.isDocumentAttachment) {
                return NSOrderedAscending;
            } else {
                return attachmentItem1.pageIndex > attachmentItem2.pageIndex;
            }
        }];
    }
}

#pragma mark <AnnotationListCellDelegate>

- (void)annotationListCellWillShowEditView:(AnnotationListCell *)cell {
    [self hideCellEditView];
}

- (void)annotationListCellDidShowEditView:(AnnotationListCell *)cell {
    self.isShowMore = YES;
    self.moreIndexPath = [self.tableView indexPathForCell:cell];
    if (!self.tapGesture) {
        self.tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTap:)];
        [self.view addGestureRecognizer:self.tapGesture];
    }
    self.tapGesture.enabled = YES;
}


- (BOOL)annotationListItemCanDescript:(AttachmentItem *)item {
    if (![_extensionsManager annotsPermissionCanModify:item.annot]) {
        return NO;
    }
    return item.annot ? item.annot.canEdit : (([_extensionsManager.permissionProvider checkPermission:FSFunctionDocumentAttachmentCanEdit] == FSPermissionStateNo) ? NO : [FSPermissionUtilty canModifyContents:_extensionsManager]);
}

- (BOOL)annotationListItemCanDelete:(AttachmentItem *)item {
    if (item.annot) {
        if (![_extensionsManager annotsPermissionCanDelete:item.annot]) {
            return NO;
        }
    }
    
    if ([_extensionsManager.config.tools containsObject:Tool_Multiple_Selection] && item.annot.canGetGroupHeader) return NO;
    assert(item);
    return item.annot ? item.annot.canEdit: (([_extensionsManager.permissionProvider checkPermission:FSFunctionDocumentAttachmentCanEdit] == FSPermissionStateNo) ? NO : [FSPermissionUtilty canModifyContents:_extensionsManager]);
}

- (BOOL)annotationListItemCanSave:(AttachmentItem *)item {
    if (item.isDocumentAttachment) {
        return [Utility canExtractContents:_pdfViewCtrl];
    }
//    return [Utility canExtractContents:_pdfViewCtrl];
    return [Utility canCopyForAssess:_pdfViewCtrl];
}

- (BOOL)annotationListItemCanFlatten:(AttachmentItem *)item {
    if (![_extensionsManager annotPermissionCanFlatten:item.annot]) {
        return NO;
    }
    if ([_extensionsManager.config.tools containsObject:Tool_Multiple_Selection] && item.annot.canGetGroupHeader) return NO;
    if ([item isKindOfClass:[AttachmentItem class]]) {
        if (![FSPermissionUtilty canAddAnnot:_extensionsManager]) return NO;
        return !((AttachmentItem *)item).isDocumentAttachment;
    }
    return NO;
}

- (void)annotationListItemDelete:(AttachmentItem *)item {
    [self deleteAnnotation:item];
    self.canDeleteAttachmentCounts -- ;
}

- (void)annotationListItemSave:(AttachmentItem *)item {
    [self saveAttachment:item];
}

- (void)annotationListItemFlatten:(AttachmentItem *)item {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kFlatten") message:FSLocalizedForKey(@"kFlattenTip") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
    UIAlertAction *flattenAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFlatten") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        if ([item.annot isEmpty]) {
            return;
        }
        
        assert(item.annot);
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:item.annot];
        if ( [annotHandler flattenAnnot:item.annot]){
            self.canDeleteAttachmentCounts -- ;
        }
    }];
    [alertController addAction:cancelAction];
    [alertController addAction:flattenAction];
    [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
}

- (void)reloadDataList:(NSNotification *)not{
    if (not.object != self){
        [self clearData];
        [self loadData];
    }
}

#pragma mark

- (void)handleTap:(UITapGestureRecognizer *)tapGesture {
    assert(self.isShowMore);
    assert(self.moreIndexPath);
    [self hideCellEditView];
}

- (void)hideCellEditView {
    if (self.isShowMore) {
        assert(self.moreIndexPath);
        AnnotationListCell *cell = (AnnotationListCell *) [self.tableView cellForRowAtIndexPath:self.moreIndexPath];
        [cell setEditViewHidden:YES];
        self.isShowMore = NO;
        self.moreIndexPath = nil;
        self.tapGesture.enabled = NO;
    }
}

#pragma mark - methods

- (void)clearData {
    if (self.loadAttachmentQueue) {
        [self.loadAttachmentQueue cancelAllOperations];
        [self.loadAttachmentQueue waitUntilAllOperationsAreFinished];
    }
    [self.allAttachmentsSections removeAllObjects];
    self.canDeleteAttachmentCounts = 0;
    dispatch_async(dispatch_get_main_queue(), ^{
        [self reloadData];
    });
    self.indexPath = nil;
}

- (void)loadData {
    if ([_pdfViewCtrl isDynamicXFA]) {
        return;
    }
    
    if (!self.loadAttachmentQueue) {
        self.loadAttachmentQueue = [[NSOperationQueue alloc] init];
        self.loadAttachmentQueue.qualityOfService = NSQualityOfServiceUtility;

    }
    FSPDFDoc *doc = _pdfViewCtrl.currentDoc;
    typeof(self) __weak weakSelf = self;

    //load attachment annotations
    int pageCount = 0;
    @try {
        pageCount = [doc getPageCount];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
        return;
    }
    NSBlockOperation *op = [[NSBlockOperation alloc] init];
    typeof(op) __weak weakOp = op;
    if (_extensionsManager.config.loadAttachment) {
        const int pagesPerExecution = 100;
        for (int pageStart = 0; pageStart < pageCount; pageStart += pagesPerExecution) {
            [op addExecutionBlock:^{
                @autoreleasepool {
                    for (int pageIndex = pageStart; pageIndex < pageStart + pagesPerExecution && pageIndex < pageCount; pageIndex++) {
                        if (weakOp.isCancelled) {
                            return;
                        }
                        FSPDFPage *page = nil;
                        @try {
                            page = [doc getPage:pageIndex];
                        } @catch (NSException *exception) {
                            continue;
                        }
                        @try {
                            NSArray *annots = [Utility getAnnotationsOfType:FSAnnotFileAttachment inPage:page];
                            if (annots.count > 0) {
                                [self loadAllAnnotAttachments:annots];
                            }
                        } @catch (NSException *exception) {
                        }
                    }
                }
            }];
        }
    }

    op.completionBlock = ^{
        if (weakOp.isCancelled) {
            return;
        }
        //load document attachments
        @try {
            [weakSelf loadAllDocumentAttachments:doc];
        } @catch (NSException *exception) {
            return;
        }
        [weakSelf resortAllAttachments];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self reloadData];
        });
    };
    [self.loadAttachmentQueue addOperation:op];
    
    self.dataSource = self.allAttachmentsSections;
}

- (void)deleteALLAttachment
{
    NSMutableArray *attachmentItems = [NSMutableArray array];
    for (NSArray *sections in self.allAttachmentsSections) {
        for (AttachmentItem *item in sections) {
            if (item && [item isKindOfClass:[AttachmentItem class]]) [attachmentItems addObject:item];
        }
    }
    
    for (AttachmentItem *item in attachmentItems) {
        [self deleteAnnotation:item];
    }
    [attachmentItems removeAllObjects];
}

// attachment could be a attachment annotation or a document attachment
- (void)deleteAnnotation:(AttachmentItem *)item {
    if (item.isDocumentAttachment) {
        [self _deleteDocumentAttachment:item addUndo:NO];
    } else {
        assert(item.annot);
        id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:item.annot];
        [annotHandler removeAnnot:item.annot];
        _extensionsManager.currentAnnot = nil;
    }
}

- (void)_deleteDocumentAttachment:(AttachmentItem *)attachmentItem addUndo:(BOOL)addUndo {
    FSPDFNameTree *nameTree = [[FSPDFNameTree alloc] initWithDocument:_pdfViewCtrl.currentDoc type:FSPDFNameTreeEmbeddedFiles];
    NSString *name = attachmentItem.keyName;
    if ([nameTree hasName:name]) {
        [nameTree removeObj:name];
        _extensionsManager.isDocModified = YES;
    } else {
        // should never reach here
        assert(NO);
        return;
    }
    
    [self onDocumentAttachmentDeleted:attachmentItem];
    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAttachmentLsitUpdate object:self];

    if (addUndo) {
        [_extensionsManager addUndoItem:[UndoItem itemWithUndo:^(UndoItem *item) {
                                if (![nameTree hasName:name]) {
                                    FSFileSpec *fileSpec = [[FSFileSpec alloc] initWithDocument:self->_extensionsManager.pdfViewCtrl.currentDoc];
                                    [fileSpec setFileName:attachmentItem.fileName];
                                    assert(attachmentItem.filePath);
                                    [fileSpec embed:attachmentItem.filePath];
                                    if ([nameTree add:name pdf_object:[fileSpec getDict]]) {
                                        [self onDocumentAttachmentAdded:attachmentItem];
                                    }
                                }
                            }
                                            redo:^(UndoItem *item) {
                                                [self _deleteDocumentAttachment:attachmentItem addUndo:NO];
                                            }
                                            pageIndex:-1]];
    }
}

- (void)saveAttachment:(AttachmentItem *)item {
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:_extensionsManager.viewCtrlManager];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
    selectDestination.expectFileType = @[@"*"];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        [controller.services dismissViewControllerAnimated:YES completion:^{
            [self copyFile:item toFolder:[destinationFolder objectAtIndex:0]];
        }];
    };
    [self->_extensionsManager.viewCtrlManager presentViewController:selectDestination animated:YES completion:nil];
}

- (void)copyFile:(AttachmentItem *)item toFolder:(NSString *)toFolder {
    if (!item.fileName || [item.fileName isEqualToString:@""]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailedSaveAttachmentBadFileName") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                         style:UIAlertActionStyleCancel
                                                       handler:nil];
        [alertController addAction:action];
        [self presentAlertController:alertController];
        return;
    }
    NSFileManager *fileManager = [[NSFileManager alloc] init];
    NSString *srcFilePath = item.filePath;
    BOOL isDir = YES;
    if (![fileManager fileExistsAtPath:toFolder isDirectory:&isDir] || !isDir) {
        return;
    }
    NSString *destFilePath = [toFolder stringByAppendingPathComponent:item.fileName];
    
    void (^block)(NSString *destFilePath, BOOL replace) = ^(NSString *destFilePath, BOOL replace){
        if (replace) {
            [fileManager removeItemAtPath:destFilePath error:nil];
        }

        NSError *error = nil;
        BOOL isOK = YES;
        if ([fileManager fileExistsAtPath:srcFilePath isDirectory:nil]) {
            isOK = [fileManager copyItemAtPath:srcFilePath toPath:destFilePath error:&error];
        } else {
            isOK = [Utility loadFileSpec:item.fileSpec toPath:destFilePath];
        }
        if (!isOK) {
            NSString *msg = [NSString stringWithFormat:FSLocalizedForKey(@"kCopyFileFailed"), item.fileName, error.localizedDescription];
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(msg) preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                             style:UIAlertActionStyleCancel
                                                           handler:^(UIAlertAction *_Nonnull action) {
                                                               [self popoverDidCancel];
                                                           }];
            [alertController addAction:action];
            [self presentAlertController:alertController];
        }else{
            FSAlertViewCtrlShow(FSLocalizedForKey(@"kSuccess"), FSLocalizedForKey(@"kSaveSuccess"));
        }
    };
    
    if ([fileManager fileExistsAtPath:destFilePath]) {
        
        int renameCount = 1;
        NSString *folderPath = [destFilePath stringByDeletingLastPathComponent];
        NSString *fileExtension = [destFilePath pathExtension];
        NSString *fileName = [[destFilePath lastPathComponent] stringByDeletingPathExtension];
        NSString *newDestFilePath = [[folderPath stringByAppendingPathComponent:fileName] stringByAppendingPathExtension:fileExtension];
        
        NSString *newFileName = fileName;
        while ([fileManager fileExistsAtPath:newDestFilePath]) {
            newFileName = [NSString stringWithFormat:@"%@(%d)",fileName, renameCount];
            newDestFilePath = [[folderPath stringByAppendingPathComponent:newFileName] stringByAppendingPathExtension:fileExtension];
            renameCount ++;
        }
        [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kSaveAs") message:nil textFieldText:newFileName actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
            if (index == 1) {
                NSString *newDestFilePath = [[folderPath stringByAppendingPathComponent:textField.text] stringByAppendingPathExtension:fileExtension];
                if ([fileManager fileExistsAtPath:newDestFilePath]) {
                    [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kSureToReplaceFile"), textField.text] actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                        if (index == 1) {
                            block(newDestFilePath, YES);
                        }
                    } onCtrl:[self fs_currentSceneTopMostViewController]];
                }else{
                    block(newDestFilePath, NO);
                }
            }
        }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
            if (index == 1) {
                return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
            }
            return YES;
        }] fs_showByViewController:[self fs_currentSceneTopMostViewController]];
    }else{
        block(destFilePath, NO);
    }
}

- (void)addNoteToAnnotation:(AttachmentItem *)item withIndexPath:(NSIndexPath *)indexPath {
    
    FSTextViewPopoverViewController *textViewVC = [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:_panelController.panel.topBar];
    textViewVC.title = FSLocalizedForKey(@"kDescription");
    textViewVC.text = item.description;
    textViewVC.okItemEnabled = ^BOOL(NSString *text) {
        return item.description.hash != text.hash;
    };
    [textViewVC setDidClickedOK:^(NSString *contents) {
        [self tryRenameAttachment:item newName:contents addUndo:!item.isDocumentAttachment];
        [self reloadRowsAtIndexPaths:indexPath];
        self->_extensionsManager.isDocModified = YES;
    }];

}

- (void)tryRenameAttachment:(AttachmentItem *)attachmentItem newName:(NSString *)newName addUndo:(BOOL)addUndo {
    if (attachmentItem.annot) {
        FSFileAttachment *annot = [[FSFileAttachment alloc] initWithAnnot:attachmentItem.annot];
        NSString *oldName = annot.contents;
        if (![newName isEqualToString:oldName]) {
            attachmentItem.annot.contents = newName;
            NSDate *oldModifiedDate = annot.modifiedDate;
            NSDate *newModifiedDate = [NSDate date];
            annot.modifiedDate = newModifiedDate;
            id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
            [annotHandler modifyAnnot:annot addUndo:NO];

            //add undo item
            if (addUndo) {
                FSPDFPage *page = [annot getPage];
                NSString *NM = annot.NM;
                [_extensionsManager addUndoItem:[UndoItem itemWithUndo:^(UndoItem *item) {
                                        FSFileAttachment *annot = [[FSFileAttachment alloc] initWithAnnot:[Utility getAnnotByNM:NM inPage:page]];
                                        if (annot && ![annot isEmpty]) {
                                            annot.contents = oldName;
                                            
                                            annot.modifiedDate = oldModifiedDate;
                                            [annotHandler modifyAnnot:annot addUndo:NO];
                                        }
                                    }
                                                    redo:^(UndoItem *item) {
                                                        FSFileAttachment *annot = [[FSFileAttachment alloc] initWithAnnot:[Utility getAnnotByNM:NM inPage:page]];
                                                        if (annot && ![annot isEmpty]) {
                                                            annot.contents = newName;
                                                            annot.modifiedDate = newModifiedDate;
                                                            [annotHandler modifyAnnot:annot addUndo:NO];
                                                        }
                                                    }
                                                    pageIndex:annot.pageIndex]];
            }
        }
    } else {
        NSString *oldDescription = attachmentItem.description;
        if (newName && [newName length] && ![newName isEqualToString:oldDescription]) {
            NSDate *oldModifiedDate = attachmentItem.modifyDate;
            NSDate *newModifiedDate = [NSDate date];
            @try {
                [attachmentItem.fileSpec setDescriptionW:newName];
            } @catch (NSException *exception) {
                NSString *msg = [NSString stringWithFormat:FSLocalizedForKey(@"kFailedChangeDocumentAttachmentDescription"), exception.description];
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(msg) preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                 style:UIAlertActionStyleCancel
                                                               handler:^(UIAlertAction *_Nonnull action) {
                                                                   [self popoverDidCancel];
                                                               }];
                [alertController addAction:action];
                [self presentAlertController:alertController];
                return;
            }
            attachmentItem.description = newName;
            [attachmentItem.fileSpec setModifiedDateTime:[Utility convert2FSDateTime:newModifiedDate]];
            [self onDocumentAttachmentModified:attachmentItem];

            if (addUndo) {
                [_extensionsManager addUndoItem:[UndoItem itemWithUndo:^(UndoItem *item) {
                                        attachmentItem.description = oldDescription;
                                        [attachmentItem.fileSpec setDescriptionW:oldDescription];
                                        [attachmentItem.fileSpec setModifiedDateTime:[Utility convert2FSDateTime:oldModifiedDate]];
                                        [self onDocumentAttachmentModified:attachmentItem];
                                    }
                                                    redo:^(UndoItem *item) {
                                                        attachmentItem.description = newName;
                                                        [attachmentItem.fileSpec setDescriptionW:newName];
                                                        [attachmentItem.fileSpec setModifiedDateTime:[Utility convert2FSDateTime:newModifiedDate]];
                                                        [self onDocumentAttachmentModified:attachmentItem];
                                                    }
                                                    pageIndex:-1]];
            }
        }
    }
}

#pragma mark - IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    self.indexPath = nil;
    if (error == FSErrSuccess && document && ![document isWrapper]) {
        [self loadData];
    }
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    [self clearData];
    
}

#pragma mark IPageEventListener

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    [self clearData];
    [self loadData];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    [self clearData];
    [self loadData];
}

- (void)onPagesInsertedAtRange:(NSRange)range {
    if (self.loadAttachmentQueue) {
        [self.loadAttachmentQueue cancelAllOperations];
        [self.loadAttachmentQueue waitUntilAllOperationsAreFinished];
    }
    [self.allAttachmentsSections removeAllObjects];

    [self loadData];
}

#pragma mark popover

- (void)presentAlertController:(UIAlertController *)alertController {
    UIViewController *rootVC = self;
    self.originalPresentedViewController = rootVC.presentedViewController;
    if (self.originalPresentedViewController) {
        [self.originalPresentedViewController dismissViewControllerAnimated:true
                                                                 completion:^{
                                                                     [rootVC presentViewController:alertController animated:YES completion:nil];
                                                                 }];
    } else {
        [rootVC presentViewController:alertController animated:YES completion:nil];
    }
}

- (void)popoverDidCancel {
    if (self.originalPresentedViewController) {
        UIViewController *rootVC = self;
        [rootVC presentViewController:self.originalPresentedViewController animated:true completion:nil];
        self.originalPresentedViewController = nil;
    }
}
@end
