/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "DigitalSignaturePanel.h"
#import "FSAnnotationListViewController.h"
#import "AnnotationStruct.h"
#import "PanelController+private.h"
#import "PanelHost.h"
#import "DigitalSignatureAnnotHandler.h"
#import "DynamicXFAHandler.h"
#import "FSDigitalSignatureViewController.h"

#define ENLARGE_EDGE 3

@interface FSViewSignatureViewController : FSViewController
@property (nonatomic, weak) UIExtensionsManager *extensionsMgr;
@end

@implementation FSViewSignatureViewController

- (BOOL)prefersStatusBarHidden {
    return self.extensionsMgr.prefersStatusBarHidden;
}


@end

@interface DigitalSignaturePanel ()

@property (nonatomic, strong) TabItem *tabItem;

@property (nonatomic, strong) UIExtensionsManager *signDocExtensionsMgr;

@property (nonatomic, strong) FSDigitalSignatureViewController *digitalSignatureVC;
@end

@implementation DigitalSignaturePanel

- (void)initialize{
    
    self.digitalSignatureVC = [[FSDigitalSignatureViewController alloc] initWithExtensionsManager:self.extensionsManager module:self];
    self.contentViewCtr = self.digitalSignatureVC;
    
    self.digitalSignatureVC.digitalsigDict = @{}.mutableCopy;
    self.digitalSignatureVC.view.frame = self.contentView.bounds;
    [self.contentView addSubview:self.digitalSignatureVC.view];
    [self.digitalSignatureVC.view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left);
        make.right.equalTo(self.contentView.mas_right);
        make.top.equalTo(self.contentView.mas_top);
        make.bottom.equalTo(self.contentView.mas_bottom);
    }];
    
    self.extensionsManager.isViewSignedDocument = NO;
    
    self.title =  FSLocalizedForKey(@"kDigitalSignatureTitle");
    [self.rightItem removeFromSuperview];
}


- (void)load {
    [self.pdfViewCtrl registerDocEventListener:self];
    [self.pdfViewCtrl registerPageEventListener:self];
    [self.panelController.panel insertSpec:self atIndex:4];
    [self.extensionsManager registerAnnotEventListener:self];
    [self.panelController registerPanelChangedListener:self];
}

- (void)unload {
    [self.pdfViewCtrl unregisterDocEventListener:self];
    [self.pdfViewCtrl unregisterPageEventListener:self];
    [self.panelController.panel removeSpec:self];
    [self.extensionsManager unregisterAnnotEventListener:self];
    [self.panelController unregisterPanelChangedListener:self];
}

- (void)cancelAction {
    self.panelController.isHidden = YES;
}



#pragma mark IPanelSpec
- (int)getType {
    return FSPanelTypeDigitalSignature;
}


- (TabItem *)getTabItem{
    if (!_tabItem) {
        _tabItem = [TabItem new];
        _tabItem.image = FSImageNamedAndNormalColor(@"panel_signature_gray");
        _tabItem.selectImage = [_tabItem.image fs_imageWithTintColor:self.fs_primaryColor];
    }
    return _tabItem;
}

- (void)onActivated {
    [super onActivated];
    [self reloadData];
    
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingPanelDigitalSignature];
}

- (void)onDeactivated {
    [super onDeactivated];
    self.extensionsManager.isViewSignedDocument = NO;
}

#pragma mark - IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden {

}

#pragma mark - IAnnotEventListener

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self reloadData];
}

-(void)onAnnotDeleted:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self reloadData];
}

-(void)onAnnotFlattened:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self reloadData];
}

- (void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self reloadData];
}

- (void)onCurrentAnnotChanged:(FSAnnot *)lastAnnot currentAnnot:(FSAnnot *)currentAnnot {
}

#pragma mark IDocEventListener
- (void)onDocWillOpen {
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    
    if (self.extensionsManager.isViewSignedDocument) {
        return ;
    }
    if (self.signDocExtensionsMgr.isViewSignedDocument) {
        return ;
    }
//    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, 2 * NSEC_PER_SEC);
//    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        [self reloadData];
//    });
}

- (void)onDocWillClose:(FSPDFDoc *)document{
    if ([document isEqual:[self.signDocExtensionsMgr.pdfViewCtrl currentDoc]]) {
        self.signDocExtensionsMgr.isViewSignedDocument = NO;
        self.extensionsManager.isViewSignedDocument = NO;
    }
}

- (void)onDocWillSave:(FSPDFDoc *)document {
}

#pragma mark IPageEventListener
- (void)onPagesWillRemove:(NSArray<NSNumber *> *)indexes {
}

- (void)onPagesWillMove:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
}

- (void)onPagesWillRotate:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    [self reloadData];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    [self reloadData];
}

- (void)onPagesInsertedAtRange:(NSRange)range {
    [self reloadData];
}

-(void)signSignature:(int)pageIndex signatureIndex:(int)signatureIndex uuid:(NSString *)uuid{
    @try {
        if ([self.pdfViewCtrl isDynamicXFA]) {
            FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
            FSXFAPage *page = [xfaDoc getPage:pageIndex];
            FSXFAWidget *xfaWidget = [page getWidgetByFullName:uuid];
            
            if (![xfaWidget isEmpty] && [xfaWidget getType] == FSXFAWidgetWidgetTypeSignature) {
                
                DynamicXFAHandler *annotHandler = nil;
                annotHandler = (DynamicXFAHandler *)[self.extensionsManager getDynamicAnnotHandler];
                annotHandler.handlerCurrentWidget = nil;
                [annotHandler.formNaviBar setHidden:YES];
                
                self.panelController.isHidden = YES;
                [self.pdfViewCtrl gotoPage:pageIndex animated:YES];
                
                BOOL isShowSignList = YES;
                if (isShowSignList) {
                    [self performSelector:@selector(setCurrentXFAWidgetItem:) withObject:xfaWidget afterDelay:0.5];
                }
                
                return ;
            }
            return;
        }
        
        FSSignature *signature = [self.pdfViewCtrl.currentDoc getSignature:signatureIndex];
        if ([signature isEmpty]) {
            return ;
        }
        FSWidget *widget = [[signature getControl:0] getWidget];
        
        @try {
            int pageIndex = [[widget getPage] getIndex];
            if (DEVICE_iPHONE) {
                self.panelController.isHidden = YES;
            }
            [self.pdfViewCtrl gotoPage:pageIndex animated:YES];
            [self performSelector:@selector(setCurrentAnnotionItem:) withObject:widget afterDelay:0.5];
            
        } @catch (NSException *exception) {
            [UIAlertController fs_alert_OK_WithTitle:@"kWarning" message:@"kUnknownError" onCtrl:[self.pdfViewCtrl fs_currentSceneTopMostViewController]];
        }
        
    } @catch (NSException *exception) {
    }
}

-(void)verifySignature:(int)pageIndex signatureIndex:(int)signatureIndex uuid:(NSString *)uuid{
    @try {
        if ([self.pdfViewCtrl isDynamicXFA]) {
            FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
            FSXFAPage *page = [xfaDoc getPage:pageIndex];
            FSXFAWidget *xfaWidget = [page getWidgetByFullName:uuid];
            
            if (![xfaWidget isEmpty] && [xfaWidget getType] == FSXFAWidgetWidgetTypeSignature) {
                
                FSSignature *signature = [xfaWidget getSignature];
                if ([signature isEmpty]) return ;
                
                CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[xfaWidget getRect] pageIndex:pageIndex];
                CGRect resultRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:newRect pageIndex:pageIndex];
                
                DynamicXFASignatureAnnotHandler *signatureAnnotHandler = self.extensionsManager.dynamicXFASignatureAnnotHandler;
                signatureAnnotHandler.isDynamicXFAFile = YES;
                signatureAnnotHandler.menuRect = resultRect;
                signatureAnnotHandler.editXFAWidget = xfaWidget;
                [signatureAnnotHandler dynamicXFASignatureSelected: signature shouldShowMenu:NO];
                [signatureAnnotHandler verify];
                
                return ;
            }
            return;
        }
        
        FSSignature *signature = [self.pdfViewCtrl.currentDoc getSignature:signatureIndex];
        if ([signature isEmpty]) {
            return ;
        }
        FSWidget *widget = [[signature getControl:0] getWidget];
        
        DigitalSignatureAnnotHandler *annotHandler = nil;
        annotHandler = (DigitalSignatureAnnotHandler *)[self.extensionsManager getAnnotHandlerByAnnot:widget];
        annotHandler.currentSelectSign = signature;
        [annotHandler verify];
        
    } @catch (NSException *exception) {
    }
}

-(void)viewSignature:(int)pageIndex signatureIndex:(int)signatureIndex uuid:(NSString *)uuid{
    @try {
        if ([self.pdfViewCtrl isDynamicXFA]) {
            FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
            FSXFAPage *page = [xfaDoc getPage:pageIndex];
            FSXFAWidget *xfaWidget = [page getWidgetByFullName:uuid];
            
            if (![xfaWidget isEmpty] && [xfaWidget getType] == FSXFAWidgetWidgetTypeSignature) {

                unsigned long fileLength = [self.pdfViewCtrl.currentDoc getFileSize];
                FSInt32Array *byteRanges = [[xfaWidget getSignature] getByteRangeArray];
                unsigned int r1 = 0;
                unsigned int r2 = 0;
                if ([byteRanges getSize] == 4) {
                    r1 = [byteRanges getAt:2];
                    r2 = [byteRanges getAt:3];
                }
                
                if (fileLength < (r1 + r2)) {
                    [self.contentView fs_showHUDMessage:FSLocalizedForKey(@"kDigitalSignatureFaildView")];
                }else if (fileLength == (r1 + r2)){
                    [self.contentView fs_showHUDMessage:FSLocalizedForKey(@"kDigitalSignatureAllReadyView")];
                }else{
                    FSPDFDoc *doc = [[xfaWidget getSignature] getSignedVersionDocument:self.pdfViewCtrl.filePath];
                    if ([doc isEmpty]) return;
                    if ([doc load:nil] == FSErrSuccess) {
                        BOOL isUseTabToView = NO;
                        if (self.extensionsManager.isMultiFileMode && isUseTabToView) {
                            NSString *tempDirectory = NSTemporaryDirectory();
                            NSString *tmpPath = [tempDirectory stringByAppendingString:[NSString stringWithFormat:@"tmpdigs_%@.pdf",[[xfaWidget getSignature] getName]]];
                            [doc saveAs:tmpPath save_flags:FSPDFDocSaveFlagNormal];
                            
                            NSString *path = tmpPath;
                            [self.extensionsManager.delegate uiextensionsManager:self.extensionsManager openNewDocAtPath:path shouldCloseCurrentDoc:NO completionCallBack:^{}];
                            
                        } else{
                            [self previewSignedVersionDocument:doc];
                        }
                        
                        
                    }else {
                        [self.contentView fs_showHUDMessage:FSLocalizedForKey(@"kFileErrorNotLoaded")];
                        
                        self.extensionsManager.isViewSignedDocument = NO;
                    }
                    
                }
            }
            return;
        }
        
        FSSignature *signature = [self.pdfViewCtrl.currentDoc getSignature:signatureIndex];
        if ([signature isEmpty]) {
            return ;
        }
//        FSWidget *widget = [[signature getControl:0] getWidget];
        
        unsigned long fileLength = [self.pdfViewCtrl.currentDoc getFileSize];
        FSInt32Array *byteRanges = [signature getByteRangeArray];
        unsigned int r1 = 0;
        unsigned int r2 = 0;
        if ([byteRanges getSize] == 4) {
            r1 = [byteRanges getAt:2];
            r2 = [byteRanges getAt:3];
        }
        
        if (fileLength < (r1 + r2)) {
            [self.contentView fs_showHUDMessage:FSLocalizedForKey(@"kDigitalSignatureFaildView")];
        }else if (fileLength == (r1 + r2)){
            [self.contentView fs_showHUDMessage:FSLocalizedForKey(@"kDigitalSignatureAllReadyView")];
        }else{
            
            FSPDFDoc *doc = [signature getSignedVersionDocument:self.pdfViewCtrl.filePath];
            if ([doc isEmpty]) return;
            if ([doc load:nil] == FSErrSuccess) {
                BOOL isUseTabToView = NO;
                if (self.extensionsManager.isMultiFileMode && isUseTabToView) {
                    NSString *tempDirectory = NSTemporaryDirectory();
                    NSString *tmpPath = [tempDirectory stringByAppendingString:[NSString stringWithFormat:@"tmpdigs_%@.pdf",[signature getName]]];
                    [doc saveAs:tmpPath save_flags:FSPDFDocSaveFlagNormal];
                    
                    NSString *path = tmpPath;
                    [self.extensionsManager.delegate uiextensionsManager:self.extensionsManager openNewDocAtPath:path shouldCloseCurrentDoc:NO completionCallBack:^{}];
                    
                } else{
                    [self previewSignedVersionDocument:doc];
                }
            }else {
                [self.contentView fs_showHUDMessage:FSLocalizedForKey(@"kFileErrorNotLoaded")];
                self.extensionsManager.isViewSignedDocument = NO;
            }
        }
    } @catch (NSException *exception) {
        NSLog(@"%@",exception);
    }
}

-(void)previewSignedVersionDocument:(FSPDFDoc *)doc {
    FSPDFViewCtrl *pdfviewctrl = [[FSPDFViewCtrl alloc] initWithFrame:self.pdfViewCtrl.bounds];
    [pdfviewctrl setRMSAppClientId:@"972b6681-fa03-4b6b-817b-c8c10d38bd20" redirectURI:@"com.foxitsoftware.com.mobilepdf-for-ios://authorize"];
    [pdfviewctrl registerDocEventListener:self];
    
    self.signDocExtensionsMgr = [[UIExtensionsManager alloc] initWithPDFViewControl:pdfviewctrl configurationObject:self.extensionsManager.config];
    
    pdfviewctrl.extensionsManager = self.signDocExtensionsMgr;
    [pdfviewctrl setViewSignedDocument:YES];
    [pdfviewctrl setDoc:doc];
    [pdfviewctrl setValue:self.pdfViewCtrl.filePath forKey:@"filePath"];
    
    self.extensionsManager.isViewSignedDocument = YES;
    self.signDocExtensionsMgr.isViewSignedDocument = YES;
    
    FSViewSignatureViewController *pdfViewController = [[FSViewSignatureViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
    pdfViewController.extensionsMgr = self.signDocExtensionsMgr;
    [pdfViewController.view addSubview: pdfviewctrl];
    [pdfviewctrl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.bottom.top.mas_equalTo(pdfviewctrl.superview);
    }];
    
    
    [pdfViewController fs_setCustomTransitionAnimator:self.extensionsManager.presentedVCAnimator];
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.extensionsManager.viewCtrlManager presentViewController:pdfViewController animated:YES completion:nil];
    });

    __weak typeof(self) weakSelf = self;
    self.signDocExtensionsMgr.goBack = ^() {
        __strong typeof(self) strongSelf = weakSelf;

        if (pdfViewController != nil || strongSelf.signDocExtensionsMgr.isViewSignedDocument) {
            [pdfViewController.services dismissViewControllerAnimated:YES completion:nil];
        }
    };
}

-(void)gotoPage:(int)pageIndex signatureIndex:(int)signatureIndex uuid:(NSString *)uuid isShowSignList:(BOOL)isShowSignList {
    @try {
        if ([self.pdfViewCtrl.currentDoc isXFA] && [self.pdfViewCtrl isDynamicXFA] ) {
            FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
            FSXFAPage *page = [xfaDoc getPage:pageIndex];
            FSXFAWidget *xfaWidget = [page getWidgetByFullName:uuid];
            
            if (![xfaWidget isEmpty] && [xfaWidget getType] == FSXFAWidgetWidgetTypeSignature) {
                
                self.panelController.isHidden = YES;
                [self.pdfViewCtrl gotoPage:pageIndex animated:YES];
                
                BOOL isShowSignList = YES;
                if (isShowSignList) {
                    [self performSelector:@selector(setCurrentXFAWidgetItem:) withObject:xfaWidget afterDelay:0.5];
                }
                
                return ;
            }
            return;
        }
        
        FSSignature *signature = [self.pdfViewCtrl.currentDoc getSignature:signatureIndex];
        if ([signature isEmpty]) {
            return ;
        }
        FSWidget *widget = [[signature getControl:0] getWidget];
        if ([widget isEmpty]) return;
        @try {
            int pageIndex = [[widget getPage] getIndex];
            self.panelController.isHidden = YES;
            [self.pdfViewCtrl gotoPage:pageIndex animated:YES];
            
            if (isShowSignList) {
                [self performSelector:@selector(setCurrentAnnotionItem:) withObject:widget afterDelay:0.5];
            }
            
        } @catch (NSException *exception) {
        }
        
    } @catch (NSException *exception) {
    }
}

- (void)setCurrentAnnotionItem:(FSWidget *)widget {
    if ([self.pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW) {
        [self.extensionsManager setCurrentAnnot:widget];
    }
}

- (void)setCurrentXFAWidgetItem:(FSXFAWidget *)widget {
    if ([self.pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW) {
        [self.extensionsManager setCurrentWidget:widget];
        
        int pageIndex = [[widget getXFAPage] getIndex];
        FSSignature *signature = [widget getSignature];
        if ([signature isEmpty]) return ;
        
        CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[widget getRect] pageIndex:pageIndex];
        CGRect resultRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:newRect pageIndex:pageIndex];
        
        DynamicXFASignatureAnnotHandler *signatureAnnotHandler = self.extensionsManager.dynamicXFASignatureAnnotHandler;
        signatureAnnotHandler.isDynamicXFAFile = YES;
        signatureAnnotHandler.menuRect = resultRect;
        signatureAnnotHandler.editXFAWidget = widget;
        [signatureAnnotHandler dynamicXFASignatureSelected:signature shouldShowMenu:YES];
    }
}

-(void)getSignaturesInXFADoc {
    @try {
        FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
        int pageCount = [xfaDoc getPageCount];
        
        NSMutableArray *array = @[].mutableCopy;
        for (int i = 0 ; i < pageCount ; i++) {
            FSXFAPage *page = [xfaDoc getPage:i];
            int widgetCount = [page getWidgetCount];
            
            int signIndex = 1;
            for (int m = 0 ; m < widgetCount; m++) {
                FSXFAWidget *xfaWidget = [page getWidget:m];
                if (![xfaWidget isEmpty] && [xfaWidget getType] == FSXFAWidgetWidgetTypeSignature) {
                    FSSignature *signature = [xfaWidget getSignature];
                    if ([signature isEmpty]) {
                        continue;
                    }
                    
                    if ([signature isEmpty]) {
                        continue;
                    }
                    NSMutableDictionary *itemData = @{}.mutableCopy;
                    [itemData setObject:@([signature isSigned]) forKey:@"isSigned"];
                    if ([signature isSigned]) {
                        NSString *signer = nil;
                        
                        NSString *certSubject = [signature getCertificateInfo:@"Subject"];
                        NSRange r = [certSubject rangeOfString:@"CN="];
                        if (0 < r.length) {
                            certSubject = [certSubject substringFromIndex:r.location + 3];
                            r = [certSubject rangeOfString:@","];
                            if (0 < r.length) {
                                certSubject = [certSubject substringToIndex:r.location];
                            }
                        }
                        signer = certSubject;
                        
                        if (signer == nil || signer.length == 0) {
                            signer = [signature getKeyValue:FSSignatureKeyNameSigner];
                        }
                        
                        if (signer == nil || signer.length == 0) {
                            signer = FSLocalizedForKey(@"kDigitalSignatureListSignerUnknown");
                        }

                        [itemData setObject:signer forKey:@"signer"];
                        
                        [itemData setObject:[signature getSignTime] forKey:@"signdate"];
                        [itemData setObject:@(signIndex++) forKey:@"signIndex"];
                    }else{
                        [itemData setObject:[signature getName] forKey:@"signName"];
                    }
                    
                    [itemData setObject:@(NO) forKey:@"readOnly"];
                    if ([signature getFlags] & FSFieldFlagReadOnly) {
                        [itemData setObject:@(YES) forKey:@"readOnly"];
                    }
                    
                    [itemData setObject:@(i) forKey:@"pageIndex"];
                    
                    [itemData setObject:[xfaWidget getName:FSXFAWidgetWidgetNameTypeFullName] forKey:@"uuid"];
                    
                    [array addObject:itemData];
                }
            }
        }
        
        if (array.count > 0) {
            [self.digitalSignatureVC.digitalsigDict setObject:array forKey:@(-1)];
        }
    } @catch (NSException *exception) {
        
    }
}

-(void)getSignatureListData:(FSPDFDoc *)document {
    @try {
        if (!document || [document isEmpty]) return;
        if ([document isXFA] && [self.pdfViewCtrl isDynamicXFA] ) {
            [self.pdfViewCtrl lockRefresh];
            [self getSignaturesInXFADoc];
            [self.pdfViewCtrl unlockRefresh];
            return ;
        }
    } @catch (NSException *exception) {
        return;
    }
    
    @try {
        int signIndex = 1;
        int signatureCount = [document getSignatureCount];
        
        NSMutableArray *array = @[].mutableCopy;
        for (int i = 0 ; i < signatureCount; i ++) {
            FSSignature *signature = [document getSignature:i];

            if ([signature isEmpty]) {
                continue;
            }
            NSMutableDictionary *itemData = @{}.mutableCopy;
            [itemData setObject:@([signature isSigned]) forKey:@"isSigned"];
            if ([signature isSigned]) {
                
                NSString *certSubject = [signature getCertificateInfo:@"Subject"];
                NSRange r = [certSubject rangeOfString:@"CN="];
                if (0 < r.length) {
                    certSubject = [certSubject substringFromIndex:r.location + 3];
                    r = [certSubject rangeOfString:@","];
                    if (0 < r.length) {
                        certSubject = [certSubject substringToIndex:r.location];
                    }
                }
          
                NSString *signer  = certSubject;
                
                if (!signer.length) {
                    NSMutableArray *arr =  [FSPDFCertUtil getCertFromSignatureContent:signature];
                    if (arr != nil) {
                        NSDictionary *dic = [FSPDFCertUtil getTrustCertificateInformation:[arr lastObject]];
                        signer = dic[@"name"];
                    }
                }
                
                if (signer == nil || signer.length == 0) {
                    signer = FSLocalizedForKey(@"kDigitalSignatureListSignerUnknown");
                }
                
                [itemData setObject:signer forKey:@"signer"];
                
                [itemData setObject:[signature getSignTime] forKey:@"signdate"];
                [itemData setObject:@(signIndex++) forKey:@"signIndex"];
            }else{
                [itemData setObject:[signature getName] forKey:@"signName"];
            }
            
            [itemData setObject:@(-1) forKey:@"pageIndex"];
            
            [itemData setObject:@(NO) forKey:@"readOnly"];
            if ([signature getFlags] & FSFieldFlagReadOnly) {
                [itemData setObject:@(YES) forKey:@"readOnly"];
            }
            
            FSWidget *widget = [[signature getControl:0] getWidget];
            @try {
                if (![widget isEmpty]){
                    [itemData setObject:[widget NM] forKey:@"uuid"];
                }
                [array addObject:itemData];
        
            }@catch (NSException *exception){
                continue;
            }
        }
        if (array.count > 0) {
            [self.digitalSignatureVC.digitalsigDict setObject:array forKey:@(-1)];
        }

    }@catch (NSException *exception) {
    }
}

- (void)reloadData {
    [self.digitalSignatureVC.digitalsigDict removeAllObjects];
    [self getSignatureListData:self.pdfViewCtrl.currentDoc];
    [self.digitalSignatureVC reloadData];
}

@end
