/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PanelHost.h"
#import "PanelController.h"
#import "AnnotationPanel.h"
#import "AttachmentPanel.h"
#import "OutlinePanel.h"
#import "PanelController+private.h"
#import "PanelHost.h"
#import "ReadingBookmarkPanel.h"
#import "DigitalSignaturePanel.h"

@interface FSPanelController ()
@property (nonatomic,strong) NSArray *allPanelTypes;
@end

@implementation FSPanelController {
    __weak UIView *_superView;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    __weak UIExtensionsManager *_extensionsManager;
    AnnotationPanel *annotationPanel;
    OutlinePanel *outlinePanel;
    ReadingBookmarkPanel *bookmarkPanel;
    AttachmentPanel *attachmentPanel;
    DigitalSignaturePanel *digitalSignaturePanel;
    UIControl *_maskControl;
}

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _superView = extensionsManager.containerView;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;
        _isHidden = YES;

        self.panel = [[PanelHost alloc] initWithExtensionsManager:extensionsManager panelController:self];
        self.panelListeners = [[NSMutableArray alloc] init];
        CGRect screenFrame = _pdfViewCtrl.bounds;
        if (DEVICE_iPHONE) {
            self.panel.contentView.frame = CGRectMake(0, 0, screenFrame.size.width, screenFrame.size.height);
        } else {
            self.panel.contentView.frame = CGRectMake(0, 0, 300, screenFrame.size.height);
        }
        
        self.allPanelTypes = @[@(FSPanelTypeReadingBookmark),@(FSPanelTypeOutline),@(FSPanelTypeAnnotation),@(FSPanelTypeAttachment),@(FSPanelTypeDigitalSignature)];

        NSArray<NSNumber *> *panelTypes = ({
            UIExtensionsConfig *config = _extensionsManager.config;
            NSMutableArray<NSNumber *> *panelTypes = @[].mutableCopy;
            if (config.loadReadingBookmark) {
                [panelTypes addObject:@(FSPanelTypeReadingBookmark)];
            }
            if (config.loadOutline) {
                [panelTypes addObject:@(FSPanelTypeOutline)];
            }
            if (config.hasAnnotTools && [config.tools fs_containsAnyObjectNotInArray:@[ Tool_Select, Tool_Signature ]]) {
                [panelTypes addObject:@(FSPanelTypeAnnotation)];
            }
            if (config.loadAttachment) {
                [panelTypes addObject:@(FSPanelTypeAttachment)];
            }
            if (config.loadSignature) {
                [panelTypes addObject:@(FSPanelTypeDigitalSignature)];
            }

            panelTypes;
        });
        
//        load bookmark panel
        if ([panelTypes containsObject:@(FSPanelTypeReadingBookmark)]) {
            bookmarkPanel = [[ReadingBookmarkPanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
            [bookmarkPanel load];
        }
        //load outline panel
        if ([panelTypes containsObject:@(FSPanelTypeOutline)]) {
            outlinePanel = [[OutlinePanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
            [outlinePanel load];
        }
        //Load annotation panel
        if ([panelTypes containsObject:@(FSPanelTypeAnnotation)]) {
            annotationPanel = [[AnnotationPanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
            [annotationPanel load];
        }
        //load attachment panel
        if ([panelTypes containsObject:@(FSPanelTypeAttachment)]) {
            attachmentPanel = [[AttachmentPanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
            [attachmentPanel load];
        }
        //Load digital signature panel
        if ([panelTypes containsObject:@(FSPanelTypeDigitalSignature)]) {
            digitalSignaturePanel = [[DigitalSignaturePanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
            [digitalSignaturePanel load];
        }
        [self.panel reloadData];
    }
    return self;
}

- (void)viewDidLoad{
    [super viewDidLoad];
    
    [self.transitionCoordinator notifyWhenInteractionChangesUsingBlock:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {}];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(willEnterForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
}

- (void)willEnterForeground:(NSNotification *)notification {
    [self refreshPanelView];
}

- (void)setView:(UIView *)view{
    [super setView:self.panel.contentView];
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    
    //rotating
#if !_MAC_CATALYST_
    [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        if (!(DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) && !self.isHidden) {
            [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
                [self.panel.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.bottom.equalTo(self->_superView);
                    make.left.equalTo(self->_superView).priorityHigh();
                    make.top.equalTo([self->_extensionsManager.topToolbar getToolbarContentView] ? [self->_extensionsManager.topToolbar getToolbarContentView].mas_bottom : self->_extensionsManager.containerView.mas_top);
                    make.width.mas_equalTo(self->_superView).multipliedBy(self->_pdfViewCtrl.fs_width > self->_pdfViewCtrl.fs_height ? 0.3 : 0.4);
                }];
            } completion:^(BOOL finished) {
                if (finished) {
                    [self.panel reloadData];
                }
            }];
        }
    } completion:^(BOOL finished) {
        if (finished) {
            [self.panel reloadData];
        }
    }];
#endif
    //add shadow

    self.view.layer.masksToBounds = NO;
    self.view.layer.shadowColor = self.fs_dividerColor_P3.CGColor;
    self.view.layer.shadowOffset = CGSizeMake(0,0);
    self.view.layer.shadowOpacity = 0.5;
    self.view.layer.shadowRadius = 3;
    
    float shadowPathWidth = self.view.layer.shadowRadius;
    CGRect shadowRect = CGRectMake(self.view.bounds.size.width, 0, shadowPathWidth , self.view.bounds.size.height);
    UIBezierPath *path = [UIBezierPath bezierPathWithRect:shadowRect];
    self.view.layer.shadowPath = path.CGPath;
}

#if !_MAC_CATALYST_
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    for ( id<IPanelSpec> thePanel in self.panel.specs) {
        if ([thePanel respondsToSelector:@selector(resetToolBar)]) {
            [thePanel performSelector:@selector(resetToolBar)];
        }
    }
    
    [self refreshPanelView];
}
#endif

- (void)refreshPanelView{
    if (!self.isHidden){
        [self.panel.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self->_superView);
            make.right.equalTo(self->_superView.mas_left).priorityMedium();
            make.left.equalTo(self->_superView);
            if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                make.top.width.equalTo(self->_superView);
            } else {
                make.top.equalTo([self->_extensionsManager.topToolbar getToolbarContentView] ? [self->_extensionsManager.topToolbar getToolbarContentView].mas_bottom : self->_extensionsManager.containerView.mas_top);
                make.width.mas_equalTo(self->_superView).multipliedBy(self->_pdfViewCtrl.fs_width > self->_pdfViewCtrl.fs_height ? 0.3 : 0.4);
            }
        }];
//        [self.panel.contentView layoutIfNeeded];
//        [self->_superView layoutIfNeeded];
        [self.panel reloadData];

        FSBasePanel *pannel = (FSBasePanel*)self.panel.currentSpec;
        [pannel quitSearch];
        
    }
    
    if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        if (!self.isHidden && self.view.superview) {
            [self.view.superview bringSubviewToFront:self.view];
        }
        
    }
}

#pragma mark - get panels show/hide status
-(NSMutableDictionary *)getItemHiddenStatus {
    NSMutableDictionary *panelInfo = [@{
                                        @"ReadingBookmark":@"YES",
                                        @"Outline":@"YES",
                                        @"Annotation":@"YES",
                                        @"Attachment":@"YES",
                                        @"DigitalSignature":@"YES",
                                        } mutableCopy];
    
    NSMutableDictionary *panelsTags = [@{
                                         [NSNumber numberWithInt:FSPanelTypeReadingBookmark]:@"ReadingBookmark",
                                         [NSNumber numberWithInt:FSPanelTypeOutline]:@"Outline",
                                         [NSNumber numberWithInt:FSPanelTypeAnnotation]:@"Annotation",
                                         [NSNumber numberWithInt:FSPanelTypeAttachment]:@"Attachment",
                                         [NSNumber numberWithInt:FSPanelTypeDigitalSignature]:@"DigitalSignature",
                                         } mutableCopy];
    
    NSArray *tempArr = nil;
    
    for ( id<IPanelSpec> thePanel in self.panel.specs) {
        int tag = [thePanel getType];
        tempArr = [panelsTags allKeys];
        
        if ([tempArr containsObject: [NSNumber numberWithInt:tag]]) {
            [panelInfo setObject:@"NO" forKey:[panelsTags objectForKey:[NSNumber numberWithInt:tag]]];
        }
    }
    
    return panelInfo;
}

- (id<IPanelSpec>)getPanel:(FSPanelType) type {
    for ( id<IPanelSpec> thePanel in self.panel.specs) {
        int getType = [thePanel getType];
        if(getType == type)
            return thePanel;
    }
    return nil;
}

- (BOOL)isPanelHidden:(FSPanelType) type {
    for ( id<IPanelSpec> thePanel in self.panel.specs) {
        int getType = [thePanel getType];
        if(getType == type)
            return NO;
    }
    return YES;
}

- (void)setPanelHidden:(BOOL)isHidden type:(FSPanelType)type {
    if (isHidden) {
        [self removePanelOfType:type];
    } else {
        [self addPanelOfType:type];
    }
}

- (void)removePanelOfType:(FSPanelType)type {
    id<IPanelSpec> thePanel = nil;
    switch (type) {
        case FSPanelTypeOutline:
            thePanel = outlinePanel;
            [outlinePanel unload];
            outlinePanel = nil;
            break;
        case FSPanelTypeAnnotation:
            thePanel = annotationPanel;
            [annotationPanel unload];
            annotationPanel = nil;
            break;
        case FSPanelTypeAttachment:
            thePanel = attachmentPanel;
            [attachmentPanel unload];
            attachmentPanel = nil;
            break;
        case FSPanelTypeReadingBookmark:
            thePanel = bookmarkPanel;
            [bookmarkPanel unload];
            bookmarkPanel = nil;
            break;
        case FSPanelTypeDigitalSignature:
            thePanel = digitalSignaturePanel;
            [digitalSignaturePanel unload];
            digitalSignaturePanel = nil;
            break;
        default:
            break;
    }
    if(!thePanel) return;
    [self.panel reloadData];
}

- (void)addPanelOfType:(FSPanelType)type {
    switch (type) {
        case FSPanelTypeOutline:
            if (_extensionsManager.config.loadOutline && outlinePanel == nil) {
                outlinePanel = [[OutlinePanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
                [outlinePanel load];
            }
            break;
        case FSPanelTypeAnnotation:
            if (_extensionsManager.config.tools.count > 0 && annotationPanel == nil) {
                annotationPanel = [[AnnotationPanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
                [annotationPanel load];
            }
            break;
        case FSPanelTypeAttachment:
            if (_extensionsManager.config.loadAttachment && attachmentPanel == nil) {
                attachmentPanel = [[AttachmentPanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
                [attachmentPanel load];
            }
            break;
        case FSPanelTypeReadingBookmark:
            if (_extensionsManager.config.loadReadingBookmark && bookmarkPanel == nil) {
                bookmarkPanel = [[ReadingBookmarkPanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
                [bookmarkPanel load];
            }
        break;
        case FSPanelTypeDigitalSignature:
            if (_extensionsManager.config.loadSignature && digitalSignaturePanel == nil) {
                digitalSignaturePanel = [[DigitalSignaturePanel alloc] initWithUIExtensionsManager:_extensionsManager panelController:self];
                [digitalSignaturePanel load];
            }
            break;
        default:
            break;
    }
    
    NSMutableArray<id<IPanelSpec>> *tempSpecs = [NSMutableArray<id<IPanelSpec>> array];
    for (NSNumber *typeNum in self.allPanelTypes) {
        for ( id<IPanelSpec> thePanel in self.panel.specs) {
            if ([thePanel getType] == [typeNum intValue]) {
                [tempSpecs addObject:thePanel];
            }
        }
    }
    self.panel.specs = tempSpecs;

    [self.panel reloadData];
}

- (void)setIsHidden:(BOOL)isHidden {
    [self setIsHidden:isHidden animated:YES];
}

- (void)setIsHidden:(BOOL)isHidden animated:(BOOL)animated {
    if (_isHidden == isHidden) {
        return;
    }
    _isHidden = isHidden;
    if (_isHidden) {
        if (self.panel.contentView.superview) {
            [self.panel.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.bottom.equalTo(self->_superView);
                make.right.equalTo(self->_superView.mas_left);
                if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.top.width.equalTo(self->_superView);
                }else{
                   make.top.equalTo([self->_extensionsManager.topToolbar getToolbarContentView] ? [self->_extensionsManager.topToolbar getToolbarContentView].mas_bottom : self->_extensionsManager.containerView.mas_top);
                    make.width.mas_equalTo(self->_superView).multipliedBy(_pdfViewCtrl.fs_width > _pdfViewCtrl.fs_height ? 0.3 : 0.4);
                }
            }];
            if (_maskControl) {
                [_maskControl removeFromSuperview];
                _maskControl = nil;
            }
            [_superView setNeedsUpdateConstraints];
            [_superView updateConstraintsIfNeeded];
        }
        if (animated) {
            [UIView animateWithDuration:0.4
                animations:^{
                    [self.panel.contentView layoutIfNeeded];
                    [self->_superView layoutIfNeeded];
                }
                completion:^(BOOL finished) {
                    self.panel.contentView.hidden = self->_isHidden;
                }];
        } else {
            self.panel.contentView.hidden = _isHidden;
        }
        [self.panel.currentSpec onDeactivated];
    } else {
        
        [_extensionsManager setCurrentAnnot:nil];
        //If on iPhone device, move the PanelView to the top level
        if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact] || DEVICE_iPHONE) {
            if (self.view.superview) {
                [self.view.superview bringSubviewToFront:self.view];
            }
        }

        //adjust the toolbar width
        [self loadViewIfNeeded];
        if (![_superView.subviews containsObject:self.panel.contentView]) {
            [_superView addSubview:self.panel.contentView];
        }
        [self.panel.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self->_superView);
            make.right.equalTo(self->_superView.mas_left).priorityMedium();
            make.left.equalTo(self->_superView).priorityLow();
            if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                make.top.width.equalTo(self->_superView);
            } else {
                make.top.equalTo([self->_extensionsManager.topToolbar getToolbarContentView] ? [self->_extensionsManager.topToolbar getToolbarContentView].mas_bottom : self->_extensionsManager.containerView.mas_top);
                make.width.mas_equalTo(self->_superView).multipliedBy(_pdfViewCtrl.fs_width > _pdfViewCtrl.fs_height ? 0.3 : 0.4);
            }
        }];
        
        if (!_maskControl) {
            _maskControl = [[UIControl alloc] init];
            @weakify(self)
            [_maskControl setBlockForControlEvents:UIControlEventTouchUpInside block:^(id  _Nonnull sender) {
                @strongify(self)
                self.isHidden = YES;
            }];
        }
        [_superView addSubview:_maskControl];
        [_maskControl mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.bottom.fs_equalToSuperview();
            make.top.equalTo(self.panel.contentView);
            make.left.equalTo(self.panel.contentView.mas_right);
        }];
        [_superView setNeedsUpdateConstraints];
        [_superView updateConstraintsIfNeeded];
        [_superView layoutIfNeeded];

        self.panel.contentView.hidden = _isHidden;
        
        [self.panel.contentView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self->_superView).priorityHigh();
            make.right.equalTo(self->_superView.mas_left).priorityLow();
            if (DEVICE_iPHONE) {
                make.right.equalTo(self->_superView).priorityHigh();
            }
        }];
        
        [_superView setNeedsUpdateConstraints];
        [_superView updateConstraintsIfNeeded];
        if (animated) {
            [UIView animateWithDuration:0.4
                             animations:^{
                [self->_superView layoutIfNeeded];
            }
                             completion:^(BOOL finished){
            }];
        }
        [self.panel reloadData];
        [self.panel.currentSpec onActivated];
    }
    
    for (id<IPanelChangedListener> listener in self.panelListeners) {
        if ([listener respondsToSelector:@selector(onPanelChanged:)]) {
            [listener onPanelChanged:_isHidden];
        }
    }
    
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingPanel];
}

- (void)canelPanel:(UIButton *)button {
    self.isHidden = YES;
}

- (void)dismiss:(id)sender {
    self.isHidden = YES;
}

- (void)registerPanelChangedListener:(id<IPanelChangedListener>)listener {
    if (self.panelListeners) {
        [self.panelListeners addObject:listener];
    }
}

- (void)unregisterPanelChangedListener:(id<IPanelChangedListener>)listener {
    if ([self.panelListeners containsObject:listener]) {
        [self.panelListeners removeObject:listener];
    }
}

- (void)reloadReadingBookmarkPanel {
    [bookmarkPanel reloadData];
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

@end
