/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PanelHost.h"
#import "PanelController.h"
#import "PanelHost+private.h"
#import "FSPagerView.h"
#import "FSBasePanel.h"

@interface PanelHost ()<FSPagerViewDataSource, FSPagerViewDelegate, FSTabPagerBarDelegate, FSTabPagerBarDataSource>

@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSPanelController *panelController;
@end

@implementation PanelHost

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager panelController:(FSPanelController *)panelController {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _panelController = panelController;
        self.specs = [[NSMutableArray alloc] init];
        self.contentView = [UIView new];
        self.contentView.backgroundColor = self.fs_backgroundColor_B1;
        [self addToobar];
        [self adddPagerView];
    }
    return self;
}

- (void)addToobar{
    FSToolbar *topBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
    topBar.contentBarHeight = TOOL_BAR_HEIGHT_NORMAL * 2;

    StateBarThemeKeyPath(topBar.stateBar, backgroundColor);
    [BindThemeKeySel(topBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
        StateBarThemeKeyPath(target, backgroundColor);
    }];
    [self.contentView addSubview:topBar];
    
    [topBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.fs_equalToSuperview();
    }];
    self.topBar = topBar;
}

- (void)adddPagerView{
    _tabPagerBar = [FSTabPagerBar new];
    _tabPagerBar.delegate = self;
    _tabPagerBar.dataSource = self;
    _tabPagerBar.itemEdging = 0;
    _tabPagerBar.itemContentMode = FSTabPagerBarItemContentModeScaleAspectFit;
    BindThemeKeyPath(_tabPagerBar, progressColor);
    [self.topBar addSubview:_tabPagerBar];
    [_tabPagerBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.bottom.fs_equalToSuperview();
        make.height.mas_equalTo(TOOL_BAR_HEIGHT_NORMAL);
    }];
    
    _pagerContentView = [FSPagerView new];
    _pagerContentView.delegate = self;
    _pagerContentView.dataSource = self;
    [self.contentView addSubview:_pagerContentView];
    [_pagerContentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.pagerContentView.superview.mas_left);
        make.right.mas_equalTo(self.pagerContentView.superview.mas_right);
        make.bottom.fs_equalToSuperview();
        make.top.mas_equalTo(self.topBar.mas_bottom);
    }];
}

- (void)reloadData{
    
    FSMainToolbarItem *panelItem = [self.extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypePanel];
    panelItem.item.enabled = self.specs.count;
    
    if (!_currentSpec) {
        [self setCurrentSpec:[self.specs firstObject]];
    }
    if (!_currentSpec) return;
    
    [_pagerContentView reloadData];
    [_tabPagerBar reloadData];
}

#pragma mark <SegmentDelegate>

- (void)updateSpec:(id<IPanelSpec>)spec  {
    UIView *topToolbar = [spec getTopToolbar];
    
    [self.topBar addSubview:topToolbar];
    [topToolbar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.right.fs_equalToSuperview();
        make.bottom.mas_equalTo(self.tabPagerBar.mas_top);
    }];
    
    [topToolbar setHidden:YES];
}

- (void)addSpec:(id<IPanelSpec>)spec {
    [self.specs addObject:spec];
    [self updateSpec:spec];
}

- (void)insertSpec:(id<IPanelSpec>)spec atIndex:(int)index {
    if(index > self.specs.count)
        [self.specs addObject:spec];
    else
        [self.specs insertObject:spec atIndex:index];
    [self updateSpec:spec];
    
    NSInteger curIndex = [self.specs indexOfObject:self.currentSpec];
    if (curIndex != NSNotFound && _tabPagerBar.curIndex != curIndex) {
        [_tabPagerBar scrollToItemAtIndex:curIndex animated:NO];
        [_pagerContentView scrollToViewAtIndex:curIndex animate:NO];
    }
}

- (void)removeSpec:(id<IPanelSpec>)spec {
    if (spec == self.currentSpec) {
        self.currentSpec = ({
            id<IPanelSpec> firstOtherPanel = nil;
            for (id<IPanelSpec> panel in self.specs) {
                if (panel != spec) {
                    firstOtherPanel = panel;
                    break;
                }
            }
            firstOtherPanel;
        });
    }
    [[spec getTopToolbar] removeFromSuperview];
    [self.specs removeObject:spec];
}

- (void)setCurrentSpec:(id<IPanelSpec>)currentSpec {
    
    if ((currentSpec && ![self.specs containsObject:currentSpec]) || _currentSpec == currentSpec) {
        return;
    }
    
    if ([self.specs containsObject:_currentSpec]) {
        [[_currentSpec getTopToolbar] setHidden:YES];
        [self.currentSpec onDeactivated];
    }
    
    _currentSpec = currentSpec;
    [[_currentSpec getTopToolbar] setHidden:NO];
    if (!_panelController.isHidden) {
        [_currentSpec onActivated];
    }
    
    [self.specs enumerateObjectsUsingBlock:^(id<IPanelSpec>  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if (obj == currentSpec) {
            [_tabPagerBar scrollToItemAtIndex:idx animated:NO];
            [_pagerContentView scrollToViewAtIndex:idx animate:YES];
            *stop = YES;
        }
    }];
}

- (void)setCurrentSpecWithIndex:(NSUInteger)index{
    id<IPanelSpec>spec = self.specs[index];
    [self setCurrentSpec:spec];
}

- (UIView *)contentView {
    return _contentView;
}

#pragma mark -FSPagerViewDataSource
- (NSInteger)numberOfViewsInPagerView:(FSPagerView *)pagerView;{
    return self.specs.count;
}

- (UIView *)pagerView:(FSPagerView *)pagerView viewForIndex:(NSInteger)index{
    id<IPanelSpec>spec = self.specs[index];
    if ([spec isKindOfClass:[FSBasePanel class]]) {
        FSBasePanel *panel = (FSBasePanel *)spec;
        UIEdgeInsets layoutMargins = pagerView.safeAreaInsets;
        panel.contentViewCtr.needMakeUpSectionHeaderInset = YES;
        panel.contentViewCtr.editBar.subviewContentViewLayoutMargins = layoutMargins;
        panel.contentViewCtr.bottomBar.subviewContentViewLayoutMargins = layoutMargins;
        layoutMargins.top = 0;
        layoutMargins.bottom = 0;
        panel.contentViewCtr.subviewTableViewLayoutMargins = layoutMargins;
    }
    return [spec getContentView];
}

#pragma mark -FSPagerViewDelegate

- (void)pagerView:(FSPagerView *)pagerView transitionFromIndex:(NSInteger)fromIndex toIndex:(NSInteger)toIndex progress:(CGFloat)progress{
    [_tabPagerBar scrollToItemFromIndex:fromIndex toIndex:toIndex progress:progress];
    if (progress == 1) [self setCurrentSpecWithIndex:toIndex];
}

#pragma mark -FSTabPagerBarDelegate

- (NSInteger)numberOfItemsInPagerTabBar:(FSTabPagerBar *)pagerTabBar{
    return self.specs.count;
}

- (__kindof UIView *)pagerTabBar:(FSTabPagerBar *)pagerTabBar cellForItemAtIndex:(NSInteger)index{
    if (!self.specs || self.specs.count < index + 1) {
        return nil;
    }
    id<IPanelSpec>spec = self.specs[index];
    
    FSItem *tabItam = [FSItem createItemWithImage:[spec getTabItem].image];
    if ([spec isKindOfClass:[FSBasePanel class]]) {
        FSBasePanel *panel = (FSBasePanel *)spec;
        tabItam.accessibilityLabel = panel.title;
        tabItam.accessibilityIdentifier = [NSString stringWithFormat:@"segement_%@_id",panel.title];
    }
    [BindThemeKeySel(tabItam, setImage:forState:)
     fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[spec getTabItem].selectImage forState:UIControlStateSelected];
    }];
    return tabItam;
}

#pragma mark -FSTabPagerBarDelegate

- (CGFloat)pagerTabBar:(FSTabPagerBar *)pagerTabBar widthForItemAtIndex:(NSInteger)index{
    return pagerTabBar.fs_width / self.specs.count;
}

- (void)pagerTabBar:(FSTabPagerBar *)pagerTabBar didSelectItemAtIndex:(NSInteger)index{
    [self setCurrentSpecWithIndex:index];
}

- (void)pagerTabBar:(FSTabPagerBar *)pagerTabBar transitionFromItem:(__kindof UIView *)fromItem toItem:(__kindof UIView *)toItem animated:(BOOL)animated{}

@end

@implementation TabItem @end

