/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PasswordModule.h"
#import "EncryptOptionViewController.h"
#import "MoreView.h"
#import "FSSettingBar+private.h"
#import "MenuView+Private.h"

@implementation PasswordSecurityInfo

- (id)initWithOpenPassword:(NSString *)openPassword ownerPassword:(NSString *)ownerPassword print:(BOOL)print printHigh:(BOOL)printHigh fillForm:(BOOL)fillForm addAnnot:(BOOL)addAnnot assemble:(BOOL)assemble modify:(BOOL)modify copyForAccess:(BOOL)copyForAccesst copy:(BOOL)copy {
    if (self = [super init]) {
        self.openPassword = openPassword;
        self.ownerPassword = ownerPassword;
        self.allowPrint = print;
        self.allowPrintHigh = printHigh;
        self.allowFillForm = fillForm;
        self.allowAddAnnot = addAnnot;
        self.allowAssemble = assemble;
        self.allowModify = modify;
        self.allowCopy = copy;
        self.allowCopyForAccess = copyForAccesst;
    }
    return self;
}

@end

typedef void (^PasswordCallBack)(BOOL isInputed, NSString *password);

@interface PasswordModule () <MoreItemActionProtocol, IDocEventListener>
@property (nonatomic, copy) PasswordCallBack passwordCallback;
@property (nonatomic, assign) FSPDFDocPasswordType passwordType;
@property (nonatomic, strong) NSObject *currentVC;
@property (nonatomic, strong) UIAlertAction *passwordOkAction;
@property (nonatomic, strong) MoreMenuItem *pwdItem;
@property (nonatomic, strong) FSReadToolbarItem *passwdToolbarItem;
@end

@implementation PasswordModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return Module_Password;
}

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;

        self.passwordCallback = nil;
        self.inputPassword = nil;
        self.passwordType = FSPDFDocPwdInvalid;

        if (_extensionsManager.config.loadEncryption) {
            MoreMenuGroup *group = [_extensionsManager.more getGroup:TAG_GROUP_PROTECT];
            if (!group) {
                group = [[MoreMenuGroup alloc] init];
                group.tag = TAG_GROUP_PROTECT;

                group.title = FSLocalizedForKey(@"kProtect");
                [_extensionsManager.more addGroup:group];
            }
            self.pwdItem = [[MoreMenuItem alloc] init];
            self.pwdItem.tag = TAG_ITEM_PASSWORD;
            self.pwdItem.title = FSLocalizedForKey(@"kEncryption");
            self.pwdItem.callBack = self;
            self.pwdItem.image = FSImageNamedAndNormalColor(@"more_password_file_encryption");
            [_extensionsManager.more addMenuItem:TAG_GROUP_PROTECT withItem:self.pwdItem];
            
            _passwdToolbarItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypePassWord withItemContentType:FSItemContentTypeImageAndText];
            [_extensionsManager.toolbarManager addReadItem:_passwdToolbarItem atToolTag:FSMainTopbarToolTagProtect];
        }

        [_pdfViewCtrl registerDocEventListener:self];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getInputPassword:) name:@"NSNotificationNameInputPassword" object:nil];
    }
    return self;
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [self onClick:self.pwdItem];
}

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)getInputPassword:(NSNotification *)not{
    self.inputPassword = not.object;
}

- (void)getPasswordWithTitle:(NSString *)msgStr callback:(PasswordCallBack)callback{

    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:msgStr preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [[NSNotificationCenter defaultCenter] removeObserver:self];
        self.inputPassword = nil;
        callback(NO,nil);
        
    }];
    [alertController addAction:cancelAction];
    
    self.passwordOkAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [[NSNotificationCenter defaultCenter] removeObserver:self];
        NSString *passwordStr = alertController.textFields.firstObject.text;
        self.inputPassword = passwordStr;
        callback(YES,passwordStr);
    }];
    [alertController addAction:self.passwordOkAction];
    self.passwordOkAction.enabled = NO;
    
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        
        textField.text = @"";
        textField.clearButtonMode = UITextFieldViewModeWhileEditing;
        textField.secureTextEntry = YES;
        [textField addTarget:self action:@selector(fs_textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
    }];

    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
}

- (void)fs_textFieldDidChange:(UITextField *)textField{
    if (self.passwordOkAction) {
        self.passwordOkAction.enabled = textField.text.length;
    }
}

- (void)tryLoadPDFDocument:(FSPDFDoc *)document guessPassword:(NSString *)guessPassword success:(void (^)(NSString *password))success error:(void (^)(NSString *description))error abort:(void (^)(void))abort {
    self.pdfDoc = document;
    __weak typeof(self) weakSelf = self;
    FSErrorCode status = [document load:guessPassword];
    if (status == FSErrSuccess) {
        if (success) {
            success(guessPassword);
        }
    } else if (status == FSErrPassword) {
        NSString *title = guessPassword ? FSLocalizedForKey(@"kDocPasswordError") : FSLocalizedForKey(@"kInputOwnerPassword");
        [self getPasswordWithTitle:title
                                callback:^(BOOL isInputed, NSString *newPassword) {
                                    if (isInputed) {
                                        [weakSelf tryLoadPDFDocument:document guessPassword:newPassword success:success error:error abort:abort];
                                    } else if (abort) {
                                        abort();
                                    }
                                }];
    } else {
        if (error) {
            error([Utility getErrorCodeDescription:status]);
        }
    }
}

#pragma mark - MoreMenuItemAction

- (void)onClick:(MoreMenuItem *)item {
    if ([_extensionsManager.permissionProvider checkPermission:FSFunctionPDFEsign] == FSPermissionStateNo) return;
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionStandEncrypt checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            if ([self->_pdfViewCtrl isDynamicXFA]) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kXFAUnsupportedOP") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                [alertController addAction:action];
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                return;
            }
            if ([self->_pdfViewCtrl isRMSProtected]) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSAlreadyEncrypt") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                [alertController addAction:action];
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                return;
            }
            
            item.enable = NO;
            
            if (self.pdfDoc.isProtected) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kHadProtect") message:nil preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                [alertController addAction:action];
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                self.currentVC = alertController;
                
                item.enable = YES;
                return;
            }
            
            if ([item.title isEqualToString:FSLocalizedForKey(@"kEncryption")]) {
                [self handleEncryptDocument];
                [self->_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreProtectFileEncryption];
            } else {
                [self handleRemoveEncryption];
            }
            item.enable = YES;
        }
    }];
}

- (BOOL)isOwner {
    if (self.passwordType == FSPDFDocPwdInvalid) {
        [self updatePasswordType];
    }
    if (self.passwordType == FSPDFDocPwdNoPassword ||
        self.passwordType == FSPDFDocPwdOwner) {
        return YES;
    }
    return NO;
}

-(void)exitCropMode{
    [_pdfViewCtrl setCropMode:PDF_CROP_MODE_NONE];
}

- (BOOL)encryptDocument:(FSPDFDoc *)doc {
    NSString *originalPDFPath = _pdfViewCtrl.filePath;
    NSString *encryptedPDFPath = nil;

    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL isOK = NO;

    @autoreleasepool {
        FSStdSecurityHandler *stdSecurityHandler = [[FSStdSecurityHandler alloc] init];
        FSStdEncryptData *data = [[FSStdEncryptData alloc] init];
        data.cipher = FSSecurityHandlerCipherAES;
        data.key_length = 16;
        data.is_encrypt_metadata = YES;
        data.user_permissions = [self permissionsFromInfo];
        isOK = [stdSecurityHandler initialize:data user_password:self.securityInfo.openPassword owner_password:self.securityInfo.ownerPassword];
        if (isOK) {
            isOK = [doc setSecurityHandler:stdSecurityHandler];
        }

        if (isOK) {
            NSString *fileName = [originalPDFPath lastPathComponent] ?: @".pdf";
            encryptedPDFPath = [NSTemporaryDirectory() stringByAppendingString:[NSString stringWithFormat:@"_encryped_%@", fileName]];

            if ([fileManager fileExistsAtPath:encryptedPDFPath]) {
                [fileManager removeItemAtPath:encryptedPDFPath error:nil];
            }
            isOK = [doc saveAs:encryptedPDFPath save_flags:FSPDFDocSaveFlagNormal];
        }
        if (isOK) {
            isOK = [fileManager removeItemAtPath:originalPDFPath error:nil];
        }
        if (isOK) {
            isOK = [fileManager moveItemAtPath:encryptedPDFPath toPath:originalPDFPath error:nil];
        }
        if (isOK) {
            [self exitCropMode];
            NSString *password = self.securityInfo.ownerPassword?:self.securityInfo.openPassword;
            [_extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self, NSString *password) {
                @autoreleasepool {
                    [self.pdfViewCtrl closeDoc:nil];
                    self.pdfViewCtrl.currentDoc = nil;
                }
                [self.pdfViewCtrl openDoc:originalPDFPath password:password completion:nil];
            } obj:password];
        }
    }
    return isOK;
}

- (BOOL)removeEncrypt:(FSPDFDoc *)pdfDoc {
    BOOL isOK = NO;
    NSString *decryptedPDFPath = nil;
    NSFileManager *fileManager = [NSFileManager defaultManager];

    @autoreleasepool {
        isOK = [pdfDoc removeSecurity];
        if (isOK) {
            NSString *fileName = [_pdfViewCtrl.filePath lastPathComponent];
            decryptedPDFPath = [NSTemporaryDirectory() stringByAppendingString:[NSString stringWithFormat:@"_decryped_%@", fileName]];
            if ([fileManager fileExistsAtPath:decryptedPDFPath isDirectory:nil]) {
                [fileManager removeItemAtPath:decryptedPDFPath error:nil];
            }
            isOK = [pdfDoc saveAs:decryptedPDFPath save_flags:FSPDFDocSaveFlagNormal];
        }
        if (isOK) {
            isOK = [fileManager removeItemAtPath:_pdfViewCtrl.filePath error:nil];

            if (isOK) {
                isOK = [fileManager moveItemAtPath:decryptedPDFPath toPath:_pdfViewCtrl.filePath error:nil];
            }
        }
        if (isOK) {
            [self exitCropMode];
            NSString *filePath = _pdfViewCtrl.filePath;
            /* close Document first.*/
            
            void(^SupportsMultipleScenesBlock)(UIExtensionsManager *) = ^(UIExtensionsManager *_ext){
                @autoreleasepool {
                    [_ext.pdfViewCtrl closeDoc:nil];
                    _ext.pdfViewCtrl.currentDoc = nil;
                }
                
                [_ext.pdfViewCtrl openDoc:filePath password:nil completion:nil];
            };
            
            NSArray *otherScenes = [_extensionsManager someDocFromOtherScenes];
            SupportsMultipleScenesBlock(_extensionsManager);
            for (UIExtensionsManager *ext in otherScenes) {
                SupportsMultipleScenesBlock(ext);
            }
        }
    }

    return isOK;
}

#pragma mark - IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess)
        return;
    self.pdfDoc = document;

    [self updatePasswordType];

    NSString *text = FSLocalizedForKey(@"kEncryption");
    UIImage *image = FSImageNamedAndNormalColor(@"more_password_file_encryption");
    NSString *imageName = @"more_password_file_encryption";
    if (self.passwordType == FSPDFDocPwdUser ||
        self.passwordType == FSPDFDocPwdOwner) {
        text = FSLocalizedForKey(@"kDecryption");
        image = FSImageNamedAndNormalColor(@"more_password_remove_password");
        imageName = @"more_password_remove_password";
    }
    [self updateItemText:text image:image imageName:imageName];
    
    if ([self->_pdfViewCtrl isDynamicXFA] || [self->_pdfViewCtrl isRMSProtected] || !_extensionsManager.enableModifyDocument || document.isProtected) {
        self.pwdItem.enable = NO;
    }else{
        self.pwdItem.enable = YES;
    }
    [self updateReadItemState:self.pwdItem.enable];
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    if (self.currentVC) {
        if ([self.currentVC isKindOfClass:[UIAlertController class]]) {
            [(UIAlertController *) self.currentVC dismissViewControllerAnimated:NO completion:nil];
        } else if ([self.currentVC isKindOfClass:[EncryptOptionViewController class]]) {
            if ([((EncryptOptionViewController *) self.currentVC).currentVC isKindOfClass:[UIAlertController class]]) {
                UIAlertController *alert = (UIAlertController *) ((EncryptOptionViewController *) self.currentVC).currentVC;
                [alert dismissViewControllerAnimated:NO completion:nil];
            }
            ((EncryptOptionViewController *) self.currentVC).currentVC = nil;
            [(EncryptOptionViewController *) self.currentVC dismissViewControllerAnimated:NO completion:nil];
        }
        self.currentVC = nil;
    }
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    self.pdfDoc = nil;
    self.passwordType = FSPDFDocPwdInvalid;
}

#pragma mark - encrypt document

- (void)handleEncryptDocument {
    EncryptOptionViewController *encryptOptionCtrl = [[EncryptOptionViewController alloc] initWithNibName:@"EncryptOptionViewController" bundle:[NSBundle bundleForClass:[self class]]];
    self.currentVC = encryptOptionCtrl;

    __weak typeof(self) weakSelf = self;
    NSString *path = _pdfViewCtrl.filePath;

    @weakify(self)
    encryptOptionCtrl.optionHandler = ^(EncryptOptionViewController *ctrl, BOOL isCancel, NSString *openPassword, NSString *ownerPassword, BOOL print, BOOL printHigh, BOOL fillForm, BOOL addAnnot, BOOL assemble, BOOL modify, BOOL copyForAccess, BOOL copy) {
        @strongify(self)
        if (isCancel)
            return;
        if (openPassword == nil && ownerPassword == nil) {
            return;
        }

        self.securityInfo = [[PasswordSecurityInfo alloc] initWithOpenPassword:openPassword ownerPassword:ownerPassword print:print printHigh:printHigh fillForm:fillForm addAnnot:addAnnot assemble:assemble modify:modify copyForAccess:copyForAccess copy:copy];

        if (self->_extensionsManager.isDocModified) { // whether to save before encryption?
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kShouldSaveEncryptionDocument") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo")
                                                                   style:UIAlertActionStyleCancel
                                                                 handler:^(UIAlertAction *action) {
                                                                     FSPDFDoc *pdfDoc = [[FSPDFDoc alloc] initWithPath:path];
                                                                     FSErrorCode error = [pdfDoc load:weakSelf.inputPassword];
                                                                     if (error == FSErrSuccess) {
                                                                         weakSelf.pdfDoc = pdfDoc;
                                                                     } else {
                                                                         //todo
                                                                     }
                                                                     [weakSelf encryptDocument:weakSelf.pdfDoc];

                                                                 }];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                             style:UIAlertActionStyleDefault
                                                           handler:^(UIAlertAction *action) {
                                                                self->_extensionsManager.currentAnnot = nil;
                                                                self->_extensionsManager.currentToolHandler = nil;
                                                               [self encryptDocument:self.pdfDoc];

                                                           }];
            [alertController addAction:cancelAction];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            self.currentVC = alertController;
        } else {
            [self encryptDocument:openPassword ownerPassword:ownerPassword print:print printHigh:printHigh fillForm:fillForm addAnnot:addAnnot modify:modify assemble:assemble copyForAccess:copyForAccess copy:copy];
        }
    };

    // show permission setting option
    dispatch_async(dispatch_get_main_queue(), ^{
        FSNavigationController *encryptOptionNavCtrl = [[FSNavigationController alloc] initWithRootViewController:encryptOptionCtrl];
        encryptOptionNavCtrl.delegate = encryptOptionCtrl;
        if (DEVICE_iPHONE) {
            encryptOptionNavCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
        } else {
            encryptOptionNavCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
        }

        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:encryptOptionNavCtrl animated:YES completion:nil];
    });
}

- (void)encryptDocument:(NSString *)openPassword ownerPassword:(NSString *)ownerPassword print:(BOOL)print printHigh:(BOOL)printHigh fillForm:(BOOL)fillForm addAnnot:(BOOL)addAnnot modify:(BOOL)modify assemble:(BOOL)assemble copyForAccess:(BOOL)copyForAccess copy:(BOOL)copy {
    _extensionsManager.currentAnnot = nil;
    _extensionsManager.currentToolHandler = nil;
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{

        dispatch_async(dispatch_get_main_queue(), ^{
            BOOL isOK = [self encryptDocument:self.pdfDoc];
            if (!isOK) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kTryAgain") message:FSLocalizedForKey(@"kEncryptAddFail") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo") style:UIAlertActionStyleCancel handler:nil];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                                 style:UIAlertActionStyleDefault
                                                               handler:^(UIAlertAction *action) {
                                                                   [self encryptDocument:openPassword ownerPassword:ownerPassword print:print printHigh:printHigh fillForm:fillForm addAnnot:addAnnot modify:modify assemble:assemble copyForAccess:copyForAccess copy:copy];
                                                               }];
                [alertController addAction:cancelAction];
                [alertController addAction:action];
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                self.currentVC = alertController;
            } else {
                [self updatePasswordType];
                UIImage *image = FSImageNamedAndNormalColor(@"more_password_file_encryption");
                [self updateItemText:FSLocalizedForKey(@"kDecryption") image:image imageName:@"more_password_file_encryption"];

//                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kEncryptAddSuccess") preferredStyle:UIAlertControllerStyleAlert];
//                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
//                [alertController addAction:action];
//                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
//                self.currentVC = alertController;

                // if this file is encrypted by open doc password,
                // clear thumbnail cache. other cache is remained as user can still open it and see.
                if (openPassword != nil) {
                }
            }
        });
    });
}

- (void)handleRemoveEncryption {
    if (![self isEncrypted]) {
        return;
    }
    __weak typeof(self) weakSelf = self;
    typedef void (^SuccessCallback)(void);
    __block void (^promptForOwnerPassword)(NSString *title, SuccessCallback success) = ^(NSString *title, SuccessCallback success) {
        [weakSelf getPasswordWithTitle:title
                                    callback:^(BOOL isInputed, NSString *password) {
                                        if (isInputed) {
                                            BOOL isOwner = ([weakSelf.pdfDoc getPasswordType] == FSPDFDocPwdNoPassword) || ([weakSelf.pdfDoc checkPassword:password] == FSPDFDocPwdOwner);
                                            if (!isOwner) {
                                                promptForOwnerPassword(FSLocalizedForKey(@"kDocPasswordError"), success);
                                            } else if (success) {
                                                promptForOwnerPassword = nil; // fix strong-reference-cycle
                                                success();
                                            }
                                        } else {
                                            promptForOwnerPassword = nil; // fix strong-reference-cycle
                                        }
                                    }];
    };
    void (^tryRemoveEncryption)(void) = ^{
        if ([weakSelf isOwner]) {
            [weakSelf removeEncryption];
        } else {
            promptForOwnerPassword(FSLocalizedForKey(@"kInputOwnerPassword"), ^{
                [weakSelf removeEncryption];
            });
        }
    };
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kEncryptRemovePass") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo")
                                                           style:UIAlertActionStyleCancel
                                                         handler:^(UIAlertAction *action) {
                                                             self.currentVC = nil;
                                                         }];
    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                     style:UIAlertActionStyleDefault
                                                   handler:^(UIAlertAction *action) {
                                                       if (self->_extensionsManager.isDocModified) { // whether to save before remove encryption?
                                                           UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kShouldSaveEecryptionDocument") preferredStyle:UIAlertControllerStyleAlert];
                                                           UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo")
                                                                                                                  style:UIAlertActionStyleCancel
                                                                                                                handler:^(UIAlertAction *action) {
                                                                                                                    FSPDFDoc *pdfDoc = [[FSPDFDoc alloc] initWithPath:self->_pdfViewCtrl.filePath];
                                                                                                                    FSErrorCode error = [pdfDoc load:self.inputPassword];
                                                                                                                    if (error == FSErrSuccess) {
                                                                                                                        weakSelf.pdfDoc = pdfDoc;
                                                                                                                    } else {
                                                                                                                        //todo
                                                                                                                    }
                                                                                                                    self.currentVC = nil;
                                                                                                                    tryRemoveEncryption();
                                                                                                                    
                                                                                                                }];
                                                           UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                                                                            style:UIAlertActionStyleDefault
                                                                                                          handler:^(UIAlertAction *action) {
                                                                                                              self.currentVC = nil;
                                                                                                              tryRemoveEncryption();
                                                                                                          }];
                                                           [alertController addAction:cancelAction];
                                                           [alertController addAction:action];
                                                           [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                                                           self.currentVC = alertController;
                                                       }else{
                                                           tryRemoveEncryption();
                                                           self.currentVC = nil;
                                                       }
                                                   }];
    [alertController addAction:cancelAction];
    [alertController addAction:action];
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    self.currentVC = alertController;
}

- (void)removeEncryption {
    _extensionsManager.currentAnnot = nil;
    _extensionsManager.currentToolHandler = nil;

    BOOL isOK = [self removeEncrypt:self.pdfDoc];
    if (!isOK) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kTryAgain") message:FSLocalizedForKey(@"kEncryptAddFail") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo")
                                                               style:UIAlertActionStyleCancel
                                                             handler:^(UIAlertAction *action) {
                                                                 self.currentVC = nil;
                                                             }];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                         style:UIAlertActionStyleDefault
                                                       handler:^(UIAlertAction *action) {
                                                           [self removeEncryption];
                                                           self.currentVC = nil;
                                                       }];
        [alertController addAction:cancelAction];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        self.currentVC = alertController;
    } else {
        UIImage *image = FSImageNamedAndNormalColor(@"more_password_file_encryption");
        [self updateItemText:FSLocalizedForKey(@"kEncryption") image:image imageName:@"more_password_file_encryption"];
        
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kEncryptRemoveSuccess") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        self.currentVC = alertController;
        self.securityInfo = nil;
        [_extensionsManager.settingBar updateLayoutButtonsWithLayout:[_pdfViewCtrl getPageLayoutMode]];
    }
}

#pragma mark - private method

- (BOOL)isEncrypted {
    return [self.pdfDoc getEncryptionType] == FSPDFDocEncryptPassword;
}

- (void)updatePasswordType {
    self.passwordType = [self.pdfDoc getPasswordType];
}

- (void)updateItemText:(NSString *)text image:(UIImage*)image imageName:(NSString *)imageName {
    MoreMenuGroup *securityGroup = [_extensionsManager.more getGroup:TAG_GROUP_PROTECT];
    for (MoreMenuItem *item in [securityGroup getItems]) {
        if (item.tag == TAG_ITEM_PASSWORD) {
            item.title = text;
            item.image = image;
            break;
        }
    }
    [_extensionsManager.more reloadData];
    if (self.passwdToolbarItem) {
        self.passwdToolbarItem.title = text;
        self.passwdToolbarItem.imageName = imageName;
        self.passwdToolbarItem.image = image;
        [self.passwdToolbarItem.toolbarItem.item setImage:image forState:UIControlStateNormal];
    }
    NSArray<FSMainTopbarToolTagItem *> *toolTagItemArray = _extensionsManager.topToolbar.tagItems;
    for (int i = 0; i < toolTagItemArray.count; i++) {
        FSMainTopbarToolTagItem *item = toolTagItemArray[i];
        NSArray<FSReadToolbarItem *> *readtoolItems = item.readToolbarItems;
        if (item.toolTag == FSMainTopbarToolTagHome) {
            for (FSReadToolbarItem *toolbarItem in readtoolItems) {
                if (toolbarItem.settingsType == FSReadToolSettingsTypePassWord) {
                    toolbarItem.title = text;
                    toolbarItem.imageName = imageName;
                    toolbarItem.image = image;
                    [toolbarItem.toolbarItem.item setImage:image forState:UIControlStateNormal];
                }
            }
        }
    }
}

- (void)updateReadItemState:(BOOL)enable
{
    if (self.passwdToolbarItem) {
        self.passwdToolbarItem.enabled = enable;
    }
    NSArray<FSMainTopbarToolTagItem *> *toolTagItemArray = _extensionsManager.topToolbar.tagItems;
    for (int i = 0; i < toolTagItemArray.count; i++) {
        FSMainTopbarToolTagItem *item = toolTagItemArray[i];
        NSArray<FSReadToolbarItem *> *readtoolItems = item.readToolbarItems;
        if (item.toolTag == FSMainTopbarToolTagHome) {
            for (FSReadToolbarItem *toolbarItem in readtoolItems) {
                if (toolbarItem.settingsType == FSReadToolSettingsTypePassWord) {
                    toolbarItem.enabled = enable;
                }
            }
        }
    }
}

- (int)permissionsFromInfo {
    unsigned int permission = 0x0;
    if (self.securityInfo.allowPrint)
        permission |= FSPDFDocPermPrint;
    if (self.securityInfo.allowPrintHigh)
        permission |= FSPDFDocPermPrint | FSPDFDocPermPrintHigh;
    if (self.securityInfo.allowFillForm)
        permission |= FSPDFDocPermFillForm;
    if (self.securityInfo.allowAddAnnot)
        permission |= FSPDFDocPermAnnotForm | FSPDFDocPermFillForm;
    if (self.securityInfo.allowAssemble)
        permission |= FSPDFDocPermAssemble;
    if (self.securityInfo.allowModify)
        permission |= FSPDFDocPermModify | FSPDFDocPermAssemble | FSPDFDocPermFillForm;
    if (self.securityInfo.allowCopyForAccess)
        permission |= FSPDFDocPermExtractAccess;
    if (self.securityInfo.allowCopy)
        permission |= FSPDFDocPermExtract | FSPDFDocPermExtractAccess;
    return permission;
}

@end

