/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "EraseToolHandler.h"
#import "ReplyTableViewController.h"
#import "ReplyUtil.h"
#import "EraseUtil.h"
#import "PaintUtility.h"
#import <CoreImage/CIFilter.h>

@interface EraseToolHandler ()
@property (nonatomic) int radius;
@property (nonatomic) int changedPointCount;
@property (nonatomic, strong) NSMutableArray *pencilAnnotationArray;
- (void)resetInk:(FSInk *)inkAnnot arrayLines:(NSMutableArray<NSMutableArray<FSPointF *> *> *)arrayLines;
- (void)isBeginUpdateInkAnnot:(FSInk *)inkAnnot;
- (void)reloadCurlayer;
- (void)modifyInk:(FSAnnot *)annot;
- (void)addInk:(FSAnnot *)annot;
- (void)removeInk:(FSAnnot *)annot;
@end

@interface EraseCAShapeObject : NSObject
@property (nonatomic, strong) FSInk *ink;
@property (nonatomic, weak) CAShapeLayer *layer;
@property (nonatomic, strong) NSMutableArray<NSMutableArray<NSValue *> *> *pathPtFs;
@property (nonatomic) BOOL isChanged;
@end

@implementation EraseCAShapeObject

@end

@interface EraseCAShapeView : UIView
@property (nonatomic) int pageIndex;
@property (nonatomic, weak) FSPDFViewCtrl *pdfView;
@property (nonatomic, weak) EraseToolHandler *eraseToolHandler;
@property (nonatomic, strong) NSMutableDictionary <NSString *, EraseCAShapeObject *>*inkForLayer;

- (NSMutableArray<NSValue *> *)updatePencil:(CGPoint)point eraseStyle:(FSEraseStyle)eraseStyle;
@end

@implementation EraseCAShapeView

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self){
        self.inkForLayer = @{}.mutableCopy;
        self.userInteractionEnabled = NO;
    }
    return self;
}

- (void)resetInkPath{
    for (NSString *key in self.inkForLayer.allKeys) {
        EraseCAShapeObject *r = self.inkForLayer[key];
        if (r.isChanged){
            NSMutableArray<NSMutableArray<FSPointF *> *> *arrayLines = @[].mutableCopy;
            for (NSMutableArray<NSValue *> *pathPt in r.pathPtFs){
                NSMutableArray<FSPointF *> *path = @[].mutableCopy;
                for (NSValue *ptValue in pathPt){
                    [path addObject:[self.pdfView convertPageViewPtToPdfPt:[ptValue CGPointValue] pageIndex:r.ink.pageIndex]];
                }
                [arrayLines addObject:path];
            }
            [self.eraseToolHandler resetInk:r.ink arrayLines:arrayLines];
        }
    }
}

- (UIBezierPath *)drawBezierPath:(UIBezierPath *)path line:(NSMutableArray<NSValue *> *)line{
    NSInteger ptnum = line.count;
    CGPoint vertice[4];
    CGPoint startPoint = [line[0] CGPointValue];
    
    [path moveToPoint:startPoint];
    if(ptnum == 1){
        [path addLineToPoint:startPoint];
        return path;
    }
    
    BOOL isClosed = CGPointEqualToPoint([line[0] CGPointValue], [line[ptnum - 1] CGPointValue]);

    vertice[0] = isClosed ? [line[ptnum - 2] CGPointValue] : [line[0] CGPointValue];

    
   for (NSInteger j = 0; j < ptnum - 1; j++) {
       vertice[1] = [line[j] CGPointValue];
       vertice[2] = [line[j + 1] CGPointValue];

       if (j == ptnum - 2) {
           vertice[3] = isClosed ? [line[1] CGPointValue] : vertice[2];
       } else {
           vertice[3] = [line[j + 2] CGPointValue];
       }
       NSArray *cp = [PaintUtility calcBesselControlPoints:vertice rate:0.8f];
       [path addCurveToPoint:vertice[2] controlPoint1:[cp[0] CGPointValue] controlPoint2:[cp[1] CGPointValue]];
       vertice[0] = vertice[1];
       startPoint = vertice[2];
   }
    return path;
}

- (void)convertToCAShapeLayer{
    for (FSInk *ink in self.eraseToolHandler.pencilAnnotationArray.copy) {
        [self addInk:ink];
    }
}

- (void)setFrame:(CGRect)frame{
    CGRect curFrame = self.frame;
    [super setFrame:frame];
    if (CGRectEqualToRect(curFrame, frame)) return;
    [self reloadCurlayer];
}

- (void)addInk:(FSAnnot *)annot{
    if ([annot isEmpty]) return;
    FSInk *ink = [[FSInk alloc] initWithAnnot:annot];
    EraseCAShapeObject *record = self.inkForLayer[ink.NM];
    if (!record){
        NSMutableArray<NSMutableArray<FSPointF *> *> * _pathPtFs = [EraseUtil getPointLinesFromInkAnnot:ink];
        
        CAShapeLayer *curlayer = [[CAShapeLayer alloc] init];
        curlayer.zPosition = 1;
        curlayer.lineCap = @"round";
        curlayer.lineJoin = @"round";
        curlayer.backgroundColor = [UIColor clearColor].CGColor;
        curlayer.fillColor = [UIColor clearColor].CGColor;
        curlayer.lineWidth = [Utility convertWidth:ink.lineWidth fromPDFToPageView:self.pdfView pageIndex:self.pageIndex];
        curlayer.strokeColor = [UIColor colorWithRGB:ink.color alpha:ink.opacity].CGColor;
        
        UIBezierPath *path = [UIBezierPath bezierPath];
        NSMutableArray<NSMutableArray<NSValue *> *> * pathCgPtFs = @[].mutableCopy;
        
        for (int i = 0; i < _pathPtFs.count; i ++){
            NSMutableArray<FSPointF *> *ptFs = _pathPtFs[i];
            if (!ptFs.count) continue;
            NSMutableArray<NSValue *> * line = @[].mutableCopy;
            for (int j = 0; j < ptFs.count; j ++){
                FSPointF * ptF = ptFs[j];
                CGPoint pt = [self.pdfView convertPdfPtToPageViewPt:ptF pageIndex:self.pageIndex];
                [line addObject:[NSValue valueWithCGPoint:pt]];
            }
            path = [self drawBezierPath:path line:line];
            [pathCgPtFs addObject:line];
        }

        curlayer.path = path.CGPath;
        [self.layer addSublayer:curlayer];
        
        record = [[EraseCAShapeObject alloc] init];
        record.layer = curlayer;
        record.ink = ink;
        record.pathPtFs = pathCgPtFs;
        self.inkForLayer[ink.NM] = record;
    }
}


- (void)removeInk:(FSAnnot *)annot{
    if ([annot isEmpty]) return;
    FSInk *ink = [[FSInk alloc] initWithAnnot:annot];
    EraseCAShapeObject *record = self.inkForLayer[ink.NM];
    CAShapeLayer *curlayer = record.layer;
    [curlayer removeFromSuperlayer];
    [self.inkForLayer removeObjectForKey:ink.NM];
}

- (void)modifyInk:(FSAnnot *)annot{
    if ([annot isEmpty]) return;
    FSInk *ink = [[FSInk alloc] initWithAnnot:annot];
    EraseCAShapeObject *record = self.inkForLayer[ink.NM];
    if (record){
        NSMutableArray<NSMutableArray<FSPointF *> *> * _pathPtFs = [EraseUtil getPointLinesFromInkAnnot:ink];
        CAShapeLayer *curlayer = record.layer;
        
        UIBezierPath *path = [UIBezierPath bezierPath];
        NSMutableArray<NSMutableArray<NSValue *> *> * pathCgPtFs = @[].mutableCopy;
        
        for (int i = 0; i < _pathPtFs.count; i ++){
            NSMutableArray<FSPointF *> *ptFs = _pathPtFs[i];
            if (!ptFs.count) continue;
            NSMutableArray<NSValue *> * line = @[].mutableCopy;
            for (int j = 0; j < ptFs.count; j ++){
                FSPointF * ptF = ptFs[j];
                CGPoint pt = [self.pdfView convertPdfPtToPageViewPt:ptF pageIndex:self.pageIndex];
                [line addObject:[NSValue valueWithCGPoint:pt]];
            }
            path = [self drawBezierPath:path line:line];
            [pathCgPtFs addObject:line];
        }
        curlayer.path = path.CGPath;
        record.pathPtFs = pathCgPtFs;
    }
}

- (void)reloadCurlayer{
    for (NSString *NM in self.inkForLayer.allKeys){
        EraseCAShapeObject *record = self.inkForLayer[NM];
        if (record){
            FSInk *ink = record.ink;
            if ([ink isEmpty]) {
                CAShapeLayer *curlayer = record.layer;
                [curlayer removeFromSuperlayer];
                continue;
            }
            [self modifyInk:ink];
        }
    }
}

- (NSMutableArray<NSValue *> *)updatePencil:(CGPoint)point eraseStyle:(FSEraseStyle)eraseStyle{
    NSMutableArray<NSValue *> *arrayAffectedPoints = [NSMutableArray<NSValue *> array];
    const int reachPointCount = 3;

    BOOL isChanged = NO;
    NSMutableArray<NSValue *> *allpoint = @[].mutableCopy;
    
    NSArray <EraseCAShapeObject *> *records = self.inkForLayer.allValues;
    if (eraseStyle == FSEraseStyleWhole) {
        for (EraseCAShapeObject *record in records){
            for (NSMutableArray<NSValue *> *lines in record.pathPtFs){
                [allpoint addObjectsFromArray:lines];
            }
        }
    }
    
    for (EraseCAShapeObject *record in records.copy){
        FSInk *inkAnnot = record.ink;
        [self.eraseToolHandler isBeginUpdateInkAnnot:inkAnnot];
        NSMutableArray<NSMutableArray<NSValue *> *> *arrayLines = record.pathPtFs;
        for (int i = 0; i < arrayLines.count; i++) {
            BOOL isLineChanged = NO;
            NSMutableArray<NSValue *> *pointArray = arrayLines[i];

            if (pointArray.count <= 1) {
                CGPoint onePoint = [pointArray[0] CGPointValue];
                if ([EraseUtil cg_isInsideCircle:onePoint circleCenterPoint:point radius:self.eraseToolHandler.radius]) {
                    isChanged = YES;
                    if (eraseStyle == FSEraseStylePartial) {
                        [arrayAffectedPoints addObject:pointArray[0]];
                        self.eraseToolHandler.changedPointCount++;
                        [arrayLines removeObjectAtIndex:i];
                        i--;
                    }else{
                        [arrayAffectedPoints addObjectsFromArray:allpoint];
                        self.eraseToolHandler.changedPointCount++;
                        [arrayLines removeAllObjects];
                    }
                    
                }
            } else {
                NSMutableArray *arrayNewLines = [NSMutableArray array];
                for (int j = 0; j < pointArray.count - 1; j++) {
                    @autoreleasepool {
                        CGPoint point1 = [pointArray[j] CGPointValue];
                        CGPoint point2 = [pointArray[j + 1] CGPointValue];

                        double u1, u2;
                        [EraseUtil cg_getU:point1 point2:point2 circleCenterPoint:point u1:&u1 u2:&u2 radius:self.eraseToolHandler.radius];

                        if ((u1 < 0 && u2 < 0) || (u1 > 1 && u2 > 1)) //no cross point, outside the circle
                        {
                        } else if ((u1 < 0 && u2 > 1) || (u2 < 0 && u1 > 1)) //no cross point, inside the circle
                        {
                            
                            if (eraseStyle == FSEraseStylePartial) {
                                isLineChanged = YES;
                                [arrayAffectedPoints addObject:pointArray[j]];
                                self.eraseToolHandler.changedPointCount++;
                                [pointArray removeObjectAtIndex:j];
                                j--;
                            }else{
                                isChanged = YES;
                                [arrayAffectedPoints addObjectsFromArray:allpoint];
                                self.eraseToolHandler.changedPointCount++;
                                [arrayLines removeAllObjects];
                                break;
                            }
                        } else if (((u1 > 0.01 && u1 < 0.99) && (u2 < 0.01 || u2 > 0.99)) || ((u2 > 0.01 && u2 < 0.99) && (u1 < 0.01 || u1 > 0.99))) //one cross point, not on circle
                        {
                            double u;
                            if (u1 > 0.01 && u1 < 0.99) {
                                u = u1;
                            } else {
                                u = u2;
                            }

                            CGPoint pointCross = {point1.x + u * (point2.x - point1.x), point1.y + u * (point2.y - point1.y)};

                            if (![EraseUtil cg_isInsideCircle:point1 circleCenterPoint:point radius:self.eraseToolHandler.radius]) //enter circle, point1 is out of circle, point2 is in circle
                            {
                                
                                if (eraseStyle == FSEraseStylePartial) {
                                    isLineChanged = YES;
                                    [arrayAffectedPoints addObjectsFromArray:[pointArray subarrayWithRange:NSMakeRange(MAX(0, j + 1 - reachPointCount), MIN(reachPointCount, j + 1))]];
                                    [arrayAffectedPoints addObject:[NSValue valueWithCGPoint:pointCross]];
                                    self.eraseToolHandler.changedPointCount++;
                                    NSMutableArray<NSValue *> *arrayNewPoints = [NSMutableArray<NSValue *> array];
                                    [arrayNewPoints addObjectsFromArray:[pointArray subarrayWithRange:NSMakeRange(0, j + 1)]];
                                    [arrayNewPoints addObject:[NSValue valueWithCGPoint:pointCross]];
                                    [arrayNewLines addObject:arrayNewPoints];
                                    [pointArray removeObjectsInRange:NSMakeRange(0, j + 1)];
                                    j = -1;
                                }else{
                                    isChanged = YES;
                                    [arrayAffectedPoints addObjectsFromArray:allpoint];
                                    self.eraseToolHandler.changedPointCount++;
                                    [arrayLines removeAllObjects];
                                    break;
                                }
                            } else //leave circle
                            {
                                if (j == 0) {
                                    
                                    if (eraseStyle == FSEraseStylePartial) {
                                        isLineChanged = YES;
                                        [arrayAffectedPoints addObject:pointArray[j]];
                                        [pointArray removeObjectAtIndex:j];
                                        [pointArray insertObject:[NSValue valueWithCGPoint:pointCross] atIndex:0];
                                        [arrayAffectedPoints addObjectsFromArray:[pointArray subarrayWithRange:NSMakeRange(0, MIN(reachPointCount, pointArray.count))]];
                                        self.eraseToolHandler.changedPointCount++;
                                    }else{
                                        isChanged = YES;
                                        [arrayAffectedPoints addObjectsFromArray:allpoint];
                                        self.eraseToolHandler.changedPointCount++;
                                        [arrayLines removeAllObjects];
                                        break;
                                    }
                                }
                            }
                        } else if ((u1 > 0.01 && u1 < 0.99) && (u2 > 0.01 && u2 < 0.99)) //two cross point
                        {
                            
                            if (eraseStyle == FSEraseStylePartial) {
                                isLineChanged = YES;
                                CGPoint pointCross1 = {point1.x + u1 * (point2.x - point1.x), point1.y + u1 * (point2.y - point1.y)};
                                CGPoint pointCross2 = {point1.x + u2 * (point2.x - point1.x), point1.y + u2 * (point2.y - point1.y)};
                                
                                double au1, au2;
                                [EraseUtil cg_getU:pointCross1 point2:point1 circleCenterPoint:point u1:&au1 u2:&au2 radius:self.eraseToolHandler.radius];
                                
                                CGPoint pointCrossStart;
                                CGPoint pointCrossEnd;
                                
                                if ((au1 >= 0 && au1 <= 1) && (au2 >= 0 && au2 <= 1)) {
                                    pointCrossStart = pointCross2;
                                    pointCrossEnd = pointCross1;
                                } else {
                                    pointCrossStart = pointCross1;
                                    pointCrossEnd = pointCross2;
                                }
                                
                                [arrayAffectedPoints addObjectsFromArray:[pointArray subarrayWithRange:NSMakeRange(MAX(0, j + 1 - reachPointCount), MIN(reachPointCount, j + 1))]];
                                [arrayAffectedPoints addObject:[NSValue valueWithCGPoint:pointCrossStart]];
                                NSMutableArray *arrayNewPoints = [NSMutableArray array];
                                [arrayNewPoints addObjectsFromArray:[pointArray subarrayWithRange:NSMakeRange(0, j + 1)]];
                                [arrayNewPoints addObject:[NSValue valueWithCGPoint:pointCrossStart]];
                                [arrayNewLines addObject:arrayNewPoints];
                                [pointArray removeObjectsInRange:NSMakeRange(0, j + 1)];
                                [pointArray insertObject:[NSValue valueWithCGPoint:pointCrossEnd] atIndex:0];
                                [arrayAffectedPoints addObjectsFromArray:[pointArray subarrayWithRange:NSMakeRange(0, MIN(reachPointCount, pointArray.count))]];
                                self.eraseToolHandler.changedPointCount++;
                                j = 1;
                            }else{
                                isChanged = YES;
                                [arrayAffectedPoints addObjectsFromArray:allpoint];
                                self.eraseToolHandler.changedPointCount++;
                                [arrayLines removeAllObjects];
                                break;
                            }
                            
                        } else //one cross point, line exactly on the circle
                        {
                        }
                    }
                }

                if (isLineChanged) {
                    isChanged = YES;

                    if (pointArray.count > 1) {
                        [arrayNewLines addObject:pointArray];
                    } else {
                        [arrayAffectedPoints addObjectsFromArray:pointArray];
                        self.eraseToolHandler.changedPointCount++;
                    }

                    [arrayLines removeObjectAtIndex:i];
                    i--;

                    if (arrayNewLines.count > 0) {
                        [arrayLines addObjectsFromArray:arrayNewLines];
                    }
                }
            }
        }

        if (isChanged) {
            record.isChanged = YES;
            CAShapeLayer *curlayer = record.layer;
            
            UIBezierPath *path = [UIBezierPath bezierPath];
            for (int i = 0; i < record.pathPtFs.count; i ++){
                NSMutableArray<NSValue *> *ptFs = record.pathPtFs[i];
                path = [self drawBezierPath:path line:ptFs];
            }
            curlayer.path = path.CGPath;
            
        }
    }

    return arrayAffectedPoints;
}
@end

@interface EraseToolHandler ()<IAnnotEventListener>

@property (nonatomic, strong) NSMutableArray *affectedPencilAnnotationArray;
@property (nonatomic, strong) NSMutableArray *needRemoveAnnotAttArr;
@property (nonatomic, assign) int annotationEditPageIndex;
@property (nonatomic, assign) CGRect currentEditRect;
@property (nonatomic, strong) NSMutableArray<FSAnnotAttributes *> *originalAttributesArray;
@property (nonatomic, strong) NSMutableDictionary *bezierViews;
@end

@implementation EraseToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotUnknownType; // todo

        self.pencilAnnotationArray = nil;
        self.affectedPencilAnnotationArray = nil;
        self.needRemoveAnnotAttArr = nil;
        _bezierViews = @{}.mutableCopy;
        _isMoving = NO;
        _isZooming = NO;
        _lastPoint = CGPointZero;
        _isBegin = NO;
        _radius = 0;
        _isChanged = NO;
        _changedPointCount = 0;
        _annotationEditPageIndex = -1;
        [_extensionsManager registerAnnotEventListener:self];
    }
    return self;
}

- (NSString *)getName {
    return Tool_Eraser;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
    if (self.pencilAnnotationArray.count){
        NSArray *tmp = self.pencilAnnotationArray.copy;
        self.pencilAnnotationArray = nil;
        for (FSAnnot *annot in tmp){
            [_pdfViewCtrl refresh:annot.pageIndex];
            break;
        }
    }
    if (self.pencilAnnotationArray){
        self.pencilAnnotationArray = nil;
    }
    [_pdfViewCtrl lockRefresh];
    for (EraseCAShapeView *eraseView in self.bezierViews.allValues.copy) {
        int pageIndex = eraseView.pageIndex;
        [eraseView removeFromSuperview];
        self.bezierViews[@(pageIndex)] = nil;
    }
    [_pdfViewCtrl unlockRefresh];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    [_pdfViewCtrl refresh:pageIndex needRender:NO];

    _isMoving = NO;
    return NO;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    if (recognizer.state == UIGestureRecognizerStateCancelled || recognizer.state == UIGestureRecognizerStateEnded ) {
        if (_isMoving) {
            return [self endEraseSuccessed:pageIndex];
        }

    }
    return YES;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return YES;
}

- (CGRect)getRectFromPointArray:(NSMutableArray<FSPointF *> *)pointArray {
    CGFloat greatestXValue = INT32_MIN;
    CGFloat greatestYValue = INT32_MIN;
    CGFloat smallestXValue = INT32_MIN;
    CGFloat smallestYValue = INT32_MIN;

    for (FSPointF *point in pointArray) {
        float x = point.x;
        float y = point.y;
        if (greatestXValue == INT32_MIN) {
            greatestXValue = x;
            greatestYValue = y;
            smallestXValue = x;
            smallestYValue = y;
        } else {
            greatestXValue = MAX(greatestXValue, x);
            greatestYValue = MAX(greatestYValue, y);
            smallestXValue = MIN(smallestXValue, x);
            smallestYValue = MIN(smallestYValue, y);
        }
    }

    CGRect rect;
    rect.origin = CGPointMake(smallestXValue, smallestYValue);
    rect.size.width = greatestXValue - smallestXValue;
    rect.size.height = greatestYValue - smallestYValue;

    return rect;
}

- (CGRect)getRectFromPencilAnnotation {
    CGRect ret = CGRectZero;

    for (FSAnnot *annot in self.pencilAnnotationArray) {
        CGRect rect = [Utility FSRectF2CGRect:annot.fsrect];
        if (CGRectEqualToRect(ret, CGRectZero)) {
            ret = rect;
        } else {
            ret = CGRectUnion(rect, ret);
        }
    }

    return ret;
}

- (void)isBeginUpdateInkAnnot:(FSInk *)inkAnnot{
    if (_isBegin) {
        [[_extensionsManager getAnnotHandlerByAnnot:inkAnnot] onAnnotSelected:inkAnnot];
        _isBegin = NO;
    }
}

- (void)resetInk:(FSInk *)inkAnnot arrayLines:(NSMutableArray<NSMutableArray<FSPointF *> *> *)arrayLines{
    if ([inkAnnot isEmpty]) return;
    @try {
        FSPath *newPath = [[FSPath alloc] init];
        for (int i = 0; i < arrayLines.count; i++) {
            NSArray<FSPointF *> *arrayPoints = [arrayLines objectAtIndex:i];
            if(arrayPoints.count > 1)
                [newPath moveTo:arrayPoints[0]];
            for (int j = 1; j < arrayPoints.count; j++) {
                [newPath lineTo:arrayPoints[j]];
            }
        }
        
        if (![self.affectedPencilAnnotationArray containsObject:inkAnnot]) {
            [self.originalAttributesArray addObject:[FSAnnotAttributes attributesWithAnnot:inkAnnot]];
        }
        
        if ([newPath getPointCount] <= 1){
            FSAnnotAttributes *annotAttributes = [FSAnnotAttributes attributesWithAnnot:inkAnnot];
            if (![self.needRemoveAnnotAttArr containsObject:annotAttributes]) {
                [self.needRemoveAnnotAttArr addObject:annotAttributes];
            }
            inkAnnot.opacity = 0;
        }else{
            [_pdfViewCtrl lockRefresh];
            [inkAnnot setInkList:newPath];
            [inkAnnot resetAppearanceStream];
            [_pdfViewCtrl unlockRefresh];
        }
    } @catch (NSException *exception) {
        [_pdfViewCtrl unlockRefresh];
    }

    if (![self.affectedPencilAnnotationArray containsObject:inkAnnot]) {
        [self.affectedPencilAnnotationArray addObject:inkAnnot];
    }
}

- (NSMutableArray<FSInk *> *)getInksInPage:(int)pageIndex {
    NSMutableArray<FSInk *> *inkAnnots = [NSMutableArray<FSInk *> array];
    FSPDFPage *page = [[_pdfViewCtrl getDoc] getPage:pageIndex];
    for (int i = 0; i < [page getAnnotCount]; i++) {
        FSAnnot *annot = [page getAnnot:i];
        if (annot.type == FSAnnotInk) {
            [inkAnnots addObject:[[FSInk alloc] initWithAnnot:annot]];
        }
    }
    return inkAnnots;
}

- (BOOL)endEraseSuccessed:(int)pageIndex{
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    if (pageIndex < 0) {
        return NO;
    }
    
    if (_isChanged){
        for (EraseCAShapeView *eraseView in self.bezierViews.allValues) {
            [eraseView resetInkPath];
        }
    }
    
    NSArray<FSAnnotAttributes *> *originalAttributesArray = [NSArray<FSAnnotAttributes *> arrayWithArray:self.originalAttributesArray];
    self.originalAttributesArray = nil;
    NSMutableArray<NSString *> *removedAnnotNMArray = [NSMutableArray<NSString *> array];
    NSMutableArray<FSAnnotAttributes *> *changedAttributesArray = [NSMutableArray<FSAnnotAttributes *> arrayWithCapacity:self.affectedPencilAnnotationArray.count];

    if (_isChanged) {
        id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:FSAnnotInk];
        for (int i = 0; i < self.affectedPencilAnnotationArray.count; i++) {
            FSInk *annot = self.affectedPencilAnnotationArray[i];
            
            BOOL needRemove = NO;
            @try {
                for (FSAnnotAttributes *annotAttributes in self.needRemoveAnnotAttArr){
                    if ([annotAttributes.NM isEqualToString:annot.NM]){
                        [annotAttributes resetAnnot:annot];
                        needRemove = YES;
                        break;
                    }
                }
            } @catch (NSException *exception) {
                
            }
            if ([[annot getInkList] getPointCount] <= 1 || needRemove) // remove if not visibale
            {
                [removedAnnotNMArray addObject:annot.NM];
                [annotHandler removeAnnot:annot addUndo:NO];
            } else {
                [changedAttributesArray addObject:[FSAnnotAttributes attributesWithAnnot:annot]];
                [annotHandler modifyAnnot:annot addUndo:NO];
            }
        }

        FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
        [_extensionsManager addUndoItem:[UndoItem itemWithUndo:^(UndoItem *item) {
                                NSMutableArray<FSAnnot *> *reAddedAnnotArray = [NSMutableArray<FSAnnot *> arrayWithCapacity:[removedAnnotNMArray count]];
                                FSRectF *tmpRect = [Utility makeFSRectWithLeft:0 top:0 right:10 bottom:10]; // casual value, we will reset rect later
                                CGRect unionRect = CGRectZero;
                                for (NSString *NM in removedAnnotNMArray) {
                                    FSAnnot *annot = [page addAnnot:FSAnnotInk rect:tmpRect];
                                    annot.NM = NM;
                                    [reAddedAnnotArray addObject:annot];
                                }
                                for (FSAnnotAttributes *attributes in originalAttributesArray) {
                                    FSAnnot *annot = [Utility getAnnotByNM:attributes.NM inPage:page];
                                    if (annot && ![annot isEmpty]) {
                                        CGRect rect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
                                        unionRect = CGRectIsEmpty(unionRect) ? rect : CGRectUnion(unionRect, rect);

                                        [attributes resetAnnot:annot];
                                        [annotHandler modifyAnnot:annot addUndo:NO];
                                        rect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
                                        unionRect = CGRectIsEmpty(unionRect) ? rect : CGRectUnion(unionRect, rect);
                                    }
                                }
                                for (FSAnnot *annot in reAddedAnnotArray) {
                                    [annotHandler addAnnot:annot addUndo:NO];
                                }
                                if (!CGRectIsEmpty(unionRect)) {
                                    [self->_pdfViewCtrl refresh:CGRectInset(unionRect, -10, -10) pageIndex:pageIndex];
                                }
                            }
                                            redo:^(UndoItem *item) {
                                                CGRect unionRect = CGRectZero;
                                                for (NSString *NM in removedAnnotNMArray) {
                                                    FSAnnot *annot = [Utility getAnnotByNM:NM inPage:page];
                                                    if (annot && ![annot isEmpty]) {
                                                        CGRect rect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
                                                        unionRect = CGRectIsEmpty(unionRect) ? rect : CGRectUnion(unionRect, rect);

                                                        [annotHandler removeAnnot:annot addUndo:NO];
                                                    }
                                                }
                                                for (FSAnnotAttributes *attributes in changedAttributesArray) {
                                                    FSAnnot *annot = [Utility getAnnotByNM:attributes.NM inPage:page];
                                                    if (annot && ![annot isEmpty]) {
                                                        CGRect rect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
                                                        unionRect = CGRectIsEmpty(unionRect) ? rect : CGRectUnion(unionRect, rect);

                                                        [attributes resetAnnot:annot];
                                                        [annotHandler modifyAnnot:annot addUndo:NO];
                                                        rect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
                                                        unionRect = CGRectIsEmpty(unionRect) ? rect : CGRectUnion(unionRect, rect);
                                                    }
                                                }
                                                if (!CGRectIsEmpty(unionRect)) {
                                                    [self->_pdfViewCtrl refresh:CGRectInset(unionRect, -10, -10) pageIndex:pageIndex];
                                                }
                                            }
                                            pageIndex:pageIndex
                                            attributes:originalAttributesArray]];
    }

    self.affectedPencilAnnotationArray = nil;
    self.needRemoveAnnotAttArr = nil;

    _isBegin = NO;
    _isMoving = NO;
    _isChanged = NO;
    _changedPointCount = 0;

    return YES;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [[touches anyObject] locationInView:pageView];

    if (_isBegin) {
        return NO;
    }
    _isBegin = YES;

    _isChanged = NO;


    self.annotationEditPageIndex = pageIndex;

    self.pencilAnnotationArray = [self getInksInPage:pageIndex];

    if (self.pencilAnnotationArray.count == 0) {
        return NO;
    }
    _radius = self.toolSettings.eraserLineWidth;

    self.affectedPencilAnnotationArray = [NSMutableArray array];
    self.originalAttributesArray = [NSMutableArray<FSAnnotAttributes *> array];
    self.needRemoveAnnotAttArr = @[].mutableCopy;

    //make overylay always draw background with rect
    _allRect = [self getRectFromPencilAnnotation];
    _penclRect = [Utility CGRect2FSRectF:_allRect];
    _allRect = [_pdfViewCtrl convertPdfRectToPageViewRect:_penclRect pageIndex:pageIndex];
    _allRect = [Utility getStandardRect:_allRect];
    _changedPointCount = 0;

    float scale = pageView.bounds.size.width / 1000.0;
    CGRect rect = CGRectMake(point.x - _radius * scale, point.y - _radius * scale, _radius * scale * 2, _radius * scale * 2);

    FSPointF *pdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    
    EraseCAShapeView *eraseView = self.bezierViews[@(pageIndex)];
    if (!eraseView){
        eraseView = [[EraseCAShapeView alloc] init];
        eraseView.backgroundColor = [UIColor clearColor];
        eraseView.userInteractionEnabled = NO;
        eraseView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
        self.bezierViews[@(pageIndex)] = eraseView;
    }

    CGRect b = pageView.bounds;
    eraseView.frame = b;
    eraseView.pageIndex = pageIndex;
    eraseView.pdfView = _pdfViewCtrl;
    eraseView.eraseToolHandler = self;
    [eraseView convertToCAShapeLayer];
    
    if ([eraseView updatePencil:point eraseStyle:self.toolSettings.eraseStyle].count > 0) {
        _isChanged = YES;
    }
    
    [self->_pdfViewCtrl refresh:pageIndex];

    _lastPoint = point;
    _isMoving = YES;
    return YES;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    
    if (_isMoving && !_isZooming) {
        if (pageIndex != self.annotationEditPageIndex) {
            [self onPageViewTouchesEnded:self.annotationEditPageIndex touches:touches withEvent:event];
            return YES;
        }
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [[touches anyObject] locationInView:pageView];
        FSPointF *pdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];

        EraseCAShapeView *eraseView = self.bezierViews[@(pageIndex)];
        if (!eraseView){
            eraseView = [[EraseCAShapeView alloc] init];
            eraseView.backgroundColor = [UIColor clearColor];
            self.bezierViews[@(pageIndex)] =  eraseView;
        }
        
        NSMutableArray<FSPointF *> *pointArray = [NSMutableArray<FSPointF *> array];
        [pointArray addObject:pdfPoint];
        FSPointF *lastPdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:_lastPoint pageIndex:pageIndex];
        [EraseUtil addCenterPoint:pdfPoint p2:lastPdfPoint radius:_radius array:pointArray];
        [pointArray enumerateObjectsUsingBlock:^(FSPointF *obj, NSUInteger idx, BOOL *stop) {
            CGPoint pt = [self->_pdfViewCtrl convertPdfPtToPageViewPt:obj pageIndex:pageIndex];
            if ([eraseView updatePencil:pt eraseStyle:self.toolSettings.eraseStyle].count){
                _isChanged = YES;
            }
        }];

        float scale = pageView.bounds.size.width / 1000.0;
        CGRect rect = [Utility convertToCGRect:point p2:_lastPoint];
        rect = CGRectInset(rect, -_radius * scale * 2, -_radius * scale * 2);
        dispatch_async(dispatch_get_main_queue(), ^{
            [self->_pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:NO];
        });

        if (CGRectEqualToRect(self.currentEditRect, CGRectZero)) {
            self.currentEditRect = rect;
        } else {
            self.currentEditRect = CGRectUnion(self.currentEditRect, rect);
        }
        _penclRect = [_pdfViewCtrl convertPageViewRectToPdfRect:self.currentEditRect pageIndex:pageIndex];

        _lastPoint = point;
        return YES;
    }
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    float scale = pageView.bounds.size.width / 1000.0;
    CGPoint point = [[touches anyObject] locationInView:pageView];
    CGRect rect = [Utility convertToCGRect:point p2:_lastPoint];
    rect = CGRectInset(rect, -_radius * scale * 2, -_radius * scale * 2);
    dispatch_async(dispatch_get_main_queue(), ^{
        [self->_pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:NO];
    });
    return [self endEraseSuccessed:pageIndex];
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

#pragma mark - IAnnotEventListener

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (!self.isEnabled) return;
    if (annot.type == FSAnnotInk) {
        [self.pencilAnnotationArray addObject:annot];
        EraseCAShapeView *eraseView = self.bezierViews[@(annot.pageIndex)];
        [eraseView addInk:annot];
    }
}

- (void)onAnnotWillDelete:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (!self.isEnabled) return;
    if (annot.type == FSAnnotInk) {
        for (FSAnnot *_annot in self.pencilAnnotationArray.copy){
            if ([_annot isEqualToAnnot:annot]) {
                [self.pencilAnnotationArray removeObject:_annot];
                EraseCAShapeView *eraseView = self.bezierViews[@(annot.pageIndex)];
                [eraseView removeInk:annot];
                break;
            }
        }
    }
}

- (void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (!self.isEnabled) return;
    if (annot.type == FSAnnotInk) {
        for (FSAnnot *_annot in self.pencilAnnotationArray){
            if ([_annot isEqualToAnnot:annot]) {
                EraseCAShapeView *eraseView = self.bezierViews[@(annot.pageIndex)];
                [eraseView modifyInk:_annot];
                break;
            }
        }
    }
}


- (BOOL)shouldDrawAnnot:(FSAnnot *)annot inPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    for (FSAnnot *_annot in self.pencilAnnotationArray){
        if ([_annot isEqualToAnnot:annot]) {
            EraseCAShapeView *eraseView = self.bezierViews[@(annot.pageIndex)];
            dispatch_main_async_safe(^{
                if (eraseView && !eraseView.superview){
                    UIView *pageView = [self->_pdfViewCtrl getPageView:annot.pageIndex];
                    CGRect b = pageView.bounds;
                    eraseView.frame = b;
                    [pageView addSubview:eraseView];
                }
            });
            return NO;
        }
    }
    if (annot.type == FSAnnotInk) {
        EraseCAShapeView *eraseView = self.bezierViews[@(annot.pageIndex)];
        dispatch_main_async_safe(^{
            if (eraseView && eraseView.superview){
                [eraseView removeFromSuperview];
            }
        });
    }
    return YES;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (pageIndex == self.annotationEditPageIndex) {
        EraseCAShapeView *eraseView = self.bezierViews[@(pageIndex)];
        if (eraseView && eraseView.superview){
            UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
            CGRect b = pageView.bounds;
            if (!CGRectEqualToRect(b, eraseView.frame)) eraseView.frame = b;
        }
        if (_isMoving && !_isZooming){
            float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000.0;
            
            // draw gray circle
            CGContextSetLineWidth(context, 2); // set the line width
            CGContextSetFillColorWithColor(context, [UIColor colorWithRed:0 green:0 blue:0 alpha:0.2].CGColor);

            CGPoint center = _lastPoint;              // get the circle centre
            CGFloat radius = _radius * scale;         // little scaling needed
            CGFloat startAngle = -((float) M_PI / 2); // 90 degrees
            CGFloat endAngle = ((2 * (float) M_PI) + startAngle);
            CGContextAddArc(context, center.x, center.y, radius + 4, startAngle, endAngle, 0); // create an arc the +4 just adds some pixels because of the polygon line thickness
            CGContextFillPath(context);
        }
    }
}
@synthesize toolSettings;

@end

