/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
#if !TARGET_OS_MACCATALYST
#import <Foundation/Foundation.h>
#import "RecognizedInk.h"



/**
 * Protocol used by the `StrokeManager` to send requests back to the `ViewController` to update the
 * display.
 */
@protocol StrokeManagerDelegate <NSObject>

/** Display the given message to the user. */
- (void)displayMessage:(NSString *)message;

- (void)downModelSuccess:(NSString *) tag;

- (void)inkSuccess:(NSString *)text;
@end

/**
 * The `StrokeManager` object is responsible for storing the ink and recognition results, and
 * managing the interaction with the recognizer. It receives the touch points as the user is drawing
 * from the `ViewController` (which takes care of rendering the ink), and stores them into an array
 * of `Stroke`s. When the user taps "recognize", the strokes are collected together into an `Ink`
 * object, and passed to the recognizer. The `StrokeManagerDelegate` protocol is used to inform the
 * `ViewController` when the display needs to be updated.
 *
 * The `StrokeManager` provides additional methods to handle other buttons in the UI, including
 * selecting a recognition language, downloading or deleting the recognition model, or clearing the
 * ink.
 */
@interface StrokeManager : NSObject

/**
 * Array of `RecognizedInk`s that have been sent to the recognizer along with any recognition
 * results.
 */
//@property(readonly, nonatomic) NSArray<RecognizedInk *> *recognizedInks;

/**
 * Initializes internal state and stores a pointer to the view to allow for redrawing when ink is
 * sent to the recognizer or recognition results come back.
 */
- (nullable instancetype)initWithDelegate:(nullable id<StrokeManagerDelegate>)delegate
    NS_DESIGNATED_INITIALIZER;
- (nullable instancetype)init NS_UNAVAILABLE;

- (void)setDelegate:(nullable id<StrokeManagerDelegate>)delegate;

/** Function called by the `ViewController` to create the first point of a stroke. */
- (void)startStrokeAtPoint:(CGPoint)point time:(NSTimeInterval)t;

/** Function called by the `ViewController` to add a point to a stroke. */
- (void)continueStrokeAtPoint:(CGPoint)point time:(NSTimeInterval)t;

/** Function called by the `ViewController` to end a stroke. */
- (void)endStrokeAtPoint:(CGPoint)point time:(NSTimeInterval)t;



- (void)addAtPoint:(FSPDFViewCtrl *_Nullable)pdfViewCtrl annots:(NSMutableArray<NSMutableArray<NSMutableArray<FSPointF *> *> *> *_Nullable)inkListss index:(int)index;


/** Clears all ink. */
- (void)clear;

/** Informs the `StrokeManager` of which recognizer to use for subsequent recognitions. */
- (void)selectLanguage:(NSString *_Nullable)languageTag;

/**
 * Asks the `StrokeManager` to start downloading the recognition model indicated by
 * `selectLanguage:`.
 */
- (void)downloadModel;

- (void)downloadModel:(NSString *_Nullable)languageTag;

/**
 * Check whether the model for the given language tag is already downloaded.
 */
- (BOOL)isLanguageDownloaded:(NSString *_Nullable)languageTag;

/** Asks the `StrokeManager` to delete the reocngition model indicated by `selectLanguage:`. */
- (void)deleteModel;

- (void)deleteModel:(NSString *_Nullable)languageTag;

/**
 * Asks the `StrokeManager` to recognize the unrecognized ink using the recognition model indicated
 * by `selectLanguage:`.
 */
- (void)recognizeInk;

@end
#endif
