/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PolyLineToolHandler.h"
#import "PaintUtility.h"

@interface PolyLineToolHandler ()

@property (nonatomic, strong) FSPointFArray *vertexes;
@property (nonatomic, strong) UIImage *annotImage;
@property (nonatomic) int pageIndex;
@property (nonatomic, strong) PaintUtility *paintUtility;

@end

@implementation PolyLineToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotPolyLine;
        _vertexes = [[FSPointFArray alloc] init];
        _annot = nil;
        _annotImage = nil;
        _minVertexDistance = 5;
        _pageIndex = -1;
        [_pdfViewCtrl registerGestureEventListener:self];
    }
    return self;
}

- (NSString *)getName {
    return Tool_PolyLine;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)reset {
    if (self.annot) {
        FSPolyLine *annot = self.annot;
        self.annot = nil;
        self.annotImage = nil;
        int pageIndex = annot.pageIndex;
        id<IAnnotHandler> annothandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
        [annothandler addAnnot:annot addUndo:YES];
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        
        if ([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE) {
            [_pdfViewCtrl refresh];
        }else{
            [_pdfViewCtrl refresh:CGRectInset(pvRect, -30, -30) pageIndex:pageIndex needRender:YES];
        }
    }else{
        [_pdfViewCtrl refresh:self.pageIndex];
    }

    [self.vertexes removeAll];
    self.pageIndex = -1;
}

- (CGRect)getRectFromVertexes:(int)pageIndex{
    NSMutableArray *points = @[].mutableCopy;
    for (int i = 0; i < [self.vertexes getSize]; i++) {
        CGPoint vertex  = [_pdfViewCtrl convertPdfPtToPageViewPt:[self.vertexes getAt:i] pageIndex:pageIndex];
        [points addObject:@(vertex)];
    }
    return [Utility getRectFromPoints:points];
}
- (void)onDeactivate {
    [self updateAnnotVertexesInPage:self.pageIndex];
    [self reset];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self ) {
        return NO;
    }
    
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    self.paintUtility = [[PaintUtility alloc] init];
    self.paintUtility.annotLineWidth = 2;
    self.paintUtility.annotColor = self.toolSettings.settingsColor;
    self.paintUtility.annotOpacity = self.toolSettings.opacity;
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        
        if (self.pageIndex != -1 && pageIndex != -1 && self.pageIndex != pageIndex) {
            if (self.vertexes.getSize > 0) {
                [self completeAnnotation];
            }
        }
        
        self.pageIndex = pageIndex;
        [self.vertexes add:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        [_pdfViewCtrl refresh:pageIndex];
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (pageIndex != self.pageIndex) {
            return YES;
        }
        
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        if (point.x < 0) {
            point.x = 0;
        }else if (point.x > pageView.bounds.size.width){
            point.x = pageView.bounds.size.width;
        }
        
        if (point.y < 0) {
            point.y = 0;
        }else if (point.y > pageView.bounds.size.height){
            point.y = pageView.bounds.size.height;
        }
        
        if (self.vertexes.getSize <= 1) {
            [self.vertexes add:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        }
        [self.vertexes setAt:self.vertexes.getSize-1 newElement:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        [_pdfViewCtrl refresh:pageIndex];
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        if (pageIndex != self.pageIndex) {
            pageIndex = self.pageIndex;
            pageview = [_pdfViewCtrl getPageView:pageIndex];
            point = [recognizer locationInView:pageview];
        }
        
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        if (point.x < 0) {
            point.x = 0;
        }else if (point.x > pageView.bounds.size.width){
            point.x = pageView.bounds.size.width;
        }
        
        if (point.y < 0) {
            point.y = 0;
        }else if (point.y > pageView.bounds.size.height){
            point.y = pageView.bounds.size.height;
        }
        
        [self.vertexes setAt:self.vertexes.getSize-1 newElement:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        [_pdfViewCtrl refresh:pageIndex];
    }
    return YES;
}

- (BOOL)isVertexValid:(CGPoint)vertex pageIndex:(int)pageIndex {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGRect validRect = CGRectInset(pageView.bounds, 5, 5);
    if (!CGRectContainsPoint(validRect, vertex)) {
        return NO;
    }
    if ([self.vertexes getSize] > 0) {
        CGPoint lastPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:[self.vertexes getAt:[self.vertexes getSize] - 1] pageIndex:pageIndex];
        CGFloat dx = lastPoint.x - vertex.x;
        CGFloat dy = lastPoint.y - vertex.y;
        if (dx * dx + dy * dy < self.minVertexDistance * self.minVertexDistance) {
            return NO;
        }
    }
    return YES;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (self.pageIndex != pageIndex) {
        return;
    }
    
    UIImage *dot = [UIImage imageNamed:@"annotation_drag_highlight.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *highlightDot = [UIImage imageNamed:@"annotation_drag_highlight.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    CGSize dotSize = dot.size;
    if (self.annotImage) {
        CGRect annotRect = [Utility getAnnotRect:self.annot pdfViewCtrl:_pdfViewCtrl];
        CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
        annotRect.origin.x = rect.origin.x;
        annotRect.origin.y = rect.origin.y;
        CGContextSaveGState(context);
        CGContextTranslateCTM(context, annotRect.origin.x, annotRect.origin.y);
        CGContextTranslateCTM(context, 0, annotRect.size.height);
        CGContextScaleCTM(context, 1.0, -1.0);
        CGContextTranslateCTM(context, -annotRect.origin.x, -annotRect.origin.y);
        CGContextDrawImage(context, annotRect, [self.annotImage CGImage]);
        CGContextRestoreGState(context);
    }else{
        
        float lineWidth = [Utility convertWidth:self.toolSettings.thickness fromPDFToPageView:_extensionsManager.pdfViewCtrl pageIndex:pageIndex];
        CGContextSetLineWidth(context, lineWidth);
        
        UIColor *color = [UIColor colorWithRGB:self.toolSettings.settingsColor];
        CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],self.toolSettings.opacity);
        
        CGContextBeginPath(context);
        CGContextSetLineCap(context, kCGLineCapRound);
        CGContextSetLineJoin(context, kCGLineJoinRound);
        CGContextSetBlendMode(context, kCGBlendModeDestinationAtop);
        
        for (int i = 0; i < [self.vertexes getSize]; i++) {
            FSPointF *vertex = [self.vertexes getAt:i];
            CGPoint point = [_pdfViewCtrl convertPdfPtToPageViewPt:vertex pageIndex:pageIndex];
            if (i == 0) {
                CGContextMoveToPoint(context, point.x, point.y);
            } else {
                CGContextAddLineToPoint(context, point.x, point.y);
            }

        }
        CGContextStrokePath(context);
    }
    
    for (int i = 0; i < [self.vertexes getSize]; i++) {
        FSPointF *vertex = [self.vertexes getAt:i];
        CGPoint point = [_pdfViewCtrl convertPdfPtToPageViewPt:vertex pageIndex:pageIndex];
        if (i == 0) {
            point.x -= highlightDot.size.width / 2;
            point.y -= highlightDot.size.height / 2;
            [highlightDot drawAtPoint:point];
        } else {
            point.x -= dotSize.width / 2;
            point.y -= dotSize.height / 2;
            [dot drawAtPoint:point];
        }
    }
}

- (void)updateAnnotVertexesInPage:(int)pageIndex {
    if (self.annot == nil && [self.vertexes getSize] > 2) {
        self.annot = [self addAnnotToPage:pageIndex];
    }
    if (self.annot) {
        [self.annot setBorderInfo:({
            FSBorderInfo *borderInfo = [[FSBorderInfo alloc] init];
            [borderInfo setWidth:self.toolSettings.thickness];
            [borderInfo setStyle:FSBorderInfoSolid];
            borderInfo;
        })];

        [self.annot setVertexes:self.vertexes];
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        self.annotImage = [Utility getAnnotImage:self.annot pdfViewCtrl:_pdfViewCtrl];
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
        [_pdfViewCtrl refresh:CGRectInset(pvRect, -30, -30) pageIndex:pageIndex needRender:NO];
    } else if ([self.vertexes getSize] < 3 && [self.vertexes getSize] > 0) {
        [_pdfViewCtrl refresh:CGRectInset([self getRectFromVertexes:pageIndex], -30, -30) pageIndex:pageIndex needRender:NO];
        [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddPolylineFailedHints")];
    }
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    if (self.annot && self.annot.pageIndex != pageIndex) {
        return YES;
    }
    if (self.pageIndex != -1 && self.pageIndex != pageIndex) {
        if (self.vertexes.getSize > 0) {
            [self completeAnnotation];
        }
    }
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    if (![self isVertexValid:point pageIndex:pageIndex]) {
        return YES;
    }
    self.pageIndex = pageIndex;
    [self.vertexes add:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
    [_pdfViewCtrl refreshOverlay];
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    if ([gestureRecognizer isKindOfClass:[UITapGestureRecognizer class]]) {
        if (((UITapGestureRecognizer *) gestureRecognizer).numberOfTapsRequired == 2) {
            if (_extensionsManager.currentToolHandler == self) {
                return YES;
            }
        }
    }
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (FSPolyLine *)addAnnotToPage:(int)pageIndex {
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty])
        return nil;

    FSPolyLine *annot = nil;
    FSRectF *rect = [[FSRectF alloc] init];
    [rect setLeft:0];
    [rect setBottom:0];
    [rect setRight:0];
    [rect setTop:0];
    @try {
        annot = [[FSPolyLine alloc] initWithAnnot:[page addAnnot:self.type rect:rect]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
        return nil;
    }
    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.color = self.toolSettings.settingsColor;
    annot.opacity = self.toolSettings.opacity;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;
    [annot setBorderInfo:({
               FSBorderInfo *borderInfo = [[FSBorderInfo alloc] init];
               [borderInfo setWidth:self.toolSettings.thickness];
               [borderInfo setStyle:FSBorderInfoSolid];
               borderInfo;
           })];
//    annot.intent = @"PolyLineDimension";
    annot.subject = @"PolyLine";
    return annot;
}

#pragma mark <IAnnotPropertyListener>

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.annot && toolSettings.settingsType == self.toolSettings.settingsType) {
        self.annot.color = toolSettings.settingsColor;
        self.annot.opacity = toolSettings.opacity;
        self.annot.lineWidth = toolSettings.thickness;
        [self updateAnnotProperty];
    }
}

- (void)updateAnnotProperty {
    if (self.annot) {
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        self.annotImage = [Utility getAnnotImage:self.annot pdfViewCtrl:_pdfViewCtrl];
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:self.pageIndex];
        [_pdfViewCtrl refresh:CGRectInset(pvRect, -30, -30) pageIndex:self.pageIndex needRender:NO];
    }
}

- (void)completeAnnotation
{
    [self updateAnnotVertexesInPage:self.pageIndex];
    [self reset];
}

#pragma mark - IGestureEventListener
- (BOOL)onDoubleTap:(UITapGestureRecognizer *)gestureRecognizer{
    if (_extensionsManager.currentToolHandler == self) {
        [self completeAnnotation];
        return YES;
    }
    return NO;
}

@synthesize toolSettings;

@end
