/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PolygonToolHandler.h"
#import "FSAnnotExtent.h"
#import "Utility.h"
#import "PaintUtility.h"

@interface PolygonToolHandler ()

@property (nonatomic, strong) FSPointFArray *vertexes;
@property (nonatomic, strong) UIImage *annotImage;
@property (nonatomic) int pageIndex;
@property (nonatomic, strong) PaintUtility *paintUtility;
@property (nonatomic, assign) CGRect oldRect;
@end

@implementation PolygonToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotPolygon;
        _vertexes = [[FSPointFArray alloc] init];
        _annot = nil;
        _annotImage = nil;
        _minVertexDistance = 5;
        _pageIndex = -1;
        [_pdfViewCtrl registerGestureEventListener:self];
    }
    return self;
}

- (NSString *)getName {
    return Tool_Polygon;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)reset {
    if (self.annot) {
        FSPolygon *annot = self.annot;
        self.annot = nil;
        self.annotImage = nil;

        int pageIndex = annot.pageIndex;
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        if ([self.vertexes getSize] <= 2) {
            if (self.isPolygon) {
                [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddPolygonFailedHints")];
            }else{
                [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddCloudFailedHints")];
            }
            [annot.getPage removeAnnot:annot];
        }else{
            id<IAnnotHandler> annothandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
            [annothandler addAnnot:annot addUndo:YES];
        }

        if ([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE) {
            [_pdfViewCtrl refresh];
        }else{
            [_pdfViewCtrl refresh:CGRectInset(pvRect, -30, -30) pageIndex:pageIndex needRender:YES];
        }
    }else {
        if ([self.vertexes getSize] <= 2 && [self.vertexes getSize] >= 1) {
            if (self.isPolygon) {
                [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddPolygonFailedHints")];
            }else{
                [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddCloudFailedHints")];
            }
        }
        [_pdfViewCtrl refresh:self.pageIndex];
    }
    [self.vertexes removeAll];
    self.pageIndex = -1;
}

- (void)removeAnnot {
    if (self.annot) {
        FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:self.pageIndex];
        [page removeAnnot:self.annot];
        self.annot = nil;
        self.annotImage = nil;
    }else{
        [_pdfViewCtrl refresh:self.pageIndex];
    }
    [self.vertexes removeAll];
    self.pageIndex = -1;
}

- (void)onDeactivate {
    [self updateAnnotVertexesInPage:self.pageIndex];
    [self reset];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    self.paintUtility = [[PaintUtility alloc] init];
    self.paintUtility.annotLineWidth = self.toolSettings.thickness;
    self.paintUtility.annotColor = self.toolSettings.settingsColor;
    self.paintUtility.annotFillColor = (self.toolSettings.fillColor.rgbaValue == 0xFFFFFF00 ? 0xFFFFFF00 : self.toolSettings.fillColor.rgbValue);
    self.paintUtility.annotOpacity = self.toolSettings.opacity;
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        
        if (self.pageIndex != -1 && pageIndex != -1 && self.pageIndex != pageIndex) {
            if (self.vertexes.getSize > 0) {
                [self completeAnnotation];
            }
        }
        
        self.pageIndex = pageIndex;
        [self.vertexes add:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        if (self.isPolygon) {
            [_pdfViewCtrl refresh:pageIndex];
        }else{
            [self updateAnnotVertexesInPage:self.pageIndex];
        }
        
    }else if (recognizer.state == UIGestureRecognizerStateChanged ) {
        if (pageIndex != self.pageIndex) {
            return YES;
        }
        
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        if (point.x < 0) {
            point.x = 0;
        }else if (point.x > pageView.bounds.size.width){
            point.x = pageView.bounds.size.width;
        }
        
        if (point.y < 0) {
            point.y = 0;
        }else if (point.y > pageView.bounds.size.height){
            point.y = pageView.bounds.size.height;
        }
        
        if (self.vertexes.getSize <= 1) {
            [self.vertexes add:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        }
        [self.vertexes setAt:self.vertexes.getSize-1 newElement:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        if (self.isPolygon) {
            [_pdfViewCtrl refresh:pageIndex];
        }else{
            [self updateAnnotVertexesInPage:self.pageIndex];
        }
    }
    else if (recognizer.state == UIGestureRecognizerStateCancelled || recognizer.state == UIGestureRecognizerStateEnded) {
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        if (pageIndex != self.pageIndex) {
            pageIndex = self.pageIndex;
            pageview = [_pdfViewCtrl getPageView:pageIndex];
            point = [recognizer locationInView:pageview];
        }
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        if (point.x < 0) {
            point.x = 0;
        }else if (point.x > pageView.bounds.size.width){
            point.x = pageView.bounds.size.width;
        }
        
        if (point.y < 0) {
            point.y = 0;
        }else if (point.y > pageView.bounds.size.height){
            point.y = pageView.bounds.size.height;
        }
        
        [self.vertexes setAt:self.vertexes.getSize-1 newElement:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
        if (self.isPolygon) {
            [_pdfViewCtrl refresh:pageIndex];
        }else{
            [self updateAnnotVertexesInPage:self.pageIndex];
        }
    }
        
    return YES;
}

- (BOOL)isVertexValid:(CGPoint)vertex pageIndex:(int)pageIndex {
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGRect validRect = CGRectInset(pageView.bounds, 5, 5);
    if (!CGRectContainsPoint(validRect, vertex)) {
        return NO;
    }
    int vertexCount = [self.vertexes getSize];
    if (vertexCount > 0) {
        CGPoint lastPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:[self.vertexes getAt:vertexCount - 1] pageIndex:pageIndex];
        CGFloat dx = lastPoint.x - vertex.x;
        CGFloat dy = lastPoint.y - vertex.y;
        if (dx * dx + dy * dy < self.minVertexDistance * self.minVertexDistance) {
            return NO;
        }
    }
    return YES;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (self.pageIndex != pageIndex) {
        return;
    }
    if (self.annotImage) {
        CGRect annotRect = [Utility getAnnotRect:self.annot pdfViewCtrl:_pdfViewCtrl];
        CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
        annotRect.origin.x = rect.origin.x;
        annotRect.origin.y = rect.origin.y;
        CGContextSaveGState(context);
        CGContextTranslateCTM(context, annotRect.origin.x, annotRect.origin.y);
        CGContextTranslateCTM(context, 0, annotRect.size.height);
        CGContextScaleCTM(context, 1.0, -1.0);
        CGContextTranslateCTM(context, -annotRect.origin.x, -annotRect.origin.y);
        CGContextDrawImage(context, annotRect, [self.annotImage CGImage]);
        CGContextRestoreGState(context);
    }
    UIImage *dot = [UIImage imageNamed:@"annotation_drag_highlight.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *highlightDot = [UIImage imageNamed:@"annotation_drag_highlight.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    CGSize dotSize = dot.size;
    CGFloat lineWidth = [Utility convertWidth:self.paintUtility.annotLineWidth fromPDFToPageView:_pdfViewCtrl pageIndex:pageIndex];
    for (int i = 0; i < [self.vertexes getSize]; i++) {
        FSPointF *vertex = [self.vertexes getAt:i];
        CGPoint point = [_pdfViewCtrl convertPdfPtToPageViewPt:vertex pageIndex:pageIndex];
        if (i == 0) {
            point.x -= highlightDot.size.width / 2;
            point.y -= highlightDot.size.height / 2;
            [highlightDot drawAtPoint:point];
        } else {
            point.x -= dotSize.width / 2;
            point.y -= dotSize.height / 2;
            [dot drawAtPoint:point];
            
            if (self.isPolygon) {
                FSPointF *startVertex = [self.vertexes getAt:i-1];
                CGPoint startCGPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:startVertex pageIndex:pageIndex];
                CGPoint endCGPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:vertex pageIndex:pageIndex];
                [self.paintUtility drawLineFromStartPoint:startCGPoint toEndPoint:endCGPoint lineWidth:lineWidth inContext:context isArrowLine:NO];
            }
        }
    }
}

- (void)updateAnnotVertexesInPage:(int)pageIndex {
    if (self.annot == nil && [self.vertexes getSize] > 1) {
        self.annot = [self addAnnotToPage:pageIndex];
    }
    if (self.annot) {
        [self.annot setVertexes:self.vertexes];
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        self.annotImage = [Utility getAnnotImage:self.annot pdfViewCtrl:_pdfViewCtrl];
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
        CGRect unionRect = CGRectUnion(pvRect, self.oldRect);
        [_pdfViewCtrl refresh:CGRectInset(unionRect, -30, -30) pageIndex:pageIndex needRender:NO];
        self.oldRect = pvRect;

    } else if ([self.vertexes getSize] == 1) {
        CGPoint vertex = [_pdfViewCtrl convertPdfPtToPageViewPt:[self.vertexes getAt:0] pageIndex:pageIndex];
        [_pdfViewCtrl refresh:CGRectInset(CGRectMake(vertex.x, vertex.y, 0, 0), -30, -30) pageIndex:pageIndex needRender:NO];
    }
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    if (self.pageIndex != -1 && self.pageIndex != pageIndex) {
        if (self.vertexes.getSize > 0) {
            [self completeAnnotation];
        }
    }
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    if (![self isVertexValid:point pageIndex:pageIndex]) {
        return YES;
    }
    self.pageIndex = pageIndex;
    [self.vertexes add:[_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex]];
    
    if (self.isPolygon)
    {
        self.paintUtility = [[PaintUtility alloc] init];
        self.paintUtility.annotLineWidth = self.toolSettings.thickness;
        self.paintUtility.annotColor = self.toolSettings.settingsColor;
        self.paintUtility.annotFillColor = (self.toolSettings.fillColor.rgbaValue == 0xFFFFFF00 ? 0xFFFFFF00 : self.toolSettings.fillColor.rgbValue);
        self.paintUtility.annotOpacity = self.toolSettings.opacity;
        [_pdfViewCtrl refreshOverlay];
    } else {
        [self updateAnnotVertexesInPage:self.pageIndex];
    }
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    if ([gestureRecognizer isKindOfClass:[UITapGestureRecognizer class]]) {
        if (((UITapGestureRecognizer *) gestureRecognizer).numberOfTapsRequired == 2) {
            if (_extensionsManager.currentToolHandler == self) {
                return YES;
            }
        }
    }
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    return YES;
}

- (FSPolygon *)addAnnotToPage:(int)pageIndex {
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty])
        return nil;

    FSPolygon *annot = nil;
    FSRectF *rect = [[FSRectF alloc] init];
    [rect setLeft:0];
    [rect setBottom:0];
    [rect setRight:0];
    [rect setTop:0];
    @try {
        annot = [[FSPolygon alloc] initWithAnnot:[page addAnnot:self.type rect:rect]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
        return nil;
    }
    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.color = self.toolSettings.settingsColor;
    annot.exfillColor = (self.toolSettings.fillColor.rgbaValue == 0xFFFFFF00 ? 0xFFFFFF00 : self.toolSettings.fillColor.rgbValue);
    annot.opacity = self.toolSettings.opacity;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;
    [annot setBorderInfo:({
               FSBorderInfo *borderInfo = [[FSBorderInfo alloc] init];
               [borderInfo setWidth:self.toolSettings.thickness];
               if (self.isPolygon) {
                   [borderInfo setStyle:FSBorderInfoSolid];
               } else {
                   [borderInfo setStyle:FSBorderInfoCloudy];
                   [borderInfo setCloud_intensity:2.0];
               }
               borderInfo;
           })];
    if (!self.isPolygon) {
        annot.subject = @"Cloud";
    } else {
        annot.subject = @"Polygon";
    }
    
    return annot;
}

#pragma mark <IAnnotPropertyListener>



- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.annot && toolSettings.settingsType == self.toolSettings.settingsType) {
        self.annot.color = toolSettings.settingsColor;
        self.annot.opacity = toolSettings.opacity;
        self.annot.lineWidth = toolSettings.thickness;
        [self updateAnnotProperty];
    }
}

- (void)updateAnnotProperty {
    if (self.annot) {
        [_pdfViewCtrl lockRefresh];
        [self.annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        self.annotImage = [Utility getAnnotImage:self.annot pdfViewCtrl:_pdfViewCtrl];
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:self.pageIndex];
        [_pdfViewCtrl refresh:CGRectInset(pvRect, -30, -30) pageIndex:self.pageIndex needRender:NO];
    }
}

- (void)completeAnnotation
{
    if ([self.vertexes getSize] <= 2 && [self.vertexes getSize] >= 1) {
        if (self.isPolygon) {
            [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddPolygonFailedHints")];
        }else{
            [_pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kAddCloudFailedHints")];
        }
    }
    [self updateAnnotVertexesInPage:self.pageIndex];
    [self reset];
}

#pragma mark - IGestureEventListener
- (BOOL)onDoubleTap:(UITapGestureRecognizer *)gestureRecognizer{
    if (_extensionsManager.currentToolHandler == self) {
        [self completeAnnotation];
        return YES;
    }
    return NO;
}

@synthesize toolSettings;

@end
