/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReadingBookmarkViewController.h"
#import "UIExtensionsManager.h"
#import "FSNavigationController.h"
#import "FSDataItem.h"
#import "FSPermissionUtilty.h"

@interface FSReadingBookmarkItem : FSDataItem
@property (nonatomic, copy) NSString *title;
@property (nonatomic, assign, readonly) int pageIndex;
@property (nonatomic, strong, readonly) FSReadingBookmark *bookmark;
@end

@implementation FSReadingBookmarkItem

- (instancetype)initWithBookmark:(FSReadingBookmark *)bookmark{
    self = [super init];
    if (self) {
        _pageIndex = [bookmark getPageIndex];
        _bookmark = bookmark;
    }
    return self;
}

- (NSString *)title{
    @try {
        return [_bookmark getTitle];
    } @catch (NSException *exception) {
        return @"";
    }
    return @"";
}

- (void)setTitle:(NSString *)title{
    [_bookmark setTitle:title];
}

- (void)replaceBookmark:(FSReadingBookmark *)bookmark{
    _pageIndex = [bookmark getPageIndex];
    _bookmark = bookmark;
}

@end

@interface ReadingBookmarkViewController ()

@property (nonatomic, strong) UIViewController *originalPresentedViewController;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@end


@implementation ReadingBookmarkViewController

@synthesize bookmarkGotoPageHandler = _bookmarkGotoPageHandler;
@synthesize bookmarkSelectionHandler = _bookmarkSelectionHandler;
@synthesize isContentEditing = _isContentEditing;
@synthesize arrayBookmarks = _arrayBookmarks;

#pragma mark - rewrite

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar{
    if (self.editing) {
        self.editing = NO;
    }
    [_panelController.panel.pagerContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_panelController.panel.contentView.fs_mas_left);
        make.right.mas_equalTo(_panelController.panel.contentView.fs_mas_right);
        make.bottom.top.fs_equalToSuperview();
    }];
    [_panelController.panel.contentView layoutIfNeeded];
    _panelController.panel.pagerContentView.scrollEnabled = NO;
    return YES;
}

- (void)clickedSearchCancelItem{
    [super clickedSearchCancelItem];
    [_panelController.panel.pagerContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_panelController.panel.contentView.mas_left);
        make.right.mas_equalTo(_panelController.panel.contentView.mas_right);
        make.bottom.fs_equalToSuperview();
        make.top.mas_equalTo(_panelController.panel.topBar.mas_bottom);
    }];
    [_panelController.panel.contentView layoutIfNeeded];
    _panelController.panel.pagerContentView.scrollEnabled = YES;
    self.panel.rightItem.selected = self.editing;
}

- (void)enteredSearchState{
    [super enteredSearchState];
    [self reloadData];
}

- (void)setSelectedIndexPaths:(NSArray<NSIndexPath *> *)selectedIndexPaths{
    [super setSelectedIndexPaths:selectedIndexPaths];
    NSArray *bookmark = [[self dataSource] firstObject];
    if (bookmark.count) {
        self.editLeftItem.selected = self.selectedIndexPaths.count ==  bookmark.count;
    }
}

- (void)setEditing:(BOOL)editing{
    [super setEditing:editing];
    _panel.rightItem.selected = editing;
}

- (void)clickedEditLeftItem{
    if ([self getBookmarkCount] > 0) {
        self.editLeftItem.selected = !self.editLeftItem.selected;
        if (self.editLeftItem.selected) {
            [self selectAll];
        }else{
            [self deselectAll];
        }
    }
}

- (void)clickedEditRightItem{
    if ([self selectedIndexPaths].count > 0 && [self selectedDataSource].count > 0) {
        void(^block)(void) = ^{
            for (FSReadingBookmarkItem *deletedBookmark in self.selectedDataSource) {
                [self->_pdfViewCtrl.currentDoc removeReadingBookmark:deletedBookmark.bookmark];
                [self->_arrayBookmarks removeObject:deletedBookmark];
            }
            [self.tableView deleteRowsAtIndexPaths:self.selectedIndexPaths withRowAnimation:UITableViewRowAnimationFade];
            [self reloadEmptyDataSet];
            [super clickedEditRightItem];
            self.bookmarkDeleteHandler();
        };
        
        if (self.selectedDataSource.count != self.getBookmarkCount) {
            block();
        }else{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kClearBookmark") actions:@[ FSLocalizedForKey(@"kNo"), FSLocalizedForKey(@"kYes")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    block();
                }
            }onCtrl:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
        }
    }
}

- (void)loadData{
    NSMutableArray *bookmarks = [[NSMutableArray alloc] init];
    @try {
        int count = [_pdfViewCtrl.currentDoc getReadingBookmarkCount];
        
        NSMutableArray *selectedDataSource = @[].mutableCopy;
        NSMutableArray *tmp = self.selectedDataSource.mutableCopy;
        for (int i = 0; i < count; i++) {
            FSReadingBookmark *bookmark = [_pdfViewCtrl.currentDoc getReadingBookmark:i];
            FSReadingBookmarkItem *bookmarkItem = [[FSReadingBookmarkItem alloc] initWithBookmark:bookmark];
            for (FSReadingBookmarkItem *selectedBookmark in tmp) {
                if (selectedBookmark.pageIndex == bookmark.pageIndex) {
                    [selectedBookmark replaceBookmark:bookmark];
                    [selectedDataSource addObject:selectedBookmark];
                    [tmp removeObject:selectedBookmark];
                    bookmarkItem = selectedBookmark;
                    break;
                }
            }
            [bookmarks addObject:bookmarkItem];
        }
        NSMutableArray *selectedIndexPaths = @[].mutableCopy;
        for (FSReadingBookmarkItem *bookmarkItem in selectedDataSource) {
            NSUInteger index = [bookmarks indexOfObject:bookmarkItem];
            if (index != NSNotFound) {
                [selectedIndexPaths addObject:[NSIndexPath indexPathForRow:index inSection:0]];
            }
        }
        self.selectedIndexPaths = selectedIndexPaths;
        self.selectedDataSource = selectedDataSource;
        if (bookmarks.count == self.selectedDataSource.count && self.selectedDataSource.count) {
            self.editLeftItem.selected = YES;
        }else{
            self.editLeftItem.selected = NO;
        }
        
    } @catch (NSException *exception) {
        NSLog(@"Failed to load reading bookmark: '%@'", exception.description);
    } @finally {
        self.arrayBookmarks = bookmarks;
    }
    self.dataSource = @[_arrayBookmarks];
}

- (BOOL)evaluateWithObject:(FSReadingBookmarkItem *)object{
    @try {
        return [super evaluateWithObject:object.title];
    } @catch (NSException *exception) {
        return NO;
    }
    return NO;
}

- (NSArray<UILabel *> *)keywordsHighlightLabels:(FSTitleEstimatedTableViewCell *)cell{
    return @[cell.estimatedTitleLabel];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSReadingBookmarkItem *)bookmarkItem{
    if (!cell.installed) {
        cell.editTopOffset = ITEM_IMAGE_TOP_MARGIN_NORMAL;
        cell.leftMargin = 9.f;
    }
    cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
    if (!cell.rightView) {
        FSItem *moreItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
        moreItem.accessibilityLabel = FSLocalizedForKey(@"kMore");
        cell.rightView = moreItem;
    }
    
    FSItem *moreItem = (FSItem *)cell.rightView;
    @weakify(self)
    @weakify(cell)
    moreItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        @strongify(cell)
        NSIndexPath *indexPath = cell.indexPath;
        NSMutableArray *items = @[].mutableCopy;
        
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kRename") image:FSImageNamedAndNormalColor(@"common_rename") action:^(FSPopoverItem * _Nonnull item, int index) {
            [self renameBookmarkWithIndex:indexPath.row];

        }]];
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            [self->_arrayBookmarks removeObject:bookmarkItem];
            [self.searchResultSoruce removeObject:bookmarkItem];
            [self->_pdfViewCtrl.currentDoc removeReadingBookmark:bookmarkItem.bookmark];
            
            [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [self reloadEmptyDataSet];
            if (self.bookmarkDeleteHandler) {
                self.bookmarkDeleteHandler();
            }
        }]];
        
        if ([self fs_currentSceneIsCompact]) [cell setHighlighted:YES color:self.fs_backgroundColor_B2 custom:YES];
        bookmarkItem.isShowMore = YES;
        [self->_extensionsManager.viewCtrlManager showMoreMenuWithItems:items.copy dismissCompletion:^{
            [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
            bookmarkItem.isShowMore = NO;
        } sourceView:![self fs_currentSceneIsCompact] ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return _item;
        } : nil];
    };
    
    cell.estimatedTitleLabel.text = [bookmarkItem title];
    cell.estimatedTitleLabel.accessibilityIdentifier = @"page_bookmark_id";
    
    moreItem.enabled = YES;
    if ([FSPermissionUtilty canAssemble:_extensionsManager]) {
        cell.rightView.hidden = self.enteredSearch;
    } else {
        cell.rightView.hidden = NO;
        moreItem.enabled = NO;
    }
    cell.editDragItemImage = FSImageNamed(@"panel_outline_more");
    [super configureCell:cell atIndexPath:indexPath withDataItem:bookmarkItem];
    if ([self fs_currentSceneIsCompact]) [cell setHighlighted:bookmarkItem.isShowMore color:self.fs_backgroundColor_B2 custom:YES];
    else [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    BOOL emptyDataSetShouldDisplay = [super emptyDataSetShouldDisplay:tableView];
    [self.panel setEditEnable:!emptyDataSetShouldDisplay];
    if (emptyDataSetShouldDisplay) {
        if (self.editing) {
            self.editing = NO;
        }
        if (!self.searchBarContentView.hidden && !self.keywords.length) {
            [self setSearchBarHidden:YES];
        }
    }
    return emptyDataSetShouldDisplay;
}

- (NSString *)titleForEmptyDataSet{
    if (self.keywords.length) return FSLocalizedForKey(@"kNoResultsTip");
    return FSLocalizedForKey(@"kNoBookmarks");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    if (self.keywords.length) return FSImageNamedAndPrimaryColor(@"search_result_no_data");
    return FSImageNamedAndPrimaryColor(@"panel_bookmark_no_data_gray");
}

#pragma mark - View lifecycle

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager panelController:(FSPanelController *)panelController {
    self = [super initWithServices:extensionsManager.viewCtrlManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _panelController = panelController;
        self.arrayBookmarks = [[NSMutableArray alloc] init];
        _bookmarkGotoPageHandler = nil;
        _bookmarkSelectionHandler = nil;

        self.moreIndexPath = nil;

        [panelController registerPanelChangedListener:self];

    }
    return self;
}

- (void)dealloc {
    self.arrayBookmarks = nil;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.placeholder = FSLocalizedForKey(@"kSearch");
    self.cancelTitle = FSLocalizedForKey(@"kCancel");
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kSelectAll") forState:UIControlStateNormal];
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
    [self.editRightItem setTitle:@"" forState:UIControlStateNormal];
    [self.editRightItem setImage:FSImageNamedAndNormalColor(@"common_delete_gray") forState:UIControlStateNormal];
    self.editRightItem.accessibilityLabel = FSLocalizedForKey(@"kDelete");
    self.cellSelectedColor = UIColor.clearColor;
}

/*
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return YES;
}
*/

- (void)onPanelChanged:(BOOL)isHidden {
    if (isHidden) {
        if (self.cancelItem.superview) {
            [self clickedSearchCancelItem];
        }

        self.editing = NO;
    }
}

- (void)hideCellEditView{
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editing) {
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
        return;
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    FSReadingBookmarkItem *bookmarkItem = [_arrayBookmarks objectAtIndex:indexPath.row];
    if (self.isContentEditing) {
        [tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
        if (self.bookmarkSelectionHandler) {
            self.bookmarkSelectionHandler();
        }
    } else {
        _panelController.isHidden = YES;
        [_pdfViewCtrl gotoPage:[bookmarkItem pageIndex] animated:NO];
    }
    [self clickedSearchCancelItem];
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(self.arrayBookmarks.count > 1){
        return self.panel.rightItem.enabled;
    }else{
        return NO;
    }
    
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    if (sourceIndexPath.row == destinationIndexPath.row) {
        return;
    }

    FSReadingBookmarkItem *sourceBookmark = [_arrayBookmarks objectAtIndex:sourceIndexPath.row];
    NSString *title = sourceBookmark.title;
    int pageIndex = sourceBookmark.pageIndex;
    [[_pdfViewCtrl currentDoc] removeReadingBookmark:sourceBookmark.bookmark];
    [[_pdfViewCtrl currentDoc] insertReadingBookmark:(int)destinationIndexPath.row title:title dest_page_index:pageIndex];
    [self.extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
        [(ReadingBookmarkPanel *)[ext.panelController getPanel:FSPanelTypeReadingBookmark] reloadData];
    }];
    _extensionsManager.isDocModified = YES;
}


#pragma mark status bar

- (void)renameBookmarkWithIndex:(NSInteger)index {
    FSReadingBookmarkItem *bookmark = [_arrayBookmarks objectAtIndex:index];
    [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kRenameBookmark") message:FSLocalizedForKey(@"kInputNewName") textFieldText:[bookmark title] actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
        if (index == 1) {
            if ([textField.text compare:[bookmark title]] != NSOrderedSame) {
                [bookmark setTitle:textField.text];
                [self renameBookmark:bookmark];
            }
        }
    }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
        if (index == 1) {
            return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length && textField.text.hash != [bookmark title].hash;
        }
        return YES;
    }] fs_showByViewController:self];
}

#pragma mark <TSAlertViewDelegate>
- (void)clearData:(BOOL)fromPDF;
{
    if (fromPDF) {
        for (FSReadingBookmarkItem *item in _arrayBookmarks) {
            [_pdfViewCtrl.currentDoc removeReadingBookmark:item.bookmark];
        }
    }

    [_arrayBookmarks removeAllObjects];
    [self reloadData];
}

- (NSUInteger)getBookmarkCount {
    return [_arrayBookmarks count];
}

- (void)renameBookmark:(FSReadingBookmarkItem *)renameBookmark {
    NSUInteger rowIndex = [_arrayBookmarks indexOfObject:renameBookmark];
    [self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:rowIndex inSection:0]] withRowAnimation:UITableViewRowAnimationNone];
    ((UIExtensionsManager*)(_pdfViewCtrl.extensionsManager)).isDocModified = YES;
}
@end
