/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "RedactAnnotHandler.h"
#import "FSUndoGroup.h"
#import "AudioPlayerControl.h"
#import "FSPermissionUtilty.h"

@implementation RedactAnnotHandler {
    __weak FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super initWithUIExtensionsManager:extensionsManager];
    if (self) {
        _taskServer = extensionsManager.taskServer;
        self.annotBorderEdgeInsets = UIEdgeInsetsMake(-5, -5, -5, -5);
        self.annotSelectionEdgeInsets = UIEdgeInsetsMake(0, 0, 0, 0);
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotRedact;
}

- (BOOL)canMoveAnnot:(FSAnnot *)annot {
    if (![self.extensionsManager annotsPermissionCanModify:annot]) {
        return NO;
    }
    FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
    return !redact.quads.count && redact.canMove;
}

- (BOOL)canResizeAnnot:(FSAnnot *)annot {
    if ([self.extensionsManager annotsPermissionCanModify:annot]) {
        FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
        return !redact.quads.count && redact.canResize;
    }
    return NO;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionOpen;
    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    
    if (self.extensionsManager.isSupportRedaction && [FSPermissionUtilty canApplyRedaction:self.extensionsManager]) {
        if ([self.extensionsManager annotsPermissionCanModify:annot]) {
            options |= FSMenuOptionApplyRedaction;
        }
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionNone;
    }
    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options |= FSMenuOptionStyle;
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    if (annot.canReply && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        if ([self.extensionsManager annotsPermissionCanReply:annot]) {
            options |= FSMenuOptionReply;
        }
    }
    return options;
}

- (void)applyRedaction{
    UIAlertController *alert = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRedactTip") actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
        if (index == 1) {
            FSRedact *annot = [[FSRedact alloc] initWithAnnot:self.extensionsManager.currentAnnot];
            [self.extensionsManager setCurrentAnnot:nil];
            int pageIndex = annot.pageIndex;
            [Utility applyRedactAnnot:annot completed:^(NSArray<FSAnnotAttributes *> * _Nonnull replyAttributes) {
                [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAttachmentLsitUpdate object:nil];
                [self.pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex needRender:YES clearCache:YES];
                self.extensionsManager.isDocModified = YES;
                [self.extensionsManager clearUndoRedo];
            }];
        }else{
            [self showMenu];
        }
    }];
    [self.pdfViewCtrl.fs_viewController presentViewController:alert animated:YES completion:nil];
}

- (BOOL)removeAnnot:(FSAnnot *)annot {
    return [super removeAnnot:annot addUndo:self.extensionsManager.isSupportRedaction];
}

- (void)fixInvalidFontForFreeText:(FSAnnot *)annot {
    FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
    FSDefaultAppearance *ap = [redact getDefaultAppearance];
    if (!ap.font || [ap.font isEmpty] || ap.text_size == 0) {
        if (!ap.font || [ap.font isEmpty]) {
            FSFont *font = [[FSFont alloc] initWithFont_id:(FSFontStandardID)[Utility toStandardFontID:self.extensionsManager.config.defaultSettings.annotations.redaction.textFace]];
            ap.font = font;
        }
        if (ap.text_size == 0) {
            ap.text_size = self.extensionsManager.config.defaultSettings.annotations.redaction.textSize;
        }
        ap.text_color = self.extensionsManager.config.defaultSettings.annotations.redaction.textColor.rgbValue;
        ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
        [redact setDefaultAppearance:ap];
    }
}

- (void)onAnnotSelected:(FSAnnot *)annot {
    [self fixInvalidFontForFreeText:annot];
    [super onAnnotSelected:annot];
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *_Nullable)annot{
    [super onDraw:pageIndex inContext:context annot:annot];
    if (pageIndex == annot.pageIndex && [Utility isAnnot:annot uniqueToAnnot:self.annot]) {
        FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
        NSMutableArray *quadPoints = [[NSMutableArray alloc] initWithCapacity:[redact.quadPoints getSize]];
        for (int i = 0; i < [redact.quadPoints getSize]; i++) {
            [quadPoints addObject:[redact.quadPoints getAt:i]];
        }
        
        if (!quadPoints.count) {
            CGRect fillRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:redact.fsrect pageIndex:annot.pageIndex];
            CGContextAddRect(context, fillRect);
            [[UIColor colorWithRGB:redact.applyFillColor] set];
            CGContextFillPath(context);
            return;
        }
        
        [quadPoints enumerateObjectsUsingBlock:^(FSQuadPoints *quad, NSUInteger idx, BOOL * _Nonnull stop) {
            FSRectF *dibRect = [Utility convertToFSRect:quad.first p2:quad.fourth];
            CGRect fillRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:annot.pageIndex];
            CGContextAddRect(context, fillRect);
            [[UIColor colorWithRGB:redact.applyFillColor] set];
            CGContextFillPath(context);
        }];
    }
}

- (void)resetProperties{
    FSRedact *annot = [[FSRedact alloc] initWithAnnot:self.extensionsManager.currentAnnot];
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    
    FSDefaultAppearance *appearance = [annot getDefaultAppearance];
    int fontColor = 0;
    float fontSize = 12.f;
    FSFont *font = [appearance getFont];
    NSString *fontName = @"Courier";
    if (font && ![font isEmpty]) {
        fontName = [font getName];
        fontSize = [appearance getText_size];
        fontColor = [appearance getText_color];
    }
    
    [propertyMainView setFillColor:annot.applyFillColor];
    [propertyMainView setPropertyColor:fontColor];
    [propertyMainView setText:annot.overlayText];
    [propertyMainView setFontName:fontName];
    [propertyMainView setFontSize:fontSize];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];
    [propertyMainView resetProperties:@[@(FSPropertyTypeBackGroundColor), @(FSPropertyTypeTextColor), @(FSPropertyTypeTextRedact), @(FSPropertyTypeFontName), @(FSPropertyTypeFontSizeFixed)]];
}


- (BOOL)isHitAnnot:(FSAnnot *)annot point:(FSPointF *)point {
    if (annot.type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]]) {
        return NO;
    }
    UIImage *annotDotImage;
    CGRect pvRect = [self getAnnotRect:annot];
    pvRect = UIEdgeInsetsInsetRect(pvRect, self.annotSelectionEdgeInsets);
    if (!annotDotImage) annotDotImage = FSImageNamed(@"annotation_drag");
    pvRect = CGRectInset(pvRect, -(annotDotImage.size.width+10), -(annotDotImage.size.height+10));
    CGPoint pvPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:point pageIndex:annot.pageIndex];
    if (CGRectContainsPoint(pvRect, pvPoint)) {
        return YES;
    }
    return NO;
}

- (BOOL)addAnnots:(NSArray<FSAnnot *> *)annots addUndo:(BOOL)addUndo{
    NSMutableArray<FSAnnotAttributes*> *attributesArray = [NSMutableArray array];
    NSMutableArray<NSNumber*> *allPageArray = [NSMutableArray array];
    NSMutableArray<NSNumber*> *singlePageArray = [NSMutableArray array];
    for (FSAnnot *annot in annots) {
        FSPDFPage *page = [annot getPage];
        
        if (!page && [page isEmpty]) {
            continue;
        }
        BOOL bFound = NO;
        for (NSNumber *number in singlePageArray) {
            if ([number isEqualToNumber:[NSNumber numberWithInt:[page getIndex]]]) {
                bFound = YES;
                break;
            }
        }
        if (!bFound) {
            [singlePageArray addObject:[NSNumber numberWithInt:[page getIndex]]];
        }
        
        [self.extensionsManager addPopupWithAnnot:annot];
        
        
        [allPageArray addObject:[NSNumber numberWithInt:[page getIndex]]];
        FSAnnotAttributes *attributes = [FSAnnotAttributes attributesWithAnnot:annot];
        [attributesArray addObject:attributes];
        
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        for (NSNumber *number in singlePageArray) {
            [self.pdfViewCtrl refresh:[number intValue] needRender:YES];
        }
    });

    if (addUndo) {
        
        [self.extensionsManager addUndoItem:[FSUndoGroup itemForUndoAddRdactAnnotsWithAttributes:attributesArray pageArray:allPageArray annotHandler:self curDoc:[self.pdfViewCtrl getDoc]]];
    }
    [self.extensionsManager onAnnotsAdded:annots];
    
    return YES;
}

- (BOOL)removeAnnots:(NSArray<FSAnnot*> *)annots addUndo:(BOOL)addUndo;{
    [self hideMenu];
    if (self.extensionsManager.propertyMainView.isShowing) {
        [self.extensionsManager.propertyMainView dismiss];
    }

    [self.extensionsManager onAnnotsWillDelete:annots];
    NSMutableArray<FSAnnotAttributes*> *attributesArray = [NSMutableArray array];
    NSMutableArray<NSNumber*> *allPageArray = [NSMutableArray array];
    NSMutableArray<NSNumber*> *singlePageArray = [NSMutableArray array];
    
    BOOL ret = NO;
    BOOL bCloseAudio = NO;
    [self.pdfViewCtrl lockRefresh];
    for (FSAnnot *annot in annots) {
        FSPDFPage *page = [annot getPage];
        if (!page || [page isEmpty]) {
            continue;
        }
        
        BOOL bFound = NO;
        for (NSNumber *number in singlePageArray) {
            if ([number isEqualToNumber:[NSNumber numberWithInt:[page getIndex]]]) {
                bFound = YES;
                break;
            }
        }
        if (!bFound) {
            [singlePageArray addObject:[NSNumber numberWithInt:[page getIndex]]];
        }
        
        int pageIndex = annot.pageIndex;
        CGRect rect = [self getAnnotRect:annot];
        rect = UIEdgeInsetsInsetRect(rect, self.annotRefreshRectEdgeInsets);
        rect = CGRectIntersection(rect, [self.pdfViewCtrl getPageView:pageIndex].bounds);



        AudioPlayerControl *audioPlay = [AudioPlayerControl sharedControl];
        NSString *medioFilePath = [Utility getDocumentRenditionTempFilePath:annot.ha_fileSpec PDFPath:self.extensionsManager.pdfViewCtrl.filePath];
        NSString *playFilePath = audioPlay.filePath.path;
        if ([medioFilePath isEqualToString:playFilePath]) {
            bCloseAudio = YES;
        }

        [self.extensionsManager deletePopupWithAnnot:annot];

        [allPageArray addObject:[NSNumber numberWithInt:[page getIndex]]];
        FSAnnotAttributes *attributes = [FSAnnotAttributes attributesWithAnnot:annot];
        [attributesArray addObject:attributes];
        
        NSArray *replyAnnots = annot.replyAnnots;
        if (replyAnnots.count) {
            FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
            [markup removeAllReplies];
        }
        ret = [page removeAnnot:annot];
        
    }
    [self.pdfViewCtrl unlockRefresh];

    if (bCloseAudio){
        AudioPlayerControl *audioPlay = [AudioPlayerControl sharedControl];
        [audioPlay dismiss];
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        for (NSNumber *number in singlePageArray) {
            [self.pdfViewCtrl refresh:[number intValue] needRender:YES];
        }
        
        [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
    });

    if (ret) {
        
        if (addUndo) {
            
            [self.extensionsManager addUndoItem:[FSUndoGroup itemForUndoDeleteRdactAnnotsWithAttributes:attributesArray pageArray:allPageArray annotHandler:self curDoc:[self.pdfViewCtrl getDoc]]];
        }
        
        self.attributesBeforeModify = nil;
        self.annotImage = nil;
//        self.annot = nil;
    }
    return ret;

}


@end
