/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "RedactModule+Private.h"
#import "RedactToolHandler.h"
#import "RedactAnnotHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>
#import "UIExtensionsConfig.h"
#import "MenuView+Private.h"
#import "FSRedactThumbnailViewController.h"
#import "UIExtensionsManager+Private.h"
#import "SearchModule.h"
#import "FSPermissionUtilty.h"
#import "PageNavigationModule.h"

@interface RedactModule ()<IDocEventListener, MoreItemActionProtocol,IGestureEventListener, IStateChangeListener, IPageEventListener, IModule, IFullScreenListener,IWindowFrameChangedListener, UIViewAdapter, IViewAppleEventListener> {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSAnnotType _annotType;
}

@property(nonatomic, strong) FSToolbar *topToolBar;
@property(nonatomic, strong) FSToolbar *bottomToolBar;
@property(nonatomic, assign) BOOL toolBarHiden;
@property(nonatomic, strong) RedactToolHandler* toolHandler;
@property(nonatomic, strong) RedactAnnotHandler* annotHandler;
@property(nonatomic, strong) FSItem *redactItem;
@property(nonatomic, strong) FSItem *applyAllItem;
@property(nonatomic, strong) MoreMenuItem *moreRedactItem;
@property(nonatomic, strong) FSItem *titleItem;
@property(nonatomic, strong) FSReadToolbarItem *redactToolbarItem;
@end

@implementation RedactModule

- (NSString *)getName {
    return @"Redact";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _toolBarHiden = YES;
        _bCanRedact = NO;
        [self loadModule];

        [_extensionsManager registerWindowFrameChangedListener:self];
        [_extensionsManager registerAnnotPropertyListener:self];
    }
    return self;
}

- (void)dealloc
{
    [_extensionsManager unregisterWindowFrameChangedListener:self];
    [_extensionsManager unregisterStateChangeListener:self];
    [_extensionsManager unregisterToolHandler:self.toolHandler];
    [_extensionsManager unregisterAnnotHandler:self.annotHandler];
    [_extensionsManager unregisterFullScreenListener:self];
    
    [_pdfViewCtrl unregisterGestureEventListener:self];
    [_pdfViewCtrl unregisterDocEventListener:self];
    [_pdfViewCtrl unregisterPageEventListener:self];
    [_pdfViewCtrl unregisterViewAppleEventListener:self];
}

- (FSReadToolSettings *)getToolSettings{
    return [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeRedact];
}

- (void)setToolSettings:(FSReadToolSettings *)toolSettings{
    _extensionsManager.defaultReadToolSettingItems[@(FSReadToolSettingsTypeRedact)] = toolSettings;
}

- (void)loadModule {

    self.toolHandler = [[RedactToolHandler alloc] initWithUIExtensionsManager:_extensionsManager];
    [_extensionsManager registerToolHandler:self.toolHandler];
    self.annotHandler = [[RedactAnnotHandler alloc] initWithUIExtensionsManager:_extensionsManager];
    [_extensionsManager registerAnnotHandler:self.annotHandler];
    
    [_pdfViewCtrl registerViewAppleEventListener:self];
    
    [_pdfViewCtrl registerGestureEventListener:self];
    [_extensionsManager registerStateChangeListener:self];
    [_pdfViewCtrl registerDocEventListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager registerFullScreenListener:self];
    
    
    MoreMenuGroup *group = [_extensionsManager.more getGroup:TAG_GROUP_PROTECT];
    if (!group) {
        group = [[MoreMenuGroup alloc] init];
        group.tag = TAG_GROUP_PROTECT;

        group.title = FSLocalizedForKey(@"kProtect");
        [_extensionsManager.more addGroup:group];
    }
    self.moreRedactItem = [[MoreMenuItem alloc] init];
    self.moreRedactItem.tag = TAG_ITEM_REDACTION;
    self.moreRedactItem.title = FSLocalizedForKey(@"kRedaction");
    self.moreRedactItem.callBack = self;
    self.moreRedactItem.image = FSImageNamedAndNormalColor(@"more_redact_redaction");
    [_extensionsManager.more addMenuItem:TAG_GROUP_PROTECT withItem:self.moreRedactItem];
    
    _redactToolbarItem = [[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeRedact withItemContentType:FSItemContentTypeImageAndText];
    [_extensionsManager.toolbarManager addReadItem:_redactToolbarItem atToolTag:FSMainTopbarToolTagProtect];
    
    [self initToolBar];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [self onClick:self.moreRedactItem];
}

- (void)enterRedactionMode:(BOOL)flag
{
    if (flag) {

        dispatch_async(dispatch_get_main_queue(), ^{
            if (DEVICE_iPAD) {
                self->_extensionsManager.panelController.isHidden= YES;
            }
        });

    } else {
        
        [self setToolBarHiden:YES animation:YES];
        [self stopRedact];

        if (_extensionsManager.currentState == FSUIManagerStateRedaction) {
            [_extensionsManager changeState:FSUIManagerStateNormal];
        }

    }

    [_pdfViewCtrl layoutIfNeeded];
}

- (void)initToolBar{
    
    [self arrangeTopToolBar];
    [self arrangeBottomBar];
    
}

-(FSToolbar *)topToolBar{
    if (!_topToolBar) {
        
        _topToolBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        _topToolBar.tag = TOPBAR_PAGESLIDER_TAG;
        StateBarThemeKeyPath(_topToolBar.stateBar, backgroundColor);
        [BindThemeKeySel(_topToolBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
            StateBarThemeKeyPath(target, backgroundColor);
        }];
        [_extensionsManager.containerView addSubview:_topToolBar];
        [_topToolBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.equalTo(_pdfViewCtrl);
            make.bottom.equalTo(_pdfViewCtrl.mas_top);
        }];
        _topToolBar.hidden = YES;
    }
    return _topToolBar;
}

- (FSToolbar *)bottomToolBar{
    if (!_bottomToolBar) {
        _bottomToolBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        [_extensionsManager.containerView addSubview:_bottomToolBar];
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            [self.bottomToolBar mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(_pdfViewCtrl.mas_left).offset(0);
                make.right.equalTo(_pdfViewCtrl.mas_right).offset(0);
                make.top.equalTo(_pdfViewCtrl.mas_bottom).offset(0);
            }];
            _bottomToolBar.hasDivide = YES;
        }else{
            [_bottomToolBar mas_makeConstraints:^(MASConstraintMaker *make) {
                make.top.equalTo(_pdfViewCtrl.mas_bottom);
                make.centerX.equalTo(_pdfViewCtrl);
                make.width.mas_equalTo(208);
            }];
            _bottomToolBar.hasDivide = NO;
            _bottomToolBar.layer.cornerRadius = 6;
            _bottomToolBar.clipsToBounds = YES;
        }
    }
    _bottomToolBar.hidden = ![_extensionsManager annotsPermissionCanAdd];
    return _bottomToolBar;
}

- (void)arrangeTopToolBar{
    
    FSItem *leftItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
    [leftItem setTitleColor:self.fs_primaryColor forState:UIControlStateNormal];
    [BindThemeKeySel(leftItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    @weakify(self)
    leftItem.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        [self cancelRedaction];
    };
    [self.topToolBar addSubview: leftItem];
    [leftItem mas_makeConstraints:^(MASConstraintMaker *make) {

        make.left.mas_equalTo(leftItem.superview).offset(10);
        make.centerY.mas_equalTo(leftItem.superview);
    }];
    
    FSItem *titleItem = [FSItem createHeadlineItemWithTitle:FSLocalizedForKey(@"kRedaction")];
    [titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateDisabled];
    titleItem.enabled = NO;
    [self.topToolBar addSubview:titleItem];
    self.titleItem = titleItem;
    self.titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
    self.titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
    titleItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    [titleItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(titleItem.superview);
    }];
    
    self.applyAllItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kApplyAll")];
    [self.applyAllItem setTitleColor:self.fs_primaryColor forState:UIControlStateNormal];
    [BindThemeKeySel(self.applyAllItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    self.applyAllItem.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        [self applyAllRedact];
    };
    [self.topToolBar addSubview: self.applyAllItem];
    [self.applyAllItem mas_makeConstraints:^(MASConstraintMaker *make) {

        make.right.mas_equalTo(self.applyAllItem.superview).offset(-10);
        make.centerY.mas_equalTo(self.applyAllItem.superview);
    }];
}

- (void)applyAllRedact{
    
    FSPDFDoc *doc = [self->_extensionsManager.pdfViewCtrl getDoc];
    if (![self pdfDocHasRedactAnnote:doc]) {
        [self fs_toastWithMsg:FSLocalizedForKey(@"kRedactNoAnnote") inView:[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view];
        return;
    }
    
    if (self->_extensionsManager.currentAnnot) {
        [self->_extensionsManager setCurrentAnnot:nil];
    }
    
    @weakify(self)
    UIAlertController *alert = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRedactTip") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
          if (index == 1){
              
              @strongify(self)

              FSRedaction *redaction = [[FSRedaction alloc] initWithDocument:[self->_extensionsManager.pdfViewCtrl getDoc]];
              [redaction apply];
              
              [self stopRedact];
              self->_extensionsManager.isDocModified = YES;
              [self->_extensionsManager.pdfViewCtrl clearRenderCache];
              [self->_extensionsManager.pdfViewCtrl refresh];
              [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
              [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAttachmentLsitUpdate object:nil];
              [self->_extensionsManager clearUndoRedo];
          }
      }];
      [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alert animated:YES completion:nil];

}

- (BOOL)pdfDocHasRedactAnnote:(FSPDFDoc*)pdfDoc{
    int pageCount = [pdfDoc getPageCount];
    int annoteCount = 0;
    
    if (pdfDoc && ![pdfDoc isEmpty]) {

        for (int i=0; i<pageCount; i++) {
            FSPDFPage *pdfPage = [pdfDoc getPage:i];
            annoteCount = [pdfPage getAnnotCount];
            for (int i=0; i<annoteCount; i++) {
                FSAnnot* annote= [pdfPage getAnnot:i];
                if (annote.getType == FSAnnotRedact) {
                    return YES;
                }
                
            }

        }
    }
    
    return NO;
}

- (void)arrangeBottomBar{
    
    BOOL bLoadSearch = _extensionsManager.config.loadSearch;
    
    self.redactItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"more_redact_redaction_left")];
    self.redactItem.accessibilityLabel = FSLocalizedForKey(@"kRedactTextVoiceOver");
    self.redactItem.layer.cornerRadius = 2;
    self.redactItem.clipsToBounds = YES;
    @weakify(self)
    __weak FSItem *leftItem = self.redactItem;
    leftItem.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        leftItem.selected = !leftItem.selected;
        self.bCanRedact = leftItem.selected;
        if (leftItem.selected) {
            leftItem.backgroundColor = self.fs_dividerColor_P2;
            [self startRedact];
            [self fs_toastWithMsg:FSLocalizedForKey(@"kSelectRedactText") inView:[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view];
        }else{
            [self stopRedact];
        }
        
    };
    [self.bottomToolBar addSubview: leftItem];
    
    
    FSItem *midItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"more_redact_redaction_mid")];
    midItem.accessibilityLabel = FSLocalizedForKey(@"kRedactPageVoiceOver");
    midItem.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        [self stopRedact];
        FSRedactThumbnailViewController *redactPagesVC = [[FSRedactThumbnailViewController alloc] initWithServices:self->_extensionsManager.viewCtrlManager];
        FSNavigationController *thumbnailNav = [[FSNavigationController alloc] initWithRootViewController:redactPagesVC];
        [self->_extensionsManager.viewCtrlManager presentViewController:thumbnailNav animated:YES completion:nil];
    };
    [self.bottomToolBar addSubview: midItem];
    [midItem mas_makeConstraints:^(MASConstraintMaker *make) {
        if (bLoadSearch) {
            make.centerX.equalTo(midItem.superview);
        }else{
            make.centerX.equalTo(midItem.superview).offset(30);
        }
        make.centerY.mas_equalTo(midItem.superview);
    }];
    
    [leftItem mas_makeConstraints:^(MASConstraintMaker *make) {
        if (bLoadSearch) {
            make.right.mas_equalTo(midItem.mas_left).offset(-10);
        }else{
            make.centerX.equalTo(leftItem.superview).offset(-30);
        }
        
        make.centerY.mas_equalTo(leftItem.superview);
    }];
    
    if (bLoadSearch) {
        FSItem *rightItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"more_redact_redaction_right")];
        rightItem.accessibilityLabel = FSLocalizedForKey(@"kRedactSearchVoiceOver");
        rightItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self showSearchBar];
        };
        [self.bottomToolBar addSubview: rightItem];
        [rightItem mas_makeConstraints:^(MASConstraintMaker *make) {

            make.left.mas_equalTo(midItem.mas_right).offset(10);
            make.centerY.mas_equalTo(rightItem.superview);
        }];
    }
    

}

- (void)showSearchBar{
    SearchModule *module = (SearchModule *)[_extensionsManager getModuleByName:@"Search"];
    if (module) {
        module.isRedactChoose = YES;
    }
    
    [_extensionsManager changeState:FSUIManagerStateSearch];
}

- (void)setToolBarHiden:(BOOL)toolBarHiden{
    [self setToolBarHiden:toolBarHiden animation:YES];
}

- (void)setToolBarHiden:(BOOL)toolBarHiden animation:(BOOL)animation{
    if (_toolBarHiden == toolBarHiden) {
        return;
    }

    if (!toolBarHiden) {
        [self delayHideReactTopBottomToolBar];
    }
    _toolBarHiden = toolBarHiden;
    
    [self setTopToolBarHidden:toolBarHiden animated:YES];
    [self setBottomToolBarHidden:toolBarHiden animated:YES];
}

- (void)setTopToolBarHidden:(BOOL)hidden animated:(BOOL)animated{
    if (!hidden) {
        _topToolBar.hidden = hidden;
    }
    if (hidden) {
        
        [_topToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.equalTo(_pdfViewCtrl);
            make.bottom.equalTo(_pdfViewCtrl.mas_top);
        }];

    } else {
      
        [_topToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.mas_equalTo(_pdfViewCtrl);
        }];
    }
    if (animated) {
        [UIView animateWithDuration:0.3 animations:^{
            [self->_extensionsManager.pdfViewCtrl layoutIfNeeded];
        } completion:^(BOOL finished) {
            if (hidden) {
                _topToolBar.hidden = YES;
            }
        }];
    }else{
        [self->_extensionsManager.pdfViewCtrl layoutIfNeeded];
    }

}

- (void)setBottomToolBarHidden:(BOOL)hidden animated:(BOOL)animated{
    if (hidden) {
        
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            [self.bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(_extensionsManager.pdfViewCtrl.mas_left).offset(0);
                make.right.equalTo(_extensionsManager.pdfViewCtrl.mas_right).offset(0);
                make.top.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(0);
            }];
        }else{
            [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerX.equalTo(_pdfViewCtrl);
                make.width.mas_equalTo(208);
                make.top.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(0);
            }];
        }


    } else {
            _bottomToolBar.clipsToBounds = YES;
            if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                [self.bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(_extensionsManager.pdfViewCtrl.mas_left).offset(0);
                    make.right.equalTo(_extensionsManager.pdfViewCtrl.mas_right).offset(0);
                    make.bottom.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(0);
                    
                }];
                _bottomToolBar.layer.cornerRadius = 0;
            }else{

                [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.centerX.equalTo(_pdfViewCtrl);
                    make.width.mas_equalTo(208);
                    
                    if (@available(iOS 11.0, *)) {
                        make.bottom.mas_equalTo(_extensionsManager.pdfViewCtrl.mas_safeAreaLayoutGuideBottom).offset(-40);
                    } else {
                        make.bottom.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(-40);
                    }
                }];
                _bottomToolBar.layer.cornerRadius = 6;
            }

    }
    [self->_extensionsManager.pdfViewCtrl bringSubviewToFront:self.bottomToolBar];
    if (animated) {
        [UIView animateWithDuration:0.3 animations:^{
            [self->_extensionsManager.pdfViewCtrl layoutIfNeeded];
        } completion:^(BOOL finished) {
            [(PageNavigationModule *)[self->_extensionsManager getModuleByName:@"PageNavigation"] showPageNum:!hidden];
        } ];
    }else{
        [self->_extensionsManager.pdfViewCtrl layoutIfNeeded];
        [(PageNavigationModule *)[self->_extensionsManager getModuleByName:@"PageNavigation"] showPageNum:!hidden];
    }
}

- (void)reLayoutBottomBar{
    if (self.toolBarHiden) {
        
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            [self.bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(_extensionsManager.pdfViewCtrl.mas_left).offset(0);
                make.right.equalTo(_extensionsManager.pdfViewCtrl.mas_right).offset(0);
                make.top.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(0);
            }];
        }else{
            [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerX.equalTo(_pdfViewCtrl);
                make.width.mas_equalTo(208);
                make.top.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(0);
            }];
        }


    } else {
            _bottomToolBar.clipsToBounds = YES;
            if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                [self.bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(_extensionsManager.pdfViewCtrl.mas_left).offset(0);
                    make.right.equalTo(_extensionsManager.pdfViewCtrl.mas_right).offset(0);
                    make.bottom.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(0);
                    
                }];
                _bottomToolBar.layer.cornerRadius = 0;
            }else{

                [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.centerX.equalTo(_pdfViewCtrl);
                    make.width.mas_equalTo(208);
                    
                    if (@available(iOS 11.0, *)) {
                        make.bottom.mas_equalTo(_extensionsManager.pdfViewCtrl.mas_safeAreaLayoutGuideBottom).offset(-40);
                    } else {
                        make.bottom.equalTo(_extensionsManager.pdfViewCtrl.mas_bottom).offset(-40);
                    }
                }];
                _bottomToolBar.layer.cornerRadius = 6;
            }

        }

}

- (void)delayHideReactTopBottomToolBar{
    if (_extensionsManager.isFullScreen) {
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
        [self performSelector:@selector(autoFullScreen) withObject:nil afterDelay:5.f];
    }
    [_extensionsManager cancelAutoFullScreen];
    [_extensionsManager delayAutoFullScreen];
}

- (void)suspendHideToolBar{
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
}

- (void)autoFullScreen{
//    [self setToolBarHiden:YES animation:YES];
//    [_extensionsManager setFullScreen:YES];
}

- (void)cancelRedaction {
    
    [self stopRedact];
    
    [_extensionsManager setCurrentToolHandler:nil];
    [_extensionsManager changeState:FSUIManagerStateNormal];
    [_extensionsManager setFullScreen:NO];
}

- (void)startRedact{
    if (!_extensionsManager.isSupportRedaction) {

        [UIAlertController fs_alert_OK_WithTitle:@"kWarning" message:@"kFailedLoadModuleForLicenseTip" actionBack:nil onCurrentSceneResponder:self->_pdfViewCtrl];
        return;
    }
    self.bCanRedact = YES;
    [self suspendHideToolBar];
}

- (void)stopRedact{
    self.bCanRedact = NO;
    
    if (self.redactItem) {
        self.redactItem.backgroundColor = [UIColor clearColor];
        self.redactItem.selected = NO;
    }

    [self delayHideReactTopBottomToolBar];
}

- (void)updateReadItemState:(BOOL)enable
{
    if (self.redactToolbarItem) {
        self.redactToolbarItem.enabled = enable;
    }
    NSArray<FSMainTopbarToolTagItem *> *toolTagItemArray = _extensionsManager.topToolbar.tagItems;
    for (int i = 0; i < toolTagItemArray.count; i++) {
        FSMainTopbarToolTagItem *item = toolTagItemArray[i];
        NSArray<FSReadToolbarItem *> *readtoolItems = item.readToolbarItems;
        if (item.toolTag == FSMainTopbarToolTagHome) {
            for (FSReadToolbarItem *toolbarItem in readtoolItems) {
                if (toolbarItem.settingsType == FSReadToolSettingsTypeRedact) {
                    toolbarItem.enabled = enable;
                }
            }
        }
    }
}

#pragma mark -- <MoreItemActionProtocol>
- (void)onClick:(MoreMenuItem*)item{
    if (!_extensionsManager.isSupportRedaction) {
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailedLoadModuleForLicenseTip") actionBack:nil onCurrentSceneResponder:self->_pdfViewCtrl];
        return;
    }
    
    if (![FSPermissionUtilty canAddAnnot:_extensionsManager]) {
        FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kWarning"), FSLocalizedForKey(@"kRMSNoAccess"), self->_pdfViewCtrl);
        return;
    }
    
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionRedaction checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            [self->_extensionsManager changeState:FSUIManagerStateRedaction];
        }
    }];
    
    if (item.tag == TAG_ITEM_REDACTION) {
        [self->_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreProtectRedaction];
    }
}

- (void)resetTitleItem
{
    [self.applyAllItem mas_remakeConstraints:^(MASConstraintMaker *make) {

        make.right.mas_equalTo(self.applyAllItem.superview).offset(-10);
        make.centerY.mas_equalTo(self.applyAllItem.superview);
        if (_pdfViewCtrl.fs_width <= 320) {
            make.width.mas_lessThanOrEqualTo(70);
        }
    }];
    
    if (self.titleItem) {
        CGSize closeItemSize = [Utility getTextSize:FSLocalizedForKey(@"kClose") fontSize:15 maxSize:CGSizeMake(150, 30)];
        CGFloat titleItemWidth = (_pdfViewCtrl.fs_width/2 - 16*2 - closeItemSize.width)*2;
        [self.titleItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.mas_equalTo(self.titleItem.superview);
            make.size.mas_equalTo(CGSizeMake(titleItemWidth, 20));
        }];
    }
}

#pragma mark -- <IDocEventListener>
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error{
    _extensionsManager.isSupportRedaction = [FSLibrary hasModuleLicenseRight:FSModuleNameRedaction];
    
    if (self.applyAllItem) {
        self.applyAllItem.enabled = _extensionsManager.isSupportRedaction && [FSPermissionUtilty canApplyRedaction:_extensionsManager] && [_extensionsManager allAnnotPermission];
    }

    if (self.moreRedactItem) {
        BOOL canAdd = [_extensionsManager annotsPermissionCanAdd];
        self.moreRedactItem.enable = self.applyAllItem.enabled || canAdd;
        [self updateReadItemState:canAdd];
    }
}

- (void)onDocWillClose:(FSPDFDoc *)document
{
    if ([self->_extensionsManager getState] == FSUIManagerStateRedaction) {
        [self cancelRedaction];
    }
}

#pragma mark IGestureEventListener
- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    if (!_extensionsManager.isFullScreen) {
        return NO;
    }
    if ([_extensionsManager getState] == FSUIManagerStateRedaction) {
        if (_toolBarHiden) {
            [self setToolBarHiden:NO animation:YES];
            [_extensionsManager setFullScreen:NO];
        } else {
            [self setToolBarHiden:YES animation:YES];
             [_extensionsManager setFullScreen:YES];
        }
        return YES;
    }
    return NO;
}

- (BOOL)onPan:(UIPanGestureRecognizer *)recognizer {

    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (self.toolHandler != nil && self.bCanRedact) {
        if ([self.toolHandler onPageViewPan:pageIndex recognizer:recognizer]) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)onShouldBegin:(UIGestureRecognizer *)recognizer {
    
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (self.toolHandler != nil && self.bCanRedact) {
        if ([self.toolHandler respondsToSelector:@selector(onPageViewShouldBegin:recognizer:)]) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)onTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (self.toolHandler != nil && self.bCanRedact) {
        if ([self.toolHandler onPageViewTouchesBegan:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    }
    
    return NO;
}
- (BOOL)onTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (self.toolHandler != nil && self.bCanRedact) {
        if ([self.toolHandler onPageViewTouchesMoved:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)onTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
   
    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (self.toolHandler != nil && self.bCanRedact) {
        if ([self.toolHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)onTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (self.toolHandler != nil && self.bCanRedact) {
        if ([self.toolHandler onPageViewTouchesCancelled:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)gestureRecognizer {

    return YES;
}

#pragma mark IStateChangeListener

- (void)onStateChanged:(FSUIManagerState)state {
    if (state == FSUIManagerStateRedaction) {
        [self setToolBarHiden:NO animation:YES];
        [_extensionsManager setFullScreen:NO];

    }else{
        [self setToolBarHiden:YES animation:YES];
    }
}

#pragma mark IPageEventListener

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {

    if ([_extensionsManager getState] == FSUIManagerStateReflow && _extensionsManager.isFullScreen) {
        [self delayHideReactTopBottomToolBar];
    }
}

- (FSAnnot *)addAnnotToPage:(int)pageIndex withRect:(FSRectF *)rect{
    if (self.toolHandler) {
        return [self.toolHandler addAnnotToPage:pageIndex withRect:rect];
    }
    
    return nil;
}

- (FSAnnot *)addAnnotToPage:(int)pageIndex withRect:(FSRectF *)rect  needQuads:(BOOL)needQuads {
    if (self.toolHandler) {
        return [self.toolHandler addAnnotToPage:pageIndex withRect:rect needQuads:needQuads];
    }
    
    return nil;
}

#pragma mark -- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD && changed) {
        if (_extensionsManager.currentState == FSUIManagerStateRedaction) {
            [self reLayoutBottomBar];
        }
        [self delayHideReactTopBottomToolBar];
    }
}

- (void)onFullScreen:(BOOL)isFullScreen {
    if ([_extensionsManager getState] == FSUIManagerStateRedaction) {
        [_extensionsManager setStatusBarHidden:isFullScreen];
        [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
        [self resetPdfViewCtrlSafeArea];
        
        [self setToolBarHiden:isFullScreen animation:YES];
    }
}

- (void)resetPdfViewCtrlSafeArea{
    UIEdgeInsets insets = _extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets;
    if (UIEdgeInsetsEqualToEdgeInsets(insets, UIEdgeInsetsZero)) {
        insets = _pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets;
    }
    [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    _extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets = insets;
    if (UIEdgeInsetsEqualToEdgeInsets(insets,_pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets)) {
        insets = UIEdgeInsetsZero;
    }
    _extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets = insets;
}

#pragma mark -- IViewAppleEventListener
- (void)layoutSubviewsFromPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    if (pdfViewCtrl == _pdfViewCtrl) {
        [self resetTitleItem];
    }
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if ([self getToolSettings] == toolSettings) {
        
    }
}
 
@end
