/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReflowModule.h"
#import "PanelController+private.h"
#import "FSSettingBar+private.h"
#import "DigitalSignaturePanel.h"
#import "AnnotationPanel.h"
#import "PageNavigationModule.h"
#import "FSPermissionUtilty.h"
#import "SelectToolHandler.h"
#import "TextMKAnnotHandler.h"

@interface ReflowModule ()  <IAnnotEventListener>

@property (nonatomic, strong) FSItem *bookmarkItem;
@property (nonatomic, strong) FSItem *backItem;
@property (nonatomic, strong) UILabel *title;

@property (strong, nonatomic) FSToolbar *toolBarTopReflow;
@property (strong, nonatomic) FSToolbar *toolBarBottomReflow;
@property (nonatomic ,strong) FSItem *panelItem;
@property (strong, nonatomic) FSItem * biggerItem;
@property (strong, nonatomic) FSItem * smallerItem;
@property (strong, nonatomic) FSItem * showGraphItem;
@property (strong, nonatomic) FSItem * previousPageItem;
@property (strong, nonatomic) FSItem * nextPageItem;
@property (strong, nonatomic) FSItem* exitItem;

@property (assign, nonatomic) BOOL needShowImageForReflow;
@property (assign, nonatomic) BOOL isReflowBarShowing;
@property (assign, nonatomic) BOOL isReflowByClick;
@property (strong, nonatomic) NSMutableDictionary *oldItemHiddenStatus;
@property (nonatomic, assign) BOOL isSelfState;
@property (strong, nonatomic) UIPanGestureRecognizer *pan;
@end

@implementation ReflowModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;

    PDF_LAYOUT_MODE currentPageLayoutMode;
    PDF_LAYOUT_MODE oldPageLayoutMode;
}

- (NSString *)getName {
    return @"Reflow";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _oldItemHiddenStatus = @{}.mutableCopy;
        [self loadModule];
    }
    return self;
}

+ (ReflowPageManager *)getReflowManager {
    return reflowManager;
}

- (void)loadModule {
    
    [_pdfViewCtrl registerGestureEventListener:self];
    [_extensionsManager registerStateChangeListener:self];
    [_pdfViewCtrl registerDocEventListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager registerFullScreenListener:self];
    [_extensionsManager registerAnnotEventListener:self];
    [_extensionsManager.settingBar.viewSettingManager registerEventListener:self];
    [_extensionsManager.panelController registerPanelChangedListener:self];
    [_extensionsManager.thumbnailModule registerThumbnailEventListener:self];
    [_extensionsManager registerWindowFrameChangedListener:self];
    self.needShowImageForReflow = YES;
    _isReflowByClick = NO;
    reflowManager = [[ReflowPageManager alloc] initWithExtensionsManager:_extensionsManager];
    if (_extensionsManager.config.loadReadingBookmark) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(bookmarkUpdateNotificationButtonState:) name:FSBookmarkUpdateNotification object:nil];
    }
}

- (void)enterReflowMode:(BOOL)flag {
    if (flag) {
        currentPageLayoutMode = [_pdfViewCtrl getPageLayoutMode];
        oldPageLayoutMode = currentPageLayoutMode;
        if (oldPageLayoutMode == PDF_LAYOUT_MODE_REFLOW) {
            oldPageLayoutMode = PDF_LAYOUT_MODE_SINGLE;
        }
        [_pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_REFLOW];
        [self showReflowBar:YES animation:YES];
        _isReflowByClick = YES;
        if (!self.needShowImageForReflow) {
            [self.showGraphItem setImage:FSImageNamedAndNormalColor(@"reflow_graph_none") forState:UIControlStateNormal];
            self.showGraphItem.accessibilityLabel = FSLocalizedForKey(@"kImgHidVoiceOver");
            [_pdfViewCtrl setReflowMode:PDF_REFLOW_ONLYTEXT];
        } else {
            [self.showGraphItem setImage:FSImageNamedAndNormalColor(@"reflow_graph") forState:UIControlStateNormal];
            self.showGraphItem.accessibilityLabel = FSLocalizedForKey(@"kImgShVoiceOver");
            [_pdfViewCtrl setReflowMode:PDF_REFLOW_WITHIMAGE];
        }
        [_extensionsManager setFullScreen:NO];
        
        FSPDFViewCtrl *pdfViewCtrl = self->_pdfViewCtrl;
        int currentPage = [pdfViewCtrl getCurrentPage];
        FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:pdfViewCtrl.currentDoc page:currentPage];
        self.bookmarkItem.selected = !(!bookmark || [bookmark isEmpty]);
    } else {
        [self showReflowBar:NO animation:YES];
        [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:REFLOW selected:NO];
        currentPageLayoutMode = oldPageLayoutMode;
        if (currentPageLayoutMode == PDF_LAYOUT_MODE_UNKNOWN) {
            currentPageLayoutMode = PDF_LAYOUT_MODE_SINGLE;
        }
        if (!_isReflowByClick) {
            currentPageLayoutMode = PDF_LAYOUT_MODE_SINGLE;
        }
        [_pdfViewCtrl setPageLayoutMode:currentPageLayoutMode];
        [_extensionsManager changeState:FSUIManagerStateNormal];
    }
    self.biggerItem.enabled = ([_pdfViewCtrl getScale] < 5.f);
    self.smallerItem.enabled = ([_pdfViewCtrl getScale] > 1.0f);
    [self setPreviousAndNextBtnEnable];
    [_pdfViewCtrl layoutIfNeeded];
}

- (FSToolbar *)toolBarTopReflow {
    if (!_toolBarTopReflow) {
        
        _toolBarTopReflow = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        _toolBarTopReflow.tag = TOPBAR_PAGESLIDER_TAG;
        [_toolBarTopReflow setHidden:YES];
        StateBarThemeKeyPath(_toolBarTopReflow.stateBar, backgroundColor);
        [BindThemeKeySel(_toolBarTopReflow.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
            StateBarThemeKeyPath(target, backgroundColor);
        }];
        self.backItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
        self.backItem.font = self.fs_standardGeneralFont_GF2;
        [BindThemeKeySel(self.backItem, setTextColor:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        
        @fs_weakify(self)
        self.backItem.onTapClick = ^(FSItem *item) {
            @fs_strongify(self)
            self.needShowImageForReflow = YES;
            [reflowManager clear];
            [(SelectToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Select] deSelect];
            [self cancelButtonClicked];
            self.isSelfState = NO;
        };
        
        [_toolBarTopReflow addItem: self.backItem atPosition:FSItemPositionLeft];

        self.title = [[UILabel alloc] init];
        self.title.accessibilityTraits = UIAccessibilityTraitHeader; // assign to var to keep a strong reference, can't directly assign to weak property of self
        self.title.text = FSLocalizedForKey(@"kReflow");
        self.title.font = self.fs_standardGeneralFont_GF1;
        self.title.textColor = self.fs_textColor_T4;
        [_toolBarTopReflow addItem:self.title atPosition:FSItemPositionCenter];
        
        if (_extensionsManager.config.loadReadingBookmark) {
            self.bookmarkItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"read_bookmark_gray")];
            self.bookmarkItem.accessibilityLabel = FSLocalizedForKey(@"kAddBookMarkItemVoiceOver");
            [BindThemeKeySel(self.bookmarkItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull _item, SEL  _Nonnull sel) {
                [_item setImage:FSImageNamedAndPrimaryColor(@"read_bookmark_select") forState:UIControlStateSelected];
            }];
            self.bookmarkItem.enabled = [FSPermissionUtilty canAssemble:self->_extensionsManager];
            @fs_weakify(self)
            self.bookmarkItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                FSPDFViewCtrl *pdfViewCtrl = self->_pdfViewCtrl;
                int currentPage = [pdfViewCtrl getCurrentPage];
                FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:pdfViewCtrl.currentDoc page:currentPage];
                if (!bookmark || [bookmark isEmpty]) {
                    @try {
                        [pdfViewCtrl.currentDoc insertReadingBookmark:-1 title:[NSString stringWithFormat: FSLocalizedForKey(@"kPage"), currentPage + 1] dest_page_index:currentPage];
                    } @catch (NSException *exception) {
                        return;
                    }
                } else {
                    [pdfViewCtrl.currentDoc removeReadingBookmark:bookmark];
                }
                [[NSNotificationCenter defaultCenter] postNotificationName:FSBookmarkUpdateNotification object:self];
                [self->_extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
                    [ext.panelController reloadReadingBookmarkPanel];
                }];
                self->_extensionsManager.isDocModified = YES;
            };
            [_toolBarTopReflow addItem:self.bookmarkItem atPosition:FSItemPositionRight];
        }
        [_extensionsManager.containerView addSubview:_toolBarTopReflow];
        [_toolBarTopReflow mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.mas_equalTo(self->_pdfViewCtrl);
        }];
        _toolBarTopReflow.hidden = YES;
    }
    return _toolBarTopReflow;
}

- (FSToolbar *)toolBarBottomReflow {
    if (!_toolBarBottomReflow) {
        
        FSToolbar* bottomToolbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
//        if (DEVICE_iPAD) {
//            UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:_extensionsManager.settingBar.viewSettingManager action:@selector(moveBottombarWithRecognizer:)];
//            [bottomToolbar addGestureRecognizer:pan];
//        }
        
        [bottomToolbar setHidden:YES];
        bottomToolbar.hasDivide = NO;
        [_extensionsManager.containerView addSubview:bottomToolbar ];
        _toolBarBottomReflow = bottomToolbar;
        @weakify(self)
        
        _biggerItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"reflow_bigger")];
        _biggerItem.accessibilityIdentifier = @"reflow_bigger_id";
        _biggerItem.accessibilityLabel = FSLocalizedForKey(@"kIncSizeVoiceOver");
        _biggerItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canAutoHideTopBottomToolBar]) {
                [self->_extensionsManager delayHideTopBottomToolBar];
            }
            [self biggerClicked];
        };
        
        _smallerItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"reflow_smaller")];
        _smallerItem.accessibilityLabel = FSLocalizedForKey(@"kDecSizeVoiceOver");
        _smallerItem.accessibilityIdentifier = @"reflow_smaller_id";
        _smallerItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canAutoHideTopBottomToolBar]) {
                [self->_extensionsManager delayHideTopBottomToolBar];
            }
            [self smallerClicked];
        };
        
        _showGraphItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"reflow_graph")];
        _showGraphItem.accessibilityIdentifier = @"reflow_graph_id";
        _showGraphItem.accessibilityLabel = FSLocalizedForKey(@"kImgHidVoiceOver");
        _showGraphItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canAutoHideTopBottomToolBar]) {
                [self->_extensionsManager delayHideTopBottomToolBar];
            }
            [self showGraphClicked];
        };
        
        _previousPageItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"reflow_pre")];
        _previousPageItem.accessibilityLabel = FSLocalizedForKey(@"kPanZoomPreviousPageVoiceOver");
        _previousPageItem.accessibilityIdentifier = @"reflow_pre_id";
        _previousPageItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canAutoHideTopBottomToolBar]) {
                [self->_extensionsManager delayHideTopBottomToolBar];
            }
            [self previousPageClicked];
        };
        
        _nextPageItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"reflow_next")];
        _nextPageItem.accessibilityLabel = FSLocalizedForKey(@"kPanZoomNextPageVoiceOver");
        _nextPageItem.accessibilityIdentifier = @"reflow_next_id";
        _nextPageItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canAutoHideTopBottomToolBar]) {
                [self->_extensionsManager delayHideTopBottomToolBar];
            }
            [self nextPageClicked];
        };

        [self reLayoutBottomToolbar];
        self.needShowImageForReflow = YES;
    }
    return _toolBarBottomReflow;
}

- (void)showReflowBar:(BOOL)isShow animation:(BOOL)animation {
    if (self.toolBarBottomReflow.hidden != isShow) {
        return;
    }
    if (!isShow) {
        if (![self checkIsIphoneDevice]) {
            if (self.isSelfState) {
                [_extensionsManager setHiddenTopToolbar:YES];
            }
        }
        self.toolBarTopReflow.hidden = YES;
        self.toolBarBottomReflow.hidden  = YES;
    } else {
        self.toolBarBottomReflow.hidden  = NO;
        if (![self checkIsIphoneDevice]) {
            self.toolBarTopReflow.hidden = YES;
            [_extensionsManager setHiddenTopToolbar:NO];
            [self.toolBarBottomReflow.superview bringSubviewToFront:self.toolBarBottomReflow];
        }else{
            [_extensionsManager setHiddenTopToolbar:YES];
            [_extensionsManager setHiddenBottomToolbar:YES];
            self.toolBarTopReflow.hidden = NO;
        }
        self.bookmarkItem.hidden = ([_extensionsManager.permissionProvider checkPermission:FSFunctionPDFBookMark] == FSPermissionStateHide);
    }
    if (animation) {
        [Utility addAnimation:_toolBarTopReflow.layer type:_toolBarTopReflow.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:_toolBarTopReflow.hidden ? kCATransitionFromTop : kCATransitionFromBottom timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
        [Utility addAnimation:_toolBarBottomReflow.layer type:_toolBarBottomReflow.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:_toolBarBottomReflow.hidden ? kCATransitionFromBottom : kCATransitionFromTop timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
    }
    [(PageNavigationModule*)[_extensionsManager getModuleByName:@"PageNavigation"] showPageNum: isShow];
    if (![self checkIsIphoneDevice]) {
        [_extensionsManager setStatusBarHidden:_extensionsManager.currentIsFullScreen && _extensionsManager.topToolbar.hidden];
        [self.toolBarBottomReflow fs_setShadowColor:self.fs_dividerColor_P3];
    }else{
        [_extensionsManager setStatusBarHidden:_extensionsManager.currentIsFullScreen && _toolBarTopReflow.hidden];
        [self.toolBarBottomReflow fs_setShadowColor:nil];
    }
    
    [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    [_extensionsManager.settingBar.viewSettingManager resetPdfViewCtrlSafeArea];
    
    [(PageNavigationModule *)[_extensionsManager getModuleByName:@"PageNavigation"] layoutPdfViewContentInsets:NO];
}

#pragma buttonClickEvent-- - Click on the button below
- (void)bookmarkClicked{
    _extensionsManager.hiddenPanel = NO;
}

- (void)biggerClicked{
    [reflowManager clear];
    [(SelectToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Select] deSelect];
    float newZoom = [_pdfViewCtrl getScale] + 1;
    [_pdfViewCtrl setZoom:newZoom];
    self.biggerItem.enabled = (newZoom < 5);
    self.smallerItem.enabled = YES;
    [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:FITPAGE selected:NO];
    [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:FITWIDTH selected:NO];
}

- (void)smallerClicked{
    [reflowManager clear];
    [(SelectToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Select] deSelect];
    [_pdfViewCtrl setZoom:[_pdfViewCtrl getScale] - 1];
    self.smallerItem.enabled = ([_pdfViewCtrl getScale] > 1.f);
    self.biggerItem.enabled = YES;
    [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:FITPAGE selected:NO];
    [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:FITWIDTH selected:NO];
}

- (void)showGraphClicked{
    [reflowManager clear];
    [(SelectToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Select] deSelect];
    if (self.needShowImageForReflow) {
        [self.showGraphItem setImage:FSImageNamedAndNormalColor(@"reflow_graph_none") forState:UIControlStateNormal];
        self.showGraphItem.accessibilityLabel = FSLocalizedForKey(@"kImgHidVoiceOver");
        [_pdfViewCtrl setReflowMode:PDF_REFLOW_ONLYTEXT];
    } else {
        [self.showGraphItem setImage:FSImageNamedAndNormalColor(@"reflow_graph") forState:UIControlStateNormal];
        self.showGraphItem.accessibilityLabel = FSLocalizedForKey(@"kImgShVoiceOver");
        [_pdfViewCtrl setReflowMode:PDF_REFLOW_WITHIMAGE];
    }
    self.needShowImageForReflow = !self.needShowImageForReflow;
    if (UIAccessibilityIsVoiceOverRunning()) {
        UIAccessibilityPostNotification(UIAccessibilityAnnouncementNotification, self.showGraphItem);
    }
}

- (void)previousPageClicked {
    [_pdfViewCtrl gotoPrevPage:NO];
    [self setPreviousAndNextBtnEnable];
}

- (void)nextPageClicked {
    [_pdfViewCtrl gotoNextPage:NO];
    [self setPreviousAndNextBtnEnable];
}

- (void)cancelButtonClicked {
    [self enterReflowMode:NO];
    [self->_extensionsManager setFullScreen:NO ];
}

- (void)setPreviousAndNextBtnEnable {
    if ([_pdfViewCtrl getCurrentPage] <= 0) {
        self.previousPageItem.enabled = NO;
    } else {
        self.previousPageItem.enabled = YES;
    }

    if ([_pdfViewCtrl getCurrentPage] >= [_pdfViewCtrl getPageCount] - 1) {
        self.nextPageItem.enabled = NO;
    } else {
        self.nextPageItem.enabled = YES;
    }
}


- (void)bookmarkUpdateNotificationButtonState:(NSNotification *)not{
    FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:[_pdfViewCtrl currentDoc] page:[_pdfViewCtrl getCurrentPage]];
    self.bookmarkItem.selected = !(!bookmark || [bookmark isEmpty]);
}

#pragma mark IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error{
    [[UIApplication sharedApplication] beginReceivingRemoteControlEvents];
    if ([_pdfViewCtrl isDynamicXFA] && _isReflowByClick) {
        [self enterReflowMode:NO];
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        self.bookmarkItem.enabled = [FSPermissionUtilty canAssemble:self->_extensionsManager];
    });
}

#pragma mark -- IDocEventListener
- (void)onDocWillClose:(FSPDFDoc *)document{
    _isReflowByClick = NO;
}

#pragma mark -- IFullScreenListener
- (void)onFullScreen:(BOOL)isFullScreen {
    if ([_extensionsManager getState] == FSUIManagerStateReflow) {
        [self showReflowBar:!isFullScreen  animation:YES];
    }
}

#pragma IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    
    if ([_extensionsManager getState] == FSUIManagerStateReflow) {
        [_extensionsManager setFullScreen:!self.toolBarBottomReflow.hidden];
        return YES;
    }
    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

#pragma IStateChangeListener

- (void)onStateChanged:(FSUIManagerState)state {
    if (state == FSUIManagerStateReflow) {
       
//        if (_extensionsManager.currentAnnot) {
//            [_extensionsManager setCurrentAnnot:nil];
//        }
        if (_extensionsManager.topToolbar.currentTagItem.toolTag != FSUIManagerStateReflow) {
            [_extensionsManager.topToolbar switchToTag:FSMainTopbarToolTagView];
        }
        if ([self checkIsIphoneDevice]) {
            [_toolBarBottomReflow mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.right.bottom.mas_equalTo(self->_pdfViewCtrl);
            }];
        }else{
            [_toolBarBottomReflow mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.bottom.mas_equalTo(self.toolBarBottomReflow.superview.fs_mas_bottom).offset(-40);
                make.centerX.equalTo(self->_pdfViewCtrl);
                make.width.mas_equalTo(352);
            }];
        }
        
        self.isSelfState = YES;
        [self showReflowBar:YES animation:YES];
        self.biggerItem.enabled = ([_pdfViewCtrl getScale] < 5.0f);
        self.smallerItem.enabled = ([_pdfViewCtrl getScale] > 1.0f);
        [self setPreviousAndNextBtnEnable];
        _oldItemHiddenStatus = [_extensionsManager.panelController getItemHiddenStatus].mutableCopy;
        if (![_extensionsManager.panelController isPanelHidden:FSPanelTypeDigitalSignature]) {
            [_extensionsManager.panelController setPanelHidden:YES type:FSPanelTypeDigitalSignature];
        }
        if (![_extensionsManager.panelController isPanelHidden:FSPanelTypeAnnotation]) {
            [_extensionsManager.panelController setPanelHidden:NO type:FSPanelTypeAnnotation];
        }
        NSArray *curSpecs = [NSArray arrayWithArray:_extensionsManager.panelController.panel.specs];
        for (id<IPanelSpec> spec in curSpecs) {
            if([spec getType] == FSPanelTypeDigitalSignature){
                [_extensionsManager.panelController.panel removeSpec:spec];
            }
        }
        [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:REFLOW selected:YES];
        _panelItem.enabled = _extensionsManager.panelController.panel.specs.count > 0;
    }else{
        if (self.isSelfState) {
            self.isSelfState = NO;
            [self showReflowBar:NO animation:YES];
            currentPageLayoutMode = oldPageLayoutMode;
            if (currentPageLayoutMode == PDF_LAYOUT_MODE_UNKNOWN) {
                currentPageLayoutMode = PDF_LAYOUT_MODE_SINGLE;
            }
            if (!_isReflowByClick) {
                currentPageLayoutMode = PDF_LAYOUT_MODE_SINGLE;
            }
            [_pdfViewCtrl setPageLayoutMode:currentPageLayoutMode];
            [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:REFLOW selected:NO];
            if ([_oldItemHiddenStatus allKeys].count) {
                if (![_oldItemHiddenStatus[@"Annotation"] boolValue]) {
                    if ([_extensionsManager.panelController isPanelHidden:FSPanelTypeAnnotation]) {
                        [_extensionsManager.panelController setPanelHidden:NO type:FSPanelTypeAnnotation];
                        AnnotationPanel* annotPanel =(AnnotationPanel*)[_extensionsManager.panelController getPanel:FSPanelTypeAnnotation];
                        [annotPanel onDocOpened:_pdfViewCtrl.currentDoc error:0];
                    }
                }
                
                if (![_oldItemHiddenStatus[@"DigitalSignature"] boolValue]) {
                    if ([_extensionsManager.panelController isPanelHidden:FSPanelTypeDigitalSignature]) {
                        [_extensionsManager.panelController setPanelHidden:NO type:FSPanelTypeDigitalSignature];
                        DigitalSignaturePanel* digtalPanel =(DigitalSignaturePanel*)[_extensionsManager.panelController getPanel:FSPanelTypeDigitalSignature];
                        [digtalPanel onDocOpened:_pdfViewCtrl.currentDoc error:0];
                    }
                }

            }
            [_extensionsManager.settingBar.viewSettingManager setPageColorWithNormalState];
            [_extensionsManager.settingBar updateLayoutButtonsWithLayout:currentPageLayoutMode];
        }
    }
}

#pragma IPageEventListener

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    [reflowManager clear];
    [(SelectToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Select] deSelect];
    [self setPreviousAndNextBtnEnable];
    if ([_extensionsManager getState] == FSUIManagerStateReflow) {
        FSPDFViewCtrl *pdfViewCtrl = self->_pdfViewCtrl;
        int currentPage = [pdfViewCtrl getCurrentPage];
        FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:pdfViewCtrl.currentDoc page:currentPage];
        self.bookmarkItem.selected = !(!bookmark || [bookmark isEmpty]);
        if (_extensionsManager.isFullScreen) {
            // [self delayHideReflowTopBottomToolBar];
        }
    }
}

#pragma mark --IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStateReflow) {
        if ([self checkIsIphoneDevice]) {
            self.toolBarBottomReflow.hidden = !isHidden;
        } else if (!isHidden) {
            [self.toolBarBottomReflow.superview bringSubviewToFront:self.toolBarBottomReflow];
        }
    }
}

#pragma mark -- IThumbnailEventListener
- (void)onThumbnailChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStateReflow) {
        self.toolBarBottomReflow.hidden = !isHidden;
        if (isHidden) {
            [_extensionsManager setFullScreen:NO];
        }
    }
}

-(void)reLayoutBottomToolbar
{
    [_toolBarBottomReflow removeAllItems];
    CGFloat spacing = ITEM_LABEL_TOP_MARGIN_NORMAL;
    if ([self checkIsIphoneDevice]) {
        _toolBarBottomReflow.layer.cornerRadius = 0;
        [_toolBarBottomReflow mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.mas_equalTo(_toolBarBottomReflow.superview);
        }];
        
        if (_pan && [_toolBarBottomReflow.gestureRecognizers containsObject:_pan]) {
            [_toolBarBottomReflow removeGestureRecognizer:_pan];
        }
        if (!_panelItem) {
            _panelItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"reflow_bookmark")];
            _panelItem.accessibilityLabel = FSLocalizedForKey(@"kPanelListVoiceover");
            _panelItem.accessibilityIdentifier = @"reflow_bookmark_id";
            @fs_weakify(self)
            _panelItem.onTapClick = ^(FSItem * _Nonnull _item) {
              @fs_strongify(self)
                [self bookmarkClicked];
            };
        }

        [_toolBarBottomReflow addItem:_panelItem spacing:spacing atPosition:FSItemPositionCenter];
        if (_extensionsManager.panelController.panel.specs.count == 0) {
            _panelItem.enabled = NO;
        }
        _panelItem.enabled = !(!_extensionsManager.config.loadReadingBookmark && !_extensionsManager.config.loadAttachment && !_extensionsManager.config.loadOutline);
        
        [_toolBarBottomReflow addItem:_biggerItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_smallerItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_showGraphItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_previousPageItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_nextPageItem spacing:spacing atPosition:FSItemPositionCenter];
        
    }else {
        _toolBarBottomReflow.layer.cornerRadius = 6;
        [_toolBarBottomReflow mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(_toolBarBottomReflow.superview.fs_mas_bottom).offset(-40);
            make.centerX.equalTo(_toolBarBottomReflow.superview);
            make.width.mas_equalTo(352);
        }];
        
        if (!_pan) {
            _pan = [[UIPanGestureRecognizer alloc] initWithTarget:_extensionsManager.settingBar.viewSettingManager action:@selector(moveBottombarWithRecognizer:)];
        }
        if (![_toolBarBottomReflow.gestureRecognizers containsObject:_pan]) {
            [_toolBarBottomReflow addGestureRecognizer:_pan];
        }
        
        if (!_exitItem) {
            _exitItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_exit")];
            _exitItem.accessibilityLabel = FSLocalizedForKey(@"kExitVoiceOver");
            @fs_weakify(self)
            _exitItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                [self->_extensionsManager changeState:FSUIManagerStateNormal];
            };
        }
        [_toolBarBottomReflow addItem:_biggerItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_smallerItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_showGraphItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_previousPageItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_nextPageItem spacing:spacing atPosition:FSItemPositionCenter];
        [_toolBarBottomReflow addItem:_exitItem spacing:spacing atPosition:FSItemPositionCenter];
    }
}
#pragma mark-- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    if (DEVICE_iPAD && changed) {
        [self reLayoutBottomToolbar];
        if ([_extensionsManager getState] == FSUIManagerStateReflow) {
            self.toolBarBottomReflow.hidden = YES;
            [_extensionsManager setCurrentIsFullScreen:NO];
            [self showReflowBar:YES animation:NO];
        }
    }
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}
@end
