/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SelectToolHandler.h"
#import "FtToolHandler.h"
#import "MagnifierView.h"
#import "SignToolHandler.h"
#import "RedactModule+Private.h"
#import "MarkupModule.h"
#import "FSPermissionUtilty.h"
#import "ReflowModule.h"

@interface SelectToolHandler () <IDocEventListener>

@property (nonatomic, assign) BOOL isTextSelect;
@property (nonatomic, assign) int currentPageIndex;
@property (nonatomic, strong) FSPointF *currentPoint;
@property (nonatomic, assign) int startPosIndex;
@property (nonatomic, assign) int endPosIndex;
@property (nonatomic, assign) int wordStartPosIndex;
@property (nonatomic, assign) int wordEndPosIndex;
@property (nonatomic, assign) int startReadPosIndex;
@property (nonatomic, assign) int endReadPosIndex;
@property (nonatomic, assign) BOOL isReading;
@property (nonatomic, assign) CGPoint longPressBeginPoint;
@property (nonatomic, assign) int currentReadPageIndex;
@property (nonatomic, strong) NSArray *arraySelectedRect;
@property (nonatomic, strong) FSRectF *currentEditPdfRect;
@property (nonatomic, strong) MagnifierView *magnifierView;
@end

@implementation SelectToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
}

@synthesize type = _type;

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _startPosIndex = -1;
        _endPosIndex = -1;
        _wordStartPosIndex = -1;
        _wordEndPosIndex = -1;
    }
    return self;
}

- (NSArray *)_getTextRects:(FSTextPage *)fstextPage start:(int)start end:(int)end {
    __block NSArray *ret = nil;
    FSTask *task = [[FSTask alloc] init];
    task.run = ^() {
        ret = [Utility getTextRects:fstextPage startCharIndex:start endCharIndex:end];
    };
    [_taskServer executeSync:task];
    return ret;
}

//get word range of string, including space
- (NSArray *)getUnitWordBoundary:(NSString *)str {
    NSMutableArray *array = [NSMutableArray array];
    CFStringTokenizerRef tokenizer = CFStringTokenizerCreate(kCFAllocatorDefault,
                                                             (CFStringRef) str,
                                                             CFRangeMake(0, [str length]),
                                                             kCFStringTokenizerUnitWordBoundary,
                                                             NULL);
    CFStringTokenizerTokenType tokenType = kCFStringTokenizerTokenNone;
    while ((tokenType = CFStringTokenizerAdvanceToNextToken(tokenizer)) != kCFStringTokenizerTokenNone) {
        CFRange tokenRange = CFStringTokenizerGetCurrentTokenRange(tokenizer);
        NSRange range = NSMakeRange(tokenRange.location, tokenRange.length);
        [array addObject:[NSValue valueWithRange:range]];
    }
    if (tokenizer) {
        CFRelease(tokenizer);
    }
    return array;
}

- (int)getCharIndexAtPos:(int)pageIndex point:(CGPoint)point {
    if (pageIndex < 0) return -1;
    FSTextPage *textPage;
    FSPointF *dibPoint;
    try {
        dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];

        int index =  [textPage getIndexAtPos:dibPoint.x y:dibPoint.y tolerance:5];
    } catch (NSException *exception) {
        return -1;
    }
    if ([textPage isEmpty]) return -1;
    return (int) [textPage getIndexAtPos:dibPoint.x y:dibPoint.y tolerance:5];
}


- (NSArray *)getCurrentSelectRects:(int)pageIndex {
    NSMutableArray *retArray = [NSMutableArray array];
    __block CGRect unionRect = CGRectZero;
    FSTextPage *textPage;
    textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    
    NSArray *array = [self _getTextRects:textPage start:self.startPosIndex end:self.endPosIndex];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        CGRect rect = [[obj objectAtIndex:0] CGRectValue];
        [retArray addObject:[NSValue valueWithCGRect:rect]];
        if (CGRectEqualToRect(unionRect, CGRectZero)) {
            unionRect = rect;
        } else {
            unionRect = CGRectUnion(unionRect, rect);
        }
    }];

    self.currentEditPdfRect = [Utility CGRect2FSRectF:unionRect];
    return retArray;
}

- (void)clearSelection {
    self.startPosIndex = -1;
    self.endPosIndex = -1;
    self.wordStartPosIndex = -1;
    self.wordEndPosIndex = -1;
    self.arraySelectedRect = nil;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
    if (self.isEdit && [_extensionsManager.menuControl isMenuVisible]) {
        [_extensionsManager.menuControl hideMenu];
    }
    [self clearSelection];
    [self closeMagnifier];
    [_pdfViewCtrl refresh:self.currentPageIndex];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentAnnot) {
        [_extensionsManager setCurrentAnnot:nil];
    }

    if ([self handleLongPressAndPan:pageIndex gestureRecognizer:recognizer]) {
        return YES;
    }
    return NO;
}

- (void )deSelect {
    if (_extensionsManager.currentToolHandler == self) {
        [_extensionsManager setCurrentToolHandler:nil];
    }
    if (_extensionsManager.currentAnnot) {
        [_extensionsManager setCurrentAnnot:nil];
    }
    self.isEdit = NO;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    id<IToolHandler> originToolHandler = _extensionsManager.currentToolHandler;
    if (_extensionsManager.currentToolHandler == self) {
        [_extensionsManager setCurrentToolHandler:nil];
    }
    if (!self.isEdit) {
        if ((originToolHandler == self || originToolHandler == nil ) && [_extensionsManager.menuControl isMenuVisible]) {
            [_extensionsManager.menuControl hideMenu];
            return YES;
        }
        return NO;
    }
    if (_extensionsManager.currentAnnot) {
        [_extensionsManager setCurrentAnnot:nil];
    }

    self.isEdit = NO;
    [self clearSelection];
    [_pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex needRender:NO];
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    if (!self.isEdit) {
        return NO;
    }
    if ([self handleLongPressAndPan:pageIndex gestureRecognizer:recognizer]) {
        return YES;
    }
    MenuControl *annotMenu = _extensionsManager.menuControl;
    if ([annotMenu isMenuVisible]) {
        [annotMenu hideMenu];
    }
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    if (_extensionsManager.currentToolHandler != self || _extensionsManager.currentAnnot) {
        return NO;
    }
    if (self.isEdit) {
        CGPoint  point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        
        int index = [self verifyPointRange:point pageIndex:pageIndex];
        if (index > -1) {
            return YES;
        }
    }
    return NO;
}
#pragma mark - private methods

- (BOOL)handleLongPressAndPan:(int)pageIndex gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer {
    CGPoint point;

    point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
        MenuControl *annotMenu = _extensionsManager.menuControl;
        if ([annotMenu isMenuVisible]) {
            [annotMenu hideMenu];
        }
        if ([gestureRecognizer isKindOfClass:[UILongPressGestureRecognizer class]]) //only start when long press
        {
            self.currentPageIndex = pageIndex;
            self.longPressBeginPoint = point;
            //start a new range of select text
            int index = [self getCharIndexAtPos:pageIndex point:point];
            [self clearSelection];
            if (index > -1) //has some selection, make first character selected
            {
                self.isEdit = YES;
                self.isTextSelect = YES;
                FSTextPage *textPage;
                FSPointF* fPoint = [[FSPointF alloc] init];
                textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
                fPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
                FSRange *range =[textPage getWordAtPos:[fPoint getX] y:[fPoint getY] tolerance:5];
                if(![range isEmpty] && [range getSegmentCount] == 1){
                    self.startPosIndex = [range getSegmentStart:0];
                    self.endPosIndex =  [range getSegmentEnd:0];
                }else{
                    self.isTextSelect = NO;
                    return NO;
                }
                self.wordStartPosIndex = self.startPosIndex;
                self.wordEndPosIndex = self.endPosIndex;
                [self showMagnifier:pageIndex index:index point:point];

                [_pdfViewCtrl refresh:pageIndex needRender:NO];
            } else {
                self.isTextSelect = NO;
                return NO;
            }
        } else if ([gestureRecognizer isKindOfClass:[UIPanGestureRecognizer class]]) //pan start is used to drag the dot
        {
            int index = [self verifyPointRange:point pageIndex:pageIndex];
            if (index > -1) {
                if (index == self.startPosIndex) {
                    //when drag dot always make drag end
                    int tmp = self.endPosIndex;
                    self.endPosIndex = self.startPosIndex;
                    self.startPosIndex = tmp;
                }
                [self showMagnifier:pageIndex index:index point:point];
                [_pdfViewCtrl refresh:pageIndex needRender:NO];
            }
            self.wordStartPosIndex = -1;
            self.wordEndPosIndex = -1;
        }
    } else if (gestureRecognizer.state == UIGestureRecognizerStateChanged) {
        
        if ([gestureRecognizer isKindOfClass:[UILongPressGestureRecognizer class]]){
            if (CGPointEqualToPoint(self.longPressBeginPoint, point)) {
                return YES;
            }
        }
        if (self.currentPageIndex != pageIndex) {
            return YES;
        }
        //position change, update the selected range
        if (self.startPosIndex == -1 || self.endPosIndex == -1) {
            return YES; //not start successfully
        }
        int index = [self getCharIndexAtPos:pageIndex point:point];
        if (index > -1) {
            self.endPosIndex = index;
            if (index < self.wordStartPosIndex && self.wordEndPosIndex != -1) {
                self.startPosIndex = self.wordEndPosIndex;
            }else if (index > self.wordStartPosIndex && self.wordStartPosIndex != -1){
                self.startPosIndex = self.wordStartPosIndex;
            }
            [_pdfViewCtrl refresh:pageIndex needRender:NO];
            [self showMagnifier:pageIndex index:index point:point];
            [self moveMagnifier:pageIndex index:index point:point];
        }
        return YES;
    } else if (gestureRecognizer.state == UIGestureRecognizerStateEnded || gestureRecognizer.state == UIGestureRecognizerStateCancelled) {
        [self closeMagnifier];
        if (self.currentPageIndex != pageIndex) {
            [self showTextMenu:self.currentPageIndex rect:[self convertIsReflowPageViewRect:self.currentEditPdfRect pageIndex:self.currentPageIndex]];
            return YES;
        }

        //getsture end. if have selection, pop up menu to let user choose; otherwise change to none op.
        if (self.startPosIndex == -1 || self.endPosIndex == -1) {
            return NO;
        } else {
            [self getCurrentSelectRects:pageIndex];
            
            [self showTextMenu:pageIndex rect:[self convertIsReflowPageViewRect:self.currentEditPdfRect pageIndex:self.currentPageIndex]];
            return YES;
        }
    }
    return YES;
}

- (CGRect)convertIsReflowPageViewRect:(FSRectF *)rect pageIndex:(int)pageIndex
{
    if([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
        FSReflowPage * reflowPage = [[ReflowModule getReflowManager] getParsedPage:pageIndex];
        return [_pdfViewCtrl convertReflowPageRectToPageViewRect:rect reflowPage:reflowPage pageIndex:pageIndex];
    }else{
        return [_pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:pageIndex];
    }
}

- (CGPoint)convertIsReflowPageViewPt:(FSPointF *)point pageIndex:(int)pageIndex
{
    if([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
        FSReflowPage * reflowPage = [[ReflowModule getReflowManager] getParsedPage:pageIndex];
        return [_pdfViewCtrl convertReflowPagePtToPageViewPt:point reflowPage:reflowPage pageIndex:pageIndex];
    }else{
        return [_pdfViewCtrl convertPdfPtToPageViewPt:point pageIndex:pageIndex];
    }
}


- (int)verifyPointRange:(CGPoint)point pageIndex:(int)pageIndex
{
    int tolrance = 60;
    int control = 20;
    CGRect startRect = CGRectZero;
    CGRect endRect = CGRectZero;
    FSTextPage *textPage;
   
    textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
   
    NSArray* startRects = [Utility getTextRects:textPage startCharIndex:self.startPosIndex endCharIndex:self.startPosIndex];
    NSArray* endRects = [Utility getTextRects:textPage startCharIndex:self.endPosIndex endCharIndex:self.endPosIndex];

    if (startRects.count > 0) {
       FSRectF *dibRect = [Utility CGRect2FSRectF:[[[startRects objectAtIndex:0] objectAtIndex:0] CGRectValue]];
       
       startRect = [self convertIsReflowPageViewRect:dibRect pageIndex:pageIndex];
    }
    if (endRects.count > 0) {
       FSRectF *dibRect = [Utility CGRect2FSRectF:[[[endRects objectAtIndex:0] objectAtIndex:0] CGRectValue]];
       endRect = [self convertIsReflowPageViewRect:dibRect pageIndex:pageIndex];
    }
    if (CGRectContainsPoint(startRect, point))
       return self.startPosIndex;
    if (CGRectContainsPoint(endRect, point))
       return self.endPosIndex;
    if (startRect.size.width < tolrance)
       startRect = CGRectInset(startRect, -(tolrance - startRect.size.width) / 2, -(tolrance - startRect.size.height) / 2);
    if (endRect.size.height < tolrance)
       endRect = CGRectInset(endRect, -(tolrance - endRect.size.width) / 2, -(tolrance - endRect.size.height) / 2);
    startRect.size.height += control;
    endRect.size.height += control;

    if (CGRectContainsPoint(startRect, point) && !CGRectContainsPoint(endRect, point))
       return self.startPosIndex;
    if (!CGRectContainsPoint(startRect, point) && CGRectContainsPoint(endRect, point))
       return self.endPosIndex;
    if (CGRectContainsPoint(startRect, point) && CGRectContainsPoint(endRect, point)) {
       CGFloat toStartX = point.x-(startRect.origin.x+startRect.size.width/2);
       CGFloat toStartY = point.y-(startRect.origin.y+startRect.size.height/2);
       CGFloat toEndX = point.x-(endRect.origin.x+endRect.size.width/2);
       CGFloat toEndY = point.y-(endRect.origin.y+endRect.size.height/2);
       if (toStartX*toStartX + toStartY*toStartY > toEndX*toEndX + toEndY*toEndY) {
           return self.endPosIndex;
       }else{
           return self.startPosIndex;
       }
    }
    return -1;
}

- (int)verifyIndexRange:(int)index count:(int)count {
    int ret = -1;
    if (index == -1) {
        ret = index;
    } else if (index >= self.startPosIndex - count && index <= self.startPosIndex + count) {
        ret = self.startPosIndex;
    } else if (index >= self.endPosIndex - count && index <= self.endPosIndex + count) {
        ret = self.endPosIndex;
    }
    return ret;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)showTextMenu:(int)pageIndex rect:(CGRect)rect {
    
    id<IModule> module = [_extensionsManager getModuleByName:@"Redact"];
    RedactModule *redactModule = (RedactModule *)module;
    if (_extensionsManager.currentState==FSUIManagerStateRedaction
        && redactModule && redactModule.bCanRedact) {
        [self addRedact];
        return;
    }

    self.currentPageIndex = pageIndex;
    NSMutableArray *array = [NSMutableArray array];
    MenuItem *copyTextItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kCopyText") object:self action:@selector(copyText)];
    MenuItem *hightLightItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kHighlight") object:self action:@selector(addHighlight)];
    MenuItem *squigglyItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSquiggly") object:self action:@selector(addSquiggly)];
    MenuItem *strikeOutItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kStrikeout") object:self action:@selector(addStrikeout)];
    MenuItem *underlineItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kUnderline") object:self action:@selector(addUnderline)];
    MenuItem *redactItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kRedaction") object:self action:@selector(addRedact)];
    MenuItem *speechItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSpeak") object:self action:@selector(speech)];

    if ([Utility canCopyText:_pdfViewCtrl] && _extensionsManager.config.copyText) {
        [array addObject:copyTextItem];
    }

    if ([_extensionsManager annotsPermissionCanAdd]) {
        if ([_extensionsManager.config.tools containsObject:Tool_Highlight]) {
            [array addObject:hightLightItem];
        }
        if ([_extensionsManager.config.tools containsObject:Tool_Underline]) {
            [array addObject:underlineItem];
        }
        if ([_extensionsManager.config.tools containsObject:Tool_Strikeout]) {
            [array addObject:strikeOutItem];
        }
        if ([_extensionsManager.config.tools containsObject:Tool_Squiggly]) {
            [array addObject:squigglyItem];
        }
        if ([_extensionsManager.config.tools containsObject:Tool_Redaction]) {
            if ([_extensionsManager.permissionProvider checkPermission:FSFunctionRedaction] != FSPermissionStateHide && [_pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW) [array addObject:redactItem];
        }
    }
    
    BOOL canSpeak = [Utility canCopyText:_pdfViewCtrl] && _extensionsManager.config.copyText && ![_pdfViewCtrl isDynamicXFA] && [_pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW;
    if (canSpeak) {
        [array addObject:speechItem];
    }
    if([_pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW){
        array = [_extensionsManager didForMenuControlWillShow:FSMenuControlSceneSelectedText array:array];
    }
    if (array.count > 0) {
        CGRect dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:rect pageIndex:self.currentPageIndex];
        MenuControl *annotMenu = _extensionsManager.menuControl;
        annotMenu.menuItems = array;
        [annotMenu setRect:dvRect];
        [annotMenu showMenu];
    }
}

- (NSString *)copyText;
{
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressCopyText];
    
    self.isEdit = NO;
    NSMutableString *str = [NSMutableString stringWithFormat:@""];
    FSTextPage *textSelect;
    textSelect = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:self.currentPageIndex];
     
    NSString *selStr = [textSelect getChars:MIN(self.startPosIndex, self.endPosIndex) count:ABS(self.endPosIndex - self.startPosIndex) + 1];
    if (selStr && selStr.length > 0) {
        [str appendString:selStr];
    }
    if (str && ![str isEqualToString:@""]) {
        UIPasteboard *board = [UIPasteboard generalPasteboard];
        board.string = str;
    }
    [_pdfViewCtrl refresh:self.currentPageIndex needRender:NO];
    [self clearSelection];
    return str;
}

- (void)addHighlight {
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressHighlight];
    
    self.isEdit = NO;
    [self addMarkup:FSAnnotHighlight];
    [self clearSelection];
}

- (void)addSquiggly {
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressSquiggly];
    
    self.isEdit = NO;
    [self addMarkup:FSAnnotSquiggly];
    [self clearSelection];
}

- (void)addStrikeout {
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressStrikeout];
    
    self.isEdit = NO;
    [self addMarkup:FSAnnotStrikeOut];
    [self clearSelection];
}

- (void)addUnderline {
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressUnderline];
    
    self.isEdit = NO;
    [self addMarkup:FSAnnotUnderline];
    [self clearSelection];
}

- (void)addRedact{
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressRedaction];
    
    if (!_extensionsManager.isSupportRedaction) {
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailedLoadModuleForLicenseTip") actionBack:nil onCurrentSceneResponder:self->_pdfViewCtrl];
        [self clearSelection];
        return;
    }
    
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionRedaction checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            self.isEdit = NO;
            [self addMarkup:FSAnnotRedact];
            [self clearSelection];
        }
    }];
}

- (void)speech{
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingLongPressSpeak];
    
    NSMutableArray *array = [NSMutableArray array];
    MenuItem *speechSeclectItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSpeakString") object:self action:@selector(speechSeclect)];
    MenuItem *speechFromItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSpeakStart") object:self action:@selector(speechFrom)];
    [array addObject:speechSeclectItem];
    [array addObject:speechFromItem];
    
    if (array.count > 0) {
        CGRect dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect: [_pdfViewCtrl convertPdfRectToPageViewRect:self.currentEditPdfRect pageIndex:self.currentPageIndex] pageIndex:self.currentPageIndex];
        MenuControl *annotMenu = _extensionsManager.menuControl;
        annotMenu.menuItems = array;
        [annotMenu setRect:dvRect];
        [self performSelector:@selector(delayShowMenu) withObject:nil afterDelay:0.5f];
    }
}

- (void)delayShowMenu
{
    MenuControl *annotMenu = _extensionsManager.menuControl;
    [annotMenu showMenu];
}

- (void)speechSeclect
{
    self.isEdit = NO;
    _startReadPosIndex = MIN(_startPosIndex, _endPosIndex);
    _endReadPosIndex =   MAX(_startPosIndex, _endPosIndex);
    _currentReadPageIndex = _currentPageIndex;
    SpeechModule *speech = _extensionsManager.speechModule;
    [speech speakSelectPageIndex:self.currentPageIndex startCharPos:_startReadPosIndex endCharPos:_endReadPosIndex];
    _isReading = YES;
    [self getCurrentSelectRects:_currentPageIndex];
    [self clearSelection];
}

- (void)speechFrom
{
    self.isEdit = NO;

    _startReadPosIndex = MIN(_startPosIndex, _endPosIndex);
    _endReadPosIndex =   MAX(_startPosIndex, _endPosIndex);
    _currentReadPageIndex = _currentPageIndex;
    SpeechModule *speech = _extensionsManager.speechModule;
    [speech speakFromPageIndex:self.currentPageIndex startCharPos:_startReadPosIndex];
    [self getCurrentSelectRects:_currentPageIndex];
    [self clearSelection];
    return;
}

//- (void)popoverControllerDidDismissPopover:(UIPopoverController *)popoverController {
//    self.isEdit = NO;
//    [_pdfViewCtrl refresh:CGRectZero pageIndex:self.currentPageIndex needRender:NO];
//}

- (void)addMarkup:(FSAnnotType)type {
    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:self.currentPageIndex];
    NSArray *array = [self _getTextRects:textPage start:self.startPosIndex end:self.endPosIndex];
    NSMutableArray *arrayQuads = [NSMutableArray array];
    for (int i = 0; i < array.count; i++) {
        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:i] objectAtIndex:0] CGRectValue]];
        int direction = [[[array objectAtIndex:i] objectAtIndex:1] intValue];
        CGPoint point1;
        CGPoint point2;
        CGPoint point3;
        CGPoint point4;
        if (direction == 0 || direction == 4) //text is horizontal or unknown, left to right
        {
            point1.x = dibRect.left;
            point1.y = dibRect.top;
            point2.x = dibRect.right;
            point2.y = dibRect.top;
            point3.x = dibRect.left;
            point3.y = dibRect.bottom;
            point4.x = dibRect.right;
            point4.y = dibRect.bottom;
        } else if (direction == 1 || direction == 3) // test is vertical, left to right
        {
            point4.x = dibRect.right;
            point4.y = dibRect.top;
            point3.x = dibRect.right;
            point3.y = dibRect.bottom;
            point2.x = dibRect.left;
            point2.y = dibRect.top;
            point1.x = dibRect.left;
            point1.y = dibRect.bottom;
        } else if (direction == 2) //text is horizontal, right to left
        {
            point4.x = dibRect.left;
            point4.y = dibRect.top;
            point3.x = dibRect.right;
            point3.y = dibRect.top;
            point2.x = dibRect.left;
            point2.y = dibRect.bottom;
            point1.x = dibRect.right;
            point1.y = dibRect.bottom;
        // } else if (direction == 3) //text is vertical, right to left
        // {
        //     point1.x = dibRect.right;
        //     point1.y = dibRect.top;
        //     point2.x = dibRect.right;
        //     point2.y = dibRect.bottom;
        //     point3.x = dibRect.left;
        //     point3.y = dibRect.top;
        //     point4.x = dibRect.left;
        //     point4.y = dibRect.bottom;
        } else {
            continue;
        }

        FSQuadPoints *fsqp = [[FSQuadPoints alloc] init];
        FSPointF *pt1 = [[FSPointF alloc] init];
        [pt1 set:point1.x y:point1.y];
        FSPointF *pt2 = [[FSPointF alloc] init];
        [pt2 set:point2.x y:point2.y];
        FSPointF *pt3 = [[FSPointF alloc] init];
        [pt3 set:point3.x y:point3.y];
        FSPointF *pt4 = [[FSPointF alloc] init];
        [pt4 set:point4.x y:point4.y];
        [fsqp setFirst:pt1];
        [fsqp setSecond:pt2];
        [fsqp setThird:pt3];
        [fsqp setFourth:pt4];
        [arrayQuads addObject:fsqp];
    }
    if (0 == arrayQuads.count)
        return;
    CGRect insetRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.currentEditPdfRect pageIndex:self.currentPageIndex];
    FSRectF *rect = [_pdfViewCtrl convertPageViewRectToPdfRect:insetRect pageIndex:self.currentPageIndex];

    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:self.currentPageIndex];
    if (!page || [page isEmpty])
        return;

    FSMarkup *annot = nil;
    if (type == FSAnnotRedact) {
        @try {
            FSRedaction *redaction = [[FSRedaction alloc] initWithDocument:page.getDocument];
            FSRectFArray *rects = [[FSRectFArray alloc] init];
            [rects add:rect];
            annot = [redaction markRedactAnnot:page rects:rects];
        } @catch (NSException *exception) {
            return;
        }
    }else{
        annot = [[FSMarkup alloc] initWithAnnot:[page addAnnot:type rect:rect]];
    }
    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.quads = arrayQuads;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;

    unsigned int color = 0;
    float opacity = 0;
    if (type == FSAnnotHighlight) {
        FSReadToolSettings* toolSettings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeHighlight];
        color = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeHighlight].settingsColor;
        opacity = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeHighlight].opacity;
        color = toolSettings ? (unsigned int)toolSettings.color.rgbValue : color;
        opacity = toolSettings ? toolSettings.opacity : opacity;
        annot.subject = @"Highlight";
        annot.color = color;
    } else if (type == FSAnnotSquiggly) {
        FSReadToolSettings* toolSettings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeSquiggly];
        color = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeSquiggly].settingsColor;
        opacity = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeSquiggly].opacity;
        color = toolSettings ? (unsigned int)toolSettings.color.rgbValue : color;
        opacity = toolSettings ? toolSettings.opacity : opacity;
        annot.subject = @"Squiggly";
        annot.color = color;
    } else if (type == FSAnnotStrikeOut) {
        FSReadToolSettings* toolSettings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeStrikeout];
        color = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeStrikeout].settingsColor;
        opacity = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeStrikeout].opacity;
        color = toolSettings ? (unsigned int)toolSettings.color.rgbValue : color;
        opacity = toolSettings ? toolSettings.opacity : opacity;
        annot.subject = @"Strikeout";
        annot.color = color;
    } else if (type == FSAnnotUnderline) {
        FSReadToolSettings* toolSettings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeUnderline];
        color = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeUnderline].settingsColor;
        opacity = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeUnderline].opacity;
        color = toolSettings ? (unsigned int)toolSettings.color.rgbValue : color;
        opacity = toolSettings ? toolSettings.opacity : opacity;
        annot.subject = @"Underline";
        annot.color = color;
    }else if (type == FSAnnotRedact) {
        unsigned int fillColor = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeRedact].fillColor.rgbValue;
        opacity = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeRedact].opacity;
        annot.subject = @"Redact";
        annot.contents = @"";
        FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
        redact.applyFillColor = fillColor;
    }

    
    annot.opacity = opacity;

    if (textPage && type != FSAnnotRedact) {
        NSString *tmp = @"";
        tmp = [textPage getChars:MIN(self.startPosIndex, self.endPosIndex) count:ABS(self.endPosIndex - self.startPosIndex) + 1];
//        for (int i = 0; i < arrayQuads.count; i++) {
//            FSQuadPoints *arrayQuad = [arrayQuads objectAtIndex:i];
//            FSRectF *rect = [Utility convertToFSRect:arrayQuad.getFirst p2:arrayQuad.getFourth];
//            NSString *text = [textPage getTextInRect:rect];
//            if (text.length > 0) {
//                tmp = [tmp stringByAppendingString:[textPage getTextInRect:rect]];
//            }
//        }
        [annot setContents:tmp];
    }
    [_pdfViewCtrl lockRefresh];
    [annot resetAppearanceStream];
    [_pdfViewCtrl unlockRefresh];

    FSTask *task = [[FSTask alloc] init];
    task.run = ^() {
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
        [annotHandler addAnnot:annot];
    
        CGRect cgRect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:self.currentPageIndex];
        cgRect = CGRectInset(cgRect, -20, -20);

        [self->_pdfViewCtrl refresh:cgRect pageIndex:self.currentPageIndex];
        [self clearSelection];
    };
    [_taskServer executeSync:task];
}

- (void)showBlankMenu:(int)pageIndex point:(CGPoint)point {
    self.currentPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    self.currentPageIndex = pageIndex;
    NSMutableArray *array = [NSMutableArray array];
    MenuItem *commentItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kNote") object:self action:@selector(comment)];
    MenuItem *typeWriterItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kTypewriter") object:self action:@selector(typeWriter)];
    MenuItem *signatureItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSignAction") object:self action:@selector(signature)];

    if ([FSPermissionUtilty canAddAnnot:_extensionsManager]) {
        if ([_extensionsManager.config.tools containsObject:Tool_Note]) {
            [array addObject:commentItem];
        }
        if ([_extensionsManager.config.tools containsObject:Tool_Freetext]) {
            [array addObject:typeWriterItem];
        }
    }
    if ([FSPermissionUtilty canAddSign:_extensionsManager]) {
        if (_extensionsManager.config.loadSignature)
            [array addObject:signatureItem];
    }

    if (array.count > 0) {
        CGRect dvRect = CGRectMake(point.x, point.y, 2, 2);
        dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:dvRect pageIndex:pageIndex];
        MenuControl *annotMenu = _extensionsManager.menuControl;
        annotMenu.menuItems = array;
        [annotMenu setRect:dvRect];
        [annotMenu showMenu];
    }
}

- (void)comment {
    self.isEdit = NO;
    unsigned int color = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeNote].settingsColor;
    float pageWidth = [_pdfViewCtrl getPageViewWidth:self.currentPageIndex];
    float pageHeight = [_pdfViewCtrl getPageViewHeight:self.currentPageIndex];

    float scale = pageWidth / 1000.0;
    CGPoint pvPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.currentPoint pageIndex:self.currentPageIndex];

    if (pvPoint.x > pageWidth - NOTE_ANNOTATION_WIDTH * scale * 2)
        pvPoint.x = pageWidth - NOTE_ANNOTATION_WIDTH * scale * 2;
    if (pvPoint.y > pageHeight - NOTE_ANNOTATION_WIDTH * scale * 2)
        pvPoint.y = pageHeight - NOTE_ANNOTATION_WIDTH * scale * 2;

    CGRect rect = CGRectMake(pvPoint.x - NOTE_ANNOTATION_WIDTH * scale / 2, pvPoint.y - NOTE_ANNOTATION_WIDTH * scale / 2, NOTE_ANNOTATION_WIDTH * scale, NOTE_ANNOTATION_WIDTH * scale);
    FSRectF *dibRect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:self.currentPageIndex];

    
    FSTextViewPopoverViewController *noteDialog = [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:_extensionsManager.topToolbar.getToolbarContentView];
    noteDialog.okItemEnabled = ^BOOL(NSString *text) {
        return YES;
    };
    noteDialog.title = FSLocalizedForKey(@"kNote");
    @fs_weakify(self)
    noteDialog.didClickedOK = ^(NSString *text) {
        @fs_strongify(self)
        FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:self.currentPageIndex];
        if (!page || [page isEmpty])
            return;

        FSNote *note = [[FSNote alloc] initWithAnnot:[page addAnnot:FSAnnotNote rect:dibRect]];
        note.color = color;
        int opacity = [self->_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeNote].opacity;
        note.opacity = opacity / 100.0f;
        note.icon = [self->_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeNote].noteIcon;
        note.author = self-> _extensionsManager.annotAuthor;
        note.contents = text;
        note.NM = [Utility getUUID];
        note.lineWidth = 2;
        note.modifiedDate = [NSDate date];
        note.createDate = [NSDate date];
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:note];
        [annotHandler addAnnot:note];
    };
    if (_extensionsManager.currentToolHandler == self) {
        [_extensionsManager setCurrentToolHandler:nil];
    }
}

- (void)typeWriter {
    self.isEdit = NO;
    FtToolHandler *toolHandler = (FtToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Freetext];
    toolHandler.toolSettings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeTypewriter];
    [_extensionsManager setCurrentToolHandler:toolHandler];
    toolHandler.freeTextStartPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.currentPoint pageIndex:self.currentPageIndex];
    [toolHandler onPageViewTap:self.currentPageIndex recognizer:nil];
    toolHandler.isTypewriterToolbarActive = NO;
    if (_extensionsManager.currentToolHandler == self) {
        [_extensionsManager setCurrentToolHandler:nil];
    }
}

- (void)signature {
    self.isEdit = NO;
    SignToolHandler *toolHandler = (SignToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Signature];
    [_extensionsManager setCurrentToolHandler:toolHandler];
    toolHandler.isAdded = NO;
    toolHandler.signatureStartPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.currentPoint pageIndex:self.currentPageIndex];
    [toolHandler onPageViewTap:self.currentPageIndex recognizer:nil];
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (pageIndex != self.currentPageIndex) {
        return;
    }

    if (self.isEdit && self.isTextSelect) {
        if (self.startPosIndex == -1 || self.endPosIndex == -1) {
            return;
        }

        self.arraySelectedRect = [self getCurrentSelectRects:pageIndex];
        [self.arraySelectedRect enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect selfRect = [obj CGRectValue];
            FSRectF *docRect = [Utility CGRect2FSRectF:selfRect];
            CGRect pvRect = [self convertIsReflowPageViewRect:docRect pageIndex:pageIndex];
            
            UIColor *highlightColor = self->_extensionsManager.selectionHighlightColor;
            CGFloat red, green, blue, alpha;
            [highlightColor getRed:&red green:&green blue:&blue alpha:&alpha];
            CGContextSetRGBFillColor(context, red, green, blue, alpha);
            CGContextFillRect(context, pvRect);

            //draw the drag dot
            if (idx == 0) {
                UIImage *dragDot = [UIImage imageNamed:@"annotation_dragdot.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
                CGRect leftCursor = CGRectMake((int) (pvRect.origin.x - 7.5), (int) (pvRect.origin.y - 12), 15, 17);
                [dragDot drawAtPoint:leftCursor.origin];
            }
            if (idx + 1 == self.arraySelectedRect.count) {
                UIImage *dragDot = [UIImage imageNamed:@"annotation_dragdot.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
                CGRect rightCursor = CGRectMake((int) (pvRect.origin.x + pvRect.size.width - 7.5), (int) (pvRect.origin.y + pvRect.size.height - 5), 15, 17);
                [dragDot drawAtPoint:rightCursor.origin];
            }
        }];
    }
}

#pragma mark - IPageEventListener

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    if (oldIndex != currentIndex) {
        if (!self.isEdit) {
            return;
        }
        if (_extensionsManager.currentAnnot) {
            [_extensionsManager setCurrentAnnot:nil];
        }

        MenuControl *annotMenu = _extensionsManager.menuControl;
        if ([annotMenu isMenuVisible]) {
            [annotMenu hideMenu];
        }
        self.isEdit = NO;
        [self clearSelection];

        [_pdfViewCtrl refreshOverlay];
    }
}

#pragma mark IRotationEventListener

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    [self dismissAnnotMenu];
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    [self showAnnotMenu];
}

#pragma mark IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

- (void)onScrollViewWillBeginDragging:(UIScrollView *)scrollView {
    [self dismissAnnotMenu];
}

- (void)onScrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate;
{
    [self showAnnotMenu];
}

- (void)onScrollViewWillBeginDecelerating:(UIScrollView *)scrollView {
    [self dismissAnnotMenu];
}

- (void)onScrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    [self showAnnotMenu];
}

- (void)onScrollViewWillBeginZooming:(UIScrollView *)scrollView {
    [self dismissAnnotMenu];
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView {
    [self showAnnotMenu];
}

- (void)showAnnotMenu {
    if (self.isEdit) {
        double delayInSeconds = .05;
        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
        dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
            MenuControl *annotMenu = self->_extensionsManager.menuControl;
            if (self.isTextSelect) {
                
                CGRect dvRect = [self convertIsReflowPageViewRect:self.currentEditPdfRect pageIndex:self.currentPageIndex];
                dvRect = [self->_pdfViewCtrl convertPageViewRectToDisplayViewRect:dvRect pageIndex:self.currentPageIndex];

                CGRect rectDisplayView = [[self->_pdfViewCtrl getDisplayView] bounds];
                if (CGRectIsEmpty(dvRect) || CGRectIsNull(CGRectIntersection(dvRect, rectDisplayView)))
                    return;

                [annotMenu setRect:dvRect];
                [annotMenu showMenu];
            } else {
                
                CGPoint dvPoint = [self convertIsReflowPageViewPt:self.currentPoint pageIndex:self.currentPageIndex];
                CGRect dvRect = CGRectMake(dvPoint.x, dvPoint.y, 2, 2);

                CGRect rectDisplayView = [[self->_pdfViewCtrl getDisplayView] bounds];
                if (CGRectIsEmpty(dvRect) || CGRectIsNull(CGRectIntersection(dvRect, rectDisplayView)))
                    return;

                dvRect = [self->_pdfViewCtrl convertPageViewRectToDisplayViewRect:dvRect pageIndex:self.currentPageIndex];
                [annotMenu setRect:dvRect];
                [annotMenu showMenu];
            }
        });
    }
}

- (void)dismissAnnotMenu {
    if (self.isEdit) {
        MenuControl *annotMenu = _extensionsManager.menuControl;
        if ([annotMenu isMenuVisible]) {
            [annotMenu hideMenu];
        }
    }
}

#pragma mark - Magnifier

- (void)showMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    if (_magnifierView == nil) {
        FSTextPage *textPage;
        textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
        NSArray *array = [self _getTextRects:textPage start:index end:index + 1];
        if (array.count > 0) {
            FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
            CGRect rect = [self convertIsReflowPageViewRect:dibRect pageIndex:pageIndex];
            point = CGPointMake(point.x, CGRectGetMidY(rect));
        }
        _magnifierView = [[MagnifierView alloc] init];
        _magnifierView.viewToMagnify = [_pdfViewCtrl getDisplayView];
        _magnifierView.touchPoint = point;
        _magnifierView.magnifyPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getDisplayView]];
        [[_pdfViewCtrl getPageView:pageIndex] addSubview:_magnifierView];
    
    }
}

- (void)moveMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    FSTextPage *textPage;
    textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    NSArray *array = [self _getTextRects:textPage start:index end:index + 1];
    if (array.count > 0) {
        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
        CGRect rect = [self convertIsReflowPageViewRect:dibRect pageIndex:pageIndex];
        point = CGPointMake(point.x, CGRectGetMidY(rect));
    }
    _magnifierView.touchPoint = point;
    _magnifierView.magnifyPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getDisplayView]];
    
}


- (void)closeMagnifier {
    [_magnifierView removeFromSuperview];
    _magnifierView = nil;
}

- (NSString *)getName {
    return Tool_Select;
}

#pragma mark IDocEventListener

- (void)onDocWillClose:(FSPDFDoc *)document {
    self.isEdit = NO;
    [self clearSelection];
}

#pragma mark --IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD) {
        [self showAnnotMenu];
    }
}

@synthesize toolSettings;

@end
