/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSSettingBar.h"
#import "FSSettingBar+private.h"
#import "FSSettingBarHorizontalCell.h"
#import "PasswordModule.h"

typedef NS_ENUM(NSUInteger, KDividLineType) {
    KDividLineTypeHorizontal,
    KDividLineTypeVertical
};

@interface FSSettingBar () <IDocEventListener,IWindowFrameChangedListener>
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, assign) BOOL isNightMode;
@property (nonatomic, assign) PDF_LAYOUT_MODE lastLayoutMode;
@property (nonatomic, assign) CGFloat pdfViewWidth;
@end

@implementation FSSettingBar

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _viewSettingManager = [[FSViewSettingManager alloc] initWithExtensionsManager:extensionsManager];
        self.pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self.pdfViewCtrl registerDocEventListener:self];
        self.extensionsManager = extensionsManager;
       
        [_extensionsManager registerWindowFrameChangedListener:self];
        UIDevice *device = [UIDevice currentDevice];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChanged:) name:UIDeviceOrientationDidChangeNotification object:device];
    }
    return self;
}

- (void)dealloc
{
    [self.pdfViewCtrl unregisterDocEventListener:self];
    [_extensionsManager unregisterWindowFrameChangedListener:self];
}

- (void)initContentView
{
    CGFloat width = _pdfViewCtrl.fs_width;
    CGFloat height = _pdfViewCtrl.fs_height > width ? 458 : (_pdfViewCtrl.fs_height-80);
    self.contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, [self checkIsIphoneDevice] ? MIN(_pdfViewCtrl.fs_height,width) : 375, [self checkIsIphoneDevice] ? height : 271)];
    self.contentView.backgroundColor = self.fs_backgroundColor_B1;
    _settingView = [[FSSettingContentView alloc] initWithFrame:self.contentView.bounds extensionsManager:_extensionsManager];
    [self.contentView addSubview:_settingView];
    [_settingView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.bottom.mas_equalTo(0);
    }];
}

- (void)updateContentViewLayoutHeight{
    if (![self checkIsIphoneDevice]) {
        CGFloat layoutHeight = _settingView.layoutHeight;
        if (layoutHeight) {
            self.contentView.fs_height = layoutHeight + 33;
        }
    }
}

- (void)updateBtnLayout {
    [self updateLayoutButtonsWithLayout:[_pdfViewCtrl getPageLayoutMode]];
}

- (void)didClickItemWithType:(SettingItemType)type
{
    switch (type) {
        case SINGLE:
            if ([self.delegate respondsToSelector:@selector(settingBarSinglePageLayout:)]) {
                [self.delegate settingBarSinglePageLayout:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewSinglePage];
            break;
        case DOUBLEPAGE:
            if ([self.delegate respondsToSelector:@selector(settingBarDoublePageLayout:)]) {
                [self.delegate settingBarDoublePageLayout:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewFacing];
            break;
        case COVERPAGE:
            if ([self.delegate respondsToSelector:@selector(settingBarCoverPageLayout:)]) {
                [self.delegate settingBarCoverPageLayout:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewCoverFacing];
            break;
        case DAYMODE:
            if ([self.delegate respondsToSelector:@selector(settingBar:isNightMode:)]) {
                [self.delegate settingBar:self isNightMode:NO];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewDay];
            break;
        case PAGECOLOR:
        {
            if ([self.delegate respondsToSelector:@selector(settingBarPageColor:)]) {
                [self.delegate settingBarPageColor:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewPageColor];
        }
            break;
        case NIGHTMODE:
            if ([self.delegate respondsToSelector:@selector(settingBar:isNightMode:)]) {
                [self.delegate settingBar:self isNightMode:YES];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewNight];
            break;
        case CONTINUOUS:
            [self.pdfViewCtrl setContinuous:[self.viewSettingManager getContinuousON]];
            if ([self.delegate respondsToSelector:@selector(settingBarContinuousLayout:)]) {
                [self.delegate settingBarContinuousLayout:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewContinuousScrolling];
            break;
        case PAGERTL:
        {
            [self.pdfViewCtrl setPageBindingEdge:[self.viewSettingManager getPageRTLON] ? PDF_PAGE_BINDING_RIGHT_EDGE : PDF_PAGE_BINDING_LEFT_EDGE];
            if ([self.delegate respondsToSelector:@selector(settingBarDidChangePageEdge:)]) {
                [self.delegate settingBarDidChangePageEdge:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewPageRTL];
            break;
        }
        case FITPAGE:
        {
            PDF_LAYOUT_MODE currentLayoutMode = [_pdfViewCtrl getPageLayoutMode];
            if (currentLayoutMode == PDF_LAYOUT_MODE_UNKNOWN ||  currentLayoutMode == PDF_LAYOUT_MODE_REFLOW) {
                return;
            }
            if ([self.delegate respondsToSelector:@selector(settingBarFitPage:)]) {
                [self.delegate settingBarFitPage:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewFitPage];
        }
            break;
        case FITWIDTH:
        {
            PDF_LAYOUT_MODE currentLayoutMode = [_pdfViewCtrl getPageLayoutMode];
            if (currentLayoutMode == PDF_LAYOUT_MODE_UNKNOWN ||  currentLayoutMode == PDF_LAYOUT_MODE_REFLOW) {
                return;
            }
            if ([self.delegate respondsToSelector:@selector(settingBarFitWidth:)]) {
                [self.delegate settingBarFitWidth:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewFitWidth];
        }
            break;
        case REFLOW:
            if ([self.delegate respondsToSelector:@selector(settingBarReflow:)]) {
                [self.delegate settingBarReflow:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewReflow];
            break;
        case CROPPAGE:
            if ([self.delegate respondsToSelector:@selector(settingBarCrop:)]) {
                [self.delegate settingBarCrop:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewCropMode];
            break;
        case SPEECH:
            if ([self.delegate respondsToSelector:@selector(settingBarSpeech:)]) {
                [self.delegate settingBarSpeech:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewSpeak];
            break;
        case AUTOFLIP:
            if ([self.delegate respondsToSelector:@selector(settingBarAutoFlip:)]) {
                [self.delegate settingBarAutoFlip:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewAutoFlip];
            break;
        case ROTATE:
        {
            PDF_LAYOUT_MODE currentLayoutMode = [_pdfViewCtrl getPageLayoutMode];
            if (currentLayoutMode == PDF_LAYOUT_MODE_UNKNOWN ||  currentLayoutMode == PDF_LAYOUT_MODE_REFLOW) {
                return;
            }
            if ([self.delegate respondsToSelector:@selector(settingBarRotate:)]) {
                [self.delegate settingBarRotate:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewRotateView];
        }
            break;
        case PANZOOM:
            if ([self.delegate respondsToSelector:@selector(settingBarPanAndZoom:)]) {
                [self.delegate settingBarPanAndZoom:self];
            }
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingViewPanZoom];
            break;
        default:
            break;
    }
}

# pragma mark - get setting bar items show/hide status.
-(NSMutableDictionary *)getItemHiddenStatus{
    NSMutableDictionary *settingbarInfo = [@{} mutableCopy];
    
    for (int i = SINGLE; i <= PANZOOM; i++) {
        BOOL hidden = [_viewSettingManager getItemHiddenWithType:i];
        [settingbarInfo setObject:hidden ? @"YES" : @"NO" forKey:[self getItemKeyByIndexKey:i]];
    }
    return settingbarInfo;
}

- (NSString*)getItemKeyByIndexKey:(int)cellType
{
    NSString* itemKey;
    switch (cellType) {
        case SINGLE:
            itemKey = @"SingleViewBtn";
            break;
        case DOUBLEPAGE:
            itemKey = @"DoubleViewBtn";
            break;
        case COVERPAGE:
            itemKey = @"CoverPageBtn";
            break;
        case DAYMODE:
            itemKey = @"DayMode";
            break;
        case PAGECOLOR:
            itemKey = @"PageColor";
            break;
        case NIGHTMODE:
            itemKey = @"NightMode";
            break;
        case CONTINUOUS:
            itemKey = @"ContinueSwith";
            break;
        case PAGERTL:
            itemKey = @"PageRLT";
            break;
        case FITPAGE:
            itemKey = @"FitpageBtn";
            break;
        case FITWIDTH:
            itemKey = @"FitwidthBtn";
            break;
        case REFLOW:
            itemKey = @"Reflow";
            break;
        case CROPPAGE:
            itemKey = @"CropBtn";
            break;
        case SPEECH:
            itemKey = @"SpeechBtn";
            break;
        case AUTOFLIP:
            itemKey = @"AutoFlip";
            break;
        case ROTATE:
            itemKey = @"RotateBtn";
            break;
        case PANZOOM:
            itemKey = @"PanAndZoomBtn";
            break;
        default:
            break;
    }
    return itemKey;
}

- (BOOL)isItemHidden:(SettingItemType) type {
    
    return [_viewSettingManager getItemHiddenWithType:type];
}

- (void)setItem:(SettingItemType)itemType hidden:(BOOL)hidden {
    [_viewSettingManager setItemHiddenWithType:itemType hidden:hidden];
}

- (void)updateLayoutButtonsWithLayout:(PDF_LAYOUT_MODE)layout {
    
    [_viewSettingManager setItemSelectedWithType:SINGLE selected:layout == PDF_LAYOUT_MODE_SINGLE];
    [_viewSettingManager setContinuousON:[self.extensionsManager.pdfViewCtrl isContinuous]];
    [_viewSettingManager setPageRTLON:[self.extensionsManager.pdfViewCtrl pageBindingEdge] == PDF_PAGE_BINDING_RIGHT_EDGE];
    [_viewSettingManager setItemSelectedWithType:DOUBLEPAGE selected:layout == PDF_LAYOUT_MODE_TWO];
    [_viewSettingManager setItemSelectedWithType:COVERPAGE selected:layout == PDF_LAYOUT_MODE_TWO_RIGHT];
    [_viewSettingManager setItemSelectedWithType:REFLOW selected:layout == PDF_LAYOUT_MODE_REFLOW];
    [_viewSettingManager setItemSelectedWithType:NIGHTMODE selected:_pdfViewCtrl.isNightMode];
    
    float currentPageScale = [self.extensionsManager.pdfViewCtrl getScale];
    float defaultPageScale = [self.extensionsManager.pdfViewCtrl getDefaultPageScale];
    currentPageScale = [[NSString stringWithFormat:@"%.6f",currentPageScale] floatValue];
    defaultPageScale = [[NSString stringWithFormat:@"%.6f",defaultPageScale] floatValue];

//    BOOL fitpagSelected = [_pdfViewCtrl getZoomMode] == PDF_DISPLAY_ZOOMMODE_FITPAGE && currentPageScale == defaultPageScale;
//    [_viewSettingManager setItemSelectedWithType:FITPAGE selected:fitpagSelected];
//    BOOL fitWidthSelected = [_pdfViewCtrl getZoomMode] == PDF_DISPLAY_ZOOMMODE_FITWIDTH && currentPageScale == defaultPageScale;
//    [_viewSettingManager setItemSelectedWithType:FITWIDTH selected:fitWidthSelected];

    [_viewSettingManager setItemSelectedWithType:CROPPAGE selected:[_pdfViewCtrl getCropMode] != PDF_CROP_MODE_NONE ? YES : NO];
//    [_viewSettingManager setItemSelectedWithType:FITPAGE selected:[_pdfViewCtrl getZoomMode] == PDF_DISPLAY_ZOOMMODE_FITPAGE];
//    [_viewSettingManager setItemSelectedWithType:FITWIDTH selected:[_pdfViewCtrl getZoomMode] == PDF_DISPLAY_ZOOMMODE_FITWIDTH];
    
    BOOL canSpeak = [Utility canCopyText:self.extensionsManager.pdfViewCtrl] && self.extensionsManager.config.copyText && ![_pdfViewCtrl isDynamicXFA];
    FSPDFDocUserPermissions permission = [_extensionsManager.pdfViewCtrl getDocPermissions];
    [_viewSettingManager setItemDefaultEnabledWithType:SPEECH enabled:canSpeak && ((permission & FSPDFDocPermExtractAccess) == FSPDFDocPermExtractAccess)];
    BOOL isDynamicXFA = [_pdfViewCtrl isDynamicXFA];
    [_viewSettingManager setItemDefaultEnabledWithType:REFLOW enabled:!isDynamicXFA];
    [_viewSettingManager setItemDefaultEnabledWithType:PANZOOM enabled:!isDynamicXFA];
    [_viewSettingManager setItemDefaultEnabledWithType:CROPPAGE enabled:!isDynamicXFA];
    
    [_settingView reloadData];
    
    [_viewSettingManager setItemsInIPAD];
}

#pragma mark IAppLifecycleListener

- (void)applicationWillResignActive:(UIApplication *)application {
}

- (void)applicationDidEnterBackground:(UIApplication *)application {

}

- (void)applicationWillEnterForeground:(UIApplication *)application {

}

- (void)applicationDidBecomeActive:(UIApplication *)application {

}

#pragma mark IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    [self updateLayoutButtonsWithLayout:[_pdfViewCtrl getPageLayoutMode]];
}

// todo
- (void)readStarted {

}

- (void)readDestroy {

}

- (void)orientationChanged:(NSNotification *)note {
}

#pragma mark-- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    if (DEVICE_iPAD ) {
        if (!changed) {
            if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && self.viewSettingManager.isShowSettingView) {
                
                self.contentView.frame = CGRectMake(0, 0, newFrame.size.width, self.contentView.frame.size.height);
                [_settingView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.top.bottom.mas_equalTo(0);
                    make.width.mas_equalTo(newFrame.size.width);
                }];
                [_settingView reloadDataWithFrameChanged:newFrame.size.width];
            }
            if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && self.viewSettingManager.isShowPageColorView) {
                [self.viewSettingManager resetPageColorViewSize:newFrame.size.width];
            }
        }
    }
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}

@end
