/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "MarkupModule.h"
#import "TextMKAnnotHandler.h"
#import "TextMKToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>
#import "UIExtensionsConfig.h"

@interface MarkupModule () {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSAnnotType _annotType;
}
@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem* highlightReadtoolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *areaHiItem;
@property (nonatomic, strong) FSReadToolbarItem* squigglyReadtoolbarItem;
@property (nonatomic, strong) FSReadToolbarItem* strikeOutReadtoolbarItem;
@property (nonatomic, strong) FSReadToolbarItem* underlineReadtoolbarItem;
@end

@implementation MarkupModule

- (NSString *)getName {
    return @"Markup";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        MKToolHandler* toolHandler = [[MKToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        MKAnnotHandler* annotHandler = [[MKAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];
    }
    return self;
}

- (void)loadModule {
    
    if ([_extensionsManager.config.tools containsObject:Tool_Highlight]) {
        [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeHighlight] atToolTag:FSMainTopbarToolTagHome];
        
        _highlightReadtoolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeHighlight]];
        [_extensionsManager.toolbarManager addReadItem:_highlightReadtoolbarItem atToolTag:FSMainTopbarToolTagComment];
        _areaHiItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeAreaHighlight]];
        [_extensionsManager.toolbarManager addReadItem:_areaHiItem atToolTag:FSMainTopbarToolTagComment];
    }
    if ([_extensionsManager.config.tools containsObject:Tool_Underline]) {
        _underlineReadtoolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeUnderline]];
        [_extensionsManager.toolbarManager addReadItem:_underlineReadtoolbarItem atToolTag:FSMainTopbarToolTagComment];
    }
    if ([_extensionsManager.config.tools containsObject:Tool_Strikeout]) {
        _strikeOutReadtoolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeStrikeout]];
        [_extensionsManager.toolbarManager addReadItem:_strikeOutReadtoolbarItem atToolTag:FSMainTopbarToolTagComment];
    }
    if ([_extensionsManager.config.tools containsObject:Tool_Squiggly]) {
        _squigglyReadtoolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeSquiggly]];
        [_extensionsManager.toolbarManager addReadItem:_squigglyReadtoolbarItem atToolTag:FSMainTopbarToolTagComment];
    }
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)setAnnotType:(FSAnnotType)annotType{
    _annotType = annotType;
}

- (FSAnnotType)annotType{
    return _annotType;
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Markup];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    switch (_annotType) {
    case FSAnnotHighlight:
    case FSAnnotSquiggly:
    case FSAnnotStrikeOut:
    case FSAnnotUnderline:
        toolHandler.type = _annotType;
        [_extensionsManager setCurrentToolHandler:toolHandler];
        break;

    default:
        break;
    }
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;

    FSUIElementType elementType = -1;
    switch (_annotType) {
        case FSAnnotHighlight:
            elementType = FSUIElementReadingCommentBarHighlight;
            if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
                elementType = FSUIElementReadingCommentBarAreaHighlight;
            }
            break;
        case FSAnnotSquiggly:
            elementType = FSUIElementReadingCommentBarSquiggly;
            break;
        case FSAnnotStrikeOut:
            elementType = FSUIElementReadingCommentBarStrikeout;
            break;
        case FSAnnotUnderline:
            elementType = FSUIElementReadingCommentBarUnderline;
            break;
        default:
            break;
    }
    
    [_extensionsManager onUIInteractElementClicked:elementType];
}

- (void)dismissAnnotationContinue {
    [Utility dismissAnnotationContinue:_pdfViewCtrl];
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_highlightReadtoolbarItem.toolSettings == toolSettings){
        _highlightReadtoolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_areaHiItem.toolSettings == toolSettings){
        _areaHiItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_underlineReadtoolbarItem.toolSettings == toolSettings){
        _underlineReadtoolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_squigglyReadtoolbarItem.toolSettings == toolSettings){
        _squigglyReadtoolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_strikeOutReadtoolbarItem.toolSettings == toolSettings){
        _strikeOutReadtoolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}
                    
@synthesize annotType;

@end
