/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "TextMKAnnotHandler.h"
#import "UIExtensionsConfig+private.h"
#import "MarkupModule.h"
#import "MagnifierView.h"
#import "FSPermissionUtilty.h"

@interface MKAnnotHandler ()
{
    float _maxWidth;
    float _minWidth;
    float _maxHeight;
    float _minHeight;
    MagnifierView *_magnifierView;
    BOOL _shouldDrawMarkupAnnot;
}

@property (nonatomic, assign) int startReadPos;
@property (nonatomic, assign) int endReadPos;

@property (nonatomic, assign) CGPoint oldPoint;
@property (nonatomic) BOOL shouldShowMenu;
@property (nonatomic) BOOL shouldShowPropertyBar;

@property (nonatomic, strong) FSRectF *oldRect;

@property (nonatomic, assign) int startPosIndex;
@property (nonatomic, assign) int endPosIndex;

@property (nonatomic, assign) CGRect currentEditRect;
@property (nonatomic, assign) CGRect oldEditRect;

@property (nonatomic, strong) NSArray *markupQuadsArrForRender;

@end



@implementation MKAnnotHandler {
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super initWithUIExtensionsManager:extensionsManager];
    if (self) {
        _taskServer = extensionsManager.taskServer;
        self.annotBorderEdgeInsets = UIEdgeInsetsMake(-5, -5, -5, -5);
        _shouldDrawMarkupAnnot = YES;
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotHighlight;
}

- (BOOL)shouldDrawAnnot:(FSAnnot *)annot inPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl {
    if (!_shouldDrawMarkupAnnot && [self.annot isEqualToAnnot:annot]) {
        return NO;
    }
    return YES;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionOpen;
    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionOpen;
    }
    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options |= FSMenuOptionStyle;
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    if (annot.canReply && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        if ([self.extensionsManager annotsPermissionCanReply:annot]) {
            options |= FSMenuOptionReply;
        }
    }
   
    if (annot.canCopyText && [Utility canCopyText:self.extensionsManager.pdfViewCtrl] && self.extensionsManager.config.copyText) {
        if ([annot getFilterAnnotType] == FSAnnotAreaHighlight) {
            FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:[annot pageIndex]];
            NSString *text = [Utility getTextFromRect:textPage fsrect:annot.fsrect];
            if (text && text.length > 0) {
                options |= FSMenuOptionCopyText;
                options |= FSMenuOptionSpeech;
            }
        }else{
            options |= FSMenuOptionCopyText;
            options |= FSMenuOptionSpeech;
        }
    }
    return options;
}

- (BOOL)canMoveAnnot:(FSAnnot *)annot {
    if (![self.extensionsManager annotsPermissionCanModify:annot]) {
        return NO;
    }
    return [annot getFilterAnnotType] == FSAnnotAreaHighlight;
}

- (BOOL)canResizeAnnot:(FSAnnot *)annot {
    if (![self.extensionsManager annotsPermissionCanModify:annot]) {
        return NO;
    }
    return [annot getFilterAnnotType] == FSAnnotAreaHighlight;
}

- (void)resetProperties{

    FSAnnot *annot = self.extensionsManager.currentAnnot;
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setPropertyColor:annot.color];
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];

    [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity)]];
}

- (void)speech{
    
    SpeechModule *speech = self.extensionsManager.speechModule;
    [speech speakFromTextMarkup:[[FSTextMarkup alloc] initWithAnnot:self.annot]];
    self.extensionsManager.currentAnnot = nil;
}


- (BOOL)isHitAnnot:(FSAnnot *)annot point:(FSPointF *)point {
    if (annot.type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]]) {
        // this is a |replace| annot
        return NO;
    }
    return [super isHitAnnot:annot point:point];
}

-(void)onAnnotSelected:(FSAnnot *)annot
{
    _shouldDrawMarkupAnnot = NO;
    self.markupQuadsArrForRender = annot.quads;
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:annot.pageIndex];
    if ([textPage getCharCount] < 1) self.markupQuadsArrForRender = nil;
    [super onAnnotSelected:annot];
}

-(void)onAnnotDeselected:(FSAnnot *)annot
{
    [super onAnnotDeselected:annot];
    _shouldDrawMarkupAnnot = YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    if (self.extensionsManager.currentAnnot == annot) {
        if (pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint]) {
            [self.extensionsManager showMenu];
            return YES;
        } else {
            [self.extensionsManager setCurrentAnnot:nil];
            return YES;
        }
    } else {
        [self.extensionsManager setCurrentAnnot:annot];
//        [self comment];
        return YES;
    }
    return NO;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer annot:(FSAnnot *)annot
{
    if (self.extensionsManager.currentAnnot != annot)
    {
        return NO;
    }
    
    
    if (![self.extensionsManager annotsPermissionCanModify:annot]) {
        if (self.extensionsManager.currentAnnot) {
            [self showMenu];
        }
        return YES;
    }
    
    if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        if (self.extensionsManager.currentAnnot &&
            [self.extensionsManager.currentAnnot getFilterAnnotType] == FSAnnotAreaHighlight) {
            self.extensionsManager.menuControl.menuItems = [self getMenuItems];
        }
    }
    
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
    
    _maxWidth = [self.pdfViewCtrl getPageViewWidth:pageIndex];
    _minWidth = 10;
    _maxHeight = [self.pdfViewCtrl getPageViewHeight:pageIndex];
    _minHeight = 10;
    if ([annot.intent isEqualToString:@"AreaHi"]) {
        BOOL canMove = [self canMoveAnnot:annot];
        BOOL canResize = [self canResizeAnnot:annot];
        if (!canMove && !canResize) {
            if (self.extensionsManager.currentAnnot) {
                [self showMenu];
            }
            return YES;
        }
        
        if (self.oldPoint.x == 0 && self.oldPoint.y == 0) {
            self.oldPoint = point;
        }
        
        if (pageIndex != annot.pageIndex)
        {
            self.oldPoint = CGPointZero;
            return NO;
        }
        
        if (recognizer.state == UIGestureRecognizerStateBegan)
        {
            self.oldPoint = CGPointZero;
            
            self.shouldShowMenu = self.extensionsManager.shouldShowMenu;
            self.shouldShowPropertyBar = self.extensionsManager.shouldShowPropertyBar;
            [self hideMenu];
            [self.extensionsManager hidePropertyBar];
            
            self.editType = FSAnnotEditTypeFull;
            
            CGRect rect = [Utility getAnnotRect:annot pdfViewCtrl:self.pdfViewCtrl];
            rect = CGRectInset(rect, -10,-10);
            
            self.editType = [ShapeUtil getAnnotEditTypeWithPoint:point rect:rect keepAspectRatio:NO defaultEditType:FSAnnotEditTypeFull];
        }
        else if (recognizer.state == UIGestureRecognizerStateChanged)
        {
            self.oldRect = CopyRectF(annot.fsrect) ;
            
            float tw = point.x - self.oldPoint.x;
            float th = point.y - self.oldPoint.y;
            self.oldPoint = point;
            
            CGRect realRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            FSRectF *rect = [Utility CGRect2FSRectF:realRect];
            
            if (self.editType == FSAnnotEditTypeLeftTop ||
                self.editType == FSAnnotEditTypeLeftMiddle ||
                self.editType == FSAnnotEditTypeLeftBottom ||
                self.editType == FSAnnotEditTypeFull)
            {
                if (!annot.canModify) {
                    return YES;
                }
                
                rect.left += tw;
                
                if (rect.left < _minWidth) {
                    return NO;
                }
                
                if (self.editType != FSAnnotEditTypeFull)
                {
                    // Not left over right
                    if ((rect.left + _minWidth) > rect.right)
                    {
                        rect.right = rect.left + _minWidth;
                        if (rect.right > _maxWidth - _minWidth) {
                            return NO;
                        }
                    }
                    else if (ABS(rect.right - rect.left) > _maxWidth)
                    {
                        rect.left -= tw;
                    }
                }
            }
            if (self.editType == FSAnnotEditTypeRightTop ||
                self.editType == FSAnnotEditTypeRightMiddle ||
                self.editType == FSAnnotEditTypeRightBottom ||
                self.editType == FSAnnotEditTypeFull)
            {
                if (!annot.canModify) {
                    return YES;
                }
                rect.right +=tw;
                if (rect.right > _maxWidth - _minWidth) {
                    return NO;
                }
                if (self.editType != FSAnnotEditTypeFull)
                {
                    if ((rect.left + _minWidth) > rect.right)
                    {
                        rect.left = rect.right - _minWidth;
                        if (rect.left < _minWidth) {
                            return NO;
                        }
                    }
                    else if (ABS(rect.right - rect.left) > _maxWidth)
                    {
                        rect.right -= tw;
                    }
                }
            }
            
            if (self.editType == FSAnnotEditTypeLeftTop ||
                self.editType == FSAnnotEditTypeMiddleTop ||
                self.editType == FSAnnotEditTypeRightTop ||
                self.editType == FSAnnotEditTypeFull)
            {
                if (!annot.canModify) {
                    return YES;
                }
                rect.bottom += th;
                if (rect.bottom < _minHeight) {
                    return NO;
                }
                if (self.editType != FSAnnotEditTypeFull)
                {
                    if ((rect.bottom + _minHeight) > rect.top)
                    {
                        rect.top = rect.bottom + _minHeight;
                        if (rect.bottom > _maxHeight - _minHeight) {
                            return NO;
                        }
                    }
                    else if (ABS(rect.bottom - rect.top) > _maxHeight)
                    {
                        rect.bottom -= th;
                    }
                }
            }
            if (self.editType == FSAnnotEditTypeLeftBottom ||
                self.editType == FSAnnotEditTypeMiddleBottom||
                self.editType == FSAnnotEditTypeRightBottom ||
                self.editType == FSAnnotEditTypeFull)
            {
                if (!annot.canModify) {
                    return YES;
                }
                rect.top += th;
                if (rect.top > _maxHeight - _minHeight) {
                    return NO;
                }
                if (self.editType != FSAnnotEditTypeFull)
                {
                    if ((rect.bottom + _minHeight) > rect.top)
                    {
                        rect.bottom = rect.top - _minHeight;
                        if (rect.bottom < _minHeight) {
                            return NO;
                        }
                    }
                    else if (ABS(rect.bottom - rect.top) > _maxHeight)
                    {
                        rect.top -= th;
                    }
                }
            }
            
            CGRect newRect = [Utility FSRectF2CGRect:rect];
            
            annot.fsrect = [self.pdfViewCtrl convertPageViewRectToPdfRect:newRect pageIndex:pageIndex];
            
            [self.pdfViewCtrl lockRefresh];
            [annot resetAppearanceStream];
            [self.pdfViewCtrl unlockRefresh];
            
            CGRect oldRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:pageIndex];

            newRect = CGRectInset(CGRectUnion(newRect, oldRect), -30, -30);
            [self.pdfViewCtrl refresh:newRect pageIndex:pageIndex needRender:NO];
        }
        
        else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled)
        {
            self.editType = FSAnnotEditTypeFull;
            
            if (annot.canModify) {
                //[self modifyAnnot:pageView dmAnnot:shapeAnnot addUndo:NO];
            }
            
            CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            
            CGRect oldRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:pageIndex];
            
//            CGRect showRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:newRect pageIndex:pageIndex];
            
            if (self.extensionsManager.shouldShowPropertyBar && !DEVICE_iPHONE)
            {
                self.shouldShowMenu = NO;
                self.shouldShowPropertyBar = YES;
                [self.extensionsManager showPropertyBar];
            }
            else
            {
                self.shouldShowMenu = YES;
                self.shouldShowPropertyBar = NO;
                [self.extensionsManager showMenu];
            }
            
            newRect = CGRectUnion(newRect, oldRect);
            newRect = CGRectInset(newRect, -30, -30);
            self.oldPoint = CGPointZero;
            
            [self.pdfViewCtrl refresh:newRect pageIndex:pageIndex needRender:YES];
            
        }
    }
    else
    {
        if (pageIndex != annot.pageIndex)
        {
            self.oldPoint = CGPointZero;
            return NO;
        }
        
        if (recognizer.state == UIGestureRecognizerStateBegan)
        {
            self.shouldShowMenu = NO;
            self.shouldShowPropertyBar = NO;
            
            [self hideMenu];
            [self.extensionsManager hidePropertyBar];

            self.editType = FSAnnotEditTypeFull;
            
            NSArray *array = annot.quads;
            
            self.markupQuadsArrForRender = array;
            
            
            for (int i = 0; i < array.count; i++)
            {
                FSQuadPoints *arrayQuad = [array objectAtIndex:i];
                CGPoint startPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:arrayQuad.first pageIndex:pageIndex];
                CGPoint endPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:arrayQuad.fourth pageIndex:pageIndex];

                CGRect pvRect = [self getRectFromStartPoint:startPoint endPoint:endPoint];

                if (i == 0)
                {
                    int index = [self getCharIndexAtPos:pageIndex point:CGPointMake(pvRect.origin.x, pvRect.origin.y)];
                    self.startPosIndex = index;
                }

                if (i == array.count - 1)
                {
                    int index = [self getCharIndexAtPos:pageIndex point: CGPointMake(pvRect.origin.x + pvRect.size.width, pvRect.origin.y + pvRect.size.height)];
                    self.endPosIndex = index;
                }
            }
            
            for (int i = 0; i < array.count; i++)
            {
                FSQuadPoints *arrayQuad = [array objectAtIndex:i];
                CGPoint startPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:arrayQuad.first pageIndex:pageIndex];
                CGPoint endPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:arrayQuad.fourth pageIndex:pageIndex];
                
                CGRect pvRect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
                
                if (i == 0)
                {
                    int index = [self getCharIndexAtPos:pageIndex point:CGPointMake(pvRect.origin.x, pvRect.origin.y)];
                    self.startPosIndex = index;
                    
                    CGRect leftCursor = CGRectMake((int)(pvRect.origin.x-17.5), (int)(pvRect.origin.y-22), 35, 37);
                    
                    if (CGRectContainsPoint(leftCursor, point)) {
                        self.editType = FSAnnotEditTypeLeftMiddle;
                        [self showMagnifier:pageIndex index:self.startPosIndex point:point];
                        break;
                    }
                    
                }
                if (i == array.count - 1)
                {
                    int index = [self getCharIndexAtPos:pageIndex point: CGPointMake(pvRect.origin.x + pvRect.size.width, pvRect.origin.y + pvRect.size.height)];
                    self.endPosIndex = index;
                    
                    CGRect rightCursor = CGRectMake((int)(pvRect.origin.x+pvRect.size.width-17.5), (int)(pvRect.origin.y+pvRect.size.height-15), 35, 37);
                    
                    if (CGRectContainsPoint(rightCursor, point)) {
                        self.editType = FSAnnotEditTypeRightMiddle;
                        [self showMagnifier:pageIndex index:self.startPosIndex point:point];
                        break;
                    }
                }
                
            }
        }
        else if (recognizer.state == UIGestureRecognizerStateChanged)
        {
            if (self.editType == FSAnnotEditTypeLeftMiddle)
            {
                if (!annot.canModify) {
                    return YES;
                }
                
                int index = [self getCharIndexAtPos:pageIndex point:point];
                
                if (index > -1)
                {
                    if (index < self.endPosIndex) {
                        self.startPosIndex = index;
                    }
                    else if (index > self.endPosIndex)
                    {
                        self.startPosIndex = self.endPosIndex;
                        self.endPosIndex = index;
                    }
                   
                    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
                    int start = MIN(_startPosIndex, _endPosIndex);
                    int count = ABS(_endPosIndex - _startPosIndex) + 1;
                    NSArray *array = [Utility getTextRects:textPage start:start count:count];
                    
                    NSMutableArray *arrayQuads = [NSMutableArray array];
                    
                    for (int i = 0; i < array.count; i++) {
                        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:i] objectAtIndex:0] CGRectValue]];
                        int direction = [[[array objectAtIndex:i] objectAtIndex:1] intValue];
                        FSQuadPoints *fsqps = [[FSQuadPoints alloc] init];
                        if (direction == 0 || direction == 4) //text is horizontal or unknown, left to right
                        {
                            FSPointF *first = [[FSPointF alloc] init];
                            [first set:dibRect.left y:dibRect.top];
                            FSPointF *second = [[FSPointF alloc] init];
                            [second set:dibRect.right y:dibRect.top];
                            FSPointF *third = [[FSPointF alloc] init];
                            [third set:dibRect.left y:dibRect.bottom];
                            FSPointF *fourth = [[FSPointF alloc] init];
                            [fourth set:dibRect.right y:dibRect.bottom];
                            [fsqps set:first second:second third:third fourth:fourth];
                        } else if (direction == 1 || direction == 3) // test is vertical, left to right
                        {
                            FSPointF *first = [[FSPointF alloc] init];
                            [first set:dibRect.left y:dibRect.bottom];
                            FSPointF *second = [[FSPointF alloc] init];
                            [second set:dibRect.left y:dibRect.top];
                            FSPointF *third = [[FSPointF alloc] init];
                            [third set:dibRect.right y:dibRect.bottom];
                            FSPointF *fourth = [[FSPointF alloc] init];
                            [fourth set:dibRect.right y:dibRect.top];
                            [fsqps set:first second:second third:third fourth:fourth];
                        } else if (direction == 2) //text is horizontal, right to left
                        {
                            FSPointF *first = [[FSPointF alloc] init];
                            [first set:dibRect.right y:dibRect.bottom];
                            FSPointF *second = [[FSPointF alloc] init];
                            [second set:dibRect.left y:dibRect.bottom];
                            FSPointF *third = [[FSPointF alloc] init];
                            [third set:dibRect.right y:dibRect.top];
                            FSPointF *fourth = [[FSPointF alloc] init];
                            [fourth set:dibRect.left y:dibRect.top];
                            [fsqps set:first second:second third:third fourth:fourth];
            //            } else if (direction == 3) //text is vertical, right to left
            //            {
            //                FSPointF *first = [[FSPointF alloc] init];
            //                [first set:dibRect.right y:dibRect.top];
            //                FSPointF *second = [[FSPointF alloc] init];
            //                [second set:dibRect.right y:dibRect.bottom];
            //                FSPointF *third = [[FSPointF alloc] init];
            //                [third set:dibRect.left y:dibRect.top];
            //                FSPointF *fourth = [[FSPointF alloc] init];
            //                [fourth set:dibRect.left y:dibRect.bottom];
            //                [fsqps set:first second:second third:third fourth:fourth];
                        }
                        [arrayQuads addObject:fsqps];
                    }
                    
                    self.markupQuadsArrForRender = arrayQuads;
                    
//                    annot.quads = arrayQuads;
                    
//                    FSCRT_QUADPOINTSF *quads = [DmAnnot convertArrayToQuads:annot.quads];
//                    FSPDF_Annot_SetQuadPoints(MKGAnnot, quads, (FS_INT32)annot.quads.count);
                    
//                    [self.pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex];
                    [self.pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex needRender:NO];
                    
                    [self moveMagnifier:pageIndex index:index point:point];
                }
            }
            
            if (self.editType == FSAnnotEditTypeRightMiddle)
            {
                if (!annot.canModify) {
                    return YES;
                }
                
                int index = [self getCharIndexAtPos:pageIndex point:point];
                
                if (index > -1)
                {
                    if (index > self.startPosIndex) {
                        self.endPosIndex = index;
                    }
                    else if (index < self.startPosIndex)
                    {
                        self.endPosIndex = self.startPosIndex;
                        self.startPosIndex = index;
                    }
                    
                    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
                    assert(_startPosIndex != -1 && _endPosIndex != -1);
                    int start = MIN(_startPosIndex, _endPosIndex);
                    int count = ABS(_endPosIndex - _startPosIndex) + 1;
                    NSArray *array = [Utility getTextRects:textPage start:start count:count];
                    
                    NSMutableArray *arrayQuads = [NSMutableArray array];
                    
                    for (int i = 0; i < array.count; i++) {
                        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:i] objectAtIndex:0] CGRectValue]];
                        int direction = [[[array objectAtIndex:i] objectAtIndex:1] intValue];
                        FSQuadPoints *fsqps = [[FSQuadPoints alloc] init];
                        if (direction == 0 || direction == 4) //text is horizontal or unknown, left to right
                        {
                            FSPointF *first = [[FSPointF alloc] init];
                            [first set:dibRect.left y:dibRect.top];
                            FSPointF *second = [[FSPointF alloc] init];
                            [second set:dibRect.right y:dibRect.top];
                            FSPointF *third = [[FSPointF alloc] init];
                            [third set:dibRect.left y:dibRect.bottom];
                            FSPointF *fourth = [[FSPointF alloc] init];
                            [fourth set:dibRect.right y:dibRect.bottom];
                            [fsqps set:first second:second third:third fourth:fourth];
                        } else if (direction == 1 || direction == 3) // test is vertical, left to right
                        {
                            FSPointF *first = [[FSPointF alloc] init];
                            [first set:dibRect.left y:dibRect.bottom];
                            FSPointF *second = [[FSPointF alloc] init];
                            [second set:dibRect.left y:dibRect.top];
                            FSPointF *third = [[FSPointF alloc] init];
                            [third set:dibRect.right y:dibRect.bottom];
                            FSPointF *fourth = [[FSPointF alloc] init];
                            [fourth set:dibRect.right y:dibRect.top];
                            [fsqps set:first second:second third:third fourth:fourth];
                        } else if (direction == 2) //text is horizontal, right to left
                        {
                            FSPointF *first = [[FSPointF alloc] init];
                            [first set:dibRect.right y:dibRect.bottom];
                            FSPointF *second = [[FSPointF alloc] init];
                            [second set:dibRect.left y:dibRect.bottom];
                            FSPointF *third = [[FSPointF alloc] init];
                            [third set:dibRect.right y:dibRect.top];
                            FSPointF *fourth = [[FSPointF alloc] init];
                            [fourth set:dibRect.left y:dibRect.top];
                            [fsqps set:first second:second third:third fourth:fourth];
            //            } else if (direction == 3) //text is vertical, right to left
            //            {
            //                FSPointF *first = [[FSPointF alloc] init];
            //                [first set:dibRect.right y:dibRect.top];
            //                FSPointF *second = [[FSPointF alloc] init];
            //                [second set:dibRect.right y:dibRect.bottom];
            //                FSPointF *third = [[FSPointF alloc] init];
            //                [third set:dibRect.left y:dibRect.top];
            //                FSPointF *fourth = [[FSPointF alloc] init];
            //                [fourth set:dibRect.left y:dibRect.bottom];
            //                [fsqps set:first second:second third:third fourth:fourth];
                        }
                        [arrayQuads addObject:fsqps];
                    }
                    self.markupQuadsArrForRender = arrayQuads;
//                    annot.quads = arrayQuads;
                    
//                    FSCRT_QUADPOINTSF *quads = [DmAnnot convertArrayToQuads:annot.quads];
//                    FSPDF_Annot_SetQuadPoints(MKGAnnot, quads, (FS_INT32)annot.quads.count);

                    [self.pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex needRender:NO];
                    [self moveMagnifier:pageIndex index:index point:point];
                }
            }
        }
        else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled)
        {

            [self closeMagnifier];
            
            if (self.startPosIndex == -1 || self.endPosIndex == -1)
            {
                return YES;
            }
            
            FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
            assert(_startPosIndex != -1 && _endPosIndex != -1);
            int start = MIN(_startPosIndex, _endPosIndex);
            int count = ABS(_endPosIndex - _startPosIndex) + 1;
            NSArray *array = [Utility getTextRects:textPage start:start count:count];
            
            NSMutableArray *arrayQuads = [NSMutableArray array];
            
            for (int i = 0; i < array.count; i++) {
                FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:i] objectAtIndex:0] CGRectValue]];
                int direction = [[[array objectAtIndex:i] objectAtIndex:1] intValue];
                FSQuadPoints *fsqps = [[FSQuadPoints alloc] init];
                if (direction == 0 || direction == 4) //text is horizontal or unknown, left to right
                {
                    FSPointF *first = [[FSPointF alloc] init];
                    [first set:dibRect.left y:dibRect.top];
                    FSPointF *second = [[FSPointF alloc] init];
                    [second set:dibRect.right y:dibRect.top];
                    FSPointF *third = [[FSPointF alloc] init];
                    [third set:dibRect.left y:dibRect.bottom];
                    FSPointF *fourth = [[FSPointF alloc] init];
                    [fourth set:dibRect.right y:dibRect.bottom];
                    [fsqps set:first second:second third:third fourth:fourth];
                } else if (direction == 1 || direction == 3) // test is vertical, left to right
                {
                    FSPointF *first = [[FSPointF alloc] init];
                    [first set:dibRect.left y:dibRect.bottom];
                    FSPointF *second = [[FSPointF alloc] init];
                    [second set:dibRect.left y:dibRect.top];
                    FSPointF *third = [[FSPointF alloc] init];
                    [third set:dibRect.right y:dibRect.bottom];
                    FSPointF *fourth = [[FSPointF alloc] init];
                    [fourth set:dibRect.right y:dibRect.top];
                    [fsqps set:first second:second third:third fourth:fourth];
                } else if (direction == 2) //text is horizontal, right to left
                {
                    FSPointF *first = [[FSPointF alloc] init];
                    [first set:dibRect.right y:dibRect.bottom];
                    FSPointF *second = [[FSPointF alloc] init];
                    [second set:dibRect.left y:dibRect.bottom];
                    FSPointF *third = [[FSPointF alloc] init];
                    [third set:dibRect.right y:dibRect.top];
                    FSPointF *fourth = [[FSPointF alloc] init];
                    [fourth set:dibRect.left y:dibRect.top];
                    [fsqps set:first second:second third:third fourth:fourth];
    //            } else if (direction == 3) //text is vertical, right to left
    //            {
    //                FSPointF *first = [[FSPointF alloc] init];
    //                [first set:dibRect.right y:dibRect.top];
    //                FSPointF *second = [[FSPointF alloc] init];
    //                [second set:dibRect.right y:dibRect.bottom];
    //                FSPointF *third = [[FSPointF alloc] init];
    //                [third set:dibRect.left y:dibRect.top];
    //                FSPointF *fourth = [[FSPointF alloc] init];
    //                [fourth set:dibRect.left y:dibRect.bottom];
    //                [fsqps set:first second:second third:third fourth:fourth];
                }
                [arrayQuads addObject:fsqps];
            }
            annot.quads = arrayQuads;
            
            [self.pdfViewCtrl lockRefresh];
            [annot resetAppearanceStream];
            [self.pdfViewCtrl unlockRefresh];
            
            self.markupQuadsArrForRender = arrayQuads;
//            FSCRT_QUADPOINTSF *quads = [DmAnnot convertArrayToQuads:annot.quads];
//            FSPDF_Annot_SetQuadPoints(MKGAnnot, quads, (FS_INT32)annot.quads.count);
            
            [self getCurrentSelectRects:pageIndex];

            FSRectF *rect = [self.pdfViewCtrl convertPageViewRectToPdfRect:self.currentEditRect pageIndex:pageIndex];
            
            annot.fsrect = rect;
            
            
            CGRect newRect = self.currentEditRect;
            
            CGRect oldRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.oldRect pageIndex:pageIndex];
            
            if (self.extensionsManager.shouldShowPropertyBar && !DEVICE_iPHONE)
            {
                self.shouldShowMenu = NO;
                self.shouldShowPropertyBar = YES;
                [self.extensionsManager showPropertyBar];
            }
            else
            {
                self.shouldShowMenu = YES;
                self.shouldShowPropertyBar = NO;
                [self.extensionsManager showMenu];
            }
            
            newRect = CGRectUnion(newRect, oldRect);
            newRect = CGRectInset(newRect, -30, -30);
            
            self.oldPoint = CGPointZero;
            [self.pdfViewCtrl refresh:newRect pageIndex:pageIndex needRender:YES];
        }
    }
    return YES;
}

#pragma mark - draw annot
- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *)annot {
    
    if (self.extensionsManager.currentAnnot == annot && pageIndex == annot.pageIndex)
    {
        CGRect rect = [Utility getAnnotRect:annot pdfViewCtrl:self.pdfViewCtrl];
        CGContextSaveGState(context);
        
        if ([annot.intent isEqualToString:@"AreaHi"]) {
            CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
            CGContextFillRect(context, rect);
            
            CGContextRestoreGState(context);
            rect = CGRectInset(rect, -10,-10);
            CGContextSetLineWidth(context, 2.0);
            CGFloat dashArray[] = {3,3,3,3};
            CGContextSetLineDash(context, 3, dashArray, 4);
            
            CGContextSetStrokeColorWithColor(context, [[UIColor colorWithRed:0 green:0 blue:1 alpha:1] CGColor]);
            CGContextStrokeRect(context, rect);
            
            if ([annot canResize] && [self.extensionsManager annotsPermissionCanModify:annot]) {
                UIImage *dragDot = [UIImage imageNamed:@"annotation_drag.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
                NSArray *movePointArray = [ShapeUtil getMovePointInRect:rect];
                [movePointArray enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    CGRect dotRect = [obj CGRectValue];
                    CGPoint point = CGPointMake(dotRect.origin.x - 2, dotRect.origin.y -2);
                    [dragDot drawAtPoint:point];
                }];
            }
        }else{
            
            if (self.markupQuadsArrForRender == nil) {
                return;
            }
//            NSArray *tmparray = [self getCurrentSelectRects:pageIndex];
//            [tmparray enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
//                CGRect selfRect = [obj CGRectValue];
//                CGContextSetRGBFillColor(context, 1, 0, 0, 1);
//                CGContextFillRect(context, selfRect);
//            }];
            
            NSArray *array = self.markupQuadsArrForRender;//annot.quads;
            
            for (int i = 0; i < array.count; i++)
            {
//                NSArray *arrayQuad = [array objectAtIndex:i];
//                CGPoint point1 = [[arrayQuad objectAtIndex:0] CGPointValue];
//                CGPoint point4 = [[arrayQuad objectAtIndex:3] CGPointValue];
//
//                FSPointF *p1 = [[FSPointF alloc]initWithX:point1.x Y:point1.y];
//                FSPointF *p2 = [[FSPointF alloc]initWithX:point4.x Y:point4.y];
                
                FSQuadPoints *arrayQuad = [array objectAtIndex:i];
                CGPoint startPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:arrayQuad.first pageIndex:pageIndex];
                CGPoint endPoint = [self.pdfViewCtrl convertPdfPtToPageViewPt:arrayQuad.fourth pageIndex:pageIndex];
                
                CGRect pvRect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
                
                BOOL isFacingMode = [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT || [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE;
                
                if (isFacingMode) {
                    CGRect displayRect = [self.pdfViewCtrl getPageView:pageIndex].bounds;
                    pvRect = CGRectIntersection(pvRect, displayRect);
                }
                
                CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
                CGContextFillRect(context, pvRect);
                
                if ([annot canResize] && [self.extensionsManager annotsPermissionCanModify:annot]) {
                    if (i == 0)
                    {
                        UIImage *dragDot = [UIImage imageNamed:@"annotation_drag.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
                        CGRect leftCursor = CGRectMake((int)(pvRect.origin.x-7.5), (int)(pvRect.origin.y-12), 15, 17);
                        [dragDot drawAtPoint:leftCursor.origin];
                        
                    }
                    if (i == array.count - 1)
                    {
                        UIImage *dragDot = [UIImage imageNamed:@"annotation_drag.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
                        CGRect rightCursor = CGRectMake((int)(pvRect.origin.x+pvRect.size.width-7.5), (int)(pvRect.origin.y+pvRect.size.height-5), 15, 17);
                        [dragDot drawAtPoint:rightCursor.origin];
                    }
                }
            }
        }
    }
}

- (CGRect)getRectFromStartPoint:(CGPoint)startPoint endPoint:(CGPoint)endPoint {
    CGFloat xminValue = MIN(startPoint.x, endPoint.x);
    CGFloat yminValue = MIN(startPoint.y, endPoint.y);
    
    CGFloat xmaxValue = MAX(startPoint.x, endPoint.x);
    CGFloat ymaxValue = MAX(startPoint.y, endPoint.y);
    
    CGRect result = CGRectMake(xminValue, yminValue, fabs(xmaxValue-xminValue), fabs(ymaxValue-yminValue));
    return result;
}

- (int)getCharIndexAtPos:(int)pageIndex point:(CGPoint)point {
    FSPointF *dibPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    if ([textPage isEmpty]) return -1;
    return (int) [textPage getIndexAtPos:dibPoint.x y:dibPoint.y tolerance:15];
}

- (void)clearSelection {
    _startPosIndex = -1;
    _endPosIndex = -1;
//    _startPoint = nil;
//    _endPoint = nil;
//    self.arraySelectedRect = nil;
    self.currentEditRect = CGRectZero;
    self.oldEditRect = CGRectZero;
}

- (NSArray *)getCurrentSelectRects:(int)pageIndex {
    UIView *view = [self.pdfViewCtrl getPageView:pageIndex];
    CGSize size = view.frame.size;
    int offsetY = 0;
    int offsetX = 0;
    __block CGRect unionRect = CGRectZero;
    NSMutableArray *ret = [NSMutableArray array];

    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    assert(_startPosIndex != -1 && _endPosIndex != -1);
    int start = MIN(_startPosIndex, _endPosIndex);
    int count = ABS(_endPosIndex - _startPosIndex) + 1;
    NSArray *array = [Utility getTextRects:textPage start:start count:count];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        CGRect rect = [[obj objectAtIndex:0] CGRectValue];
        FSRectF *dibRect = [Utility CGRect2FSRectF:rect];
        CGRect selfRect = [self getRealRectWithOptions:pageIndex dibRect:dibRect size:size offsetY:offsetY offsetX:offsetX];
        [ret addObject:[NSValue valueWithCGRect:selfRect]];
        if (CGRectEqualToRect(unionRect, CGRectZero)) {
            unionRect = selfRect;
        } else {
            unionRect = CGRectUnion(unionRect, selfRect);
        }
    }];
    self.oldEditRect = self.currentEditRect;
    self.currentEditRect = unionRect;
    return ret;
}

- (CGRect)getRealRectWithOptions:(int)pageIndex dibRect:(FSRectF *)dibRect size:(CGSize)size offsetY:(int)offsetY offsetX:(int)offsetX {
    CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
    return rect;
}

#pragma mark - Magnifier

- (void)showMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    if (_magnifierView == nil) {
        FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
        NSArray *array = [Utility getTextRects:textPage start:index count:1];
        if (array.count > 0) {
            FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
            CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
            point = CGPointMake(point.x, CGRectGetMidY(rect));
        }
        _magnifierView = [[MagnifierView alloc] init];
        _magnifierView.viewToMagnify = [self.pdfViewCtrl getDisplayView];
        
        CGPoint locationPoint = [[self.pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[self.pdfViewCtrl getOverlayView:pageIndex]];
        _magnifierView.touchPoint = locationPoint;
        _magnifierView.magnifyPoint = [[self.pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[self.pdfViewCtrl getDisplayView]];
        
        [[self.pdfViewCtrl getOverlayView:pageIndex] addSubview:_magnifierView];
    }
}

- (void)moveMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    NSArray *array = [Utility getTextRects:textPage start:index count:1];
    if (array.count > 0) {
        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
        CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
        point = CGPointMake(point.x, CGRectGetMidY(rect));
    }
    
    CGPoint locationPoint = [[self.pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[self.pdfViewCtrl getOverlayView:pageIndex]];
    _magnifierView.touchPoint = locationPoint;
    _magnifierView.magnifyPoint = [[self.pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[self.pdfViewCtrl getDisplayView]];
    
    [_magnifierView setNeedsDisplay];
}

- (void)closeMagnifier {
    [_magnifierView removeFromSuperview];
    _magnifierView = nil;
}

@end
