/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "TextMKToolHandler.h"
#import "PaintUtility.h"

@interface MKToolHandler ()

@property (nonatomic, assign) int pageindex;
@property (nonatomic, strong) NSArray *arraySelectedRect;
@property (nonatomic, assign) CGRect currentEditRect;
@property (nonatomic, assign) CGRect oldEditRect;
@property (nonatomic, assign) int pageIndex;

@property (nonatomic, strong) PaintUtility *paintUtility;
@property (nonatomic, strong) FSPointF *startPoint;
@property (nonatomic, strong) FSPointF *endPoint;
@property (nonatomic, assign) int wordStartPosIndex;
@property (nonatomic, assign) int wordEndPosIndex;
@property (nonatomic, assign) CGRect rect;
@end

@implementation MKToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotHighlight;
        [self clearSelection];
    }
    return self;
}

- (int)getCharIndexAtPos:(int)pageIndex point:(CGPoint)point {
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    if ([textPage isEmpty]) return -1;
    return (int) [textPage getIndexAtPos:dibPoint.x y:dibPoint.y tolerance:5];
}

- (NSArray *)getCurrentSelectRects:(int)pageIndex {
    UIView *view = [_pdfViewCtrl getPageView:pageIndex];
    CGSize size = view.frame.size;
    int offsetY = 0;
    int offsetX = 0;
    __block CGRect unionRect = CGRectZero;
    NSMutableArray *ret = [NSMutableArray array];

    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    assert(_startPosIndex != -1 && _endPosIndex != -1);
    int start = MIN(_startPosIndex, _endPosIndex);
    int count = ABS(_endPosIndex - _startPosIndex) + 1;
    NSArray *array = [Utility getTextRects:textPage start:start count:count];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        CGRect rect = [[obj objectAtIndex:0] CGRectValue];
        FSRectF *dibRect = [Utility CGRect2FSRectF:rect];
        CGRect selfRect = [self getRealRectWithOptions:pageIndex dibRect:dibRect size:size offsetY:offsetY offsetX:offsetX];
        
        BOOL isFacingMode = [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE;
        
        if (isFacingMode) {
            CGRect displayRect = [_pdfViewCtrl getPageView:pageIndex].bounds;
            selfRect = CGRectIntersection(selfRect, displayRect);
        }
        
        [ret addObject:[NSValue valueWithCGRect:selfRect]];
        if (CGRectEqualToRect(unionRect, CGRectZero)) {
            unionRect = selfRect;
        } else {
            unionRect = CGRectUnion(unionRect, selfRect);
        }
    }];
    self.oldEditRect = self.currentEditRect;
    self.currentEditRect = unionRect;
    return ret;
}

- (CGRect)getRealRectWithOptions:(int)pageIndex dibRect:(FSRectF *)dibRect size:(CGSize)size offsetY:(int)offsetY offsetX:(int)offsetX {
    CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
    return rect;
}

- (void)clearSelection {
    _startPosIndex = -1;
    _endPosIndex = -1;
    _wordStartPosIndex = -1;
    _wordEndPosIndex = -1;
    _startPoint = nil;
    _endPoint = nil;
    self.arraySelectedRect = nil;
    self.currentEditRect = CGRectZero;
    self.oldEditRect = CGRectZero;
}

#pragma mark - Magnifier

- (void)showMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    if (_magnifierView == nil) {
        FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
        NSArray *array = [Utility getTextRects:textPage start:index count:1];
        if (array.count > 0) {
            FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
            CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
            point = CGPointMake(point.x, CGRectGetMidY(rect));
        }
        _magnifierView = [[MagnifierView alloc] init];
        _magnifierView.viewToMagnify = [_pdfViewCtrl getDisplayView];
        
        CGPoint locationPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getOverlayView:pageIndex]];
        _magnifierView.touchPoint = locationPoint;
        _magnifierView.magnifyPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getDisplayView]];
        
        [[_pdfViewCtrl getOverlayView:pageIndex] addSubview:_magnifierView];
    }
}

- (void)moveMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    NSArray *array = [Utility getTextRects:textPage start:index count:1];
    if (array.count > 0) {
        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
        CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
        point = CGPointMake(point.x, CGRectGetMidY(rect));
    }
    
    CGPoint locationPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getOverlayView:pageIndex]];
    _magnifierView.touchPoint = locationPoint;
    _magnifierView.magnifyPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getDisplayView]];
    
//    _magnifierView.touchPoint = point;
//    _magnifierView.magnifyPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getDisplayView]];
    [_magnifierView setNeedsDisplay];
}

- (void)closeMagnifier {
    [_magnifierView removeFromSuperview];
    _magnifierView = nil;
}

- (NSArray *)getAnnotationQuad:(FSTextMarkup *)annot {
    NSMutableArray *array = [[NSMutableArray alloc] init];
    FSTextMarkup *markup = [[FSTextMarkup alloc] initWithAnnot:annot];
    FSQuadPointsArray *quads = [markup getQuadPoints];
    unsigned long quadCount = [quads getSize];
    if (quadCount == 0) {
        return nil;
    }

    for (int i = 0; i < quadCount; i++) {
        FSQuadPoints *quadPoints = [quads getAt:i];
        if (!quadPoints) {
            goto END;
        }
        [array addObject:quadPoints];
    }

END:
    return array;
}

- (NSString *)getName {
    return Tool_Markup;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
}

- (void)addUndoFromReflow:(FSMarkup *)at type:(FSAnnotType)type{
    int pageIndex = [[at getPage] getIndex];
    FSPDFPage *page = [[_pdfViewCtrl getDoc] getPage:pageIndex];
    try {
        if(type == FSAnnotHighlight){
            at.subject = @"Highlight";
        }else if(type == FSAnnotUnderline){
            at.subject = @"Underline";
        }else if(type == FSAnnotSquiggly){
            at.subject = @"Squiggly";
        }else if(type == FSAnnotStrikeOut){
            at.subject = @"Strikeout";
        }
        at.NM = [Utility getUUID];
        at.author = _extensionsManager.annotAuthor;
          
        at.createDate = [NSDate date];
        at.modifiedDate = [NSDate date];
        at.flags = FSAnnotFlagPrint;

        [_pdfViewCtrl lockRefresh];
        [at resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        FSTask *task = [[FSTask alloc] init];
        task.run = ^() {
//            FSAnnotAttributes *attributes = [FSAnnotAttributes attributesWithAnnot:at];
            id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:at];
            [annotHandler addAnnot:at];
            [self clearSelection];

        };
        [_taskServer executeSync:task];
            
    } catch (NSException *e) {
    }
}

- (void)addAnnot:(int)pageIndex {
    FSRectF *rect = [_pdfViewCtrl convertPageViewRectToPdfRect:self.currentEditRect pageIndex:pageIndex];
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page)
        return;
    
    FSTextPage *textPage;
    NSArray *array;
    NSMutableArray *arrayQuads = [NSMutableArray array];
    
    if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
        CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:pageIndex];
        CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:pageIndex];
        
        CGRect rect = [self getRectFromStartPoint:startPoint endPoint:endPoint];

        FSPointF *first = [_pdfViewCtrl convertPageViewPtToPdfPt:CGPointMake(CGRectGetMinX(rect), CGRectGetMinY(rect)) pageIndex:pageIndex];
        FSPointF *second = [_pdfViewCtrl convertPageViewPtToPdfPt:CGPointMake(CGRectGetMaxX(rect), CGRectGetMinY(rect)) pageIndex:pageIndex];
        FSPointF *third =[_pdfViewCtrl convertPageViewPtToPdfPt:CGPointMake(CGRectGetMinX(rect), CGRectGetMaxY(rect)) pageIndex:pageIndex];
        FSPointF *fourth = [_pdfViewCtrl convertPageViewPtToPdfPt:CGPointMake(CGRectGetMaxX(rect), CGRectGetMaxY(rect)) pageIndex:pageIndex];
        
        FSQuadPoints *fsqps = [[FSQuadPoints alloc] initWithFirst:first second:second third:third fourth:fourth];
        
        [arrayQuads addObject:fsqps];
    }else{
        textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:self.pageindex];
        array = [Utility getTextRects:textPage start:MIN(self.startPosIndex, self.endPosIndex) count:ABS(self.endPosIndex - self.startPosIndex) + 1];
        for (int i = 0; i < array.count; i++) {
            FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:i] objectAtIndex:0] CGRectValue]];
            int direction = [[[array objectAtIndex:i] objectAtIndex:1] intValue];
            FSQuadPoints *fsqps = [[FSQuadPoints alloc] init];
            if (direction == 0 || direction == 4) //text is horizontal or unknown, left to right
            {
                FSPointF *first = [[FSPointF alloc] init];
                [first set:dibRect.left y:dibRect.top];
                FSPointF *second = [[FSPointF alloc] init];
                [second set:dibRect.right y:dibRect.top];
                FSPointF *third = [[FSPointF alloc] init];
                [third set:dibRect.left y:dibRect.bottom];
                FSPointF *fourth = [[FSPointF alloc] init];
                [fourth set:dibRect.right y:dibRect.bottom];
                [fsqps set:first second:second third:third fourth:fourth];
            } else if (direction == 1 || direction == 3) // test is vertical, left to right
            {
                FSPointF *first = [[FSPointF alloc] init];
                [first set:dibRect.left y:dibRect.bottom];
                FSPointF *second = [[FSPointF alloc] init];
                [second set:dibRect.left y:dibRect.top];
                FSPointF *third = [[FSPointF alloc] init];
                [third set:dibRect.right y:dibRect.bottom];
                FSPointF *fourth = [[FSPointF alloc] init];
                [fourth set:dibRect.right y:dibRect.top];
                [fsqps set:first second:second third:third fourth:fourth];
            } else if (direction == 2) //text is horizontal, right to left
            {
                FSPointF *first = [[FSPointF alloc] init];
                [first set:dibRect.right y:dibRect.bottom];
                FSPointF *second = [[FSPointF alloc] init];
                [second set:dibRect.left y:dibRect.bottom];
                FSPointF *third = [[FSPointF alloc] init];
                [third set:dibRect.right y:dibRect.top];
                FSPointF *fourth = [[FSPointF alloc] init];
                [fourth set:dibRect.left y:dibRect.top];
                [fsqps set:first second:second third:third fourth:fourth];
            }
            [arrayQuads addObject:fsqps];
        }
    }
    
    FSMarkup *annot = [[FSMarkup alloc] initWithAnnot:[page addAnnot:self.type rect:rect]];
    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.color = self.toolSettings.settingsColor;
    annot.opacity = self.toolSettings.opacity;
    annot.quads = arrayQuads;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;

    if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
        annot.intent = @"AreaHi";
    }else{
        if (textPage) {
            NSString *tmp = @"";
            for (int i = 0; i < arrayQuads.count; i++) {
                FSQuadPoints *arrayQuad = [arrayQuads objectAtIndex:i];
                FSRectF *rect = [Utility convertToFSRect:arrayQuad.getFirst p2:arrayQuad.getFourth];
                NSString *text = [textPage getTextInRect:rect];
                if (text.length > 0) {
                    tmp = [tmp stringByAppendingString:[textPage getTextInRect:rect]];
                }
            }
            [annot setContents:tmp];
        }
    }

    if (self.type == FSAnnotHighlight) {
        annot.subject = @"Highlight";
    } else if (self.type == FSAnnotSquiggly) {
        annot.subject = @"Squiggly";
    } else if (self.type == FSAnnotStrikeOut) {
        annot.subject = @"Strikeout";
    } else if (self.type == FSAnnotUnderline) {
        annot.subject = @"Underline";
    }
    
    [_pdfViewCtrl lockRefresh];
    [annot resetAppearanceStream];
    [_pdfViewCtrl unlockRefresh];

    FSTask *task = [[FSTask alloc] init];
    task.run = ^() {

        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
        [annotHandler addAnnot:annot];

        CGRect cgRect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:self.pageindex];
        cgRect = CGRectInset(cgRect, -20, -20);

        [self->_pdfViewCtrl refresh:cgRect pageIndex:self.pageindex];
        [self clearSelection];

    };
    [_taskServer executeSync:task];
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
        if (pageIndex < 0) {
        pageIndex = self.pageindex;
    }
    self.pageIndex = pageIndex;

    UIView *view = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:view];
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.rect = CGRectZero;
        self.pageindex = pageIndex;
        if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
            _startPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
            self.paintUtility = [[PaintUtility alloc] init];
            self.paintUtility.annotLineWidth = 1.f;
            self.paintUtility.annotColor = self.toolSettings.settingsColor;
            self.paintUtility.annotOpacity = self.toolSettings.opacity * 0.75;
        }else{
            [self clearSelection];
            int index = [self getCharIndexAtPos:pageIndex point:point];
            if (index > -1) {
                if ([recognizer isKindOfClass:[UILongPressGestureRecognizer class]]) {
                    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
                    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
                    FSRange *range = [textPage getWordAtPos:dibPoint.x y:dibPoint.y tolerance:10];
                    if (!range.isEmpty && range.getSegmentCount == 1) {
                        self.startPosIndex = [range getSegmentStart:0];
                        self.endPosIndex = [range getSegmentEnd:0];
                    }else{
                        FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
                        NSRange range = [Utility getWordByTextIndex:index textPage:textPage];

                        self.startPosIndex = (int) range.location;
                        self.endPosIndex = (int) (range.location + range.length - 1);
                    }
                    self.wordStartPosIndex = self.startPosIndex;
                    self.wordEndPosIndex = self.endPosIndex;
                    [self getCurrentSelectRects:pageIndex];
                    CGRect refreshRect = CGRectIsEmpty(self.oldEditRect) ? self.currentEditRect : CGRectUnion(self.currentEditRect, self.oldEditRect);
                    [_pdfViewCtrl refresh:CGRectInset(refreshRect, RECT_INSET, RECT_INSET) pageIndex:pageIndex needRender:NO];
                } else {
                    self.startPosIndex = index;
                    self.endPosIndex = index;
                }
                [self showMagnifier:pageIndex index:index point:point];
            }
        }
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (pageIndex != self.pageindex) {
            return YES;
        }
        CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
            _endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
            
            CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:pageIndex];
            CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:pageIndex];
            
            CGRect rect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
            CGRect refreshRect = CGRectInset(rect, RECT_INSET*2, RECT_INSET*2);
            [_pdfViewCtrl refresh:CGRectUnion(refreshRect, self.rect) pageIndex:pageIndex needRender:NO];
            self.rect = refreshRect;
        }else{
            int index = [self getCharIndexAtPos:pageIndex point:point];
            if (index > -1) {
                if (self.startPosIndex == -1) {
                    self.startPosIndex = index;
                    self.endPosIndex = index;
                } else {
                    if (index < self.wordStartPosIndex) {
                        self.startPosIndex = self.wordEndPosIndex;
                    }else if (index > self.wordStartPosIndex && self.wordStartPosIndex > -1){
                        self.startPosIndex = self.wordStartPosIndex;
                    }
                    self.endPosIndex = index;
                }
                [self getCurrentSelectRects:pageIndex];
                CGRect refreshRect = CGRectIsEmpty(self.oldEditRect) ? self.currentEditRect : CGRectUnion(self.currentEditRect, self.oldEditRect);
                [_pdfViewCtrl refresh:CGRectInset(refreshRect, RECT_INSET, RECT_INSET) pageIndex:pageIndex needRender:NO];
                [self moveMagnifier:pageIndex index:index point:point];
            }
        }

    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        if (self.toolSettings.settingsType != FSReadToolSettingsTypeAreaHighlight) {
            [self closeMagnifier];
            if (self.startPosIndex == -1 || self.endPosIndex == -1) {
                return YES;
            }
        }else if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight && !self.endPoint) {
            return YES;
        }
        
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        if (pageIndex != self.pageindex) {
            pageIndex = self.pageindex;
            pageview = [_pdfViewCtrl getPageView:pageIndex];
            point = [recognizer locationInView:pageview];
        }
        [self addAnnot:pageIndex];
        [self clearSelection];
        return YES;
    }
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
        return NO;
    }
    self.pageindex = pageIndex;
    UIView *view = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:view];
    [self clearSelection];
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    FSRange *range = [textPage getWordAtPos:dibPoint.x y:dibPoint.y tolerance:10];
    if (!range.isEmpty && range.getSegmentCount == 1) {
        self.startPosIndex = [range getSegmentStart:0];
        self.endPosIndex = [range getSegmentEnd:0];
        [self getCurrentSelectRects:pageIndex];
        CGRect refreshRect = CGRectIsEmpty(self.oldEditRect) ? self.currentEditRect : CGRectUnion(self.currentEditRect, self.oldEditRect);
        [_pdfViewCtrl refresh:CGRectInset(refreshRect, RECT_INSET, RECT_INSET) pageIndex:pageIndex needRender:NO];
        [self addAnnot:pageIndex];
        [self clearSelection];
    }
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (_extensionsManager.currentToolHandler != self || pageIndex != self.pageindex) {
        return;
    }
    
    if (self.toolSettings.settingsType == FSReadToolSettingsTypeAreaHighlight) {
        if (!self.startPoint || !self.endPoint) {
            return;
        }
        CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:pageIndex];
        CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:pageIndex];
        
        CGRect rect = [self getRectFromStartPoint:startPoint endPoint:endPoint];
        [self.paintUtility drawMultipleSelectWithRect:rect inContext:context];
    }else{
        if (self.startPosIndex == -1 || self.endPosIndex == -1) {
            return;
        }
        self.arraySelectedRect = [self getCurrentSelectRects:pageIndex];
        [self.arraySelectedRect enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect selfRect = [obj CGRectValue];
            CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
            CGContextFillRect(context, selfRect);
        }];
    }
}
- (CGRect)getRectFromStartPoint:(CGPoint)startPoint endPoint:(CGPoint)endPoint {
    CGFloat xminValue = MIN(startPoint.x, endPoint.x);
    CGFloat yminValue = MIN(startPoint.y, endPoint.y);
    
    CGFloat xmaxValue = MAX(startPoint.x, endPoint.x);
    CGFloat ymaxValue = MAX(startPoint.y, endPoint.y);
    
    CGRect result = CGRectMake(xminValue, yminValue, fabs(xmaxValue-xminValue), fabs(ymaxValue-yminValue));
    return result;
}


@synthesize toolSettings;

@end
