/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "TextboxToolHandler.h"
#import "StringDrawUtil.h"
#import "FSEasyDoneTextView.h"
#import "FreetextUtil.h"

@interface TextboxToolHandler () <UITextViewDelegate, IRotationEventListener, IWindowFrameChangedListener, IPageEventListener,IScrollViewEventListener, IPropertyViewEventListener, IFSUndoEventListener>{
    FSEasyDoneTextView *_textView;
}

@property (nonatomic, assign) int pageIndex;
@property (nonatomic, assign) BOOL pageIsAlreadyExist;
@property (nonatomic, strong) FSPointF *startPoint;
@property (nonatomic, strong) FSPointF *endPoint;
@property (nonatomic, assign) CGRect rect;
@property (nonatomic, strong) FSRectF *dibRect;
@property (nonatomic, assign) BOOL isPanCreate;
@property (nonatomic, strong) FSRectF *beginScrollFSRect;
@end

@implementation TextboxToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
    CGFloat minAnnotPageInset;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _pageIsAlreadyExist = NO;
        _type = FSAnnotFreeText;
        minAnnotPageInset = 5;
        [_extensionsManager registerRotateChangedListener:self];
        [_extensionsManager registerWindowFrameChangedListener:self];
        [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
        [extensionsManager.pdfViewCtrl registerScrollViewEventListener:self];
        [extensionsManager registerPropertyViewEvnetListener:self];
    }
    return self;
}

- (NSString *)getName {
    return Tool_Textbox;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
    [self save];
}

//set textView font and properties
- (UIFont*)setTextViewFont:(int)pageIndex
{
    float fontSize = self.toolSettings.textSize;
    fontSize = [Utility convertWidth:fontSize fromPDFToPageView:_pdfViewCtrl pageIndex:pageIndex];
    NSString *fontName = self.toolSettings.textFace;
    UIFont *font = [self getSysFont:fontName size:fontSize];
    if (!font) {
        font = [UIFont boldSystemFontOfSize:fontSize];
    }
    return font;
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    id<IAnnotHandler> annotHandler = nil;
    FSAnnot *annot = _extensionsManager.currentAnnot;
    if (annot != nil && ![annot isEmpty]) {
        annotHandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
        if (recognizer.state == UIGestureRecognizerStateBegan) {
            if ([annotHandler onPageViewShouldBegin:pageIndex recognizer:recognizer annot:annot]) {
                [annotHandler onPageViewPan:pageIndex recognizer:(UIPanGestureRecognizer*)recognizer annot:annot];
                return YES;
            } else {
                _extensionsManager.currentAnnot = nil;
            }
        } else {
            [annotHandler onPageViewPan:pageIndex recognizer:(UIPanGestureRecognizer*)recognizer annot:annot];
            return YES;
        }
    }
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        if (_textView) {
            [self save];
        }
        self.pageIndex = pageIndex;
        self.startPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (pageIndex != self.pageIndex) {
            return YES;
        }
        
        self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        self.dibRect = [Utility convertToFSRect:self.startPoint p2:self.endPoint];
        CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.dibRect pageIndex:pageIndex];
        rect = CGRectIntersection(rect, pageView.bounds);
        [_pdfViewCtrl refresh:CGRectUnion(rect, self.rect) pageIndex:pageIndex needRender:NO];
        self.rect = rect;
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        if (pageIndex != self.pageIndex) {
            pageIndex = self.pageIndex;
            pageView = [_pdfViewCtrl getPageView:pageIndex];
            point = [recognizer locationInView:pageView];
        }
        self.endPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        CGPoint startPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.startPoint pageIndex:self.pageIndex];
        CGPoint endPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:self.endPoint pageIndex:self.pageIndex];
        if (CGPointEqualToPoint(startPoint, endPoint)) {
            return [self onPageViewTap:pageIndex recognizer:(UITapGestureRecognizer*)recognizer];
        }
        if (!self.pageIsAlreadyExist) {
            self.pageIndex = pageIndex;
            self.pageIsAlreadyExist = YES;
        }
        
        CGRect rect = self.rect;
        _isSaved = NO;
        self.rect = rect;
        
        if (_extensionsManager.currentToolHandler == self) {
            _textView = [[FSEasyDoneTextView alloc] initWithFrame:CGRectMake(self.rect.origin.x, self.rect.origin.y, self.rect.size.width, self.rect.size.height)];
            _textView.accessibilityIdentifier = @"textboxaccessibilityID";
            [_pdfViewCtrl refresh:_textView.frame pageIndex:pageIndex needRender:NO];
            @fs_weakify(self)
            _textView.doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                [self save];
            };
            
            UIFont *annotFont = [self setTextViewFont:pageIndex];
            _textView.font = annotFont;
            //set text view properties
            [FreetextUtil setTextViewProperties:_textView UIExtensionsManager:_extensionsManager toolSettings:self.toolSettings isCallout:NO];
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(keyboardWasShown:)
                                                         name:UIKeyboardDidShowNotification
                                                       object:nil];
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(keyboardWasHidden:)
                                                         name:UIKeyboardWillHideNotification
                                                       object:nil];
            [pageView addSubview:_textView];
            _textView.delegate = self;
            [_textView becomeFirstResponder];
            self.isPanCreate = YES;
            return YES;
        }
        return YES;
    }
    return YES;
}

- (BOOL)addAnnot:(int)pageIndex point:(CGPoint)point {
    if (!self.pageIsAlreadyExist) {
        self.pageIndex = pageIndex;
        self.pageIsAlreadyExist = YES;
    }
    [self save];
    _isSaved = NO;

    if (_extensionsManager.currentToolHandler == self) {
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        UIFont* font = [self setTextViewFont:pageIndex];
        CGSize testSize = [Utility getTestSize:font];

        CGFloat pageViewWidth = [_pdfViewCtrl getPageView:pageIndex].frame.size.width;
        if(DEVICE_iPAD)
            _textView = [[FSEasyDoneTextView alloc] initWithFrame:CGRectMake(point.x, point.y, pageViewWidth - point.x >= 300 ? 300 : pageViewWidth - point.x, testSize.height)];
        else
           _textView = [[FSEasyDoneTextView alloc] initWithFrame:CGRectMake(point.x, point.y, pageViewWidth - point.x >= 100 ? 100 : pageViewWidth - point.x, testSize.height)];
        
        @weakify(self)
        _textView.doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self save];
        };
        _textView.accessibilityIdentifier = @"textboxaccessibilityID";
        _textView.font = font;
        
        [self adjustTextViewFrame:_textView inPageView:pageView forMinPageInset:minAnnotPageInset];
        
        //set text view properties
        [FreetextUtil setTextViewProperties:_textView UIExtensionsManager:_extensionsManager toolSettings:self.toolSettings isCallout:NO];
        CGRect frame = _textView.frame;
        CGSize constraintSize = CGSizeMake(frame.size.width, MAXFLOAT);
        CGSize size = [_textView sizeThatFits:constraintSize];
        size.height += 3;
        _textView.frame = CGRectMake(frame.origin.x, frame.origin.y, frame.size.width, size.height);
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasShown:)
                                                     name:UIKeyboardDidShowNotification
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasHidden:)
                                                     name:UIKeyboardWillHideNotification
                                                   object:nil];
        
        [pageView addSubview:_textView];
        _textView.delegate = self;
        [_textView becomeFirstResponder];
    }
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    CGPoint point = CGPointZero;
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    if (recognizer) {
        point = [recognizer locationInView:pageView];
    } else {
        point = _freeTextStartPoint;
    }
    return [self addAnnot:pageIndex point:point];
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (_extensionsManager.currentToolHandler != self) {
        return;
    }
    if (!_textView) {
        CGRect selfRect = self.rect;
        CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
        CGContextFillRect(context, selfRect);
    }
}

- (UIFont *)getSysFont:(NSString *)name size:(float)size {
    UIFont *font = [UIFont fontWithName:[Utility convert2SysFontString:name] size:size];
    if (!font) {
        font = [UIFont systemFontOfSize:size];
    }
    return font;
}

#pragma mark - UITextViewDelegate

- (void)textViewDidChange:(UITextView *)textView {
    UIView *pageView = [_pdfViewCtrl getPageView:self.pageIndex];
    if (pageView) {
        CGRect frame = textView.frame;
        CGSize constraintSize = CGSizeMake(frame.size.width, MAXFLOAT);
        CGSize size = [textView sizeThatFits:constraintSize];
        size.height += 3;
        if (self.isPanCreate) {
            if (size.height < frame.size.height) {
                size.height = frame.size.height;
            }
        }

        textView.frame = CGRectMake(frame.origin.x, frame.origin.y, frame.size.width, size.height);
        if (textView.frame.size.height >= (CGRectGetHeight(pageView.frame) - 20)) {
            [textView endEditing:YES];
        }
        if (textView.frame.size.width >= (CGRectGetWidth(pageView.frame) - 20)) {
            [textView endEditing:YES];
        }
        [self adjustTextViewFrame:textView inPageView:pageView forMinPageInset:minAnnotPageInset];
    }
}

- (void)textViewDidEndEditing:(UITextView *)textView {
    if (textView == _textView) {
        [self save];
    }
    
}

- (void)adjustTextViewFrame:(UITextView *)textView inPageView:(UIView *)pageView forMinPageInset:(CGFloat)inset {
    CGRect bounds = CGRectInset(pageView.bounds, inset, inset);
    if (!CGRectIntersectsRect(textView.frame, bounds)) {
        return;
    }
    if (CGRectGetMinX(textView.frame) < CGRectGetMinX(bounds)) {
        CGPoint center = textView.center;
        center.x += CGRectGetMinX(bounds) - CGRectGetMinX(textView.frame);
        textView.center = center;
    }
    if (CGRectGetMaxX(textView.frame) > CGRectGetMaxX(bounds)) {
        CGPoint center = textView.center;
        center.x -= CGRectGetMaxX(textView.frame) - CGRectGetMaxX(bounds);
        textView.center = center;
    }
    if (CGRectGetMinY(textView.frame) < CGRectGetMinY(bounds)) {
        CGPoint center = textView.center;
        center.y += CGRectGetMinY(bounds) - CGRectGetMinY(textView.frame);
        textView.center = center;
    }
    if (CGRectGetMaxY(textView.frame) > CGRectGetMaxY(bounds)) {
        CGPoint center = textView.center;
        center.y -= CGRectGetMaxY(textView.frame) - CGRectGetMaxY(bounds);
        textView.center = center;
    }
}

#pragma mark - keyboard

- (void)keyboardWasShown:(NSNotification *)notification {
    if (_keyboardShown) {
        return;
    }
    _keyboardShown = YES;
    
    [FreetextUtil afterKeyboardShown:notification textView:_textView UIExtensionsManager:_extensionsManager pageIndex:self.pageIndex];
}

- (void)keyboardWasHidden:(NSNotification *)notification {
    _keyboardShown = NO;
    
    [FreetextUtil adjustBottomOffset:_pdfViewCtrl pageIndex:self.pageIndex];
}

- (void)onPageChangedFrom:(int)oldIndex to:(int)newIndex {
    [self save];
}

- (void)save {
    if (_textView && !_isSaved) {
        _isSaved = YES;

        NSString *checker = [NSString stringWithFormat:@"%C", 8198];
        if ([_textView.text rangeOfString:checker].length) {
            _textView.text  = [_textView.text stringByReplacingOccurrencesOfString:checker withString:@""];
        }
        
        CGRect textFrame = _textView.frame;
        NSString *content = _textView.text;
        
        FSRectF *rect = [_pdfViewCtrl convertPageViewRectToPdfRect:textFrame pageIndex:self.pageIndex];
        if (ABS(rect.left - rect.right) < 1E-5 || ABS(rect.top - rect.bottom) < 1E-5) {
            return;
        }
        FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:self.pageIndex];
        if (!page || [page isEmpty] || (textFrame.size.width < 1 && textFrame.size.height < 1)) {
            return;
        }
        FSFreeText *annot = [[FSFreeText alloc] initWithAnnot:[page addAnnot:FSAnnotFreeText rect:rect]];
        annot.NM = [Utility getUUID];
        annot.author = _extensionsManager.annotAuthor;
        
        [annot setDefaultAppearance:({
            FSDefaultAppearance *appearance = [annot getDefaultAppearance];
            appearance.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
            NSString *fontName = self.toolSettings.textFace;
            int fontID = [Utility toStandardFontID:fontName];
            if (fontID == -1) {
                [appearance setFont:[[FSFont alloc] initWithName:fontName styles:0 charset:FSFontCharsetDefault weight:0]];
            } else {
                [appearance setFont:[[FSFont alloc] initWithFont_id:fontID]];
            }
            [appearance setText_size:self.toolSettings.textSize];
            unsigned int color = self.toolSettings.settingsColor;
            [appearance setText_color:color];
            appearance;
        })];
        annot.opacity = self.toolSettings.opacity;
        annot.contents = content;
        annot.createDate = [NSDate date];
        annot.modifiedDate = [NSDate date];
        annot.subject = @"Textbox";
        annot.flags = FSAnnotFlagPrint;
        [annot setBorderColor:_extensionsManager.config.defaultSettings.annotations.textbox.color.rgbValue];
        
        int rotation = ([page getRotation] +[_pdfViewCtrl getViewRotation]) % 4;
        rotation = rotation == 0 ? rotation : 4 - rotation;
        annot.rotation = rotation ;
        [annot allowTextOverflow:NO];
        [_pdfViewCtrl lockRefresh];
        [annot resetAppearanceStream];
        [_pdfViewCtrl unlockRefresh];
        // move annot if exceed page edge, as the annot size may be changed after reset ap (especially for Chinese char, the line interspace is changed)
        {
            FSRectF *rect = annot.fsrect;
            if (rect.bottom < 0) {
                rect.top -= rect.bottom;
                rect.bottom = 0;
                annot.fsrect = rect;
                [_pdfViewCtrl lockRefresh];
                [annot resetAppearanceStream];
                [_pdfViewCtrl unlockRefresh];
            }
        }
        [annot allowTextOverflow:YES];
        if (annot && ![annot isEmpty]) {
            id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
            [annotHandler addAnnot:annot addUndo:YES];
        }

        [_textView resignFirstResponder];
        [_textView removeFromSuperview];
        _textView = nil;
        self.rect = CGRectZero;
        self.isPanCreate = NO;
        self.pageIsAlreadyExist = NO;

        [[NSNotificationCenter defaultCenter] removeObserver:self];
    }
}

// useless ?
- (UIView *)hitTest:(CGPoint)point withEvent:(UIEvent *)event onView:(UIView *)view {
    if (_textView != nil) {
        CGPoint pt = [_textView convertPoint:point fromView:view];
        if (CGRectContainsPoint(_textView.bounds, pt)) {
            return _textView;
        }
    }
    return nil;
}

#pragma mark <IFSUndoEventListener>

- (void)onWillRedo {
    [self save];
}

- (void)onWillUndo {
    [self save];
}

#pragma mark -- IRotationEventListener
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation{
    [self save];
}

#pragma mark -- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD) {
        [self save];
    }
}

#pragma mark -- IPageEventListener
- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex{
    [self save];
}

#pragma mark <IScrollViewEventListener>
- (void)onScrollViewWillBeginZooming:(UIScrollView *)scrollView {
    if (_textView) {
        self.beginScrollFSRect = [_pdfViewCtrl convertPageViewRectToPdfRect:_textView.frame pageIndex:_pdfViewCtrl.getCurrentPage];
    }
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView {

    if (_textView && self.beginScrollFSRect) {
        
        CGRect textFrame = [_pdfViewCtrl convertPdfRectToPageViewRect:self.beginScrollFSRect pageIndex:_pdfViewCtrl.getCurrentPage];
        int pageIndex = _pdfViewCtrl.getCurrentPage;
        UIFont *annotFont = [self setTextViewFont:pageIndex];
        _textView.frame = textFrame;
        _textView.font = annotFont;
        [[_pdfViewCtrl getPageView:_pdfViewCtrl.getCurrentPage] addSubview:_textView];
    }
}

#pragma mark --IPropertyViewEventListener
- (void)onPropertyViewDidShow{
    [self save];
}

@synthesize toolSettings;

@end
