/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSThumbnailCell.h"
#import "FSThumbnailView.h"
#import "FSThumbnailViewController.h"
#import "FSInsertPageConfigVCtrl.h"
#import "FSThumbnailChildViewController.h"
#import "PhotoToPDFViewController.h"
#import "FSMediaFileToPDFViewController.h"
#import "FSPermissionUtilty.h"

typedef NS_ENUM(NSUInteger, FSThumbnailSegmentShowType) {
    FSThumbnailSegmentShowTypeAll = 0,
    FSThumbnailSegmentShowTypeAnnoted,
    FSThumbnailSegmentShowTypeBookmark,
};

static NSArray<NSNumber *> *arrayFromRange(NSRange range);
static NSArray<NSIndexPath *> *indexPathsFromRange(NSRange range);

@interface FSThumbnailViewController () <FSInsertPageConfigVCtrlDelegate, FSMediaFileToPDFViewControllerDelegate, IFSUIExtensionsModifiedEventListener>

@property (nonatomic, assign) FSThumbnailSegmentShowType segmentShowType;

@property (nonatomic, strong) UIView *editBarContainer;
@property (nonatomic, strong) FSToolbar *editSubTopBar;
@property (nonatomic, strong) FSToolbar *editSubBottomBar;

@property (nonatomic, strong) FSItem *editSubSelectAllItem;
@property (nonatomic, strong) FSItem *editSubTitleItem;
@property (nonatomic, strong) FSItem *editSubDoneItem;
@property (nonatomic, strong) FSItem *editItem;

@property (nonatomic, strong) FSItem *editSubInsertItem;
@property (nonatomic, strong) FSItem *editSubDeleteItem;
@property (nonatomic, strong) FSItem *editSubDuplicateItem;
@property (nonatomic, strong) FSItem *editSubPasteItem;
@property (nonatomic, strong) FSItem *editSubLeftRotateItem;
@property (nonatomic, strong) FSItem *editSubRightRotateItem;
@property (nonatomic, strong) FSItem *editSubExtractItemItem;
@property (nonatomic, strong) FSItem *editSubShareItemItem;

@property (nonatomic, strong) UIView *segmentContainer;
@property (nonatomic, strong) FSItem *segmentCurrentItem;
@property (nonatomic, strong) NSArray<FSItem *> *segmentItems;

@property (nonatomic, strong) FSThumbnailChildViewController *thumbnailChildViewControllerAll;
@property (nonatomic, strong) FSThumbnailChildViewController *thumbnailChildViewControllerAnnoted;
@property (nonatomic, strong) FSThumbnailChildViewController *thumbnailChildViewControllerBookmark;

@property (nonatomic, strong) NSMutableArray *bookmarkPageIndexs;
@property (nonatomic, strong) FSPDFDoc *copyedDoc;

@property (nonatomic, strong, readwrite) FSViewControllerManager *services;

// misc
@property (nonatomic) NSInteger insertIndex;
@property (nonatomic, assign) BOOL needPauseNow;
@end

@implementation FSThumbnailViewController

@dynamic services;

- (instancetype)initWithServices:(id<FSViewControllerServices>)services{
    self = [super initWithServices:services];
    if (self) {
        self.extensionsManager = self.services.extensionsManager;
        self.pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        
        self.delegate = nil;
        self.document = self.pdfViewCtrl.currentDoc;
        self.isEditing = NO;

        self.segmentShowType = FSThumbnailSegmentShowTypeAll;
        
        self.bookmarkPageIndexs = @[].mutableCopy;
        [self loadReadingBookmarkPageIndex];
        self.currentThumbnailChildViewController = self.thumbnailChildViewControllerAll;
        
        [self.extensionsManager registerExtensionsModifiedEventListener:self];
    }
    return self;
}

- (void)dealloc{
    [self.extensionsManager unregisterExtensionsModifiedEventListener:self];
}

- (UIView *)editBarContainer{
    if(!_editBarContainer){
        _editBarContainer = [UIView new];
        
        self.editSubTopBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleDefault];
        
        FSItem *tmpItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kSelectAll")];
        tmpItem.font = self.fs_standardGeneralFont_GF2;
        [tmpItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
        @weakify(self)
        tmpItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            _item.selected = !_item.selected;
            [self selectOrDeselectAll];
        };
        [BindThemeKeySel(tmpItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateSelected];
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        
        [self.editSubTopBar addSubview:tmpItem];
        self.editSubSelectAllItem = tmpItem;
        
        tmpItem = [FSItem createHeadlineItemWithTitle:FSLocalizedForKey(@"kSelected")];
        tmpItem.font = self.fs_standardGeneralFont_GF2;
        tmpItem.accessibilityTraits = UIAccessibilityTraitNone;
        tmpItem.customAccessibilityTraits = UIAccessibilityTraitNone;
        self.editSubTitleItem = tmpItem;
        [self.editSubTopBar addItem:tmpItem atPosition:FSItemPositionCenter];
        
        tmpItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kDone")];
        tmpItem.font = self.fs_standardGeneralFont_GF2;
        tmpItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            self.isEditing = NO;
        };
        [BindThemeKeySel(tmpItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateSelected];
        }];
        
        [self.editSubTopBar addItem:tmpItem atPosition:FSItemPositionRight];
        
        [_editBarContainer addSubview:self.editSubTopBar];
        
        [self.editSubTopBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.fs_equalToSuperview();
            make.height.offset(TOOL_BAR_HEIGHT_NORMAL);
        }];
        
        self.editSubBottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleDefault];
        
        NSArray *tmpImageNames = @[@"thumbnail_add", @"thumbnail_rotate_left", @"thumbnail_rotate_right", @"thumbnail_extract", @"thumbnail_copy",@"thumbnail_paste", @"common_delete_gray"];
        if (_extensionsManager.thumbnailModule.showShareExtactItem) {
            tmpImageNames = @[@"thumbnail_add", @"thumbnail_rotate_left", @"thumbnail_rotate_right", @"thumbnail_extract", @"thumbnail_copy",@"thumbnail_paste", @"common_delete_gray", @"thumbnail_share"];
        }
        for (int i = 0; i < tmpImageNames.count; i ++) {
            FSItem *item = [FSItem createItemWithImage:FSImageNamedAndNormalColor(tmpImageNames[i])];
            item.enabled = NO;
            [self.editSubBottomBar addItem:item spacing:ITEM_TOP_ICON_MARGIN atPosition:FSItemPositionCenter];
            @weakify(self)
            if (i == 0) {
                item.accessibilityLabel = FSLocalizedForKey(@"kAddPage");
                item.enabled = (self.segmentShowType == FSThumbnailSegmentShowTypeAll);
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    self.insertIndex = [self.pdfViewCtrl getCurrentPage]+1;
                    [self showInsertMenu:_item];
                };
                self.editSubInsertItem = item;
            }
            else if (i == 1) {
                item.accessibilityLabel = FSLocalizedForKey(@"kLeftRaotatePageVoiceOver");
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self rotateSelectedWithClockwise:NO];
                    _item.enabled = self.collectionView.indexPathsForSelectedItems.count > 0;
                };
                self.editSubLeftRotateItem = item;
            } else if (i == 2) {
                item.accessibilityLabel = FSLocalizedForKey(@"kRightRaotatePageVoiceOver");
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self rotateSelectedWithClockwise:YES];
                    _item.enabled = self.collectionView.indexPathsForSelectedItems.count > 0;
                };
                self.editSubRightRotateItem = item;
            }else if (i == 3) {
                item.accessibilityLabel = FSLocalizedForKey(@"kExtractPageVoiceOver");
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self extractSelected];
                };
                self.editSubExtractItemItem = item;
            }else if (i == 4) {
                item.accessibilityLabel = FSLocalizedForKey(@"kCopyPageVoiceOver");
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self.view fs_scheduledMainQueueShowHUDLoading:FSLocalizedForKey(@"kCopying")
                                         process:^{
                        [self.currentThumbnailChildViewController copyDocForSelectedPageIndexs];
                        if (!self.copyedDoc) return;
                        [self onCellSelectedOrDeselected];
                        self.editSubPasteItem.enabled = YES;
                        _item.enabled = NO;
                    }];
                };
                self.editSubDuplicateItem = item;
            }else if (i == 5) {
                item.accessibilityLabel = FSLocalizedForKey(@"kPastePageVoiceOver");
                item.enabled = NO;
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self.currentThumbnailChildViewController insertCopyItem];
                    [self onCellSelectedOrDeselected];
                    self.editSubDuplicateItem.enabled = NO;
                    _item.enabled = NO;
                };
                self.editSubPasteItem = item;
            }else if (i == 6) {
                item.accessibilityLabel = FSLocalizedForKey(@"kDeleteVoiceOver");
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self deleteSelected];
                };
                self.editSubDeleteItem = item;
            }else if (i == 7) {
                item.accessibilityLabel = FSLocalizedForKey(@"kShareVoiceOver");
                item.onTapClick = ^(FSItem * _Nonnull _item) {
                    @strongify(self)
                    [self shareExtactDoc];
                };
                self.editSubShareItemItem = item;
            }
        }
        
        [_editBarContainer addSubview:self.editSubBottomBar];
        [self.editSubBottomBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.editSubTopBar.mas_bottom);
            make.left.right.bottom.fs_equalToSuperview();
        }];
        
    }
    return _editBarContainer;
}

- (FSThumbnailChildViewController *)thumbnailChildViewControllerAll{
    if(!_thumbnailChildViewControllerAll){
        _thumbnailChildViewControllerAll = [[FSThumbnailChildViewController alloc] initWithServices:self.services params:@(FSThumbnailChildTypeAll)];
        _thumbnailChildViewControllerAll.thumbnailParentViewController = self;
        _thumbnailChildViewControllerAll.bookmarkPageIndexs = self.bookmarkPageIndexs;
    }
    return _thumbnailChildViewControllerAll;
}

- (FSThumbnailChildViewController *)thumbnailChildViewControllerAnnoted{
    if(!_thumbnailChildViewControllerAnnoted){
        _thumbnailChildViewControllerAnnoted = [[FSThumbnailChildViewController alloc] initWithServices:self.services params:@(FSThumbnailChildTypeAnnoted)];
        _thumbnailChildViewControllerAnnoted.thumbnailParentViewController = self;
        _thumbnailChildViewControllerAnnoted.bookmarkPageIndexs = self.bookmarkPageIndexs;
    }
    return _thumbnailChildViewControllerAnnoted;
}

- (FSThumbnailChildViewController *)thumbnailChildViewControllerBookmark{
    if(!_thumbnailChildViewControllerBookmark){
        _thumbnailChildViewControllerBookmark = [[FSThumbnailChildViewController alloc] initWithServices:self.services params:@(FSThumbnailChildTypeBookmark)];
        _thumbnailChildViewControllerBookmark.thumbnailParentViewController = self;
        _thumbnailChildViewControllerBookmark.bookmarkPageIndexs = self.bookmarkPageIndexs;
    }
    return _thumbnailChildViewControllerBookmark;
}

- (UICollectionView *)collectionView{
    return self.currentThumbnailChildViewController.collectionView;
}

- (FSPDFDoc *)copyedDoc{
    return self.currentThumbnailChildViewController.copyedDoc;
}

- (BOOL)hasPasteItemInChildViewController{
    return _thumbnailChildViewControllerAll.needPasteItem || _thumbnailChildViewControllerAnnoted.needPasteItem || _thumbnailChildViewControllerBookmark.needPasteItem;
}

- (int)getPageCount{
    BOOL isDynamciXFA = [_pdfViewCtrl isDynamicXFA];
    if (!isDynamciXFA) {
        return [self.document getPageCount];
    }
    return [[_pdfViewCtrl getXFADoc] getPageCount];
}

- (void)makeConstraints{
    [super makeConstraints];
    
    [self.segmentContainer mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.segmentContainer.superview.fs_mas_left).offset(ITEM_MARGIN_NORMAL).priorityHigh();
        make.right.equalTo(self.segmentContainer.superview.fs_mas_right).offset(-ITEM_MARGIN_NORMAL).priorityHigh();
        make.top.equalTo(self.navBar.mas_bottom).offset(ITEM_MARGIN_NORMAL);
        make.centerX.fs_equalToSuperview();
        make.width.mas_lessThanOrEqualTo(MIN_BAR_WIDTH_IN_VIEW(self.view));
        make.height.offset(28.f);
    }];
    [self makeCurrentThumbnailChildViewControllerViewConstraints];

}

- (void)makeCurrentThumbnailChildViewControllerViewConstraints{
    UIView *target = self.currentThumbnailChildViewController.view;
    [target mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(target.superview.fs_mas_left);
        make.right.equalTo(target.superview.fs_mas_right);
        make.bottom.equalTo(target.superview.fs_mas_bottom);
        make.top.equalTo(self.segmentContainer.mas_bottom).offset(ITEM_MARGIN_NORMAL);
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    CGRect rect = CGRectMake(0, 0, self.pdfViewCtrl.frame.size.width, self.pdfViewCtrl.frame.size.height);
    [self.view setFrame:rect];
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    self.title = FSLocalizedForKey(@"kThumbnail");

    if ([FSPermissionUtilty canAssemble:_extensionsManager] && ![_pdfViewCtrl.currentDoc isXFA]) {
        FSItem *editItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kEdit")];
        editItem.font = self.fs_standardGeneralFont_GF2;
        [BindThemeKeySel(editItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        @weakify(self)
        editItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self.extensionsManager.permissionProvider checkPermission:FSFunctionEdit checkPermissionState:^(FSPermissionState state) {
                if (state == FSPermissionStateShow) {
                    self.isEditing = YES;
                }
            }];
        };
        self.editItem = editItem;
        self.editItem.hidden = ([self.extensionsManager.permissionProvider checkPermission:FSFunctionEdit] == FSPermissionStateHide);
        [self.navBar addItem:editItem atPosition:FSItemPositionRight];
    }
    
    [self buildSegment];
    
    [self addChildViewController:self.currentThumbnailChildViewController];
    [self.view addSubview:_currentThumbnailChildViewController.view];
    [_currentThumbnailChildViewController didMoveToParentViewController:self];
    
    self.view.backgroundColor = self.collectionView.backgroundColor;
}

- (void)buildSegment{
    UIView *segmentContainer = [UIView new];
    [segmentContainer fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_1];
    segmentContainer.backgroundColor = self.collectionView.backgroundColor;
    self.segmentContainer = segmentContainer;
    [self.view addSubview:self.segmentContainer];
    segmentContainer.layer.cornerRadius = 7;
    segmentContainer.clipsToBounds = YES;
    
    NSArray *tmp = @[FSLocalizedForKey(@"kAll"), FSLocalizedForKey(@"kAnnotated"), FSLocalizedForKey(@"kBookmarked") ];
    NSMutableArray *segmentItems = @[].mutableCopy;
    for (int i = 0; i < tmp.count; i++) {
        FSItem *item = [FSItem createItemWithTitle:tmp[i]];
        item.tag = i;
        item.titleLabel.textAlignment = NSTextAlignmentCenter;
        item.font = self.fs_standardGeneralFont_GF3;
        [item setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [item setTitleColor:self.fs_iconColor_I1 forState:UIControlStateSelected];
        [item setBackgroundImage:[UIImage fs_imageWithColor:self.fs_backgroundColor_B2 size:CGSizeMake(1, 1)] forState:UIControlStateNormal | UIControlStateHighlighted];
        
        @weakify(self)
        item.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if (self.segmentCurrentItem != _item) {
                self.segmentCurrentItem.selected = NO;
                self.segmentCurrentItem = _item;
                self.segmentCurrentItem.selected = YES;
                self.segmentShowType = _item.tag;
                
                self.editSubInsertItem.enabled = NO;
                FSThumbnailChildViewController *currentVC = self.currentThumbnailChildViewController;
                [currentVC willMoveToParentViewController:nil];
                [currentVC.view removeFromSuperview];
                [currentVC removeFromParentViewController];
                
                
                if (self.segmentShowType == FSThumbnailSegmentShowTypeAll) {
                    self.currentThumbnailChildViewController = self.thumbnailChildViewControllerAll;
                }
                else if (self.segmentShowType == FSThumbnailSegmentShowTypeAnnoted) {
                    self.currentThumbnailChildViewController = self.thumbnailChildViewControllerAnnoted;
                }
                else if (self.segmentShowType == FSThumbnailSegmentShowTypeBookmark) {
                    self.currentThumbnailChildViewController = self.thumbnailChildViewControllerBookmark;
                }
                
                self.currentThumbnailChildViewController.isEditing = currentVC.isEditing;
                currentVC = self.currentThumbnailChildViewController;
                
                [self addChildViewController:currentVC];
                [self.view addSubview:currentVC.view];
                [currentVC didMoveToParentViewController:self];
                
                [self makeCurrentThumbnailChildViewControllerViewConstraints];

                [self onCellSelectedOrDeselected];
            }
        };
        
        [BindThemeKeySel(item, setBackgroundImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setBackgroundImage:[UIImage fs_imageWithColor:target.fs_primaryColor size:CGSizeMake(1, 1)] forState:UIControlStateSelected];
        }];
        [self.segmentContainer addSubview:item];
        
        FSItem *lastItem = [segmentItems lastObject];
        [item mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.bottom.fs_equalToSuperview();
            if (i == tmp.count - 1) {
                make.right.fs_equalToSuperview();
            }
            if (lastItem) {
                make.left.equalTo(lastItem.mas_right).offset(DIVIDE_VIEW_THICKNES);
                make.width.equalTo(lastItem);
            }else{
                make.left.fs_equalToSuperview();
            }
        }];
        if (i == 0) {
            item.selected = YES;
            self.segmentCurrentItem = item;
        }
        [segmentItems addObject:item];
    }
    self.segmentItems = segmentItems.copy;
}

- (void)resetTopBar{
    if (self.isEditing) {
        self.navBar.contentBarHeight = TOOL_BAR_HEIGHT_NORMAL * 2;
        [self.navBar addSubview:self.editBarContainer];
        self.editBarContainer.accessibilityViewIsModal = YES;
        [self.editBarContainer mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.top.right.bottom.fs_equalToSuperview();
        }];
    }else{
        [self.editBarContainer removeFromSuperview];
        self.navBar.contentBarHeight = TOOL_BAR_HEIGHT_NORMAL;
    }
}

- (void)loadReadingBookmarkPageIndex{
    [self.bookmarkPageIndexs removeAllObjects];
    int readingBookmarkCount = [self.document getReadingBookmarkCount];
    for (int i = 0; i < readingBookmarkCount; i++) {
        FSReadingBookmark *bookmark = [self.document getReadingBookmark:i];
        if (![self.bookmarkPageIndexs containsObject:@(bookmark.pageIndex)]) {
            [self.bookmarkPageIndexs addObject:@(bookmark.pageIndex)];
        }
    }
}

- (void)reloadThumbnailChildIndexPathsForSelectedItems:(NSArray<NSIndexPath *> *)indexPathsForSelectedItems{
    
    NSArray<NSIndexPath *> *selectedIndexPaths = indexPathsForSelectedItems;
    NSMutableArray *pageIndexs = @[].mutableCopy;
    for (NSIndexPath *selectedIndexPath in selectedIndexPaths) {
        NSNumber *pageIndex = @([self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:selectedIndexPath.item]);
        [pageIndexs addObject:pageIndex];
    }
    
    void (^block)(FSThumbnailChildViewController *childViewController) =^(FSThumbnailChildViewController *childViewController){
        if (pageIndexs.count) {
            NSMutableArray <NSIndexPath *> *tmp = @[].mutableCopy;
            for (NSIndexPath *visibleItems in childViewController.collectionView.indexPathsForVisibleItems) {
                if ([pageIndexs containsObject:@([childViewController getPageIndexInCollectionViewAtItem:visibleItems.item])]) {
                    [tmp addObject:visibleItems];
                    if (tmp.count == pageIndexs.count) {
                        break;
                    }
                }
            }
            if (tmp.count) {
                NSArray<NSIndexPath *> *selectedIndexPaths = childViewController.collectionView.indexPathsForSelectedItems;
                [childViewController.collectionView reloadItemsAtIndexPaths:tmp];
    
                [childViewController.collectionView performBatchUpdates:^{
                    [selectedIndexPaths enumerateObjectsUsingBlock:^(NSIndexPath *_Nonnull obj, NSUInteger idx, BOOL *_Nonnull stop) {
                        [childViewController.collectionView selectItemAtIndexPath:obj animated:NO scrollPosition:UICollectionViewScrollPositionNone];
                    }];
                } completion:nil];
            }
        }
    };
    if (self.thumbnailChildViewControllerAll) {
        block(self.thumbnailChildViewControllerAll);
    }
    if (_thumbnailChildViewControllerAnnoted) {
        block(_thumbnailChildViewControllerAnnoted);
    }
    if (_thumbnailChildViewControllerBookmark) {
        block(_thumbnailChildViewControllerBookmark);
    }
}

- (void)thumbnailChildInsertPages:(NSArray<NSIndexPath *> *)insertPages atIndex:(NSUInteger)index{
    void (^block)(FSThumbnailChildViewController *childViewController) =^(FSThumbnailChildViewController *childViewController){
        [childViewController insertPagesInIndexPaths:insertPages atIndex:index];
    };
    [self loadReadingBookmarkPageIndex];
    
    block(_thumbnailChildViewControllerAll);
    block(_thumbnailChildViewControllerAnnoted);
    block(_thumbnailChildViewControllerBookmark);
}

#pragma mark tool bars

- (void)navigationBarBackTouch{
    if (self.delegate != nil && [(NSObject*)self.delegate respondsToSelector:@selector(exitThumbnailViewController:)]) {
        [self.delegate exitThumbnailViewController:self];
    }
}

#pragma mark event handler

- (void)setIsEditing:(BOOL)isEditing {
    if (_isEditing != isEditing) {
        _isEditing = isEditing;
        //update tool bars
        [self resetTopBar];

        [self.currentThumbnailChildViewController setIsEditing:isEditing];
        
        if (!isEditing) {
            [self thumbnailChildViewControllerDeleteCopyItem];
        }
    }
}

- (void)thumbnailChildViewControllerDeleteCopyItem{
    [_thumbnailChildViewControllerAll deleteCopyItem];
    [_thumbnailChildViewControllerAnnoted deleteCopyItem];
    [_thumbnailChildViewControllerBookmark deleteCopyItem];
}

- (void)selectOrDeselectAll {
    [self.currentThumbnailChildViewController selectOrDeselectAll];
}

- (void)rotateSelectedWithClockwise:(BOOL)clockwise{
    NSArray<NSIndexPath *> *selectedIndexPaths = self.collectionView.indexPathsForSelectedItems;
    NSMutableArray *pageIndexs = @[].mutableCopy;
    for (NSIndexPath *selectedIndexPath in selectedIndexPaths) {
        NSNumber *pageIndex = @([self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:selectedIndexPath.item]);
        [pageIndexs addObject:pageIndex];
    }
    if (pageIndexs.count > 0) {
        [self cancelDrawingThumbnailsAtIndexPaths:selectedIndexPaths];
        if ([self.pageManipulationDelegate rotatePagesAtIndexes:pageIndexs clockwise:clockwise]) {
            [self reloadThumbnailChildIndexPathsForSelectedItems:selectedIndexPaths];
        } else {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailRotatePage") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
        }
    }
}

- (void)deleteSelected {
    NSArray *indexPaths = self.collectionView.indexPathsForSelectedItems;
    if (indexPaths.count == 0) {
        [self onCellSelectedOrDeselected];
        return;
    } else if (indexPaths.count == [self getPageCount]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kNotAllowDeleteAll") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
        return;
    }

    void (^deletePages)(void) = ^{
        NSMutableArray <NSNumber *> *pageIndexs = @[].mutableCopy;
        for (NSIndexPath *indexPath in indexPaths) {
            [pageIndexs addObject:@([self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:indexPath.item])];
        }
        [self willDeleteChildViewControllerAtPageIndexs:pageIndexs];
        [self cancelChildViewControllerDrawingThumbnails];
        
        [self.view fs_showHUDLoading:FSLocalizedForKey(@"kDeleting") process:^{
            dispatch_semaphore_t wait = dispatch_semaphore_create(0);
            dispatch_async(dispatch_get_main_queue(), ^{
                BOOL bDeleted = [self.pageManipulationDelegate deletePagesAtIndexes:pageIndexs];
                if (!bDeleted) {
                    [self convenientAlert:FSLocalizedForKey(@"kFailDelete")];
                    [self onCellSelectedOrDeselected];
                    return;
                }
                self.editSubPasteItem.enabled = NO;
                [self loadReadingBookmarkPageIndex];
                [self didDeleteChildViewControllerAtPageIndexs:pageIndexs];
                [self onCellSelectedOrDeselected];
                dispatch_semaphore_signal(wait);
            });
            
            dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
        }];
        

    };
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kSureToDeletePage") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                           style:UIAlertActionStyleCancel
                                                         handler:^(UIAlertAction *_Nonnull action) {
        [self onCellSelectedOrDeselected];
    }];
    UIAlertAction *deleteAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *_Nonnull action) {
        deletePages();
    }];
    [alertController addAction:cancelAction];
    [alertController addAction:deleteAction];
    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
    

}

- (void)extractSelected {
    id<FSMenuView> customView = [self.services.extensionsManager.menuViewManager getMenuViewForMenuViewType:FSMenuViewTypeExtract];
    if (customView) {
        if ([customView respondsToSelector:@selector(shouldPresentByRDK)]) {
            if (![customView shouldPresentByRDK]) {
                if ( [customView respondsToSelector:@selector(presentActionInMenuView)]) {
                    [customView presentActionInMenuView];
                }
                return;
            }
        }
        if ([customView respondsToSelector:@selector(getCustomView)]) {
            customView.containerController =  [self.services presentViewCtrlType:FSViewCtrlTypeCustomViewContainer params:[customView getCustomView]];
        }
        return;
    }
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.services];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
    selectDestination.expectFileType = @[@"*"];
    selectDestination.titleForTypeSelectFolder = FSLocalizedForKey(@"kExtract");
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        [controller dismissViewControllerAnimated:YES completion:^{
            if (destinationFolder.count == 0)
                return;
            
            __block void (^inputFileName)(NSString *path) = ^(NSString *path) {
                BOOL isDir = NO;
                NSFileManager *fileManager = [NSFileManager defaultManager];
                if (![fileManager fileExistsAtPath:path isDirectory:&isDir]) {
                    return;
                }
                if (!isDir) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                               style:UIAlertActionStyleCancel
                                                                             handler:^(UIAlertAction *action) {
                                                                                 dispatch_async(dispatch_get_main_queue(), ^{
                                                                                     inputFileName([path stringByDeletingLastPathComponent]);
                                                                                 });
                                                                             }];
                        [alertController addAction:cancelAction];
                        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                    });
                    return;
                }
                
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInputNewFileName") preferredStyle:UIAlertControllerStyleAlert];
                [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
                    textField.text = @"";
                }];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                 style:UIAlertActionStyleDefault
                                                               handler:^(UIAlertAction *action) {
                                                                   UITextField *textField = alertController.textFields[0];
                                                                   NSString *newName = textField.text;
                                                                   if (newName.length < 1) {
                                                                       UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInputNewFileName") preferredStyle:UIAlertControllerStyleAlert];
                                                                       UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
                                                                           dispatch_async(dispatch_get_main_queue(), ^{
                                                                               inputFileName(path);
                                                                           });
                                                                       }];
                                                                       [alertController addAction:okAction];
                                                                       [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                                                   } else if ([fileManager fileExistsAtPath:[path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.pdf", newName]]]) {
                                                                       dispatch_async(dispatch_get_main_queue(), ^{
                                                                           UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                                                                           UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                                                                                  style:UIAlertActionStyleCancel
                                                                                                                                handler:^(UIAlertAction *action) {
                                                                                                                                    inputFileName(destinationFolder[0]);
                                                                                                                                }];
                                                                           UIAlertAction *OKAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                                                                                  style:UIAlertActionStyleDefault
                                                                                                                                handler:^(UIAlertAction *action) {
                                                                                                                                    dispatch_async(dispatch_get_main_queue(), ^{
                                                                                                                                        [fileManager removeItemAtPath:[path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.pdf", newName]] error:nil];
                                                                                                                                        [self extractSelectedToPath:[path stringByAppendingString:[NSString stringWithFormat:@"/%@.pdf", newName]]];
                                                                                                                                        inputFileName = nil;
                                                                                                                                    });
                                                                                                                                }];
                                                                           [alertController addAction:cancelAction];
                                                                           [alertController addAction:OKAction];
                                                                           [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                                                       });
                                                                   } else {
                                                                       [self extractSelectedToPath:[path stringByAppendingString:[NSString stringWithFormat:@"/%@.pdf", newName]]];
                                                                       inputFileName = nil;
                                                                   }
                                                               }];
                [alertController addAction:cancelAction];
                [alertController addAction:action];
                [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
            };
            
            inputFileName(destinationFolder[0]);
        }];
        
    };
    [self.services presentViewController:selectDestination animated:YES completion:nil];
}

- (void)movePageIndexs:(NSArray <NSNumber *> *)pageIndexs toIndex:(NSUInteger)index beforeOrAfter:(BOOL)before{
    if (self.currentThumbnailChildViewController != self.thumbnailChildViewControllerAll) {
        if (before) {
            if (index == 0) {
                index = [self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:index];
            }
        }else{
            index = [self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:index-1]+1;
        }
        
    }
    BOOL isOK = [self.pageManipulationDelegate movePagesFromIndexes:pageIndexs toIndex:index];
    if (!isOK) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailMovePage") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
    }else{
        [self moveChildViewControllerPageIndexs:pageIndexs];
    }
}

- (void)extractSelectedToPath:(NSString *)destinationPath {
    
    [self.view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
        FSPDFDoc *doc = [self.currentThumbnailChildViewController getDocForSelectedPageIndexs];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            if (!doc) {
                [self convenientAlert:FSLocalizedForKey(@"kFailExtract")];
            } else {
                NSDate *date = [NSDate date];
                FSMetadata *metadata = [[FSMetadata alloc] initWithDocument:doc];
                [metadata setCreationDateTime:[Utility convert2FSDateTime:date]];
                if (![doc saveAs:destinationPath save_flags:FSPDFDocSaveFlagNormal]) {
                    [self convenientAlert:FSLocalizedForKey(@"kFailExtract")];
                } else {
                    [self convenientAlert:FSLocalizedForKey(@"kSaveSuccess")];
                }
            }
            [self onCellSelectedOrDeselected];
        });

    }];
    
    
}

- (void)duplicatePages:(void (^)(NSString *))completionBlock {

    NSArray<NSNumber *> *sourcePagesIndexes = arrayFromRange(NSMakeRange(0, [self.copyedDoc getPageCount]));
    [self.pageManipulationDelegate insertPagesFromDocument:self.copyedDoc withSourceIndexes:sourcePagesIndexes flags:FSPDFDocImportFlagNormal layerName:nil atIndex:[self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:self.currentThumbnailChildViewController.insertIndex] success:^(NSString * _Nonnull result) {
        NSUInteger pageIndex = [self.currentThumbnailChildViewController getPageIndexInCollectionViewAtItem:self.currentThumbnailChildViewController.insertIndex];
        [self thumbnailChildInsertPages:indexPathsFromRange(NSMakeRange(pageIndex, sourcePagesIndexes.count)) atIndex:self.currentThumbnailChildViewController.insertIndex];
        completionBlock ? completionBlock(nil) : nil;
    } error:^(NSString * _Nonnull errorInfo) {
        if ([errorInfo isEqualToString:@"FSErrUnsupported"]) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kUnsupportedDocFormat") preferredStyle:UIAlertControllerStyleAlert];

            UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:okAction];

            [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
        }else{
            [self convenientAlert:FSLocalizedForKey(@"kFailCopy")];
            completionBlock ? completionBlock(errorInfo) : nil;
        }
    }];
}

- (void)shareExtactDoc
{
    [self.view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
        FSPDFDoc *doc = [self.currentThumbnailChildViewController getDocForSelectedPageIndexs];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            if (!doc) {
                [self convenientAlert:FSLocalizedForKey(@"kFailExtract")];
            } else {
                NSString *filePath = self.pdfViewCtrl.filePath;
                NSString *tempDir = NSTemporaryDirectory();
                NSString *intermediateFilePath = [tempDir stringByAppendingPathComponent:[filePath lastPathComponent]];
                NSDate *date = [NSDate date];
                FSMetadata *metadata = [[FSMetadata alloc] initWithDocument:doc];
                [metadata setCreationDateTime:[Utility convert2FSDateTime:date]];
                if (![doc saveAs:intermediateFilePath save_flags:FSPDFDocSaveFlagNormal]) {
                    [self convenientAlert:FSLocalizedForKey(@"kFailExtract")];
                } else {
                    dispatch_main_async_safe(^{
                        NSURL *shareURL = [NSURL fileURLWithPath:intermediateFilePath];
                        NSArray *activityItems = [[NSArray alloc] initWithObjects:shareURL, nil];

                        UIActivityViewController *vc = [[UIActivityViewController alloc] initWithActivityItems:activityItems applicationActivities:nil];

                        UIActivityViewControllerCompletionWithItemsHandler myBlock = ^(UIActivityType activityType, BOOL completed, NSArray *returnedItems, NSError *activityError) {
                            [vc dismissViewControllerAnimated:YES completion:nil];
                        };
                        vc.completionWithItemsHandler = myBlock;
                        if ([vc respondsToSelector:@selector(popoverPresentationController)]) {vc.popoverPresentationController.sourceView = self.editSubShareItemItem;}
                        vc.completionWithItemsHandler = myBlock;
                        [self.pdfViewCtrl.fs_viewController presentViewController:vc animated:YES completion:^{

                        }];
                    });
                }
            }
            [self onCellSelectedOrDeselected];
        });

    }];
}

- (FSPDFDoc *)getCurrnetChildViewOfDocForSelectedPageIndexs{
    return [self.currentThumbnailChildViewController getDocForSelectedPageIndexs];
}

#pragma mark - InsertPageConfigVCtrlDelegate

- (void)insertPageSucceed:(FSPDFDoc *)document count:(int)count{
    
    NSMutableArray<NSIndexPath *> *indexPaths = [[NSMutableArray alloc] init];
    int index = (int)self.insertIndex;
    for (int i = index; i < index + count; i++) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForItem:i inSection:0];
        [indexPaths addObject:indexPath];
    }
    [self thumbnailChildInsertPages:indexPaths atIndex:index];
}

#pragma mark insert pages and images

- (void)showInsertMenu:(UIView *)view {
    
    id<FSMenuView> customView = [self.extensionsManager.menuViewManager getMenuViewForMenuViewType:FSMenuViewTypeThumbnailInsertBlankPagePop];
    if (customView) {
        if ([customView respondsToSelector:@selector(shouldPresentByRDK)]) {
            if (![customView shouldPresentByRDK]) {
                if ( [customView respondsToSelector:@selector(presentActionInMenuView)]) {
                    [customView presentActionInMenuView];
                }
                return;
            }
        }
        if ([customView respondsToSelector:@selector(getCustomView)]) {
            customView.containerController = [self.extensionsManager.viewCtrlManager popPopoverCustomView:[customView getCustomView] sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
                popver.permittedArrowDirections = UIPopoverArrowDirectionUp;
                return view;
            } : nil];
        }
        return;
    }
    
    NSArray <FSMenuItemGroup *> *groups = [self.extensionsManager.menuViewManager getMenuItemGroupsForMenuViewType:FSMenuViewTypeThumbnailInsertBlankPagePop];
    
    NSMutableArray *tmpGroups = @[].mutableCopy;
    for (int i = 0; i < groups.count; i++) {
        FSMenuItemGroup *group = groups[i];
        
        NSMutableArray *tmpItems = @[].mutableCopy;
        for (FSMenuItem *customItem in group.items) {
            @weakify(self)
            [tmpItems addObject:[[FSPopoverItem alloc] initWithTitle:customItem.title image:customItem.image action:^(FSPopoverItem * _Nonnull item, int index) {
                if (customItem.action) {
                    customItem.action(customItem);
                }else{
                    @strongify(self)
                    if (customItem.actionTag != -1) {
                        [self popoverItemActionForActionTag:customItem.actionTag];
                    }
                }
            }]];
        }
        
        [tmpGroups addObject:[[FSPopoverItemGroup alloc] initWithTitle:group.title items:tmpItems]];
    }
    
    if (!tmpGroups.count) {
        return;
    }
    [self->_extensionsManager.viewCtrlManager popPopoverWithGroups:tmpGroups sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
        popver.permittedArrowDirections = UIPopoverArrowDirectionUp;
        return view;
    } : nil];
}

- (void)popoverItemActionForActionTag:(NSInteger)tag{
    if (tag == 0) {
        FSInsertPageConfigVCtrl *insertPageConfigVCtrl = [self.services presentViewCtrlType:FSViewCtrlTypeInsertPageConfig];
        insertPageConfigVCtrl.delegate = self;
    }else if (tag == 1){
        FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
        mediaFileToPDFViewController.delegate = self;
        if (DEVICE_iPAD) {
            mediaFileToPDFViewController.modalPresentationStyle = UIModalPresentationFormSheet;
        }else{
            mediaFileToPDFViewController.modalPresentationStyle = UIModalPresentationFullScreen;
        }
        [self presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
        [mediaFileToPDFViewController openAlbum];
    }else if (tag == 2){
        FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
        mediaFileToPDFViewController.delegate = self;
        [self presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
        [mediaFileToPDFViewController openCamera];
    }else if (tag == 3){
        
    }else if (tag == 4){
        FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.services];
        selectDestination.isRootFileDirectory = YES;
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFile;
        selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeImport;
        selectDestination.expectFileType = @[ @"pdf", @"jbig2", @"jpx", @"tif", @"gif", @"png", @"jpg", @"bmp" ];
        [selectDestination loadFilesWithPath:DOCUMENT_PATH];
        selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
            [controller.services dismissViewControllerAnimated:YES completion:^{
                if (destinationFolder.count > 0) {
                    NSString *srcPath = destinationFolder[0];
                    if ([[Utility isSupportImageFormats] containsObject:srcPath.pathExtension.lowercaseString]) {
                        UIImage *image = [UIImage imageWithContentsOfFile:srcPath];
                        if (image) {
                            [self insertPageFromImage:image atIndex:(int)self.insertIndex];
                        }
                    } else {
                        [self insertPages:srcPath success:nil error:nil];
                    }
                }
            }];
        };
        [self.extensionsManager.viewCtrlManager presentViewController:selectDestination animated:YES completion:nil];
    }
}

- (void)insertPages:(NSString *)path success:(void (^)(NSString *result))success error:(void (^)(NSString *errorInfo))error{
    [self insertPages:path atIndex:(int)self.insertIndex success:success error:error];
}

- (void)insertPages:(NSString *)path atIndex:(int)index success:(void (^)(NSString *result))success error:(void (^)(NSString *errorInfo))error {
    void (^insertPagesFromLoadedDocument)(FSPDFDoc *document) = ^(FSPDFDoc *document) {
        NSArray<NSNumber *> *sourcePagesIndexes = arrayFromRange(NSMakeRange(0, [document getPageCount]));
        [self.pageManipulationDelegate insertPagesFromDocument:document withSourceIndexes:sourcePagesIndexes flags:FSPDFDocImportFlagNormal layerName:nil atIndex:index success:^(NSString * result) {
            NSMutableArray<NSIndexPath *> *indexPaths = [[NSMutableArray alloc] init];
            for (int i = index; i < index + [document getPageCount]; i++) {
                NSIndexPath *indexPath = [NSIndexPath indexPathForItem:i inSection:0];
                [indexPaths addObject:indexPath];
            }
            [self thumbnailChildInsertPages:indexPaths atIndex:index];
            ! success ? : success(result);
        } error:^(NSString * _Nonnull errorInfo) {
            if ([errorInfo isEqualToString:@"FSErrUnsupported"]) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kUnsupportedDocFormat") preferredStyle:UIAlertControllerStyleAlert];
                
                UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                [alertController addAction:okAction];
                
                [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
            }else{
                [self convenientAlert:FSLocalizedForKey(@"kFailInsert")];
            }
            ! error ? : error(errorInfo);
        }];
    };

    [self.view fs_scheduledMainQueueShowHUDLoading:FSLocalizedForKey(@"kInserting") process:^{
        FSPDFDoc *srcDoc = [[FSPDFDoc alloc] initWithPath:path];
        [srcDoc load:nil];
        if ([srcDoc isWrapper] || [Utility checkFileisRMS:path password:@""]) {
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kNonSupportDoc"), [path lastPathComponent]] actions:@[FSLocalizedForKey(@"kOK")] actionBack:nil onCtrl:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
        }else{
            srcDoc = [[FSPDFDoc alloc] initWithPath:path];
            [Utility tryLoadDocument:srcDoc
                withPassword:@""
                success:^(NSString *password) {
                    insertPagesFromLoadedDocument(srcDoc);
                }
                error:^(NSString *description) {
                    [self convenientAlert:description];
                }
                abort:nil sceneProduct:self];

            // reset selectall
            [self onCellSelectedOrDeselected];
        }
    }];
}

- (void)insertPageFromImage:(UIImage *)image atIndex:(int)index {
    [self.view fs_scheduledMainQueueShowHUDLoading:FSLocalizedForKey(@"kInserting") process:^{
        if ([self.pageManipulationDelegate insertPageFromImage:image atIndex:index]) {
            NSIndexPath *indexPath = [NSIndexPath indexPathForItem:index inSection:0];
            [self thumbnailChildInsertPages:@[indexPath] atIndex:index];
        } else {
            [self convenientAlert:FSLocalizedForKey(@"kFailInsert")];
        }
        // reset selectall
        [self onCellSelectedOrDeselected];
    }];
}

- (void)convenientAlert:(NSString *)title {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:nil preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                           style:UIAlertActionStyleCancel
                                                         handler:nil];
    [alertController addAction:cancelAction];
    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - inner methods

- (void)onCellSelectedOrDeselected {
    int selectedCount = (int) self.collectionView.indexPathsForSelectedItems.count;
    NSUInteger currentItems = [self.currentThumbnailChildViewController getNumberOfItemsInCollectionView] - (self.currentThumbnailChildViewController.needPasteItem ? 1 : 0);
    BOOL isAllPagesSelected = (selectedCount == currentItems);
    self.editSubSelectAllItem.enabled = currentItems > 0 ? YES : NO;
    self.editSubSelectAllItem.selected = isAllPagesSelected;

    if (self.isEditing) {
        [self.editSubTitleItem setTitle:[NSString stringWithFormat:FSLocalizedForKey(@"kFormatSelected"),selectedCount] forState:UIControlStateNormal];
    }

    BOOL isAnyCellSelected = selectedCount > 0;
    self.editSubPasteItem.enabled = self.copyedDoc ? YES : NO;
    if (self.hasPasteItemInChildViewController) {
        isAnyCellSelected = NO;
        self.editSubSelectAllItem.enabled = isAnyCellSelected;
        self.editSubPasteItem.enabled = isAnyCellSelected;
    }
    self.editSubDuplicateItem.enabled = isAnyCellSelected;
    if (self.currentThumbnailChildViewController == _thumbnailChildViewControllerBookmark) {
        self.editSubDuplicateItem.enabled = NO;
        self.editSubPasteItem.enabled = NO;
    }
    if (self.currentThumbnailChildViewController.needPasteItem){
        isAnyCellSelected = NO;
        self.editSubInsertItem.enabled = isAnyCellSelected;
    }else{
        if (self.segmentShowType == FSThumbnailSegmentShowTypeAll) {
            self.editSubInsertItem.enabled = !self.hasPasteItemInChildViewController;
        }
    }
    self.editSubDeleteItem.enabled = isAnyCellSelected;
    self.editSubLeftRotateItem.enabled = isAnyCellSelected;
    self.editSubRightRotateItem.enabled = isAnyCellSelected;
    self.editSubExtractItemItem.enabled = isAnyCellSelected && [Utility canExtractContentsInDocument:self.document];
    self.editSubShareItemItem.enabled = isAnyCellSelected && [Utility canExtractContentsInDocument:self.document];
    
    if (self.editBarContainer.superview) {
        CGSize titleItemSize = [Utility getTextSize:self.editSubTitleItem.currentTitle fontSize:self.editSubTitleItem.titleLabel.font.pointSize maxSize:CGSizeMake(300, 40)];
        CGFloat leftItemWidth = SCREEN_WIDTH/2 - titleItemSize.width/2 - 16*2;
        [self.editSubSelectAllItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.editSubSelectAllItem.superview).offset(16);
            make.centerY.mas_equalTo(self.editSubSelectAllItem.superview);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(leftItemWidth).priorityMedium();
            make.right.lessThanOrEqualTo(self.editSubTitleItem.mas_left).offset(-10);
        }];
    }
}

- (void) moveChildViewControllerPageIndexs:(NSArray<NSNumber *> *)pageIndexs{
    [self loadReadingBookmarkPageIndex];
    void (^block)(FSThumbnailChildViewController *childViewController) =^(FSThumbnailChildViewController *childViewController){
        if (childViewController != self.currentThumbnailChildViewController) {
            [childViewController movePageIndexsAfterReload];
        }
    };
    block(_thumbnailChildViewControllerAll);
    block(_thumbnailChildViewControllerAnnoted);
    block(_thumbnailChildViewControllerBookmark);
}

- (void) willDeleteChildViewControllerAtPageIndexs:(NSArray<NSNumber *> *)pageIndexs{
    [_thumbnailChildViewControllerAll willDeletePageIndexs:pageIndexs];
    [_thumbnailChildViewControllerAnnoted willDeletePageIndexs:pageIndexs];
    [_thumbnailChildViewControllerBookmark willDeletePageIndexs:pageIndexs];
}
- (void) didDeleteChildViewControllerAtPageIndexs:(NSArray<NSNumber *> *)pageIndexs{
    [_thumbnailChildViewControllerAll didDeletePageIndexs:pageIndexs];
    [_thumbnailChildViewControllerAnnoted didDeletePageIndexs:pageIndexs];
    [_thumbnailChildViewControllerBookmark didDeletePageIndexs:pageIndexs];
}

- (void)cancelChildViewControllerDrawingThumbnails {
    [_thumbnailChildViewControllerAll cancelDrawingThumbnails];
    [_thumbnailChildViewControllerAnnoted cancelDrawingThumbnails];
    [_thumbnailChildViewControllerBookmark cancelDrawingThumbnails];
}

- (void)cancelDrawingThumbnailsAtIndexPaths:(NSArray<NSIndexPath *> *)indexPaths {
    [self.currentThumbnailChildViewController cancelDrawingThumbnailsAtIndexPaths:indexPaths];
}

#pragma mark <IRotationEventListener>

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    [_thumbnailChildViewControllerAll.collectionView.collectionViewLayout invalidateLayout];
    [_thumbnailChildViewControllerAnnoted.collectionView.collectionViewLayout invalidateLayout];
    [_thumbnailChildViewControllerBookmark.collectionView.collectionViewLayout invalidateLayout];
    if (self.editBarContainer.superview) {
        CGSize titleItemSize = [Utility getTextSize:self.editSubTitleItem.currentTitle fontSize:self.editSubTitleItem.titleLabel.font.pointSize maxSize:CGSizeMake(300, 40)];
        CGFloat leftItemWidth = SCREEN_WIDTH/2 - titleItemSize.width/2 - 16*2;
        [self.editSubSelectAllItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.editSubSelectAllItem.superview).offset(16);
            make.centerY.mas_equalTo(self.editSubSelectAllItem.superview);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(leftItemWidth);
        }];
    }
}
#pragma mark <FSMediaFileToPDFViewControllerDelegate>
- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectedImage:(UIImage *)image{
    if (image) {
        [self insertPageFromImage:image atIndex:(int)self.insertIndex];
    }
}
- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectedVideo:(AVPlayer *)video{}
#pragma mark <IFSUIExtensionsModifiedEventListener>
- (void)extensions:(UIExtensionsManager *)extensions modifiedEventDealSel:(SEL)sel args:(NSArray *)args{
    if (extensions != self.extensionsManager) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            void (^block)(FSThumbnailChildViewController *childViewController) =^(FSThumbnailChildViewController *childViewController){
                [childViewController cancelDrawingThumbnails];
                [extensions.thumbnailModule clearThumbnailCachesForPDFAtPath:extensions.pdfViewCtrl.filePath];
                [childViewController reloadData];
            };
            block(self->_thumbnailChildViewControllerAll);
            block(self->_thumbnailChildViewControllerAnnoted);
            block(self->_thumbnailChildViewControllerBookmark);
        });
    }
}
@end


static NSArray<NSNumber *> *arrayFromRange(NSRange range) {
    if (range.location == NSNotFound || range.length == 0) {
        return nil;
    }
    NSMutableArray<NSNumber *> *array = [NSMutableArray<NSNumber *> arrayWithCapacity:range.length];
    for (NSUInteger i = range.location; i < range.location + range.length; i++) {
        [array addObject:@(i)];
    }
    return array;
}

static NSArray<NSIndexPath *> *indexPathsFromRange(NSRange range) {
    if (range.location == NSNotFound || range.length == 0) {
        return nil;
    }
    NSMutableArray<NSIndexPath *> *indexPaths = [NSMutableArray<NSIndexPath *> arrayWithCapacity:range.length];
    for (NSUInteger i = range.location; i < range.location + range.length; i++) {
        [indexPaths addObject:[NSIndexPath indexPathForItem:i inSection:0]];
    }
    return indexPaths;
}
