/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ThumbnailModule+Private.h"
#import "FSThumbnailViewController.h"
#import "FSSettingBar+private.h"
#import "MultipleSelectionToolHandler.h"

@interface ThumbnailModule ()<FSThumbnailViewControllerDelegate>
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong, nullable) FSThumbnailViewController *thumbnailViewController;
@property (nonatomic, strong) NSMutableSet *thumbnailListeners;
@end
@implementation ThumbnailModule


- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _thumbnailListeners = [[NSMutableSet alloc] init];
        [self loadModule];
    }
    return self;

}

- (FSPDFViewCtrl *)pdfViewCtrl{
    return self.extensionsManager.pdfViewCtrl;
}

- (void)loadModule {
    self.thumbnailCache = [[FSThumbnailCache alloc] initWithUIExtenionsManager:self.extensionsManager];
    [self.pdfViewCtrl registerPageEventListener:self.thumbnailCache];
    [self.extensionsManager registerAnnotEventListener:self.thumbnailCache];
    
    FSMainToolbarItem *thumbnailItem = [self.extensionsManager.toolbarManager addMainItemWithType:FSMainToolbarItemTypeThumbnail];
    [thumbnailItem.item addTarget:self action:@selector(didClickedMainItem:) forControlEvents:UIControlEventTouchUpInside];
}

- (void)didClickedMainItem:(FSItem *)mainItem {
    if (self.extensionsManager.currentToolHandler) {
        [self.extensionsManager setCurrentToolHandler:nil];
    }
    if (self.extensionsManager.currentAnnot) {
        [self.extensionsManager setCurrentAnnot:nil];
    }
    if ([self.extensionsManager.currentToolHandler isKindOfClass:[MultipleSelectionToolHandler class]] ||
        [self.extensionsManager.currentToolHandler isKindOfClass:[FillSignToolHandler class]]) {
        [self.extensionsManager setCurrentToolHandler:nil];
    }
    [self.extensionsManager.thumbnailModule showThumbnailView];
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingThumbnial];
}

#pragma mark - IModule
- (FSModuleStringName)getName {
    return Module_Thumbnail;
}

#pragma mark thumbnail

- (void)showThumbnailView {
    if (!self.thumbnailViewController) {
        self.thumbnailViewController = [[FSThumbnailViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
        self.thumbnailViewController.delegate = self;
        self.thumbnailViewController.pageManipulationDelegate = self.pdfViewCtrl;
        self.thumbnailViewController.pdfViewCtrl = self.pdfViewCtrl;
        self.thumbnailViewController.extensionsManager = self.extensionsManager;
        [self.extensionsManager registerRotateChangedListener:self.thumbnailViewController];
    }

    [self.extensionsManager cancelAutoFullScreen];
    [self.pdfViewCtrl addSubview:self.thumbnailViewController.view];
    [UIView transitionWithView:self.pdfViewCtrl duration:0.8 options:UIViewAnimationOptionTransitionFlipFromRight animations:nil completion:^(BOOL finished) {
        if (UIAccessibilityIsVoiceOverRunning()) self.thumbnailViewController.view.accessibilityViewIsModal = YES;
    }];
    for (id<IThumbnailEventListener> listener in self.thumbnailListeners) {
        if ([listener respondsToSelector:@selector(onThumbnailChanged:)]) {
            [listener onThumbnailChanged:NO];
        }
    }
    if (self.extensionsManager.shouldShowMenu) {
        [self.extensionsManager hideMenu];
        self.extensionsManager.shouldShowMenu = YES;
    }
    
}

- (BOOL)isThumbnailViewHidden{
    if (self.thumbnailViewController && self.thumbnailViewController.view.superview) {
        return NO;
    }else{
        return YES;
    }
}

- (FSPDFDoc *)getCurrnetDocForSelectedPageIndexs{
    return [self.thumbnailViewController getCurrnetChildViewOfDocForSelectedPageIndexs];
}

- (void)improtDocumentFromPath:(NSString *)path success:(void (^)(NSString *result))success error:(void (^)(NSString *errorInfo))error{
    [self.thumbnailViewController insertPages:path success:success error:error];
}

- (void)removeThumbnailView{
    if (self.thumbnailViewController) {
        [self.extensionsManager unregisterRotateChangedListener:self.thumbnailViewController];
        [self.thumbnailViewController.view removeFromSuperview];
        self.thumbnailViewController = nil; //to delete
    }
    
    [self.extensionsManager delayAutoFullScreen];
    for (id<IThumbnailEventListener> listener in self.thumbnailListeners) {
        if ([listener respondsToSelector:@selector(onThumbnailChanged:)]) {
            [listener onThumbnailChanged:YES];
        }
    }
}

- (void)removeThumbnailCacheOfPageAtIndex:(NSUInteger)pageIndex {
    [self.thumbnailCache removeThumbnailCacheOfPageAtIndex:pageIndex];
}

- (void)clearThumbnailCachesForPDFAtPath:(NSString *)path {
    [self.thumbnailCache clearThumbnailCachesForPDFAtPath:path];
}

#pragma mark <FSThumbnailViewControllerDelegate>

- (void)exitThumbnailViewController:(FSThumbnailViewController *)thumbnailViewController {
    [self removeThumbnailView];
    [UIView transitionWithView:self.pdfViewCtrl duration:0.8 options:UIViewAnimationOptionTransitionFlipFromLeft animations:nil completion:nil];
    [self.extensionsManager.settingBar updateLayoutButtonsWithLayout:[self.pdfViewCtrl getPageLayoutMode]];
}

- (void)thumbnailViewController:(FSThumbnailViewController *)thumbnailViewController openPage:(int)page {
    [self removeThumbnailView];
    [self.pdfViewCtrl gotoPage:page animated:NO];
    [UIView transitionWithView:self.pdfViewCtrl duration:0.8 options:UIViewAnimationOptionTransitionFlipFromLeft animations:nil completion:nil];
    [self.extensionsManager.settingBar updateLayoutButtonsWithLayout:[self.pdfViewCtrl getPageLayoutMode]];
}

- (void)thumbnailViewController:(FSThumbnailViewController *)thumbnailViewController getThumbnailForPageAtIndex:(NSUInteger)index thumbnailSize:(CGSize)thumbnailSize needPause:(BOOL (^__nullable)(void))needPause callback:(void (^__nonnull)(UIImage *))callback {
    [self.thumbnailCache getThumbnailForPageAtIndex:index withThumbnailSize:thumbnailSize needPause:needPause callback:callback];
}

- (void)registerThumbnailEventListener:(id<IThumbnailEventListener>)listener
{
    [self.thumbnailListeners addObject:listener];
}

- (void)unregisterThumbnailEventListener:(id<IThumbnailEventListener>)listener
{
    if ([self.thumbnailListeners containsObject:listener]) {
        [self.thumbnailListeners removeObject:listener];
    }
}
@end
