/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSUndoGroup.h"
#import "FSAnnotHandler.h"
#import "RedactAnnotHandler.h"

@interface FSUndoGroup ()

@property (nonatomic, strong) NSMutableArray<FSAnnotAttributes *> *attrArray;
@property (nonatomic, strong) FSPDFDoc *pdfDoc;
@end

@implementation FSUndoGroup

- (NSArray<NSNumber*> *)getIndexArray
{
    return self.pageArray;
}

- (void)removePageIndex:(NSArray<NSNumber*> *)pageIndexs
{
    if (!pageIndexs || pageIndexs.count < 0) {
        return;
    }
    NSMutableArray *removePageArray = [NSMutableArray array];
    NSMutableArray *removeAttrArray = [NSMutableArray array];
    for (int i=0; i < pageIndexs.count; i++) {
        NSNumber *removePageIndex = [pageIndexs objectAtIndex:i];
        for (int j=0; j<self.pageArray.count; j++) {
            NSNumber *pageIndex = [self.pageArray objectAtIndex:j];
            if ([pageIndex intValue] == [removePageIndex intValue]) {
                [removePageArray addObject:pageIndex];
                if (j < self.attrArray.count) {
                    [removeAttrArray addObject:[self.attrArray objectAtIndex:j]];
                }
            }
        }
    }
    if (removePageArray.count > 0) {
        [self.pageArray removeObjectsInArray:removePageArray];
    }
    if (removeAttrArray.count > 0) {
        [self.attrArray removeObjectsInArray:removeAttrArray];
    }
}

- (void)removePageAttributes:(FSAnnotAttributes*)attributes{
    int totalCount = (int)self.attrArray.count;
    for (int i=0; i < totalCount; i++) {
        FSAnnotAttributes *attr = [self.attrArray objectAtIndex:i];
        if ([attributes.NM isEqualToString:attr.NM] && attributes.pageIndex == attr.pageIndex) {
            [self.attrArray removeObjectAtIndex:i];
            if (self.pageArray.count > i) {
                [self.pageArray removeObjectAtIndex:i];
            }
            break;
        }
    }
}

+ (instancetype)itemWithUndo:(UndoBlock)undo redo:(RedoBlock)redo pageIndexs:(NSArray<NSNumber*> *)pageIndexs attributesArray:(NSArray<FSAnnotAttributes *> *)attributesArray  curDoc:(FSPDFDoc*)curDoc{
    return [[FSUndoGroup alloc] initWithUndo:undo redo:redo pageIndexs:pageIndexs attributesArray:attributesArray curDoc:curDoc];
}


- (instancetype)initWithUndo:(UndoBlock)undo redo:(RedoBlock)redo pageIndexs:(NSArray<NSNumber*> *)pageIndexs attributesArray:(NSArray<FSAnnotAttributes *> *)attributesArray curDoc:(FSPDFDoc*)curDoc
{
    if (self = [super init]) {
        self.undo = undo;
        self.redo = redo;
        self.pageArray = [pageIndexs mutableCopy];
        self.attrArray = [attributesArray mutableCopy];
    }
    return self;
}

+ (instancetype)itemForUndoAddRdactAnnotsWithAttributes:(NSArray<FSAnnotAttributes*> *)attributesArray pageArray:(NSArray<NSNumber*> *)pageArray annotHandler:(RedactAnnotHandler*)annotHandler curDoc:(FSPDFDoc*)curDoc{
    
    if (attributesArray && attributesArray.count>0 && pageArray && pageArray.count>0 && annotHandler) {
        
        return [FSUndoGroup itemWithUndo:^(UndoItem *undoItem) {
            
            if (![undoItem isKindOfClass:[FSUndoGroup class]]) {
                return;
            }
            FSUndoGroup *item = (FSUndoGroup*)undoItem;
            
            [[annotHandler.pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                NSInteger count = item.attrArray.count;
                NSMutableArray<FSAnnot*> *annotArray = [NSMutableArray array];
                for (int i=0; i<count; i++) {
                    FSPDFPage* page = nil;
                    FSAnnot *annot = nil;
                    FSAnnotAttributes* attributes = [item.attrArray objectAtIndex:i];
                    if (i < item.pageArray.count) {
                        NSNumber* pageIndex = [item.pageArray objectAtIndex:i];
                        page = [curDoc getPage:[pageIndex intValue]];
                    }
                    
                    if (attributes && page && ![page isEmpty]) {
                        annot = [Utility getAnnotByNM:attributes.NM inPage:page];
                        
                        if (annot && ![annot isEmpty]) {
                            [annotArray addObject:annot];
                        }
                    }
                    
                }
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (annotArray.count > 0) {
                        if ([annotHandler isKindOfClass:[RedactAnnotHandler class]]) {
                            [annotHandler removeAnnots:annotArray addUndo:NO];
                        }
                    }
                });

            }];
            
        } redo:^(UndoItem *undoItem) {
            
            if (![undoItem isKindOfClass:[FSUndoGroup class]]) {
                return;
            }
            FSUndoGroup *item = (FSUndoGroup*)undoItem;
            
            [[annotHandler.pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                NSInteger count = item.attrArray.count;
                NSMutableArray<FSAnnot*> *annotArray = [NSMutableArray array];
                for (int i=0; i<count; i++) {
                    FSPDFPage* page = nil;
                    FSAnnot *annot = nil;
                    FSAnnotAttributes* attributes = [item.attrArray objectAtIndex:i];
                    if (i < item.pageArray.count) {
                        NSNumber * pageIndex = [item.pageArray objectAtIndex:i];
                        page = [curDoc getPage:[pageIndex intValue]];
                    }
                    
                    BOOL isReply = (attributes.type == FSAnnotNote && ((FSNoteAttributes *) attributes).replyTo.length > 0);
                    BOOL isRedact = (attributes.type == FSAnnotRedact);
                    BOOL isForm = (attributes.type == FSAnnotWidget);
                    if (isRedact) {
                        @try {
                            FSRedaction *redaction = [[FSRedaction alloc] initWithDocument:page.getDocument];
                            FSRedact *redact = [redaction markRedactAnnot:page rects:((FSRedactAttributes *)attributes).rects];
                            annot = redact;
                        } @catch (NSException *exception) {
                            return ;
                        }
                    }else if (isForm){
                        FSWidgetAttributes *widgetAttributes = (FSWidgetAttributes *)attributes;
                        annot = [widgetAttributes redoWidgetWithPage:page];
                    }
                    else if (!isReply) {
                        annot = [page addAnnot:attributes.type rect:attributes.rect];
                    } else {
                        FSMarkup *parent = [[FSMarkup alloc] initWithAnnot:[Utility getAnnotByNM:((FSNoteAttributes *) attributes).replyTo inPage:page]];
                        annot = [parent addReply];
                    }
                    
                    if (annot && ![annot isEmpty]) {
                        [attributes resetAnnot:annot];
                        [annotArray addObject:annot];
                    }
                }
                
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (annotArray.count > 0) {
                        if ([annotHandler isKindOfClass:[RedactAnnotHandler class]]) {
                            [annotHandler addAnnots:annotArray addUndo:NO];
                        }
                    }
                });
            }];
        } pageIndexs:pageArray attributesArray:attributesArray curDoc:curDoc];
        
    }
    return nil;
}


+ (instancetype)itemForUndoDeleteRdactAnnotsWithAttributes:(NSArray<FSAnnotAttributes*> *)attributesArray pageArray:(NSArray<NSNumber*> *)pageArray annotHandler:(RedactAnnotHandler*)annotHandler curDoc:(FSPDFDoc*)curDoc{
    
    if (attributesArray && attributesArray.count>0 && pageArray && pageArray.count>0 && annotHandler) {
        
        return [FSUndoGroup itemWithUndo:^(UndoItem *undoItem) {
            
            if (![undoItem isKindOfClass:[FSUndoGroup class]]) {
                return;
            }
            FSUndoGroup *item = (FSUndoGroup*)undoItem;
            
            [[annotHandler.pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                NSInteger count = item.attrArray.count;
                NSMutableArray<FSAnnot*> *annotArray = [NSMutableArray array];
                for (int i=0; i<count; i++) {
                    FSPDFPage* page = nil;
                    FSAnnot *annot = nil;
                    FSAnnotAttributes* attributes = [item.attrArray objectAtIndex:i];
                    if (i < item.pageArray.count) {
                        NSNumber * pageIndex = [item.pageArray objectAtIndex:i];
                        page = [curDoc getPage:[pageIndex intValue]];
                    }
                    
                    BOOL isReply = (attributes.type == FSAnnotNote && ((FSNoteAttributes *) attributes).replyTo.length > 0);
                    BOOL isRedact = (attributes.type == FSAnnotRedact);
                    BOOL isForm = (attributes.type == FSAnnotWidget);
                    if (isRedact) {
                        @try {
                            FSRedaction *redaction = [[FSRedaction alloc] initWithDocument:page.getDocument];
                            FSRedact *redact = [redaction markRedactAnnot:page rects:((FSRedactAttributes *)attributes).rects];
                            annot = redact;
                        } @catch (NSException *exception) {
                            return ;
                        }
                    }else if (isForm){
                        FSWidgetAttributes *widgetAttributes = (FSWidgetAttributes *)attributes;
                        annot = [widgetAttributes redoWidgetWithPage:page];
                    }
                    else if (!isReply) {
                        annot = [page addAnnot:attributes.type rect:attributes.rect];
                    } else {
                        FSMarkup *parent = [[FSMarkup alloc] initWithAnnot:[Utility getAnnotByNM:((FSNoteAttributes *) attributes).replyTo inPage:page]];
                        annot = [parent addReply];
                    }
                    
                    if (annot && ![annot isEmpty]) {
                        [attributes resetAnnot:annot];
                        [annotArray addObject:annot];
                    }
                }
                
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (annotArray.count > 0) {
                        if ([annotHandler isKindOfClass:[RedactAnnotHandler class]]) {
                            [annotHandler addAnnots:annotArray addUndo:NO];
                        }
                    }
                });
            }];
            
        } redo:^(UndoItem *undoItem) {
            
            if (![undoItem isKindOfClass:[FSUndoGroup class]]) {
                return;
            }
            FSUndoGroup *item = (FSUndoGroup*)undoItem;
            
            [[annotHandler.pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{
                NSInteger count = item.attrArray.count;
                NSMutableArray<FSAnnot*> *annotArray = [NSMutableArray array];
                for (int i=0; i<count; i++) {
                    FSPDFPage* page = nil;
                    FSAnnot *annot = nil;
                    FSAnnotAttributes* attributes = [item.attrArray objectAtIndex:i];
                    if (i < item.pageArray.count) {
                        NSNumber* pageIndex = [item.pageArray objectAtIndex:i];
                        page = [curDoc getPage:[pageIndex intValue]];
                    }
                    
                    if (attributes && page && ![page isEmpty]) {
                        annot = [Utility getAnnotByNM:attributes.NM inPage:page];
                        
                        if (annot && ![annot isEmpty]) {
                            [annotArray addObject:annot];
                        }
                    }
                }
                
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (annotArray.count > 0) {
                        if ([annotHandler isKindOfClass:[RedactAnnotHandler class]]) {
                            [annotHandler removeAnnots:annotArray addUndo:NO];
                        }
                    }
                });
            }];
        } pageIndexs:pageArray attributesArray:attributesArray curDoc:curDoc];
        
    }
    return nil;
}

@end

