/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UndoModule.h"
#import <FoxitRDK/FSPDFViewControl.h>
#import "FSToolbarManager.h"
#import "FSMenuController.h"

@interface UndoModule ()<FSMenuControllerDelegate>
{
    NSMutableArray *_undoItems;
    NSMutableArray *_redoItems;
    NSMutableArray <FSMenuItem *>*_menuItem;
}

@end

@implementation UndoModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSUIManagerState oldManagerState;
}

- (id)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        [_extensionsManager registerUndoEventListener:self];
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
    }
    return self;
}

- (NSArray<FSMainToolbarItem *> *)undoItems{
    return _undoItems.copy;
}

- (NSArray<FSMainToolbarItem *> *)redoItems{
    return _redoItems.copy;
}

- (void)loadModule {
    _undoItems = @[].mutableCopy;
    _redoItems = @[].mutableCopy;
    
    @weakify(self)
    FSMenuItem *undoItem = [[FSMenuItem alloc] initWithTitle:@"" image:FSImageNamedAndNormalColor(@"read_undo") action:^(FSMenuItem * _Nonnull _item) {
        @strongify(self)
        if ([self->_extensionsManager canUndo]) {
            [self->_extensionsManager undo];
        }
    }];
    undoItem.accessibilityLabel = FSLocalizedForKey(@"kUndoItemVoiceOver");
    FSMenuItem *redoItem = [[FSMenuItem alloc] initWithTitle:@"" image:FSImageNamedAndNormalColor(@"read_redo") action:^(FSMenuItem * _Nonnull _item) {
        @strongify(self)
        if ([self->_extensionsManager canRedo]) {
            [self->_extensionsManager redo];
        }
    }];
    redoItem.accessibilityLabel = FSLocalizedForKey(@"kRedoItemVoiceOver");
    _menuItem = [NSMutableArray arrayWithObjects:undoItem, redoItem, nil];
    
    if ([[NSFileManager defaultManager] fileExistsAtPath:UNDO_CACHE_PATH]) {
        [[NSFileManager defaultManager] removeItemAtPath:UNDO_CACHE_PATH error:nil];
    }
}

- (void)showdMenuControllerInItem:(FSItem *)item{
    if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        FSMenuController *menuController = [FSMenuController sharedMenuController];
        [menuController setMenuItems:self->_menuItem];
        menuController.delegate = self;
        menuController.arrowDirection = FSMenuControllerArrowDirectionUp;
        [menuController setTargetRect:item.bounds inView:item];
        [menuController setMenuVisible:YES animated:YES];
        
        [_extensionsManager cancelAutoFullScreen];
    }
}

- (BOOL)addUndoItem:(FSMainToolbarItem *)undoItem{
    if (undoItem.itemType == FSMainToolbarItemTypeUndo) {
        @weakify(self)
        undoItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canUndo]) {
                [self->_extensionsManager undo];
            }else{
                [self showdMenuControllerInItem:_item];
            }
        };
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            undoItem.item.enabled = [_extensionsManager canUndo] || [_extensionsManager canRedo];
        }else{
            undoItem.item.enabled = [_extensionsManager canUndo];
        }
        undoItem.item.onLongPress = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if (DEVICE_iPHONE || [self->_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                [self showdMenuControllerInItem:_item];
            }
        };
        [_undoItems addObject:undoItem];
        return YES;
    }
    return NO;
}
- (BOOL)addRedoItem:(FSMainToolbarItem *)redoItem{
    if (redoItem.itemType == FSMainToolbarItemTypeRedo) {
        @weakify(self)
        redoItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if ([self->_extensionsManager canRedo]) {
                [self->_extensionsManager redo];
            }
        };
        redoItem.item.enabled = [_extensionsManager canRedo];
        [_redoItems addObject:redoItem];
        return YES;
    }
    return NO;
}

#pragma mark IFSUndoEventListener

- (void)onUndoChanged {
    for (FSMainToolbarItem *undoItem in _undoItems) {
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            undoItem.item.enabled = [_extensionsManager canUndo] || [_extensionsManager canRedo];
        }else{
            undoItem.item.enabled = [_extensionsManager canUndo];
        }
    }
    
    for (FSMainToolbarItem *redoItem in _redoItems) {
        redoItem.item.enabled = [_extensionsManager canRedo];
    }
}

#pragma mark -FSMenuControllerDelegate
- (BOOL)menuControllerEnableAtIndex:(NSUInteger)index{
    if (index == 0) {
        return [_extensionsManager canUndo];
    }
    return [_extensionsManager canRedo];
}

- (void)menuControllerDidDisappear{
    [_extensionsManager delayAutoFullScreen];
}

@end
