/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <MessageUI/MessageUI.h>

#import <ifaddrs.h>
#import <netinet/in.h>
#import <sys/socket.h>

#import "NSSet+containsAnyObjectInArray.h"
#import "UINavigationItem+IOS7PaddingAdditions.h"
#import "PrivateDefine.h"
#import "UIAlertController+Extensions.h"

typedef void (^CallBackInt)(long property, int value);

@class FSTaskServer;
@class FSAnnotAttributes;
NS_ASSUME_NONNULL_BEGIN


#define kPreventRepeatClickTime(_seconds_) \
static BOOL shouldPrevent; \
if (shouldPrevent) return; \
shouldPrevent = YES; \
dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)((_seconds_) * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{ \
shouldPrevent = NO; \
}); \

#define FOXIT_LOG_ON YES

#define FSLocalizedForKey(key) [Utility localizedString:key]
#define FSLocalizedForKeyWithLanguage(key, lan) [Utility localizedString:key language:lan]
#define FSLocalizedFormatForKey(key, ...) [NSString stringWithFormat:FSLocalizedForKey(key),##__VA_ARGS__]

#define FSAlertViewCtrlShow(tit, msg) \
if ([self isKindOfClass:[UIViewController class]] || [self isKindOfClass:[UIView class]]) {\
    [UIAlertController fs_alert_OK_WithTitle:tit message:msg actionBack:nil onCtrl:[self fs_currentSceneTopMostViewController]];\
}\

#define FSAlertViewCtrlShowOnCtrl(tit, msg, ctrl) \
if ([ctrl isKindOfClass:[UIViewController class]] || [ctrl isKindOfClass:[UIView class]]) {\
    [UIAlertController fs_alert_OK_WithTitle:tit message:msg actionBack:nil onCtrl:[ctrl fs_currentSceneTopMostViewController]];\
}\



#define BookmarkRootFolder @"Foxit Bookmark"

FOUNDATION_EXPORT void FoxitLog(NSString *format, ...);


typedef enum {
    ScreenSizeMode_35 = 0, //3.5 inches
    ScreenSizeMode_40,     //4 inches
    ScreenSizeMode_47,     //4.7 inches
    ScreenSizeMode_55,     //5.5 inches
    ScreenSizeMode_97      //9.7 inches
} ScreenSizeMode;

typedef enum : NSUInteger {
    FileNameErrorCode_Success,
    FileNameErrorCode_NameNULL,
    FileNameErrorCode_NameLong,
    FileNameErrorCode_IllegalChar,
} FileNameErrorCode;

/**
 * @name Macro Definitions for icon type.
 */
/**@{*/
/** @brief note icon type Check. */
#define FPDF_ICONTYPE_NOTE_CHECK 0
/** @brief note icon type Circle. */
#define FPDF_ICONTYPE_NOTE_CIRCLE 1
/** @brief note icon type Comment. */
#define FPDF_ICONTYPE_NOTE_COMMENT 2
/** @brief note icon type Cross. */
#define FPDF_ICONTYPE_NOTE_CROSS 3
/** @brief note icon type Help. */
#define FPDF_ICONTYPE_NOTE_HELP 4
/** @brief note icon type Insert. */
#define FPDF_ICONTYPE_NOTE_INSERT 5
/** @brief note icon type Key. */
#define FPDF_ICONTYPE_NOTE_KEY 6
/** @brief note icon type New Paragraph. */
#define FPDF_ICONTYPE_NOTE_NEWPARAGRAPH 7
/** @brief note icon type Note. */
#define FPDF_ICONTYPE_NOTE_NOTE 8
/** @brief note icon type Paragraph. */
#define FPDF_ICONTYPE_NOTE_PARAGRAPH 9
/** @brief note icon type Right Arrow. */
#define FPDF_ICONTYPE_NOTE_RIGHTARROW 10
/** @brief note icon type Right Pointer. */
#define FPDF_ICONTYPE_NOTE_RIGHTPOINTER 11
/** @brief note icon type Star. */
#define FPDF_ICONTYPE_NOTE_STAR 12
/** @brief note icon type Up Arrow. */
#define FPDF_ICONTYPE_NOTE_UPARROW 13
/** @brief note icon type Upleft Arrow. */
#define FPDF_ICONTYPE_NOTE_UPLEFTARROW 14

/** @brief file attachment icon type Graph. */
#define FPDF_ICONTYPE_FILEATTACH_GRAPH 0
/** @brief file attachment icon type PushPin. */
#define FPDF_ICONTYPE_FILEATTACH_PUSHPIN 1
/** @brief file attachment icon type PaperClip. */
#define FPDF_ICONTYPE_FILEATTACH_PAPERCLIP 2
/** @brief file attachment icon type Tag. */
#define FPDF_ICONTYPE_FILEATTACH_TAG 3

/** @brief Unknown icon type. */
#define FPDF_ICONTYPE_UNKNOWN -1
/**@}*/

#define UX_BG_COLOR_NIGHT_PAGEVIEW 0xFF00001b
#define UX_TEXT_COLOR_NIGHT 0xFF5d5b71

#define NSNotificationNameAnnotLsitUpdate @"NSNotificationNameAnnotLsitUpdate"
#define NSNotificationNameAttachmentLsitUpdate @"NSNotificationNameAttachmentLsitUpdate"

@interface Utility : NSObject {
}
#ifdef _UIEXTENSIONS_BUILD_
//get the xib name according to iPhone or iPad
+ (NSString *)getXibName:(NSString *)baseXibName;
+ (NSString *)displayDateInMDYHMS:(NSDate *)date;
+ (NSString *)displayDateInMDYHM:(NSDate *)date;
+ (NSString *)displayDateInMDY:(NSDate *)date;
/** @brief display date in yyyy-MM-dd HH:mm formate. */
+ (NSString *)displayDateInYMDHM:(NSDate *)date;
/** @brief display date in "yyyy-MM-dd HH:mm" or "yyyyMMddHHmm". */
+ (NSString *)displayDateInYMDHM:(NSDate *)date hasSymbol:(BOOL)hasSymbol;
/** @brief Verify if point is in polygon. */
+ (BOOL)isPointInPolygon:(CGPoint)p polygonPoints:(NSArray *)polygonPoints;
/** @Get Rect by two points. */
+ (CGRect)convertToCGRect:(CGPoint)p1 p2:(CGPoint)p2;
/** @Convert rect to insets. */
+ (UIEdgeInsets)convertCGRect2Insets:(CGRect)rect size:(CGSize)size;
/** @Convert rect with margin. */
+ (CGRect)convertCGRectWithMargin:(CGRect)rect size:(CGSize)size margin:(int)margin;
/** @brief Get Rect by two points. */
+ (FSRectF *)convertToFSRect:(FSPointF *)p1 p2:(FSPointF *)p2;
/** @brief Increases the width and height of the rectangle. */
+ (FSRectF *)inflateFSRect:(FSRectF *)rect width:(float)width height:(float)height;
/** @brief Does it contain provided point ? check if the point is inside this rect. */
+ (BOOL)isPointInFSRect:(FSRectF *)rect point:(FSPointF*)point;
/** @brief Standard Rect. */
+ (CGRect)getStandardRect:(CGRect)rect;
+ (CGRect)getRectFromPoints:(NSArray *)array;
/** @brief Get UUID. */
+ (NSString *)getUUID;
/** @brief Get all the annotations from page. */
+ (NSArray<FSAnnot *> *)getAnnotsInPage:(FSPDFPage *)page predicateBlock:(BOOL (^)(FSAnnot *_Nonnull))predicateBlock;
+ (NSArray<FSAnnot *> *)getAnnotationsOfType:(FSAnnotType)type inPage:(FSPDFPage *)page;
/** @brief Whether annot is a replace text, which consists of a caret and strikeout. */
+ (BOOL)isReplaceText:(FSMarkup *)annot;
+ (FSRectF *)getCaretAnnotRect:(FSMarkup *)markup;
/** @brief Get text selection handler from specified page. */
+ (FSTextPage *)getTextSelect:(FSPDFDoc *)doc pageIndex:(int)index;

/** @brief Get array of text rectangles in specified text index range */
+ (NSArray *)getTextRects:(FSTextPage *)fstextPage start:(int)start count:(int)count;
/** @brief Get text from text rectangle  */
+ (NSString *)getTextFromRect:(FSTextPage *)fstextPage fsrect:(FSRectF *)fsrect;
+ (NSArray *)getTextRects:(FSTextPage *)fstextPage start:(int)start count:(int)count limitRect:(FSRectF *)limitRect;
/** @brief Get word range of specified text index */
+ (NSRange)getWordByTextIndex:(int)index textPage:(FSTextPage *)fstextPage;
/** @brief Verify file type */
+ (BOOL)isGivenPath:(NSString *)path type:(NSString *)type;
+ (BOOL)isGivenExtension:(NSString *)extension type:(NSString *)type;
+ (FSRectF *)getPageRealRect:(FSPDFPage *)page;

#pragma mark -
/** @brief Convert bitmap buffer to UImage. */
+ (UIImage *)dib2img:(void *)pBuf size:(int)size dibWidth:(int)dibWidth dibHeight:(int)dibHeight withAlpha:(BOOL)withAlpha;
+ (UIImage *)dib2imgNoColorChannel:(void *)pBuf size:(int)size dibWidth:(int)dibWidth dibHeight:(int)dibHeight withAlpha:(BOOL)withAlpha;
+ (UIImage *)rgbDib2img:(const void *)pBuf size:(int)size dibWidth:(int)dibWidth dibHeight:(int)dibHeight withAlpha:(BOOL)withAlpha freeWhenDone:(BOOL)b;
/** @brief Compare the two rect. */
+ (BOOL)rectEqualToRect:(FSRectF *)rect rect:(FSRectF *)rect1;
+ (BOOL)quadsEqualToQuads:(FSQuadPoints *)quads1 quads:(FSQuadPoints *)quads2;
+ (BOOL)pointEqualToPoint:(FSPointF *)point1 point:(FSPointF *)point2;
+ (BOOL)inkListEqualToInkList:(FSPath *)inkList1 inkList:(FSPath *)inkList2;

+ (CGRect)FSRectF2CGRect:(FSRectF *)fsrect;
+ (FSRectF *)CGRect2FSRectF:(CGRect)rect;

+ (NSDate *)convertFSDateTime2NSDate:(FSDateTime *)time;
+ (FSDateTime *)convert2FSDateTime:(NSDate *)date;

/** @brief Draw a page tumbnail with specified PDF document and page index. */
+ (UIImage *)drawPageThumbnailWithPDFPath:(NSString *)pdfPath pageIndex:(int)pageIndex pageSize:(CGSize)size rotation:(FSRotation)rotation;
/** @brief Get page size from specified PDF document and page index. */
+ (CGSize)getPDFPageSizeWithIndex:(NSUInteger)index pdfPath:(NSString *)path;
/** @brief Get security type of specified PDF document. */
+ (FSPDFDocEncryptType)getDocumentSecurityType:(NSString *)filePath taskServer:(FSTaskServer *_Nullable)taskServer;

+ (NSString *)convert2SysFontString:(NSString *)str;
+ (NSString *)convert2AppFontString:(NSString *)str;

+ (CGSize)getTestSize:(UIFont *)font;

/** @brief Get mininum horizontal/vertical margin of annotation. Called when add a new annot at the edge of page. */
+ (float)getAnnotMinXMarginInPDF:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex;
+ (float)getAnnotMinYMarginInPDF:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex;

+ (float)convertWidth:(float)width fromPDFToPageView:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex;
+ (float)convertWidth:(float)width fromPageViewToPDF:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex;

/* get annot rect in page view, taken acount of zoom factor */
+ (CGRect)getAnnotRect:(FSAnnot *)annot pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl;
+ (UIImage *)getAnnotImage:(FSAnnot *)annot pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl;
+ (UIImage *)drawPage:(FSPDFPage *)page targetSize:(CGSize)targetSize shouldDrawAnnotation:(BOOL)shouldDrawAnnotation needPause:(BOOL (^__nullable)(void))needPause rotation:(FSRotation)rotation enableForPrint:(BOOL)enableForPrint pdfViewCtrl:(FSPDFViewCtrl * _Nullable)pdfViewCtrl;
+ (UIImage *)drawPage:(FSPDFPage *)page targetSize:(CGSize)targetSize shouldDrawAnnotation:(BOOL)shouldDrawAnnotation needPause:(BOOL (^__nullable)(void))needPause rotation:(FSRotation)rotation enableForPrint:(BOOL)enableForPrint pdfViewCtrl:(FSPDFViewCtrl * _Nullable)pdfViewCtrl transparent:(BOOL) isTransparent;
+ (UIImage *)drawPage:(FSPDFPage *)page targetSize:(CGSize)targetSize shouldDrawAnnotation:(BOOL)shouldDrawAnnotation progressive:(void (^__nullable)(UIImage * image, BOOL))progressive needPause:(BOOL (^ __nullable)(void))needPause rotation:(FSRotation)rotation enableForPrint:(BOOL)enableForPrint pdfViewCtrl:(FSPDFViewCtrl * _Nullable)pdfViewCtrl;
+ (UIImage *)drawPage:(FSPDFPage *)page targetSize:(CGSize)targetSize shouldDrawAnnotation:(BOOL)shouldDrawAnnotation progressive:(void (^__nullable)(UIImage * image, BOOL))progressive needPause:(BOOL (^ __nullable)(void))needPause rotation:(FSRotation)rotation enableForPrint:(BOOL)enableForPrint pdfViewCtrl:(FSPDFViewCtrl * _Nullable)pdfViewCtrl transparent:(BOOL) isTransparent;

+ (void)printPage:(FSPDFPage *)page inContext:(CGContextRef)context inRect:(CGRect)rect shouldDrawAnnotation:(BOOL)shouldDrawAnnotation;

+ (int)toStandardFontID:(NSString *)fontName;

+ (FSRectF *)normalizeFSRect:(FSRectF *)dibRect;
+ (CGRect)normalizeCGRect:(CGRect)rect;
+ (FSRectF *)makeFSRectWithLeft:(float)left top:(float)top right:(float)right bottom:(float)bottom;
+ (FSPointF *)makeFSPointWithX:(float)x y:(float)y;

+ (NSString *)getErrorCodeDescription:(FSErrorCode)error;

+ (FSAnnot *)getAnnotByNM:(NSString *)nm inPage:(FSPDFPage *)page;
+ (FSRectF *)cloneRect:(FSRectF *)rect;
+ (FSPointF *)clonePoint:(FSPointF *)point;
+ (FSPath *)cloneInkList:(FSPath *)inkList;

+ (UIImage *)scaleToSize:(UIImage *)oriImage size:(CGSize)size;
+ (NSString *)formAddControlFieldName:(FSForm *)form forType:(FSFieldType)fieldType isImageField:(BOOL)isImageField;

+ (BOOL)isPDFPath:(NSString *)path;
+ (BOOL)isPDFExtension:(NSString *)extension;
+ (BOOL)isSupportFormat:(NSString *)path;
+ (NSArray *)isSupportImageFormats;
+ (NSArray *)isSupportAudioFormats;
+ (NSArray *)isSupportVideoFormats;
+ (NSString *)getIconName:(NSString *)path;
+ (NSString *)displayFileSize:(unsigned long long)byte;

+ (ScreenSizeMode)getScreenSizeMode;

+ (NSString *)getAttachmentTempFilePath:(FSFileAttachment *)attachment;
+ (NSString *)getDocumentAttachmentTempFilePath:(FSFileSpec *)attachmentFile PDFPath:(NSString *)PDFPath;
+ (NSString *)getDocumentRenditionTempFilePath:(FSFileSpec *)renditionFile PDFPath:(NSString *)PDFPath;
+ (NSString *)getDocumentSoundTempFilePath:(FSSound *)sound PDFPath:(NSString *)PDFPath;
+ (NSString *)getDocumentSoundStreamTempFilePath:(FSPDFStream *)soundStream PDFPath:(NSString *)PDFPath;
+ (BOOL)loadAttachment:(FSFileAttachment *)annot toPath:(NSString *)attachmentPath;
+ (BOOL)loadFileSpec:(FSFileSpec *)fileSpec toPath:(NSString *)path;

+ (NSString *)getStringMD5:(NSString *)string;

+ (FSBitmap *)imgDataToBitmap:(NSData *)imgData;

//+ (NSDictionary<NSString *, FSPDFObject *> *)getNSDictionaryFromPDFDictionary:(FSPDFDictionary *)pdfDict;

+ (int)getIconTypeWithIconName:(NSString *)iconName annotType:(FSAnnotType)annotType;
+ (NSString *)getIconNameWithIconType:(int)iconType annotType:(FSAnnotType)annotType;
+ (NSString *)getAnnotImageNameForAnnot:(FSAnnot *)annot;
+ (BOOL)isValidIconName:(NSString *)iconName annotType:(FSAnnotType)annotType;
+ (NSArray<NSString *> *)getAllIconLowercaseNames;
+ (FileNameErrorCode)validateFileName:(NSString *)fileName;

+ (BOOL)isSignatureWithAnnot:(FSAnnot *)annot;
+ (BOOL)simpleCheckPDFA:(FSPDFDoc *)document;
+ (BOOL)hasSignatureInDocument:(FSPDFDoc *)document;
+ (BOOL)isOwnerOfDoucment:(FSPDFDoc *)document;
+ (BOOL)isDocumentSigned:(FSPDFDoc *)document;
+ (BOOL)canNotEnterForm:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canAddAnnot:(FSPDFViewCtrl *)viewctrl;
//edit annots: FSAnnotAudio, FSAnnotVideo, FSAnnotLink
+ (BOOL)canAddEditAnnot:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canCopyText:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canFillForm:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canAddSign:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canAssemble:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canCopyForAssess:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canModifyContents:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canExtractContents:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canPrint:(FSPDFViewCtrl *)viewctrl;
+ (BOOL)canAddAnnotToDocument:(FSPDFDoc *)document;
//edit annots: FSAnnotAudio, FSAnnotVideo, FSAnnotLink
+ (BOOL)canAddEditAnnotToDocument:(FSPDFDoc *)document;
+ (BOOL)canCopyTextInDocument:(FSPDFDoc *)document;
+ (BOOL)canFillFormInDocument:(FSPDFDoc *)document;
+ (BOOL)canAddSignToDocument:(FSPDFDoc *)document;
+ (BOOL)canAssembleDocument:(FSPDFDoc *)document;
+ (BOOL)canCopyForAssessInDocument:(FSPDFDoc *)document;
+ (BOOL)canModifyContentsInDocument:(FSPDFDoc *)document;
+ (BOOL)canExtractContentsInDocument:(FSPDFDoc *)document;
+ (BOOL)canPrintDocument:(FSPDFDoc *)document;
+ (BOOL)canApplyRedaction:(FSPDFDoc *)document;

+ (int)getMDPDigitalSignPermissionInDocument:(FSPDFDoc *)document;

+ (void)assignImage:(UIImageView *)imageView rawFrame:(CGRect)frame image:(UIImage *)image;

+ (NSString *)temporaryPath;

+ (FSImage *)uiimageToFSImage:(UIImage *)image;

+ (FSBitmap *)uiimageToFSBitmap:(UIImage *)image;

+ (UIImage *)bitmapToUIImage:(FSBitmap *)bitmap;

+ (UIImage *)bitmapToUIImageNoColorChannel:(FSBitmap *)bitmap;

+ (NSArray *)searchFilesWithFolder:(NSString *)folder recursive:(BOOL)recursive;

+ (void)tryLoadDocument:(FSPDFDoc *)document withPassword:(NSString *)password success:(void (^)(NSString *password))success error:(void (^_Nullable)(NSString *description))error abort:(void (^_Nullable)(void))abort sceneProduct:(id)sceneProduct;

+ (void)tryLoadDocument:(FSPDFDoc *)document fileName:(NSString* _Nullable)fileName withPassword:(NSString *)password success:(void (^)(NSString *password))success error:(void (^_Nullable)(NSString *description))error abort:(void (^_Nullable)(void))abort sceneProduct:(id)sceneProduct;
    
//File/Folder existance
+ (BOOL)isFileOrFolderExistAtPath:(NSString *)path fileOrFolderName:(NSString *)fileOrFolderName;

//alert user whether tool will keep selected
+ (BOOL)showAnnotationContinue:(BOOL)isContinue pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl siblingSubview:(UIView *)siblingSubview;
+ (void)dismissAnnotationContinue:(UIView *)superView;

//show user name and icon of current tool
+ (BOOL)showAnnotationType:(NSString *)annotType type:(FSAnnotType)type pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl belowSubview:(UIView *)siblingSubview;
+ (void)showTips:(NSString *)tipsContent pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl belowSubview:(UIView *)siblingSubview ;
    
//Add animation
+ (void)addAnimation:(CALayer *)layer type:(NSString *)type subType:(NSString *)subType timeFunction:(NSString *)timeFunction duration:(float)duration;

+ (FSReadingBookmark *)getReadingBookMarkAtPage:(FSPDFDoc *)doc page:(int)page;

+ (BOOL)parsePage:(FSPDFPage *)page;
+ (BOOL)parsePage:(FSPDFPage *)page flag:(unsigned int)flag pause:(id<FSPauseCallback> _Nullable)pause;
+ (FSRectF*)getPageBoundary:(FSPDFPage*)page;

+ (UIButton *)createButtonWithImage:(UIImage *)image;
+ (UIButton *)createButtonWithTitle:(NSString *)title;
+ (UIButton *)createButtonWithImageAndTitle:(nullable NSString *)title
                                imageNormal:(UIImage *)imageNormal
                              imageSelected:(UIImage *)imageSelected
                               imageDisable:(UIImage *)imageDisabled;

+ (CGFloat)getUIToolbarPaddingX;

+ (BOOL)isSinceiOS:(const NSString *)requiredVersion;

+ (void)printDoc:(FSPDFDoc *)doc animated:(BOOL)animated inWindow:(UIWindow *)window jobName:(nullable NSString *)jobName delegate:(nullable id<UIPrintInteractionControllerDelegate>)delegate completionHandler:(nullable UIPrintInteractionCompletionHandler)completion;

+ (void)printDoc:(FSPDFDoc *)doc fromRect:(CGRect)rect inView:(UIView *)view animated:(BOOL)animated inWindow:(UIWindow *)window jobName:(nullable NSString *)jobName delegate:(nullable id<UIPrintInteractionControllerDelegate>)delegate completionHandler:(nullable UIPrintInteractionCompletionHandler)completion;

////Take screen shot
+ (UIImage *)screenShot:(UIView *)view;

//remove all gesture
+ (void)removeAllGestureRecognizer:(UIView *)view;
//Crop image
+ (UIImage*)cropImage:(UIImage*)img rect:(CGRect)rect;
//rotation image
+ (UIImage *)image:(UIImage *)image rotation:(UIImageOrientation)orientation;

// get text rectangles
+ (NSArray *)getTextRects:(FSTextPage *)fstextPage startCharIndex:(int)startCharIndex endCharIndex:(int)endCharIndex;

// get distance unit info
+ (NSArray *)getDistanceUnitInfo:(NSString *)measureRatio ;
    
// get distance between to cgpoint
+ (float)getDistanceFromX:(FSPointF *)start toY:(FSPointF *)end;

// get distance between to cgpoint with unit
+ (float)getDistanceFromX:(FSPointF *)start toY:(FSPointF *)end withUnit:(NSString *)measureRatio;

+ (float)distanceFromPoint:(CGPoint)pt toLineStartPt:(CGPoint)startPt toLineEndPt:(CGPoint)endPt;

+ (BOOL)isPointVerticalIntersect:(CGPoint)pt onLineStartPt:(CGPoint)startPt  onLineEndPt:(CGPoint)endPt;

// move rect inside container rect
+ (CGRect)boundedRectForRect:(CGRect)rect containerRect:(CGRect)containerRect;

+ (FSRotation)rotationForValue:(NSValue *)value;
+ (int)valueForRotation:(FSRotation)rotation;
+ (UIImageOrientation)imageOrientationForRotation:(FSRotation)rotation;

//+ (NSArray<FSPointF *> *)getPolygonVertexes:(FSPolygon *)polygon;
//
//+ (NSArray<FSPointF *> *)getPolyLineVertexes:(FSPolyLine *)polyline;

+ (BOOL)isAnnot:(FSAnnot *)annot1 uniqueToAnnot:(FSAnnot *)annot2;

+ (NSDictionary *)getACLInfo:(NSString *)aclstr ;
//+ (BOOL)CDRMPageAlow2Access:(int)pageIndex filePath:(NSString *)filePath;

+ (UIImage *)drawCDRMNonAccessImage:(int)dibWidth dibHeight:(int)dibHeight;

+ (NSString *)encodeEmojiString:(NSString *)encodeString;

+ (NSString *)decodeEmojiString:(NSString *)decodeString;

+ (NSString *)filterStrEmoji:(NSString *)emojiStr;
+ (NSString *)regularFilterStrEmoji:(NSString *)text;

+ (BOOL)stringContainsEmoji:(NSString *)string;

+ (UIImage *)drawXFAPage:(FSXFAPage *)page dibWidth:(int)dibWidth dibHeight:(int)dibHeight shouldDrawAnnotation:(BOOL)shouldDrawAnnotation  needPause:(BOOL (^__nullable)(void))needPause rotation:(FSRotation)rotation;

+ (void)printXFADoc:(FSXFADoc *)doc animated:(BOOL)animated jobName:(nullable NSString *)jobName delegate:(nullable id<UIPrintInteractionControllerDelegate>)delegate completionHandler:(nullable UIPrintInteractionCompletionHandler)completion;

+ (void)printXFADoc:(FSXFADoc *)doc fromRect:(CGRect)rect inView:(UIView *)view animated:(BOOL)animated jobName:(nullable NSString *)jobName delegate:(nullable id<UIPrintInteractionControllerDelegate>)delegate completionHandler:(nullable UIPrintInteractionCompletionHandler)completion;

+ (void)printXFAPage:(FSXFAPage *)page inContext:(CGContextRef)context inRect:(CGRect)rect shouldDrawAnnotation:(BOOL)shouldDrawAnnotation ;

+ (BOOL)flattenAnnot:(FSAnnot *)annot completed:(void (^)(NSArray<FSAnnotAttributes *> *replyAttributes))completed;

+ (void)applyRedactAnnot:(FSRedact *)annot completed:(void (^)(NSArray<FSAnnotAttributes *> *replyAttributes))completed;

+ (BOOL)isVertexValid:(CGPoint)vertex pageView:(UIView *)pageView;

+ (NSArray<FSAnnot *> *)getAnnotationsForPageAtIndex:(FSPDFPage *)page withUIConfig:(UIExtensionsConfig *)config;
+ (NSArray<FSAnnot *> *)getAnnotationsFromAnnots:(NSArray *)annots withUIConfig:(UIExtensionsConfig *)config;

+(FSRectF *)convertPageViewRectToPdfRect:(FSRectF *)pageRect pageDisplayMatrix:(FSMatrix2D *)pageDisplayMatrix annot:(FSAnnot *)annot oldPdfRect:(FSRectF *)oldPdfRect rotation:(int)rotation;

+ (void)tryLoadDoc:(NSString *)filePath withPassword:(NSString *)password success:(void (^)(NSString *password))success error:(void (^_Nullable)(NSString *description))error abort:(void (^_Nullable)(void))abort sceneProduct:(id)sceneProduct;

+(BOOL)checkFileisRMS:(NSString *)filePath password:(NSString *)password;

+(CGFloat)GetStatusBarHeight;

+ (FSRectF *)normalizePDFRect:(FSRectF *)rectF ;
+ (UIImage *)getStampImageWithIcon:(int)icon;
+ (void)bookmarkMigration:(FSPDFDoc *)document;
+ (void)cancelRecognizer:(UIGestureRecognizer *)recognizer;

#endif
/** @brief Get the adjusted size according to current scale. */
+ (CGFloat)realPX:(CGFloat)wantPX;
/** @brief Get the texts bounds. */
+ (CGSize)getTextSize:(NSString *)text fontSize:(float)fontSize maxSize:(CGSize)maxSize;
/** @brief Get the texts size from attributed string. */
+ (CGSize)getAttributedTextSize:(NSAttributedString *)attributedString maxSize:(CGSize)maxSize;
// get top viewcontroller for UI viewcontrollers stack
+ (UIImage *)unableImage:(UIImage*)originImage;
+ (UIImage *)unableDynamicImage:(UIImage *)originDynamicImage;
+ (UIImage *)commonDynamicBackgroundImage;
/** @brief Apply alpha with the origin image. */
+ (UIImage *)imageByApplyingAlpha:(UIImage *)image alpha:(CGFloat)alpha;
+ (NSString *)localizedString:(NSString *)translation_key;
+ (NSString *)localizedString:(NSString *)translation_key language:(NSString *)language;

+ (BOOL)isIPadHorizontalCompactWithView:(UIView*)view;

+ (BOOL)isPathOfWindows:(NSString*)pathStr;

+ (NSString *)getDisplayPageLabel:(int)pageIndex needTotal:(BOOL)needTotal useLogicalPageNumbers:(BOOL)useLogicalPageNumbers pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl;
+ (NSString *)getPageLabelTitle:(int)index doc:(FSPDFDoc *)doc;

+ (NSString*)convertToJSONData:(id)infoDict;
+ (NSMutableDictionary *)dictionaryWithJsonString:(NSString *)jsonString;
@end

NS_ASSUME_NONNULL_END

