/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PlatformAPIDemo.h"
#import <CoreGraphics/CoreGraphics.h>
#import <UIKit/UIKit.h>
#import "PlatformAPIDemoCallBack.h"
#import "ExportAnnotData.h"

@interface PlatformAPIDemo (Util)

- (BOOL)setPropertiesOfBookmarks:(FSBookmark *)bookmark;
- (BOOL)save2PNGWithBitmap:(FSBitmap *)fsBitmap CGAffineTransform:(CGAffineTransform *)transform Path:(NSString *)path;

@end

@interface PlatformAPIDemo ()
@property (nonatomic, copy) NSString *docPath;
@end
@implementation PlatformAPIDemo

+ (instancetype)loadDocFromPath:(NSString *)path withPasswd:(NSString *)passwd {
    PlatformAPIDemo *demo = [[PlatformAPIDemo alloc] init];
    if (demo) {
        demo.doc = [[FSPDFDoc alloc] initWithPath:path];
        if (FSErrSuccess != [demo.doc load:passwd]) {
            return nil;
        }
        demo.docPath = path;
    }
    return demo;
}

- (BOOL)extractTextOfDoc:(FSPDFDoc *)doc toPath:(NSString *)path {
    if (!doc || !path) {
        return NO;
    }

    int nPage = [doc getPageCount];
    if (nPage < 1) {
        return NO;
    }

    NSFileManager *fileMgr = [[NSFileManager alloc] init];
    [fileMgr createFileAtPath:path contents:nil attributes:nil];
    NSFileHandle *outFile = [NSFileHandle fileHandleForWritingAtPath:path];
    if (!outFile) {
        return NO;
    }
    for (int i = 0; i < nPage; i++) {
        FSPDFPage *page = [doc getPage:i];
        [page startParse:FSPDFPageParsePageNormal pause:nil is_reparse:YES];
        FSTextPage *textSelect = [[FSTextPage alloc] initWithPage:page flags:FSTextPageParseTextNormal];
        if (!textSelect) {
            continue;
        }

        NSString *fullText = [textSelect getChars:0 count:-1];
        if (!fullText) {
            continue;
        }

        [outFile writeData:[fullText dataUsingEncoding:NSUTF8StringEncoding]];
    }

    [outFile closeFile];

    return YES;
}

- (BOOL)modifyBookmarkOfDoc:(FSPDFDoc *)doc saveAsNewDocAtPath:(NSString *)path {
    if (!doc || !path) {
        return NO;
    }

    FSBookmark *rootBMark = [doc getRootBookmark];
    FSBookmark *child = [rootBMark getFirstChild];

    if (![self setPropertiesOfBookmarks:child]) {
        return NO;
    }

    [doc saveAs:path save_flags:FSPDFDocSaveFlagNormal];

    return YES;
}

- (BOOL)addAnnotToPage:(FSPDFPage *)page saveAsNewDocAtPath:(NSString *)path {
    if (!page || !path) {
        return NO;
    }

    if (![page isParsed]) {
        [page startParse:FSPDFPageParsePageNormal pause:nil is_reparse:YES];
    }

    FSDateTime *(^getLocalDateTime)(void) = ^(){
        NSCalendar *gregorian = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
        unsigned unitFlags = NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay | NSCalendarUnitHour | NSCalendarUnitMinute | NSCalendarUnitSecond;
        NSDateComponents *comps = [gregorian components:unitFlags fromDate:[NSDate date]];
        FSDateTime *time = [[FSDateTime alloc] init];
        time.year = [comps year];
        time.month = [comps month];
        time.day = [comps day];
        time.hour = [comps hour];
        time.minute = [comps minute];
        time.second = [comps second];
        
        NSTimeZone* localTimeZone = [NSTimeZone localTimeZone];
        NSInteger gmtOffset = [localTimeZone secondsFromGMTForDate:[NSDate date]];
        [time setUtc_hour_offset:gmtOffset/3600];
        [time setUtc_minute_offset:(gmtOffset % 3600) / 60];
        return time;
    };
    
    NSString *(^getUUID)(void) = ^(){
        CFUUIDRef uuid = CFUUIDCreate(NULL);
        CFStringRef strUUID = CFUUIDCreateString(NULL, uuid);
        NSString *ret = [(__bridge NSString *) strUUID lowercaseString];
        CFRelease(strUUID);
        CFRelease(uuid);
        return ret;
    };
    
    FSRectF *rectF = [[FSRectF alloc] initWithLeft1:0 bottom1:650 right1:100 top1:750];
    {
        // Add line annotation
        // No special intent, as a common line.
        
        FSLine *line = [[FSLine alloc] initWithAnnot:[page addAnnot:FSAnnotLine rect:rectF]];
        [line setStartPoint:[[FSPointF alloc] initWithX:20 Y:650]];
        [line setEndPoint:[[FSPointF alloc] initWithX:100 Y:740]];
        // Intent, as line arrow.
        line.content = @"A line arrow annotation";
        line.intent = @"LineArrow";
        line.subject = @"Arrow";
        line.title = @"Foxit SDK";
        line.creationDateTime = getLocalDateTime();
        line.modifiedDateTime = getLocalDateTime();
        [line setUniqueID:getUUID()];
        // Appearance should be reset.
        [line resetAppearanceStream];
        
        [rectF setTop:760];
        line = [[FSLine alloc] initWithAnnot:[page addAnnot:FSAnnotLine rect:rectF]];
        // Set foxit RGB color
        [line setBorderColor:0x00FF00];
        [line setStartPoint:[[FSPointF alloc] initWithX:10 Y:650]];
        [line setEndPoint:[[FSPointF alloc] initWithX:100 Y:750]];
        line.content = @"A common line.";
        line.lineStartStyle = FSMarkupEndingStyleSquare;
        line.lineEndStyle = FSMarkupEndingStyleOpenArrow;
        // Show text in line
        [line enableCaption:YES];
        [line setCaptionOffset:[[FSPointF alloc] initWithX:0 Y:5]];

        line.subject = @"Line";
        line.title = @"Foxit SDK";
        line.creationDateTime = getLocalDateTime();
        line.modifiedDateTime = getLocalDateTime();
        [line setUniqueID:getUUID()];
        // Appearance should be reset.
        [line resetAppearanceStream];
        NSLog(@"Add a line annotations.");
    }

    {
        // Add circle annotation
        rectF = [[FSRectF alloc] initWithLeft1:100 bottom1:650 right1:200 top1:750];
        FSCircle *circle = [[FSCircle alloc] initWithAnnot:[page addAnnot:FSAnnotCircle rect:rectF]];
        [circle setInnerRect:[[FSRectF alloc] initWithLeft1:120 bottom1:660 right1:160 top1:740]];
        circle.subject = @"Circle";
        circle.title = @"Foxit SDK";
        circle.creationDateTime = getLocalDateTime();
        circle.modifiedDateTime = getLocalDateTime();
        [circle setUniqueID:getUUID()];
        // Appearance should be reset.
        [circle resetAppearanceStream];
        NSLog(@"Add a circle annotation.");
    }

    {
        // Add square annotation
        rectF = [[FSRectF alloc] initWithLeft1:200 bottom1:650 right1:300 top1:750];
        FSSquare *square = [[FSSquare alloc] initWithAnnot:[page addAnnot:FSAnnotSquare rect:rectF]];
        [square setFillColor:0x00FF00];
        [square setInnerRect:[[FSRectF alloc] initWithLeft1:220 bottom1:660 right1:260 top1:740]];
        square.subject = @"Square";
        square.title = @"Foxit SDK";
        square.creationDateTime = getLocalDateTime();
        square.modifiedDateTime = getLocalDateTime();
        [square setUniqueID:getUUID()];
        // Appearance should be reset.
        [square resetAppearanceStream];
        NSLog(@"Add a square annotation.");
    }

    {
        // Add polygon annotation, as cloud.
        rectF = [[FSRectF alloc] initWithLeft1:300 bottom1:650 right1:500 top1:750];
        FSPolygon *polygon = [[FSPolygon alloc] initWithAnnot:[page addAnnot:FSAnnotPolygon rect:rectF]];
        polygon.intent = @"PolygonCloud";
        [polygon setFillColor:0x0000FF];
        FSPointFArray *vertexe_array = [FSPointFArray new];
        [vertexe_array add:[[FSPointF alloc] initWithX:335 Y:665]];
        [vertexe_array add:[[FSPointF alloc] initWithX:365 Y:665]];
        [vertexe_array add:[[FSPointF alloc] initWithX:385 Y:705]];
        [vertexe_array add:[[FSPointF alloc] initWithX:365 Y:740]];
        [vertexe_array add:[[FSPointF alloc] initWithX:335 Y:740]];
        [vertexe_array add:[[FSPointF alloc] initWithX:315 Y:705]];
        polygon.vertexes = vertexe_array;
        polygon.subject = @"Cloud";
        polygon.title = @"Foxit SDK";
        polygon.creationDateTime = getLocalDateTime();
        polygon.modifiedDateTime = getLocalDateTime();
        [polygon setUniqueID:getUUID()];
        // Appearance should be reset.
        [polygon resetAppearanceStream];
        NSLog(@"Add a polygon annotation.");
    }

    {
        // Add polygon annotation, with dashed border.
        FSBorderInfo *borderinfo = [FSBorderInfo new];
        borderinfo.cloud_intensity = 2.0f;
        borderinfo.width = 2.0f;
        borderinfo.style = FSBorderInfoDashed;
        borderinfo.dash_phase = 3.f;
        FSFloatArray *dashes = [FSFloatArray new];
        [dashes setAt:0 newElement:2];
        [dashes setAt:1 newElement:2];
        borderinfo.dashes = dashes;
        
        rectF = [[FSRectF alloc] initWithLeft1:400 bottom1:650 right1:500 top1:750];
        FSPolygon *polygon = [[FSPolygon alloc] initWithAnnot:[page addAnnot:FSAnnotPolygon rect:rectF]];
        [polygon setFillColor:0x0000FF];
        [polygon setBorderInfo:borderinfo];
        
        FSPointFArray *vertexe_array = [FSPointFArray new];
        [vertexe_array add:[[FSPointF alloc] initWithX:435 Y:665]];
        [vertexe_array add:[[FSPointF alloc] initWithX:465 Y:665]];
        [vertexe_array add:[[FSPointF alloc] initWithX:485 Y:705]];
        [vertexe_array add:[[FSPointF alloc] initWithX:465 Y:740]];
        [vertexe_array add:[[FSPointF alloc] initWithX:435 Y:740]];
        [vertexe_array add:[[FSPointF alloc] initWithX:415 Y:705]];
        polygon.vertexes = vertexe_array;
        
        polygon.subject = @"Polygon";
        polygon.title = @"Foxit SDK";
        polygon.creationDateTime = getLocalDateTime();
        polygon.modifiedDateTime = getLocalDateTime();
        [polygon setUniqueID:getUUID()];
        // Appearance should be reset.
        [polygon resetAppearanceStream];
        
        NSLog(@"Add a polygon cloud annotation.");
    }

    {
      // Add polyline annotation
        rectF = [[FSRectF alloc] initWithLeft1:500 bottom1:650 right1:600 top1:700];
        FSPolyLine *polyline = [[FSPolyLine alloc] initWithAnnot:[page addAnnot:FSAnnotPolyLine rect:rectF]];
        
        FSPointFArray *vertexe_array = [FSPointFArray new];
        [vertexe_array add:[[FSPointF alloc] initWithX:515 Y:705]];
        [vertexe_array add:[[FSPointF alloc] initWithX:535 Y:740]];
        [vertexe_array add:[[FSPointF alloc] initWithX:565 Y:740]];
        [vertexe_array add:[[FSPointF alloc] initWithX:585 Y:705]];
        [vertexe_array add:[[FSPointF alloc] initWithX:565 Y:665]];
        [vertexe_array add:[[FSPointF alloc] initWithX:535 Y:665]];
        polyline.vertexes = vertexe_array;
        
        polyline.subject = @"PolyLine";
        polyline.title = @"Foxit SDK";
        polyline.creationDateTime = getLocalDateTime();
        polyline.modifiedDateTime = getLocalDateTime();
        [polyline setUniqueID:getUUID()];
        // Appearance should be reset.
        [polyline resetAppearanceStream];
        
        NSLog(@"Add a polyline annotation.");
    }

    {
        // Add freetext annotation, as type writer.
        rectF = [[FSRectF alloc] initWithLeft1:10 bottom1:550 right1:220 top1:600];
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:[page addAnnot:FSAnnotFreeText rect:rectF]];
        
        // Set default appearance
        FSDefaultAppearance *default_ap = [FSDefaultAppearance new];
        default_ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagFontSize | FSDefaultAppearanceFlagTextColor;
        [default_ap setFont:[[FSFont alloc] initWithFont_id:FSFontStdIDHelvetica]];
        default_ap.text_size = 12.0f;
        default_ap.text_color = 0x000000;
        // Set default appearance for form.
        [freetext setDefaultAppearance:default_ap];
        [freetext setAlignment:FSAlignmentLeft];
        freetext.intent = @"FreeTextTypewriter";
        freetext.content = @"A typewriter annotation";
        freetext.subject = @"FreeTextTypewriter";
        freetext.title = @"Foxit SDK";
        freetext.creationDateTime = getLocalDateTime();
        freetext.modifiedDateTime = getLocalDateTime();
        [freetext setUniqueID:getUUID()];
        // Appearance should be reset.
        [freetext resetAppearanceStream];
        
        NSLog(@"Add a typewriter freetext annotation.");
    }
    {
        // Add freetext annotation, as call-out.
        rectF = [[FSRectF alloc] initWithLeft1:300 bottom1:550 right1:400 top1:600];
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:[page addAnnot:FSAnnotFreeText rect:rectF]];
        
        // Set default appearance
        FSDefaultAppearance *default_ap = [FSDefaultAppearance new];
        default_ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagFontSize | FSDefaultAppearanceFlagTextColor;
        [default_ap setFont:[[FSFont alloc] initWithFont_id:FSFontStdIDHelveticaB]];
        default_ap.text_size = 12.0f;
        default_ap.text_color = 0x000000;
        // Set default appearance for form.
        [freetext setDefaultAppearance:default_ap];
        [freetext setAlignment:FSAlignmentCenter];
        //
        freetext.intent = @"FreeTextCallout";
        FSPointFArray *callout_points = [FSPointFArray new];
        [callout_points add:[[FSPointF alloc] initWithX:250 Y:540]];
        [callout_points add:[[FSPointF alloc] initWithX:280 Y:570]];
        [callout_points add:[[FSPointF alloc] initWithX:300 Y:570]];
        freetext.calloutLinePoints = callout_points;
        
        freetext.calloutLineEndingStyle = FSMarkupEndingStyleOpenArrow;
        freetext.content = @"A callout annotation.";
        freetext.subject = @"FreeTextCallout";
        freetext.title = @"Foxit SDK";
        freetext.creationDateTime = getLocalDateTime();
        freetext.modifiedDateTime = getLocalDateTime();
        [freetext setUniqueID:getUUID()];
        // Appearance should be reset.
        [freetext resetAppearanceStream];
        
        NSLog(@"Add a callout freetext annotation.");
    }
    {
        // Add freetext annotation, as text box.
        rectF = [[FSRectF alloc] initWithLeft1:450 bottom1:550 right1:550 top1:600];
        FSFreeText *freetext = [[FSFreeText alloc] initWithAnnot:[page addAnnot:FSAnnotFreeText rect:rectF]];
        
        // Set default appearance
        FSDefaultAppearance *default_ap = [FSDefaultAppearance new];
        default_ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagFontSize | FSDefaultAppearanceFlagTextColor;
        [default_ap setFont:[[FSFont alloc] initWithFont_id:FSFontStdIDHelveticaI]];
        default_ap.text_size = 12.0f;
        default_ap.text_color = 0x000000;
        // Set default appearance for form.
        [freetext setDefaultAppearance:default_ap];
        
        [freetext setAlignment:FSAlignmentCenter];
        freetext.content = @"A text box annotation.";
        freetext.subject = @"Textbox";
        freetext.title = @"Foxit SDK";
        freetext.creationDateTime = getLocalDateTime();
        freetext.modifiedDateTime = getLocalDateTime();
        [freetext setUniqueID:getUUID()];
        // Appearance should be reset.
        [freetext resetAppearanceStream];
        NSLog(@"Add a text box freetext annotation.");
    }
    {
        // Add highlight annotation.
        rectF = [[FSRectF alloc] initWithLeft1:10 bottom1:450 right1:100 top1:550];
        FSHighlight *highlight = [[FSHighlight alloc] initWithAnnot:[page addAnnot:FSAnnotHighlight rect:rectF]];
        highlight.content = @"Highlight";
        
        FSQuadPoints *quad_points = [FSQuadPoints new];
        [quad_points setFirst:[[FSPointF alloc] initWithX:10 Y:500]];
        [quad_points setSecond:[[FSPointF alloc] initWithX:90 Y:500]];
        [quad_points setThird:[[FSPointF alloc] initWithX:10 Y:480]];
        [quad_points setFourth:[[FSPointF alloc] initWithX:90 Y:480]];
        
        FSQuadPointsArray *quad_points_array = [FSQuadPointsArray new];
        [quad_points_array add:quad_points];
        
        [highlight setQuadPoints:quad_points_array];
        highlight.subject = @"Highlight";
        highlight.title = @"Foxit SDK";
        highlight.creationDateTime = getLocalDateTime();
        highlight.modifiedDateTime = getLocalDateTime();
        [highlight setUniqueID:getUUID()];
        // Appearance should be reset.
        [highlight resetAppearanceStream];
        NSLog(@"Add a highlight annotation.");
    }
    {
        // Add underline annotation.
        rectF = [[FSRectF alloc] initWithLeft1:100 bottom1:450 right1:200 top1:550];
        FSUnderline *underline = [[FSUnderline alloc] initWithAnnot:[page addAnnot:FSAnnotUnderline rect:rectF]];
        
        FSQuadPoints *quad_points = [FSQuadPoints new];
        [quad_points setFirst:[[FSPointF alloc] initWithX:110 Y:500]];
        [quad_points setSecond:[[FSPointF alloc] initWithX:190 Y:500]];
        [quad_points setThird:[[FSPointF alloc] initWithX:110 Y:480]];
        [quad_points setFourth:[[FSPointF alloc] initWithX:190 Y:480]];
        
        FSQuadPointsArray *quad_points_array = [FSQuadPointsArray new];
        [quad_points_array add:quad_points];
        
        [underline setQuadPoints:quad_points_array];
        underline.subject = @"Underline";
        underline.title = @"Foxit SDK";
        underline.creationDateTime = getLocalDateTime();
        underline.modifiedDateTime = getLocalDateTime();
        [underline setUniqueID:getUUID()];
        // Appearance should be reset.
        [underline resetAppearanceStream];

        NSLog(@"Add a underline annotation.");
    }
    {
        // Add squiggly annotation.
        rectF = [[FSRectF alloc] initWithLeft1:200 bottom1:450 right1:300 top1:550];
        FSSquiggly *squiggly = [[FSSquiggly alloc] initWithAnnot:[page addAnnot:FSAnnotSquiggly rect:rectF]];
        squiggly.intent = @"Squiggly";

        FSQuadPoints *quad_points = [FSQuadPoints new];
        [quad_points setFirst:[[FSPointF alloc] initWithX:210 Y:500]];
        [quad_points setSecond:[[FSPointF alloc] initWithX:290 Y:500]];
        [quad_points setThird:[[FSPointF alloc] initWithX:210 Y:480]];
        [quad_points setFourth:[[FSPointF alloc] initWithX:290 Y:480]];

        FSQuadPointsArray *quad_points_array = [FSQuadPointsArray new];
        [quad_points_array add:quad_points];
        
        [squiggly setQuadPoints:quad_points_array];
        squiggly.subject = @"Squiggly";
        squiggly.title = @"Foxit SDK";
        squiggly.creationDateTime = getLocalDateTime();
        squiggly.modifiedDateTime = getLocalDateTime();
        [squiggly setUniqueID:getUUID()];
        // Appearance should be reset.
        [squiggly resetAppearanceStream];

        NSLog(@"Add a squiggly annotation.");
    }
    {
      // Add strikeout annotation.
        
        rectF = [[FSRectF alloc] initWithLeft1:300 bottom1:450 right1:400 top1:550];
        FSStrikeOut *strikeout = [[FSStrikeOut alloc] initWithAnnot:[page addAnnot:FSAnnotStrikeOut rect:rectF]];

        FSQuadPoints *quad_points = [FSQuadPoints new];
        [quad_points setFirst:[[FSPointF alloc] initWithX:310 Y:500]];
        [quad_points setSecond:[[FSPointF alloc] initWithX:390 Y:500]];
        [quad_points setThird:[[FSPointF alloc] initWithX:310 Y:480]];
        [quad_points setFourth:[[FSPointF alloc] initWithX:390 Y:480]];

        FSQuadPointsArray *quad_points_array = [FSQuadPointsArray new];
        [quad_points_array add:quad_points];
        
        [strikeout setQuadPoints:quad_points_array];
        strikeout.subject = @"StrikeOut";
        strikeout.title = @"Foxit SDK";
        strikeout.creationDateTime = getLocalDateTime();
        strikeout.modifiedDateTime = getLocalDateTime();
        [strikeout setUniqueID:getUUID()];
        // Appearance should be reset.
        [strikeout resetAppearanceStream];
        
        NSLog(@"Add a strikeout annotation.");
    }
    {
        // Add caret annotation.
        rectF = [[FSRectF alloc] initWithLeft1:400 bottom1:450 right1:420 top1:470];
        FSCaret *caret = [[FSCaret alloc] initWithAnnot:[page addAnnot:FSAnnotCaret rect:rectF]];
        [caret setInnerRect:[[FSRectF alloc] initWithLeft1:410 bottom1:450 right1:430 top1:470]];
        caret.content = @"Caret annotation";
        caret.subject = @"Caret";
        caret.title = @"Foxit SDK";
        caret.creationDateTime = getLocalDateTime();
        caret.modifiedDateTime = getLocalDateTime();
        [caret setUniqueID:getUUID()];
        // Appearance should be reset.
        [caret resetAppearanceStream];

        NSLog(@"Add a caret annotation.");
    }
    {
        // Add note annotation
        rectF = [[FSRectF alloc] initWithLeft1:10 bottom1:350 right1:50 top1:400];
        FSNote *note = [[FSNote alloc] initWithAnnot:[page addAnnot:FSAnnotNote rect:rectF]];
        note.iconName = @"Comment";
        note.content = @"Note annotation.";
        note.subject = @"Note";
        note.title = @"Foxit SDK";
        note.creationDateTime = getLocalDateTime();
        note.modifiedDateTime = getLocalDateTime();
        [note setUniqueID:getUUID()];
        
        // Add popup to note annotation
        rectF = [[FSRectF alloc] initWithLeft1:300 bottom1:450 right1:500 top1:550];
        FSPopup *popup = [[FSPopup alloc] initWithAnnot:[page addAnnot:FSAnnotPopup rect:rectF]];
        popup.borderColor = 0x00FF00;
        popup.openStatus = NO;
        
        popup.modifiedDateTime = getLocalDateTime();
        note.popup = popup;

        // Add reply annotation to note annotation
        FSNote *reply = [note addReply];
        reply.content = @"reply";
        reply.modifiedDateTime = getLocalDateTime();
        reply.title = @"Foxit SDK";
        [note setUniqueID:getUUID()];
        [reply resetAppearanceStream];

        // Add state annotation to note annotation
        FSNote *state = [note addStateAnnot:@"Foxit SDK" model:FSMarkupStateModelReview state:FSMarkupStateAccepted];
        state.content = @"Accepted set by Foxit SDK";
        [state setUniqueID:getUUID()];
        // Appearance should be reset.
        [state resetAppearanceStream];
        
        NSLog(@"Add a note annotation.");
    }
    {
        // Add ink annotation
        FSRectF* annot_rect = [[FSRectF alloc] initWithLeft1:100 bottom1:350 right1:200 top1:450];
        FSInk* ink = [[FSInk alloc] initWithAnnot:[page addAnnot:FSAnnotInk rect:annot_rect]];
        
        FSPath* inklist = [FSPath new];
        float width = 100;
        float height = 100;
        float out_width = MIN(width, height) * 2 / 3.f;
        float inner_width = out_width * sin(18.f / 180.f * 3.14f) / sin(36.f / 180.f * 3.14f);
        FSPointF* center = [FSPointF new];
        [center set:150 y:400];
        float x = out_width;
        float y = 0;
        FSPointF* point = [FSPointF new];
        [point set:(center.x + x) y:(center.y + y)];
        [inklist moveTo:point];
        for (int i = 0; i < 5; i++) {
            x = out_width * cos(72.f * i / 180.f * 3.14f);
            y = out_width * sin(72.f * i / 180.f * 3.14f);
            [point set:(center.x + x) y:(center.y + y)];
            [inklist lineTo:point];
            
            x = inner_width * cos((72.f * i + 36) / 180.f * 3.14f);
            y = inner_width * sin((72.f * i + 36) / 180.f * 3.14f);
            [point set:(center.x + x) y:(center.y + y)];
            [inklist lineTo:point];
        }
        [point set:(center.x + out_width) y:(center.y + 0)];
        [inklist lineTo:point];
        [inklist closeFigure];
        [ink setInkList:inklist];
        [ink setSubject:@"Ink"];
        [ink setTitle:@"Foxit SDK"];
        [ink setContent:@"Ink annotation."];
        FSDateTime* local_time = getLocalDateTime();
        [ink setCreationDateTime:local_time];
        [ink setModifiedDateTime:local_time];
        [ink setUniqueID:getUUID()];
        // Appearance should be reset.
        [ink resetAppearanceStream];
        NSLog(@"Add an ink annotation.");
    }
    {
        // Add file attachment annotation
        NSString *pdf_path = [[NSBundle mainBundle] pathForResource:@"test_files/AboutFoxit" ofType:@"pdf"] ;
        FSRectF* annot_rect = [[FSRectF alloc] initWithLeft1:280 bottom1:350 right1:300 top1:380];
        FSFileAttachment* file_attachment = [[FSFileAttachment alloc] initWithAnnot:[page addAnnot:FSAnnotFileAttachment rect:annot_rect]];
        [file_attachment setIconName:@"Graph"];
        
        FSFileSpec* file_spec = [[FSFileSpec alloc] initWithDocument:[page getDocument]];
        [file_spec setFileName:@"attachment.pdf"];
        FSDateTime* local_time = getLocalDateTime();
        [file_spec setCreationDateTime:local_time];
        [file_spec setDescription:@"The original file"];
        [file_spec setModifiedDateTime:local_time];
        [file_spec embed:pdf_path];
        [file_attachment setFileSpec:file_spec];
        [file_attachment setSubject:@"File Attachment"];
        [file_attachment setTitle:@"Foxit SDK"];
        // Appearance should be reset.
        [file_attachment resetAppearanceStream];
        NSLog(@"Add an attachment annotation.");
    }
    {
        // Add link annotation
        FSRectF* annot_rect = [[FSRectF alloc] initWithLeft1:350 bottom1:350 right1:380 top1:400];
        FSLink* link = [[FSLink alloc] initWithAnnot:[page addAnnot:FSAnnotLink rect:annot_rect]];
        [link setHighlightingMode:FSAnnotHighlightingToggle];
        
        // Add action for link annotation
        FSPDFDoc* doc = [page getDocument];
        FSURIAction* action = [[FSURIAction alloc] initWithAction:[FSAction create:doc action_type:FSActionTypeURI]];
        [action setTrackPositionFlag:YES];
        [action setURI:@"www.foxitsoftware.com"];
        [link setAction:action];
        // Appearance should be reset.
        [link resetAppearanceStream];
        NSLog(@"Add a link annotation.");
    }
    {
        // Set icon provider for annotation to Foxit PDF SDK.
        NSString *input_path =[self.docPath stringByDeletingLastPathComponent];
        NSString *icon_folder = [input_path stringByAppendingPathComponent:@"Stamps"];
        MyIconProvider* icon_provider = [[MyIconProvider alloc] initWithPath:icon_folder];
        [icon_provider setUseDynamicStamp:NO];
        [FSLibrary setAnnotIconProviderCallback:icon_provider];
        
        // Add common stamp annotation.
        [FSLibrary setActionCallback:nil];
        
        FSRectF* annot_rect = [[FSRectF alloc] initWithLeft1:110 bottom1:150 right1:200 top1:250];
        FSStamp* static_stamp = [[FSStamp alloc] initWithAnnot:[page addAnnot:FSAnnotStamp rect:annot_rect]];
        [static_stamp setIconName:@"Approved"];
        // Appearance should be reset.
        [static_stamp resetAppearanceStream];
        
        // Add dynamic stamp annotation.
        [FSLibrary setActionCallback:[MyActionEventHandler new]];
        [icon_provider setUseDynamicStamp:YES];
        
        annot_rect = [[FSRectF alloc] initWithLeft1:10 bottom1:150 right1:100 top1:250];
        FSStamp* dynamic_stamp = [[FSStamp alloc] initWithAnnot:[page addAnnot:FSAnnotStamp rect:annot_rect]];
        [dynamic_stamp setIconName:@"Approved"];
        
        // Appearance should be reset.
        [dynamic_stamp resetAppearanceStream];
        NSLog(@"Add stamp annotations.");

    }
     
    FSPDFDoc *doc = [page getDocument];
    [doc saveAs:path save_flags:FSPDFDocSaveFlagNormal];

    [ExportAnnotData exportAnnotDataToJson:doc toPath:[[path stringByDeletingLastPathComponent] stringByAppendingPathComponent:@"annotations.json"]];
    return YES;
}

- (BOOL)exportDocInfoOfDoc:(FSPDFDoc *)doc toPath:(NSString *)path {
    if (!doc || !path) {
        return NO;
    }

    int nPage = [doc getPageCount];
    if (nPage < 1) {
        return NO;
    }
    NSMutableString *string = [NSMutableString stringWithFormat:@"PageCount: %d pages\n", nPage];

    //get from metadata
    FSMetadata *metadata = [[FSMetadata alloc] initWithDocument:doc];
    NSArray *arrayOfItem = [NSArray arrayWithObjects:@"Title", @"Author", @"Subject", @"Keywords", nil];
    for (NSString *item in arrayOfItem) {
        [string appendFormat:@"%@: %@\n", item, [metadata getValues:item][0]];
    }

    NSFileManager *fileMgr = [[NSFileManager alloc] init];
    [fileMgr createFileAtPath:path contents:nil attributes:nil];
    NSFileHandle *outFile = [NSFileHandle fileHandleForWritingAtPath:path];
    if (!outFile) {
        return NO;
    }

    [outFile writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];

    [outFile closeFile];

    return YES;
}

- (BOOL)renderPage:(FSPDFPage *)page atPath:(NSString *)path {
    if (!page || !path) {
        return NO;
    }

    if (![page isParsed]) {
        FSProgressive* progress = [page startParse:FSPDFPageParsePageNormal pause:nil is_reparse:YES];
        [progress resume];
    }

    //create bitmap
    float pageWidth = [page getWidth];
    float pageHeight = [page getHeight];
    int size = pageWidth * pageHeight * 4;
    void *pBuf = malloc(size);
    memset(pBuf, 0x00, size);
    FSBitmap *bmp = [[FSBitmap alloc] initWithWidth:pageWidth height:pageHeight format:FSBitmapDIBArgb buffer:[NSData dataWithBytesNoCopy:pBuf length:size freeWhenDone:NO] pitch:pageWidth * 4];
    
//    FSBitmap *bmp = [[FSBitmap alloc] initWithWidth:pageWidth height:pageHeight format:FSBitmapDIBArgb buffer:nil pitch:pageWidth * 4];
    if (!bmp) {
        return NO;
    }
    [bmp fillRect:0xffffffff rect:nil];

    //init renderer
    FSRenderer *render = [[FSRenderer alloc] initWithBitmap:bmp is_rgb_order:YES];
    if (!render) {
        return NO;
    }

    //set different kinds of properties
    [render setRenderContentFlags:FSRendererRenderPage];
    [render setColorMode:FSRendererColorModeNormal];
    [render setMappingModeColors:0xffffffff foreground_color:0x0000ffff];
    [render setForceHalftone:YES];

    FSMatrix2D *mat = [page getDisplayMatrix:0 top:0 width:pageWidth height:pageHeight rotate:FSRotation0];
    FSProgressive *progress = [render startRender:page matrix:mat pause:nil];
    if (progress) {
        FSProgressiveState stat = [progress resume];
        while (stat == FSProgressiveToBeContinued)
            stat = [progress resume];
    }

    BOOL isOK = [self save2PNGWithBitmap:bmp CGAffineTransform:nil Path:path];
    free(pBuf);
    return isOK;
}

- (BOOL)addSignatureToPage:(FSPDFPage *)page saveAsNewDocAtPath:(NSString *)path {
    if (!page || !path) {
        return NO;
    }

    FSRectF *rect = [[FSRectF alloc] init];
    [rect setLeft:100];
    [rect setBottom:0];
    [rect setRight:200];
    [rect setTop:100];
    FSSignature *signature = [page addSignature:rect];

    NSDate *now = [[NSDate alloc] init];
    NSTimeZone *timeZone = [[NSTimeZone alloc] init];
    timeZone = [NSTimeZone systemTimeZone];
    int secondsFromGMTForDate = (int) [timeZone secondsFromGMTForDate:now];

    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSUInteger unitFlags = NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay | NSCalendarUnitHour | NSCalendarUnitMinute | NSCalendarUnitSecond;
    NSDateComponents *dateComponent = [calendar components:unitFlags fromDate:now];

    int year = (int) [dateComponent year];
    int month = (int) [dateComponent month];
    int day = (int) [dateComponent day];
    int hour = (int) [dateComponent hour];
    int minute = (int) [dateComponent minute];
    int second = (int) [dateComponent second];
    int UTHourOffset = secondsFromGMTForDate / 3600;
    int UTMinuteOffset = secondsFromGMTForDate % 3600;

    FSDateTime *dateTime = [[FSDateTime alloc] init];
    [dateTime set:year month:month day:day hour:hour minute:minute second:second milliseconds:0 utc_hour_offset:UTHourOffset utc_minute_offset:UTMinuteOffset];
    FSBitmap *bitmap = [[FSBitmap alloc] initWithWidth:100 height:100 format:FSBitmapDIBRgb32 buffer:nil pitch:0];
    [bitmap fillRect:0xff0000ff rect:nil];
    //"adbe.pkcs7.sha1"
    [signature setFilter:@"Adobe.PPKLite"];
    [signature setSubFilter:@"adbe.pkcs7.detached"];
    [signature setKeyValue:FSSignatureKeyNameDN value:@"dn"];
    [signature setKeyValue:FSSignatureKeyNameLocation value:@"location"];
    [signature setKeyValue:FSSignatureKeyNameReason value:@"reason"];
    [signature setKeyValue:FSSignatureKeyNameContactInfo value:@"contactInfo"];
    [signature setKeyValue:FSSignatureKeyNameSigner value:@"signer"];
    [signature setKeyValue:FSSignatureKeyNameText value:@"text"];
    [signature setSignTime:dateTime];
    [signature setAppearanceFlags:FSSignatureAPFlagFoxitFlag | FSSignatureAPFlagLabel | FSSignatureAPFlagReason | FSSignatureAPFlagSigningTime | FSSignatureAPFlagDN | FSSignatureAPFlagLocation | FSSignatureAPFlagSigner | FSSignatureAPFlagBitmap | FSSignatureAPFlagText];

    //note:if use FSSignatureAPFlagBitmap flag. you need call FSSignature::setBitmap function
    [signature setBitmap:bitmap];
     
    NSString *certPath = [[NSBundle mainBundle] pathForResource:@"test_files/foxit_all" ofType:@"pfx"];
    NSString *password = @"123456";
    // set customized appearance.
    FSProgressive *progress = [signature startSign:certPath cert_password:password digest_algorithm:FSSignatureDigestSHA256 save_path:path client_data:nil pause:nil];

    FSProgressiveState state = FSProgressiveFinished;
    if (progress != nil) {
        state = [progress resume];
        while (state == FSProgressiveToBeContinued)
            state = [progress resume];
    }

    if (FSProgressiveFinished != state) {
        return NO;
    }
    FSPDFDoc *pDoc = [page getDocument];
    pDoc = nil;
    signature = nil;

    // load the signed doc, and verify it.
    pDoc = [[FSPDFDoc alloc] initWithPath:path];
    if ([pDoc load:nil] != FSErrSuccess) {
        return NO;
    }
    signature = [pDoc getSignature:0];

    FSProgressive *verifyProgress = [signature startVerify:0 pause:nil];
    FSProgressiveState verifyProgressState = [verifyProgress resume];

    while (verifyProgressState == FSProgressiveToBeContinued) {
        verifyProgressState = [verifyProgress resume];
    }

    FSSignatureStates stat = [signature getState];
    if (!(FSSignatureStateVerifyValid == (stat & FSSignatureStateVerifyValid) || FSSignatureStateVerifyNoChange == (stat & FSSignatureStateVerifyNoChange)))
        return NO;
    return YES;
}

- (BOOL)addWatermark:(NSString *)input_file saveAsNewDocAtPath:(NSString *)path{
    
    void (^AddTextWatermark)(FSPDFDoc *doc, FSPDFPage *page) = ^(FSPDFDoc *doc, FSPDFPage *page){
        FSWatermarkSettings *settings = [FSWatermarkSettings new];
        settings.flags = FSWatermarkSettingsFlagASPageContents | FSWatermarkSettingsFlagOnTop;
        settings.offset_x = 0;
        settings.offset_y = 0;
        settings.opacity = 90;
        settings.position = FSPosTopRight;
        settings.rotation = -45.f;
        settings.scale_x = 1.f;
        settings.scale_y = 1.f;
        
        FSWatermarkTextProperties *text_properties = [FSWatermarkTextProperties new];
        text_properties.alignment = FSAlignmentCenter;
        text_properties.color = 0xF68C21;
        text_properties.font_style = FSWatermarkTextPropertiesFontStyleNormal;
        text_properties.line_space = 1;
        text_properties.font_size = 12.f;
        FSFont *font = [[FSFont alloc] initWithFont_id:FSFontStdIDTimesB];
        text_properties.font = font;
        
        FSWatermark *watermark = [[FSWatermark alloc] initWithDocument:doc text:@"Foxit PDF SDK\nwww.foxitsoftware.com" properties:text_properties settings:settings];
        [watermark insertToPage:page];
        
    };
    void (^AddBitmapWatermark)(FSPDFDoc *doc, FSPDFPage *page, NSString *bitmap_file) = ^(FSPDFDoc *doc, FSPDFPage *page, NSString *bitmap_file){
        FSWatermarkSettings *settings = [FSWatermarkSettings new];
        settings.flags = FSWatermarkSettingsFlagASPageContents | FSWatermarkSettingsFlagOnTop;
        settings.offset_x = 0;
        settings.offset_y = 0;
        settings.opacity = 60.f;
        settings.position = FSPosCenterLeft;
        settings.rotation = 90.f;
        
        FSImage *image = [[FSImage alloc] initWithPath:bitmap_file];
        FSBitmap *bitmap = [image getFrameBitmap:0];
        settings.scale_x = page.getHeight * 1.0f / bitmap.getWidth;
        settings.scale_y = settings.scale_x;
        
        FSWatermark *watermark = [[FSWatermark alloc] initWithDocument:doc bitmap:bitmap settings:settings];
        [watermark insertToPage:page];
    };
    void (^AddImageWatermark)(FSPDFDoc *doc, FSPDFPage *page, NSString *image_file) = ^(FSPDFDoc *doc, FSPDFPage *page, NSString *image_file){
        FSWatermarkSettings *settings = [FSWatermarkSettings new];
        settings.flags = FSWatermarkSettingsFlagASPageContents | FSWatermarkSettingsFlagOnTop;
        settings.offset_x = 0;
        settings.offset_y = 0;
        settings.opacity = 20.f;
        settings.position = FSPosCenter;
        settings.rotation = 0;

        FSImage *image = [[FSImage alloc] initWithPath:image_file];
        FSBitmap *bitmap = [image getFrameBitmap:0];
        settings.scale_x = page.getWidth * 0.618f / bitmap.getWidth;
        settings.scale_y = settings.scale_x;

        FSWatermark *watermark = [[FSWatermark alloc] initWithDocument:doc image:image frame_index:0 settings:settings];
        [watermark insertToPage:page];
    };
    void (^AddSingleWatermark)(FSPDFDoc *doc, FSPDFPage *page) = ^(FSPDFDoc *doc, FSPDFPage *page){
        FSWatermarkSettings *settings = [FSWatermarkSettings new];
        settings.flags = FSWatermarkSettingsFlagASPageContents | FSWatermarkSettingsFlagOnTop;
        settings.offset_x = 0;
        settings.offset_y = 0;
        settings.opacity = 90;
        settings.position = FSPosBottomRight;
        settings.rotation = 0.f;
        settings.scale_x = 0.1f;
        settings.scale_y = 0.1f;
        
        FSWatermark *watermark = [[FSWatermark alloc] initWithDocument:doc page:page settings:settings];
        [watermark insertToPage:page];
    };
    
    int page_count = self.doc.getPageCount;
    for(int i=0; i<page_count; i++)
    {
        FSPDFPage *page = [self.doc getPage:i];
        [page startParse:0 pause:nil is_reparse:NO];
        AddTextWatermark(self.doc, page);
        AddBitmapWatermark(self.doc, page,[input_file stringByAppendingPathComponent:@"watermark.bmp"]);
        AddImageWatermark(self.doc, page,[input_file stringByAppendingPathComponent:@"rdk.png"]);
        AddSingleWatermark(self.doc, page);
        
    }
    [self.doc saveAs:path save_flags:FSPDFDocSaveFlagNormal];
    return YES;
}
- (BOOL)searchWithSaveResultsPath:(NSString *)path{
    @try {
        
        NSMutableString *textOutput = @"".mutableCopy;
        
        FSPDFDoc *doc = self.doc;
        // sample 1: search for all pages of doc.
        FSTextSearch *search = [[FSTextSearch alloc] initWithDocument:self.doc cancel:nil flags:FSTextPageParseTextNormal];

        int start_index = 0, end_index = doc.getPageCount - 1;
        [search setStartPage:start_index];
        [search setEndPage:end_index];
        
        NSString *pattern = @"Foxit";
        [search setPattern:pattern];

        FSTextSearchSearchFlags flags = FSTextSearchSearchNormal;
        // If want to specify flags, you can do as followings:
        // flags |= FSTextSearchSearchMatchCase;
        // flags |= FSTextSearchSearchMatchWholeWord;
        // flags |= FSTextSearchSearchConsecutive;
        [search setSearchFlags:(int)flags];
        
        [textOutput appendFormat:@"Begin search %@ at %@.\n", pattern, _docPath];
        [textOutput appendFormat:@"Start index:\t%d\r\n", start_index];
        [textOutput appendFormat:@"End index:\t%d\r\n", end_index];
        [textOutput appendFormat:@"Match key:\t%@\r\n", pattern];
        
        NSString *match_case = flags & FSTextSearchSearchMatchCase ? @"Yes" : @"No";
        [textOutput appendFormat:@"Match case:\t%@\r\n", match_case];
        NSString *match_whole_word = flags & FSTextSearchSearchMatchWholeWord ? @"Yes" : @"No";
        [textOutput appendFormat:@"Match whole word:\t%@\r\n", match_whole_word];
        NSString *match_consecutive = flags & FSTextSearchSearchConsecutive ? @"Yes" : @"No";
        [textOutput appendFormat:@"Consecutive:\t%@\r\n", match_consecutive];
        int match_count = 0;
        while (search.findNext) {
            int page_index = search.getMatchPageIndex;
            int matched_index = match_count;
            [textOutput appendFormat:@"Index of matched pattern:\t%d\r\n", matched_index];
            [textOutput appendFormat:@"\tpage:\t%d\r\n", page_index];
            [textOutput appendFormat:@"\tmatch char start index:\t%d\r\n", search.getMatchStartCharIndex];
            [textOutput appendFormat:@"\tmatch char end index:\t%d\r\n", search.getMatchEndCharIndex];
            [textOutput appendFormat:@"\tmatch sentence start index:\t%d\r\n", search.getMatchSentenceStartIndex];
            [textOutput appendFormat:@"\tmatch sentence:\t%@\r\n", search.getMatchSentence];
            
            FSRectFArray *rect_array = [search getMatchRects];
            int rect_count = rect_array.getSize;
            [textOutput appendFormat:@"\tmatch rectangles count:\t%d\r\n", rect_count];
            for (int i = 0; i < rect_count; i++) {
                FSRectF *rect = [rect_array getAt:i];
                [textOutput appendFormat:@"\trectangle(in PDF space) :%d\t[left = %.4f, bottom = %.4f, right = %.4f, top = %.4f]\r\n", i,
                 rect.left, rect.bottom, rect.right, rect.top];
            }
            match_count ++;
        }
        NSLog(@"Marched %d counts.",match_count);
        // end of sample 1.
        NSLog(@"Search demo finished.");
        
        return [[textOutput dataUsingEncoding:NSUTF8StringEncoding] writeToFile:path atomically:YES];
    } @catch (NSException *exception) {
        return NO;
    }
    return  YES;
}

- (BOOL)exampleGraphicsObjectsInputPath:(NSString *)inputPath saveAsNewDocAtPath:(NSString *)path{
    void (^AddTextObjects)(FSPDFPage *page) = ^(FSPDFPage *page){
        long position = [page getLastGraphicsObjectPosition:FSGraphicsObjectTypeText];
        FSTextObject* text_object = [FSTextObject create];
        [text_object setFillColor:0xFFFF7F00];
        
        FSTextState *state = [FSTextState new];
        state.font_size = 80.f;
        FSFont *font = [[FSFont alloc] initWithFont_id:FSFontStdIDCourier];
        state.font = font;
        state.textmode = FSTextStateModeFill;
        [text_object setTextState:page text_state:state is_italic:NO weight:750];
        
        // Set text.
        [text_object setText:@"Foxit Software"];
        long last_position = [page insertGraphicsObject:position graphics_object:text_object];
          
        FSRectF *rect = [text_object getRect];
        float offset_x = ([page getWidth] - (rect.right - rect.left)) / 2;
        float offset_y = [page getHeight] * 0.8f - (rect.top - rect.bottom) / 2;
        FSMatrix2D *mat = [[FSMatrix2D alloc] initWithA1:1 b1:0 c1:0 d1:1 e1:offset_x f1:offset_y];
        [text_object transform:mat need_transform_clippath:NO];

        // Generate content
        [page generateContent];

        // Clone a text object from the old text object.
        text_object = [[text_object clone] getTextObject];
        
        font = [[FSFont alloc] initWithFont_id:FSFontStdIDTimes];
        state.font = font;
        state.font_size = 48;
        state.textmode = FSTextStateModeFillClip;
        
        [text_object setTextState:page text_state:state is_italic:YES weight:750];
        [text_object setText:@"www.foxitsoftware.com"];

        [text_object setFillColor:0xFFAAAAAA];
        [text_object setStrokeColor:0xFFF68C21];
        [page insertGraphicsObject:last_position graphics_object:text_object];

        rect = [text_object getRect];
        offset_x = (page.getWidth - (rect.right - rect.left)) / 2;
        offset_y = page.getHeight * 0.618f - (rect.top - rect.bottom) / 2;
        mat = [[FSMatrix2D alloc] initWithA1:1 b1:0 c1:0 d1:1 e1:offset_x f1:offset_y];
        [text_object transform:mat need_transform_clippath:NO];

        // Generate content again after transformation.
        [page generateContent];

        //Show how to get the characters' information of text object.
        int text_object_charcount = text_object.getCharCount;
        NSMutableString *text_doc = @"".mutableCopy;
        [text_doc appendFormat:@"The new text object has %d characters.\n", text_object_charcount];

        for (int i = 0; i < text_object_charcount; i++) {
            //The character's position.
            FSPointF *char_pos = [text_object getCharPos:i];
            //The character's width.
            float width = [text_object getCharWidthByIndex:i];
            //The character's height.
            float height = [text_object getCharHeightByIndex:i];

            [text_doc appendFormat:@"The position of the %d characters is (%f, %f).The width and height of the characters is (%f, %f).\n", i, char_pos.x, char_pos.y, width, height];
        }
        [[text_doc dataUsingEncoding:NSUTF8StringEncoding] writeToFile:[NSString stringWithFormat:@"%@/text_objects_info.txt",[path stringByDeletingLastPathComponent]] atomically:YES];
    };
    
    void (^AddImageObjects)(FSPDFPage *page, NSString *image_file) = ^(FSPDFPage *page, NSString *image_file){
        long position = [page getLastGraphicsObjectPosition:FSGraphicsObjectTypeImage];
        FSImage *image = [[FSImage alloc] initWithPath:image_file];
        FSImageObject* image_object = [FSImageObject create:[page getDocument]];
        [image_object setImage:image frame_index:0];

        float width = [image getWidth];
        float height = [image getHeight];

        float page_width = page.getWidth;
        float page_height = page.getHeight;

        // Please notice the matrix value.
        FSMatrix2D *mat = [[FSMatrix2D alloc] initWithA1:width b1:0 c1:0 d1:height e1:(page_width - width) / 2.0f f1:(page_height - height) / 2.0f];
        [image_object setMatrix:mat];
        
        [page insertGraphicsObject:position graphics_object:image_object];
        [page generateContent];
    };
    
    void (^AddPathObjects)(FSPDFPage *page, FSShadingObject* black_pieces, FSShadingObject* white_pieces) = ^(FSPDFPage *page, FSShadingObject* black_pieces, FSShadingObject* white_pieces){
        long position = [page getLastGraphicsObjectPosition:FSGraphicsObjectTypePath];
        FSPathObject* path_object = [FSPathObject create];
        FSPath *path = [FSPath new];
        float page_width = page.getWidth;
        float page_height = page.getHeight;

        float width = MIN(page_width, page_height) / 20.0f;
        float start_x = (page_width - width * 18.0f) / 2.0f;
        float start_y = (page_height - width * 18.0f) / 2.0f;

        // Draw a chess board
        for (int i = 0; i < 19; i++) {
            float x1 = start_x;
            float y1 = i * width + start_y;

            float x2 = start_x + 18 * width;
            [path moveTo:[[FSPointF alloc] initWithX:x1 Y:y1]];
            [path lineTo:[[FSPointF alloc] initWithX:x2 Y:y1]];

            x1 = i * width + start_x;
            y1 = start_y;

            float y2 = 18 * width + start_y;
            [path moveTo:[[FSPointF alloc] initWithX:x1 Y:y1]];
            [path lineTo:[[FSPointF alloc] initWithX:x1 Y:y2]];
        }
        int star[3] = {3, 9, 15};
        for (int i = 0; i < 3; i++) {
          for (int j = 0; j < 3; j++) {
              FSRectF *rect = [[FSRectF alloc] initWithLeft1:start_x + star[i] * width - width / 12 bottom1:start_y + star[j] * width - width / 12 right1:start_x + star[i] * width + width / 12 top1:start_y + star[j] * width + width / 12];
              [path appendRect:rect];
          }
        }
        [path_object setPathData:path];
        
        [path_object setFillColor:0xFF000000];
        [path_object setFillMode:FSFillModeAlternate];
        [path_object setStrokeState:YES];
        [path_object setStrokeColor:0xFF000000];
        
        [page insertGraphicsObject:position graphics_object:path_object];
        [page generateContent];

        // Draw pieces
        
        void (^AddPieces)(FSPDFPage *page, FSShadingObject* orignal_pieces, FSRectF *dst_rect) = ^(FSPDFPage *page, FSShadingObject* orignal_pieces, FSRectF *dst_rect){
            long position = [page getFirstGraphicsObjectPosition:FSGraphicsObjectTypeAll];
            FSShadingObject* pieces = (FSShadingObject*)[orignal_pieces clone];

            FSRectF *piece_rect = [pieces getRect];

            // Calculates the transformation matrix between dst_rect and  piece_rect.
            float a = (dst_rect.right - dst_rect.left) / (piece_rect.right - piece_rect.left);
            float d = (dst_rect.top - dst_rect.bottom) / (piece_rect.top - piece_rect.bottom);
            float e = dst_rect.left - piece_rect.left * a;
            float f = dst_rect.top - piece_rect.top * d;

           // Transform rect.
            FSMatrix2D *mat = [[FSMatrix2D alloc] initWithA1:a b1:0 c1:0 d1:d e1:e f1:f];
            [pieces transform:mat need_transform_clippath:NO];
            [page insertGraphicsObject:position graphics_object:pieces];
            
            [page generateContent];
        };
        
        FSPointF *pieces_vector[2][8] = {
            {
                [[FSPointF alloc] initWithX:3 Y:3], [[FSPointF alloc] initWithX:3 Y:7], [[FSPointF alloc] initWithX:3 Y:15], [[FSPointF alloc] initWithX:13 Y:2],
                [[FSPointF alloc] initWithX:13 Y:16], [[FSPointF alloc] initWithX:13 Y:17], [[FSPointF alloc] initWithX:15 Y:16], [[FSPointF alloc] initWithX:16 Y:16]
            },
            {
                [[FSPointF alloc] initWithX:11 Y:16], [[FSPointF alloc] initWithX:12 Y:14], [[FSPointF alloc] initWithX:14 Y:4], [[FSPointF alloc] initWithX:14 Y:15],
                [[FSPointF alloc] initWithX:15 Y:3], [[FSPointF alloc] initWithX:15 Y:9], [[FSPointF alloc] initWithX:15 Y:15], [[FSPointF alloc] initWithX:16 Y:15]
            }
        };
        for (int k = 0; k < 2; k++) {
          for (int i = 0; i < sizeof(pieces_vector[k]) / sizeof(pieces_vector[k][0]); i++) {
              int x = (int)(pieces_vector[k][i].x);
              int y = (int)(pieces_vector[k][i].y);
              FSRectF *rectf = [[FSRectF alloc] initWithLeft1:start_x + x * width - width / 2.f bottom1:start_y + y * width - width / 2.f right1:start_x + x * width + width / 2.f top1:start_y + y * width + width / 2.f];
              AddPieces(page, k % 2 ? white_pieces : black_pieces, rectf);
          }
        }
    };


    
    FSPDFDoc *doc = self.doc;
    // Get original shading objects from PDF page.
    FSPDFPage *original_page = [doc getPage:0];
    [original_page startParse:0 pause:nil is_reparse:NO];
    long position = [original_page getFirstGraphicsObjectPosition:FSGraphicsObjectTypeShading];
    if (!position) return NO;
    FSShadingObject* black_pieces = (FSShadingObject*)[original_page getGraphicsObject:position];
    position = [original_page getNextGraphicsObjectPosition:position filter:FSGraphicsObjectTypeShading];
    FSShadingObject* white_pieces = (FSShadingObject*)[original_page getGraphicsObject:position];

    // Add a new PDF page and insert text objects.
    FSPDFPage *page = [doc insertPageSize:0 size:FSPDFPageSizeLetter];
    AddTextObjects(page);

    // Add a new PDF page and insert image objects.
    page = [doc insertPageSize:1 size:FSPDFPageSizeLetter];
    NSString *image_file = [NSString stringWithFormat:@"%@/rdk.png",inputPath];
    AddImageObjects(page, image_file);

    // Add a new PDF page and insert path objects, and copy shading objects.
    page = [doc insertPageSize:2 size:FSPDFPageSizeLetter];
    AddPathObjects(page, black_pieces, white_pieces);
    
    [doc saveAs:path save_flags:FSPDFDocSaveFlagNormal];
    NSLog(@"Add graphics objects to PDF file.");
    return YES;
}

+ (BOOL)image2pdf:(NSString *)input_file saveAsNewDocAtPath:(NSString *)path{
    NSArray *imageNames = @[@"watermark.bmp",@"image_samples.jpg",@"TIF2Pages.tif",@"image005.gif"];
    for (NSString *imageName in imageNames) {
        NSString *imagePath = [input_file stringByAppendingFormat:@"/%@",imageName];
        FSImage *image = [[FSImage alloc] initWithPath:imagePath];
        int count = [image getFrameCount];
        FSPDFDoc *doc = [[FSPDFDoc alloc] init];
        for (int i = 0 ; i < count; i++) {
            FSPDFPage *page = [doc insertPage:i width:[image getWidth] height:[image getHeight]];
            [page startParse:0 pause:nil is_reparse:NO];
            [page addImage:image frame_index:i position:[[FSPointF alloc] initWithX:0 Y:0]width:[image getWidth]  height:[image getHeight] auto_generate_content:YES];
        }
        NSString *savePath = [path stringByAppendingFormat:@"/%@_%@.pdf",[imageName stringByDeletingPathExtension],[imageName pathExtension]];
        [doc saveAs:savePath save_flags:FSPDFDocSaveFlagNormal];
    }

    
    return YES;
}

- (BOOL)setPropertiesOfBookmarks:(FSBookmark *)bookmark {
    if (!bookmark) {
        return NO;
    }

    FSBookmark *bk = nil;
    for (bk = bookmark; ![bk isEmpty]; bk = [bk getNextSibling]) {
        NSString *title = [bk getTitle];
        [bk setTitle:[NSString stringWithFormat:@"%@_renamed", title]];
        [bk setColor:0x0000ffff];
        [bk setStyle:FSBookmarkStyleBold];

        FSBookmark *child = [bk getFirstChild];
        if (child && ![child isEmpty]) {
            [self setPropertiesOfBookmarks:child];
        }
    }

    return YES;
}

- (BOOL)save2PNGWithBitmap:(FSBitmap *)fsBitmap CGAffineTransform:(CGAffineTransform *)transform Path:(NSString *)path {
    if (!fsBitmap || !path) {
        return NO;
    }

    float pageWidth = [fsBitmap getWidth];
    float pageHeight = [fsBitmap getHeight];
    if (pageWidth <= 0 || pageHeight <= 0) {
        return NO;
    }

    CGDataProviderRef provider = CGDataProviderCreateWithCFData((CFDataRef)[fsBitmap getBuffer]);
    CGImageRef image = CGImageCreate(pageWidth, pageHeight, 8, 32, [fsBitmap getPitch], CGColorSpaceCreateDeviceRGB(), kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big, provider, NULL, YES, kCGRenderingIntentDefault);

    UIImage *uiImage = [UIImage imageWithCGImage:image];
    BOOL bRet = [UIImagePNGRepresentation(uiImage) writeToFile:path atomically:YES];

    CGImageRelease(image);
    CGDataProviderRelease(provider);

    return bRet;
}

@end
