/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

import UIKit

protocol TabsToolbarDelegate {
    
//    func tabsToolbar(showFileList tabsToolbar: TabsToolbar)
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, didCloseTabAt index: Int)
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, didSelectTabAt index: Int)
    
}

protocol TabsToolbarDataSource {
    
    func tabsToolbar(numberOfTabsIn tabsToolbar: TabsToolbar) -> Int
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, infoForTabAt index: Int) -> TabInfo
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, moveTabAt sourceIndex: Int, to destinationIndex: Int)
    
}

class TabsToolbar: UIView {
    
    var tabsToolbarDelegate: TabsToolbarDelegate?
    var dataSource: TabsToolbarDataSource?
    var tabs: UICollectionView?
    fileprivate let height = CGFloat(30.0)
    fileprivate let TabReuseID = "TabReuseID"
    fileprivate var tabLongPressOffsetX: CGFloat?
    fileprivate var lastIndexPath = IndexPath.init(row: 0, section: 0);
    
//    fileprivate var addTabButton: UIButton?
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.clipsToBounds = true
        setupSubviews()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func reloadData() {
        tabs!.reloadData()
    }
    
    func selectTab(at index: Int) {
        let indexPath = IndexPath(item: index, section: 0)
        let alreadySelected = tabs!.indexPathsForSelectedItems?.contains(indexPath) ?? false
        if !alreadySelected {
            tabs!.selectItem(at: indexPath, animated: true, scrollPosition: .centeredHorizontally)
        }
        lastIndexPath = indexPath;
    }
    
    func removeTab(at index: Int) {
        let indexPath = IndexPath(item: index, section: 0)
        tabs!.deleteItems(at: [ indexPath ])
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        tabs?.collectionViewLayout.invalidateLayout()
    }
}

extension TabsToolbar: UICollectionViewDataSource {
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return dataSource?.tabsToolbar(numberOfTabsIn: self) ?? 0
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let tab = collectionView.dequeueReusableCell(withReuseIdentifier: TabReuseID, for: indexPath) as! Tab
        tab.delegate = self
        tab.info = dataSource!.tabsToolbar(self, infoForTabAt: indexPath.row)
        tab.backgroundColor = UIColor.init(named: "Tab_Un_Select_Color");
        let selectedBackgroundView = UIView.init();
        selectedBackgroundView.backgroundColor = UIColor.init(named: "Tab_Select_Color");
        tab.selectedBackgroundView = selectedBackgroundView;
        return tab
    }
    
    func collectionView(_ collectionView: UICollectionView, moveItemAt sourceIndexPath: IndexPath, to destinationIndexPath: IndexPath) {
        dataSource!.tabsToolbar(self, moveTabAt: sourceIndexPath.row, to: destinationIndexPath.row)
    }
    
}

extension TabsToolbar: UICollectionViewDelegate {
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        if (lastIndexPath.section == indexPath.section && lastIndexPath.item == indexPath.item) {return}
        collectionView.scrollToItem(at: indexPath, at: .centeredHorizontally, animated: true)
        self.tabsToolbarDelegate?.tabsToolbar(self, didSelectTabAt: indexPath.row)
    }
    
}

extension TabsToolbar: UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        let num = collectionView.numberOfItems(inSection: indexPath.section)
        var w = collectionView.bounds.width / CGFloat(num)
        var minWidth = (UIDevice.current.userInterfaceIdiom == .phone) ? 200 : 300
        if (self.traitCollection.horizontalSizeClass == .compact && UIDevice.current.userInterfaceIdiom == .pad){
            minWidth = 0;
        }
        w = max(w, CGFloat(minWidth))
        return CGSize(width: w, height: collectionView.bounds.height)
    }
    
}

extension TabsToolbar: TabDelegate {
    
    func closeTab(_ tab: Tab) {
        let indexPath = tabs!.indexPath(for: tab)!
        self.tabsToolbarDelegate?.tabsToolbar(self, didCloseTabAt: indexPath.row)
        self.tabs?.reloadData()
    }
    
}

fileprivate extension TabsToolbar {
    
    func setupSubviews() {
//        let addTabButton = UIButton(type: .custom)
//        addTabButton.setImage(UIImage(named: "newTab"), for: .normal)
//        addTabButton.sizeToFit()
//        addTabButton.contentEdgeInsets = UIEdgeInsets(top: (height - addTabButton.bounds.height) / 2, left: 6, bottom: (height - addTabButton.bounds.height) / 2, right: 6)
//        addTabButton.addTarget(self, action: #selector(onClickAddTab(_:)), for: .touchUpInside)
//        addSubview(addTabButton)
//        self.addTabButton = addTabButton
        
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .horizontal
        layout.sectionInset = .zero
        layout.minimumInteritemSpacing = 0
        layout.minimumLineSpacing = 0
        tabs = UICollectionView(frame: bounds, collectionViewLayout: layout)
        tabs!.backgroundColor = UIColor.clear
        tabs!.contentInset = .zero
        tabs!.allowsMultipleSelection = false
        tabs!.register(Tab.self, forCellWithReuseIdentifier: TabReuseID)
        tabs!.showsHorizontalScrollIndicator = false
        tabs!.delegate = self
        tabs!.dataSource = self
        addSubview(tabs!)
        
        // draggable cells
        if #available(iOS 9.0, *) {
            let longPress = UILongPressGestureRecognizer(target: self, action: #selector(onLongPress(gesture:)))
            longPress.delaysTouchesBegan = true;
            tabs!.addGestureRecognizer(longPress)
        }
        
//        addTabButton.translatesAutoresizingMaskIntoConstraints = false
        tabs!.translatesAutoresizingMaskIntoConstraints = false
        let allViews = [/*"button": addTabButton, */"tabs": tabs!] as [String : Any]
        addConstraints(NSLayoutConstraint.constraints(withVisualFormat: "H:|-0-[tabs]-0-|", options: [.alignAllCenterY], metrics: nil, views: allViews))
        addConstraints(NSLayoutConstraint.constraints(withVisualFormat: "V:|-0-[tabs(\(height))]-0-|", options: [], metrics: nil, views: allViews))
    }
    
//    @IBAction func onClickAddTab(_ button: UIButton) {
//        self.tabsToolbarDelegate?.tabsToolbar(showFileList: self)
//    }
    
    @available(iOS 9.0, *)
    @IBAction func onLongPress(gesture: UILongPressGestureRecognizer) {
        
        switch gesture.state {
        case .began:
            guard let indexPath = tabs!.indexPathForItem(at: gesture.location(in: tabs!)),
                let tab = tabs!.cellForItem(at: indexPath) else {
                    return
            }
            tabs!.beginInteractiveMovementForItem(at: indexPath)
            tabLongPressOffsetX = gesture.location(in: tab).x - tab.bounds.width / 2 // offset to centerX of tab
        case .changed:
            if tabLongPressOffsetX != nil {
                let tabCenterX = gesture.location(in: tabs!).x - tabLongPressOffsetX!
                tabs!.updateInteractiveMovementTargetPosition(CGPoint(x: tabCenterX, y: tabs!.center.y))
            }
        case .ended:
            tabs!.endInteractiveMovement()
            tabLongPressOffsetX = nil
        default:
            tabs!.cancelInteractiveMovement()
            tabLongPressOffsetX = nil
        }
    }
    
}
