/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

import UIKit

public let BtnSaveTag : Int = 100000;
public let BtnSaveAsTag : Int = 100001;
public let BtnDiscardTag : Int = 100002;

protocol TabsViewControllerDelegate {
    
    func tabsViewController(showFileList tabsViewController: TabsViewController)
    
    func tabsViewController(exit tabsViewController: TabsViewController)
    
}

class TabsViewController: UIViewController {
    
    var delegate: TabsViewControllerDelegate?
    fileprivate var tabInfos: [TabInfo] = []
    fileprivate var tabsToolbar: TabsToolbar?
    
    override func viewDidLoad() {
        self.automaticallyAdjustsScrollViewInsets = false
    }
    
    var allowTabs: Bool = false {
        didSet {
            if allowTabs {
                if tabsToolbar == nil {
                    tabsToolbar = TabsToolbar()
                    tabsToolbar?.dataSource = self
                    tabsToolbar!.tabsToolbarDelegate = self
                }
            } else {
                for tabInfo in tabInfos {
                    guard let PDFView = tabInfo.PDFView else { continue }
                    PDFView.removeFromSuperview()
                    saveAndCloseDoc(in: PDFView)
                }
                tabInfos.removeAll()
                tabsToolbar?.removeFromSuperview()
                tabsToolbar = nil
            }
        }
    }
    
    func openPDF(atPath path: String, completion: ((FSErrorCode) -> Swift.Void)? = nil) {
        if let tabInfo = findTab(path: path) {
            showTab(tabInfo)
            completion?(.errSuccess)
        } else {
            addNewTab(path: path, completion: completion)
        }
    }
    
    func suspendAutoFullScreen() -> Void {
        for tabs in tabInfos {
            tabs.uiextensions?.suspendAutoFullScreen()
        }
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)
        tabsToolbar?.tabs?.collectionViewLayout.invalidateLayout()
    }
    
    var currentPDFView: FSPDFViewCtrl? {
        get {
            return view.subviews.first(where: { $0 as? FSPDFViewCtrl != nil }) as? FSPDFViewCtrl
        }
    }
    
    var currentTabInfo: TabInfo? {
        get {
            if let currentPDFView = self.currentPDFView {
                return tabInfos.first(where: { $0.PDFView == currentPDFView })
            } else {
                return nil
            }
        }
    }
    
}

extension TabsViewController: UIExtensionsManagerDelegate, UIPopoverPresentationControllerDelegate, UIAdaptivePresentationControllerDelegate {
    
    func popoverPresentationControllerDidDismissPopover(_ popoverPresentationController: UIPopoverPresentationController) {
        guard let tabInfo = self.currentTabInfo else {
            print("An error occurred when try to quit pdf.")
            return
        }
        
        tabInfo.uiextensions?.resumeAutoFullScreen()
    }
    
    func adaptivePresentationStyle(for controller: UIPresentationController) -> UIModalPresentationStyle {
        return .none
    }
    
    func popoverPresentationController(_ popoverPresentationController: UIPopoverPresentationController, willRepositionPopoverTo rect: UnsafeMutablePointer<CGRect>, in view: AutoreleasingUnsafeMutablePointer<UIView>) {
        guard let tabInfo = self.currentTabInfo else {
            print("An error occurred when try to quit pdf.")
            return
        }
        tabInfo.uiextensions?.suspendAutoFullScreen()
    }
    
    func uiextensionsManager(_ uiextensionsManager: UIExtensionsManager, onToolBar type: FSToolbarType, hidden: Bool) {
        if self.currentPDFView == uiextensionsManager.pdfViewCtrl && type == FSToolbarType.topBar {
        }
    }
    
    func uiextensionsManager(_ uiextensionsManager: UIExtensionsManager, openNewDocAtPath path: String, shouldCloseCurrentDoc closeCurrentDoc: Bool,completionCallBack : (()->Void)) -> Bool {
        if FileManager.default.fileExists(atPath: path) {
            if !allowTabs {
                let pdfView = uiextensionsManager.pdfViewCtrl
                if uiextensionsManager.isDocModified {
                    if (pdfView.filePath == path) {
                        pdfView.openDoc(path, password: nil, completion: nil)
                    }else{
                        if(closeCurrentDoc) {
                            pdfView.closeDoc({
                                pdfView.openDoc(path, password: nil, completion: nil)
                            })
                        }else{
                            saveAndCloseDoc(in: pdfView, completion: {
                                pdfView.openDoc(path, password: nil, completion: nil)
                            })
                        }
                    }
                } else {
                    pdfView.closeDoc({
                        pdfView.openDoc(path, password: nil, completion: nil)
                    })
                }
            } else {
                if(!closeCurrentDoc) {
                    if let tabInfo = findTab(path: path) {
                        self.closeTab(tabInfo, save: BtnDiscardTag)
                    }
                    DispatchQueue.main.async {
                        self.openPDF(atPath: path)
                    }
                }
                else {
                    let originalPDFFilePath = uiextensionsManager.pdfViewCtrl.filePath
                    let originaltabInfo = self.findTab(path: originalPDFFilePath!)
                    originaltabInfo!.PDFView!.closeDoc({
                        originaltabInfo!.PDFView!.openDoc(path, password: nil, completion: {errocode -> Void in
                            originaltabInfo?.path = path;
                            originaltabInfo?.doc = originaltabInfo!.PDFView!.currentDoc;
                            self.tabsToolbar?.reloadData()
                        })
                    })
                }
            }
            return true
        } else {
            return false
        }
    }
    
    func quitUIExtensionsManager(_ uiextensionsManager: UIExtensionsManager, control: UIControl) {
        if allowTabs {
            self.delegate?.tabsViewController(exit: self)
            return
        }
        
        guard let tabInfo = self.currentTabInfo else {
            print("An error occurred when try to quit pdf.")
            return
        }
        if uiextensionsManager.isDocModified == false {
            self.closeTab(tabInfo, save: BtnDiscardTag)
            return
        }
        
        let menu = UIView(frame: CGRect(x: 0, y: 0, width: 160, height: 150))
        
        let save = UIButton(frame: CGRect(x: 10, y: 15, width: 130, height: 20))
        save.setTitle("Save", for: .normal)
        save.setTitleColor(UIColor(red: 23.0 / 255.0, green: 156.0 / 255.0, blue: 216.0 / 255.0, alpha: 1), for: .normal)
        save.setTitleColor(UIColor.gray, for: .highlighted)
        save.titleLabel?.font = UIFont.systemFont(ofSize: 15)
        save.addTarget(self, action: #selector(self.buttonSaveClick), for: .touchUpInside)
        menu.addSubview(save)
        
        let saveas = UIButton(frame: CGRect(x: 10, y: 65, width: 130, height: 20))
        saveas.setTitle("SaveAs", for: .normal)
        saveas.setTitleColor(UIColor(red: 23.0 / 255.0, green: 156.0 / 255.0, blue: 216.0 / 255.0, alpha: 1), for: .normal)
        saveas.setTitleColor(UIColor.gray, for: .highlighted)
        saveas.titleLabel?.font = UIFont.systemFont(ofSize: 15)
        saveas.addTarget(self, action: #selector(self.buttonSaveAsClick), for: .touchUpInside)
        menu.addSubview(saveas)
        
        let discard = UIButton(frame: CGRect(x: 10, y: 115, width: 130, height: 20))
        discard.setTitle("Discard Changes", for: .normal)
        discard.setTitleColor(UIColor(red: 23.0 / 255.0, green: 156.0 / 255.0, blue: 216.0 / 255.0, alpha: 1), for: .normal)
        discard.setTitleColor(UIColor.gray, for: .highlighted)
        discard.titleLabel?.font = UIFont.systemFont(ofSize: 15)
        discard.addTarget(self, action: #selector(self.buttonDiscardChangeClick), for: .touchUpInside)
        menu.addSubview(discard)
        
        let line = UIView(frame: CGRect(x: 10, y: 50, width: 130, height: 1))
        line.backgroundColor = UIColor(red: 0.92, green: 0.92, blue: 0.92, alpha: 1)
        menu.addSubview(line)
        
        let line2 = UIView(frame: CGRect(x: 10, y: 100, width: 130, height: 1))
        line2.backgroundColor = UIColor(red: 0.92, green: 0.92, blue: 0.92, alpha: 1)
        menu.addSubview(line2)
        
        let rootViewController = UIApplication.shared.keyWindow?.rootViewController
        let alert = UIAlertController.init(title: nil, message: nil, preferredStyle:.actionSheet)
        
        let saveAction = UIAlertAction.init(title: "Save", style: .default, handler: { _ in
            self.closeTab(tabInfo, save: BtnSaveTag)
        })
        
        let saveasAction = UIAlertAction.init(title: "SaveAs", style: .default, handler: { _ in
            self.closeTab(tabInfo, save: BtnSaveAsTag)
        })
        
        let discardAction = UIAlertAction.init(title: "Discard Changes", style: .default, handler: { _ in
            self.closeTab(tabInfo, save: BtnDiscardTag)
        })
        
        alert.addAction(saveAction)
        alert.addAction(saveasAction)
        alert.addAction(discardAction)
        
        alert.modalPresentationStyle = UIModalPresentationStyle.popover
        let popPresenter = alert.popoverPresentationController
        popPresenter?.sourceView = control
        popPresenter?.sourceRect = control.bounds
        popPresenter?.delegate = self
        
        rootViewController?.present(alert, animated: true, completion: nil)

    }
    
    @objc func buttonSaveClick() {
        guard let tabInfo = self.currentTabInfo else {
            print("An error occurred when try to quit pdf.")
            return
        }
        self.closeTab(tabInfo, save: BtnSaveTag)
    }
    
    @objc func buttonSaveAsClick() {
        guard let tabInfo = self.currentTabInfo else {
            print("An error occurred when try to quit pdf.")
            return
        }
        self.closeTab(tabInfo, save: BtnSaveAsTag)
    }
    
    @objc func buttonDiscardChangeClick() {
        guard let tabInfo = self.currentTabInfo else {
            print("An error occurred when try to quit pdf.")
            return
        }
        self.closeTab(tabInfo, save: BtnDiscardTag)
    }
    
}

extension TabsViewController: TabsToolbarDelegate {
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, didSelectTabAt index: Int) {
        showTab(tabInfos[index])
    }
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, didCloseTabAt index: Int) {
        if (self.currentPDFView?.extensionsManager as? UIExtensionsManager)?.isDocModified == false {
            self.closeTab(self.tabInfos[index], save: BtnDiscardTag)
            return
        }
        
        let rootViewController = UIApplication.shared.keyWindow?.rootViewController
        let alert = UIAlertController.init(title: nil, message: nil, preferredStyle: (UIDevice.current.userInterfaceIdiom == .phone) ? .actionSheet : .alert)
        let cancelAction = UIAlertAction.init(title: "Cancel", style: .cancel, handler: nil)
        let saveAction = UIAlertAction.init(title: "Save", style: .default, handler: { _ in
            self.closeTab(self.tabInfos[index], save: BtnSaveTag)
        })
        
        let saveasAction = UIAlertAction.init(title: "SaveAs", style: .default, handler: { _ in
            self.closeTab(self.tabInfos[index], save: BtnSaveAsTag)
        })
        
        let discardAction = UIAlertAction.init(title: "Discard Changes", style: .default, handler: { _ in
            self.closeTab(self.tabInfos[index], save: BtnDiscardTag)
        })
        alert.addAction(cancelAction)
        alert.addAction(saveAction)
        alert.addAction(saveasAction)
        alert.addAction(discardAction)
        rootViewController?.present(alert, animated: true, completion: nil)
    }
    
}

extension TabsViewController: TabsToolbarDataSource {
    
    func tabsToolbar(numberOfTabsIn tabsToolbar: TabsToolbar) -> Int {
        return tabInfos.count
    }
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, infoForTabAt index: Int) -> TabInfo {
        return tabInfos[index]
    }
    
    func tabsToolbar(_ tabsToolbar: TabsToolbar, moveTabAt sourceIndex: Int, to destinationIndex: Int) {
        let moved = tabInfos.remove(at: sourceIndex)
        tabInfos.insert(moved, at: destinationIndex)
    }
    
}

extension TabsViewController: IDocEventListener {
    
    func onDocOpened(_ document: FSPDFDoc?, error: Int32) {
    }
    
    func onDocWillClose(_ document: FSPDFDoc) {
    }
    
}

fileprivate extension TabsViewController {
    
    func showTab(_ tabInfo: TabInfo) {
        self.currentPDFView?.removeFromSuperview()
        if let PDFView = tabInfo.PDFView {
            //Suspend the auto fullscreen mode first.
            for tab in tabInfos {
                tab.uiextensions?.suspendAutoFullScreen()
            }
            PDFView.frame = view.bounds
            DispatchQueue.main.async {
                self.view.addSubview(PDFView)
                let extensionsManager = PDFView.extensionsManager as! UIExtensionsManager
                self.wrapTopToolbar(of: extensionsManager)
                //Resume the auto fullscreen mode, if fullscreen mode is set.
                extensionsManager.resumeAutoFullScreen()
            }
        } else {
            fatalError()
        }
        tabsToolbar?.selectTab(at: tabInfos.index(where: { $0 === tabInfo })!)
    }
    
    func findTab(path: String) -> TabInfo? {
        if let index = tabInfos.index(where: { $0.path == path }) {
            return tabInfos[index]
        } else {
            return nil
        }
    }
    
    func addNewTab(path: String, completion: ((FSErrorCode) -> Swift.Void)? = nil) {
        let PDFView = FSPDFViewCtrl(frame: view.bounds)
        PDFView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        PDFView.setRMSAppClientId("972b6681-fa03-4b6b-817b-c8c10d38bd20", redirectURI: "com.foxitsoftware.com.mobilepdf-for-ios://authorize")
        PDFView.register(self as IDocEventListener)
        
        var data: Data? = nil
        if let url = Bundle.main.url(forResource: "uiextensions_config", withExtension: "json") {
            data = try? Data(contentsOf: url)
        }
        let extensionsManager = UIExtensionsManager(pdfViewControl: PDFView, configuration: data)
        let value : UIExtensionsManager? = extensionsManager
        if  value == nil {
            let errorcode: FSErrorCode = .errFile;
            completion?(errorcode)
            return;
        }
        
        PDFView.extensionsManager = extensionsManager
        extensionsManager.delegate = self
        extensionsManager.isMultiFileMode = allowTabs;
        let tabInfo = TabInfo(path: path, doc: nil, PDFView: PDFView, uiextensions: extensionsManager)
        tabInfos.append(tabInfo)
        tabsToolbar?.reloadData()
        showTab(tabInfo)
        PDFView.openDoc(path, password: nil) { [weak self, weak tabInfo, weak PDFView] (errorCode) in
            if errorCode == .errSuccess {
                if (tabInfo != nil) {
                    tabInfo!.doc = PDFView?.currentDoc
                }else{
                    if self!.allowTabs {
                        let tabInfo = TabInfo(path: path, doc: nil, PDFView: PDFView, uiextensions: PDFView?.extensionsManager as? UIExtensionsManager)
                        self!.tabInfos.append(tabInfo)
                        self!.tabsToolbar?.reloadData()
                        self!.showTab(tabInfo)
                    }
                }
            } else {
                self!.closeTab(tabInfo!, save: BtnDiscardTag)
            }
            completion?(errorCode)
        }
    }
    
    func closeTab(_ tabInfo: TabInfo, save: Int) {
        guard let PDFView = tabInfo.PDFView else { return }
        
        if (save == BtnSaveAsTag) {
            if let extensionsManager = PDFView.extensionsManager as? UIExtensionsManager {
                extensionsManager.documentSave(as: {
                    PDFView.removeFromSuperview()
                    
                    guard let index = self.tabInfos.index(where: { $0 === tabInfo }) else { return }
                    self.tabInfos.remove(at: index)
                    self.tabsToolbar?.removeTab(at: index)
                    if self.tabInfos.isEmpty {
                        self.delegate?.tabsViewController(exit: self)
                    } else {
                        let prevIndex = max(0, index - 1)
                        self.showTab(self.tabInfos[prevIndex])
                    }
                }, error: nil)
            }
            return
        }
        
        PDFView.removeFromSuperview()

        if save == BtnSaveTag {
            saveAndCloseDoc(in: PDFView)
        }
        else {
            if PDFView.currentDoc != nil {
                PDFView.closeDoc(nil)
            }
        }
        
        guard let index = tabInfos.index(where: { $0 === tabInfo }) else { return }
        tabInfos.remove(at: index)
        tabsToolbar?.removeTab(at: index)
        if tabInfos.isEmpty {
            delegate?.tabsViewController(exit: self)
        } else {
            let prevIndex = max(0, index - 1)
            showTab(self.tabInfos[prevIndex])
        }
    }
    
    func saveAndCloseDoc(in PDFView: FSPDFViewCtrl, completion: (() -> Swift.Void)? = nil) {
        guard PDFView.currentDoc != nil else { return }
        if let path = PDFView.filePath {
            
            let sourceURL = URL(fileURLWithPath: path)
            let fileName = sourceURL.lastPathComponent
            let tempURL = URL(fileURLWithPath: NSTemporaryDirectory()).appendingPathComponent(fileName)
            var saveFlag: FSPDFDocSaveFlags = .incremental
            if let extensionsManager = PDFView.extensionsManager as? UIExtensionsManager {
                saveFlag = extensionsManager.docSaveFlag
            }
            let saved = PDFView.saveDoc(tempURL.path, flag: saveFlag)
            PDFView.closeDoc({
                if saved {
                    do {
                        let _ = try FileManager.default.replaceItemAt(sourceURL, withItemAt: tempURL)
                    } catch {
                        print("failed to overwrite file: \(error)")
                    }
                }
                completion?()
            })
        } else {
            PDFView.closeDoc({
                completion?()
            })
        }
    }
    
    func wrapTopToolbar(of extensionsManager: UIExtensionsManager) {
        let originTopToolbar = extensionsManager.topToolbar
        if let tabsToolbar = tabsToolbar {
            originTopToolbar?.getContentView().addArrangedSubview(tabsToolbar);
        }
    }
}
