/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to 
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement 
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.

 */

import Foundation
import UIKit
import CoreGraphics

class PlatformAPIDemo: NSObject {
    var doc: FSPDFDoc!
    var docPath: String!
    
    /** @brief	Load document from path by using the specified password. */
    init(_ path: String, passwd: String?) {
        
        self.doc = FSPDFDoc(path: path)
        self.doc.load(passwd)
        self.docPath = path
    }
    
    /** @brief	Extract text from a PDF document to a TXT file. */
    func extractTextOfDoc(_ doc: FSPDFDoc, toPath path: String) -> Bool {
        if path == "" {
            return false
        }
        let nPage = doc.getPageCount()
        if nPage < 1 {
            return false
        }
        let fileMgr = FileManager()
        fileMgr.createFile(atPath: path, contents: nil, attributes: nil)
   
        let outFile = FileHandle.init(forWritingAtPath: path)
        if outFile == nil {
            return false
        }
        for i in 0..<nPage {
            let page = doc.getPage(i)!
            page.startParse(UInt32(FSPDFPageParseFlags.pageNormal.rawValue), pause: nil, is_reparse: true)
            let textSelect = FSTextPage(page: page, flags:1)!
//            if textSelect == nil {
//                return false
//            }
            let fullText = textSelect.getChars(0, count: -1)!
            if fullText.isEmpty {
                return false
            }
            outFile!.write((fullText.data(using: String.Encoding.utf8))!)
//            doc.closePage(i)
        }
        outFile!.closeFile()
        return true
    }
    
    /** @brief	Edit bookmark appearances and titles, and save these changes to a PDF file. */
    func modifyBookmarkOfDoc(_ doc: FSPDFDoc, saveAsNewDocAtPath path: String) -> Bool {
        if path == "" {
            return false
        }
        let rootBMark = doc.getRootBookmark()
        guard let child = rootBMark?.getFirstChild() else {
            return false
        }
        if !(self.setPropertiesOfBookmarks(child)) {
            return false
        }
        doc.save(as: path, save_flags: 0)
        return true
    }
    
    class func parsePage(_ page: FSPDFPage, reparse: Bool) -> Bool {
        let progressive = page.startParse(0, pause: nil, is_reparse: reparse)
        if progressive == nil {
            return true
        }
        
        while true {
            let state = progressive!.resume()
            switch state {
            case .finished:
                return true
            case .toBeContinued:
                continue
            case .error:
                return false
            default:
                fatalError()
            }
        }
    }
    
    /** @brief	Add annotations, and save these changes to a PDF file. */
    func addAnnotToPage(_ page: FSPDFPage, saveAsNewDocAtPath path: String) -> Bool {
        if path.isEmpty {
            return false
        }
        if !page.isParsed() {
            if !PlatformAPIDemo.parsePage(page, reparse: true) {
                return false
            }
        }
        
        let document = page.getDocument()
        
        func getLocalDateTime() -> FSDateTime {
            let gregorian = NSCalendar(calendarIdentifier: NSCalendar.Identifier.gregorian)
            let unitFlags = NSCalendar.Unit.year.rawValue | NSCalendar.Unit.month.rawValue | NSCalendar.Unit.day.rawValue | NSCalendar.Unit.hour.rawValue | NSCalendar.Unit.minute.rawValue | NSCalendar.Unit.second.rawValue
            let comps = gregorian?.components(NSCalendar.Unit(rawValue: unitFlags), from: Date())
            let time = FSDateTime()!
            time.year = UInt16((comps?.year)!)
            time.month = UInt16((comps?.month)!)
            time.day = UInt16((comps?.day)!)
            time.hour = UInt16((comps?.hour)!)
            time.minute = UInt16((comps?.minute)!)
            time.second = UInt16((comps?.second)!)
            
            let localTimeZone = NSTimeZone.local
            let gmtOffset = localTimeZone.secondsFromGMT(for: Date())
            time.utc_hour_offset = Int16(gmtOffset/3600)
            time.utc_minute_offset = UInt16((gmtOffset % 3600) / 60)
            return time;
        };
        
        func getUUID() -> String{
            return UUID().uuidString;
        };

        var rectF = FSRectF(left1: 0.0, bottom1: 650, right1: 100.0, top1: 750.0)
        // Add line annotation
        // No special intent, as a common line.
        
        var line = FSLine(annot: page.add(FSAnnotType.annotLine, rect: rectF!))
        line?.startPoint = FSPointF(x: 20, y: 650)
        line?.endPoint = FSPointF(x: 100, y: 740)
        // Intent, as line arrow.
        line?.content = "A line arrow annotation"
        line?.intent = "LineArrow"
        line?.subject = "Arrow"
        line?.title = "Foxit SDK"
        line?.creationDateTime = getLocalDateTime();
        line?.modifiedDateTime = getLocalDateTime();
        line?.uniqueID = getUUID()
        // Appearance should be reset.
        line?.resetAppearanceStream()
        
        rectF?.top = 760
        line = FSLine(annot: page.add(FSAnnotType.annotLine, rect: rectF!))
        // Set foxit RGB color
        line?.borderColor = 0x00FF00
        line?.startPoint = FSPointF(x: 10, y: 650)
        line?.endPoint = FSPointF(x: 100, y: 750)
        line?.content = "A common line."
        line?.lineStartStyle = FSMarkupEndingStyle.square
        line?.lineEndStyle = FSMarkupEndingStyle.openArrow
        // Show text in line
        line?.enableCaption(true)
        line?.captionOffset = FSPointF(x: 0, y: 5)

        line?.subject = "Line"
        line?.title = "Foxit SDK"
        line?.creationDateTime = getLocalDateTime()
        line?.modifiedDateTime = getLocalDateTime()
        line?.uniqueID = getUUID()
        // Appearance should be reset.
        line?.resetAppearanceStream()
        NSLog("Add a line annotations.")
        
        // Add circle annotation
        rectF = FSRectF(left1: 100, bottom1: 650, right1: 200.0, top1: 750.0)
        let circle = FSCircle(annot: page.add(FSAnnotType.annotCircle, rect: rectF))
        circle?.innerRect = FSRectF(left1: 120, bottom1: 660, right1: 160, top1: 740)
        circle?.subject = "Circle"
        circle?.title = "Foxit SDK"
        circle?.creationDateTime = getLocalDateTime()
        circle?.modifiedDateTime = getLocalDateTime()
        circle?.uniqueID = getUUID()
        // Appearance should be reset.
        circle?.resetAppearanceStream()
        NSLog("Add a circle annotation.")
        
        // Add square annotation
        rectF = FSRectF(left1: 200, bottom1: 650, right1: 300.0, top1: 750.0)
        let square = FSSquare(annot: page.add(FSAnnotType.annotSquare, rect: rectF))
        square?.fillColor = 0x00FF00
        square?.innerRect = FSRectF(left1: 220, bottom1: 660, right1: 260, top1: 740)
        square?.subject = "Square"
        square?.title = "Foxit SDK"
        square?.creationDateTime = getLocalDateTime()
        square?.modifiedDateTime = getLocalDateTime()
        square?.uniqueID = getUUID()
        // Appearance should be reset.
        square?.resetAppearanceStream()
        NSLog("Add a square annotation.")
        
        // Add polygon annotation, as cloud.
        rectF = FSRectF(left1: 300, bottom1: 650, right1: 500.0, top1: 750.0)
        var polygon = FSPolygon(annot: page.add(FSAnnotType.annotPolygon, rect: rectF))
        polygon?.intent = "PolygonCloud"
        polygon?.fillColor = 0x0000FF
        var vertexe_array = FSPointFArray()
        vertexe_array?.add(FSPointF(x: 335, y: 665))
        vertexe_array?.add(FSPointF(x: 365, y: 665))
        vertexe_array?.add(FSPointF(x: 385, y: 705))
        vertexe_array?.add(FSPointF(x: 365, y: 740))
        vertexe_array?.add(FSPointF(x: 335, y: 740))
        vertexe_array?.add(FSPointF(x: 315, y: 705))
        polygon?.vertexes = vertexe_array
        polygon?.subject = "Cloud"
        polygon?.title = "Foxit SDK"
        polygon?.creationDateTime = getLocalDateTime()
        polygon?.modifiedDateTime = getLocalDateTime()
        polygon?.uniqueID = getUUID()
        // Appearance should be reset.
        polygon?.resetAppearanceStream()
        NSLog("Add a polygon annotation.");
        
        // Add polygon annotation, with dashed border.
        let borderinfo = FSBorderInfo()
        borderinfo?.cloud_intensity = 2.0;
        borderinfo?.width = 2.0;
        borderinfo?.style = FSBorderInfoStyle.dashed;
        borderinfo?.dash_phase = 3.0;
        let dashes = FSFloatArray()
        dashes?.setAt(0, newElement: 2)
        dashes?.setAt(1, newElement: 2)
        borderinfo?.dashes = dashes;
        
        rectF = FSRectF(left1: 400, bottom1: 650, right1: 500.0, top1: 750.0)
        polygon = FSPolygon(annot: page.add(FSAnnotType.annotPolygon, rect: rectF))
        polygon?.fillColor = 0x0000FF
        polygon?.borderInfo = borderinfo
        
        vertexe_array?.removeAll()
        vertexe_array?.add(FSPointF(x: 435, y: 665))
        vertexe_array?.add(FSPointF(x: 465, y: 665))
        vertexe_array?.add(FSPointF(x: 485, y: 705))
        vertexe_array?.add(FSPointF(x: 465, y: 740))
        vertexe_array?.add(FSPointF(x: 435, y: 740))
        vertexe_array?.add(FSPointF(x: 415, y: 705))
        polygon?.vertexes = vertexe_array;
        
        polygon?.subject = "Polygon"
        polygon?.title = "Foxit SDK"
        polygon?.creationDateTime = getLocalDateTime()
        polygon?.modifiedDateTime = getLocalDateTime()
        polygon?.uniqueID = getUUID()
        // Appearance should be reset.
        polygon?.resetAppearanceStream()
        
        NSLog("Add a polygon cloud annotation.")
        
        
        
        // Add polyline annotation
        rectF = FSRectF(left1: 500, bottom1: 650, right1: 600.0, top1: 750.0)
        let polyline = FSPolyLine(annot: page.add(FSAnnotType.annotPolyLine, rect: rectF))
      
        vertexe_array?.removeAll()
        vertexe_array?.add(FSPointF(x: 515, y: 705))
        vertexe_array?.add(FSPointF(x: 535, y: 740))
        vertexe_array?.add(FSPointF(x: 565, y: 740))
        vertexe_array?.add(FSPointF(x: 585, y: 705))
        vertexe_array?.add(FSPointF(x: 565, y: 665))
        vertexe_array?.add(FSPointF(x: 535, y: 665))
        polyline?.vertexes = vertexe_array
      
        polyline?.subject = "PolyLine"
        polyline?.title = "Foxit SDK"
        polyline?.creationDateTime = getLocalDateTime()
        polyline?.modifiedDateTime = getLocalDateTime()
        polyline?.uniqueID = getUUID()
        // Appearance should be reset.
        polyline?.resetAppearanceStream()
      
        NSLog("Add a polyline annotation.")
        
        
        // Add freetext annotation, as type writer.
        rectF = FSRectF(left1: 10, bottom1: 550, right1: 220, top1: 600)
        var freetext = FSFreeText(annot: page.add(FSAnnotType.annotFreeText, rect: rectF))
        
        // Set default appearance
        var default_ap = FSDefaultAppearance()
        default_ap?.flags = UInt32(FSDefaultAppearanceDefAPFlags.font.rawValue | FSDefaultAppearanceDefAPFlags.fontSize.rawValue | FSDefaultAppearanceDefAPFlags.textColor.rawValue)
        default_ap?.font = FSFont(font_id: FSFontStandardID.stdIDHelvetica)
        default_ap?.text_size = 12.0
        default_ap?.text_color = 0x000000
        // Set default appearance for form.
        freetext?.setDefaultAppearance(default_ap)
        freetext?.alignment = FSAlignment.left
        freetext?.intent = "FreeTextTypewriter"
        freetext?.content = "A typewriter annotation"
        freetext?.subject = "FreeTextTypewriter"
        freetext?.title = "Foxit SDK"
        freetext?.creationDateTime = getLocalDateTime()
        freetext?.modifiedDateTime = getLocalDateTime()
        freetext?.uniqueID = getUUID()
        // Appearance should be reset.
        freetext?.resetAppearanceStream()
        
        NSLog("Add a typewriter freetext annotation.")
        
        // Add freetext annotation, as call-out.
        rectF = FSRectF(left1: 300, bottom1: 550, right1: 400, top1: 600)
        freetext = FSFreeText(annot: page.add(FSAnnotType.annotFreeText, rect: rectF))
        
        // Set default appearance
        default_ap = FSDefaultAppearance()
        default_ap?.flags = UInt32(FSDefaultAppearanceDefAPFlags.font.rawValue | FSDefaultAppearanceDefAPFlags.fontSize.rawValue | FSDefaultAppearanceDefAPFlags.textColor.rawValue)
        default_ap?.font = FSFont(font_id: FSFontStandardID.stdIDHelveticaB)
        default_ap?.text_size = 12.0
        default_ap?.text_color = 0x000000
        // Set default appearance for form.
        freetext?.setDefaultAppearance(default_ap)
        freetext?.alignment = FSAlignment.center
        
        //
        freetext?.intent = "FreeTextCallout"
        let callout_points = FSPointFArray()
        callout_points?.add(FSPointF(x: 250, y: 540))
        callout_points?.add(FSPointF(x: 280, y: 570))
        callout_points?.add(FSPointF(x: 300, y: 570))
        freetext?.calloutLinePoints = callout_points;
        
        freetext?.calloutLineEndingStyle = FSMarkupEndingStyle.openArrow;
        freetext?.content = "A callout annotation."
        freetext?.subject = "FreeTextCallout"
        freetext?.title = "Foxit SDK"
        freetext?.creationDateTime = getLocalDateTime()
        freetext?.modifiedDateTime = getLocalDateTime()
        freetext?.uniqueID = getUUID()
        // Appearance should be reset.
        freetext?.resetAppearanceStream()
        
        NSLog("Add a callout freetext annotation.")
        
        // Add freetext annotation, as text box.
        rectF = FSRectF(left1: 450, bottom1: 550, right1: 550, top1: 600)
        freetext = FSFreeText(annot: page.add(FSAnnotType.annotFreeText, rect: rectF))
        
        // Set default appearance
        // Set default appearance
        default_ap = FSDefaultAppearance()
        default_ap?.flags = UInt32(FSDefaultAppearanceDefAPFlags.font.rawValue | FSDefaultAppearanceDefAPFlags.fontSize.rawValue | FSDefaultAppearanceDefAPFlags.textColor.rawValue)
        default_ap?.font = FSFont(font_id: FSFontStandardID.stdIDHelveticaI)
        default_ap?.text_size = 12.0
        default_ap?.text_color = 0x000000
        // Set default appearance for form.
        freetext?.setDefaultAppearance(default_ap)
        freetext?.alignment = FSAlignment.center
        
        freetext?.content = "A text box annotation."
        freetext?.subject = "Textbox"
        freetext?.title = "Foxit SDK"
        freetext?.creationDateTime = getLocalDateTime()
        freetext?.modifiedDateTime = getLocalDateTime()
        freetext?.uniqueID = getUUID()
        // Appearance should be reset.
        freetext?.resetAppearanceStream()
        NSLog("Add a text box freetext annotation.")
        
        
        // Add highlight annotation.
        rectF = FSRectF(left1: 10, bottom1: 450, right1: 100, top1: 550)
        let highlight = FSHighlight(annot: page.add(FSAnnotType.annotHighlight, rect: rectF))
        highlight?.content = "Highlight"
    
        var quad_points = FSQuadPoints()
        quad_points?.first = FSPointF(x: 10, y: 500)
        quad_points?.second = FSPointF(x: 90, y: 500)
        quad_points?.third = FSPointF(x: 10, y: 480)
        quad_points?.fourth = FSPointF(x: 90, y: 480)
        
        var quad_points_array = FSQuadPointsArray()
        quad_points_array?.add(quad_points)
        
        highlight?.quadPoints = quad_points_array
        highlight?.subject = "Highlight"
        highlight?.title = "Foxit SDK"
        highlight?.creationDateTime = getLocalDateTime()
        highlight?.modifiedDateTime = getLocalDateTime()
        highlight?.uniqueID = getUUID()
        // Appearance should be reset.
        highlight?.resetAppearanceStream()
        NSLog("Add a highlight annotation.")
        
        // Add underline annotation.
        rectF = FSRectF(left1: 100, bottom1: 450, right1: 200, top1: 550)
        let underline = FSUnderline(annot: page.add(FSAnnotType.annotUnderline, rect: rectF))
        
        quad_points = FSQuadPoints()
        quad_points?.first = FSPointF(x: 110, y: 500)
        quad_points?.second = FSPointF(x: 190, y: 500)
        quad_points?.third = FSPointF(x: 110, y: 480)
        quad_points?.fourth = FSPointF(x: 190, y: 480)
        quad_points_array?.removeAll()
        quad_points_array?.add(quad_points)
        
        underline?.quadPoints = quad_points_array
        underline?.subject = "Underline"
        underline?.title = "Foxit SDK"
        underline?.creationDateTime = getLocalDateTime()
        underline?.modifiedDateTime = getLocalDateTime()
        underline?.uniqueID = getUUID()
        // Appearance should be reset.
        underline?.resetAppearanceStream()

        NSLog("Add a underline annotation.")
        
        
        // Add squiggly annotation.
        rectF = FSRectF(left1: 200, bottom1: 450, right1: 300, top1: 550)
        let squiggly = FSSquiggly(annot: page.add(FSAnnotType.annotSquiggly, rect: rectF))
        squiggly?.intent = "Squiggly"

        quad_points = FSQuadPoints()
        quad_points?.first = FSPointF(x: 210, y: 500)
        quad_points?.second = FSPointF(x: 290, y: 500)
        quad_points?.third = FSPointF(x: 210, y: 480)
        quad_points?.fourth = FSPointF(x: 290, y: 480)
        quad_points_array?.removeAll()
        quad_points_array?.add(quad_points)
        
        squiggly?.quadPoints = quad_points_array
        squiggly?.subject = "Squiggly"
        squiggly?.title = "Foxit SDK"
        squiggly?.creationDateTime = getLocalDateTime()
        squiggly?.modifiedDateTime = getLocalDateTime()
        squiggly?.uniqueID = getUUID()
        // Appearance should be reset.
        squiggly?.resetAppearanceStream()

        NSLog("Add a squiggly annotation.")
        
        // Add strikeout annotation.
        rectF = FSRectF(left1: 300, bottom1: 450, right1: 400, top1: 550)
        let strikeout = FSStrikeOut(annot: page.add(FSAnnotType.annotStrikeOut, rect: rectF))

        quad_points = FSQuadPoints()
        quad_points?.first = FSPointF(x: 310, y: 500)
        quad_points?.second = FSPointF(x: 390, y: 500)
        quad_points?.third = FSPointF(x: 310, y: 480)
        quad_points?.fourth = FSPointF(x: 390, y: 480)
        quad_points_array?.removeAll()
        quad_points_array?.add(quad_points)
        
        
        strikeout?.quadPoints = quad_points_array
        strikeout?.subject = "StrikeOut"
        strikeout?.title = "Foxit SDK"
        strikeout?.creationDateTime = getLocalDateTime()
        strikeout?.modifiedDateTime = getLocalDateTime()
        strikeout?.uniqueID = getUUID()
        // Appearance should be reset.
        strikeout?.resetAppearanceStream()
        
        NSLog("Add a strikeout annotation.")
        
        // Add caret annotation.
        rectF = FSRectF(left1: 400, bottom1: 450, right1: 420, top1: 470)
        let caret = FSCaret(annot: page.add(FSAnnotType.annotCaret, rect: rectF))
        caret?.innerRect = FSRectF(left1: 410, bottom1: 450, right1: 430, top1: 470)
        caret?.content = "Caret annotation"
        caret?.subject = "Caret"
        caret?.title = "Foxit SDK"
        caret?.creationDateTime = getLocalDateTime()
        caret?.modifiedDateTime = getLocalDateTime()
        caret?.uniqueID = getUUID()
        // Appearance should be reset.
        caret?.resetAppearanceStream()

        NSLog("Add a caret annotation.")
        
        
        // Add note annotation
        rectF = FSRectF(left1: 10, bottom1: 350, right1: 50, top1: 400)
        let note = FSNote(annot: page.add(FSAnnotType.annotNote, rect: rectF))
        note?.iconName = "Comment"
        note?.content = "Note annotation."
        note?.subject = "Note";
        note?.title = "Foxit SDK"
        note?.creationDateTime = getLocalDateTime()
        note?.modifiedDateTime = getLocalDateTime()
        note?.uniqueID = getUUID()
        
        // Add popup to note annotation
        rectF = FSRectF(left1: 300, bottom1: 450, right1: 500, top1: 550)
        let popup = FSPopup(annot: page.add(FSAnnotType.annotPopup, rect: rectF))
        popup?.borderColor = 0x00FF00;
        popup?.getOpenStatus = false;
        
        popup?.modifiedDateTime = getLocalDateTime()
        note?.popup = popup

        // Add reply annotation to note annotation
        let reply = note?.addReply()
        reply?.content = "reply"
        reply?.modifiedDateTime = getLocalDateTime()
        reply?.title = "Foxit SDK"
        note?.uniqueID = getUUID()
        reply?.resetAppearanceStream()

        // Add state annotation to note annotation
        let state = note?.addStateAnnot("Foxit SDK", model: FSMarkupStateModel.review, state: FSMarkupState.accepted)
        state?.content = "Accepted set by Foxit SDK"
        state?.uniqueID = getUUID()
        // Appearance should be reset.
        state?.resetAppearanceStream()
        
        NSLog("Add a note annotation.")
        
        // Add ink annotation
        rectF = FSRectF(left1: 100, bottom1: 350, right1: 200, top1: 450)
        let ink = FSInk(annot: page.add(FSAnnotType.annotInk, rect: rectF))
        
        let inklist = FSPath()
        let width = 100.0
        let height = 100.0
        let out_width = min(width, height) * 2 / 3.0
        let inner_width = out_width * sin(18.0 / 180.0 * 3.14) / sin(36.0 / 180.0 * 3.14)
        let center = FSPointF(x: 150, y: 400)
        var x: Float = Float(out_width)
        var y: Float = Float(0);
        var point = FSPointF(x: center!.x + x, y: center!.y + y)
        inklist?.move(to: point)
        for i in 0..<5 {
            x = Float(out_width * cos(72.0 * Double(i) / 180.0 * 3.14))
            y = Float(out_width * sin(72.0 * Double(i) / 180.0 * 3.14))
            point?.set(center!.x + x, y: center!.y + y)
            inklist?.line(to: point)
            
            x = Float(inner_width * cos((72.0 * Double(i) + 36) / 180.0 * 3.14));
            y = Float(inner_width * sin((72.0 * Double(i) + 36) / 180.0 * 3.14));
            point?.set(center!.x + x, y: center!.y + y)
            inklist?.line(to: point)
        }
        point?.set(center!.x + Float(out_width), y: center!.y)
        inklist?.line(to: point)
        inklist?.closeFigure()
        ink?.inkList = inklist
        ink?.content = "Ink annotation"
        ink?.subject = "Ink"
        ink?.title = "Foxit SDK"
        ink?.creationDateTime = getLocalDateTime()
        ink?.modifiedDateTime = getLocalDateTime()
        ink?.uniqueID = getUUID()
        // Appearance should be reset.
        ink?.resetAppearanceStream()
        NSLog("Add an ink annotation.")
        
        // Add file attachment annotation
        let pdf_path = Bundle.main.path(forResource: "test_files/AboutFoxit", ofType: "pdf")!
        rectF = FSRectF(left1: 280, bottom1: 350, right1: 300, top1: 380)
        let file_attachment = FSFileAttachment(annot: page.add(FSAnnotType.annotFileAttachment, rect: rectF))
        file_attachment?.iconName = "Graph"
        
        let file_spec = FSFileSpec(document: document)
        file_spec?.fileName = "attachment.pdf"
        file_spec?.description = "The original file"
        file_spec?.embed(pdf_path)
        file_attachment?.setFileSpec(file_spec)
        file_attachment?.subject = "File Attachment"
        file_attachment?.title = "Foxit SDK"
        file_attachment?.creationDateTime = getLocalDateTime()
        file_attachment?.modifiedDateTime = getLocalDateTime()
        file_attachment?.uniqueID = getUUID()
        // Appearance should be reset.
        file_attachment?.resetAppearanceStream()
        NSLog("Add an attachment annotation.")
        
        // Add link annotation
        rectF = FSRectF(left1: 350, bottom1: 350, right1: 380, top1: 400)
        let link = FSLink(annot: page.add(FSAnnotType.annotLink, rect: rectF))
        link?.highlightingMode = FSAnnotHighlightingMode.toggle
        
        // Add action for link annotation
        let action = FSURIAction(action: FSAction.create(document, action_type: FSActionType.URI))
        action?.setTrackPositionFlag(true)
        action?.uRI = "www.foxitsoftware.com"
        link?.action = action
        // Appearance should be reset.
        link?.resetAppearanceStream()
        NSLog("Add a link annotation.")
        
        
        // Set icon provider for annotation to Foxit PDF SDK.
        var input_path : NSString = self.docPath as NSString
        input_path = input_path.deletingLastPathComponent as NSString
        let icon_folder = input_path.appendingPathComponent("Stamps")
        let icon_provider = MyIconProvider(folder: icon_folder, use_dynamic_stamp_: false)
        FSLibrary.setAnnotIconProviderCallback(icon_provider)
        
        // Add common stamp annotation.
        FSLibrary.setActionCallback(nil)
        
        rectF = FSRectF(left1: 110, bottom1: 150, right1: 200, top1: 250)
        let static_stamp = FSStamp(annot: page.add(FSAnnotType.annotStamp, rect: rectF))
        static_stamp?.iconName = "Approved"
        // Appearance should be reset.
        static_stamp?.resetAppearanceStream()

        // Add dynamic stamp annotation.
        FSLibrary.setActionCallback(MyActionEventHandler())
        icon_provider.use_dynamic_stamp_ = true

        rectF = FSRectF(left1: 10, bottom1: 150, right1: 100, top1: 250)
        let dynamic_stamp = FSStamp(annot: page.add(FSAnnotType.annotStamp, rect: rectF))
        dynamic_stamp?.iconName = "Approved"

        // Appearance should be reset.
        dynamic_stamp?.resetAppearanceStream()
        NSLog("Add stamp annotations.");
        
        document?.save(as: path, save_flags: 0)
        
        ExportAnnotData.exportAnnotDataToJson(doc: doc, toPath: ((path as NSString).deletingLastPathComponent as NSString).appendingPathComponent("annotations.json"))
        return true
    }
    
    /** @brief	Export document info to a TXT file. */
    func exportDocInfoOfDoc(_ doc: FSPDFDoc, toPath path: String) -> Bool {
        if path == "" {
            return false
        }
        let nPage = doc.getPageCount()
        if nPage < 1 {
            return false
        }
        var string = "PageCount: \(nPage) pages\n"
        //get from metadata
        let metadata = FSMetadata.init(document: doc)!
        let arrayOfItem = ["Title", "Author", "Subject", "Keywords"]
        for item: String in arrayOfItem {
            if let val = metadata.getValues(item) {
                for i in 0..<val.count {
                    string += "\(item): \(val[i])\n"
                }
            }
        }
        let fileMgr = FileManager()
        fileMgr.createFile(atPath: path, contents: nil, attributes: nil)
    
        let outFile = FileHandle(forWritingAtPath: path)
        if !(outFile != nil) {
            return false
        }
        outFile!.write(string.data(using: String.Encoding.utf8)!)
        outFile?.closeFile()
        return true
    }
    
    /** @brief	Render a specified page to Bitmap, and save it to a PNG image. */
    func renderPage(_ page: FSPDFPage, atPath path: String) -> Bool {
        if path == "" {
            return false
        }
        if !page.isParsed() {
            if !PlatformAPIDemo.parsePage(page, reparse: true) {
                return false
            }
        }
        //create bitmap
        let pageWidth: Int = Int(page.getWidth())
        let pageHeight: Int = Int(page.getHeight())
        
        let bmp = FSBitmap(width: Int32(pageWidth), height: Int32(pageHeight), format: .dibArgb, buffer: nil, pitch: 0)!
        
        bmp.fillRect(0xffffffff, rect: nil)
        //init renderer
        let render = FSRenderer.init(bitmap: bmp, is_rgb_order: true) 
        if render == nil {
            return false
        }
        
        //set different kinds of properties
        render!.setRenderContentFlags(1)
        render!.setColorMode(FSRendererColorMode.normal)
        render!.setMappingModeColors(0xffffffff, foreground_color: 0x0000ffff)
        render!.setForceHalftone(true)
        let mat = page.getDisplayMatrix(0, top: 0, width: Int32(pageWidth), height: Int32(pageHeight), rotate: .rotation0)
        let progressive = render!.startRender(page, matrix: mat, pause: nil)
        if progressive != nil {
            while true {
                let state = progressive!.resume()
                if state == .error {
                    return false
                } else if state != .toBeContinued {
                    break;
                }
            }
        }
        if !self.save2PNGWithBitmap(bmp, transform: nil, Path: path as String) {
            return false
        }
        return true
    }

    func setPropertiesOfBookmarks(_ bookmark: FSBookmark?) -> Bool {
        if bookmark == nil {
            return false
        }
        var bk: FSBookmark!
        bk = bookmark!
        while !bk.isEmpty() {
            let title = bk.title!
            bk.title = "\(String(describing: title))_renamed" as String
            bk.color = 0x0000ffff
            bk.style = UInt32(FSBookmarkStyle.bold.rawValue)
            
            let child = bk.getFirstChild()!
            if !child.isEmpty() {
                self.setPropertiesOfBookmarks(child)
            }
            bk = bk!.getNextSibling()
        }
        return true
    }

    func save2PNGWithBitmap(_ fsBitmap: FSBitmap, transform: CGAffineTransform?, Path path: String) -> Bool {
        if path == "" {
            return false
        }
        let pageWidth: Int = Int(fsBitmap.getWidth())
        let pageHeight: Int = Int(fsBitmap.getHeight())
        if pageWidth <= 0 || pageHeight <= 0 {
            return false
        }
        
        let provider = CGDataProvider.init(data: fsBitmap.getBuffer() as CFData)

        let bitmapInfo = CGBitmapInfo(rawValue: CGImageAlphaInfo.last.rawValue | CGBitmapInfo.byteOrder32Big.rawValue)
        let image = CGImage(width: pageWidth, height: pageHeight, bitsPerComponent: 8, bitsPerPixel: 32, bytesPerRow: Int(fsBitmap.getPitch()), space: CGColorSpaceCreateDeviceRGB(), bitmapInfo: bitmapInfo, provider: provider!, decode: nil, shouldInterpolate: true, intent: .defaultIntent);

        if (image != nil){
            let uiImage = UIImage(cgImage: image!)
            let fileURL = URL(fileURLWithPath: path as String)
            do {
                try UIImagePNGRepresentation(uiImage)!.write(to: fileURL)
            } catch {
                print(error)
                return false
            }
        }
        return true
    }
    
    func addSignature(to page: FSPDFPage, path: String) -> Bool
    {
        if path == ""{
            return false
        }
        let rect = FSRectF.init(left1: 100, bottom1: 0, right1: 200, top1: 100)
        var signature = page.addSignature(rect)!
        
        let now = Date()
        let timeZone = NSTimeZone.system
        let secondsFromGMTForDate = timeZone.secondsFromGMT(for: now)
        let calendar = NSCalendar.current
        
        let dateComponent = calendar.dateComponents([.year, .month, .day, .hour, .minute, .second], from: now)
        let year = dateComponent.year!
        let month = dateComponent.month!
        let day = dateComponent.day!
        let hour = dateComponent.hour!
        let minute = dateComponent.minute!
        let second = dateComponent.second!
        let UTHourOffset = secondsFromGMTForDate / 3600
        let UTMinuteOffset = secondsFromGMTForDate % 3600
        
        let dateTime = FSDateTime()!
        dateTime.set(UInt16(year), month: UInt16(month), day: UInt16(day), hour: UInt16(hour), minute: UInt16(minute), second: UInt16(second), milliseconds: 0, utc_hour_offset: Int16(UTHourOffset), utc_minute_offset: UInt16(UTMinuteOffset))
        let bitmap = FSBitmap(width: 100, height:100, format:.dibRgb32, buffer:nil, pitch:0)
        bitmap?.fillRect(0xff0000ff, rect:nil)
        //"adbe.pkcs7.sha1"
        signature.filter = "Adobe.PPKLite"
        signature.subFilter = "adbe.pkcs7.detached"
        signature.setKeyValue(.DN, value: "dn")
        signature.setKeyValue(.location, value: "location")
        signature.setKeyValue(.reason, value: "reason")
        signature.setKeyValue(.contactInfo, value: "contactInfo")
        signature.setKeyValue(.signer, value: "signer")
        signature.setKeyValue(.text, value: "text")
        signature.signTime = dateTime
//        signature.appearanceFlags = ([.e_signatureAPFlagFoxitFlag, .e_signatureAPFlagSigningTime, .e_signatureAPFlagReason, .e_signatureAPFlagLocation, /*.e_signatureAPFlagBitmap, */.e_signatureAPFlagReason, .e_signatureAPFlagSigner, .e_signatureAPFlagText, .e_signatureAPFlagDN])
        signature.appearanceFlags = 0x01FF
        
        let certPath: String = Bundle.main.path(forResource: "test_files/foxit_all", ofType: "pfx")!
        let password = "123456"
        // set customized appearance.
        let progressive = signature.startSign(certPath, cert_password: password, digest_algorithm: .SHA256, save_path: path, client_data: nil, pause: nil)
        if progressive != nil {
            while .toBeContinued == progressive!.resume() {
                continue
            }
        }
        // load the signed doc, and verify it.
        let doc = FSPDFDoc(path: path)!
        if doc.load(nil) != .errSuccess {
            return false
        }
        signature = doc.getSignature(0)!
        let verifyProgressive = signature.startVerify(nil, pause: nil)
        if verifyProgressive != nil {
            while .toBeContinued == verifyProgressive!.resume() {
                continue
            }
        }
        let state = signature.getState()
        return state&UInt32(FSSignatureStates.verifyValid.rawValue) != 0 || state&UInt32(FSSignatureStates.verifyNoChange.rawValue) != 0
    }
    
    func image2pdf(input_file: String, saveAsNewDocAtPath path: String) -> Bool {
        let imageNames = ["watermark.bmp", "image_samples.jpg", "TIF2Pages.tif", "image005.gif"]
        for imageName in imageNames {
            let imagePath = input_file.appendingFormat("/%@", imageName)
            let image = FSImage(path: imagePath)
            let count = image?.getFrameCount()
            let doc = FSPDFDoc()
            for i: Int32 in 0..<count! {
                let page = doc?.insertPage(i, width: Float((image?.getWidth())!), height: Float((image?.getHeight())!))
                page?.startParse(0, pause: nil, is_reparse: false)
                page?.add(image, frame_index: Int32(i), position: FSPointF(x: 0, y: 0), width: Float((image?.getWidth())!), height: Float((image?.getWidth())!), auto_generate_content: true)
            }
            
            let imageNameStr : NSString = imageName as NSString;
            let savePath = path.appendingFormat("/%@_%@.pdf", imageNameStr.deletingPathExtension, imageNameStr.pathExtension);
            doc?.save(as: savePath, save_flags: UInt32(FSPDFDocSaveFlags.normal.rawValue))
        }
        
        return true;
    }
    
    func addTextWatermark(doc: FSPDFDoc, page: FSPDFPage) {
        let settings = FSWatermarkSettings();
        settings?.flags = UInt32(FSWatermarkSettingsFlags.asPageContents.rawValue | FSWatermarkSettingsFlags.onTop.rawValue);
        settings?.offset_x = 0;
        settings?.offset_y = 0;
        settings?.opacity = 90;
        settings?.position = FSPosition.posTopRight;
        settings?.rotation = -45.0;
        settings?.scale_x = 1.0;
        settings?.scale_y = 1.0;
        
        let text_properties = FSWatermarkTextProperties();
        text_properties?.alignment = FSAlignment.center;
        text_properties?.color = 0xF68C21;
        text_properties?.font_style = FSWatermarkTextPropertiesFontStyle.normal;
        text_properties?.line_space = 1;
        text_properties?.font_size = 12.0;
        let font = FSFont(font_id: FSFontStandardID.stdIDTimesB);
        text_properties?.font = font;
        
        let watermark = FSWatermark(document: doc, text: "Foxit PDF SDK\nwww.foxitsoftware.com", properties: text_properties, settings: settings);
        watermark?.insert(to: page);
    }
    
    func addBitmapWatermark(doc: FSPDFDoc, page: FSPDFPage, bitmap_file: String) {
        let settings = FSWatermarkSettings();
        settings?.flags = UInt32(FSWatermarkSettingsFlags.asPageContents.rawValue | FSWatermarkSettingsFlags.onTop.rawValue);
        settings?.offset_x = 0;
        settings?.offset_y = 0;
        settings?.opacity = 60;
        settings?.position = FSPosition.posCenterLeft;
        settings?.rotation = 90.0;
        
        let image = FSImage(path: bitmap_file);
        let bitmap = image?.getFrameBitmap(0);
        let bmpWidth : Float = Float((bitmap?.getWidth())!);
        if (bmpWidth <= 0) {
            return;
        }

        settings?.scale_x = page.getHeight() * 1.0 / bmpWidth;
        settings?.scale_y = settings!.scale_x;
        
        let watermark = FSWatermark(document: doc, bitmap: bitmap, settings: settings);
        watermark?.insert(to: page);
    }

    func addImageWatermark(doc: FSPDFDoc, page: FSPDFPage, image_file: String) {
        let settings = FSWatermarkSettings();
        settings?.flags = UInt32(FSWatermarkSettingsFlags.asPageContents.rawValue | FSWatermarkSettingsFlags.onTop.rawValue);
        settings?.offset_x = 0;
        settings?.offset_y = 0;
        settings?.opacity = 20;
        settings?.position = FSPosition.posCenter;
        settings?.rotation = 0;

        let image = FSImage(path: image_file);
        let bitmap = image?.getFrameBitmap(0);
        let bmpWidth : Float = Float((bitmap?.getWidth())!);
        if (bmpWidth <= 0) {
            return;
        }

        settings?.scale_x = page.getHeight() * 0.618 / bmpWidth;
        settings?.scale_y = settings!.scale_x;
        
        let watermark = FSWatermark(document: doc, image: image,frame_index: 0, settings: settings);
        watermark?.insert(to: page);
    }

    func addSingleWatermark(doc: FSPDFDoc, page: FSPDFPage) {
        let settings = FSWatermarkSettings();
        settings?.flags = UInt32(FSWatermarkSettingsFlags.asPageContents.rawValue | FSWatermarkSettingsFlags.onTop.rawValue);
        settings?.offset_x = 0;
        settings?.offset_y = 0;
        settings?.opacity = 90;
        settings?.position = FSPosition.posBottomRight;
        settings?.rotation = 0;
        settings?.scale_x = 0.1;
        settings?.scale_y = 0.1;
        
        let watermark = FSWatermark(document: doc, page: page, settings: settings);
        watermark?.insert(to: page);
    }

    func addWatermark(input_file: String, saveAsNewDocAtPath path: String) -> Bool {
        let page_count = self.doc.getPageCount();
        for i in 0..<page_count {
            let page = self.doc.getPage(i);
            page?.startParse(0, pause: nil, is_reparse: false);
            addTextWatermark(doc: self.doc, page: page!);
            addBitmapWatermark(doc: self.doc, page: page!, bitmap_file: Bundle.main.path(forResource: "test_files/watermark", ofType: "bmp")!);
            addImageWatermark(doc: self.doc, page:page!, image_file: Bundle.main.path(forResource: "test_files/rdk", ofType: "png")!);
            addSingleWatermark(doc: self.doc, page: page!);
            
        }

        self.doc.save(as: path, save_flags: UInt32(FSPDFDocSaveFlags.normal.rawValue));
        return true;

    }
    
    func searchWithSaveResultsPath(path: String) -> Bool {
        var textOutput = ""
        // sample 1: search for all pages of doc.
        let search = FSTextSearch(document: self.doc, cancel: nil, flags: Int32(FSTextPageTextParseFlags.parseTextNormal.rawValue))
        let page_count = self.doc.getPageCount()
        let start_index = 0, end_index = page_count - 1
        search?.setStartPage(Int32(start_index))
        search?.setEndPage(end_index)
        
        let pattern = "Foxit";
        search?.setPattern(pattern)

        var flags = FSTextSearchSearchFlags.normal
        // If want to specify flags, you can do as followings:
//        flags |= FSTextSearchSearchFlags.matchCase
//        flags |= FSTextSearchSearchFlags.matchWholeWord
//        flags |= FSTextSearchSearchFlags.consecutive
        search?.setSearchFlags(UInt32(flags.rawValue))
        textOutput = textOutput.appendingFormat("Begin search %@ at %@.\n", pattern, self.docPath)
        textOutput = textOutput.appendingFormat("Start index:\t%d\r\n", start_index)
        textOutput = textOutput.appendingFormat("End index:\t%d\r\n", end_index)
        textOutput = textOutput.appendingFormat("Match key:\t%@\r\n", pattern)
        
        let match_case = (flags.rawValue & FSTextSearchSearchFlags.matchCase.rawValue) == FSTextSearchSearchFlags.matchCase.rawValue ? "Yes" : "No"
        textOutput = textOutput.appendingFormat("Match case:\t%@\r\n", match_case)
        let match_whole_word = (flags.rawValue & FSTextSearchSearchFlags.matchWholeWord.rawValue) == FSTextSearchSearchFlags.matchWholeWord.rawValue ? "Yes" : "No"
        textOutput = textOutput.appendingFormat("Match whole word:\t%@\r\n", match_whole_word)
        let match_consecutive = (flags.rawValue & FSTextSearchSearchFlags.consecutive.rawValue) == FSTextSearchSearchFlags.consecutive.rawValue ? "Yes" : "No"
        textOutput = textOutput.appendingFormat("Consecutive:\t%@\r\n", match_consecutive)
        var match_count = 0;
        while (search?.findNext() ?? false) {
            let page_index = (search?.getMatchPageIndex())!
            let matched_index = match_count
            textOutput = textOutput.appendingFormat("Index of matched pattern:\t%d\r\n", matched_index)
            textOutput = textOutput.appendingFormat("\tpage:\t%d\r\n", page_index)
            textOutput = textOutput.appendingFormat("\tmatch char start index:\t%d\r\n", (search?.getMatchStartCharIndex())!)
            textOutput = textOutput.appendingFormat("\tmatch char end index:\t%d\r\n", (search?.getMatchEndCharIndex())!)
            textOutput = textOutput.appendingFormat("\tmatch sentence start index:\t%d\r\n", (search?.getMatchSentenceStartIndex())!)
            textOutput = textOutput.appendingFormat("\tmatch sentence:\t%@\r\n", (search?.getMatchSentence())!)
            
            let rect_array = search?.getMatchRects()
            let rect_count = (rect_array?.getSize())!
            textOutput = textOutput.appendingFormat("\tmatch rectangles count:\t%d\r\n", rect_count)
            for i in Int32(0)..<rect_count {
                let rect = (rect_array?.getAt(i))!
                textOutput = textOutput.appendingFormat("\trectangle(in PDF space) :%d\t[left = %.4f, bottom = %.4f, right = %.4f, top = %.4f]\r\n", i,
                                           rect.left, rect.bottom, rect.right, rect.top)
            }
            match_count += 1
        }
        
        NSLog("Marched %d counts.",match_count)
        // end of sample 1.
        NSLog("Search demo finished.");

        do {
           try textOutput.write(toFile: path, atomically: true, encoding: String.Encoding.utf8)
        } catch {
            return false
        }
        return true
    }
    
    func addGraphicsObjects(inputPath: String, saveAsNewDocAtPath path: String) -> Bool {
        func addTextObjects(page: FSPDFPage) {
            let position = page.getLastGraphicsObjectPosition(FSGraphicsObjectType.text)
            var text_object = FSTextObject.create()
            text_object?.fillColor = 0xFFFF7F00
            
            let state = FSTextState()
            state?.font_size = 80.0
            var font = FSFont(font_id: .stdIDCourier)
            state?.font = font
            state?.textmode = FSTextStateMode.fill
            text_object?.setTextState(page, text_state:state, is_italic:false, weight:750)
            
            // Set text.
            text_object?.text = "Foxit Software"
            let last_position = page.insertGraphicsObject(position, graphics_object:text_object)
              
            var rect = (text_object?.getRect())!
            var offset_x = (page.getWidth() - (rect.right - rect.left)) / 2
            var offset_y = page.getHeight() * 0.8 - (rect.top - rect.bottom) / 2
            var mat = FSMatrix2D.init(a1: 1, b1: 0, c1: 0, d1: 1, e1: offset_x, f1: offset_y)
            text_object?.transform(mat, need_transform_clippath:false)

            // Generate content
            page.generateContent()

            // Clone a text object from the old text object.
            text_object = (text_object?.clone())?.getTextObject()
            
            font = FSFont.init(font_id: FSFontStandardID.stdIDTimes)
            state?.font = font
            state?.font_size = 48
            state?.textmode = FSTextStateMode.fillClip

            text_object?.setTextState(page, text_state:state, is_italic:true, weight:750)
            text_object?.text = "www.foxitsoftware.com"

            text_object?.fillColor = 0xFFAAAAAA
            text_object?.strokeColor = 0xFFF68C21
            page.insertGraphicsObject(last_position, graphics_object:text_object)

            rect = (text_object?.getRect())!
            offset_x = (page.getWidth() - (rect.right - rect.left)) / 2;
            offset_y = page.getHeight() * 0.618 - (rect.top - rect.bottom) / 2;
            mat = FSMatrix2D.init(a1: 1, b1: 0, c1: 0, d1: 1, e1: offset_x, f1: offset_y)
            text_object?.transform(mat, need_transform_clippath:false)

            // Generate content again after transformation.
            page.generateContent()

            //Show how to get the characters' information of text object.
            let text_object_charcount = text_object?.getCharCount()
            var text_doc = ""
            text_doc = text_doc.appendingFormat("The new text object has %d characters.\n", text_object_charcount!)

            for i in 0..<text_object_charcount! {
                //The character's position.
                let char_pos = text_object?.getCharPos(i)
                //The character's width.
                let width = text_object?.getCharWidth(by: i)
                //The character's height.
                let height = text_object?.getCharHeight(by: i)

                
                text_doc = text_doc.appendingFormat("The position of the %d characters is (%f, %f).The width and height of the characters is (%f, %f).\n", i, char_pos!.x, char_pos!.y, width!, height!)
            }

            do {
                let nsPath = path as NSString
                try text_doc.write(toFile: String(format: "%@/text_objects_info.txt", nsPath.deletingLastPathComponent), atomically: true, encoding: String.Encoding.utf8)
            } catch {
                
            }
        };
        
        func addImageObjects(page: FSPDFPage, image_file: String) {
            let position = page.getLastGraphicsObjectPosition(FSGraphicsObjectType.image)
            let image = FSImage(path: image_file)
            let image_object = FSImageObject.create(page.getDocument())
            image_object?.setImage(image, frame_index:0)

            let width = image?.getWidth()
            let height = image?.getHeight()

            let page_width = page.getWidth()
            let page_height = page.getHeight()

            // Please notice the matrix value.
            let mat = FSMatrix2D.init(a1: Float(width!), b1: 0, c1: 0, d1: Float(height!), e1: (page_width - Float(width!)) / 2.0, f1: (page_height - Float(height!)) / 2.0)
            image_object?.matrix = mat
            
            page.insertGraphicsObject(position, graphics_object:image_object)
            page.generateContent()
        };
        
        func addPathObjects(page : FSPDFPage, black_pieces: FSShadingObject, white_pieces: FSShadingObject) {
            let position = page.getLastGraphicsObjectPosition(FSGraphicsObjectType.path)
            let path_object = FSPathObject.create()
            let path = FSPath()
            let page_width = page.getWidth()
            let page_height = page.getHeight()

            let width = min(page_width, page_height) / 20.0
            let start_x = (page_width - width * 18.0) / 2.0;
            let start_y = (page_height - width * 18.0) / 2.0;

            // Draw a chess board
            for i in 0..<19 {
                var x1 = start_x;
                var y1 = Float(i) * width + start_y;

                let x2 = start_x + 18 * width;
                path?.move(to: FSPointF(x: x1, y: y1)!)
                path?.line(to: FSPointF(x: x2, y: y1)!)

                x1 = Float(i) * width + start_x;
                y1 = start_y;

                let y2 = 18 * width + start_y;
                path?.move(to: FSPointF(x: x1, y: y1)!)
                path?.line(to: FSPointF(x: x1, y: y2)!)
            }
            let star = [3, 9, 15]
            for i in 0..<3 {
                for j in 0..<3 {
                    let left  = start_x + Float(star[i]) * width - width / 12.0
                    let bottom = start_y + Float(star[j]) * width - width / 12.0
                    let right = start_x + Float(star[i]) * width + width / 12.0
                    let top = start_y + Float(star[j]) * width + width / 12.0
                    let rect = FSRectF(left1: left, bottom1: bottom, right1: right, top1: top)
                    path!.appendRect(rect!)
              }
            }
            path_object?.pathData = path
            
            path_object?.fillColor = 0xFF000000
            path_object?.fillMode = FSFillMode.alternate
            path_object?.getStrokeState = true
            path_object?.strokeColor = 0xFF000000
            
            page.insertGraphicsObject(position, graphics_object:path_object)
            page.generateContent()

            // Draw pieces
            
            func addPieces(page: FSPDFPage, orignal_pieces: FSShadingObject, dst_rect: FSRectF) {
                let position = page.getFirstGraphicsObjectPosition(FSGraphicsObjectType.all)
                let pieces: FSShadingObject = (orignal_pieces.clone()?.getShadingObject())!

                let piece_rect = (pieces.getRect())!

                // Calculates the transformation matrix between dst_rect and  piece_rect.
                let a = (dst_rect.right - dst_rect.left) / (piece_rect.right - piece_rect.left);
                let d = (dst_rect.top - dst_rect.bottom) / (piece_rect.top - piece_rect.bottom);
                let e = dst_rect.left - piece_rect.left * a;
                let f = dst_rect.top - piece_rect.top * d;

               // Transform rect.
                let mat = FSMatrix2D.init(a1: a, b1: 0, c1: 0, d1: d, e1: e, f1: f)
                pieces.transform(mat, need_transform_clippath: false)
                page.insertGraphicsObject(position, graphics_object:pieces)
                
                page.generateContent()
            };
            
            let pieces_vector = [
                [
                    FSPointF(x: 3, y: 3)!, FSPointF(x: 3, y: 7)!, FSPointF(x: 3, y: 15)!, FSPointF(x: 13, y: 2)!,
                    FSPointF(x: 13, y: 16)!, FSPointF(x: 13, y: 17)!, FSPointF(x: 15, y: 16)!, FSPointF(x: 16, y: 16)!
                ],
                [
                    FSPointF(x: 11, y: 16)!, FSPointF(x: 12, y: 14)!, FSPointF(x: 14, y: 4)!, FSPointF(x: 14, y: 15)!,
                    FSPointF(x: 15, y: 3)!, FSPointF(x: 15, y: 9)!, FSPointF(x: 15, y: 15)!, FSPointF(x: 16, y: 15)!
                ]
            ]
            
            for k in 0..<2 {
                for i in 0..<(MemoryLayout.size(ofValue: pieces_vector[k]) / MemoryLayout.size(ofValue: pieces_vector[k][0])) {
                    let x : Int = Int(pieces_vector[k][i].x)
                    let y : Int = Int(pieces_vector[k][i].y)
                
                    let rectf = FSRectF(left1: start_x + Float(x) * width - width / 2.0, bottom1:start_y + Float(y) * width - width / 2.0, right1:start_x + Float(x) * width + width / 2.0, top1:start_y + Float(y) * width + width / 2.0)

                    addPieces(page: page, orignal_pieces: ((k % 2) != 0) ? white_pieces : black_pieces, dst_rect: rectf!)
              }
            }
        };


        
        // Get original shading objects from PDF page.
        let original_page = self.doc.getPage(0)
        original_page?.startParse(0, pause: nil, is_reparse: false)
        var position = original_page?.getFirstGraphicsObjectPosition(FSGraphicsObjectType.shading)
        if (position == nil) {
            return false
            
        }
        
        let black_pieces: FSShadingObject = (original_page?.getGraphicsObject(position!)?.getShadingObject())!
        position = original_page?.getNextGraphicsObjectPosition(position!, filter:FSGraphicsObjectType.shading)
        let white_pieces: FSShadingObject = (original_page?.getGraphicsObject(position!)?.getShadingObject())!

        // Add a new PDF page and insert text objects.
        var page = self.doc.insertPageSize(0, size:FSPDFPageSize.letter)
        addTextObjects(page: page!);

        // Add a new PDF page and insert image objects.
        page = self.doc.insertPageSize(1, size:FSPDFPageSize.letter)
        let image_file = String.init(format: "%@/rdk.png", inputPath)
        addImageObjects(page: page!, image_file: image_file);

        // Add a new PDF page and insert path objects, and copy shading objects.
        page = self.doc.insertPageSize(2, size:FSPDFPageSize.letter)
        addPathObjects(page: page!, black_pieces: black_pieces , white_pieces: white_pieces );

        self.doc.save(as: path, save_flags: UInt32(FSPDFDocSaveFlags.normal.rawValue));
        NSLog("Add graphics objects to PDF file.");
        return true
    }

}


